!==========================================================================================!
!==========================================================================================!
!     This module contains the main variable structures in ED, and several important sub-  !
! routines for allocation, de-allocation.                                                  !
!------------------------------------------------------------------------------------------!
module ed_state_vars

   use grid_coms          , only : nzg               & ! intent(in)
                                 , nzs               & ! intent(in)
                                 , ngrids            ! ! intent(in)
   use ed_max_dims        , only : max_site          & ! intent(in)
                                 , n_pft             & ! intent(in)
                                 , n_dbh             & ! intent(in)
                                 , n_mort            & ! intent(in)
                                 , n_dist_types      & ! intent(in)
                                 , n_radprof         & ! intent(in)
                                 , maxmach           & ! intent(in)
                                 , maxgrds           & ! intent(in)
                                 , str_len           ! ! intent(in)
   use disturb_coms       , only : lutime            & ! intent(in)
                                 , num_lu_trans      & ! intent(in)
                                 , max_lu_years      ! ! intent(in)
   use met_driver_coms    , only : met_driv_data     & ! intent(in)
                                 , met_driv_state    ! ! intent(in)
   use fusion_fission_coms, only : ff_nhgt           & ! intent(in)
                                 , hgt_class         ! ! intent(in)
   use phenology_coms     , only : prescribed_phen   ! ! intent(in)
   use ed_misc_coms       , only : writing_long      & ! intent(in)
                                 , writing_eorq      & ! intent(in)
                                 , writing_dcyc      & ! intent(in)
                                 , iadd_site_means   & ! intent(in)
                                 , iadd_patch_means  & ! intent(in)
                                 , iadd_cohort_means & ! intent(in)
                                 , history_fast      & ! intent(in)
                                 , history_dail      & ! intent(in)
                                 , history_eorq      & ! intent(in)
                                 , ndcycle           ! ! intent(in)

   implicit none
   !=======================================================================================!
   !=======================================================================================!





   !=======================================================================================!
   !=======================================================================================!
   ! PATCH TYPE: The following are arrays of cohorts that populate the current patch.      !
   !             This is the lowest (most nested) part of the memory structure.            !
   !---------------------------------------------------------------------------------------!  
   type patchtype
      !----- Number of cohorts in this patch.   This number can be zero (empty patch). ----!
      integer :: ncohorts
      !------------------------------------------------------------------------------------!  


      !----- Global index of the first cohort across all cohorts. -------------------------!
      integer :: coglob_id
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      ! Plant functional type.  Default values are:                                        !
      ! PFT | Name                                     | Grass   | Tropical | agriculture? !
      !-----+------------------------------------------+---------+----------+--------------!
      !   1 | C4 grass                                 |     yes |      yes |          yes !
      !   2 | Early tropical                           |      no |      yes |           no !
      !   3 | Mid tropical                             |      no |      yes |           no !
      !   4 | Late tropical                            |      no |      yes |           no !
      !   5 | Temperate C3 grass                       |     yes |       no |          yes !
      !   6 | Northern pines                           |      no |       no |           no !
      !   7 | Southern pines                           |      no |       no |           no !
      !   8 | Late conifers                            |      no |       no |           no !
      !   9 | Early temperate deciduous                |      no |       no |           no !
      !  10 | Mid temperate deciduous                  |      no |       no |           no !
      !  11 | Late temperate deciduous                 |      no |       no |           no !
      !  12 | C3 pasture                               |     yes |       no |          yes !
      !  13 | C3 crop (e.g.,wheat, rice, soybean)      |     yes |       no |          yes !
      !  14 | C4 pasture                               |     yes |      yes |          yes !
      !  15 | C4 crop (e.g.,corn/maize)                |     yes |      yes |          yes !
      !  16 | Tropical C3 grass                        |     yes |      yes |          yes !
      !  17 | Araucaria (based on 7, tropical allom.)  |      no |      yes |           no !
      !-----+------------------------------------------+---------+----------+--------------!
      integer ,pointer,dimension(:) :: pft

      ! Density of plants (number/m2)
      real ,pointer,dimension(:) :: nplant

      ! phenology_status codes:
      !  0 - plant has the maximum LAI, given its size
      !  1 - plant is growing leaves
      ! -1 - plant is actively dropping leaves
      !  2 - plant has no leaves
      integer ,pointer,dimension(:) :: phenology_status

      ! recruit_dbh codes (updated every month):
      ! 0 - new plant
      ! 1 - first month with DBH > 10cm
      ! 2 - established as DBH > 10cm
      integer, pointer, dimension(:) :: recruit_dbh

      ! census_status codes (updated every census):
      ! 0 - this cohort has never been with DBH > 10 cm at census times
      ! 1 - New recruit (DBH > 10 cm for the first census)
      ! 2 - Established (DBH > 10 cm for at least two censuses
      integer, pointer, dimension(:) :: census_status

      ! Plant height (m)
      real ,pointer,dimension(:) :: hite

      ! Above-ground biomass (kgC/plant)
      real, pointer, dimension(:) :: agb

      ! Basal area (cm2)
      real, pointer, dimension(:) :: basarea
     
      ! Rate of change in agb (kgC/plant/yr)
      real, pointer, dimension(:) :: dagb_dt
     
      ! Rate of change in agb (1/yr)
      real, pointer, dimension(:) :: dlnagb_dt
     
      ! Rate of change in basal area (cm2/yr)
      real, pointer, dimension(:) :: dba_dt
     
      ! Rate of change in basal area (1/yr)
      real, pointer, dimension(:) :: dlnba_dt
     
      ! Rate of change in dbh (cm/yr)
      real, pointer, dimension(:) :: ddbh_dt
     
      ! Rate of change in dbh ( 1/yr)
      real, pointer, dimension(:) :: dlndbh_dt

      ! Plant diameter at breast height (cm)
      real ,pointer,dimension(:) :: dbh

      ! Biomass of the structural wood (kgC/plant)
      real ,pointer,dimension(:) :: bdead

      ! Biomass of leaves (kgC/plant)
      real ,pointer,dimension(:) :: bleaf

      ! Biomass of live tissue (kgC/plant)
      real ,pointer,dimension(:) :: balive

      ! Biomass of fine roots (kgC/plant)
      real, pointer, dimension(:) :: broot

      ! Biomass of sapwood above ground (kgC/plant)
      real, pointer, dimension(:) :: bsapwooda

      ! Biomass of sapwood below ground (kgC/plant)
      real, pointer, dimension(:) :: bsapwoodb

      ! Plant storage pool of carbon [kgC/plant]
      real ,pointer,dimension(:) :: bstorage

      ! Amount of seeds produced for dispersal [kgC/plant]
      real ,pointer,dimension(:) :: bseeds

      ! Leaf area index (m2 leaf / m2 ground)
      real ,pointer,dimension(:) :: lai

      ! Wood area index (m2 wood / m2 ground)
      real ,pointer,dimension(:) :: wai

      ! Crown area (m2 crown / m2 ground)
      real ,pointer,dimension(:) :: crown_area

      ! Logical test to check whether the cohort leaves can be resolved...
      logical, pointer, dimension(:) :: leaf_resolvable
      logical, pointer, dimension(:) :: wood_resolvable
     
      ! Monthly carbon balance for past 12 months and the current month
      ! (kgC/plant) - 13th column holds the partial month integration
      real, pointer,dimension(:,:) :: cb           !(13,ncohorts)

      ! Maximum monthly carbon balance for past 12 months and the current 
      ! month  if cohort were at the top of the canopy (maximum light).
      ! (kgC/plant) - 13th column holds the partial month integration.
      real, pointer,dimension(:,:) :: cb_lightmax  !(13,ncohorts)

      ! Maximum monthly carbon balance for past 12 months and the current 
      ! month  if cohort had access to all soil moisture needed (maximum moisture/fsw).
      ! (kgC/plant) - 13th column holds the partial month integration.
      real, pointer,dimension(:,:) :: cb_moistmax  !(13,ncohorts)

      ! Maximum monthly carbon balance for past 12 months and the current 
      ! month  if cohort had access to all soil moisture needed (maximum moisture/fsw)
      ! AND the cohort was at the top of the canopy (maximum light).
      ! (kgC/plant) - 13th column holds the partial month integration.
      real, pointer,dimension(:,:) :: cb_mlmax  !(13,ncohorts)

      ! Relative carbon balance:
      !                    1         CB_lightmax             CB_watermax 
      !                  ----   = k ------------- + (1 - k) -------------
      !                   CR             CB                      CB      
      ! k is the ddmort_const given on the namelist
      real ,pointer,dimension(:) :: cbr_bar

      ! Leaf internal energy (J/m2 ground)
      real ,pointer,dimension(:) :: leaf_energy

      ! Leaf temperature (K)
      real ,pointer,dimension(:) :: leaf_temp

      ! Leaf vapour pressure deficit (Pa)
      real ,pointer,dimension(:) :: leaf_vpdef

      ! Leaf temperature of the previous step (K)
      real, pointer,dimension(:) :: leaf_temp_pv

      ! Fraction of liquid water on top of leaves (dimensionless)
      real ,pointer,dimension(:) :: leaf_fliq

      ! Leaf surface water (kg/m2 ground)
      real ,pointer,dimension(:) :: leaf_water

      ! Leaf heat capacity [ J/m2/K]
      real, pointer,dimension(:) :: leaf_hcap

      ! Wood internal energy (J/m2 ground)
      real ,pointer,dimension(:) :: wood_energy

      ! Wood temperature (K)
      real ,pointer,dimension(:) :: wood_temp

      ! Wood temperature at previous step(K)
      real ,pointer,dimension(:) :: wood_temp_pv

      ! Fraction of liquid water on top of wood (dimensionless)
      real ,pointer,dimension(:) :: wood_fliq

      ! Wood surface water (kg/m2 ground)
      real ,pointer,dimension(:) :: wood_water

      ! Wood heat capacity [ J/m2/K]
      real, pointer,dimension(:) :: wood_hcap

      ! Reduced wind at the cohort level (m/s)
      real ,pointer,dimension(:) :: veg_wind

      ! Leaf surface specific humidity for open stomata (kg/kg)
      real ,pointer,dimension(:) :: lsfc_shv_open

      ! Leaf surface specific humidity for closed stomata (kg/kg)
      real ,pointer,dimension(:) :: lsfc_shv_closed

      ! Leaf surface CO2 for open stomata (mol/mol)
      real ,pointer,dimension(:) :: lsfc_co2_open

      ! Leaf surface CO2 for closed stomata (mol/mol)
      real ,pointer,dimension(:) :: lsfc_co2_closed

      ! Leaf interecellular specific humidity (kg/kg)
      real ,pointer,dimension(:) :: lint_shv

      ! Leaf intercellular CO2 for open stomata (mol/mol)
      real ,pointer,dimension(:) :: lint_co2_open

      ! Leaf intercellular CO2 for closed stomata (mol/mol)
      real ,pointer,dimension(:) :: lint_co2_closed


      !------------------------------------------------------------------------------------!
      !     The following variables are used for growth rates.  They are for internal use  !
      ! only and are meaningless in the output files (they saved only in HISTORY).         !
      !------------------------------------------------------------------------------------!
      ! Mean leaf respiration rate (umol/m2 ground/s), averaged over 1 day
      real ,pointer,dimension(:) :: today_leaf_resp
      ! Mean root respiration rate (umol/m2 ground/s), averaged over 1 day
      real ,pointer,dimension(:) :: today_root_resp
      ! Gross primary productivity (GPP) [umol/m2 ground/s], averaged over 1 day
      real,pointer,dimension(:) :: today_gpp
      ! Potential GPP in the absence of N limitation [umol/m2 ground/s],
      ! averaged over 1 day
      real ,pointer,dimension(:) :: today_gpp_pot
      ! Maximum GPP if cohort were at the top of the canopy (maximum light)
      ! [umol/m2 ground/s], averaged over 1 day
      real ,pointer,dimension(:) :: today_gpp_lightmax
      ! Maximum GPP if cohort had all soil moisture needed (maximum soil moisture)
      ! [umol/m2 ground/s], averaged over 1 day
      real ,pointer,dimension(:) :: today_gpp_moistmax
      ! Maximum GPP if cohort had maximum soil moisture and maximum light
      ! [umol/m2 ground/s], averaged over 1 day
      real ,pointer,dimension(:) :: today_gpp_mlmax


      real,pointer,dimension(:) :: today_nppleaf
      real,pointer,dimension(:) :: today_nppfroot
      real,pointer,dimension(:) :: today_nppsapwood
      real,pointer,dimension(:) :: today_nppcroot
      real,pointer,dimension(:) :: today_nppseeds
      real,pointer,dimension(:) :: today_nppwood
      real,pointer,dimension(:) :: today_nppdaily
      !------------------------------------------------------------------------------------!



      ! Plant growth respiration (kgC/plant/day)
      real ,pointer,dimension(:) :: leaf_growth_resp
      ! Plant growth respiration (kgC/plant/day)
      real ,pointer,dimension(:) :: root_growth_resp
      ! Plant growth respiration (kgC/plant/day)
      real ,pointer,dimension(:) :: sapa_growth_resp
      ! Plant growth respiration (kgC/plant/day)
      real ,pointer,dimension(:) :: sapb_growth_resp
      ! Plant storage respiration (kgC/plant/day)
      real ,pointer,dimension(:) :: leaf_storage_resp
      real ,pointer,dimension(:) :: root_storage_resp
      real ,pointer,dimension(:) :: sapa_storage_resp
      real ,pointer,dimension(:) :: sapb_storage_resp

      ! Plant density tendency [plants/m2/month]
      real ,pointer,dimension(:) :: monthly_dndt

      ! Plant density tendency [1/month]
      real ,pointer,dimension(:) :: monthly_dlnndt

      ! Mortality rate [yr-1]
      real , pointer, dimension(:,:) :: mort_rate

      ! This specifies the index of the deepest soil layer of which the 
      ! cohort can access water.
      integer ,pointer,dimension(:) :: krdepth

      ! The model reports annual growth, mortality, cut and recruitment.  
      ! These rates are calculated with respect to two censuses.  This 
      ! is the flag specifying if a cohort was present at the first
      ! census.
      integer ,pointer,dimension(:) :: first_census

      ! Flag specifying if this cohort is a new recruit with respect
      ! to the first census.
      integer ,pointer,dimension(:) :: new_recruit_flag

      ! Light level received by this cohort, its diurnal and monthly means
      real, pointer, dimension(:) :: light_level
      real, pointer, dimension(:) :: light_level_beam
      real, pointer, dimension(:) :: light_level_diff

      ! PAR flux density centered on cohort, diurnal and monthly means
      real, pointer, dimension(:) :: par_level_beam    ! beam
      real, pointer, dimension(:) :: par_level_diffu   ! diffuse down
      real, pointer, dimension(:) :: par_level_diffd   ! diffuse up

      ! Photosynthetically active radiation (PAR) absorbed by the 
      ! cohort leaves(units are W/m2)
      real ,pointer,dimension(:)   :: par_l

      ! Photosynthetically active radiation (PAR) absorbed by the 
      ! cohort leaves (units are W/m2), beam component
      real ,pointer,dimension(:)   :: par_l_beam

      ! Photosynthetically active radiation (PAR) absorbed by the 
      ! cohort leaves (units are W/m2), diffuse component
      real ,pointer,dimension(:)   :: par_l_diffuse

      ! Total short wave radiation absorbed by the cohort leaves, W/m2
      real ,pointer,dimension(:) :: rshort_l

      ! Total short wave radiation absorbed by the cohort leaves, W/m2, beam component
      real ,pointer,dimension(:) :: rshort_l_beam

      ! Total short wave radiation absorbed by the cohort leaves, W/m2, diffuse 
      ! component
      real ,pointer,dimension(:) :: rshort_l_diffuse

      ! Total long wave radiation absorbed by the cohort leaves (W/m2)
      real ,pointer,dimension(:) :: rlong_l

      ! Total short wave radiation absorbed by the cohort wood, W/m2
      real ,pointer,dimension(:) :: rshort_w

      ! Total short wave radiation absorbed by the cohort wood, W/m2, beam component
      real ,pointer,dimension(:) :: rshort_w_beam

      ! Total short wave radiation absorbed by the cohort wood, W/m2, diffuse 
      ! component
      real ,pointer,dimension(:) :: rshort_w_diffuse

      ! Total long wave radiation absorbed by the cohort wood (W/m2)
      real ,pointer,dimension(:) :: rlong_w

      !----- Radiation profile for this patch, at thhe cohort interfaces (W/m2). ----------!
      !  1. PAR, Beam, Down                                                                !
      !  2. PAR, Beam, Up (only Medvigy's radiation, zero for others).                     !
      !  3. PAR, Diff, Down                                                                !
      !  4. PAR, Diff, Up                                                                  !
      !  5. NIR, Beam, Down                                                                !
      !  6. NIR, Beam, Up (only Medvigy's radiation, zero for others).                     !
      !  7. NIR, Diff, Down                                                                !
      !  8. NIR, Diff, Up                                                                  !
      !  9. TIR, Diff, Down                                                                !
      ! 10. TIR. Diff, Up                                                                  !
      !------------------------------------------------------------------------------------!
      real ,pointer,dimension(:,:) :: rad_profile

      ! Leaf boundary layer conductance for heat/entropy  [J/K/m2leaf/s]
      real ,pointer,dimension(:) :: leaf_gbh

      ! Leaf boundary layer conductance for water [kg/m2leaf/s]
      real ,pointer,dimension(:) :: leaf_gbw

      ! Wood boundary layer conductance for heat/entropy  [J/K/m2wood/s]
      real ,pointer,dimension(:) :: wood_gbh

      ! Wood boundary layer conductance for water [kg/m2wood/s]
      real ,pointer,dimension(:) :: wood_gbw

      ! Photosynthesis rate, open stomata (umol/m2 leaf/s)
      real ,pointer,dimension(:) :: A_open

      ! Photosynthesis rate, closed stomata (umol/m2 leaf/s)
      real ,pointer,dimension(:) :: A_closed

      ! Photosynthesis rate, light limited (umol/m2 leaf/s)
      real ,pointer,dimension(:) :: A_light

      ! Photosynthesis rate, rubisco limited (umol/m2 leaf/s)
      real ,pointer,dimension(:) :: A_rubp

      ! Photosynthesis rate, CO2 limited (umol/m2 leaf/s)
      real ,pointer,dimension(:) :: A_co2

      ! Transpiration rate, open stomata (kg/m2_leaf/s)
      real ,pointer,dimension(:) :: psi_open

      ! Transpiration rate, closed stomata (kg/m2_leaf/s)
      real ,pointer,dimension(:) :: psi_closed

      ! Stomatal conductance for water, open stomata [kg/m2leaf/s]
      real ,pointer,dimension(:) :: gsw_open

      ! Stomatal conductance for water, closed stomata [kg/m2leaf/s]
      real ,pointer,dimension(:) :: gsw_closed

      ! Net stomatal conductance [kg/m2leaf/s]
      real ,pointer,dimension(:) :: leaf_gsw

      ! Weighting factor for open and closed stomata (fsw=1 => fully open)
      real ,pointer,dimension(:) :: fsw

      ! Weighting factor for open and closed stomata due to N limitation
      real ,pointer,dimension(:) :: fsn

      ! Product of fsw and fsn
      real ,pointer,dimension(:) :: fs_open

      ! Water supply (kg_H2O/m2/s)
      real , pointer, dimension(:) :: water_supply

      ! Plant maintenance costs due to turnover of leaves and fine 
      ! roots [kgC/plant]
      real ,pointer,dimension(:) :: leaf_maintenance
      real ,pointer,dimension(:) :: root_maintenance
     
      ! Leaf loss to litter layer due to phenology [kgC/plant]
      real , pointer, dimension(:) :: leaf_drop

      ! Instantaneous values of leaf and root respiration [umol/m2/s]
      real ,pointer,dimension(:) :: leaf_respiration
      real ,pointer,dimension(:) :: root_respiration

      ! Gross Primary Productivity [umol/m2/s]
      real, pointer,dimension(:) :: gpp

      ! Plant available water, (stress function)  0 to 1 [unitless]
      real, pointer, dimension(:) :: paw_avg

      ! Elongation factor (0 - 1), factor to scale bleaf under drought.
      real, pointer, dimension(:) :: elongf

      ! Phenology-related
      real, pointer, dimension(:) :: turnover_amp
      real, pointer, dimension(:) :: llspan
      real, pointer, dimension(:) :: vm_bar
      real, pointer, dimension(:) :: sla

      !------------------------------------------------------------------------------------!
      ! These are diagnostic variables, averaged over various time scales.                 !
      ! FMEAN -- averaged over FRQSUM (the least between FRQFAST, FRQSTATE and one day).   !
      ! DMEAN -- daily averages                                                            !
      ! MMEAN -- monthly averages                                                          !
      ! MMSQU -- monthly mean sum of squares                                               !
      ! QMEAN -- mean diel                                                                 !
      ! QMSQU -- mean diel (mean sum of squares)                                           !
      !                                                                                    !
      ! pl means plant              (intensive variable)                                   !
      ! m2l means m2 of leaf area   (intensive variable)                                   !
      ! m2g means m2 of ground area (extensive variable)                                   !
      !------------------------------------------------------------------------------------!
      !----- Fast averages. ---------------------------------------------------------------!
      real,pointer,dimension(:)   :: fmean_gpp              ! Gross prim. prod. [ kgC/pl/yr]
      real,pointer,dimension(:)   :: fmean_npp              ! Net primary prod. [ kgC/pl/yr]
      real,pointer,dimension(:)   :: fmean_leaf_resp        ! Leaf respiration  [ kgC/pl/yr]
      real,pointer,dimension(:)   :: fmean_root_resp        ! Root respiration  [ kgC/pl/yr]
      real,pointer,dimension(:)   :: fmean_leaf_growth_resp ! Growth resp.      [ kgC/pl/yr]
      real,pointer,dimension(:)   :: fmean_root_growth_resp ! Growth resp.      [ kgC/pl/yr]
      real,pointer,dimension(:)   :: fmean_sapa_growth_resp ! Growth resp.      [ kgC/pl/yr]
      real,pointer,dimension(:)   :: fmean_sapb_growth_resp ! Growth resp.      [ kgC/pl/yr]
      real,pointer,dimension(:)   :: fmean_leaf_storage_resp! Storage resp.     [ kgC/pl/yr]
      real,pointer,dimension(:)   :: fmean_root_storage_resp! Storage resp.     [ kgC/pl/yr]
      real,pointer,dimension(:)   :: fmean_sapa_storage_resp! Storage resp.     [ kgC/pl/yr]
      real,pointer,dimension(:)   :: fmean_sapb_storage_resp! Storage resp.     [ kgC/pl/yr]
      real,pointer,dimension(:)   :: fmean_plresp           ! Plant resp.       [ kgC/pl/yr]
      real,pointer,dimension(:)   :: fmean_leaf_energy      ! Leaf int. energy  [     J/m2g]
      real,pointer,dimension(:)   :: fmean_leaf_water       ! Leaf sfc. water   [    kg/m2g]
      real,pointer,dimension(:)   :: fmean_leaf_hcap        ! Leaf heat cap.    [   J/m2g/K]
      real,pointer,dimension(:)   :: fmean_leaf_vpdef       ! Leaf VPD          [        Pa]
      real,pointer,dimension(:)   :: fmean_leaf_temp        ! Leaf temperature  [         K]
      real,pointer,dimension(:)   :: fmean_leaf_fliq        ! Liquid fraction   [        --]
      real,pointer,dimension(:)   :: fmean_leaf_gsw         ! Stomatal conduct. [       m/s]
      real,pointer,dimension(:)   :: fmean_leaf_gbw         ! Leaf BL conduct.  [       m/s]
      real,pointer,dimension(:)   :: fmean_wood_energy      ! Wood int. energy  [     J/m2g]
      real,pointer,dimension(:)   :: fmean_wood_water       ! Wood sfc. water   [    kg/m2g]
      real,pointer,dimension(:)   :: fmean_wood_hcap        ! Wood heat cap.    [   J/m2g/K]
      real,pointer,dimension(:)   :: fmean_wood_temp        ! Wood temperature  [         K]
      real,pointer,dimension(:)   :: fmean_wood_fliq        ! Liquid fraction   [        --]
      real,pointer,dimension(:)   :: fmean_wood_gbw         ! Wood BL conduct.  [       m/s]
      real,pointer,dimension(:)   :: fmean_fs_open          ! Net stress factor [        --]
      real,pointer,dimension(:)   :: fmean_fsw              ! Moisture stress   [        --]
      real,pointer,dimension(:)   :: fmean_fsn              ! Nitrogen stress   [        --]
      real,pointer,dimension(:)   :: fmean_a_open           ! Assim. (no stress)[umol/m2l/s]
      real,pointer,dimension(:)   :: fmean_a_closed         ! Minimum assim.    [umol/m2l/s]
      real,pointer,dimension(:)   :: fmean_a_net            ! Assim. (actual)   [umol/m2l/s]
      real,pointer,dimension(:)   :: fmean_a_light          ! Assim. (light)    [umol/m2l/s]
      real,pointer,dimension(:)   :: fmean_a_rubp           ! Assim. (RuBP)     [umol/m2l/s]
      real,pointer,dimension(:)   :: fmean_a_co2            ! Assim. (CO2)      [umol/m2l/s]
      real,pointer,dimension(:)   :: fmean_psi_open         ! Transp. no stress [  kg/m2l/s]
      real,pointer,dimension(:)   :: fmean_psi_closed       ! Transp. max st.   [  kg/m2l/s]
      real,pointer,dimension(:)   :: fmean_water_supply     ! Water supply      [        --]
      real,pointer,dimension(:)   :: fmean_light_level      ! Light lev. (Tot.) [        --]
      real,pointer,dimension(:)   :: fmean_light_level_beam ! Light lev. (Beam) [        --]
      real,pointer,dimension(:)   :: fmean_light_level_diff ! Lighy lev. (Diff) [        --]
      real,pointer,dimension(:)   :: fmean_par_level_beam   ! Par lev. (beam)   [- or W/m2?]
      real,pointer,dimension(:)   :: fmean_par_level_diffu  ! Par lev. (Diff Up)[- or W/m2?]
      real,pointer,dimension(:)   :: fmean_par_level_diffd  ! Par lev. (Diff Dn)[- or W/m2?]
      real,pointer,dimension(:)   :: fmean_par_l            ! Abs. PAR (Leaf)   [     W/m2g]
      real,pointer,dimension(:)   :: fmean_par_l_beam       ! Abs. Dir. PAR     [     W/m2g]
      real,pointer,dimension(:)   :: fmean_par_l_diff       ! Abs. Diffuse PAR  [     W/m2g]
      real,pointer,dimension(:)   :: fmean_rshort_l         ! Abs. SW (leaf)    [     W/m2g]
      real,pointer,dimension(:)   :: fmean_rlong_l          ! Abs. LW (leaf)    [     W/m2g]
      real,pointer,dimension(:)   :: fmean_sensible_lc      ! Sensible heat     [     W/m2g]
      real,pointer,dimension(:)   :: fmean_vapor_lc         ! Leaf evaporation  [  kg/m2g/s]
      real,pointer,dimension(:)   :: fmean_transp           ! Leaf transp.      [  kg/m2g/s]
      real,pointer,dimension(:)   :: fmean_intercepted_al   ! Leaf interception [  kg/m2g/s]
      real,pointer,dimension(:)   :: fmean_wshed_lg         ! Leaf shedding     [  kg/m2g/s]
      real,pointer,dimension(:)   :: fmean_rshort_w         ! Abs. SW (Wood)    [     W/m2g]
      real,pointer,dimension(:)   :: fmean_rlong_w          ! Abs. LW (Wood)    [     W/m2g]
      real,pointer,dimension(:,:) :: fmean_rad_profile      ! Rad. profile      [     W/m2g]
      real,pointer,dimension(:)   :: fmean_sensible_wc      ! Sensible heat     [     W/m2g]
      real,pointer,dimension(:)   :: fmean_vapor_wc         ! Wood evaporation  [  kg/m2g/s]
      real,pointer,dimension(:)   :: fmean_intercepted_aw   ! Wood interception [  kg/m2g/s]
      real,pointer,dimension(:)   :: fmean_wshed_wg         ! Wood shedding     [  kg/m2g/s]
      real,pointer,dimension(:)   :: fmean_lai		    ! LAI  		[     m2/m2]
      real,pointer,dimension(:)	  :: fmean_bdead	    ! Bdead		[     kg/pl]
      !----- Variables without sub-daily averages. ----------------------------------------!
      real,pointer,dimension(:)   :: dmean_nppleaf          ! Leaf NPP          [ kgC/pl/yr]
      real,pointer,dimension(:)   :: dmean_nppfroot         ! Fine root NPP     [ kgC/pl/yr]
      real,pointer,dimension(:)   :: dmean_nppsapwood       ! Sapwood NPP       [ kgC/pl/yr]
      real,pointer,dimension(:)   :: dmean_nppcroot         ! Coarse root NPP   [ kgC/pl/yr]
      real,pointer,dimension(:)   :: dmean_nppseeds         ! Seed NPP          [ kgC/pl/yr]
      real,pointer,dimension(:)   :: dmean_nppwood          ! Wood NPP          [ kgC/pl/yr]
      real,pointer,dimension(:)   :: dmean_nppdaily         ! Daily NPP         [ kgC/pl/yr]
      !----- Monthly means of variables that are integrated daily. ------------------------!
      real,pointer,dimension(:)   :: mmean_lai              ! Leaf area index   [   m2l/m2g]
      real,pointer,dimension(:)   :: mmean_bleaf            ! Leaf biomass      [    kgC/pl]
      real,pointer,dimension(:)   :: mmean_broot            ! Root biomass      [    kgC/pl]
      real,pointer,dimension(:)   :: mmean_bstorage         ! Storage biomass   [    kgC/pl]
      real,pointer,dimension(:,:) :: mmean_mort_rate        ! Mortality rate    [      1/yr]
      real,pointer,dimension(:)   :: mmean_leaf_maintenance ! Leaf maintenance  [ kgC/pl/yr]
      real,pointer,dimension(:)   :: mmean_root_maintenance ! Root mainten.     [ kgC/pl/yr]
      real,pointer,dimension(:)   :: mmean_leaf_drop        ! Leaf drop         [ kgC/pl/yr]
      real,pointer,dimension(:)   :: mmean_cb               ! 12-mon C balance  [    kgC/pl]
      !----- Daily mean (same units as fast mean). ----------------------------------------!
      real,pointer,dimension(:)     :: dmean_gpp
      real,pointer,dimension(:)     :: dmean_npp
      real,pointer,dimension(:)     :: dmean_leaf_resp
      real,pointer,dimension(:)     :: dmean_root_resp
      real,pointer,dimension(:)     :: dmean_leaf_growth_resp
      real,pointer,dimension(:)     :: dmean_root_growth_resp
      real,pointer,dimension(:)     :: dmean_sapa_growth_resp
      real,pointer,dimension(:)     :: dmean_sapb_growth_resp
      real,pointer,dimension(:)     :: dmean_leaf_storage_resp
      real,pointer,dimension(:)     :: dmean_root_storage_resp
      real,pointer,dimension(:)     :: dmean_sapa_storage_resp
      real,pointer,dimension(:)     :: dmean_sapb_storage_resp
      real,pointer,dimension(:)     :: dmean_plresp
      real,pointer,dimension(:)     :: dmean_leaf_energy
      real,pointer,dimension(:)     :: dmean_leaf_water
      real,pointer,dimension(:)     :: dmean_leaf_hcap
      real,pointer,dimension(:)     :: dmean_leaf_vpdef
      real,pointer,dimension(:)     :: dmean_leaf_temp
      real,pointer,dimension(:)     :: dmean_leaf_fliq
      real,pointer,dimension(:)     :: dmean_leaf_gsw
      real,pointer,dimension(:)     :: dmean_leaf_gbw
      real,pointer,dimension(:)     :: dmean_wood_energy
      real,pointer,dimension(:)     :: dmean_wood_water
      real,pointer,dimension(:)     :: dmean_wood_hcap
      real,pointer,dimension(:)     :: dmean_wood_temp
      real,pointer,dimension(:)     :: dmean_wood_fliq
      real,pointer,dimension(:)     :: dmean_wood_gbw
      real,pointer,dimension(:)     :: dmean_fs_open
      real,pointer,dimension(:)     :: dmean_fsw
      real,pointer,dimension(:)     :: dmean_fsn
      real,pointer,dimension(:)     :: dmean_a_open
      real,pointer,dimension(:)     :: dmean_a_closed
      real,pointer,dimension(:)     :: dmean_a_net
      real,pointer,dimension(:)     :: dmean_a_light
      real,pointer,dimension(:)     :: dmean_a_rubp
      real,pointer,dimension(:)     :: dmean_a_co2
      real,pointer,dimension(:)     :: dmean_psi_open
      real,pointer,dimension(:)     :: dmean_psi_closed
      real,pointer,dimension(:)     :: dmean_water_supply
      real,pointer,dimension(:)     :: dmean_light_level
      real,pointer,dimension(:)     :: dmean_light_level_beam
      real,pointer,dimension(:)     :: dmean_light_level_diff
      
      real,pointer,dimension(:)     :: dmean_par_level_beam
      real,pointer,dimension(:)     :: dmean_par_level_diffu
      real,pointer,dimension(:)     :: dmean_par_level_diffd
      
      real,pointer,dimension(:)     :: dmean_par_l
      real,pointer,dimension(:)     :: dmean_par_l_beam
      real,pointer,dimension(:)     :: dmean_par_l_diff
      real,pointer,dimension(:)     :: dmean_rshort_l
      real,pointer,dimension(:)     :: dmean_rlong_l
      real,pointer,dimension(:)     :: dmean_sensible_lc
      real,pointer,dimension(:)     :: dmean_vapor_lc
      real,pointer,dimension(:)     :: dmean_transp
      real,pointer,dimension(:)     :: dmean_intercepted_al
      real,pointer,dimension(:)     :: dmean_wshed_lg
      real,pointer,dimension(:)     :: dmean_rshort_w
      real,pointer,dimension(:)     :: dmean_rlong_w
      real,pointer,dimension(:,:)   :: dmean_rad_profile
      real,pointer,dimension(:)     :: dmean_sensible_wc
      real,pointer,dimension(:)     :: dmean_vapor_wc
      real,pointer,dimension(:)     :: dmean_intercepted_aw
      real,pointer,dimension(:)     :: dmean_wshed_wg
      !----- Monthly mean (same units as fast mean). --------------------------------------!
      real,pointer,dimension(:)     :: mmean_gpp
      real,pointer,dimension(:)     :: mmean_npp
      real,pointer,dimension(:)     :: mmean_leaf_resp
      real,pointer,dimension(:)     :: mmean_root_resp
      real,pointer,dimension(:)     :: mmean_leaf_growth_resp
      real,pointer,dimension(:)     :: mmean_root_growth_resp
      real,pointer,dimension(:)     :: mmean_sapa_growth_resp
      real,pointer,dimension(:)     :: mmean_sapb_growth_resp
      real,pointer,dimension(:)     :: mmean_leaf_storage_resp
      real,pointer,dimension(:)     :: mmean_root_storage_resp
      real,pointer,dimension(:)     :: mmean_sapa_storage_resp
      real,pointer,dimension(:)     :: mmean_sapb_storage_resp
      real,pointer,dimension(:)     :: mmean_plresp
      real,pointer,dimension(:)     :: mmean_leaf_energy
      real,pointer,dimension(:)     :: mmean_leaf_water
      real,pointer,dimension(:)     :: mmean_leaf_hcap
      real,pointer,dimension(:)     :: mmean_leaf_vpdef
      real,pointer,dimension(:)     :: mmean_leaf_temp
      real,pointer,dimension(:)     :: mmean_leaf_fliq
      real,pointer,dimension(:)     :: mmean_leaf_gsw
      real,pointer,dimension(:)     :: mmean_leaf_gbw
      real,pointer,dimension(:)     :: mmean_wood_energy
      real,pointer,dimension(:)     :: mmean_wood_water
      real,pointer,dimension(:)     :: mmean_wood_hcap
      real,pointer,dimension(:)     :: mmean_wood_temp
      real,pointer,dimension(:)     :: mmean_wood_fliq
      real,pointer,dimension(:)     :: mmean_wood_gbw
      real,pointer,dimension(:)     :: mmean_fs_open
      real,pointer,dimension(:)     :: mmean_fsw
      real,pointer,dimension(:)     :: mmean_fsn
      real,pointer,dimension(:)     :: mmean_a_open
      real,pointer,dimension(:)     :: mmean_a_closed
      real,pointer,dimension(:)     :: mmean_a_net
      real,pointer,dimension(:)     :: mmean_a_light
      real,pointer,dimension(:)     :: mmean_a_rubp
      real,pointer,dimension(:)     :: mmean_a_co2
      real,pointer,dimension(:)     :: mmean_psi_open
      real,pointer,dimension(:)     :: mmean_psi_closed
      real,pointer,dimension(:)     :: mmean_water_supply
      real,pointer,dimension(:)     :: mmean_light_level
      real,pointer,dimension(:)     :: mmean_light_level_beam
      real,pointer,dimension(:)     :: mmean_light_level_diff

      real,pointer,dimension(:)     :: mmean_par_level_beam
      real,pointer,dimension(:)     :: mmean_par_level_diffu
      real,pointer,dimension(:)     :: mmean_par_level_diffd

      real,pointer,dimension(:)     :: mmean_par_l
      real,pointer,dimension(:)     :: mmean_par_l_beam
      real,pointer,dimension(:)     :: mmean_par_l_diff
      real,pointer,dimension(:)     :: mmean_rshort_l
      real,pointer,dimension(:)     :: mmean_rlong_l
      real,pointer,dimension(:)     :: mmean_sensible_lc
      real,pointer,dimension(:)     :: mmean_vapor_lc
      real,pointer,dimension(:)     :: mmean_transp
      real,pointer,dimension(:)     :: mmean_intercepted_al
      real,pointer,dimension(:)     :: mmean_wshed_lg
      real,pointer,dimension(:)     :: mmean_rshort_w
      real,pointer,dimension(:)     :: mmean_rlong_w
      real,pointer,dimension(:,:)   :: mmean_rad_profile
      real,pointer,dimension(:)     :: mmean_sensible_wc
      real,pointer,dimension(:)     :: mmean_vapor_wc
      real,pointer,dimension(:)     :: mmean_intercepted_aw
      real,pointer,dimension(:)     :: mmean_wshed_wg
      real,pointer,dimension(:)     :: mmean_nppleaf
      real,pointer,dimension(:)     :: mmean_nppfroot
      real,pointer,dimension(:)     :: mmean_nppsapwood
      real,pointer,dimension(:)     :: mmean_nppcroot
      real,pointer,dimension(:)     :: mmean_nppseeds
      real,pointer,dimension(:)     :: mmean_nppwood
      real,pointer,dimension(:)     :: mmean_nppdaily
      !----- Monthly mean sum of squares. -------------------------------------------------!
      real,pointer,dimension(:)     :: mmsqu_gpp
      real,pointer,dimension(:)     :: mmsqu_npp
      real,pointer,dimension(:)     :: mmsqu_plresp
      real,pointer,dimension(:)     :: mmsqu_sensible_lc
      real,pointer,dimension(:)     :: mmsqu_vapor_lc
      real,pointer,dimension(:)     :: mmsqu_transp
      real,pointer,dimension(:)     :: mmsqu_sensible_wc
      real,pointer,dimension(:)     :: mmsqu_vapor_wc
      !----- Mean diel (same units as fast mean). -----------------------------------------!
      real,pointer,dimension(:,:)   :: qmean_gpp
      real,pointer,dimension(:,:)   :: qmean_npp
      real,pointer,dimension(:,:)   :: qmean_leaf_resp
      real,pointer,dimension(:,:)   :: qmean_root_resp
      real,pointer,dimension(:,:)   :: qmean_leaf_growth_resp
      real,pointer,dimension(:,:)   :: qmean_root_growth_resp
      real,pointer,dimension(:,:)   :: qmean_sapa_growth_resp
      real,pointer,dimension(:,:)   :: qmean_sapb_growth_resp
      real,pointer,dimension(:,:)   :: qmean_leaf_storage_resp
      real,pointer,dimension(:,:)   :: qmean_root_storage_resp
      real,pointer,dimension(:,:)   :: qmean_sapa_storage_resp
      real,pointer,dimension(:,:)   :: qmean_sapb_storage_resp
      real,pointer,dimension(:,:)   :: qmean_plresp
      real,pointer,dimension(:,:)   :: qmean_leaf_energy
      real,pointer,dimension(:,:)   :: qmean_leaf_water
      real,pointer,dimension(:,:)   :: qmean_leaf_hcap
      real,pointer,dimension(:,:)   :: qmean_leaf_vpdef
      real,pointer,dimension(:,:)   :: qmean_leaf_temp
      real,pointer,dimension(:,:)   :: qmean_leaf_fliq
      real,pointer,dimension(:,:)   :: qmean_leaf_gsw
      real,pointer,dimension(:,:)   :: qmean_leaf_gbw
      real,pointer,dimension(:,:)   :: qmean_wood_energy
      real,pointer,dimension(:,:)   :: qmean_wood_water
      real,pointer,dimension(:,:)   :: qmean_wood_hcap
      real,pointer,dimension(:,:)   :: qmean_wood_temp
      real,pointer,dimension(:,:)   :: qmean_wood_fliq
      real,pointer,dimension(:,:)   :: qmean_wood_gbw
      real,pointer,dimension(:,:)   :: qmean_fs_open
      real,pointer,dimension(:,:)   :: qmean_fsw
      real,pointer,dimension(:,:)   :: qmean_fsn
      real,pointer,dimension(:,:)   :: qmean_a_open
      real,pointer,dimension(:,:)   :: qmean_a_closed
      real,pointer,dimension(:,:)   :: qmean_a_net
      real,pointer,dimension(:,:)   :: qmean_a_light
      real,pointer,dimension(:,:)   :: qmean_a_rubp
      real,pointer,dimension(:,:)   :: qmean_a_co2
      real,pointer,dimension(:,:)   :: qmean_psi_open
      real,pointer,dimension(:,:)   :: qmean_psi_closed
      real,pointer,dimension(:,:)   :: qmean_water_supply
      real,pointer,dimension(:,:)   :: qmean_light_level
      real,pointer,dimension(:,:)   :: qmean_light_level_beam
      real,pointer,dimension(:,:)   :: qmean_light_level_diff

      ! ADDED RGK 11-2014
      real,pointer,dimension(:,:)   :: qmean_par_level_beam
      real,pointer,dimension(:,:)   :: qmean_par_level_diffu
      real,pointer,dimension(:,:)   :: qmean_par_level_diffd

      real,pointer,dimension(:,:)   :: qmean_par_l
      real,pointer,dimension(:,:)   :: qmean_par_l_beam
      real,pointer,dimension(:,:)   :: qmean_par_l_diff
      real,pointer,dimension(:,:)   :: qmean_rshort_l
      real,pointer,dimension(:,:)   :: qmean_rlong_l
      real,pointer,dimension(:,:)   :: qmean_sensible_lc
      real,pointer,dimension(:,:)   :: qmean_vapor_lc
      real,pointer,dimension(:,:)   :: qmean_transp
      real,pointer,dimension(:,:)   :: qmean_intercepted_al
      real,pointer,dimension(:,:)   :: qmean_wshed_lg
      real,pointer,dimension(:,:)   :: qmean_rshort_w
      real,pointer,dimension(:,:)   :: qmean_rlong_w
      real,pointer,dimension(:,:,:) :: qmean_rad_profile
      real,pointer,dimension(:,:)   :: qmean_sensible_wc
      real,pointer,dimension(:,:)   :: qmean_vapor_wc
      real,pointer,dimension(:,:)   :: qmean_intercepted_aw
      real,pointer,dimension(:,:)   :: qmean_wshed_wg
      !------ Mean diel of sum of squares. ------------------------------------------------!
      real,pointer,dimension(:,:)   :: qmsqu_gpp
      real,pointer,dimension(:,:)   :: qmsqu_npp
      real,pointer,dimension(:,:)   :: qmsqu_plresp
      real,pointer,dimension(:,:)   :: qmsqu_sensible_lc
      real,pointer,dimension(:,:)   :: qmsqu_vapor_lc
      real,pointer,dimension(:,:)   :: qmsqu_transp
      real,pointer,dimension(:,:)   :: qmsqu_sensible_wc
      real,pointer,dimension(:,:)   :: qmsqu_vapor_wc
      !------------------------------------------------------------------------------------!
   end type patchtype
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   ! SITE TYPE:  The following are the patch level arrays that populate the current site.  !
   !---------------------------------------------------------------------------------------!
   type sitetype
      !----- Number of patches in this site. ----------------------------------------------!
      integer :: npatches
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !  MAIN INDICES:                                                                     !
      !  PACO_ID -- The global index of the first cohort in all patches                    !
      !  PACO_N  -- The number of cohorts in each patch                                    !
      !  PAGLOB_ID -- Global index of the first patch in this vector, across all patches.  !
      !------------------------------------------------------------------------------------!
      integer,pointer,dimension(:) :: paco_id
      integer,pointer,dimension(:) :: paco_n
      integer :: paglob_id
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      !      This structure contains all cohorts in each patch.                            !
      !------------------------------------------------------------------------------------!
      type(patchtype),pointer,dimension(:) :: patch
      !------------------------------------------------------------------------------------!


      !----- Number of cohorts in the patch. ----------------------------------------------!
      integer, pointer,dimension(:) :: cohort_count
      !------------------------------------------------------------------------------------!


      !----- Patch name (used only when restarting from ED1). -----------------------------!
      character(len=str_len), pointer,dimension(:) :: pname
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      !  BASIC INFORMATION                                                                 !
      !                                                                                    !
      !  AREA      -- patch area (relative to the total SITE area).                        !
      !  AGE       -- time since last disturbance (years)                                  !
      !  DIST_TYPE -- patch type                                                           !
      !               1.  Clear cut (cropland and pasture).                                !
      !               2.  Forest plantation.                                               !
      !               3.  Tree fall.                                                       !
      !               4.  Fire.                                                            !
      !               5.  Forest regrowth.                                                 !
      !               6.  Logged forest.                                                   !
      !------------------------------------------------------------------------------------!
      real   , pointer,dimension(:) :: area
      real   , pointer,dimension(:) :: age
      integer, pointer,dimension(:) :: dist_type
      !------------------------------------------------------------------------------------!

      ! Soil carbon concentration, fast pool (kg/m2)
      real , pointer,dimension(:) :: fast_soil_C 

      ! Soil carbon concentration, slow pool (kg/m2)
      real , pointer,dimension(:) :: slow_soil_C 

      ! Soil carbon concentration, structural pool (kg/m2)
      real , pointer,dimension(:) :: structural_soil_C

      ! Soil lignin concentration, structural pool (kg/m2)
      real , pointer,dimension(:) :: structural_soil_L

      ! Soil nitrogen concentration, mineralized pool (kg/m2)
      real , pointer,dimension(:) :: mineralized_soil_N  

      ! Soil nitrogen concentration, fast pool (kg/m2)
      real , pointer,dimension(:) :: fast_soil_N 

      ! Number of degree days
      ! Degree days --- sum of daily average temperatures above 278.15 K 
      real , pointer,dimension(:) :: sum_dgd

      ! Chill days --- number of days with average temperatures below 278.15 K
      real , pointer,dimension(:) :: sum_chd  

      ! Ice-vapour equivalent potential temperature of canopy air space [K]
      real , pointer,dimension(:) :: can_theiv

      ! Vapour pressure deficit [Pa]
      real , pointer,dimension(:) :: can_vpdef

      ! Temperature (K) of canopy air
      real , pointer,dimension(:) :: can_temp

      ! Previous step's canopy air temperature
      real , pointer,dimension(:) :: can_temp_pv


      ! Water vapor specific humidity (kg/kg) of canopy air
      real , pointer,dimension(:) :: can_shv

      ! CO2 mixing ratio [umol/mol] of canopy air
      real , pointer,dimension(:) :: can_co2

      ! Density [kg/m3] of canopy air
      real , pointer,dimension(:) :: can_rhos

      ! Canopy air pressure.
      real , pointer,dimension(:) :: can_prss

      ! Canopy air potential temperature
      real , pointer,dimension(:) :: can_theta

      ! Canopy depth (m)
      real , pointer,dimension(:) :: can_depth

      ! Fraction of canopy that is open (---)
      real , pointer,dimension(:) :: opencan_frac
      
      ! Ground to canopy conductances
      real , pointer, dimension(:) :: ggbare
      real , pointer, dimension(:) :: ggveg
      real , pointer, dimension(:) :: ggnet
      real , pointer, dimension(:) :: ggsoil

      
      ! Number of surface water layers
      integer,pointer,dimension(:) :: nlev_sfcwater

      ! Surface water mass (kg/m2)
      real, pointer,dimension(:,:) :: sfcwater_mass

      ! Surface water internal energy (J/kg)
      real, pointer,dimension(:,:) :: sfcwater_energy

      ! Depth of surface water (m)
      real, pointer,dimension(:,:) :: sfcwater_depth

      ! Temperature of surface water (K)
      real, pointer,dimension(:,:) :: sfcwater_tempk
     
      ! Liquid fraction of surface water
      real, pointer,dimension(:,:) :: sfcwater_fracliq

      ! Short wave radiation absorbed by the surface water (W/m2)
      real, pointer,dimension(:,:) :: rshort_s

      ! Short wave radiation absorbed by the surface water, 
      ! beam component (W/m2)
      real, pointer,dimension(:,:) :: rshort_s_beam

      ! Short wave radiation absorbed by the surface water, 
      ! diffuse component (W/m2)
      real, pointer,dimension(:,:) :: rshort_s_diffuse

      ! PAR absorbed by the surface water (W/m2)
      real, pointer,dimension(:,:) :: par_s

      ! PAR absorbed by the surface water, 
      ! beam component (W/m2)
      real, pointer,dimension(:,:) :: par_s_beam

      ! PAR absorbed by the surface water, 
      ! diffuse component (W/m2)
      real, pointer,dimension(:,:) :: par_s_diffuse
      
      ! Soil internal energy (J/m3)
      real,    pointer,dimension(:,:) :: soil_energy
      
      ! Soil matric potential (m)
      real,    pointer,dimension(:,:) :: soil_mstpot
      
      ! Soil water (m3 water / m3 soil)
      real,    pointer,dimension(:,:) :: soil_water
           
      ! Temperature of soil (K)
      real,    pointer,dimension(:,:) :: soil_tempk
      
      ! Liquid fraction of soil
      real,    pointer,dimension(:,:) :: soil_fracliq

      ! Root density [kg/m3] over the profile
      ! Typically diagnosed for history or monthly output
      real, pointer, dimension (:,:) :: rootdense
      
      ! Effective specific humidity (kg/kg) just above soil
      real,    pointer,dimension(:) :: ground_shv
      
      ! Surface saturation specific humidity (kg/kg)
      real,    pointer,dimension(:) :: ground_ssh

      ! Ground temperature (either the top soil or top surface snow/water) (K)
      real, pointer, dimension (:) :: ground_temp

      ! Ground temperature (either the top soil or top surface snow/water) (K)
      real, pointer, dimension (:) :: ground_fliq

      ! Net roughness length (m)
      real,    pointer,dimension(:)  :: rough

      ! Maximum PAR possibly attained in this patch.  This is found by calling the radiation
      ! scheme for a single cohort (the tallest that can be solved).
      real,    pointer, dimension(:) :: par_l_max
      real,    pointer, dimension(:) :: par_l_beam_max
      real,    pointer, dimension(:) :: par_l_diffuse_max

      ! Photosynthetic rates for different PFTs, if they were at the top 
      ! of the canopy (umol/m2 leaf/s).  Used by mortality function.
      real, pointer,dimension(:,:) :: A_o_max  ! open stomata
      real, pointer,dimension(:,:) :: A_c_max  ! closed stomata

      ! Average daily temperature [K]
      real , pointer,dimension(:) :: avg_daily_temp  

      ! Average monthly ground water [kg/m2], used for fire ignition
      real , pointer,dimension(:) :: avg_monthly_gndwater

      ! Average monthly water deficit [kg/m2], used for fire ignition
      real , pointer,dimension(:) :: avg_monthly_waterdef


      !=======================================================================!
      !      These variables are terms of the water, carbon, and energy       !
      ! budget, and will compute what enters, what stays, and what leaves ED. !
      ! They are averaged over FRQSTATE, so the units are now per unit of     !
      ! time for all fluxes, and the total storage at the beginning of the    !
      ! next time step (or the final storage between the previous analysis    !
      ! and this one).                                                        !
      !=======================================================================!
      ! Mean moisture transfer from the canopy air to the atmosphere 
      ! (kg_H2O/m2/s)
      real , pointer,dimension(:) :: wbudget_loss2atm

      ! Contribution of density change on change in the final storage
      ! (kg_H2O/m2/s)
      real , pointer,dimension(:) :: wbudget_denseffect

      ! Precipitation [kg_H2O/m2/s]
      real , pointer,dimension(:) :: wbudget_precipgain

      ! Mean runoff (kg_H2O/m2/s)
      real , pointer,dimension(:) :: wbudget_loss2runoff

      ! Mean drainage (kg_H2O/m2/s)
      real , pointer,dimension(:) :: wbudget_loss2drainage

      ! Total water (soil, sfcwater, can_shv, veg_water) at the beginning
      ! of budget-averaging time. [kg_H2O/m2]
      real , pointer,dimension(:) :: wbudget_initialstorage

      ! Residual of the water budget (kg_H2O/m2/s)
      real , pointer,dimension(:) :: wbudget_residual

      ! Mean sensible heat transfer from the canopy air to the atmosphere
      ! (J/m2/s)
      real , pointer,dimension(:) :: ebudget_loss2atm

      ! Mean change in storage due to change in density
      ! (J/m2/s)
      real , pointer,dimension(:) :: ebudget_denseffect

      ! Mean change in storage due to pressure change
      ! (J/m2/s)
      real , pointer,dimension(:) :: ebudget_prsseffect

      ! Energy associated with runoff (J/m2/s)
      real , pointer,dimension(:) :: ebudget_loss2runoff

      ! Energy associated with drainage (J/m2/s)
      real , pointer,dimension(:) :: ebudget_loss2drainage

      ! Net absorbed radiation by soil, sfcwater, vegetation [J/m2/s]
      real , pointer,dimension(:) :: ebudget_netrad

      ! Energy associated with precipitation (J/m2/s)
      real , pointer,dimension(:) :: ebudget_precipgain

      ! Total energy (soil, sfcwater, can_shv, veg_water) at the beginning
      ! of budget-averaging time. [J/m2]
      real , pointer,dimension(:) :: ebudget_initialstorage

      ! Residual of the energy budget [J/m2/s]
      real, pointer, dimension(:) :: ebudget_residual

      ! Total CO2 (can_shv) at the beginning of budget-averaging 
      ! time. [umol_CO2/m2]
      real , pointer,dimension(:) :: co2budget_initialstorage

      ! Residual of the CO2 budget [umol_CO2/m2/s]
      real , pointer,dimension(:) :: co2budget_residual

      ! Flux of CO2 from the canopy air to the atmosphere [umol_CO2/m2/s]
      real , pointer,dimension(:) :: co2budget_loss2atm

      ! Change in CO2 total storage due to change in density [umol_CO2/m2/s]
      real , pointer,dimension(:) :: co2budget_denseffect

      ! Average GPP [umol_CO2/m2/s]
      real , pointer,dimension(:) :: co2budget_gpp

      ! Average plant respiration [umol_CO2/m2/s]
      real , pointer,dimension(:) :: co2budget_plresp

      ! Average heterotrophic respiration [umol_CO2/m2/s]
      real , pointer,dimension(:) :: co2budget_rh

      ! Daily average of A_decomp, the temperature and moisture dependence
      ! of heterotrophic respiration.  The "today" variable is used in the
      ! model, whereas dmean and mmean are for output only.
      real , pointer,dimension(:) :: today_A_decomp

      ! Daily average of the product A_decomp * f_decomp, which incorporates
      ! temperature, moisture, and N dependence of decomposition.  The "today" 
      ! variable is used in the model, whereas dmean and mmean are for output only.
      real , pointer,dimension(:) :: today_Af_decomp

      ! Carbon available to establish recruits [kgC/m2]
      real, pointer,dimension(:,:) :: repro    !(n_pft,npatches)  

      ! Vegetation roughness length (m)
      real , pointer,dimension(:) :: veg_rough

      ! Vegetation height (m)
      real , pointer,dimension(:) :: veg_height

      ! 0-plane displacement height (m)
      real , pointer,dimension(:) :: veg_displace

      ! Input to fast soil carbon pool [kgC/m2/day]
      real , pointer,dimension(:) :: fsc_in

      ! Input to structural soil carbon pool [kgC/m2/day]
      real , pointer,dimension(:) :: ssc_in

      ! Input to soil lignin pool [kgC/m2/day]
      real , pointer,dimension(:) :: ssl_in  

      ! Input to fast soil nitrogen pool [kgN/m2/day]
      real , pointer,dimension(:) :: fsn_in  

      ! Plant nitrogen update summed over all cohorts [kgN/m2/day]
      real , pointer,dimension(:) :: total_plant_nitrogen_uptake

      !real, pointer,dimension(:) :: Nnet_min
      !real, pointer,dimension(:) :: Ngross_min
      real, pointer,dimension(:) :: mineralized_N_input
      real, pointer,dimension(:) :: mineralized_N_loss

      ! Short wave radiation absorbed by the ground (W/m2)
      real , pointer,dimension(:) :: rshort_g
      
      ! Short wave radiation absorbed by the ground, beam component (W/m2)
      real , pointer,dimension(:) :: rshort_g_beam
      
      ! Short wave radiation absorbed by the ground, diffuse component (W/m2)
      real , pointer,dimension(:) :: rshort_g_diffuse

      ! PAR absorbed by the ground (W/m2)
      real , pointer,dimension(:) :: par_g
      
      ! PAR absorbed by the ground, beam component (W/m2)
      real , pointer,dimension(:) :: par_g_beam
      
      ! PAR absorbed by the ground, diffuse component (W/m2)
      real , pointer,dimension(:) :: par_g_diffuse

      ! Photosynthetically active radiation incident at the ground(W/m2)
      real , pointer,dimension(:) :: par_b

      ! Photosynthetically active radiation incident at the ground, beam component (W/m2)
      real , pointer,dimension(:) :: par_b_beam

      ! Photosynthetically active radiation incident at the ground, diffuse component (W/m2)
      real , pointer,dimension(:) :: par_b_diffuse

      ! Near infrared radiation incident at the ground(W/m2)
      real , pointer,dimension(:) :: nir_b

      ! Near infrared radiation incident at the ground, beam component (W/m2)
      real , pointer,dimension(:) :: nir_b_beam

      ! Near infrared radiation incident at the ground, diffuse component (W/m2)
      real , pointer,dimension(:) :: nir_b_diffuse

      ! Long wave radiation absorbed by the ground (W/m2)
      real , pointer,dimension(:) :: rlong_g

      ! Long wave radiation absorbed by the surface water (W/m2)
      real , pointer,dimension(:) :: rlong_s

      ! Patch albedo
      real , pointer,dimension(:) :: albedo

      ! Patch albedo, PAR component
      real , pointer,dimension(:) :: albedo_par

      ! Patch albedo, NIR component
      real , pointer,dimension(:) :: albedo_nir

      ! Albedo for long wave radiation (whatever it means...)
      real , pointer,dimension(:) :: rlong_albedo

      ! Net radiation at the top of the canopy (W/m2)
      real , pointer,dimension(:) :: rnet

      ! Upward long wave radiation at the top of the canopy (W/m2)
      real , pointer,dimension(:) :: rlongup

      ! Upward PAR radiation at the top of the canopy (W/m2)
      real , pointer,dimension(:) :: parup

      ! Upward NIR radiation at the top of the canopy (W/m2)
      real , pointer,dimension(:) :: nirup

      ! Upward short wave radiation at the top of the canopy (W/m2)
      real , pointer,dimension(:) :: rshortup

      ! Total snow depth as calculated in the radiation scheme.  Used for 
      ! checking if cohorts are buried.  [m]
      real , pointer,dimension(:) :: total_sfcw_depth

      ! Fraction of vegetation covered with snow.  Used for computing 
      ! surface roughness.
      real , pointer,dimension(:) :: snowfac

      ! limitation of heterotrophic respiration due to physical 
      ! environmental factors (0-1 coefficient)
      real , pointer,dimension(:) :: A_decomp

      ! damping of decomposition due to nitrogen immobilization 
      ! (0-1 coefficient)
      real , pointer,dimension(:) :: f_decomp

      ! total heterotrophic respiration (umol/m2/s)
      real , pointer,dimension(:) :: rh

      ! coarse woody debris contribution to rh (umol/m2/s)
      real , pointer,dimension(:) :: cwd_rh

      ! Plant density broken down into size and PFT bins.  Used in patch fusion
      real, pointer,dimension(:,:,:) :: cumlai_profile !(n_pft,ff_nhgt,npatches)

      ! Above ground biomass in this patch [kgC/m2]
      real , pointer,dimension(:) :: plant_ag_biomass

      ! Last time step successfully completed by integrator.
      real, pointer,dimension(:)  :: htry

      ! Last previous time step successfully completed by the integrator
      real, pointer,dimension(:)  :: hprev

      ! Friction velocity [m/s]
      real, pointer,dimension(:)  :: ustar 

      ! Characteristic temperature gradient [K]
      real, pointer,dimension(:)  :: tstar 

      ! Characteristic specific humidity gradient [kg/kg]
      real, pointer,dimension(:)  :: qstar 

      ! Characteristic CO2 mixing ratio gradient [umol/mol]
      real, pointer,dimension(:)  :: cstar 

      ! Height / Obukhov length
      real, pointer,dimension(:)  :: zeta

      ! Bulk Richardson number
      real, pointer,dimension(:)  :: ribulk 

      real, pointer,dimension(:)  :: upwp !eddy mom. flux u-prime w-prime

      real, pointer,dimension(:)  :: tpwp !eddy heat flux t-prime w-prime

      real, pointer,dimension(:)  :: qpwp !eddy moist. flux q-prime w-prime

      real, pointer,dimension(:)  :: cpwp !eddy CO2 flux flux c-prime w-prime

      real, pointer,dimension(:)  :: wpwp !eddy v. mom. flux w-prime w-prime

      !----- Hydrology variables ----------------------------------------------------------!
      real,pointer,dimension(:)   :: watertable
      real,pointer,dimension(:)   :: moist_dz
      real,pointer,dimension(:)   :: ksat
      real,pointer,dimension(:)   :: soil_sat_energy
      real,pointer,dimension(:)   :: soil_sat_water
      real,pointer,dimension(:)   :: soil_sat_heat
      real,pointer,dimension(:,:) :: runoff_A ! Runoff parameters, 1st dimension is 3.
      real,pointer,dimension(:)   :: runoff_rate
      real,pointer,dimension(:)   :: runoff
      real,pointer,dimension(:)   :: qrunoff

      !====================================================================================!
      !====================================================================================!
      !     Patch-level diagnostic variables:  these are either fluxes or state variables, !
      ! averaged over different scales, for each patch.                                    !
      !                                                                                    !
      !   FMEAN -- average over one frqsum time interval (usually sub-daily)               !
      !   DMEAN -- daily mean                                                              !
      !   MMEAN -- monthly mean                                                            !
      !   MMSQU -- monthly mean sum of squares                                             !
      !   QMEAN -- mean diel                                                               !
      !   QMSQU -- mean sum of squares for the diel                                        !
      !                                                                                    !
      !   Acronyms used in the description below:                                          !
      !   ATM       -- air above canopy                                                    !
      !   CAS       -- canopy air space                                                    !
      !   TSPL      -- temporary snow/pounding layer                                       !
      !   PAR       -- photosynthetically active radiation                                 !
      !   NIR       -- near infra-red                                                      !
      !   SW        -- short wave                                                          !
      !   LW        -- long wave                                                           !
      !   VPD       -- Vapour pressure deficit                                             !
      !   Theta_Eiv -- ice vapour equivalent potential temperature                         !
      !------------------------------------------------------------------------------------!
      !----- Photosynthesis/Decomposition. ------------------------------------------------!
      real,pointer,dimension(:)   :: fmean_rh              ! Heterotr. resp.    [umol/m2/s]
      real,pointer,dimension(:)   :: fmean_cwd_rh          ! CWD respiration    [umol/m2/s]
      real,pointer,dimension(:)   :: fmean_nep             ! Net Ecosyst. Prod. [umol/m2/s]
      !----- State variables. -------------------------------------------------------------!
      real,pointer,dimension(:)   :: fmean_rk4step         ! RK4 time step      [        s]
      real,pointer,dimension(:)   :: fmean_available_water ! Available water    [    kg/m2]
      real,pointer,dimension(:)   :: fmean_can_theiv       ! CAS Theta_Eiv      [        K]
      real,pointer,dimension(:)   :: fmean_can_theta       ! CAS Potential temp.[        K]
      real,pointer,dimension(:)   :: fmean_can_vpdef       ! CAS Vap. Pres. Def.[       Pa]
      real,pointer,dimension(:)   :: fmean_can_temp        ! CAS Temperature    [        K]
      real,pointer,dimension(:)   :: fmean_can_shv         ! CAS Specific hum.  [    kg/kg]
      real,pointer,dimension(:)   :: fmean_can_co2         ! CAS CO2 mix. ratio [ umol/mol]
      real,pointer,dimension(:)   :: fmean_can_rhos        ! CAS air density    [    kg/m3]
      real,pointer,dimension(:)   :: fmean_can_prss        ! CAS pressure       [       Pa]
      real,pointer,dimension(:)   :: fmean_gnd_temp        ! Ground temperature [        K]
      real,pointer,dimension(:)   :: fmean_gnd_shv         ! Ground spec. hum.  [    kg/kg]
      real,pointer,dimension(:)   :: fmean_can_ggnd        ! Net canopy conduct.[      m/2]
      real,pointer,dimension(:)   :: fmean_sfcw_depth      ! TPSL depth         [        m]
      real,pointer,dimension(:)   :: fmean_sfcw_energy     ! TPSL int. energy   [     J/kg]
      real,pointer,dimension(:)   :: fmean_sfcw_mass       ! TPSL water mass    [    kg/m2]
      real,pointer,dimension(:)   :: fmean_sfcw_temp       ! TPSL temperature   [        K]
      real,pointer,dimension(:)   :: fmean_sfcw_fliq       ! TPSL liquid frac.  [       --]
      real,pointer,dimension(:,:) :: fmean_soil_energy     ! Soil int. energy   [     J/m3]
      real,pointer,dimension(:,:) :: fmean_soil_mstpot     ! Soil matric potl.  [        m]
      real,pointer,dimension(:,:) :: fmean_soil_water      ! Soil water content [    m3/m3]
      real,pointer,dimension(:,:) :: fmean_soil_temp       ! Soil temperature   [        K]
      real,pointer,dimension(:,:) :: fmean_soil_fliq       ! Soil liquid frac.  [       --]
      !----- Radiation --------------------------------------------------------------------!
      real,pointer,dimension(:)   :: fmean_rshort_gnd   ! Gnd. absp. SW Rad.    [     W/m2]
      real,pointer,dimension(:)   :: fmean_par_gnd      ! Absorbed PAR          [     W/m2]
      real,pointer,dimension(:)   :: fmean_rlong_gnd    ! Gnd. absp. LW Rad.    [     W/m2]
      real,pointer,dimension(:)   :: fmean_rlongup      ! Outgoing LW Rad.      [     W/m2]
      real,pointer,dimension(:)   :: fmean_parup        ! Outgoing PAR          [     W/m2]
      real,pointer,dimension(:)   :: fmean_nirup        ! Outgoing NIR          [     W/m2]
      real,pointer,dimension(:)   :: fmean_rshortup     ! Outgoing SW Rad.      [     W/m2]
      real,pointer,dimension(:)   :: fmean_rnet         ! Net radiation         [     W/m2]
      real,pointer,dimension(:)   :: fmean_albedo       ! Albedo                [     ----]
      real,pointer,dimension(:)   :: fmean_albedo_par   ! Direct Albedo         [     ----]
      real,pointer,dimension(:)   :: fmean_albedo_nir   ! Diffuse Albedo        [     ----]
      real,pointer,dimension(:)   :: fmean_rlong_albedo ! Longwave Albedo       [     ----]
      !----- Characteristic variables. ----------------------------------------------------!
      real,pointer,dimension(:)   :: fmean_ustar        ! Friction velocity     [      m/s]
      real,pointer,dimension(:)   :: fmean_tstar        ! Pot. temp. scale      [        K]
      real,pointer,dimension(:)   :: fmean_qstar        ! Water vapour scale    [    kg/kg]
      real,pointer,dimension(:)   :: fmean_cstar        ! CO2 scale             [ umol/mol]
      real,pointer,dimension(:)   :: fmean_carbon_ac    ! CO2 flux, ATM->CAS    [umol/m2/s]
      real,pointer,dimension(:)   :: fmean_carbon_st    ! CO2 storage           [umol/m2/s]
      real,pointer,dimension(:)   :: fmean_vapor_gc     ! Water flux, Gnd->CAS  [  kg/m2/s]
      real,pointer,dimension(:)   :: fmean_vapor_ac     ! Water flux, Atm->CAS  [  kg/m2/s]
      real,pointer,dimension(:,:) :: fmean_smoist_gg    ! Soil water flux       [  kg/m2/s]
      real,pointer,dimension(:)   :: fmean_throughfall  ! Throughfall rate      [  kg/m2/s]
      real,pointer,dimension(:,:) :: fmean_transloss    ! Water loss (transp.)  [  kg/m2/s]
      real,pointer,dimension(:)   :: fmean_runoff       ! Water runoff          [  kg/m2/s]
      real,pointer,dimension(:)   :: fmean_drainage     ! Water drainage        [  kg/m2/s]
      !----- Sensible heat ----------------------------------------------------------------!
      real,pointer,dimension(:)   :: fmean_sensible_gc  ! Sens. heat, GND->CAS  [     W/m2]
      real,pointer,dimension(:)   :: fmean_sensible_ac  ! Sens. heat, ATM->CAS  [     W/m2]
      real,pointer,dimension(:,:) :: fmean_sensible_gg  ! Soil heat flux        [     W/m2]
      real,pointer,dimension(:)   :: fmean_qthroughfall ! Throughfall rate      [     W/m2]
      real,pointer,dimension(:)   :: fmean_qrunoff      ! Surface runoff        [     W/m2]
      real,pointer,dimension(:)   :: fmean_qdrainage    ! Drainage energy loss  [     W/m2]
      !----- Variables with no sub-daily means. -------------------------------------------!
      real,pointer,dimension(:) :: dmean_A_decomp         ! Decomposition fact. [       --]
      real,pointer,dimension(:) :: dmean_Af_decomp        ! A with N immobil.   [       --]
      real,pointer,dimension(:) :: dmean_co2_residual     ! CO2 residual        [  umol/m2]
      real,pointer,dimension(:) :: dmean_energy_residual  ! Enthalpy residual   [     J/m2]
      real,pointer,dimension(:) :: dmean_water_residual   ! Water residual      [    kg/m2]
      !----- Variables that are updated once a day. ---------------------------------------!
      real,pointer,dimension(:) :: mmean_fast_soil_c    ! Fast soil carbon      [   kgC/m2]
      real,pointer,dimension(:) :: mmean_slow_soil_c    ! Slow soil carbon      [   kgC/m2]
      real,pointer,dimension(:) :: mmean_struct_soil_c  ! Struct. soil carbon   [   kgC/m2]
      real,pointer,dimension(:) :: mmean_struct_soil_l  ! Struct. soil lignin   [   kgL/m2]
      real,pointer,dimension(:) :: mmean_fast_soil_n    ! Fast soil nitrogen    [   kgN/m2]
      real,pointer,dimension(:) :: mmean_mineral_soil_n ! Mineralised soil N    [   kgN/m2]
      !----- Daily means.  Units are the same as the "avg" variables. ---------------------!
      real,pointer,dimension(:)     :: dmean_rh
      real,pointer,dimension(:)     :: dmean_cwd_rh
      real,pointer,dimension(:)     :: dmean_nep
      real,pointer,dimension(:)     :: dmean_rk4step
      real,pointer,dimension(:)     :: dmean_available_water
      real,pointer,dimension(:)     :: dmean_can_theiv
      real,pointer,dimension(:)     :: dmean_can_theta
      real,pointer,dimension(:)     :: dmean_can_vpdef
      real,pointer,dimension(:)     :: dmean_can_temp
      real,pointer,dimension(:)     :: dmean_can_shv
      real,pointer,dimension(:)     :: dmean_can_co2
      real,pointer,dimension(:)     :: dmean_can_rhos
      real,pointer,dimension(:)     :: dmean_can_prss
      real,pointer,dimension(:)     :: dmean_gnd_temp
      real,pointer,dimension(:)     :: dmean_gnd_shv
      real,pointer,dimension(:)     :: dmean_can_ggnd
      real,pointer,dimension(:)     :: dmean_sfcw_depth
      real,pointer,dimension(:)     :: dmean_sfcw_energy
      real,pointer,dimension(:)     :: dmean_sfcw_mass
      real,pointer,dimension(:)     :: dmean_sfcw_temp
      real,pointer,dimension(:)     :: dmean_sfcw_fliq
      real,pointer,dimension(:,:)   :: dmean_soil_energy
      real,pointer,dimension(:,:)   :: dmean_soil_mstpot
      real,pointer,dimension(:,:)   :: dmean_soil_water
      real,pointer,dimension(:,:)   :: dmean_soil_temp
      real,pointer,dimension(:,:)   :: dmean_soil_fliq
      real,pointer,dimension(:)     :: dmean_rshort_gnd
      real,pointer,dimension(:)     :: dmean_par_gnd
      real,pointer,dimension(:)     :: dmean_rlong_gnd
      real,pointer,dimension(:)     :: dmean_rlongup
      real,pointer,dimension(:)     :: dmean_parup
      real,pointer,dimension(:)     :: dmean_nirup
      real,pointer,dimension(:)     :: dmean_rshortup
      real,pointer,dimension(:)     :: dmean_rnet
      real,pointer,dimension(:)     :: dmean_albedo
      real,pointer,dimension(:)     :: dmean_albedo_par 
      real,pointer,dimension(:)     :: dmean_albedo_nir 
      real,pointer,dimension(:)     :: dmean_rlong_albedo
      real,pointer,dimension(:)     :: dmean_ustar
      real,pointer,dimension(:)     :: dmean_tstar
      real,pointer,dimension(:)     :: dmean_qstar
      real,pointer,dimension(:)     :: dmean_cstar
      real,pointer,dimension(:)     :: dmean_carbon_ac
      real,pointer,dimension(:)     :: dmean_carbon_st
      real,pointer,dimension(:)     :: dmean_vapor_gc
      real,pointer,dimension(:)     :: dmean_vapor_ac
      real,pointer,dimension(:,:)   :: dmean_smoist_gg
      real,pointer,dimension(:)     :: dmean_throughfall
      real,pointer,dimension(:,:)   :: dmean_transloss
      real,pointer,dimension(:)     :: dmean_runoff
      real,pointer,dimension(:)     :: dmean_drainage
      real,pointer,dimension(:)     :: dmean_sensible_gc
      real,pointer,dimension(:)     :: dmean_sensible_ac
      real,pointer,dimension(:,:)   :: dmean_sensible_gg
      real,pointer,dimension(:)     :: dmean_qthroughfall
      real,pointer,dimension(:)     :: dmean_qrunoff
      real,pointer,dimension(:)     :: dmean_qdrainage
      !----- Monthly means.  Units are the same as the "avg" variables. --------------------!
      real,pointer,dimension(:)     :: mmean_rh
      real,pointer,dimension(:)     :: mmean_cwd_rh
      real,pointer,dimension(:)     :: mmean_nep
      real,pointer,dimension(:)     :: mmean_rk4step
      real,pointer,dimension(:)     :: mmean_available_water
      real,pointer,dimension(:)     :: mmean_can_theiv
      real,pointer,dimension(:)     :: mmean_can_theta
      real,pointer,dimension(:)     :: mmean_can_vpdef
      real,pointer,dimension(:)     :: mmean_can_temp
      real,pointer,dimension(:)     :: mmean_can_shv
      real,pointer,dimension(:)     :: mmean_can_co2
      real,pointer,dimension(:)     :: mmean_can_rhos
      real,pointer,dimension(:)     :: mmean_can_prss
      real,pointer,dimension(:)     :: mmean_gnd_temp
      real,pointer,dimension(:)     :: mmean_gnd_shv
      real,pointer,dimension(:)     :: mmean_can_ggnd
      real,pointer,dimension(:)     :: mmean_sfcw_depth
      real,pointer,dimension(:)     :: mmean_sfcw_energy
      real,pointer,dimension(:)     :: mmean_sfcw_mass
      real,pointer,dimension(:)     :: mmean_sfcw_temp
      real,pointer,dimension(:)     :: mmean_sfcw_fliq
      real,pointer,dimension(:,:)   :: mmean_soil_energy
      real,pointer,dimension(:,:)   :: mmean_soil_mstpot
      real,pointer,dimension(:,:)   :: mmean_soil_water
      real,pointer,dimension(:,:)   :: mmean_soil_temp
      real,pointer,dimension(:,:)   :: mmean_soil_fliq
      real,pointer,dimension(:)     :: mmean_rshort_gnd
      real,pointer,dimension(:)     :: mmean_par_gnd
      real,pointer,dimension(:)     :: mmean_rlong_gnd
      real,pointer,dimension(:)     :: mmean_rlongup
      real,pointer,dimension(:)     :: mmean_parup
      real,pointer,dimension(:)     :: mmean_nirup
      real,pointer,dimension(:)     :: mmean_rshortup
      real,pointer,dimension(:)     :: mmean_rnet
      real,pointer,dimension(:)     :: mmean_albedo
      real,pointer,dimension(:)     :: mmean_albedo_par 
      real,pointer,dimension(:)     :: mmean_albedo_nir 
      real,pointer,dimension(:)     :: mmean_rlong_albedo
      real,pointer,dimension(:)     :: mmean_ustar
      real,pointer,dimension(:)     :: mmean_tstar
      real,pointer,dimension(:)     :: mmean_qstar
      real,pointer,dimension(:)     :: mmean_cstar
      real,pointer,dimension(:)     :: mmean_carbon_ac
      real,pointer,dimension(:)     :: mmean_carbon_st
      real,pointer,dimension(:)     :: mmean_vapor_gc
      real,pointer,dimension(:)     :: mmean_vapor_ac
      real,pointer,dimension(:,:)   :: mmean_smoist_gg
      real,pointer,dimension(:)     :: mmean_throughfall
      real,pointer,dimension(:,:)   :: mmean_transloss
      real,pointer,dimension(:)     :: mmean_runoff
      real,pointer,dimension(:)     :: mmean_drainage
      real,pointer,dimension(:)     :: mmean_sensible_gc
      real,pointer,dimension(:)     :: mmean_sensible_ac
      real,pointer,dimension(:,:)   :: mmean_sensible_gg
      real,pointer,dimension(:)     :: mmean_qthroughfall
      real,pointer,dimension(:)     :: mmean_qrunoff
      real,pointer,dimension(:)     :: mmean_qdrainage
      real,pointer,dimension(:)     :: mmean_A_decomp
      real,pointer,dimension(:)     :: mmean_Af_decomp
      real,pointer,dimension(:)     :: mmean_co2_residual
      real,pointer,dimension(:)     :: mmean_energy_residual
      real,pointer,dimension(:)     :: mmean_water_residual
      !----- Monthly means sum of squares.  Units are the square of "avg" variables. -------!
      real,pointer,dimension(:)     :: mmsqu_rh
      real,pointer,dimension(:)     :: mmsqu_cwd_rh
      real,pointer,dimension(:)     :: mmsqu_nep
      real,pointer,dimension(:)     :: mmsqu_rlongup
      real,pointer,dimension(:)     :: mmsqu_parup
      real,pointer,dimension(:)     :: mmsqu_nirup
      real,pointer,dimension(:)     :: mmsqu_rshortup
      real,pointer,dimension(:)     :: mmsqu_rnet
      real,pointer,dimension(:)     :: mmsqu_albedo
      real,pointer,dimension(:)     :: mmsqu_ustar
      real,pointer,dimension(:)     :: mmsqu_carbon_ac
      real,pointer,dimension(:)     :: mmsqu_carbon_st
      real,pointer,dimension(:)     :: mmsqu_vapor_gc
      real,pointer,dimension(:)     :: mmsqu_vapor_ac
      real,pointer,dimension(:)     :: mmsqu_transp
      real,pointer,dimension(:)     :: mmsqu_sensible_gc
      real,pointer,dimension(:)     :: mmsqu_sensible_ac
      !----- Mean diel.  Units are the same as the "avg" variables. ------------------------!
      real,pointer,dimension(:,:)   :: qmean_rh
      real,pointer,dimension(:,:)   :: qmean_cwd_rh
      real,pointer,dimension(:,:)   :: qmean_nep
      real,pointer,dimension(:,:)   :: qmean_rk4step
      real,pointer,dimension(:,:)   :: qmean_available_water
      real,pointer,dimension(:,:)   :: qmean_can_theiv
      real,pointer,dimension(:,:)   :: qmean_can_theta
      real,pointer,dimension(:,:)   :: qmean_can_vpdef
      real,pointer,dimension(:,:)   :: qmean_can_temp
      real,pointer,dimension(:,:)   :: qmean_can_shv
      real,pointer,dimension(:,:)   :: qmean_can_co2
      real,pointer,dimension(:,:)   :: qmean_can_rhos
      real,pointer,dimension(:,:)   :: qmean_can_prss
      real,pointer,dimension(:,:)   :: qmean_gnd_temp
      real,pointer,dimension(:,:)   :: qmean_gnd_shv
      real,pointer,dimension(:,:)   :: qmean_can_ggnd
      real,pointer,dimension(:,:)   :: qmean_sfcw_depth
      real,pointer,dimension(:,:)   :: qmean_sfcw_energy
      real,pointer,dimension(:,:)   :: qmean_sfcw_mass
      real,pointer,dimension(:,:)   :: qmean_sfcw_temp
      real,pointer,dimension(:,:)   :: qmean_sfcw_fliq
      real,pointer,dimension(:,:,:) :: qmean_soil_energy
      real,pointer,dimension(:,:,:) :: qmean_soil_mstpot
      real,pointer,dimension(:,:,:) :: qmean_soil_water
      real,pointer,dimension(:,:,:) :: qmean_soil_temp
      real,pointer,dimension(:,:,:) :: qmean_soil_fliq
      real,pointer,dimension(:,:)   :: qmean_rshort_gnd
      real,pointer,dimension(:,:)   :: qmean_par_gnd
      real,pointer,dimension(:,:)   :: qmean_rlong_gnd
      real,pointer,dimension(:,:)   :: qmean_rlongup
      real,pointer,dimension(:,:)   :: qmean_parup
      real,pointer,dimension(:,:)   :: qmean_nirup
      real,pointer,dimension(:,:)   :: qmean_rshortup
      real,pointer,dimension(:,:)   :: qmean_rnet
      real,pointer,dimension(:,:)   :: qmean_albedo
      real,pointer,dimension(:,:)   :: qmean_albedo_par 
      real,pointer,dimension(:,:)   :: qmean_albedo_nir 
      real,pointer,dimension(:,:)   :: qmean_rlong_albedo
      real,pointer,dimension(:,:)   :: qmean_ustar
      real,pointer,dimension(:,:)   :: qmean_tstar
      real,pointer,dimension(:,:)   :: qmean_qstar
      real,pointer,dimension(:,:)   :: qmean_cstar
      real,pointer,dimension(:,:)   :: qmean_carbon_ac
      real,pointer,dimension(:,:)   :: qmean_carbon_st
      real,pointer,dimension(:,:)   :: qmean_vapor_gc
      real,pointer,dimension(:,:)   :: qmean_vapor_ac
      real,pointer,dimension(:,:,:) :: qmean_smoist_gg
      real,pointer,dimension(:,:)   :: qmean_throughfall
      real,pointer,dimension(:,:,:) :: qmean_transloss
      real,pointer,dimension(:,:)   :: qmean_runoff
      real,pointer,dimension(:,:)   :: qmean_drainage
      real,pointer,dimension(:,:)   :: qmean_sensible_gc
      real,pointer,dimension(:,:)   :: qmean_sensible_ac
      real,pointer,dimension(:,:,:) :: qmean_sensible_gg
      real,pointer,dimension(:,:)   :: qmean_qthroughfall
      real,pointer,dimension(:,:)   :: qmean_qrunoff
      real,pointer,dimension(:,:)   :: qmean_qdrainage
      !----- Means diel of the sum of squares.  Units are the square of "avg" variables. ---!
      real,pointer,dimension(:,:)   :: qmsqu_rh
      real,pointer,dimension(:,:)   :: qmsqu_cwd_rh
      real,pointer,dimension(:,:)   :: qmsqu_nep
      real,pointer,dimension(:,:)   :: qmsqu_rlongup
      real,pointer,dimension(:,:)   :: qmsqu_parup
      real,pointer,dimension(:,:)   :: qmsqu_nirup
      real,pointer,dimension(:,:)   :: qmsqu_rshortup
      real,pointer,dimension(:,:)   :: qmsqu_rnet
      real,pointer,dimension(:,:)   :: qmsqu_albedo
      real,pointer,dimension(:,:)   :: qmsqu_ustar
      real,pointer,dimension(:,:)   :: qmsqu_carbon_ac
      real,pointer,dimension(:,:)   :: qmsqu_carbon_st
      real,pointer,dimension(:,:)   :: qmsqu_vapor_gc
      real,pointer,dimension(:,:)   :: qmsqu_vapor_ac
      real,pointer,dimension(:,:)   :: qmsqu_transp
      real,pointer,dimension(:,:)   :: qmsqu_sensible_gc
      real,pointer,dimension(:,:)   :: qmsqu_sensible_ac
      !====================================================================================!
      !====================================================================================!
   end type sitetype
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   ! POLYGON TYPE: The following are the arrays of site level variables that populate the  !
   !               current polygon.                                                        !
   !---------------------------------------------------------------------------------------!
   type polygontype
      !----- Number of sites in this polygon. ---------------------------------------------!
      integer :: nsites
      !------------------------------------------------------------------------------------!

      ! Global index of the first site in this vector across all sites
      ! on the grid
      integer :: siglob_id     ! Global index of the first patch across all cohorts

      ! Pointer to the patch vectors
      type(sitetype),pointer,dimension(:) :: site

      !  The global index of the first patch in all each site
      integer,pointer,dimension(:) :: sipa_id

      ! The number of patches in each site
      integer,pointer,dimension(:) :: sipa_n

      integer,pointer,dimension(:) :: patch_count    ! number of patches per site

      integer,pointer,dimension(:) :: sitenum


      !------------------------------------------------------------------------------------!
      !     METEOROLOGICAL INFORMATION.                                                    !
      !------------------------------------------------------------------------------------!
      type(met_driv_state),pointer,dimension(:) :: met
      !------------------------------------------------------------------------------------!




      !------------------------------------------------------------------------------------!
      ! BASIC INFO
      !------------------------------------------------------------------------------------!
      real,pointer,dimension(:) :: area       ! Relative area occupied by the site
      real,pointer,dimension(:) :: patch_area ! Un-normalized sum of patch areas
      real,pointer,dimension(:) :: elevation  ! mean site elevation (meters)
      real,pointer,dimension(:) :: slope      ! mean site slope (degrees)
      real,pointer,dimension(:) :: aspect     ! mean site aspect (degrees)
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !     Soil properties.                                                               !
      ! LSL        -- Index of the lowest soil layer (simulation dependent).
      !                                                                                    !
      ! NCOL_SOIL  -- Index of the soil colour class (used to find the ground albedo)
      !       |-----------------------------------------------------------------------|    !
      !       |       |   Dry soil  |  Saturated  |       |   Dry soil  |  Saturated  |    !
      !       | Class |-------------+-------------| Class +-------------+-------------|    !
      !       |       |  VIS |  NIR |  VIS |  NIR |       |  VIS |  NIR |  VIS |  NIR |    !
      !       |-------+------+------+------+------+-------+------+------+------+------|    !
      !       |     1 | 0.36 | 0.61 | 0.25 | 0.50 |    11 | 0.24 | 0.37 | 0.13 | 0.26 |    !
      !       |     2 | 0.34 | 0.57 | 0.23 | 0.46 |    12 | 0.23 | 0.35 | 0.12 | 0.24 |    !
      !       |     3 | 0.32 | 0.53 | 0.21 | 0.42 |    13 | 0.22 | 0.33 | 0.11 | 0.22 |    !
      !       |     4 | 0.31 | 0.51 | 0.20 | 0.40 |    14 | 0.20 | 0.31 | 0.10 | 0.20 |    !
      !       |     5 | 0.30 | 0.49 | 0.19 | 0.38 |    15 | 0.18 | 0.29 | 0.09 | 0.18 |    !
      !       |     6 | 0.29 | 0.48 | 0.18 | 0.36 |    16 | 0.16 | 0.27 | 0.08 | 0.16 |    !
      !       |     7 | 0.28 | 0.45 | 0.17 | 0.34 |    17 | 0.14 | 0.25 | 0.07 | 0.14 |    !
      !       |     8 | 0.27 | 0.43 | 0.16 | 0.32 |    18 | 0.12 | 0.23 | 0.06 | 0.12 |    !
      !       |     9 | 0.26 | 0.41 | 0.15 | 0.30 |    19 | 0.10 | 0.21 | 0.05 | 0.10 |    !
      !       |    10 | 0.25 | 0.39 | 0.14 | 0.28 |    20 | 0.08 | 0.16 | 0.04 | 0.08 |    !
      !       |-----------------------------------------------------------------------|    !
      !                                                                                    !
      !   Soil type 21 is a special case in which we use the albedo method that used to be !
      ! the default in ED-2.1.                                                             !
      !                                                                                    !
      !                                                                                    !
      ! NTEXT_SOIL -- Index of the soil texture class, similar to the USDA classification  !
      !               except that clay is split into four classes (11,15,16,17).           !
      !                                                                                    !
      !   1 -- sand                |   7 -- silty clay loam     |  13 -- bedrock           !
      !   2 -- loamy sand          |   8 -- clayey loam         |  14 -- silt              !
      !   3 -- sandy loam          |   9 -- sandy clay          |  15 -- heavy clay        !
      !   4 -- silt loam           |  10 -- silty clay          |  16 -- clayey sand       !
      !   5 -- loam                |  11 -- clay                |  17 -- clayey silt       !
      !   6 -- sandy clay loam     |  12 -- peat                                           !
      !------------------------------------------------------------------------------------!
      integer,pointer,dimension(:)   :: lsl
      integer,pointer,dimension(:)   :: ncol_soil
      integer,pointer,dimension(:,:) :: ntext_soil
      !------------------------------------------------------------------------------------!




      !------------------------------------------------------------------------------------!
      !  hydrologic routing info
      !------------------------------------------------------------------------------------!
      real,pointer,dimension(:) :: TCI             ! topographic convergence index

      real,pointer,dimension(:) :: pptweight       ! lapse weight for precip

      ! The Following variables used to point to structures
      ! Now they point to the array index of the site
      !   type(site), pointer :: hydro_next  !site run-off goes to
      !   type(site), pointer :: hydro_prev  !site run-on comes from
      integer, pointer,dimension(:) :: hydro_next  !site run-off goes to
      integer, pointer,dimension(:) :: hydro_prev  !site run-on comes from
     
      real,pointer,dimension(:)  :: moist_W   !Wetness index
      real,pointer,dimension(:)  :: moist_f   ! decay of soil conductance w/ depth
      real,pointer,dimension(:)  :: moist_tau ! tau characteristic time scale for water redistribution
      real,pointer,dimension(:)  :: moist_zi  ! TOPMODEL "equilibrium" water table depth 
      real,pointer,dimension(:)  :: baseflow  ! loss of water from site to watershed discharge (kg/m2/s)

      real,pointer,dimension(:) :: runoff
      real,pointer,dimension(:) :: qrunoff




      !------------------------------------------------------------------------------------!
      ! Minimum daily-averaged temperature in this month -- determines whether
      ! or not recruits come up this month.
      !------------------------------------------------------------------------------------!
      real,pointer,dimension(:)  :: min_monthly_temp
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !     The number of years of landuse data calculated at read-in of data during       !
      ! landuse_init.                                                                      !
      !------------------------------------------------------------------------------------!
      integer     ,pointer ,dimension(:)    :: num_landuse_years
      !----- The vectorized landuse matrix is allocated in landuse_init. ------------------!
      type(lutime), pointer,dimension(:,:)  :: clutimes !(luyears,nsites)
      !----- Minimum DBH and probability of being harvest when selective logging happens. -!
      real        , pointer, dimension(:,:) :: mindbh_primary
      real        , pointer, dimension(:,:) :: probharv_primary
      real        , pointer, dimension(:,:) :: mindbh_secondary
      real        , pointer, dimension(:,:) :: probharv_secondary
      !------------------------------------------------------------------------------------!


      !-----------------------------------
      ! FORESTRY
      !-----------------------------------
      ! is the site under plantation management? (1=yes, 0=no)
      integer,pointer,dimension(:) :: plantation  ! initialized to zero in site creation

      ! Upon creating an agriculture patch in this site, stock it with this 
      ! PFT.  Set, along with the other stocking parameters, in 
      ! init_ed_site_vars().
      integer,pointer,dimension(:) :: agri_stocking_pft
     
      ! Upon creating an agriculture patch in this site, stock it with 
      ! this density of plants [plants/m2]
      real,pointer,dimension(:) :: agri_stocking_density
     
      ! Upon creating a plantation patch in this site, stock it with this PFT
      integer,pointer,dimension(:) :: plantation_stocking_pft
     
      ! Upon creating an plantation patch in this site, stock it with 
      ! this density of plants [plants/m2]
      real,pointer,dimension(:) :: plantation_stocking_density

      ! Target primary forest harvest for the current simulation year [kgC/m2].  
      ! Initialized together with secondary memory in init_ed_site_vars().
      real,pointer,dimension(:) :: primary_harvest_target
     
      ! Target secondary forest harvest for the current simulation year [kgC/m2].  
      ! Initialized together with secondary memory in init_ed_site_vars().
      real ,pointer,dimension(:):: secondary_harvest_target

      ! Unapplied primary forest harvest from previous years (save until 
      ! harvest is above minimum threshold.) [kgC/m2].  Initialized 
      ! together with secondary memory in init_ed_site_vars().
      real,pointer,dimension(:) :: primary_harvest_memory
     
      ! Unapplied secondary forest harvest from previous years (save until 
      ! harvest is above minimum threshold.) [kgC/m2]
      real ,pointer,dimension(:):: secondary_harvest_memory


      !-----------------------------------
      ! FIRE
      !-----------------------------------
      ! total fuel in the dry patches
      real,pointer,dimension(:) :: ignition_rate

      real,pointer, dimension(:,:) :: lambda_fire ! initialized in create_site !(12,nsites)
      ! Monthly rainfall [mm/month] for each month over the past 12 months.
      real,pointer,dimension(:,:)  :: avg_monthly_pcpg

      type(prescribed_phen),pointer, dimension(:) :: phen_pars

      !-----------------------------------
      ! DISTURBANCE
      !-----------------------------------

      ! if new patch is less than min size, store information in the memory
      real,pointer, dimension(:,:,:) :: disturbance_memory !(n_dist_types,n_dist_types,nsites)

      ! the disturbance matrix (to,from)
      real,pointer,dimension(:,:,:) :: disturbance_rates !(n_dist_types,n_dist_types,nsites)

      real,pointer,dimension(:,:) :: green_leaf_factor ! (n_pft,nsites)
      real,pointer,dimension(:,:) :: leaf_aging_factor ! (n_pft,nsites)

      real,pointer, dimension(:,:,:) :: basal_area        ! cm2/m2
      real,pointer, dimension(:,:,:) :: basal_area_growth ! cm2/m2/yr
      real,pointer, dimension(:,:,:) :: basal_area_mort   ! c2/m2/yr
      real,pointer, dimension(:,:,:) :: basal_area_cut    ! c2/m2/yr

      real,pointer, dimension(:,:,:) :: agb               ! kgC/m2
      real,pointer, dimension(:,:,:) :: agb_growth        ! kgC/m2/yr
      real,pointer, dimension(:,:,:) :: agb_mort          ! kgC/m2/yr
      real,pointer, dimension(:,:,:) :: agb_cut           ! kgC/m2/yr


      real,pointer,dimension(:) :: cosaoi

      !----- Length of day light, used to average light levels properly. -------------------!
      real   , pointer, dimension(:) :: daylight
      logical, pointer, dimension(:) :: nighttime
      
      !-----  Light-phenology variable.
      real, pointer, dimension(:) :: rad_avg

      !====================================================================================!
      !====================================================================================!
      !     Polygon-level diagnostic variables:  these are pretty much the meteorological  !
      ! forcing variables modified by each site configuration, averaged over different     !
      ! time scales, for each site.                                                        !
      !                                                                                    !
      !   FMEAN -- average over one frqsum time interval (usually sub-daily)               !
      !   DMEAN -- daily mean                                                              !
      !   MMEAN -- monthly mean                                                            !
      !   MMSQU -- monthly mean sum of squares                                             !
      !   QMEAN -- mean diel                                                               !
      !   QMSQU -- mean sum of squares for the diel                                        !
      !                                                                                    !
      !------------------------------------------------------------------------------------!
      !------ Fast mean. ------------------------------------------------------------------!
      real,pointer,dimension(:) :: fmean_atm_theiv       ! IV Equiv. Pot. Temp   [       K]
      real,pointer,dimension(:) :: fmean_atm_theta       ! Potential temperature [       K]
      real,pointer,dimension(:) :: fmean_atm_temp        ! Temperature           [       K]
      real,pointer,dimension(:) :: fmean_atm_vpdef       ! Vapour pres. deficit  [      Pa]
      real,pointer,dimension(:) :: fmean_atm_shv         ! Specific humidity     [   kg/kg]
      real,pointer,dimension(:) :: fmean_atm_rshort      ! Shortwave radiation   [    W/m2]
      real,pointer,dimension(:) :: fmean_atm_rshort_diff ! Diffuse SW radiation  [    W/m2]
      real,pointer,dimension(:) :: fmean_atm_par         ! Photosyn. Active Rad. [    W/m2]
      real,pointer,dimension(:) :: fmean_atm_par_diff    ! Diffuse PAR           [    W/m2]
      real,pointer,dimension(:) :: fmean_atm_rlong       ! Longwave radiation    [    W/m2]
      real,pointer,dimension(:) :: fmean_atm_vels        ! Wind speed            [     m/s]
      real,pointer,dimension(:) :: fmean_atm_rhos        ! Air density           [   kg/m3]
      real,pointer,dimension(:) :: fmean_atm_prss        ! Air pressure          [      Pa]
      real,pointer,dimension(:) :: fmean_atm_co2         ! Air CO2               [umol/mol]
      real,pointer,dimension(:) :: fmean_pcpg            ! Precipitation rate    [ kg/m2/s]
      real,pointer,dimension(:) :: fmean_qpcpg           ! Energy gain - rain    [    W/m2]
      real,pointer,dimension(:) :: fmean_dpcpg           ! Precipitation depth   [       m]
      !------ Daily mean. -----------------------------------------------------------------!
      real,pointer,dimension(:) :: dmean_atm_theiv
      real,pointer,dimension(:) :: dmean_atm_theta
      real,pointer,dimension(:) :: dmean_atm_temp
      real,pointer,dimension(:) :: dmean_atm_vpdef
      real,pointer,dimension(:) :: dmean_atm_shv
      real,pointer,dimension(:) :: dmean_atm_rshort
      real,pointer,dimension(:) :: dmean_atm_rshort_diff
      real,pointer,dimension(:) :: dmean_atm_par
      real,pointer,dimension(:) :: dmean_atm_par_diff
      real,pointer,dimension(:) :: dmean_atm_rlong
      real,pointer,dimension(:) :: dmean_atm_vels
      real,pointer,dimension(:) :: dmean_atm_rhos
      real,pointer,dimension(:) :: dmean_atm_prss
      real,pointer,dimension(:) :: dmean_atm_co2
      real,pointer,dimension(:) :: dmean_pcpg
      real,pointer,dimension(:) :: dmean_qpcpg
      real,pointer,dimension(:) :: dmean_dpcpg
      !------ Monthly mean. ---------------------------------------------------------------!
      real,pointer,dimension(:) :: mmean_atm_theiv
      real,pointer,dimension(:) :: mmean_atm_theta
      real,pointer,dimension(:) :: mmean_atm_temp
      real,pointer,dimension(:) :: mmean_atm_vpdef
      real,pointer,dimension(:) :: mmean_atm_shv
      real,pointer,dimension(:) :: mmean_atm_rshort
      real,pointer,dimension(:) :: mmean_atm_rshort_diff
      real,pointer,dimension(:) :: mmean_atm_par
      real,pointer,dimension(:) :: mmean_atm_par_diff
      real,pointer,dimension(:) :: mmean_atm_rlong
      real,pointer,dimension(:) :: mmean_atm_vels
      real,pointer,dimension(:) :: mmean_atm_rhos
      real,pointer,dimension(:) :: mmean_atm_prss
      real,pointer,dimension(:) :: mmean_atm_co2
      real,pointer,dimension(:) :: mmean_pcpg
      real,pointer,dimension(:) :: mmean_qpcpg
      real,pointer,dimension(:) :: mmean_dpcpg
      !------ Mean diel. ------------------------------------------------------------------!
      real,pointer,dimension(:,:) :: qmean_atm_theiv
      real,pointer,dimension(:,:) :: qmean_atm_theta
      real,pointer,dimension(:,:) :: qmean_atm_temp
      real,pointer,dimension(:,:) :: qmean_atm_vpdef
      real,pointer,dimension(:,:) :: qmean_atm_shv
      real,pointer,dimension(:,:) :: qmean_atm_rshort
      real,pointer,dimension(:,:) :: qmean_atm_rshort_diff
      real,pointer,dimension(:,:) :: qmean_atm_par
      real,pointer,dimension(:,:) :: qmean_atm_par_diff
      real,pointer,dimension(:,:) :: qmean_atm_rlong
      real,pointer,dimension(:,:) :: qmean_atm_vels
      real,pointer,dimension(:,:) :: qmean_atm_rhos
      real,pointer,dimension(:,:) :: qmean_atm_prss
      real,pointer,dimension(:,:) :: qmean_atm_co2
      real,pointer,dimension(:,:) :: qmean_pcpg
      real,pointer,dimension(:,:) :: qmean_qpcpg
      real,pointer,dimension(:,:) :: qmean_dpcpg
      !------------------------------------------------------------------------------------!
   end type polygontype
   !=======================================================================================!
   !=======================================================================================!





   !=======================================================================================!
   !=======================================================================================!
   ! ED TYPE: These are the arrays of polygon level variables that populate the current    !
   !          grid.  This is the most averaged and least nested dimension.                 !
   !---------------------------------------------------------------------------------------!
   type edtype

      !---- Data space indexing variables ---------------
      !
      ! These variables are the total number of each
      ! respective hierarchical type, summed across
      ! all compute nodes if it is a parallel process
      ! These values are mostly needed for output
      ! dataspace dimensioning.

      integer :: npolygons_global
      integer :: nsites_global
      integer :: npatches_global
      integer :: ncohorts_global

      ! Index offsets for total counts of cohorts, patches
      ! and sites.  If this is the nth machine writing to
      ! a file, it needs to know how many of these types
      ! had come before, so it writes data contiguously
      ! into the same file with the others

      integer :: mach_cohort_offset_index
      integer :: mach_patch_offset_index
      integer :: mach_site_offset_index
      integer :: mach_polygon_offset_index

      !
      !---------------------------------------------------


      integer :: npolygons

      integer :: pyglob_id

      type(polygontype),pointer,dimension(:) :: polygon


      !  The global index of the first site in each polygon
      integer,pointer,dimension(:) :: pysi_id

      ! The number of sites in each polygon
      integer,pointer,dimension(:) :: pysi_n

      real(kind=8),pointer,dimension(:) :: walltime_py

      ! Longitude at the middle point of this polygon
      real,pointer,dimension(:) :: lon

      ! Latitude at the middle point of this polygon
      real,pointer,dimension(:) :: lat

      integer,pointer,dimension(:) :: xatm
      
      integer,pointer,dimension(:) :: yatm
      
      ! matrix of site hydrologic adjacency
      real,pointer,dimension(:,:,:) :: site_adjacency

      real,pointer,dimension(:) :: wbar
      real,pointer,dimension(:) :: Te
      real,pointer,dimension(:) :: zbar
      real,pointer,dimension(:) :: tau
      real,pointer,dimension(:) :: sheat
      real,pointer,dimension(:) :: baseflow
      real,pointer,dimension(:) :: runoff
      real,pointer,dimension(:) :: qrunoff
      real,pointer,dimension(:) :: swliq

      integer,pointer,dimension(:) :: ilon   ! index for matching met. data
      integer,pointer,dimension(:) :: ilat   ! index for matching met. data


      !------------------------------------------------------------------------------------!
      !     This variable storages the workload of this polygon, and this is "measured" by !
      ! the total number of Runge-Kutta time steps this polygon takes on average every     !
      ! day.  This is not averaged by patch, instead this is added (because having two     !
      ! patches is computationally more intensive than having one).  Since the number of   !
      ! time steps depends on the environmental conditions and may change a lot for        !
      ! different seasons, we store the previous.  Index 13 is used for the integral of    !
      ! the current month.                                                                 !
      !------------------------------------------------------------------------------------!
      real, pointer, dimension(:,:) :: workload
      !------------------------------------------------------------------------------------!



      ! Polygon AGB (kgC/m2)
      real,pointer,dimension(:) :: total_agb

      ! Polygon basal area (cm2/m2)
      real,pointer,dimension(:) :: total_basal_area

      ! AGB accruing due to growth (kgC/m2/yr)
      real,pointer,dimension(:) :: total_agb_growth

      ! AGB lost due to mortality (kgC/m2/yr)
      real,pointer,dimension(:) :: total_agb_mort

      ! AGB used to generate recruits (kgC/m2/yr)
      real,pointer,dimension(:) :: total_agb_recruit



      ! CHANGED THE FOLLOWING UNIT DESCRIPTORS: FROM (cm2/m2/yr) RGK 6-13-08
      !------------
      ! BASAL_AREA accruing due to growth (cm2/m2/yr)
      real,pointer,dimension(:) :: total_basal_area_growth

      ! BASAL_AREA lost due to mortality (cm2/m2/yr)
      real,pointer,dimension(:) :: total_basal_area_mort

      ! BASAL_AREA used to generate recruits (cm2/m2/yr)
      real,pointer,dimension(:) :: total_basal_area_recruit
      !------------


      integer,pointer,dimension(:) :: nsites
      ! list of site numbers
      integer,pointer,dimension(:,:) :: sitenums !(max_site,npolygons)
 
      ! specification if the adjacency table was loaded from a file
      integer,pointer,dimension(:) :: load_adjacency

      !! Meteorological driver data
      type(met_driv_data),pointer,dimension(:) :: metinput
      
      !! Lapse rate transfer data
      type(met_driv_state),pointer,dimension(:) :: met, lapse

      real,pointer,dimension(:) :: cosz

      ! Total carbon (vegetation plus soil) at the beginning of budget-averaging
      ! time [kgC/m2]
      real,pointer,dimension(:) :: cbudget_initialstorage

      ! Average NEP (GPP - plant respiration - heterotrophic respiration)
      ! [kgC/m2/day], used for evaluating daily carbon budget.
      real,pointer,dimension(:) :: cbudget_nep

      ! Total nitrogen (vegetation plus soil) at the beginning of  
      ! budget-averaging time [kgN/m2]
      real,pointer,dimension(:) :: nbudget_initialstorage

      !----- Mass and Energy --------------------------------------------------!
      ! New variables for testing the model stability
      real,pointer,dimension(:) :: max_leaf_temp
      real,pointer,dimension(:) :: min_leaf_temp
      real,pointer,dimension(:) :: max_wood_temp
      real,pointer,dimension(:) :: min_wood_temp
      real,pointer,dimension(:) :: max_soil_temp
      real,pointer,dimension(:) :: min_soil_temp

      !------------------------------------------------------------------------------------!
      !      Polygon-level profile of the vegetation properties.  All variables are split  !
      ! by DBH and PFT, and units are based on area, not individuals.                      !
      !------------------------------------------------------------------------------------!
      real,pointer,dimension(:,:,:) :: nplant
      real,pointer,dimension(:,:,:) :: agb
      real,pointer,dimension(:,:,:) :: lai
      real,pointer,dimension(:,:,:) :: wai
      real,pointer,dimension(:,:,:) :: basal_area
      real,pointer,dimension(:,:,:) :: bdead
      real,pointer,dimension(:,:,:) :: balive
      real,pointer,dimension(:,:,:) :: bleaf
      real,pointer,dimension(:,:,:) :: broot
      real,pointer,dimension(:,:,:) :: bsapwooda
      real,pointer,dimension(:,:,:) :: bsapwoodb
      real,pointer,dimension(:,:,:) :: bseeds
      real,pointer,dimension(:,:,:) :: bstorage
      real,pointer,dimension(:,:,:) :: bdead_n
      real,pointer,dimension(:,:,:) :: balive_n
      real,pointer,dimension(:,:,:) :: bleaf_n
      real,pointer,dimension(:,:,:) :: broot_n
      real,pointer,dimension(:,:,:) :: bsapwooda_n
      real,pointer,dimension(:,:,:) :: bsapwoodb_n
      real,pointer,dimension(:,:,:) :: bseeds_n
      real,pointer,dimension(:,:,:) :: bstorage_n
      real,pointer,dimension(:,:,:) :: leaf_maintenance
      real,pointer,dimension(:,:,:) :: root_maintenance
      real,pointer,dimension(:,:,:) :: leaf_drop
      !------------------------------------------------------------------------------------!




      !----- Polygon-average of soil pools. -----------------------------------------------!
      real,pointer,dimension(:)   :: fast_soil_c
      real,pointer,dimension(:)   :: slow_soil_c
      real,pointer,dimension(:)   :: struct_soil_c
      real,pointer,dimension(:)   :: struct_soil_l
      real,pointer,dimension(:)   :: cwd_c
      real,pointer,dimension(:)   :: fast_soil_n
      real,pointer,dimension(:)   :: mineral_soil_n
      real,pointer,dimension(:)   :: cwd_n
      !------------------------------------------------------------------------------------!




      !-------- TOTAL CARBON AND NITROGEN POOLS  ---------------
      ! Added by MCD for NCEAS/FACE intercomparison (Apr 7 2009)
      !real,pointer,dimension(:) :: Cleaf
      !real,pointer,dimension(:) :: Croot
      !real,pointer,dimension(:) :: Cstore
      !real,pointer,dimension(:) :: Ccwd
      !real,pointer,dimension(:) :: Nleaf
      !real,pointer,dimension(:) :: Ndead
      !real,pointer,dimension(:) :: Nroot
      !real,pointer,dimension(:) :: Nstore
      !real,pointer,dimension(:) :: Ncwd
      ! MLO (8-Nov-2012) -- these were merged with the PFT/DBH structures above,
      !    CWD pools are reported, but I am not sure about them, they are simply
      !    a constant fraction of the structural and slow pools.


      !-------- TOTAL CARBON AND NITROGEN FLUX  ---------------
      ! Added by MCD for NCEAS/FACE intercomparison (Apr 7 2009)
      real,pointer,dimension(:) :: Cleaf_grow
      real,pointer,dimension(:) :: Croot_grow
      real,pointer,dimension(:) :: Cdead_grow
      real,pointer,dimension(:) :: Cstore_grow
      real,pointer,dimension(:) :: Cleaf_litter_flux
      real,pointer,dimension(:) :: Croot_litter_flux
      real,pointer,dimension(:) :: Ccwd_flux
      real,pointer,dimension(:) :: Nleaf_grow
      real,pointer,dimension(:) :: Ndead_grow
      real,pointer,dimension(:) :: Nroot_grow
      real,pointer,dimension(:) :: Nstore_grow
      real,pointer,dimension(:) :: Nleaf_litter_flux
      real,pointer,dimension(:) :: Nroot_litter_flux
      real,pointer,dimension(:) :: Ncwd_flux
      real,pointer,dimension(:) :: Nbiomass_uptake
      real,pointer,dimension(:) :: Ngross_min
      real,pointer,dimension(:) :: Nnet_min


      ! This diagnostic partitions energy flux variables into LAI regimes.
      ! The matrix is arranged as follows (LAI,VARIABLE,POLYGON)
      ! Where LAI is (<2.0,2-4,>4)
      ! Where Variable is (RNET,LHF,SHF,CANTEMP)
      real,pointer,dimension(:,:,:) :: avg_lai_ebalvars   


      !====================================================================================!
      !====================================================================================!
      !     Polygon-level diagnostic variables:  these are either fluxes or state          !
      ! variables, averaged over different time scales, for each polygon.  They are in the !
      ! output for convenience only.  Most came from the nested dimensions (sites, patches !
      ! and cohorts).                                                                      !
      !                                                                                    !
      !   FMEAN -- average over one frqsum time interval (usually sub-daily)               !
      !   DMEAN -- daily mean                                                              !
      !   MMEAN -- monthly mean                                                            !
      !   MMSQU -- monthly mean sum of squares                                             !
      !   QMEAN -- mean diel                                                               !
      !   QMSQU -- mean sum of squares for the diel                                        !
      !                                                                                    !
      !   Acronyms used in the description below:                                          !
      !   ATM       -- air above canopy                                                    !
      !   CAS       -- canopy air space                                                    !
      !   TSPL      -- temporary snow/pounding layer                                       !
      !   PAR       -- photosynthetically active radiation                                 !
      !   NIR       -- near infra-red                                                      !
      !   SW        -- short wave                                                          !
      !   LW        -- long wave                                                           !
      !   VPD       -- Vapour pressure deficit                                             !
      !   Theta_Eiv -- ice vapour equivalent potential temperature                         !


      !----- Fast averages. ---------------------------------------------------------------!
      real,pointer,dimension(:) :: fmean_gpp              ! Gross primary prod. [ kgC/m2/yr]
      real,pointer,dimension(:) :: fmean_npp              ! Net primary prod.   [ kgC/m2/yr]
      real,pointer,dimension(:) :: fmean_leaf_resp        ! Leaf respiration    [ kgC/m2/yr]
      real,pointer,dimension(:) :: fmean_root_resp        ! Root respiration    [ kgC/m2/yr]
      real,pointer,dimension(:) :: fmean_leaf_growth_resp ! Growth resp.        [ kgC/m2/yr]
      real,pointer,dimension(:) :: fmean_root_growth_resp ! Growth resp.        [ kgC/m2/yr]
      real,pointer,dimension(:) :: fmean_sapa_growth_resp ! Growth resp.        [ kgC/m2/yr]
      real,pointer,dimension(:) :: fmean_sapb_growth_resp ! Growth resp.        [ kgC/m2/yr]
      real,pointer,dimension(:) :: fmean_leaf_storage_resp     ! Storage resp.       [ kgC/m2/yr]
      real,pointer,dimension(:) :: fmean_root_storage_resp     ! Storage resp.       [ kgC/m2/yr]
      real,pointer,dimension(:) :: fmean_sapa_storage_resp     ! Storage resp.       [ kgC/m2/yr]
      real,pointer,dimension(:) :: fmean_sapb_storage_resp     ! Storage resp.       [ kgC/m2/yr]
      real,pointer,dimension(:) :: fmean_plresp           ! Plant respiration.  [ kgC/m2/yr]
      real,pointer,dimension(:) :: fmean_leaf_energy      ! Leaf int. energy    [      J/m2]
      real,pointer,dimension(:) :: fmean_leaf_water       ! Leaf sfc. water     [     kg/m2]
      real,pointer,dimension(:) :: fmean_leaf_hcap        ! Leaf heat capacity  [    J/m2/K]
      real,pointer,dimension(:) :: fmean_leaf_vpdef       ! Leaf VPD            [        Pa]
      real,pointer,dimension(:) :: fmean_leaf_temp        ! Leaf temperature    [         K]
      real,pointer,dimension(:) :: fmean_leaf_fliq        ! Liquid fraction     [        --]
      real,pointer,dimension(:) :: fmean_leaf_gsw         ! Stomatal conduct.   [       m/s]
      real,pointer,dimension(:) :: fmean_leaf_gbw         ! Leaf BL conduct.    [       m/s]
      real,pointer,dimension(:) :: fmean_wood_energy      ! Wood int. energy    [      J/m2]
      real,pointer,dimension(:) :: fmean_wood_water       ! Wood sfc. water     [     kg/m2]
      real,pointer,dimension(:) :: fmean_wood_hcap        ! Wood heat capacity  [    J/m2/K]
      real,pointer,dimension(:) :: fmean_wood_temp        ! Wood temperature    [         K]
      real,pointer,dimension(:) :: fmean_wood_fliq        ! Liquid fraction     [        --]
      real,pointer,dimension(:) :: fmean_wood_gbw         ! Wood BL conduct.    [       m/s]
      real,pointer,dimension(:) :: fmean_fs_open          ! Net stress factor   [        --]
      real,pointer,dimension(:) :: fmean_fsw              ! Moisture stress     [        --]
      real,pointer,dimension(:) :: fmean_fsn              ! Nitrogen stress     [        --]
      real,pointer,dimension(:) :: fmean_a_open           ! Assim. (no stress)  [umol/m2l/s]
      real,pointer,dimension(:) :: fmean_a_closed         ! Min. Assim.         [umol/m2l/s]
      real,pointer,dimension(:) :: fmean_a_net            ! Assim. actual       [umol/m2l/s]
      real,pointer,dimension(:) :: fmean_a_light          ! Assim. (light)      [umol/m2l/s]
      real,pointer,dimension(:) :: fmean_a_rubp           ! Assim. (RuBP)       [umol/m2l/s]
      real,pointer,dimension(:) :: fmean_a_co2            ! Assim. (CO2)        [umol/m2l/s]
      real,pointer,dimension(:) :: fmean_psi_open         ! Transp. no stress   [   kg/m2/s]
      real,pointer,dimension(:) :: fmean_psi_closed       ! Transp. max stress  [   kg/m2/s]
      real,pointer,dimension(:) :: fmean_water_supply     ! Water supply        [        --]
      real,pointer,dimension(:) :: fmean_par_l            ! Absorbed PAR (Leaf) [      W/m2]
      real,pointer,dimension(:) :: fmean_par_l_beam       ! Abs. Dir. PAR       [      W/m2]
      real,pointer,dimension(:) :: fmean_par_l_diff       ! Abs. Diffuse PAR    [      W/m2]
      real,pointer,dimension(:) :: fmean_rshort_l         ! Abs. SW (leaf)      [      W/m2]
      real,pointer,dimension(:) :: fmean_rlong_l          ! Abs. LW (leaf)      [      W/m2]
      real,pointer,dimension(:) :: fmean_sensible_lc      ! Sensible heat       [      W/m2]
      real,pointer,dimension(:) :: fmean_vapor_lc         ! Leaf evaporation    [   kg/m2/s]
      real,pointer,dimension(:) :: fmean_transp           ! Leaf transpiration  [   kg/m2/s]
      real,pointer,dimension(:) :: fmean_intercepted_al   ! Leaf interception   [   kg/m2/s]
      real,pointer,dimension(:) :: fmean_wshed_lg         ! Leaf shedding       [   kg/m2/s]
      real,pointer,dimension(:) :: fmean_rshort_w         ! Abs. SW (Wood)      [      W/m2]
      real,pointer,dimension(:) :: fmean_rlong_w          ! Abs. LW (Wood)      [      W/m2]
      real,pointer,dimension(:) :: fmean_sensible_wc      ! Sensible heat       [      W/m2]
      real,pointer,dimension(:) :: fmean_vapor_wc         ! Wood evaporation    [   kg/m2/s]
      real,pointer,dimension(:) :: fmean_intercepted_aw   ! Wood interception   [   kg/m2/s]
      real,pointer,dimension(:) :: fmean_wshed_wg         ! Wood shedding       [   kg/m2/s]
      !----- Photosynthesis/Decomposition. ------------------------------------------------!
      real,pointer,dimension(:)   :: fmean_rh              ! Heterotr. resp.    [ kgC/m2/yr]
      real,pointer,dimension(:)   :: fmean_cwd_rh          ! CWD respiration    [ kgC/m2/yr]
      real,pointer,dimension(:)   :: fmean_nep             ! Net Ecosyst. Prod. [ kgC/m2/yr]
      !----- State variables. -------------------------------------------------------------!
      real,pointer,dimension(:)   :: fmean_rk4step         ! RK4 time step      [         s]
      real,pointer,dimension(:)   :: fmean_available_water ! Available water    [     kg/m2]
      real,pointer,dimension(:)   :: fmean_can_theiv       ! CAS Theta_Eiv      [         K]
      real,pointer,dimension(:)   :: fmean_can_theta       ! CAS Potential temp.[         K]
      real,pointer,dimension(:)   :: fmean_can_vpdef       ! CAS Vap. Pres. Def.[        Pa]
      real,pointer,dimension(:)   :: fmean_can_temp        ! CAS Temperature    [         K]
      real,pointer,dimension(:)   :: fmean_can_shv         ! CAS Specific hum.  [     kg/kg]
      real,pointer,dimension(:)   :: fmean_can_co2         ! CAS CO2 mix. ratio [  umol/mol]
      real,pointer,dimension(:)   :: fmean_can_rhos        ! CAS air density    [     kg/m3]
      real,pointer,dimension(:)   :: fmean_can_prss        ! CAS pressure       [        Pa]
      real,pointer,dimension(:)   :: fmean_gnd_temp        ! Ground temperature [         K]
      real,pointer,dimension(:)   :: fmean_gnd_shv         ! Ground spec. hum.  [     kg/kg]
      real,pointer,dimension(:)   :: fmean_can_ggnd        ! Net canopy conduct.[       m/2]
      real,pointer,dimension(:)   :: fmean_sfcw_depth      ! TPSL depth         [         m]
      real,pointer,dimension(:)   :: fmean_sfcw_energy     ! TPSL int. energy   [      J/kg]
      real,pointer,dimension(:)   :: fmean_sfcw_mass       ! TPSL water mass    [     kg/m2]
      real,pointer,dimension(:)   :: fmean_sfcw_temp       ! TPSL temperature   [         K]
      real,pointer,dimension(:)   :: fmean_sfcw_fliq       ! TPSL liquid frac.  [        --]
      real,pointer,dimension(:,:) :: fmean_soil_energy     ! Soil int. energy   [      J/m3]
      real,pointer,dimension(:,:) :: fmean_soil_mstpot     ! Soil matric potl.  [         m]
      real,pointer,dimension(:,:) :: fmean_soil_water      ! Soil water content [     m3/m3]
      real,pointer,dimension(:,:) :: fmean_soil_temp       ! Soil temperature   [         K]
      real,pointer,dimension(:,:) :: fmean_soil_fliq       ! Soil liquid frac.  [        --]
      !----- Radiation --------------------------------------------------------------------!
      real,pointer,dimension(:)   :: fmean_rshort_gnd   ! Gnd. absp. SW Rad.    [      W/m2]
      real,pointer,dimension(:)   :: fmean_par_gnd      ! Absorbed PAR          [      W/m2]
      real,pointer,dimension(:)   :: fmean_rlong_gnd    ! Gnd. absp. LW Rad.    [      W/m2]
      real,pointer,dimension(:)   :: fmean_rlongup      ! Outgoing LW Rad.      [      W/m2]
      real,pointer,dimension(:)   :: fmean_parup        ! Outgoing PAR          [      W/m2]
      real,pointer,dimension(:)   :: fmean_nirup        ! Outgoing NIR          [      W/m2]
      real,pointer,dimension(:)   :: fmean_rshortup     ! Outgoing SW Rad.      [      W/m2]
      real,pointer,dimension(:)   :: fmean_rnet         ! Net radiation         [      W/m2]
      real,pointer,dimension(:)   :: fmean_albedo       ! Albedo                [      ----]
      real,pointer,dimension(:)   :: fmean_albedo_par   ! Direct Albedo         [      ----]
      real,pointer,dimension(:)   :: fmean_albedo_nir   ! Diffuse Albedo        [      ----]
      real,pointer,dimension(:)   :: fmean_rlong_albedo ! Longwave Albedo       [      ----]
      !----- Characteristic variables. ----------------------------------------------------!
      real,pointer,dimension(:)   :: fmean_ustar        ! Friction velocity     [       m/s]
      real,pointer,dimension(:)   :: fmean_tstar        ! Pot. temp. scale      [         K]
      real,pointer,dimension(:)   :: fmean_qstar        ! Water vapour scale    [     kg/kg]
      real,pointer,dimension(:)   :: fmean_cstar        ! CO2 scale             [  umol/mol]
      real,pointer,dimension(:)   :: fmean_carbon_ac    ! CO2 flux, ATM->CAS    [ umol/m2/s]
      real,pointer,dimension(:)   :: fmean_carbon_st    ! CO2 storage           [ umol/m2/s]
      real,pointer,dimension(:)   :: fmean_vapor_gc     ! Water flux, Gnd->CAS  [   kg/m2/s]
      real,pointer,dimension(:)   :: fmean_vapor_ac     ! Water flux, Atm->CAS  [   kg/m2/s]
      real,pointer,dimension(:,:) :: fmean_smoist_gg    ! Soil water flux       [   kg/m2/s]
      real,pointer,dimension(:)   :: fmean_throughfall  ! Throughfall rate      [   kg/m2/s]
      real,pointer,dimension(:,:) :: fmean_transloss    ! Water loss (transp.)  [   kg/m2/s]
      real,pointer,dimension(:)   :: fmean_runoff       ! Water runoff          [   kg/m2/s]
      real,pointer,dimension(:)   :: fmean_drainage     ! Water drainage        [   kg/m2/s]
      !----- Sensible heat ----------------------------------------------------------------!
      real,pointer,dimension(:)   :: fmean_sensible_gc  ! Sens. heat, GND->CAS  [      W/m2]
      real,pointer,dimension(:)   :: fmean_sensible_ac  ! Sens. heat, ATM->CAS  [      W/m2]
      real,pointer,dimension(:,:) :: fmean_sensible_gg  ! Soil heat flux        [      W/m2]
      real,pointer,dimension(:)   :: fmean_qthroughfall ! Throughfall rate      [      W/m2]
      real,pointer,dimension(:)   :: fmean_qrunoff      ! Surface runoff        [      W/m2]
      real,pointer,dimension(:)   :: fmean_qdrainage    ! Drainage energy loss  [      W/m2]
      !------ Met driver. -----------------------------------------------------------------!
      real,pointer,dimension(:) :: fmean_atm_theiv       ! IV Equiv. Pot. Temp   [        K]
      real,pointer,dimension(:) :: fmean_atm_theta       ! Potential temperature [        K]
      real,pointer,dimension(:) :: fmean_atm_temp        ! Temperature           [        K]
      real,pointer,dimension(:) :: fmean_atm_vpdef       ! Vapour pres. deficit  [       Pa]
      real,pointer,dimension(:) :: fmean_atm_shv         ! Specific humidity     [    kg/kg]
      real,pointer,dimension(:) :: fmean_atm_rshort      ! Shortwave radiation   [     W/m2]
      real,pointer,dimension(:) :: fmean_atm_rshort_diff ! Diffuse SW radiation  [     W/m2]
      real,pointer,dimension(:) :: fmean_atm_par         ! Photosyn. Active Rad. [     W/m2]
      real,pointer,dimension(:) :: fmean_atm_par_diff    ! Diffuse PAR           [     W/m2]
      real,pointer,dimension(:) :: fmean_atm_rlong       ! Longwave radiation    [     W/m2]
      real,pointer,dimension(:) :: fmean_atm_vels        ! Wind speed            [      m/s]
      real,pointer,dimension(:) :: fmean_atm_rhos        ! Air density           [    kg/m3]
      real,pointer,dimension(:) :: fmean_atm_prss        ! Air pressure          [       Pa]
      real,pointer,dimension(:) :: fmean_atm_co2         ! Air CO2               [ umol/mol]
      real,pointer,dimension(:) :: fmean_pcpg            ! Precipitation rate    [  kg/m2/s]
      real,pointer,dimension(:) :: fmean_qpcpg           ! Energy gain - rain    [     W/m2]
      real,pointer,dimension(:) :: fmean_dpcpg           ! Precipitation depth   [        m]
      !----- Moore Foundation variables. --------------------------------------------------!
      real,pointer,dimension(:) :: fmean_soil_wetness     ! Soil wetness index  [        --]
      real,pointer,dimension(:) :: fmean_skin_temp        ! Skin temperature    [         K]
      !--------Variables required by PEcAn.  ----------------------------------------------!
      real,pointer,dimension(:) :: fmean_lai		  ! LAI  		[     m2/m2]
      real,pointer,dimension(:)	:: fmean_bdead	    	  ! Bdead		[     kg/pl]
      !----- Variables without sub-daily averages. ----------------------------------------!
      real,pointer,dimension(:) :: dmean_nppleaf          ! Leaf NPP            [ kgC/m2/yr]
      real,pointer,dimension(:) :: dmean_nppfroot         ! Fine root NPP       [ kgC/m2/yr]
      real,pointer,dimension(:) :: dmean_nppsapwood       ! Sapwood NPP         [ kgC/m2/yr]
      real,pointer,dimension(:) :: dmean_nppcroot         ! Coarse root NPP     [ kgC/m2/yr]
      real,pointer,dimension(:) :: dmean_nppseeds         ! Seed NPP            [ kgC/m2/yr]
      real,pointer,dimension(:) :: dmean_nppwood          ! Wood NPP            [ kgC/m2/yr]
      real,pointer,dimension(:) :: dmean_nppdaily         ! Daily NPP           [ kgC/m2/yr]
      real,pointer,dimension(:) :: dmean_A_decomp         ! Decomposition fact. [        --]
      real,pointer,dimension(:) :: dmean_Af_decomp        ! A with N immobil.   [        --]
      real,pointer,dimension(:) :: dmean_co2_residual     ! CO2 residual        [   umol/m2]
      real,pointer,dimension(:) :: dmean_energy_residual  ! Enthalpy residual   [      J/m2]
      real,pointer,dimension(:) :: dmean_water_residual   ! Water residual      [     kg/m2]
      !----- Variables that are updated once a day. ---------------------------------------!
      real,pointer,dimension(:,:,:) :: mmean_lai              ! Leaf area index [     m2/m2]
      real,pointer,dimension(:,:,:) :: mmean_bleaf            ! Leaf biomass    [    kgC/m2]
      real,pointer,dimension(:,:,:) :: mmean_broot            ! Root biomass    [    kgC/m2]
      real,pointer,dimension(:,:,:) :: mmean_bstorage         ! Storage         [    kgC/m2]
      real,pointer,dimension(:,:,:) :: mmean_bleaf_n          ! Leaf N mass     [    kgN/m2]
      real,pointer,dimension(:,:,:) :: mmean_broot_n          ! Root N mass     [    kgN/m2]
      real,pointer,dimension(:,:,:) :: mmean_bstorage_n       ! Storage N       [    kgN/m2]
      real,pointer,dimension(:,:,:) :: mmean_leaf_maintenance ! Leaf maintenance[ kgC/m2/yr]
      real,pointer,dimension(:,:,:) :: mmean_root_maintenance ! Root maintenance[ kgC/m2/yr]
      real,pointer,dimension(:,:,:) :: mmean_leaf_drop        ! Leaf shedding   [ kgC/m2/yr]
      real,pointer,dimension(:)     :: mmean_fast_soil_c      ! Fast soil C     [    kgC/m2]
      real,pointer,dimension(:)     :: mmean_slow_soil_c      ! Slow soil C     [    kgC/m2]
      real,pointer,dimension(:)     :: mmean_struct_soil_c    ! Struct. soil C  [    kgC/m2]
      real,pointer,dimension(:)     :: mmean_struct_soil_l    ! Struct. soil L  [    kgL/m2]
      real,pointer,dimension(:)     :: mmean_cwd_c            ! CWD carbon      [    kgC/m2]
      real,pointer,dimension(:)     :: mmean_fast_soil_n      ! Fast soil N     [    kgN/m2]
      real,pointer,dimension(:)     :: mmean_mineral_soil_n   ! Mineral. soil N [    kgN/m2]
      real,pointer,dimension(:)     :: mmean_cwd_n            ! CWD nitrogen    [    kgN/m2]
      !----- Daily mean (same units as fast mean). ----------------------------------------!
      real,pointer,dimension(:)     :: dmean_gpp
      real,pointer,dimension(:)     :: dmean_npp
      real,pointer,dimension(:)     :: dmean_leaf_resp
      real,pointer,dimension(:)     :: dmean_root_resp
      real,pointer,dimension(:)     :: dmean_leaf_growth_resp
      real,pointer,dimension(:)     :: dmean_root_growth_resp
      real,pointer,dimension(:)     :: dmean_sapa_growth_resp
      real,pointer,dimension(:)     :: dmean_sapb_growth_resp
      real,pointer,dimension(:)     :: dmean_leaf_storage_resp
      real,pointer,dimension(:)     :: dmean_root_storage_resp
      real,pointer,dimension(:)     :: dmean_sapa_storage_resp
      real,pointer,dimension(:)     :: dmean_sapb_storage_resp
      real,pointer,dimension(:)     :: dmean_plresp
      real,pointer,dimension(:)     :: dmean_leaf_energy
      real,pointer,dimension(:)     :: dmean_leaf_water
      real,pointer,dimension(:)     :: dmean_leaf_hcap
      real,pointer,dimension(:)     :: dmean_leaf_vpdef
      real,pointer,dimension(:)     :: dmean_leaf_temp
      real,pointer,dimension(:)     :: dmean_leaf_fliq
      real,pointer,dimension(:)     :: dmean_leaf_gsw
      real,pointer,dimension(:)     :: dmean_leaf_gbw
      real,pointer,dimension(:)     :: dmean_wood_energy
      real,pointer,dimension(:)     :: dmean_wood_water
      real,pointer,dimension(:)     :: dmean_wood_hcap
      real,pointer,dimension(:)     :: dmean_wood_temp
      real,pointer,dimension(:)     :: dmean_wood_fliq
      real,pointer,dimension(:)     :: dmean_wood_gbw
      real,pointer,dimension(:)     :: dmean_fs_open
      real,pointer,dimension(:)     :: dmean_fsw
      real,pointer,dimension(:)     :: dmean_fsn
      real,pointer,dimension(:)     :: dmean_a_open
      real,pointer,dimension(:)     :: dmean_a_closed
      real,pointer,dimension(:)     :: dmean_a_net
      real,pointer,dimension(:)     :: dmean_a_light
      real,pointer,dimension(:)     :: dmean_a_rubp
      real,pointer,dimension(:)     :: dmean_a_co2
      real,pointer,dimension(:)     :: dmean_psi_open
      real,pointer,dimension(:)     :: dmean_psi_closed
      real,pointer,dimension(:)     :: dmean_water_supply
      real,pointer,dimension(:)     :: dmean_par_l
      real,pointer,dimension(:)     :: dmean_par_l_beam
      real,pointer,dimension(:)     :: dmean_par_l_diff
      real,pointer,dimension(:)     :: dmean_rshort_l
      real,pointer,dimension(:)     :: dmean_rlong_l
      real,pointer,dimension(:)     :: dmean_sensible_lc
      real,pointer,dimension(:)     :: dmean_vapor_lc
      real,pointer,dimension(:)     :: dmean_transp
      real,pointer,dimension(:)     :: dmean_intercepted_al
      real,pointer,dimension(:)     :: dmean_wshed_lg
      real,pointer,dimension(:)     :: dmean_rshort_w
      real,pointer,dimension(:)     :: dmean_rlong_w
      real,pointer,dimension(:)     :: dmean_sensible_wc
      real,pointer,dimension(:)     :: dmean_vapor_wc
      real,pointer,dimension(:)     :: dmean_intercepted_aw
      real,pointer,dimension(:)     :: dmean_wshed_wg
      real,pointer,dimension(:)     :: dmean_rh
      real,pointer,dimension(:)     :: dmean_cwd_rh
      real,pointer,dimension(:)     :: dmean_nep
      real,pointer,dimension(:)     :: dmean_rk4step
      real,pointer,dimension(:)     :: dmean_available_water
      real,pointer,dimension(:)     :: dmean_can_theiv
      real,pointer,dimension(:)     :: dmean_can_theta
      real,pointer,dimension(:)     :: dmean_can_vpdef
      real,pointer,dimension(:)     :: dmean_can_temp
      real,pointer,dimension(:)     :: dmean_can_shv
      real,pointer,dimension(:)     :: dmean_can_co2
      real,pointer,dimension(:)     :: dmean_can_rhos
      real,pointer,dimension(:)     :: dmean_can_prss
      real,pointer,dimension(:)     :: dmean_gnd_temp
      real,pointer,dimension(:)     :: dmean_gnd_shv
      real,pointer,dimension(:)     :: dmean_can_ggnd
      real,pointer,dimension(:)     :: dmean_sfcw_depth
      real,pointer,dimension(:)     :: dmean_sfcw_energy
      real,pointer,dimension(:)     :: dmean_sfcw_mass
      real,pointer,dimension(:)     :: dmean_sfcw_temp
      real,pointer,dimension(:)     :: dmean_sfcw_fliq
      real,pointer,dimension(:,:)   :: dmean_soil_energy
      real,pointer,dimension(:,:)   :: dmean_soil_mstpot
      real,pointer,dimension(:,:)   :: dmean_soil_water
      real,pointer,dimension(:,:)   :: dmean_soil_temp
      real,pointer,dimension(:,:)   :: dmean_soil_fliq
      real,pointer,dimension(:)     :: dmean_rshort_gnd
      real,pointer,dimension(:)     :: dmean_par_gnd
      real,pointer,dimension(:)     :: dmean_rlong_gnd
      real,pointer,dimension(:)     :: dmean_rlongup
      real,pointer,dimension(:)     :: dmean_parup
      real,pointer,dimension(:)     :: dmean_nirup
      real,pointer,dimension(:)     :: dmean_rshortup
      real,pointer,dimension(:)     :: dmean_rnet
      real,pointer,dimension(:)     :: dmean_albedo
      real,pointer,dimension(:)     :: dmean_albedo_par 
      real,pointer,dimension(:)     :: dmean_albedo_nir 
      real,pointer,dimension(:)     :: dmean_rlong_albedo
      real,pointer,dimension(:)     :: dmean_ustar
      real,pointer,dimension(:)     :: dmean_tstar
      real,pointer,dimension(:)     :: dmean_qstar
      real,pointer,dimension(:)     :: dmean_cstar
      real,pointer,dimension(:)     :: dmean_carbon_ac
      real,pointer,dimension(:)     :: dmean_carbon_st
      real,pointer,dimension(:)     :: dmean_vapor_gc
      real,pointer,dimension(:)     :: dmean_vapor_ac
      real,pointer,dimension(:,:)   :: dmean_smoist_gg
      real,pointer,dimension(:)     :: dmean_throughfall
      real,pointer,dimension(:,:)   :: dmean_transloss
      real,pointer,dimension(:)     :: dmean_runoff
      real,pointer,dimension(:)     :: dmean_drainage
      real,pointer,dimension(:)     :: dmean_sensible_gc
      real,pointer,dimension(:)     :: dmean_sensible_ac
      real,pointer,dimension(:,:)   :: dmean_sensible_gg
      real,pointer,dimension(:)     :: dmean_qthroughfall
      real,pointer,dimension(:)     :: dmean_qrunoff
      real,pointer,dimension(:)     :: dmean_qdrainage
      real,pointer,dimension(:)     :: dmean_atm_theiv
      real,pointer,dimension(:)     :: dmean_atm_theta
      real,pointer,dimension(:)     :: dmean_atm_temp
      real,pointer,dimension(:)     :: dmean_atm_vpdef
      real,pointer,dimension(:)     :: dmean_atm_shv
      real,pointer,dimension(:)     :: dmean_atm_rshort
      real,pointer,dimension(:)     :: dmean_atm_rshort_diff
      real,pointer,dimension(:)     :: dmean_atm_par
      real,pointer,dimension(:)     :: dmean_atm_par_diff
      real,pointer,dimension(:)     :: dmean_atm_rlong
      real,pointer,dimension(:)     :: dmean_atm_vels
      real,pointer,dimension(:)     :: dmean_atm_rhos
      real,pointer,dimension(:)     :: dmean_atm_prss
      real,pointer,dimension(:)     :: dmean_atm_co2
      real,pointer,dimension(:)     :: dmean_pcpg
      real,pointer,dimension(:)     :: dmean_qpcpg
      real,pointer,dimension(:)     :: dmean_dpcpg
      !----- Monthly mean (same units as fast mean). --------------------------------------!
      real,pointer,dimension(:)     :: mmean_gpp
      real,pointer,dimension(:)     :: mmean_npp
      real,pointer,dimension(:)     :: mmean_leaf_resp
      real,pointer,dimension(:)     :: mmean_root_resp
      real,pointer,dimension(:)     :: mmean_leaf_growth_resp
      real,pointer,dimension(:)     :: mmean_root_growth_resp
      real,pointer,dimension(:)     :: mmean_sapa_growth_resp
      real,pointer,dimension(:)     :: mmean_sapb_growth_resp
      real,pointer,dimension(:)     :: mmean_leaf_storage_resp
      real,pointer,dimension(:)     :: mmean_root_storage_resp
      real,pointer,dimension(:)     :: mmean_sapa_storage_resp
      real,pointer,dimension(:)     :: mmean_sapb_storage_resp
      real,pointer,dimension(:)     :: mmean_plresp
      real,pointer,dimension(:)     :: mmean_leaf_energy
      real,pointer,dimension(:)     :: mmean_leaf_water
      real,pointer,dimension(:)     :: mmean_leaf_hcap
      real,pointer,dimension(:)     :: mmean_leaf_vpdef
      real,pointer,dimension(:)     :: mmean_leaf_temp
      real,pointer,dimension(:)     :: mmean_leaf_fliq
      real,pointer,dimension(:)     :: mmean_leaf_gsw
      real,pointer,dimension(:)     :: mmean_leaf_gbw
      real,pointer,dimension(:)     :: mmean_wood_energy
      real,pointer,dimension(:)     :: mmean_wood_water
      real,pointer,dimension(:)     :: mmean_wood_hcap
      real,pointer,dimension(:)     :: mmean_wood_temp
      real,pointer,dimension(:)     :: mmean_wood_fliq
      real,pointer,dimension(:)     :: mmean_wood_gbw
      real,pointer,dimension(:)     :: mmean_fs_open
      real,pointer,dimension(:)     :: mmean_fsw
      real,pointer,dimension(:)     :: mmean_fsn
      real,pointer,dimension(:)     :: mmean_a_open
      real,pointer,dimension(:)     :: mmean_a_closed
      real,pointer,dimension(:)     :: mmean_a_net
      real,pointer,dimension(:)     :: mmean_a_light
      real,pointer,dimension(:)     :: mmean_a_rubp
      real,pointer,dimension(:)     :: mmean_a_co2
      real,pointer,dimension(:)     :: mmean_psi_open
      real,pointer,dimension(:)     :: mmean_psi_closed
      real,pointer,dimension(:)     :: mmean_water_supply
      real,pointer,dimension(:)     :: mmean_par_l
      real,pointer,dimension(:)     :: mmean_par_l_beam
      real,pointer,dimension(:)     :: mmean_par_l_diff
      real,pointer,dimension(:)     :: mmean_rshort_l
      real,pointer,dimension(:)     :: mmean_rlong_l
      real,pointer,dimension(:)     :: mmean_sensible_lc
      real,pointer,dimension(:)     :: mmean_vapor_lc
      real,pointer,dimension(:)     :: mmean_transp
      real,pointer,dimension(:)     :: mmean_intercepted_al
      real,pointer,dimension(:)     :: mmean_wshed_lg
      real,pointer,dimension(:)     :: mmean_rshort_w
      real,pointer,dimension(:)     :: mmean_rlong_w
      real,pointer,dimension(:)     :: mmean_sensible_wc
      real,pointer,dimension(:)     :: mmean_vapor_wc
      real,pointer,dimension(:)     :: mmean_intercepted_aw
      real,pointer,dimension(:)     :: mmean_wshed_wg
      real,pointer,dimension(:)     :: mmean_rh
      real,pointer,dimension(:)     :: mmean_cwd_rh
      real,pointer,dimension(:)     :: mmean_nep
      real,pointer,dimension(:)     :: mmean_rk4step
      real,pointer,dimension(:)     :: mmean_available_water
      real,pointer,dimension(:)     :: mmean_can_theiv
      real,pointer,dimension(:)     :: mmean_can_theta
      real,pointer,dimension(:)     :: mmean_can_vpdef
      real,pointer,dimension(:)     :: mmean_can_temp
      real,pointer,dimension(:)     :: mmean_can_shv
      real,pointer,dimension(:)     :: mmean_can_co2
      real,pointer,dimension(:)     :: mmean_can_rhos
      real,pointer,dimension(:)     :: mmean_can_prss
      real,pointer,dimension(:)     :: mmean_gnd_temp
      real,pointer,dimension(:)     :: mmean_gnd_shv
      real,pointer,dimension(:)     :: mmean_can_ggnd
      real,pointer,dimension(:)     :: mmean_sfcw_depth
      real,pointer,dimension(:)     :: mmean_sfcw_energy
      real,pointer,dimension(:)     :: mmean_sfcw_mass
      real,pointer,dimension(:)     :: mmean_sfcw_temp
      real,pointer,dimension(:)     :: mmean_sfcw_fliq
      real,pointer,dimension(:,:)   :: mmean_soil_energy
      real,pointer,dimension(:,:)   :: mmean_soil_mstpot
      real,pointer,dimension(:,:)   :: mmean_soil_water
      real,pointer,dimension(:,:)   :: mmean_soil_temp
      real,pointer,dimension(:,:)   :: mmean_soil_fliq
      real,pointer,dimension(:)     :: mmean_rshort_gnd
      real,pointer,dimension(:)     :: mmean_par_gnd
      real,pointer,dimension(:)     :: mmean_rlong_gnd
      real,pointer,dimension(:)     :: mmean_rlongup
      real,pointer,dimension(:)     :: mmean_parup
      real,pointer,dimension(:)     :: mmean_nirup
      real,pointer,dimension(:)     :: mmean_rshortup
      real,pointer,dimension(:)     :: mmean_rnet
      real,pointer,dimension(:)     :: mmean_albedo
      real,pointer,dimension(:)     :: mmean_albedo_par 
      real,pointer,dimension(:)     :: mmean_albedo_nir 
      real,pointer,dimension(:)     :: mmean_rlong_albedo
      real,pointer,dimension(:)     :: mmean_ustar
      real,pointer,dimension(:)     :: mmean_tstar
      real,pointer,dimension(:)     :: mmean_qstar
      real,pointer,dimension(:)     :: mmean_cstar
      real,pointer,dimension(:)     :: mmean_carbon_ac
      real,pointer,dimension(:)     :: mmean_carbon_st
      real,pointer,dimension(:)     :: mmean_vapor_gc
      real,pointer,dimension(:)     :: mmean_vapor_ac
      real,pointer,dimension(:,:)   :: mmean_smoist_gg
      real,pointer,dimension(:)     :: mmean_throughfall
      real,pointer,dimension(:,:)   :: mmean_transloss
      real,pointer,dimension(:)     :: mmean_runoff
      real,pointer,dimension(:)     :: mmean_drainage
      real,pointer,dimension(:)     :: mmean_sensible_gc
      real,pointer,dimension(:)     :: mmean_sensible_ac
      real,pointer,dimension(:,:)   :: mmean_sensible_gg
      real,pointer,dimension(:)     :: mmean_qthroughfall
      real,pointer,dimension(:)     :: mmean_qrunoff
      real,pointer,dimension(:)     :: mmean_qdrainage
      real,pointer,dimension(:)     :: mmean_nppleaf    
      real,pointer,dimension(:)     :: mmean_nppfroot   
      real,pointer,dimension(:)     :: mmean_nppsapwood 
      real,pointer,dimension(:)     :: mmean_nppcroot   
      real,pointer,dimension(:)     :: mmean_nppseeds   
      real,pointer,dimension(:)     :: mmean_nppwood    
      real,pointer,dimension(:)     :: mmean_nppdaily   
      real,pointer,dimension(:)     :: mmean_A_decomp
      real,pointer,dimension(:)     :: mmean_Af_decomp
      real,pointer,dimension(:)     :: mmean_co2_residual
      real,pointer,dimension(:)     :: mmean_energy_residual
      real,pointer,dimension(:)     :: mmean_water_residual
      real,pointer,dimension(:)     :: mmean_atm_theiv
      real,pointer,dimension(:)     :: mmean_atm_theta
      real,pointer,dimension(:)     :: mmean_atm_temp
      real,pointer,dimension(:)     :: mmean_atm_vpdef
      real,pointer,dimension(:)     :: mmean_atm_shv
      real,pointer,dimension(:)     :: mmean_atm_rshort
      real,pointer,dimension(:)     :: mmean_atm_rshort_diff
      real,pointer,dimension(:)     :: mmean_atm_par
      real,pointer,dimension(:)     :: mmean_atm_par_diff
      real,pointer,dimension(:)     :: mmean_atm_rlong
      real,pointer,dimension(:)     :: mmean_atm_vels
      real,pointer,dimension(:)     :: mmean_atm_rhos
      real,pointer,dimension(:)     :: mmean_atm_prss
      real,pointer,dimension(:)     :: mmean_atm_co2
      real,pointer,dimension(:)     :: mmean_pcpg
      real,pointer,dimension(:)     :: mmean_qpcpg
      real,pointer,dimension(:)     :: mmean_dpcpg
      !----- Monthly mean sum of squares. -------------------------------------------------!
      real,pointer,dimension(:)     :: mmsqu_gpp
      real,pointer,dimension(:)     :: mmsqu_npp
      real,pointer,dimension(:)     :: mmsqu_plresp
      real,pointer,dimension(:)     :: mmsqu_sensible_lc
      real,pointer,dimension(:)     :: mmsqu_vapor_lc
      real,pointer,dimension(:)     :: mmsqu_transp
      real,pointer,dimension(:)     :: mmsqu_sensible_wc
      real,pointer,dimension(:)     :: mmsqu_vapor_wc
      real,pointer,dimension(:)     :: mmsqu_rh
      real,pointer,dimension(:)     :: mmsqu_cwd_rh
      real,pointer,dimension(:)     :: mmsqu_nep
      real,pointer,dimension(:)     :: mmsqu_rlongup
      real,pointer,dimension(:)     :: mmsqu_parup
      real,pointer,dimension(:)     :: mmsqu_nirup
      real,pointer,dimension(:)     :: mmsqu_rshortup
      real,pointer,dimension(:)     :: mmsqu_rnet
      real,pointer,dimension(:)     :: mmsqu_albedo
      real,pointer,dimension(:)     :: mmsqu_ustar
      real,pointer,dimension(:)     :: mmsqu_carbon_ac
      real,pointer,dimension(:)     :: mmsqu_carbon_st
      real,pointer,dimension(:)     :: mmsqu_vapor_gc
      real,pointer,dimension(:)     :: mmsqu_vapor_ac
      real,pointer,dimension(:)     :: mmsqu_sensible_gc
      real,pointer,dimension(:)     :: mmsqu_sensible_ac
      !----- Mean diel (same units as fast mean). -----------------------------------------!
      real,pointer,dimension(:,:)   :: qmean_gpp
      real,pointer,dimension(:,:)   :: qmean_npp
      real,pointer,dimension(:,:)   :: qmean_leaf_resp
      real,pointer,dimension(:,:)   :: qmean_root_resp
      real,pointer,dimension(:,:)   :: qmean_leaf_growth_resp
      real,pointer,dimension(:,:)   :: qmean_root_growth_resp
      real,pointer,dimension(:,:)   :: qmean_sapa_growth_resp
      real,pointer,dimension(:,:)   :: qmean_sapb_growth_resp
      real,pointer,dimension(:,:)   :: qmean_leaf_storage_resp
      real,pointer,dimension(:,:)   :: qmean_root_storage_resp
      real,pointer,dimension(:,:)   :: qmean_sapa_storage_resp
      real,pointer,dimension(:,:)   :: qmean_sapb_storage_resp
      real,pointer,dimension(:,:)   :: qmean_plresp
      real,pointer,dimension(:,:)   :: qmean_leaf_energy
      real,pointer,dimension(:,:)   :: qmean_leaf_water
      real,pointer,dimension(:,:)   :: qmean_leaf_hcap
      real,pointer,dimension(:,:)   :: qmean_leaf_vpdef
      real,pointer,dimension(:,:)   :: qmean_leaf_temp
      real,pointer,dimension(:,:)   :: qmean_leaf_fliq
      real,pointer,dimension(:,:)   :: qmean_leaf_gsw
      real,pointer,dimension(:,:)   :: qmean_leaf_gbw
      real,pointer,dimension(:,:)   :: qmean_wood_energy
      real,pointer,dimension(:,:)   :: qmean_wood_water
      real,pointer,dimension(:,:)   :: qmean_wood_hcap
      real,pointer,dimension(:,:)   :: qmean_wood_temp
      real,pointer,dimension(:,:)   :: qmean_wood_fliq
      real,pointer,dimension(:,:)   :: qmean_wood_gbw
      real,pointer,dimension(:,:)   :: qmean_fs_open
      real,pointer,dimension(:,:)   :: qmean_fsw
      real,pointer,dimension(:,:)   :: qmean_fsn
      real,pointer,dimension(:,:)   :: qmean_a_open
      real,pointer,dimension(:,:)   :: qmean_a_closed
      real,pointer,dimension(:,:)   :: qmean_a_net
      real,pointer,dimension(:,:)   :: qmean_a_light
      real,pointer,dimension(:,:)   :: qmean_a_rubp
      real,pointer,dimension(:,:)   :: qmean_a_co2
      real,pointer,dimension(:,:)   :: qmean_psi_open
      real,pointer,dimension(:,:)   :: qmean_psi_closed
      real,pointer,dimension(:,:)   :: qmean_water_supply
      real,pointer,dimension(:,:)   :: qmean_par_l
      real,pointer,dimension(:,:)   :: qmean_par_l_beam
      real,pointer,dimension(:,:)   :: qmean_par_l_diff
      real,pointer,dimension(:,:)   :: qmean_rshort_l
      real,pointer,dimension(:,:)   :: qmean_rlong_l
      real,pointer,dimension(:,:)   :: qmean_sensible_lc
      real,pointer,dimension(:,:)   :: qmean_vapor_lc
      real,pointer,dimension(:,:)   :: qmean_transp
      real,pointer,dimension(:,:)   :: qmean_intercepted_al
      real,pointer,dimension(:,:)   :: qmean_wshed_lg
      real,pointer,dimension(:,:)   :: qmean_rshort_w
      real,pointer,dimension(:,:)   :: qmean_rlong_w
      real,pointer,dimension(:,:)   :: qmean_sensible_wc
      real,pointer,dimension(:,:)   :: qmean_vapor_wc
      real,pointer,dimension(:,:)   :: qmean_intercepted_aw
      real,pointer,dimension(:,:)   :: qmean_wshed_wg
      real,pointer,dimension(:,:)   :: qmean_rh
      real,pointer,dimension(:,:)   :: qmean_cwd_rh
      real,pointer,dimension(:,:)   :: qmean_nep
      real,pointer,dimension(:,:)   :: qmean_rk4step
      real,pointer,dimension(:,:)   :: qmean_available_water
      real,pointer,dimension(:,:)   :: qmean_can_theiv
      real,pointer,dimension(:,:)   :: qmean_can_theta
      real,pointer,dimension(:,:)   :: qmean_can_vpdef
      real,pointer,dimension(:,:)   :: qmean_can_temp
      real,pointer,dimension(:,:)   :: qmean_can_shv
      real,pointer,dimension(:,:)   :: qmean_can_co2
      real,pointer,dimension(:,:)   :: qmean_can_rhos
      real,pointer,dimension(:,:)   :: qmean_can_prss
      real,pointer,dimension(:,:)   :: qmean_gnd_temp
      real,pointer,dimension(:,:)   :: qmean_gnd_shv
      real,pointer,dimension(:,:)   :: qmean_can_ggnd
      real,pointer,dimension(:,:)   :: qmean_sfcw_depth
      real,pointer,dimension(:,:)   :: qmean_sfcw_energy
      real,pointer,dimension(:,:)   :: qmean_sfcw_mass
      real,pointer,dimension(:,:)   :: qmean_sfcw_temp
      real,pointer,dimension(:,:)   :: qmean_sfcw_fliq
      real,pointer,dimension(:,:,:) :: qmean_soil_energy
      real,pointer,dimension(:,:,:) :: qmean_soil_mstpot
      real,pointer,dimension(:,:,:) :: qmean_soil_water
      real,pointer,dimension(:,:,:) :: qmean_soil_temp
      real,pointer,dimension(:,:,:) :: qmean_soil_fliq
      real,pointer,dimension(:,:)   :: qmean_rshort_gnd
      real,pointer,dimension(:,:)   :: qmean_par_gnd
      real,pointer,dimension(:,:)   :: qmean_rlong_gnd
      real,pointer,dimension(:,:)   :: qmean_rlongup
      real,pointer,dimension(:,:)   :: qmean_parup
      real,pointer,dimension(:,:)   :: qmean_nirup
      real,pointer,dimension(:,:)   :: qmean_rshortup
      real,pointer,dimension(:,:)   :: qmean_rnet
      real,pointer,dimension(:,:)   :: qmean_albedo
      real,pointer,dimension(:,:)   :: qmean_albedo_par 
      real,pointer,dimension(:,:)   :: qmean_albedo_nir 
      real,pointer,dimension(:,:)   :: qmean_rlong_albedo
      real,pointer,dimension(:,:)   :: qmean_ustar
      real,pointer,dimension(:,:)   :: qmean_tstar
      real,pointer,dimension(:,:)   :: qmean_qstar
      real,pointer,dimension(:,:)   :: qmean_cstar
      real,pointer,dimension(:,:)   :: qmean_carbon_ac
      real,pointer,dimension(:,:)   :: qmean_carbon_st
      real,pointer,dimension(:,:)   :: qmean_vapor_gc
      real,pointer,dimension(:,:)   :: qmean_vapor_ac
      real,pointer,dimension(:,:,:) :: qmean_smoist_gg
      real,pointer,dimension(:,:)   :: qmean_throughfall
      real,pointer,dimension(:,:,:) :: qmean_transloss
      real,pointer,dimension(:,:)   :: qmean_runoff
      real,pointer,dimension(:,:)   :: qmean_drainage
      real,pointer,dimension(:,:)   :: qmean_sensible_gc
      real,pointer,dimension(:,:)   :: qmean_sensible_ac
      real,pointer,dimension(:,:,:) :: qmean_sensible_gg
      real,pointer,dimension(:,:)   :: qmean_qthroughfall
      real,pointer,dimension(:,:)   :: qmean_qrunoff
      real,pointer,dimension(:,:)   :: qmean_qdrainage
      real,pointer,dimension(:,:)   :: qmean_atm_theiv
      real,pointer,dimension(:,:)   :: qmean_atm_theta
      real,pointer,dimension(:,:)   :: qmean_atm_temp
      real,pointer,dimension(:,:)   :: qmean_atm_vpdef
      real,pointer,dimension(:,:)   :: qmean_atm_shv
      real,pointer,dimension(:,:)   :: qmean_atm_rshort
      real,pointer,dimension(:,:)   :: qmean_atm_rshort_diff
      real,pointer,dimension(:,:)   :: qmean_atm_par
      real,pointer,dimension(:,:)   :: qmean_atm_par_diff
      real,pointer,dimension(:,:)   :: qmean_atm_rlong
      real,pointer,dimension(:,:)   :: qmean_atm_vels
      real,pointer,dimension(:,:)   :: qmean_atm_rhos
      real,pointer,dimension(:,:)   :: qmean_atm_prss
      real,pointer,dimension(:,:)   :: qmean_atm_co2
      real,pointer,dimension(:,:)   :: qmean_pcpg
      real,pointer,dimension(:,:)   :: qmean_qpcpg
      real,pointer,dimension(:,:)   :: qmean_dpcpg
      !------ Mean diel of sum of squares. ------------------------------------------------!
      real,pointer,dimension(:,:)   :: qmsqu_gpp
      real,pointer,dimension(:,:)   :: qmsqu_npp
      real,pointer,dimension(:,:)   :: qmsqu_plresp
      real,pointer,dimension(:,:)   :: qmsqu_sensible_lc
      real,pointer,dimension(:,:)   :: qmsqu_vapor_lc
      real,pointer,dimension(:,:)   :: qmsqu_transp
      real,pointer,dimension(:,:)   :: qmsqu_sensible_wc
      real,pointer,dimension(:,:)   :: qmsqu_vapor_wc
      real,pointer,dimension(:,:)   :: qmsqu_rh
      real,pointer,dimension(:,:)   :: qmsqu_cwd_rh
      real,pointer,dimension(:,:)   :: qmsqu_nep
      real,pointer,dimension(:,:)   :: qmsqu_rlongup
      real,pointer,dimension(:,:)   :: qmsqu_parup
      real,pointer,dimension(:,:)   :: qmsqu_nirup
      real,pointer,dimension(:,:)   :: qmsqu_rshortup
      real,pointer,dimension(:,:)   :: qmsqu_rnet
      real,pointer,dimension(:,:)   :: qmsqu_albedo
      real,pointer,dimension(:,:)   :: qmsqu_ustar
      real,pointer,dimension(:,:)   :: qmsqu_carbon_ac
      real,pointer,dimension(:,:)   :: qmsqu_carbon_st
      real,pointer,dimension(:,:)   :: qmsqu_vapor_gc
      real,pointer,dimension(:,:)   :: qmsqu_vapor_ac
      real,pointer,dimension(:,:)   :: qmsqu_sensible_gc
      real,pointer,dimension(:,:)   :: qmsqu_sensible_ac
      !------------------------------------------------------------------------------------!
   end type edtype
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !  GLOBAL VARIABLES -- These variables are allocated and assigned before the parallel   !
   !                      distribution, so we don't want to keep it inside the structure.  !
   !                      In case of serial runs, we should have offset full of zeroes and !
   !                      the polygons numbered from 1 to the number of polygons.  In a    !
   !                      non-POI parallel run, we want to allocate "mpolygons" in each    !
   !                      node, but then must keep track of the actual polygon "ID" to     !
   !                      write the output correctly.                                      !
   !---------------------------------------------------------------------------------------!
   ! Number of polygons in each grid, for each machine. so this has a ngrids size. 
   integer, dimension(maxmach,maxgrds) :: gdpy

   ! Number of sites in each grid, for each machine.
   integer, dimension(maxmach,maxgrds) :: gdsi

   ! Number of patches in each grid, for each machine.
   integer, dimension(maxmach,maxgrds) :: gdpa

   ! Number of cohorts in each grid, for each machine.
   integer, dimension(maxmach,maxgrds) :: gdco
  

   ! Offset for each machine, so this has a nmachs size.
   integer, dimension(maxmach,maxgrds) :: py_off 
  
   ! Offset for each machine, so this has a nmachs size.
   integer, dimension(maxmach,maxgrds) :: si_off 

   ! Offset for each machine, so this has a nmachs size.
   integer, dimension(maxmach,maxgrds) :: pa_off 

   ! Offset for each machine, so this has a nmachs size.
   integer, dimension(maxmach,maxgrds) :: co_off 


   type(edtype),pointer,dimension(:) :: edgrid_g

   ! The following are swap variables used during
   ! deallocation-reallocation

   type(edtype)      :: edswap_g

   type(polygontype) :: polyswap_g

   type(sitetype)    :: siteswap_g

   type(patchtype)   :: patchswap_g
  
   !---------------------------------------------------------------------------------------!
   !   The following variables are for tracking the number of variables written in the     !
   ! output, this way we avoid having the same ID used twice.                              !
   !---------------------------------------------------------------------------------------!
   integer :: nioglobal, niogrid, niopoly, niosite

   !---------------------------------------------------------------------------------------!
   ! Logical switch that decides if the pointer tables for IO need to be updated
   ! The number and allocation of cohorts and patches dictates this, and they 
   ! change at a monthly frequency typically.
   !---------------------------------------------------------------------------------------!
   logical :: filltables
   !=======================================================================================!
   !=======================================================================================!


   contains



   !=======================================================================================!
   !=======================================================================================!
   !      This sub-routine allocates the global dimensions (outside the structure).        !
   !---------------------------------------------------------------------------------------!
   subroutine allocate_edglobals(ngrids)

      use ed_var_tables, only : num_var  & ! intent(in)
                              , vt_info  & ! intent(in)
                              , maxvars  ! ! intent(in)
      implicit none

      !----- Arguments. -------------------------------------------------------------------!
      integer :: ngrids
      !------------------------------------------------------------------------------------!
      


      !------------------------------------------------------------------------------------!
      !      Check that the structure is free.                                             !
      !------------------------------------------------------------------------------------!
      if (associated(edgrid_g))  then
         call fatal_error('Attempt to allocate global dimensions again!'                   &
                         ,'allocate_edglobals','ed_state_vars.f90')
      else
         nullify (edgrid_g)
         allocate(edgrid_g(ngrids))
      end if
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !      Allocate the basic variable table structures.                                 !
      !------------------------------------------------------------------------------------!
      allocate(num_var(ngrids))
      num_var = 0
      
      allocate(vt_info(maxvars,ngrids))
      vt_info(:,:)%vector_allocated = .false.


      !----- Initialize the global offsets. -----------------------------------------------!
      edgrid_g(:)%mach_cohort_offset_index  = 0
      edgrid_g(:)%mach_patch_offset_index   = 0
      edgrid_g(:)%mach_site_offset_index    = 0
      edgrid_g(:)%mach_polygon_offset_index = 0
      !------------------------------------------------------------------------------------!


      return
   end subroutine allocate_edglobals
   !=======================================================================================!
   !=======================================================================================!







   !=======================================================================================!
   !=======================================================================================!
   !     This subroutine allocates the polygon-level variables.                            !
   !---------------------------------------------------------------------------------------!
   subroutine allocate_edtype(cgrid,npolygons)
      implicit none

      !----- Arguments. -------------------------------------------------------------------!
      type(edtype), target     :: cgrid
      integer     , intent(in) :: npolygons
      !----- Local variables. -------------------------------------------------------------!
      integer                  :: max_sitep1
      !------------------------------------------------------------------------------------!


      !------ For site adjacency.---------------------------------------------------------!
      max_sitep1 = max_site + 1
      !------------------------------------------------------------------------------------!


      !----- Make sure that we nullify all pointers for a safe allocation. ----------------!
      call nullify_edtype(cgrid)
      !------------------------------------------------------------------------------------!

      !----- Define the number of polygons. -----------------------------------------------!
      cgrid%npolygons = npolygons
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      !       There may be empty grids.  It never happens in the offline runs, but it is   !
      ! common in coupled model settings: some nodes may receive areas exclusively over    !
      ! oceans, and npolygons will be 0 in these nodes.  In this case, we should not       !
      ! attempt to allocate anything.                                                      !
      !------------------------------------------------------------------------------------!
      if (npolygons == 0) return
      !------------------------------------------------------------------------------------!

      allocate(cgrid%polygon                                        (npolygons))


      allocate(cgrid%pysi_id                    (                    npolygons))
      allocate(cgrid%pysi_n                     (                    npolygons))
      allocate(cgrid%walltime_py                (                    npolygons))
      allocate(cgrid%lon                        (                    npolygons))
      allocate(cgrid%lat                        (                    npolygons))
      allocate(cgrid%xatm                       (                    npolygons))
      allocate(cgrid%yatm                       (                    npolygons))
      allocate(cgrid%site_adjacency             (max_site,max_sitep1,npolygons))
      allocate(cgrid%wbar                       (                    npolygons))
      allocate(cgrid%Te                         (                    npolygons))
      allocate(cgrid%zbar                       (                    npolygons))
      allocate(cgrid%sheat                      (                    npolygons))
      allocate(cgrid%baseflow                   (                    npolygons))
      allocate(cgrid%runoff                     (                    npolygons))
      allocate(cgrid%qrunoff                    (                    npolygons))
      allocate(cgrid%swliq                      (                    npolygons))
      allocate(cgrid%ilon                       (                    npolygons))
      allocate(cgrid%ilat                       (                    npolygons))
      allocate(cgrid%workload                   (                 13,npolygons))
      allocate(cgrid%total_agb                  (                    npolygons))
      allocate(cgrid%total_basal_area           (                    npolygons))
      allocate(cgrid%total_agb_growth           (                    npolygons))
      allocate(cgrid%total_agb_mort             (                    npolygons))
      allocate(cgrid%total_agb_recruit          (                    npolygons))
      allocate(cgrid%total_basal_area_growth    (                    npolygons))
      allocate(cgrid%total_basal_area_mort      (                    npolygons))
      allocate(cgrid%total_basal_area_recruit   (                    npolygons))
      allocate(cgrid%nsites                     (                    npolygons))
      allocate(cgrid%sitenums                   (           max_site,npolygons))
      allocate(cgrid%load_adjacency             (                    npolygons))
      allocate(cgrid%metinput                   (                    npolygons))
      allocate(cgrid%met                        (                    npolygons))
      allocate(cgrid%lapse                      (                    npolygons))
      allocate(cgrid%cosz                       (                    npolygons))
      allocate(cgrid%cbudget_initialstorage     (                    npolygons))
      allocate(cgrid%cbudget_nep                (                    npolygons))
      allocate(cgrid%nbudget_initialstorage     (                    npolygons))
      allocate(cgrid%max_leaf_temp              (                    npolygons))
      allocate(cgrid%min_leaf_temp              (                    npolygons))
      allocate(cgrid%max_wood_temp              (                    npolygons))
      allocate(cgrid%min_wood_temp              (                    npolygons))
      allocate(cgrid%max_soil_temp              (                    npolygons))
      allocate(cgrid%min_soil_temp              (                    npolygons))

      allocate(cgrid%nplant                     (   n_pft,     n_dbh,npolygons))
      allocate(cgrid%agb                        (   n_pft,     n_dbh,npolygons))
      allocate(cgrid%lai                        (   n_pft,     n_dbh,npolygons))
      allocate(cgrid%wai                        (   n_pft,     n_dbh,npolygons))
      allocate(cgrid%basal_area                 (   n_pft,     n_dbh,npolygons))
      allocate(cgrid%bdead                      (   n_pft,     n_dbh,npolygons))
      allocate(cgrid%balive                     (   n_pft,     n_dbh,npolygons))
      allocate(cgrid%bleaf                      (   n_pft,     n_dbh,npolygons))
      allocate(cgrid%broot                      (   n_pft,     n_dbh,npolygons))
      allocate(cgrid%bsapwooda                  (   n_pft,     n_dbh,npolygons))
      allocate(cgrid%bsapwoodb                  (   n_pft,     n_dbh,npolygons))
      allocate(cgrid%bseeds                     (   n_pft,     n_dbh,npolygons))
      allocate(cgrid%bstorage                   (   n_pft,     n_dbh,npolygons))
      allocate(cgrid%bdead_n                    (   n_pft,     n_dbh,npolygons))
      allocate(cgrid%balive_n                   (   n_pft,     n_dbh,npolygons))
      allocate(cgrid%bleaf_n                    (   n_pft,     n_dbh,npolygons))
      allocate(cgrid%broot_n                    (   n_pft,     n_dbh,npolygons))
      allocate(cgrid%bsapwooda_n                (   n_pft,     n_dbh,npolygons))
      allocate(cgrid%bsapwoodb_n                (   n_pft,     n_dbh,npolygons))
      allocate(cgrid%bseeds_n                   (   n_pft,     n_dbh,npolygons))
      allocate(cgrid%bstorage_n                 (   n_pft,     n_dbh,npolygons))
      allocate(cgrid%leaf_maintenance           (   n_pft,     n_dbh,npolygons))
      allocate(cgrid%root_maintenance           (   n_pft,     n_dbh,npolygons))
      allocate(cgrid%leaf_drop                  (   n_pft,     n_dbh,npolygons))
      allocate(cgrid%fast_soil_c                (                    npolygons))
      allocate(cgrid%slow_soil_c                (                    npolygons))
      allocate(cgrid%struct_soil_c              (                    npolygons))
      allocate(cgrid%struct_soil_l              (                    npolygons))
      allocate(cgrid%cwd_c                      (                    npolygons))
      allocate(cgrid%fast_soil_n                (                    npolygons))
      allocate(cgrid%mineral_soil_n             (                    npolygons))
      allocate(cgrid%cwd_n                      (                    npolygons))
      allocate(cgrid%Cleaf_grow                 (                    npolygons))
      allocate(cgrid%Croot_grow                 (                    npolygons))
      allocate(cgrid%Cdead_grow                 (                    npolygons))
      allocate(cgrid%Cstore_grow                (                    npolygons))
      allocate(cgrid%Cleaf_litter_flux          (                    npolygons))
      allocate(cgrid%Croot_litter_flux          (                    npolygons))
      allocate(cgrid%Ccwd_flux                  (                    npolygons))
      allocate(cgrid%Nleaf_grow                 (                    npolygons))
      allocate(cgrid%Ndead_grow                 (                    npolygons))
      allocate(cgrid%Nroot_grow                 (                    npolygons))
      allocate(cgrid%Nstore_grow                (                    npolygons))
      allocate(cgrid%Nleaf_litter_flux          (                    npolygons))
      allocate(cgrid%Nroot_litter_flux          (                    npolygons))
      allocate(cgrid%Ncwd_flux                  (                    npolygons))
      allocate(cgrid%Nbiomass_uptake            (                    npolygons))
      allocate(cgrid%Ngross_min                 (                    npolygons))
      allocate(cgrid%Nnet_min                   (                    npolygons))
      allocate(cgrid%avg_lai_ebalvars           (       3,         4,npolygons))

      allocate(cgrid%fmean_gpp                  (                    npolygons))
      allocate(cgrid%fmean_npp                  (                    npolygons))
      allocate(cgrid%fmean_leaf_resp            (                    npolygons))
      allocate(cgrid%fmean_root_resp            (                    npolygons))
      allocate(cgrid%fmean_leaf_growth_resp     (                    npolygons))
      allocate(cgrid%fmean_root_growth_resp     (                    npolygons))
      allocate(cgrid%fmean_sapa_growth_resp     (                    npolygons))
      allocate(cgrid%fmean_sapb_growth_resp     (                    npolygons))
      allocate(cgrid%fmean_leaf_storage_resp    (                    npolygons))
      allocate(cgrid%fmean_root_storage_resp    (                    npolygons))
      allocate(cgrid%fmean_sapa_storage_resp    (                    npolygons))
      allocate(cgrid%fmean_sapb_storage_resp    (                    npolygons))
      allocate(cgrid%fmean_plresp               (                    npolygons))
      allocate(cgrid%fmean_leaf_energy          (                    npolygons))
      allocate(cgrid%fmean_leaf_water           (                    npolygons))
      allocate(cgrid%fmean_leaf_hcap            (                    npolygons))
      allocate(cgrid%fmean_leaf_vpdef           (                    npolygons))
      allocate(cgrid%fmean_leaf_temp            (                    npolygons))
      allocate(cgrid%fmean_leaf_fliq            (                    npolygons))
      allocate(cgrid%fmean_leaf_gsw             (                    npolygons))
      allocate(cgrid%fmean_leaf_gbw             (                    npolygons))
      allocate(cgrid%fmean_wood_energy          (                    npolygons))
      allocate(cgrid%fmean_wood_water           (                    npolygons))
      allocate(cgrid%fmean_wood_hcap            (                    npolygons))
      allocate(cgrid%fmean_wood_temp            (                    npolygons))
      allocate(cgrid%fmean_wood_fliq            (                    npolygons))
      allocate(cgrid%fmean_wood_gbw             (                    npolygons))
      allocate(cgrid%fmean_fs_open              (                    npolygons))
      allocate(cgrid%fmean_fsw                  (                    npolygons))
      allocate(cgrid%fmean_fsn                  (                    npolygons))
      allocate(cgrid%fmean_a_open               (                    npolygons))
      allocate(cgrid%fmean_a_closed             (                    npolygons))
      allocate(cgrid%fmean_a_net                (                    npolygons))
      allocate(cgrid%fmean_a_light              (                    npolygons))
      allocate(cgrid%fmean_a_rubp               (                    npolygons))
      allocate(cgrid%fmean_a_co2                (                    npolygons))
      allocate(cgrid%fmean_psi_open             (                    npolygons))
      allocate(cgrid%fmean_psi_closed           (                    npolygons))
      allocate(cgrid%fmean_water_supply         (                    npolygons))
      allocate(cgrid%fmean_par_l                (                    npolygons))
      allocate(cgrid%fmean_par_l_beam           (                    npolygons))
      allocate(cgrid%fmean_par_l_diff           (                    npolygons))
      allocate(cgrid%fmean_rshort_l             (                    npolygons))
      allocate(cgrid%fmean_rlong_l              (                    npolygons))
      allocate(cgrid%fmean_sensible_lc          (                    npolygons))
      allocate(cgrid%fmean_vapor_lc             (                    npolygons))
      allocate(cgrid%fmean_transp               (                    npolygons))
      allocate(cgrid%fmean_intercepted_al       (                    npolygons))
      allocate(cgrid%fmean_wshed_lg             (                    npolygons))
      allocate(cgrid%fmean_rshort_w             (                    npolygons))
      allocate(cgrid%fmean_rlong_w              (                    npolygons))
      allocate(cgrid%fmean_sensible_wc          (                    npolygons))
      allocate(cgrid%fmean_vapor_wc             (                    npolygons))
      allocate(cgrid%fmean_intercepted_aw       (                    npolygons))
      allocate(cgrid%fmean_wshed_wg             (                    npolygons))
      allocate(cgrid%fmean_rh                   (                    npolygons))
      allocate(cgrid%fmean_cwd_rh               (                    npolygons))
      allocate(cgrid%fmean_nep                  (                    npolygons))
      allocate(cgrid%fmean_rk4step              (                    npolygons))
      allocate(cgrid%fmean_available_water      (                    npolygons))
      allocate(cgrid%fmean_can_theiv            (                    npolygons))
      allocate(cgrid%fmean_can_theta            (                    npolygons))
      allocate(cgrid%fmean_can_vpdef            (                    npolygons))
      allocate(cgrid%fmean_can_temp             (                    npolygons))
      allocate(cgrid%fmean_can_shv              (                    npolygons))
      allocate(cgrid%fmean_can_co2              (                    npolygons))
      allocate(cgrid%fmean_can_rhos             (                    npolygons))
      allocate(cgrid%fmean_can_prss             (                    npolygons))
      allocate(cgrid%fmean_gnd_temp             (                    npolygons))
      allocate(cgrid%fmean_gnd_shv              (                    npolygons))
      allocate(cgrid%fmean_can_ggnd             (                    npolygons))
      allocate(cgrid%fmean_sfcw_depth           (                    npolygons))
      allocate(cgrid%fmean_sfcw_energy          (                    npolygons))
      allocate(cgrid%fmean_sfcw_mass            (                    npolygons))
      allocate(cgrid%fmean_sfcw_temp            (                    npolygons))
      allocate(cgrid%fmean_sfcw_fliq            (                    npolygons))
      allocate(cgrid%fmean_soil_energy          (                nzg,npolygons))
      allocate(cgrid%fmean_soil_mstpot          (                nzg,npolygons))
      allocate(cgrid%fmean_soil_water           (                nzg,npolygons))
      allocate(cgrid%fmean_soil_temp            (                nzg,npolygons))
      allocate(cgrid%fmean_soil_fliq            (                nzg,npolygons))
      allocate(cgrid%fmean_rshort_gnd           (                    npolygons))
      allocate(cgrid%fmean_par_gnd              (                    npolygons))
      allocate(cgrid%fmean_rlong_gnd            (                    npolygons))
      allocate(cgrid%fmean_rlongup              (                    npolygons))
      allocate(cgrid%fmean_parup                (                    npolygons))
      allocate(cgrid%fmean_nirup                (                    npolygons))
      allocate(cgrid%fmean_rshortup             (                    npolygons))
      allocate(cgrid%fmean_rnet                 (                    npolygons))
      allocate(cgrid%fmean_albedo               (                    npolygons))
      allocate(cgrid%fmean_albedo_par           (                    npolygons))
      allocate(cgrid%fmean_albedo_nir           (                    npolygons))
      allocate(cgrid%fmean_rlong_albedo         (                    npolygons))
      allocate(cgrid%fmean_ustar                (                    npolygons))
      allocate(cgrid%fmean_tstar                (                    npolygons))
      allocate(cgrid%fmean_qstar                (                    npolygons))
      allocate(cgrid%fmean_cstar                (                    npolygons))
      allocate(cgrid%fmean_carbon_ac            (                    npolygons))
      allocate(cgrid%fmean_carbon_st            (                    npolygons))
      allocate(cgrid%fmean_vapor_gc             (                    npolygons))
      allocate(cgrid%fmean_vapor_ac             (                    npolygons))
      allocate(cgrid%fmean_smoist_gg            (                nzg,npolygons))
      allocate(cgrid%fmean_throughfall          (                    npolygons))
      allocate(cgrid%fmean_transloss            (                nzg,npolygons))
      allocate(cgrid%fmean_runoff               (                    npolygons))
      allocate(cgrid%fmean_drainage             (                    npolygons))
      allocate(cgrid%fmean_sensible_gc          (                    npolygons))
      allocate(cgrid%fmean_sensible_ac          (                    npolygons))
      allocate(cgrid%fmean_sensible_gg          (                nzg,npolygons))
      allocate(cgrid%fmean_qthroughfall         (                    npolygons))
      allocate(cgrid%fmean_qrunoff              (                    npolygons))
      allocate(cgrid%fmean_qdrainage            (                    npolygons))
      allocate(cgrid%fmean_atm_theiv            (                    npolygons))
      allocate(cgrid%fmean_atm_theta            (                    npolygons))
      allocate(cgrid%fmean_atm_temp             (                    npolygons))
      allocate(cgrid%fmean_atm_vpdef            (                    npolygons))
      allocate(cgrid%fmean_atm_shv              (                    npolygons))
      allocate(cgrid%fmean_atm_rshort           (                    npolygons))
      allocate(cgrid%fmean_atm_rshort_diff      (                    npolygons))
      allocate(cgrid%fmean_atm_par              (                    npolygons))
      allocate(cgrid%fmean_atm_par_diff         (                    npolygons))
      allocate(cgrid%fmean_atm_rlong            (                    npolygons))
      allocate(cgrid%fmean_atm_vels             (                    npolygons))
      allocate(cgrid%fmean_atm_rhos             (                    npolygons))
      allocate(cgrid%fmean_atm_prss             (                    npolygons))
      allocate(cgrid%fmean_atm_co2              (                    npolygons))
      allocate(cgrid%fmean_pcpg                 (                    npolygons))
      allocate(cgrid%fmean_qpcpg                (                    npolygons))
      allocate(cgrid%fmean_dpcpg                (                    npolygons))
      allocate(cgrid%fmean_soil_wetness         (                    npolygons))
      allocate(cgrid%fmean_skin_temp            (                    npolygons))
      allocate(cgrid%fmean_lai			(                    npolygons))
      allocate(cgrid%fmean_bdead 		(                    npolygons))



      !------------------------------------------------------------------------------------!
      !     Allocate the daily means, only if daily means, monthly means, or mean diurnal  !
      ! cycles are requested by the user.                                                  !
      !------------------------------------------------------------------------------------!
      if (writing_long) then
         allocate(cgrid%dmean_nppleaf          (                     npolygons))
         allocate(cgrid%dmean_nppfroot         (                     npolygons))
         allocate(cgrid%dmean_nppsapwood       (                     npolygons))
         allocate(cgrid%dmean_nppcroot         (                     npolygons))
         allocate(cgrid%dmean_nppseeds         (                     npolygons))
         allocate(cgrid%dmean_nppwood          (                     npolygons))
         allocate(cgrid%dmean_nppdaily         (                     npolygons))
         allocate(cgrid%dmean_A_decomp         (                     npolygons))
         allocate(cgrid%dmean_Af_decomp        (                     npolygons))
         allocate(cgrid%dmean_co2_residual     (                     npolygons))
         allocate(cgrid%dmean_energy_residual  (                     npolygons))
         allocate(cgrid%dmean_water_residual   (                     npolygons))
         allocate(cgrid%dmean_gpp              (                     npolygons))
         allocate(cgrid%dmean_npp              (                     npolygons))
         allocate(cgrid%dmean_leaf_resp        (                     npolygons))
         allocate(cgrid%dmean_root_resp        (                     npolygons))
         allocate(cgrid%dmean_leaf_growth_resp (                     npolygons))
         allocate(cgrid%dmean_root_growth_resp (                     npolygons))
         allocate(cgrid%dmean_sapa_growth_resp (                     npolygons))
         allocate(cgrid%dmean_sapb_growth_resp (                     npolygons))
         allocate(cgrid%dmean_leaf_storage_resp(                     npolygons))
         allocate(cgrid%dmean_root_storage_resp(                     npolygons))
         allocate(cgrid%dmean_sapa_storage_resp(                     npolygons))
         allocate(cgrid%dmean_sapb_storage_resp(                     npolygons))
         allocate(cgrid%dmean_plresp           (                     npolygons))
         allocate(cgrid%dmean_leaf_energy      (                     npolygons))
         allocate(cgrid%dmean_leaf_water       (                     npolygons))
         allocate(cgrid%dmean_leaf_hcap        (                     npolygons))
         allocate(cgrid%dmean_leaf_vpdef       (                     npolygons))
         allocate(cgrid%dmean_leaf_temp        (                     npolygons))
         allocate(cgrid%dmean_leaf_fliq        (                     npolygons))
         allocate(cgrid%dmean_leaf_gsw         (                     npolygons))
         allocate(cgrid%dmean_leaf_gbw         (                     npolygons))
         allocate(cgrid%dmean_wood_energy      (                     npolygons))
         allocate(cgrid%dmean_wood_water       (                     npolygons))
         allocate(cgrid%dmean_wood_hcap        (                     npolygons))
         allocate(cgrid%dmean_wood_temp        (                     npolygons))
         allocate(cgrid%dmean_wood_fliq        (                     npolygons))
         allocate(cgrid%dmean_wood_gbw         (                     npolygons))
         allocate(cgrid%dmean_fs_open          (                     npolygons))
         allocate(cgrid%dmean_fsw              (                     npolygons))
         allocate(cgrid%dmean_fsn              (                     npolygons))
         allocate(cgrid%dmean_a_open           (                     npolygons))
         allocate(cgrid%dmean_a_closed         (                     npolygons))
         allocate(cgrid%dmean_a_net            (                     npolygons))
         allocate(cgrid%dmean_a_light          (                     npolygons))
         allocate(cgrid%dmean_a_rubp           (                     npolygons))
         allocate(cgrid%dmean_a_co2            (                     npolygons))
         allocate(cgrid%dmean_psi_open         (                     npolygons))
         allocate(cgrid%dmean_psi_closed       (                     npolygons))
         allocate(cgrid%dmean_water_supply     (                     npolygons))
         allocate(cgrid%dmean_par_l            (                     npolygons))
         allocate(cgrid%dmean_par_l_beam       (                     npolygons))
         allocate(cgrid%dmean_par_l_diff       (                     npolygons))
         allocate(cgrid%dmean_rshort_l         (                     npolygons))
         allocate(cgrid%dmean_rlong_l          (                     npolygons))
         allocate(cgrid%dmean_sensible_lc      (                     npolygons))
         allocate(cgrid%dmean_vapor_lc         (                     npolygons))
         allocate(cgrid%dmean_transp           (                     npolygons))
         allocate(cgrid%dmean_intercepted_al   (                     npolygons))
         allocate(cgrid%dmean_wshed_lg         (                     npolygons))
         allocate(cgrid%dmean_rshort_w         (                     npolygons))
         allocate(cgrid%dmean_rlong_w          (                     npolygons))
         allocate(cgrid%dmean_sensible_wc      (                     npolygons))
         allocate(cgrid%dmean_vapor_wc         (                     npolygons))
         allocate(cgrid%dmean_intercepted_aw   (                     npolygons))
         allocate(cgrid%dmean_wshed_wg         (                     npolygons))
         allocate(cgrid%dmean_rh               (                     npolygons))
         allocate(cgrid%dmean_cwd_rh           (                     npolygons))
         allocate(cgrid%dmean_nep              (                     npolygons))
         allocate(cgrid%dmean_rk4step          (                     npolygons))
         allocate(cgrid%dmean_available_water  (                     npolygons))
         allocate(cgrid%dmean_can_theiv        (                     npolygons))
         allocate(cgrid%dmean_can_theta        (                     npolygons))
         allocate(cgrid%dmean_can_vpdef        (                     npolygons))
         allocate(cgrid%dmean_can_temp         (                     npolygons))
         allocate(cgrid%dmean_can_shv          (                     npolygons))
         allocate(cgrid%dmean_can_co2          (                     npolygons))
         allocate(cgrid%dmean_can_rhos         (                     npolygons))
         allocate(cgrid%dmean_can_prss         (                     npolygons))
         allocate(cgrid%dmean_gnd_temp         (                     npolygons))
         allocate(cgrid%dmean_gnd_shv          (                     npolygons))
         allocate(cgrid%dmean_can_ggnd         (                     npolygons))
         allocate(cgrid%dmean_sfcw_depth       (                     npolygons))
         allocate(cgrid%dmean_sfcw_energy      (                     npolygons))
         allocate(cgrid%dmean_sfcw_mass        (                     npolygons))
         allocate(cgrid%dmean_sfcw_temp        (                     npolygons))
         allocate(cgrid%dmean_sfcw_fliq        (                     npolygons))
         allocate(cgrid%dmean_soil_energy      (                 nzg,npolygons))
         allocate(cgrid%dmean_soil_mstpot      (                 nzg,npolygons))
         allocate(cgrid%dmean_soil_water       (                 nzg,npolygons))
         allocate(cgrid%dmean_soil_temp        (                 nzg,npolygons))
         allocate(cgrid%dmean_soil_fliq        (                 nzg,npolygons))
         allocate(cgrid%dmean_rshort_gnd       (                     npolygons))
         allocate(cgrid%dmean_par_gnd          (                     npolygons))
         allocate(cgrid%dmean_rlong_gnd        (                     npolygons))
         allocate(cgrid%dmean_rlongup          (                     npolygons))
         allocate(cgrid%dmean_parup            (                     npolygons))
         allocate(cgrid%dmean_nirup            (                     npolygons))
         allocate(cgrid%dmean_rshortup         (                     npolygons))
         allocate(cgrid%dmean_rnet             (                     npolygons))
         allocate(cgrid%dmean_albedo           (                     npolygons))
         allocate(cgrid%dmean_albedo_par       (                     npolygons))
         allocate(cgrid%dmean_albedo_nir       (                     npolygons))
         allocate(cgrid%dmean_rlong_albedo     (                     npolygons))
         allocate(cgrid%dmean_ustar            (                     npolygons))
         allocate(cgrid%dmean_tstar            (                     npolygons))
         allocate(cgrid%dmean_qstar            (                     npolygons))
         allocate(cgrid%dmean_cstar            (                     npolygons))
         allocate(cgrid%dmean_carbon_ac        (                     npolygons))
         allocate(cgrid%dmean_carbon_st        (                     npolygons))
         allocate(cgrid%dmean_vapor_gc         (                     npolygons))
         allocate(cgrid%dmean_vapor_ac         (                     npolygons))
         allocate(cgrid%dmean_smoist_gg        (                 nzg,npolygons))
         allocate(cgrid%dmean_throughfall      (                     npolygons))
         allocate(cgrid%dmean_transloss        (                 nzg,npolygons))
         allocate(cgrid%dmean_runoff           (                     npolygons))
         allocate(cgrid%dmean_drainage         (                     npolygons))
         allocate(cgrid%dmean_sensible_gc      (                     npolygons))
         allocate(cgrid%dmean_sensible_ac      (                     npolygons))
         allocate(cgrid%dmean_sensible_gg      (                 nzg,npolygons))
         allocate(cgrid%dmean_qthroughfall     (                     npolygons))
         allocate(cgrid%dmean_qrunoff          (                     npolygons))
         allocate(cgrid%dmean_qdrainage        (                     npolygons))
         allocate(cgrid%dmean_atm_theiv        (                     npolygons))
         allocate(cgrid%dmean_atm_theta        (                     npolygons))
         allocate(cgrid%dmean_atm_temp         (                     npolygons))
         allocate(cgrid%dmean_atm_vpdef        (                     npolygons))
         allocate(cgrid%dmean_atm_shv          (                     npolygons))
         allocate(cgrid%dmean_atm_rshort       (                     npolygons))
         allocate(cgrid%dmean_atm_rshort_diff  (                     npolygons))
         allocate(cgrid%dmean_atm_par          (                     npolygons))
         allocate(cgrid%dmean_atm_par_diff     (                     npolygons))
         allocate(cgrid%dmean_atm_rlong        (                     npolygons))
         allocate(cgrid%dmean_atm_vels         (                     npolygons))
         allocate(cgrid%dmean_atm_rhos         (                     npolygons))
         allocate(cgrid%dmean_atm_prss         (                     npolygons))
         allocate(cgrid%dmean_atm_co2          (                     npolygons))
         allocate(cgrid%dmean_pcpg             (                     npolygons))
         allocate(cgrid%dmean_qpcpg            (                     npolygons))
         allocate(cgrid%dmean_dpcpg            (                     npolygons))
      end if
      !------------------------------------------------------------------------------------!




      !------------------------------------------------------------------------------------!
      ! Allocate the monthly means only if monthly means or mean diurnal cycles are        !
      ! requested by the user.                                                             !
      !------------------------------------------------------------------------------------!
      if (writing_eorq) then
         allocate(cgrid%mmean_lai              (   n_pft,      n_dbh,npolygons))
         allocate(cgrid%mmean_bleaf            (   n_pft,      n_dbh,npolygons))
         allocate(cgrid%mmean_broot            (   n_pft,      n_dbh,npolygons))
         allocate(cgrid%mmean_bstorage         (   n_pft,      n_dbh,npolygons))
         allocate(cgrid%mmean_bleaf_n          (   n_pft,      n_dbh,npolygons))
         allocate(cgrid%mmean_broot_n          (   n_pft,      n_dbh,npolygons))
         allocate(cgrid%mmean_bstorage_n       (   n_pft,      n_dbh,npolygons))
         allocate(cgrid%mmean_leaf_maintenance (   n_pft,      n_dbh,npolygons))
         allocate(cgrid%mmean_root_maintenance (   n_pft,      n_dbh,npolygons))
         allocate(cgrid%mmean_leaf_drop        (   n_pft,      n_dbh,npolygons))
         allocate(cgrid%mmean_fast_soil_c      (                     npolygons)) 
         allocate(cgrid%mmean_slow_soil_c      (                     npolygons)) 
         allocate(cgrid%mmean_struct_soil_c    (                     npolygons)) 
         allocate(cgrid%mmean_struct_soil_l    (                     npolygons)) 
         allocate(cgrid%mmean_cwd_c            (                     npolygons)) 
         allocate(cgrid%mmean_fast_soil_n      (                     npolygons)) 
         allocate(cgrid%mmean_mineral_soil_n   (                     npolygons)) 
         allocate(cgrid%mmean_cwd_n            (                     npolygons)) 
         allocate(cgrid%mmean_gpp              (                     npolygons)) 
         allocate(cgrid%mmean_npp              (                     npolygons)) 
         allocate(cgrid%mmean_leaf_resp        (                     npolygons)) 
         allocate(cgrid%mmean_root_resp        (                     npolygons)) 
         allocate(cgrid%mmean_leaf_growth_resp (                     npolygons)) 
         allocate(cgrid%mmean_root_growth_resp (                     npolygons)) 
         allocate(cgrid%mmean_sapa_growth_resp (                     npolygons)) 
         allocate(cgrid%mmean_sapb_growth_resp (                     npolygons)) 
         allocate(cgrid%mmean_leaf_storage_resp(                     npolygons)) 
         allocate(cgrid%mmean_root_storage_resp(                     npolygons)) 
         allocate(cgrid%mmean_sapa_storage_resp(                     npolygons)) 
         allocate(cgrid%mmean_sapb_storage_resp(                     npolygons)) 
         allocate(cgrid%mmean_plresp           (                     npolygons)) 
         allocate(cgrid%mmean_leaf_energy      (                     npolygons)) 
         allocate(cgrid%mmean_leaf_water       (                     npolygons)) 
         allocate(cgrid%mmean_leaf_hcap        (                     npolygons)) 
         allocate(cgrid%mmean_leaf_vpdef       (                     npolygons)) 
         allocate(cgrid%mmean_leaf_temp        (                     npolygons)) 
         allocate(cgrid%mmean_leaf_fliq        (                     npolygons)) 
         allocate(cgrid%mmean_leaf_gsw         (                     npolygons)) 
         allocate(cgrid%mmean_leaf_gbw         (                     npolygons)) 
         allocate(cgrid%mmean_wood_energy      (                     npolygons)) 
         allocate(cgrid%mmean_wood_water       (                     npolygons)) 
         allocate(cgrid%mmean_wood_hcap        (                     npolygons)) 
         allocate(cgrid%mmean_wood_temp        (                     npolygons)) 
         allocate(cgrid%mmean_wood_fliq        (                     npolygons)) 
         allocate(cgrid%mmean_wood_gbw         (                     npolygons)) 
         allocate(cgrid%mmean_fs_open          (                     npolygons)) 
         allocate(cgrid%mmean_fsw              (                     npolygons)) 
         allocate(cgrid%mmean_fsn              (                     npolygons)) 
         allocate(cgrid%mmean_a_open           (                     npolygons))
         allocate(cgrid%mmean_a_closed         (                     npolygons))
         allocate(cgrid%mmean_a_net            (                     npolygons))
         allocate(cgrid%mmean_a_light          (                     npolygons))
         allocate(cgrid%mmean_a_rubp           (                     npolygons))
         allocate(cgrid%mmean_a_co2            (                     npolygons))
         allocate(cgrid%mmean_psi_open         (                     npolygons)) 
         allocate(cgrid%mmean_psi_closed       (                     npolygons)) 
         allocate(cgrid%mmean_water_supply     (                     npolygons)) 
         allocate(cgrid%mmean_par_l            (                     npolygons)) 
         allocate(cgrid%mmean_par_l_beam       (                     npolygons)) 
         allocate(cgrid%mmean_par_l_diff       (                     npolygons)) 
         allocate(cgrid%mmean_rshort_l         (                     npolygons)) 
         allocate(cgrid%mmean_rlong_l          (                     npolygons)) 
         allocate(cgrid%mmean_sensible_lc      (                     npolygons)) 
         allocate(cgrid%mmean_vapor_lc         (                     npolygons)) 
         allocate(cgrid%mmean_transp           (                     npolygons)) 
         allocate(cgrid%mmean_intercepted_al   (                     npolygons)) 
         allocate(cgrid%mmean_wshed_lg         (                     npolygons)) 
         allocate(cgrid%mmean_rshort_w         (                     npolygons)) 
         allocate(cgrid%mmean_rlong_w          (                     npolygons)) 
         allocate(cgrid%mmean_sensible_wc      (                     npolygons)) 
         allocate(cgrid%mmean_vapor_wc         (                     npolygons)) 
         allocate(cgrid%mmean_intercepted_aw   (                     npolygons)) 
         allocate(cgrid%mmean_wshed_wg         (                     npolygons)) 
         allocate(cgrid%mmean_rh               (                     npolygons)) 
         allocate(cgrid%mmean_cwd_rh           (                     npolygons)) 
         allocate(cgrid%mmean_nep              (                     npolygons)) 
         allocate(cgrid%mmean_rk4step          (                     npolygons)) 
         allocate(cgrid%mmean_available_water  (                     npolygons)) 
         allocate(cgrid%mmean_can_theiv        (                     npolygons)) 
         allocate(cgrid%mmean_can_theta        (                     npolygons)) 
         allocate(cgrid%mmean_can_vpdef        (                     npolygons)) 
         allocate(cgrid%mmean_can_temp         (                     npolygons)) 
         allocate(cgrid%mmean_can_shv          (                     npolygons)) 
         allocate(cgrid%mmean_can_co2          (                     npolygons)) 
         allocate(cgrid%mmean_can_rhos         (                     npolygons)) 
         allocate(cgrid%mmean_can_prss         (                     npolygons)) 
         allocate(cgrid%mmean_gnd_temp         (                     npolygons)) 
         allocate(cgrid%mmean_gnd_shv          (                     npolygons)) 
         allocate(cgrid%mmean_can_ggnd         (                     npolygons)) 
         allocate(cgrid%mmean_sfcw_depth       (                     npolygons)) 
         allocate(cgrid%mmean_sfcw_energy      (                     npolygons)) 
         allocate(cgrid%mmean_sfcw_mass        (                     npolygons)) 
         allocate(cgrid%mmean_sfcw_temp        (                     npolygons)) 
         allocate(cgrid%mmean_sfcw_fliq        (                     npolygons)) 
         allocate(cgrid%mmean_soil_energy      (                 nzg,npolygons)) 
         allocate(cgrid%mmean_soil_mstpot      (                 nzg,npolygons)) 
         allocate(cgrid%mmean_soil_water       (                 nzg,npolygons)) 
         allocate(cgrid%mmean_soil_temp        (                 nzg,npolygons)) 
         allocate(cgrid%mmean_soil_fliq        (                 nzg,npolygons)) 
         allocate(cgrid%mmean_rshort_gnd       (                     npolygons)) 
         allocate(cgrid%mmean_par_gnd          (                     npolygons)) 
         allocate(cgrid%mmean_rlong_gnd        (                     npolygons)) 
         allocate(cgrid%mmean_rlongup          (                     npolygons)) 
         allocate(cgrid%mmean_parup            (                     npolygons)) 
         allocate(cgrid%mmean_nirup            (                     npolygons)) 
         allocate(cgrid%mmean_rshortup         (                     npolygons)) 
         allocate(cgrid%mmean_rnet             (                     npolygons)) 
         allocate(cgrid%mmean_albedo           (                     npolygons)) 
         allocate(cgrid%mmean_albedo_par       (                     npolygons)) 
         allocate(cgrid%mmean_albedo_nir       (                     npolygons)) 
         allocate(cgrid%mmean_rlong_albedo     (                     npolygons)) 
         allocate(cgrid%mmean_ustar            (                     npolygons)) 
         allocate(cgrid%mmean_tstar            (                     npolygons)) 
         allocate(cgrid%mmean_qstar            (                     npolygons)) 
         allocate(cgrid%mmean_cstar            (                     npolygons)) 
         allocate(cgrid%mmean_carbon_ac        (                     npolygons)) 
         allocate(cgrid%mmean_carbon_st        (                     npolygons)) 
         allocate(cgrid%mmean_vapor_gc         (                     npolygons)) 
         allocate(cgrid%mmean_vapor_ac         (                     npolygons)) 
         allocate(cgrid%mmean_smoist_gg        (                 nzg,npolygons)) 
         allocate(cgrid%mmean_throughfall      (                     npolygons)) 
         allocate(cgrid%mmean_transloss        (                 nzg,npolygons)) 
         allocate(cgrid%mmean_runoff           (                     npolygons)) 
         allocate(cgrid%mmean_drainage         (                     npolygons)) 
         allocate(cgrid%mmean_sensible_gc      (                     npolygons)) 
         allocate(cgrid%mmean_sensible_ac      (                     npolygons)) 
         allocate(cgrid%mmean_sensible_gg      (                 nzg,npolygons)) 
         allocate(cgrid%mmean_qthroughfall     (                     npolygons)) 
         allocate(cgrid%mmean_qrunoff          (                     npolygons)) 
         allocate(cgrid%mmean_qdrainage        (                     npolygons)) 
         allocate(cgrid%mmean_nppleaf          (                     npolygons))
         allocate(cgrid%mmean_nppfroot         (                     npolygons))
         allocate(cgrid%mmean_nppsapwood       (                     npolygons))
         allocate(cgrid%mmean_nppcroot         (                     npolygons))
         allocate(cgrid%mmean_nppseeds         (                     npolygons))
         allocate(cgrid%mmean_nppwood          (                     npolygons))
         allocate(cgrid%mmean_nppdaily         (                     npolygons))
         allocate(cgrid%mmean_A_decomp         (                     npolygons)) 
         allocate(cgrid%mmean_Af_decomp        (                     npolygons)) 
         allocate(cgrid%mmean_co2_residual     (                     npolygons)) 
         allocate(cgrid%mmean_energy_residual  (                     npolygons)) 
         allocate(cgrid%mmean_water_residual   (                     npolygons)) 
         allocate(cgrid%mmean_atm_theiv        (                     npolygons)) 
         allocate(cgrid%mmean_atm_theta        (                     npolygons)) 
         allocate(cgrid%mmean_atm_temp         (                     npolygons)) 
         allocate(cgrid%mmean_atm_vpdef        (                     npolygons)) 
         allocate(cgrid%mmean_atm_shv          (                     npolygons)) 
         allocate(cgrid%mmean_atm_rshort       (                     npolygons)) 
         allocate(cgrid%mmean_atm_rshort_diff  (                     npolygons)) 
         allocate(cgrid%mmean_atm_par          (                     npolygons)) 
         allocate(cgrid%mmean_atm_par_diff     (                     npolygons)) 
         allocate(cgrid%mmean_atm_rlong        (                     npolygons)) 
         allocate(cgrid%mmean_atm_vels         (                     npolygons)) 
         allocate(cgrid%mmean_atm_rhos         (                     npolygons)) 
         allocate(cgrid%mmean_atm_prss         (                     npolygons)) 
         allocate(cgrid%mmean_atm_co2          (                     npolygons)) 
         allocate(cgrid%mmean_pcpg             (                     npolygons)) 
         allocate(cgrid%mmean_qpcpg            (                     npolygons)) 
         allocate(cgrid%mmean_dpcpg            (                     npolygons)) 
         allocate(cgrid%mmsqu_gpp              (                     npolygons)) 
         allocate(cgrid%mmsqu_npp              (                     npolygons)) 
         allocate(cgrid%mmsqu_plresp           (                     npolygons)) 
         allocate(cgrid%mmsqu_sensible_lc      (                     npolygons)) 
         allocate(cgrid%mmsqu_vapor_lc         (                     npolygons)) 
         allocate(cgrid%mmsqu_transp           (                     npolygons)) 
         allocate(cgrid%mmsqu_sensible_wc      (                     npolygons)) 
         allocate(cgrid%mmsqu_vapor_wc         (                     npolygons)) 
         allocate(cgrid%mmsqu_rh               (                     npolygons)) 
         allocate(cgrid%mmsqu_cwd_rh           (                     npolygons)) 
         allocate(cgrid%mmsqu_nep              (                     npolygons)) 
         allocate(cgrid%mmsqu_rlongup          (                     npolygons)) 
         allocate(cgrid%mmsqu_parup            (                     npolygons)) 
         allocate(cgrid%mmsqu_nirup            (                     npolygons)) 
         allocate(cgrid%mmsqu_rshortup         (                     npolygons)) 
         allocate(cgrid%mmsqu_rnet             (                     npolygons)) 
         allocate(cgrid%mmsqu_albedo           (                     npolygons)) 
         allocate(cgrid%mmsqu_ustar            (                     npolygons)) 
         allocate(cgrid%mmsqu_carbon_ac        (                     npolygons)) 
         allocate(cgrid%mmsqu_carbon_st        (                     npolygons)) 
         allocate(cgrid%mmsqu_vapor_gc         (                     npolygons)) 
         allocate(cgrid%mmsqu_vapor_ac         (                     npolygons)) 
         allocate(cgrid%mmsqu_sensible_gc      (                     npolygons)) 
         allocate(cgrid%mmsqu_sensible_ac      (                     npolygons)) 
      end if
      !------------------------------------------------------------------------------------!




      !------------------------------------------------------------------------------------!
      !      Allocate the mean diurnal cycles only if mean diurnal cycles are requested    !
      ! by the user.                                                                       !
      !------------------------------------------------------------------------------------!
      if (writing_dcyc) then
         allocate(cgrid%qmean_gpp              (             ndcycle,npolygons))
         allocate(cgrid%qmean_npp              (             ndcycle,npolygons))
         allocate(cgrid%qmean_leaf_resp        (             ndcycle,npolygons))
         allocate(cgrid%qmean_root_resp        (             ndcycle,npolygons))
         allocate(cgrid%qmean_leaf_growth_resp (             ndcycle,npolygons))
         allocate(cgrid%qmean_root_growth_resp (             ndcycle,npolygons))
         allocate(cgrid%qmean_sapa_growth_resp (             ndcycle,npolygons))
         allocate(cgrid%qmean_sapb_growth_resp (             ndcycle,npolygons))
         allocate(cgrid%qmean_leaf_storage_resp(             ndcycle,npolygons))
         allocate(cgrid%qmean_root_storage_resp(             ndcycle,npolygons))
         allocate(cgrid%qmean_sapa_storage_resp(             ndcycle,npolygons))
         allocate(cgrid%qmean_sapb_storage_resp(             ndcycle,npolygons))
         allocate(cgrid%qmean_plresp           (             ndcycle,npolygons))
         allocate(cgrid%qmean_leaf_energy      (             ndcycle,npolygons))
         allocate(cgrid%qmean_leaf_water       (             ndcycle,npolygons))
         allocate(cgrid%qmean_leaf_hcap        (             ndcycle,npolygons))
         allocate(cgrid%qmean_leaf_vpdef       (             ndcycle,npolygons))
         allocate(cgrid%qmean_leaf_temp        (             ndcycle,npolygons))
         allocate(cgrid%qmean_leaf_fliq        (             ndcycle,npolygons))
         allocate(cgrid%qmean_leaf_gsw         (             ndcycle,npolygons))
         allocate(cgrid%qmean_leaf_gbw         (             ndcycle,npolygons))
         allocate(cgrid%qmean_wood_energy      (             ndcycle,npolygons))
         allocate(cgrid%qmean_wood_water       (             ndcycle,npolygons))
         allocate(cgrid%qmean_wood_hcap        (             ndcycle,npolygons))
         allocate(cgrid%qmean_wood_temp        (             ndcycle,npolygons))
         allocate(cgrid%qmean_wood_fliq        (             ndcycle,npolygons))
         allocate(cgrid%qmean_wood_gbw         (             ndcycle,npolygons))
         allocate(cgrid%qmean_fs_open          (             ndcycle,npolygons))
         allocate(cgrid%qmean_fsw              (             ndcycle,npolygons))
         allocate(cgrid%qmean_fsn              (             ndcycle,npolygons))
         allocate(cgrid%qmean_a_open           (             ndcycle,npolygons))
         allocate(cgrid%qmean_a_closed         (             ndcycle,npolygons))
         allocate(cgrid%qmean_a_net            (             ndcycle,npolygons))
         allocate(cgrid%qmean_a_light          (             ndcycle,npolygons))
         allocate(cgrid%qmean_a_rubp           (             ndcycle,npolygons))
         allocate(cgrid%qmean_a_co2            (             ndcycle,npolygons))
         allocate(cgrid%qmean_psi_open         (             ndcycle,npolygons))
         allocate(cgrid%qmean_psi_closed       (             ndcycle,npolygons))
         allocate(cgrid%qmean_water_supply     (             ndcycle,npolygons))
         allocate(cgrid%qmean_par_l            (             ndcycle,npolygons))
         allocate(cgrid%qmean_par_l_beam       (             ndcycle,npolygons))
         allocate(cgrid%qmean_par_l_diff       (             ndcycle,npolygons))
         allocate(cgrid%qmean_rshort_l         (             ndcycle,npolygons))
         allocate(cgrid%qmean_rlong_l          (             ndcycle,npolygons))
         allocate(cgrid%qmean_sensible_lc      (             ndcycle,npolygons))
         allocate(cgrid%qmean_vapor_lc         (             ndcycle,npolygons))
         allocate(cgrid%qmean_transp           (             ndcycle,npolygons))
         allocate(cgrid%qmean_intercepted_al   (             ndcycle,npolygons))
         allocate(cgrid%qmean_wshed_lg         (             ndcycle,npolygons))
         allocate(cgrid%qmean_rshort_w         (             ndcycle,npolygons))
         allocate(cgrid%qmean_rlong_w          (             ndcycle,npolygons))
         allocate(cgrid%qmean_sensible_wc      (             ndcycle,npolygons))
         allocate(cgrid%qmean_vapor_wc         (             ndcycle,npolygons))
         allocate(cgrid%qmean_intercepted_aw   (             ndcycle,npolygons))
         allocate(cgrid%qmean_wshed_wg         (             ndcycle,npolygons))
         allocate(cgrid%qmean_rh               (             ndcycle,npolygons))
         allocate(cgrid%qmean_cwd_rh           (             ndcycle,npolygons))
         allocate(cgrid%qmean_nep              (             ndcycle,npolygons))
         allocate(cgrid%qmean_rk4step          (             ndcycle,npolygons))
         allocate(cgrid%qmean_available_water  (             ndcycle,npolygons))
         allocate(cgrid%qmean_can_theiv        (             ndcycle,npolygons))
         allocate(cgrid%qmean_can_theta        (             ndcycle,npolygons))
         allocate(cgrid%qmean_can_vpdef        (             ndcycle,npolygons))
         allocate(cgrid%qmean_can_temp         (             ndcycle,npolygons))
         allocate(cgrid%qmean_can_shv          (             ndcycle,npolygons))
         allocate(cgrid%qmean_can_co2          (             ndcycle,npolygons))
         allocate(cgrid%qmean_can_rhos         (             ndcycle,npolygons))
         allocate(cgrid%qmean_can_prss         (             ndcycle,npolygons))
         allocate(cgrid%qmean_gnd_temp         (             ndcycle,npolygons))
         allocate(cgrid%qmean_gnd_shv          (             ndcycle,npolygons))
         allocate(cgrid%qmean_can_ggnd         (             ndcycle,npolygons))
         allocate(cgrid%qmean_sfcw_depth       (             ndcycle,npolygons))
         allocate(cgrid%qmean_sfcw_energy      (             ndcycle,npolygons))
         allocate(cgrid%qmean_sfcw_mass        (             ndcycle,npolygons))
         allocate(cgrid%qmean_sfcw_temp        (             ndcycle,npolygons))
         allocate(cgrid%qmean_sfcw_fliq        (             ndcycle,npolygons))
         allocate(cgrid%qmean_soil_energy      (     nzg,    ndcycle,npolygons))
         allocate(cgrid%qmean_soil_mstpot      (     nzg,    ndcycle,npolygons))
         allocate(cgrid%qmean_soil_water       (     nzg,    ndcycle,npolygons))
         allocate(cgrid%qmean_soil_temp        (     nzg,    ndcycle,npolygons))
         allocate(cgrid%qmean_soil_fliq        (     nzg,    ndcycle,npolygons))
         allocate(cgrid%qmean_rshort_gnd       (             ndcycle,npolygons))
         allocate(cgrid%qmean_par_gnd          (             ndcycle,npolygons))
         allocate(cgrid%qmean_rlong_gnd        (             ndcycle,npolygons))
         allocate(cgrid%qmean_rlongup          (             ndcycle,npolygons))
         allocate(cgrid%qmean_parup            (             ndcycle,npolygons))
         allocate(cgrid%qmean_nirup            (             ndcycle,npolygons))
         allocate(cgrid%qmean_rshortup         (             ndcycle,npolygons))
         allocate(cgrid%qmean_rnet             (             ndcycle,npolygons))
         allocate(cgrid%qmean_albedo           (             ndcycle,npolygons))
         allocate(cgrid%qmean_albedo_par       (             ndcycle,npolygons))
         allocate(cgrid%qmean_albedo_nir       (             ndcycle,npolygons))
         allocate(cgrid%qmean_rlong_albedo     (             ndcycle,npolygons))
         allocate(cgrid%qmean_ustar            (             ndcycle,npolygons))
         allocate(cgrid%qmean_tstar            (             ndcycle,npolygons))
         allocate(cgrid%qmean_qstar            (             ndcycle,npolygons))
         allocate(cgrid%qmean_cstar            (             ndcycle,npolygons))
         allocate(cgrid%qmean_carbon_ac        (             ndcycle,npolygons))
         allocate(cgrid%qmean_carbon_st        (             ndcycle,npolygons))
         allocate(cgrid%qmean_vapor_gc         (             ndcycle,npolygons))
         allocate(cgrid%qmean_vapor_ac         (             ndcycle,npolygons))
         allocate(cgrid%qmean_smoist_gg        (     nzg,    ndcycle,npolygons))
         allocate(cgrid%qmean_throughfall      (             ndcycle,npolygons))
         allocate(cgrid%qmean_transloss        (     nzg,    ndcycle,npolygons))
         allocate(cgrid%qmean_runoff           (             ndcycle,npolygons))
         allocate(cgrid%qmean_drainage         (             ndcycle,npolygons))
         allocate(cgrid%qmean_sensible_gc      (             ndcycle,npolygons))
         allocate(cgrid%qmean_sensible_ac      (             ndcycle,npolygons))
         allocate(cgrid%qmean_sensible_gg      (     nzg,    ndcycle,npolygons))
         allocate(cgrid%qmean_qthroughfall     (             ndcycle,npolygons))
         allocate(cgrid%qmean_qrunoff          (             ndcycle,npolygons))
         allocate(cgrid%qmean_qdrainage        (             ndcycle,npolygons))
         allocate(cgrid%qmean_atm_theiv        (             ndcycle,npolygons))
         allocate(cgrid%qmean_atm_theta        (             ndcycle,npolygons))
         allocate(cgrid%qmean_atm_temp         (             ndcycle,npolygons))
         allocate(cgrid%qmean_atm_vpdef        (             ndcycle,npolygons))
         allocate(cgrid%qmean_atm_shv          (             ndcycle,npolygons))
         allocate(cgrid%qmean_atm_rshort       (             ndcycle,npolygons))
         allocate(cgrid%qmean_atm_rshort_diff  (             ndcycle,npolygons))
         allocate(cgrid%qmean_atm_par          (             ndcycle,npolygons))
         allocate(cgrid%qmean_atm_par_diff     (             ndcycle,npolygons))
         allocate(cgrid%qmean_atm_rlong        (             ndcycle,npolygons))
         allocate(cgrid%qmean_atm_vels         (             ndcycle,npolygons))
         allocate(cgrid%qmean_atm_rhos         (             ndcycle,npolygons))
         allocate(cgrid%qmean_atm_prss         (             ndcycle,npolygons))
         allocate(cgrid%qmean_atm_co2          (             ndcycle,npolygons))
         allocate(cgrid%qmean_pcpg             (             ndcycle,npolygons))
         allocate(cgrid%qmean_qpcpg            (             ndcycle,npolygons))
         allocate(cgrid%qmean_dpcpg            (             ndcycle,npolygons))
         allocate(cgrid%qmsqu_gpp              (             ndcycle,npolygons))
         allocate(cgrid%qmsqu_npp              (             ndcycle,npolygons))
         allocate(cgrid%qmsqu_plresp           (             ndcycle,npolygons))
         allocate(cgrid%qmsqu_sensible_lc      (             ndcycle,npolygons))
         allocate(cgrid%qmsqu_vapor_lc         (             ndcycle,npolygons))
         allocate(cgrid%qmsqu_transp           (             ndcycle,npolygons))
         allocate(cgrid%qmsqu_sensible_wc      (             ndcycle,npolygons))
         allocate(cgrid%qmsqu_vapor_wc         (             ndcycle,npolygons))
         allocate(cgrid%qmsqu_rh               (             ndcycle,npolygons))
         allocate(cgrid%qmsqu_cwd_rh           (             ndcycle,npolygons))
         allocate(cgrid%qmsqu_nep              (             ndcycle,npolygons))
         allocate(cgrid%qmsqu_rlongup          (             ndcycle,npolygons))
         allocate(cgrid%qmsqu_parup            (             ndcycle,npolygons))
         allocate(cgrid%qmsqu_nirup            (             ndcycle,npolygons))
         allocate(cgrid%qmsqu_rshortup         (             ndcycle,npolygons))
         allocate(cgrid%qmsqu_rnet             (             ndcycle,npolygons))
         allocate(cgrid%qmsqu_albedo           (             ndcycle,npolygons))
         allocate(cgrid%qmsqu_ustar            (             ndcycle,npolygons))
         allocate(cgrid%qmsqu_carbon_ac        (             ndcycle,npolygons))
         allocate(cgrid%qmsqu_carbon_st        (             ndcycle,npolygons))
         allocate(cgrid%qmsqu_vapor_gc         (             ndcycle,npolygons))
         allocate(cgrid%qmsqu_vapor_ac         (             ndcycle,npolygons))
         allocate(cgrid%qmsqu_sensible_gc      (             ndcycle,npolygons))
         allocate(cgrid%qmsqu_sensible_ac      (             ndcycle,npolygons))
      end if
      return
   end subroutine allocate_edtype
   !=======================================================================================!
   !=======================================================================================!





   !=======================================================================================!
   !=======================================================================================!
   !     This subroutine allocates the site-level variables.                               !
   !---------------------------------------------------------------------------------------!
   subroutine allocate_polygontype(cpoly,nsites)

      implicit none

      !----- Arguments. -------------------------------------------------------------------!
      type(polygontype), target     :: cpoly
      integer          , intent(in) :: nsites
      !------------------------------------------------------------------------------------!


      !----- Make sure that we nullify all pointers for a safe allocation. ----------------!
      call nullify_polygontype(cpoly)
      !------------------------------------------------------------------------------------!


      !----- Define the number of sites. --------------------------------------------------!
      cpoly%nsites = nsites
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      !       Up to now, there is no reason for an empty polygon, so the code will always  !
      ! assume that there is at least one site in the polygon.  In case nsites is zero,    !
      ! we should stop the run and warn the user.                                          !
      !------------------------------------------------------------------------------------!
      if (nsites == 0) then
         call fatal_error('Attempt to set a polygon with 0 sites. This cannot happen!'     &
                         ,'allocate_polygontype','ed_state_vars.f90')
      end if
      !------------------------------------------------------------------------------------!


      allocate(cpoly%site                          (                          nsites))

      allocate(cpoly%sipa_id                       (                          nsites))
      allocate(cpoly%sipa_n                        (                          nsites))

      allocate(cpoly%patch_count                   (                          nsites))
      allocate(cpoly%sitenum                       (                          nsites))
      allocate(cpoly%met                           (                          nsites))
      allocate(cpoly%area                          (                          nsites))
      allocate(cpoly%patch_area                    (                          nsites))
      allocate(cpoly%elevation                     (                          nsites))
      allocate(cpoly%slope                         (                          nsites))
      allocate(cpoly%aspect                        (                          nsites))
      allocate(cpoly%lsl                           (                          nsites))
      allocate(cpoly%ncol_soil                     (                          nsites))
      allocate(cpoly%ntext_soil                    (                      nzg,nsites))
      allocate(cpoly%TCI                           (                          nsites))
      allocate(cpoly%pptweight                     (                          nsites))
      allocate(cpoly%hydro_next                    (                          nsites))
      allocate(cpoly%hydro_prev                    (                          nsites))
      allocate(cpoly%moist_W                       (                          nsites))
      allocate(cpoly%moist_f                       (                          nsites))
      allocate(cpoly%moist_tau                     (                          nsites))
      allocate(cpoly%moist_zi                      (                          nsites))
      allocate(cpoly%baseflow                      (                          nsites))
      allocate(cpoly%runoff                        (                          nsites))
      allocate(cpoly%qrunoff                       (                          nsites))
      allocate(cpoly%min_monthly_temp              (                          nsites))
      allocate(cpoly%num_landuse_years             (                          nsites))
      allocate(cpoly%mindbh_primary                (                    n_pft,nsites))
      allocate(cpoly%probharv_primary              (                    n_pft,nsites))
      allocate(cpoly%mindbh_secondary              (                    n_pft,nsites))
      allocate(cpoly%probharv_secondary            (                    n_pft,nsites))
      allocate(cpoly%plantation                    (                          nsites))
      allocate(cpoly%agri_stocking_pft             (                          nsites))
      allocate(cpoly%agri_stocking_density         (                          nsites))
      allocate(cpoly%plantation_stocking_pft       (                          nsites))
      allocate(cpoly%plantation_stocking_density   (                          nsites))
      allocate(cpoly%primary_harvest_target        (                          nsites))
      allocate(cpoly%secondary_harvest_target      (                          nsites))
      allocate(cpoly%primary_harvest_memory        (                          nsites))
      allocate(cpoly%secondary_harvest_memory      (                          nsites))
      allocate(cpoly%ignition_rate                 (                          nsites))
      allocate(cpoly%lambda_fire                   (                       12,nsites))
      allocate(cpoly%avg_monthly_pcpg              (                       12,nsites))
      allocate(cpoly%phen_pars                     (                          nsites))
      allocate(cpoly%disturbance_memory            (n_dist_types,n_dist_types,nsites))
      allocate(cpoly%disturbance_rates             (n_dist_types,n_dist_types,nsites))
      allocate(cpoly%green_leaf_factor             (                    n_pft,nsites))
      allocate(cpoly%leaf_aging_factor             (                    n_pft,nsites))
      allocate(cpoly%basal_area                    (       n_pft,       n_dbh,nsites))
      allocate(cpoly%basal_area_growth             (       n_pft,       n_dbh,nsites))
      allocate(cpoly%agb                           (       n_pft,       n_dbh,nsites))
      allocate(cpoly%agb_growth                    (       n_pft,       n_dbh,nsites))
      allocate(cpoly%basal_area_mort               (       n_pft,       n_dbh,nsites))
      allocate(cpoly%basal_area_cut                (       n_pft,       n_dbh,nsites))
      allocate(cpoly%agb_mort                      (       n_pft,       n_dbh,nsites))
      allocate(cpoly%agb_cut                       (       n_pft,       n_dbh,nsites))

      allocate(cpoly%cosaoi                        (                          nsites))
      allocate(cpoly%daylight                      (                          nsites))
      allocate(cpoly%nighttime                     (                          nsites))
      allocate(cpoly%rad_avg                       (                          nsites))

      allocate(cpoly%fmean_atm_theiv               (                          nsites))
      allocate(cpoly%fmean_atm_theta               (                          nsites))
      allocate(cpoly%fmean_atm_temp                (                          nsites))
      allocate(cpoly%fmean_atm_vpdef               (                          nsites))
      allocate(cpoly%fmean_atm_shv                 (                          nsites))
      allocate(cpoly%fmean_atm_rshort              (                          nsites))
      allocate(cpoly%fmean_atm_rshort_diff         (                          nsites))
      allocate(cpoly%fmean_atm_par                 (                          nsites))
      allocate(cpoly%fmean_atm_par_diff            (                          nsites))
      allocate(cpoly%fmean_atm_rlong               (                          nsites))
      allocate(cpoly%fmean_atm_vels                (                          nsites))
      allocate(cpoly%fmean_atm_rhos                (                          nsites))
      allocate(cpoly%fmean_atm_prss                (                          nsites))
      allocate(cpoly%fmean_atm_co2                 (                          nsites))
      allocate(cpoly%fmean_pcpg                    (                          nsites))
      allocate(cpoly%fmean_qpcpg                   (                          nsites))
      allocate(cpoly%fmean_dpcpg                   (                          nsites))

      if (writing_long) then
         allocate(cpoly%dmean_atm_theiv            (                          nsites))
         allocate(cpoly%dmean_atm_theta            (                          nsites))
         allocate(cpoly%dmean_atm_temp             (                          nsites))
         allocate(cpoly%dmean_atm_vpdef            (                          nsites))
         allocate(cpoly%dmean_atm_shv              (                          nsites))
         allocate(cpoly%dmean_atm_rshort           (                          nsites))
         allocate(cpoly%dmean_atm_rshort_diff      (                          nsites))
         allocate(cpoly%dmean_atm_par              (                          nsites))
         allocate(cpoly%dmean_atm_par_diff         (                          nsites))
         allocate(cpoly%dmean_atm_rlong            (                          nsites))
         allocate(cpoly%dmean_atm_vels             (                          nsites))
         allocate(cpoly%dmean_atm_rhos             (                          nsites))
         allocate(cpoly%dmean_atm_prss             (                          nsites))
         allocate(cpoly%dmean_atm_co2              (                          nsites))
         allocate(cpoly%dmean_pcpg                 (                          nsites))
         allocate(cpoly%dmean_qpcpg                (                          nsites))
         allocate(cpoly%dmean_dpcpg                (                          nsites))
      end if

      if (writing_eorq) then
         allocate(cpoly%mmean_atm_theiv            (                          nsites))
         allocate(cpoly%mmean_atm_theta            (                          nsites))
         allocate(cpoly%mmean_atm_temp             (                          nsites))
         allocate(cpoly%mmean_atm_vpdef            (                          nsites))
         allocate(cpoly%mmean_atm_shv              (                          nsites))
         allocate(cpoly%mmean_atm_rshort           (                          nsites))
         allocate(cpoly%mmean_atm_rshort_diff      (                          nsites))
         allocate(cpoly%mmean_atm_par              (                          nsites))
         allocate(cpoly%mmean_atm_par_diff         (                          nsites))
         allocate(cpoly%mmean_atm_rlong            (                          nsites))
         allocate(cpoly%mmean_atm_vels             (                          nsites))
         allocate(cpoly%mmean_atm_rhos             (                          nsites))
         allocate(cpoly%mmean_atm_prss             (                          nsites))
         allocate(cpoly%mmean_atm_co2              (                          nsites))
         allocate(cpoly%mmean_pcpg                 (                          nsites))
         allocate(cpoly%mmean_qpcpg                (                          nsites))
         allocate(cpoly%mmean_dpcpg                (                          nsites))
      end if

      if (writing_dcyc) then
         allocate(cpoly%qmean_atm_theiv            (                  ndcycle,nsites))
         allocate(cpoly%qmean_atm_theta            (                  ndcycle,nsites))
         allocate(cpoly%qmean_atm_temp             (                  ndcycle,nsites))
         allocate(cpoly%qmean_atm_vpdef            (                  ndcycle,nsites))
         allocate(cpoly%qmean_atm_shv              (                  ndcycle,nsites))
         allocate(cpoly%qmean_atm_rshort           (                  ndcycle,nsites))
         allocate(cpoly%qmean_atm_rshort_diff      (                  ndcycle,nsites))
         allocate(cpoly%qmean_atm_par              (                  ndcycle,nsites))
         allocate(cpoly%qmean_atm_par_diff         (                  ndcycle,nsites))
         allocate(cpoly%qmean_atm_rlong            (                  ndcycle,nsites))
         allocate(cpoly%qmean_atm_vels             (                  ndcycle,nsites))
         allocate(cpoly%qmean_atm_rhos             (                  ndcycle,nsites))
         allocate(cpoly%qmean_atm_prss             (                  ndcycle,nsites))
         allocate(cpoly%qmean_atm_co2              (                  ndcycle,nsites))
         allocate(cpoly%qmean_pcpg                 (                  ndcycle,nsites))
         allocate(cpoly%qmean_qpcpg                (                  ndcycle,nsites))
         allocate(cpoly%qmean_dpcpg                (                  ndcycle,nsites))
      end if

      return
   end subroutine allocate_polygontype
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This subroutine allocates the patch-level variables.                              !
   !---------------------------------------------------------------------------------------!
   subroutine allocate_sitetype(csite,npatches)

      implicit none

      !----- Arguments. -------------------------------------------------------------------!
      type(sitetype), target     :: csite
      integer       , intent(in) :: npatches
      !----- Local variables. -------------------------------------------------------------!
      integer                    :: ipa
      !------------------------------------------------------------------------------------!


      !----- Make sure that we nullify all pointers for a safe allocation. ----------------!
      call nullify_sitetype(csite)
      !------------------------------------------------------------------------------------!


      !----- Define the number of sites. --------------------------------------------------!
      csite%npatches = npatches
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      !       Up to now, there is no reason for an empty site, so the code will always     !
      ! assume that there is at least one patch in the site.  In case npatches is zero,    !
      ! we should stop the run and warn the user.                                          !
      !------------------------------------------------------------------------------------!
      if (npatches == 0) then
         call fatal_error('Attempt to set a site with 0 patches. This cannot happen!'      &
                         ,'allocate_sitetype','ed_state_vars.f90')
      end if
      !------------------------------------------------------------------------------------!




      allocate(csite%patch  (npatches))

      !----- Assume that all patches are empty. -------------------------------------------!
      do ipa=1,npatches
         csite%patch(ipa)%ncohorts = 0
      end do
      !------------------------------------------------------------------------------------!

      allocate(csite%paco_id                       (              npatches))
      allocate(csite%paco_n                        (              npatches))
      allocate(csite%cohort_count                  (              npatches))
      allocate(csite%pname                         (              npatches))
      allocate(csite%area                          (              npatches))
      allocate(csite%age                           (              npatches))
      allocate(csite%dist_type                     (              npatches))
      allocate(csite%fast_soil_C                   (              npatches))
      allocate(csite%slow_soil_C                   (              npatches))
      allocate(csite%structural_soil_C             (              npatches))
      allocate(csite%structural_soil_L             (              npatches))
      allocate(csite%mineralized_soil_N            (              npatches))
      allocate(csite%fast_soil_N                   (              npatches))
      allocate(csite%sum_dgd                       (              npatches))
      allocate(csite%sum_chd                       (              npatches))
      allocate(csite%can_theiv                     (              npatches))
      allocate(csite%can_vpdef                     (              npatches))
      allocate(csite%can_temp                      (              npatches))
      allocate(csite%can_temp_pv                   (              npatches))
      allocate(csite%can_shv                       (              npatches))
      allocate(csite%can_co2                       (              npatches))
      allocate(csite%can_rhos                      (              npatches))
      allocate(csite%can_prss                      (              npatches))
      allocate(csite%can_theta                     (              npatches))
      allocate(csite%can_depth                     (              npatches))
      allocate(csite%opencan_frac                  (              npatches))
      allocate(csite%ggbare                        (              npatches))
      allocate(csite%ggveg                         (              npatches))
      allocate(csite%ggnet                         (              npatches))
      allocate(csite%ggsoil                        (              npatches))
      allocate(csite%nlev_sfcwater                 (              npatches))
      allocate(csite%sfcwater_mass                 (          nzs,npatches))
      allocate(csite%sfcwater_energy               (          nzs,npatches))
      allocate(csite%sfcwater_depth                (          nzs,npatches))
      allocate(csite%sfcwater_tempk                (          nzs,npatches))
      allocate(csite%sfcwater_fracliq              (          nzs,npatches))
      allocate(csite%rshort_s                      (          nzs,npatches))
      allocate(csite%rshort_s_beam                 (          nzs,npatches))
      allocate(csite%rshort_s_diffuse              (          nzs,npatches))
      allocate(csite%par_s                         (          nzs,npatches))
      allocate(csite%par_s_beam                    (          nzs,npatches))
      allocate(csite%par_s_diffuse                 (          nzs,npatches))
      allocate(csite%soil_energy                   (          nzg,npatches))
      allocate(csite%soil_mstpot                   (          nzg,npatches))
      allocate(csite%soil_water                    (          nzg,npatches))
      allocate(csite%soil_tempk                    (          nzg,npatches))
      allocate(csite%soil_fracliq                  (          nzg,npatches))
      allocate(csite%rootdense                     (          nzg,npatches))
      allocate(csite%ground_shv                    (              npatches))
      allocate(csite%ground_ssh                    (              npatches))
      allocate(csite%ground_temp                   (              npatches))
      allocate(csite%ground_fliq                   (              npatches))
      allocate(csite%rough                         (              npatches))
      allocate(csite%par_l_max                     (              npatches))
      allocate(csite%par_l_beam_max                (              npatches))
      allocate(csite%par_l_diffuse_max             (              npatches))
      allocate(csite%A_o_max                       (        n_pft,npatches))
      allocate(csite%A_c_max                       (        n_pft,npatches))
      allocate(csite%avg_daily_temp                (              npatches))
      allocate(csite%avg_monthly_gndwater          (              npatches))
      allocate(csite%avg_monthly_waterdef          (              npatches))
      allocate(csite%wbudget_loss2atm              (              npatches))
      allocate(csite%wbudget_denseffect            (              npatches))
      allocate(csite%wbudget_precipgain            (              npatches))
      allocate(csite%wbudget_loss2runoff           (              npatches))
      allocate(csite%wbudget_loss2drainage         (              npatches))
      allocate(csite%wbudget_initialstorage        (              npatches))
      allocate(csite%wbudget_residual              (              npatches))
      allocate(csite%ebudget_loss2atm              (              npatches))
      allocate(csite%ebudget_denseffect            (              npatches))
      allocate(csite%ebudget_prsseffect            (              npatches))
      allocate(csite%ebudget_loss2runoff           (              npatches))
      allocate(csite%ebudget_loss2drainage         (              npatches))
      allocate(csite%ebudget_netrad                (              npatches))
      allocate(csite%ebudget_precipgain            (              npatches))
      allocate(csite%ebudget_initialstorage        (              npatches))
      allocate(csite%ebudget_residual              (              npatches))
      allocate(csite%co2budget_initialstorage      (              npatches))
      allocate(csite%co2budget_residual            (              npatches))
      allocate(csite%co2budget_loss2atm            (              npatches))
      allocate(csite%co2budget_denseffect          (              npatches))
      allocate(csite%co2budget_gpp                 (              npatches))
      allocate(csite%co2budget_plresp              (              npatches))
      allocate(csite%co2budget_rh                  (              npatches))
      allocate(csite%today_A_decomp                (              npatches))
      allocate(csite%today_Af_decomp               (              npatches))
      allocate(csite%repro                         (        n_pft,npatches))
      allocate(csite%veg_rough                     (              npatches))
      allocate(csite%veg_height                    (              npatches))
      allocate(csite%veg_displace                  (              npatches))
      allocate(csite%fsc_in                        (              npatches))
      allocate(csite%ssc_in                        (              npatches))
      allocate(csite%ssl_in                        (              npatches))
      allocate(csite%fsn_in                        (              npatches))
      allocate(csite%total_plant_nitrogen_uptake   (              npatches))
      allocate(csite%mineralized_N_loss            (              npatches))
      allocate(csite%mineralized_N_input           (              npatches))
      allocate(csite%rshort_g                      (              npatches))
      allocate(csite%rshort_g_beam                 (              npatches))
      allocate(csite%rshort_g_diffuse              (              npatches))
      allocate(csite%par_g                         (              npatches))
      allocate(csite%par_g_beam                    (              npatches))
      allocate(csite%par_g_diffuse                 (              npatches))
      allocate(csite%par_b                         (              npatches))
      allocate(csite%par_b_beam                    (              npatches))
      allocate(csite%par_b_diffuse                 (              npatches))
      allocate(csite%nir_b                         (              npatches))
      allocate(csite%nir_b_beam                    (              npatches))
      allocate(csite%nir_b_diffuse                 (              npatches))
      allocate(csite%rlong_g                       (              npatches))
      allocate(csite%rlong_s                       (              npatches))
      allocate(csite%albedo                        (              npatches))
      allocate(csite%albedo_par                    (              npatches))
      allocate(csite%albedo_nir                    (              npatches))
      allocate(csite%rlong_albedo                  (              npatches))
      allocate(csite%rnet                          (              npatches))
      allocate(csite%rlongup                       (              npatches))
      allocate(csite%parup                         (              npatches))
      allocate(csite%nirup                         (              npatches))
      allocate(csite%rshortup                      (              npatches))
      allocate(csite%total_sfcw_depth              (              npatches))
      allocate(csite%snowfac                       (              npatches))
      allocate(csite%A_decomp                      (              npatches))
      allocate(csite%f_decomp                      (              npatches))
      allocate(csite%rh                            (              npatches))
      allocate(csite%cwd_rh                        (              npatches))
      allocate(csite%cumlai_profile                (n_pft,ff_nhgt,npatches))
      allocate(csite%plant_ag_biomass              (              npatches))
      allocate(csite%htry                          (              npatches))
      allocate(csite%hprev                         (              npatches))
      allocate(csite%ustar                         (              npatches))
      allocate(csite%tstar                         (              npatches))
      allocate(csite%qstar                         (              npatches))
      allocate(csite%cstar                         (              npatches))
      allocate(csite%zeta                          (              npatches))
      allocate(csite%ribulk                        (              npatches))
      allocate(csite%upwp                          (              npatches))
      allocate(csite%qpwp                          (              npatches))
      allocate(csite%cpwp                          (              npatches))
      allocate(csite%tpwp                          (              npatches))
      allocate(csite%wpwp                          (              npatches))
      allocate(csite%watertable                    (              npatches))
      allocate(csite%moist_dz                      (              npatches))
      allocate(csite%ksat                          (              npatches))
      allocate(csite%soil_sat_energy               (              npatches))
      allocate(csite%soil_sat_water                (              npatches))
      allocate(csite%soil_sat_heat                 (              npatches))
      allocate(csite%runoff_A                      (            3,npatches))
      allocate(csite%runoff_rate                   (              npatches))
      allocate(csite%runoff                        (              npatches))
      allocate(csite%qrunoff                       (              npatches))
      allocate(csite%fmean_rh                      (              npatches))
      allocate(csite%fmean_cwd_rh                  (              npatches))
      allocate(csite%fmean_nep                     (              npatches))
      allocate(csite%fmean_rk4step                 (              npatches))
      allocate(csite%fmean_available_water         (              npatches))
      allocate(csite%fmean_can_theiv               (              npatches))
      allocate(csite%fmean_can_theta               (              npatches))
      allocate(csite%fmean_can_vpdef               (              npatches))
      allocate(csite%fmean_can_temp                (              npatches))
      allocate(csite%fmean_can_shv                 (              npatches))
      allocate(csite%fmean_can_co2                 (              npatches))
      allocate(csite%fmean_can_rhos                (              npatches))
      allocate(csite%fmean_can_prss                (              npatches))
      allocate(csite%fmean_gnd_temp                (              npatches))
      allocate(csite%fmean_gnd_shv                 (              npatches))
      allocate(csite%fmean_can_ggnd                (              npatches))
      allocate(csite%fmean_sfcw_depth              (              npatches))
      allocate(csite%fmean_sfcw_energy             (              npatches))
      allocate(csite%fmean_sfcw_mass               (              npatches))
      allocate(csite%fmean_sfcw_temp               (              npatches))
      allocate(csite%fmean_sfcw_fliq               (              npatches))
      allocate(csite%fmean_soil_energy             (          nzg,npatches))
      allocate(csite%fmean_soil_mstpot             (          nzg,npatches))
      allocate(csite%fmean_soil_water              (          nzg,npatches))
      allocate(csite%fmean_soil_temp               (          nzg,npatches))
      allocate(csite%fmean_soil_fliq               (          nzg,npatches))
      allocate(csite%fmean_rshort_gnd              (              npatches))
      allocate(csite%fmean_par_gnd                 (              npatches))
      allocate(csite%fmean_rlong_gnd               (              npatches))
      allocate(csite%fmean_rlongup                 (              npatches))
      allocate(csite%fmean_parup                   (              npatches))
      allocate(csite%fmean_nirup                   (              npatches))
      allocate(csite%fmean_rshortup                (              npatches))
      allocate(csite%fmean_rnet                    (              npatches))
      allocate(csite%fmean_albedo                  (              npatches))
      allocate(csite%fmean_albedo_par              (              npatches))
      allocate(csite%fmean_albedo_nir              (              npatches))
      allocate(csite%fmean_rlong_albedo            (              npatches))
      allocate(csite%fmean_ustar                   (              npatches))
      allocate(csite%fmean_tstar                   (              npatches))
      allocate(csite%fmean_qstar                   (              npatches))
      allocate(csite%fmean_cstar                   (              npatches))
      allocate(csite%fmean_carbon_ac               (              npatches))
      allocate(csite%fmean_carbon_st               (              npatches))
      allocate(csite%fmean_vapor_gc                (              npatches))
      allocate(csite%fmean_vapor_ac                (              npatches))
      allocate(csite%fmean_smoist_gg               (          nzg,npatches))
      allocate(csite%fmean_throughfall             (              npatches))
      allocate(csite%fmean_transloss               (          nzg,npatches))
      allocate(csite%fmean_runoff                  (              npatches))
      allocate(csite%fmean_drainage                (              npatches))
      allocate(csite%fmean_sensible_gc             (              npatches))
      allocate(csite%fmean_sensible_ac             (              npatches))
      allocate(csite%fmean_sensible_gg             (          nzg,npatches))
      allocate(csite%fmean_qthroughfall            (              npatches))
      allocate(csite%fmean_qrunoff                 (              npatches))
      allocate(csite%fmean_qdrainage               (              npatches))

      if (writing_long) then
         allocate(csite%dmean_A_decomp             (              npatches))
         allocate(csite%dmean_Af_decomp            (              npatches))
         allocate(csite%dmean_co2_residual         (              npatches))
         allocate(csite%dmean_energy_residual      (              npatches))
         allocate(csite%dmean_water_residual       (              npatches))
         allocate(csite%dmean_rh                   (              npatches))
         allocate(csite%dmean_cwd_rh               (              npatches))
         allocate(csite%dmean_nep                  (              npatches))
         allocate(csite%dmean_rk4step              (              npatches))
         allocate(csite%dmean_available_water      (              npatches))
         allocate(csite%dmean_can_theiv            (              npatches))
         allocate(csite%dmean_can_theta            (              npatches))
         allocate(csite%dmean_can_vpdef            (              npatches))
         allocate(csite%dmean_can_temp             (              npatches))
         allocate(csite%dmean_can_shv              (              npatches))
         allocate(csite%dmean_can_co2              (              npatches))
         allocate(csite%dmean_can_rhos             (              npatches))
         allocate(csite%dmean_can_prss             (              npatches))
         allocate(csite%dmean_gnd_temp             (              npatches))
         allocate(csite%dmean_gnd_shv              (              npatches))
         allocate(csite%dmean_can_ggnd             (              npatches))
         allocate(csite%dmean_sfcw_depth           (              npatches))
         allocate(csite%dmean_sfcw_energy          (              npatches))
         allocate(csite%dmean_sfcw_mass            (              npatches))
         allocate(csite%dmean_sfcw_temp            (              npatches))
         allocate(csite%dmean_sfcw_fliq            (              npatches))
         allocate(csite%dmean_soil_energy          (          nzg,npatches))
         allocate(csite%dmean_soil_mstpot          (          nzg,npatches))
         allocate(csite%dmean_soil_water           (          nzg,npatches))
         allocate(csite%dmean_soil_temp            (          nzg,npatches))
         allocate(csite%dmean_soil_fliq            (          nzg,npatches))
         allocate(csite%dmean_rshort_gnd           (              npatches))
         allocate(csite%dmean_par_gnd              (              npatches))
         allocate(csite%dmean_rlong_gnd            (              npatches))
         allocate(csite%dmean_rlongup              (              npatches))
         allocate(csite%dmean_parup                (              npatches))
         allocate(csite%dmean_nirup                (              npatches))
         allocate(csite%dmean_rshortup             (              npatches))
         allocate(csite%dmean_rnet                 (              npatches))
         allocate(csite%dmean_albedo               (              npatches))
         allocate(csite%dmean_albedo_par           (              npatches))
         allocate(csite%dmean_albedo_nir           (              npatches))
         allocate(csite%dmean_rlong_albedo         (              npatches))
         allocate(csite%dmean_ustar                (              npatches))
         allocate(csite%dmean_tstar                (              npatches))
         allocate(csite%dmean_qstar                (              npatches))
         allocate(csite%dmean_cstar                (              npatches))
         allocate(csite%dmean_carbon_ac            (              npatches))
         allocate(csite%dmean_carbon_st            (              npatches))
         allocate(csite%dmean_vapor_gc             (              npatches))
         allocate(csite%dmean_vapor_ac             (              npatches))
         allocate(csite%dmean_smoist_gg            (          nzg,npatches))
         allocate(csite%dmean_throughfall          (              npatches))
         allocate(csite%dmean_transloss            (          nzg,npatches))
         allocate(csite%dmean_runoff               (              npatches))
         allocate(csite%dmean_drainage             (              npatches))
         allocate(csite%dmean_sensible_gc          (              npatches))
         allocate(csite%dmean_sensible_ac          (              npatches))
         allocate(csite%dmean_sensible_gg          (          nzg,npatches))
         allocate(csite%dmean_qthroughfall         (              npatches))
         allocate(csite%dmean_qrunoff              (              npatches))
         allocate(csite%dmean_qdrainage            (              npatches))
      end if
      if (writing_eorq) then
         allocate(csite%mmean_fast_soil_c          (              npatches))
         allocate(csite%mmean_slow_soil_c          (              npatches))
         allocate(csite%mmean_struct_soil_c        (              npatches))
         allocate(csite%mmean_struct_soil_l        (              npatches))
         allocate(csite%mmean_fast_soil_n          (              npatches))
         allocate(csite%mmean_mineral_soil_n       (              npatches))
         allocate(csite%mmean_co2_residual         (              npatches))
         allocate(csite%mmean_energy_residual      (              npatches))
         allocate(csite%mmean_water_residual       (              npatches))
         allocate(csite%mmean_rh                   (              npatches))
         allocate(csite%mmean_cwd_rh               (              npatches))
         allocate(csite%mmean_nep                  (              npatches))
         allocate(csite%mmean_A_decomp             (              npatches))
         allocate(csite%mmean_Af_decomp            (              npatches))
         allocate(csite%mmean_rk4step              (              npatches))
         allocate(csite%mmean_available_water      (              npatches))
         allocate(csite%mmean_can_theiv            (              npatches))
         allocate(csite%mmean_can_theta            (              npatches))
         allocate(csite%mmean_can_vpdef            (              npatches))
         allocate(csite%mmean_can_temp             (              npatches))
         allocate(csite%mmean_can_shv              (              npatches))
         allocate(csite%mmean_can_co2              (              npatches))
         allocate(csite%mmean_can_rhos             (              npatches))
         allocate(csite%mmean_can_prss             (              npatches))
         allocate(csite%mmean_gnd_temp             (              npatches))
         allocate(csite%mmean_gnd_shv              (              npatches))
         allocate(csite%mmean_can_ggnd             (              npatches))
         allocate(csite%mmean_sfcw_depth           (              npatches))
         allocate(csite%mmean_sfcw_energy          (              npatches))
         allocate(csite%mmean_sfcw_mass            (              npatches))
         allocate(csite%mmean_sfcw_temp            (              npatches))
         allocate(csite%mmean_sfcw_fliq            (              npatches))
         allocate(csite%mmean_soil_energy          (          nzg,npatches))
         allocate(csite%mmean_soil_mstpot          (          nzg,npatches))
         allocate(csite%mmean_soil_water           (          nzg,npatches))
         allocate(csite%mmean_soil_temp            (          nzg,npatches))
         allocate(csite%mmean_soil_fliq            (          nzg,npatches))
         allocate(csite%mmean_rshort_gnd           (              npatches))
         allocate(csite%mmean_par_gnd              (              npatches))
         allocate(csite%mmean_rlong_gnd            (              npatches))
         allocate(csite%mmean_rlongup              (              npatches))
         allocate(csite%mmean_parup                (              npatches))
         allocate(csite%mmean_nirup                (              npatches))
         allocate(csite%mmean_rshortup             (              npatches))
         allocate(csite%mmean_rnet                 (              npatches))
         allocate(csite%mmean_albedo               (              npatches))
         allocate(csite%mmean_albedo_par           (              npatches))
         allocate(csite%mmean_albedo_nir           (              npatches))
         allocate(csite%mmean_rlong_albedo         (              npatches))
         allocate(csite%mmean_ustar                (              npatches))
         allocate(csite%mmean_tstar                (              npatches))
         allocate(csite%mmean_qstar                (              npatches))
         allocate(csite%mmean_cstar                (              npatches))
         allocate(csite%mmean_carbon_ac            (              npatches))
         allocate(csite%mmean_carbon_st            (              npatches))
         allocate(csite%mmean_vapor_gc             (              npatches))
         allocate(csite%mmean_vapor_ac             (              npatches))
         allocate(csite%mmean_smoist_gg            (          nzg,npatches))
         allocate(csite%mmean_throughfall          (              npatches))
         allocate(csite%mmean_transloss            (          nzg,npatches))
         allocate(csite%mmean_runoff               (              npatches))
         allocate(csite%mmean_drainage             (              npatches))
         allocate(csite%mmean_sensible_gc          (              npatches))
         allocate(csite%mmean_sensible_ac          (              npatches))
         allocate(csite%mmean_sensible_gg          (          nzg,npatches))
         allocate(csite%mmean_qthroughfall         (              npatches))
         allocate(csite%mmean_qrunoff              (              npatches))
         allocate(csite%mmean_qdrainage            (              npatches))
         allocate(csite%mmean_A_decomp             (              npatches))
         allocate(csite%mmean_Af_decomp            (              npatches))
         allocate(csite%mmean_co2_residual         (              npatches))
         allocate(csite%mmean_energy_residual      (              npatches))
         allocate(csite%mmean_water_residual       (              npatches))
         allocate(csite%mmsqu_rh                   (              npatches))
         allocate(csite%mmsqu_cwd_rh               (              npatches))
         allocate(csite%mmsqu_nep                  (              npatches))
         allocate(csite%mmsqu_rlongup              (              npatches))
         allocate(csite%mmsqu_parup                (              npatches))
         allocate(csite%mmsqu_nirup                (              npatches))
         allocate(csite%mmsqu_rshortup             (              npatches))
         allocate(csite%mmsqu_rnet                 (              npatches))
         allocate(csite%mmsqu_albedo               (              npatches))
         allocate(csite%mmsqu_ustar                (              npatches))
         allocate(csite%mmsqu_carbon_ac            (              npatches))
         allocate(csite%mmsqu_carbon_st            (              npatches))
         allocate(csite%mmsqu_vapor_gc             (              npatches))
         allocate(csite%mmsqu_vapor_ac             (              npatches))
         allocate(csite%mmsqu_sensible_gc          (              npatches))
         allocate(csite%mmsqu_sensible_ac          (              npatches))
      end if
      if (writing_dcyc) then
         allocate(csite%qmean_rh                   (      ndcycle,npatches))
         allocate(csite%qmean_cwd_rh               (      ndcycle,npatches))
         allocate(csite%qmean_nep                  (      ndcycle,npatches))
         allocate(csite%qmean_rk4step              (      ndcycle,npatches))
         allocate(csite%qmean_available_water      (      ndcycle,npatches))
         allocate(csite%qmean_can_theiv            (      ndcycle,npatches))
         allocate(csite%qmean_can_theta            (      ndcycle,npatches))
         allocate(csite%qmean_can_vpdef            (      ndcycle,npatches))
         allocate(csite%qmean_can_temp             (      ndcycle,npatches))
         allocate(csite%qmean_can_shv              (      ndcycle,npatches))
         allocate(csite%qmean_can_co2              (      ndcycle,npatches))
         allocate(csite%qmean_can_rhos             (      ndcycle,npatches))
         allocate(csite%qmean_can_prss             (      ndcycle,npatches))
         allocate(csite%qmean_gnd_temp             (      ndcycle,npatches))
         allocate(csite%qmean_gnd_shv              (      ndcycle,npatches))
         allocate(csite%qmean_can_ggnd             (      ndcycle,npatches))
         allocate(csite%qmean_sfcw_depth           (      ndcycle,npatches))
         allocate(csite%qmean_sfcw_energy          (      ndcycle,npatches))
         allocate(csite%qmean_sfcw_mass            (      ndcycle,npatches))
         allocate(csite%qmean_sfcw_temp            (      ndcycle,npatches))
         allocate(csite%qmean_sfcw_fliq            (      ndcycle,npatches))
         allocate(csite%qmean_soil_energy          (  nzg,ndcycle,npatches))
         allocate(csite%qmean_soil_mstpot          (  nzg,ndcycle,npatches))
         allocate(csite%qmean_soil_water           (  nzg,ndcycle,npatches))
         allocate(csite%qmean_soil_temp            (  nzg,ndcycle,npatches))
         allocate(csite%qmean_soil_fliq            (  nzg,ndcycle,npatches))
         allocate(csite%qmean_rshort_gnd           (      ndcycle,npatches))
         allocate(csite%qmean_par_gnd              (      ndcycle,npatches))
         allocate(csite%qmean_rlong_gnd            (      ndcycle,npatches))
         allocate(csite%qmean_rlongup              (      ndcycle,npatches))
         allocate(csite%qmean_parup                (      ndcycle,npatches))
         allocate(csite%qmean_nirup                (      ndcycle,npatches))
         allocate(csite%qmean_rshortup             (      ndcycle,npatches))
         allocate(csite%qmean_rnet                 (      ndcycle,npatches))
         allocate(csite%qmean_albedo               (      ndcycle,npatches))
         allocate(csite%qmean_albedo_par           (      ndcycle,npatches))
         allocate(csite%qmean_albedo_nir           (      ndcycle,npatches))
         allocate(csite%qmean_rlong_albedo         (      ndcycle,npatches))
         allocate(csite%qmean_ustar                (      ndcycle,npatches))
         allocate(csite%qmean_tstar                (      ndcycle,npatches))
         allocate(csite%qmean_qstar                (      ndcycle,npatches))
         allocate(csite%qmean_cstar                (      ndcycle,npatches))
         allocate(csite%qmean_carbon_ac            (      ndcycle,npatches))
         allocate(csite%qmean_carbon_st            (      ndcycle,npatches))
         allocate(csite%qmean_vapor_gc             (      ndcycle,npatches))
         allocate(csite%qmean_vapor_ac             (      ndcycle,npatches))
         allocate(csite%qmean_smoist_gg            (  nzg,ndcycle,npatches))
         allocate(csite%qmean_throughfall          (      ndcycle,npatches))
         allocate(csite%qmean_transloss            (  nzg,ndcycle,npatches))
         allocate(csite%qmean_runoff               (      ndcycle,npatches))
         allocate(csite%qmean_drainage             (      ndcycle,npatches))
         allocate(csite%qmean_sensible_gc          (      ndcycle,npatches))
         allocate(csite%qmean_sensible_ac          (      ndcycle,npatches))
         allocate(csite%qmean_sensible_gg          (  nzg,ndcycle,npatches))
         allocate(csite%qmean_qthroughfall         (      ndcycle,npatches))
         allocate(csite%qmean_qrunoff              (      ndcycle,npatches))
         allocate(csite%qmean_qdrainage            (      ndcycle,npatches))
         allocate(csite%qmsqu_rh                   (      ndcycle,npatches))
         allocate(csite%qmsqu_cwd_rh               (      ndcycle,npatches))
         allocate(csite%qmsqu_nep                  (      ndcycle,npatches))
         allocate(csite%qmsqu_rlongup              (      ndcycle,npatches))
         allocate(csite%qmsqu_parup                (      ndcycle,npatches))
         allocate(csite%qmsqu_nirup                (      ndcycle,npatches))
         allocate(csite%qmsqu_rshortup             (      ndcycle,npatches))
         allocate(csite%qmsqu_rnet                 (      ndcycle,npatches))
         allocate(csite%qmsqu_albedo               (      ndcycle,npatches))
         allocate(csite%qmsqu_ustar                (      ndcycle,npatches))
         allocate(csite%qmsqu_carbon_ac            (      ndcycle,npatches))
         allocate(csite%qmsqu_carbon_st            (      ndcycle,npatches))
         allocate(csite%qmsqu_vapor_gc             (      ndcycle,npatches))
         allocate(csite%qmsqu_vapor_ac             (      ndcycle,npatches))
         allocate(csite%qmsqu_sensible_gc          (      ndcycle,npatches))
         allocate(csite%qmsqu_sensible_ac          (      ndcycle,npatches))
      end if

      return
   end subroutine allocate_sitetype
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================! 
   !=======================================================================================! 
   !     This subroutine allocates the cohort-level variables.                             !
   !---------------------------------------------------------------------------------------!
   subroutine allocate_patchtype(cpatch,ncohorts)

      implicit none

      !----- Arguments. -------------------------------------------------------------------!
      type(patchtype), target     :: cpatch
      integer        , intent(in) :: ncohorts
      !------------------------------------------------------------------------------------!
 

      !----- Make sure that we nullify all pointers for a safe allocation. ----------------!
      call nullify_patchtype   (cpatch)
      !------------------------------------------------------------------------------------!


      !----- Define the number of cohorts. ------------------------------------------------!
      cpatch%ncohorts = ncohorts
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      !       Some cohorts may be empty.  If all plants die, or if this simulation is a    !
      ! true bare ground run, no cohort should exist, in which case we return.             !
      !------------------------------------------------------------------------------------!
      if (ncohorts == 0) return
      !------------------------------------------------------------------------------------!


      allocate(cpatch%pft                          (                    ncohorts))
      allocate(cpatch%nplant                       (                    ncohorts))
      allocate(cpatch%phenology_status             (                    ncohorts))
      allocate(cpatch%recruit_dbh                  (                    ncohorts))
      allocate(cpatch%census_status                (                    ncohorts))
      allocate(cpatch%hite                         (                    ncohorts))
      allocate(cpatch%agb                          (                    ncohorts))
      allocate(cpatch%basarea                      (                    ncohorts))
      allocate(cpatch%dagb_dt                      (                    ncohorts))
      allocate(cpatch%dlnagb_dt                    (                    ncohorts))
      allocate(cpatch%dba_dt                       (                    ncohorts))
      allocate(cpatch%dlnba_dt                     (                    ncohorts))
      allocate(cpatch%ddbh_dt                      (                    ncohorts))
      allocate(cpatch%dlndbh_dt                    (                    ncohorts))
      allocate(cpatch%dbh                          (                    ncohorts))
      allocate(cpatch%bdead                        (                    ncohorts))
      allocate(cpatch%bleaf                        (                    ncohorts))
      allocate(cpatch%balive                       (                    ncohorts))
      allocate(cpatch%broot                        (                    ncohorts))
      allocate(cpatch%bsapwooda                    (                    ncohorts))
      allocate(cpatch%bsapwoodb                    (                    ncohorts))
      allocate(cpatch%bstorage                     (                    ncohorts))
      allocate(cpatch%bseeds                       (                    ncohorts))
      allocate(cpatch%lai                          (                    ncohorts))
      allocate(cpatch%wai                          (                    ncohorts))
      allocate(cpatch%crown_area                   (                    ncohorts))
      allocate(cpatch%leaf_resolvable              (                    ncohorts))
      allocate(cpatch%wood_resolvable              (                    ncohorts))
      allocate(cpatch%cb                           (                 13,ncohorts))
      allocate(cpatch%cb_lightmax                  (                 13,ncohorts))
      allocate(cpatch%cb_moistmax                  (                 13,ncohorts))
      allocate(cpatch%cb_mlmax                     (                 13,ncohorts))
      allocate(cpatch%cbr_bar                      (                    ncohorts))
      allocate(cpatch%leaf_energy                  (                    ncohorts))
      allocate(cpatch%leaf_temp                    (                    ncohorts))
      allocate(cpatch%leaf_vpdef                   (                    ncohorts))
      allocate(cpatch%leaf_temp_pv                 (                    ncohorts))
      allocate(cpatch%leaf_hcap                    (                    ncohorts))
      allocate(cpatch%leaf_fliq                    (                    ncohorts))
      allocate(cpatch%leaf_water                   (                    ncohorts))
      allocate(cpatch%wood_energy                  (                    ncohorts))
      allocate(cpatch%wood_temp                    (                    ncohorts))
      allocate(cpatch%wood_temp_pv                 (                    ncohorts))
      allocate(cpatch%wood_hcap                    (                    ncohorts))
      allocate(cpatch%wood_fliq                    (                    ncohorts))
      allocate(cpatch%wood_water                   (                    ncohorts))
      allocate(cpatch%veg_wind                     (                    ncohorts))
      allocate(cpatch%lsfc_shv_open                (                    ncohorts))
      allocate(cpatch%lsfc_shv_closed              (                    ncohorts))
      allocate(cpatch%lsfc_co2_open                (                    ncohorts))
      allocate(cpatch%lsfc_co2_closed              (                    ncohorts))
      allocate(cpatch%lint_shv                     (                    ncohorts))
      allocate(cpatch%lint_co2_open                (                    ncohorts))
      allocate(cpatch%lint_co2_closed              (                    ncohorts))
      allocate(cpatch%today_leaf_resp              (                    ncohorts))
      allocate(cpatch%today_root_resp              (                    ncohorts))
      allocate(cpatch%today_gpp                    (                    ncohorts))
      allocate(cpatch%today_gpp_pot                (                    ncohorts))
      allocate(cpatch%today_gpp_lightmax           (                    ncohorts))
      allocate(cpatch%today_gpp_moistmax           (                    ncohorts))
      allocate(cpatch%today_gpp_mlmax              (                    ncohorts))
      allocate(cpatch%today_nppleaf                (                    ncohorts))
      allocate(cpatch%today_nppfroot               (                    ncohorts))
      allocate(cpatch%today_nppsapwood             (                    ncohorts))
      allocate(cpatch%today_nppcroot               (                    ncohorts))
      allocate(cpatch%today_nppseeds               (                    ncohorts))
      allocate(cpatch%today_nppwood                (                    ncohorts))
      allocate(cpatch%today_nppdaily               (                    ncohorts))
      allocate(cpatch%leaf_growth_resp             (                    ncohorts))
      allocate(cpatch%root_growth_resp             (                    ncohorts))
      allocate(cpatch%sapa_growth_resp             (                    ncohorts))
      allocate(cpatch%sapb_growth_resp             (                    ncohorts))
      allocate(cpatch%leaf_storage_resp            (                    ncohorts))
      allocate(cpatch%root_storage_resp            (                    ncohorts))
      allocate(cpatch%sapa_storage_resp            (                    ncohorts))
      allocate(cpatch%sapb_storage_resp            (                    ncohorts))
      allocate(cpatch%monthly_dndt                 (                    ncohorts))
      allocate(cpatch%monthly_dlnndt               (                    ncohorts))
      allocate(cpatch%mort_rate                    (             n_mort,ncohorts))

      allocate(cpatch%krdepth                      (                    ncohorts))
      allocate(cpatch%first_census                 (                    ncohorts))
      allocate(cpatch%new_recruit_flag             (                    ncohorts))
      allocate(cpatch%light_level                  (                    ncohorts))
      allocate(cpatch%light_level_beam             (                    ncohorts))
      allocate(cpatch%light_level_diff             (                    ncohorts))

      allocate(cpatch%par_level_beam               (                    ncohorts))
      allocate(cpatch%par_level_diffu              (                    ncohorts))
      allocate(cpatch%par_level_diffd              (                    ncohorts))

      allocate(cpatch%par_l                        (                    ncohorts))
      allocate(cpatch%par_l_beam                   (                    ncohorts))
      allocate(cpatch%par_l_diffuse                (                    ncohorts))
      allocate(cpatch%rshort_l                     (                    ncohorts))
      allocate(cpatch%rshort_l_beam                (                    ncohorts))
      allocate(cpatch%rshort_l_diffuse             (                    ncohorts))
      allocate(cpatch%rlong_l                      (                    ncohorts))
      allocate(cpatch%rshort_w                     (                    ncohorts))
      allocate(cpatch%rshort_w_beam                (                    ncohorts))
      allocate(cpatch%rshort_w_diffuse             (                    ncohorts))
      allocate(cpatch%rlong_w                      (                    ncohorts))
      allocate(cpatch%rad_profile                  (          n_radprof,ncohorts))
      allocate(cpatch%leaf_gbh                     (                    ncohorts))
      allocate(cpatch%leaf_gbw                     (                    ncohorts))
      allocate(cpatch%wood_gbh                     (                    ncohorts))
      allocate(cpatch%wood_gbw                     (                    ncohorts))
      allocate(cpatch%A_open                       (                    ncohorts))
      allocate(cpatch%A_closed                     (                    ncohorts))
      allocate(cpatch%A_light                      (                    ncohorts))
      allocate(cpatch%A_rubp                       (                    ncohorts))
      allocate(cpatch%A_co2                        (                    ncohorts))
      allocate(cpatch%psi_open                     (                    ncohorts))
      allocate(cpatch%psi_closed                   (                    ncohorts))
      allocate(cpatch%gsw_open                     (                    ncohorts))
      allocate(cpatch%gsw_closed                   (                    ncohorts))
      allocate(cpatch%leaf_gsw                     (                    ncohorts))
      allocate(cpatch%fsw                          (                    ncohorts))
      allocate(cpatch%fsn                          (                    ncohorts))
      allocate(cpatch%fs_open                      (                    ncohorts))
      allocate(cpatch%water_supply                 (                    ncohorts))
      allocate(cpatch%leaf_maintenance             (                    ncohorts))
      allocate(cpatch%root_maintenance             (                    ncohorts))
      allocate(cpatch%leaf_drop                    (                    ncohorts))
      allocate(cpatch%leaf_respiration             (                    ncohorts))
      allocate(cpatch%root_respiration             (                    ncohorts))
      allocate(cpatch%gpp                          (                    ncohorts))
      allocate(cpatch%paw_avg                      (                    ncohorts))
      allocate(cpatch%elongf                       (                    ncohorts))
      allocate(cpatch%turnover_amp                 (                    ncohorts))
      allocate(cpatch%llspan                       (                    ncohorts))
      allocate(cpatch%vm_bar                       (                    ncohorts))
      allocate(cpatch%sla                          (                    ncohorts))
      allocate(cpatch%fmean_gpp                    (                    ncohorts))
      allocate(cpatch%fmean_npp                    (                    ncohorts))
      allocate(cpatch%fmean_leaf_resp              (                    ncohorts))
      allocate(cpatch%fmean_root_resp              (                    ncohorts))
      allocate(cpatch%fmean_leaf_growth_resp       (                    ncohorts))
      allocate(cpatch%fmean_root_growth_resp       (                    ncohorts))
      allocate(cpatch%fmean_sapa_growth_resp       (                    ncohorts))
      allocate(cpatch%fmean_sapb_growth_resp       (                    ncohorts))
      allocate(cpatch%fmean_leaf_storage_resp      (                    ncohorts))
      allocate(cpatch%fmean_root_storage_resp      (                    ncohorts))
      allocate(cpatch%fmean_sapa_storage_resp      (                    ncohorts))
      allocate(cpatch%fmean_sapb_storage_resp      (                    ncohorts))
      allocate(cpatch%fmean_plresp                 (                    ncohorts))
      allocate(cpatch%fmean_leaf_energy            (                    ncohorts))
      allocate(cpatch%fmean_leaf_water             (                    ncohorts))
      allocate(cpatch%fmean_leaf_hcap              (                    ncohorts))
      allocate(cpatch%fmean_leaf_vpdef             (                    ncohorts))
      allocate(cpatch%fmean_leaf_temp              (                    ncohorts))
      allocate(cpatch%fmean_leaf_fliq              (                    ncohorts))
      allocate(cpatch%fmean_leaf_gsw               (                    ncohorts))
      allocate(cpatch%fmean_leaf_gbw               (                    ncohorts))
      allocate(cpatch%fmean_wood_energy            (                    ncohorts))
      allocate(cpatch%fmean_wood_water             (                    ncohorts))
      allocate(cpatch%fmean_wood_hcap              (                    ncohorts))
      allocate(cpatch%fmean_wood_temp              (                    ncohorts))
      allocate(cpatch%fmean_wood_fliq              (                    ncohorts))
      allocate(cpatch%fmean_wood_gbw               (                    ncohorts))
      allocate(cpatch%fmean_fs_open                (                    ncohorts))
      allocate(cpatch%fmean_fsw                    (                    ncohorts))
      allocate(cpatch%fmean_fsn                    (                    ncohorts))
      allocate(cpatch%fmean_a_open                 (                    ncohorts))
      allocate(cpatch%fmean_a_closed               (                    ncohorts))
      allocate(cpatch%fmean_a_net                  (                    ncohorts))
      allocate(cpatch%fmean_A_light                (                    ncohorts))
      allocate(cpatch%fmean_A_rubp                 (                    ncohorts))
      allocate(cpatch%fmean_A_co2                  (                    ncohorts))
      allocate(cpatch%fmean_psi_open               (                    ncohorts))
      allocate(cpatch%fmean_psi_closed             (                    ncohorts))
      allocate(cpatch%fmean_water_supply           (                    ncohorts))
      allocate(cpatch%fmean_light_level            (                    ncohorts))
      allocate(cpatch%fmean_light_level_beam       (                    ncohorts))
      allocate(cpatch%fmean_light_level_diff       (                    ncohorts))

      allocate(cpatch%fmean_par_level_beam         (                    ncohorts))
      allocate(cpatch%fmean_par_level_diffu        (                    ncohorts))
      allocate(cpatch%fmean_par_level_diffd        (                    ncohorts))

      allocate(cpatch%fmean_par_l                  (                    ncohorts))
      allocate(cpatch%fmean_par_l_beam             (                    ncohorts))
      allocate(cpatch%fmean_par_l_diff             (                    ncohorts))
      allocate(cpatch%fmean_rshort_l               (                    ncohorts))
      allocate(cpatch%fmean_rlong_l                (                    ncohorts))
      allocate(cpatch%fmean_sensible_lc            (                    ncohorts))
      allocate(cpatch%fmean_vapor_lc               (                    ncohorts))
      allocate(cpatch%fmean_transp                 (                    ncohorts))
      allocate(cpatch%fmean_intercepted_al         (                    ncohorts))
      allocate(cpatch%fmean_wshed_lg               (                    ncohorts))
      allocate(cpatch%fmean_rshort_w               (                    ncohorts))
      allocate(cpatch%fmean_rlong_w                (                    ncohorts))
      allocate(cpatch%fmean_rad_profile            (          n_radprof,ncohorts))
      allocate(cpatch%fmean_sensible_wc            (                    ncohorts))
      allocate(cpatch%fmean_vapor_wc               (                    ncohorts))
      allocate(cpatch%fmean_intercepted_aw         (                    ncohorts))
      allocate(cpatch%fmean_wshed_wg               (                    ncohorts))
      allocate(cpatch%fmean_lai                    (                    ncohorts))
      allocate(cpatch%fmean_bdead                  (                    ncohorts))


      if (writing_long) then
         allocate(cpatch%dmean_nppleaf             (                    ncohorts))
         allocate(cpatch%dmean_nppfroot            (                    ncohorts))
         allocate(cpatch%dmean_nppsapwood          (                    ncohorts))
         allocate(cpatch%dmean_nppcroot            (                    ncohorts))
         allocate(cpatch%dmean_nppseeds            (                    ncohorts))
         allocate(cpatch%dmean_nppwood             (                    ncohorts))
         allocate(cpatch%dmean_nppdaily            (                    ncohorts))
         allocate(cpatch%dmean_gpp                 (                    ncohorts))
         allocate(cpatch%dmean_npp                 (                    ncohorts))
         allocate(cpatch%dmean_leaf_resp           (                    ncohorts))
         allocate(cpatch%dmean_root_resp           (                    ncohorts))
         allocate(cpatch%dmean_leaf_growth_resp    (                    ncohorts))
         allocate(cpatch%dmean_root_growth_resp    (                    ncohorts))
         allocate(cpatch%dmean_sapa_growth_resp    (                    ncohorts))
         allocate(cpatch%dmean_sapb_growth_resp    (                    ncohorts))
         allocate(cpatch%dmean_leaf_storage_resp   (                    ncohorts))
         allocate(cpatch%dmean_root_storage_resp   (                    ncohorts))
         allocate(cpatch%dmean_sapa_storage_resp   (                    ncohorts))
         allocate(cpatch%dmean_sapb_storage_resp   (                    ncohorts))
         allocate(cpatch%dmean_plresp              (                    ncohorts))
         allocate(cpatch%dmean_leaf_energy         (                    ncohorts))
         allocate(cpatch%dmean_leaf_water          (                    ncohorts))
         allocate(cpatch%dmean_leaf_hcap           (                    ncohorts))
         allocate(cpatch%dmean_leaf_vpdef          (                    ncohorts))
         allocate(cpatch%dmean_leaf_temp           (                    ncohorts))
         allocate(cpatch%dmean_leaf_fliq           (                    ncohorts))
         allocate(cpatch%dmean_leaf_gsw            (                    ncohorts))
         allocate(cpatch%dmean_leaf_gbw            (                    ncohorts))
         allocate(cpatch%dmean_wood_energy         (                    ncohorts))
         allocate(cpatch%dmean_wood_water          (                    ncohorts))
         allocate(cpatch%dmean_wood_hcap           (                    ncohorts))
         allocate(cpatch%dmean_wood_temp           (                    ncohorts))
         allocate(cpatch%dmean_wood_fliq           (                    ncohorts))
         allocate(cpatch%dmean_wood_gbw            (                    ncohorts))
         allocate(cpatch%dmean_fs_open             (                    ncohorts))
         allocate(cpatch%dmean_fsw                 (                    ncohorts))
         allocate(cpatch%dmean_fsn                 (                    ncohorts))
         allocate(cpatch%dmean_a_open              (                    ncohorts))
         allocate(cpatch%dmean_a_closed            (                    ncohorts))
         allocate(cpatch%dmean_a_net               (                    ncohorts))
         allocate(cpatch%dmean_A_light             (                    ncohorts))
         allocate(cpatch%dmean_A_rubp              (                    ncohorts))
         allocate(cpatch%dmean_A_co2               (                    ncohorts))
         allocate(cpatch%dmean_psi_open            (                    ncohorts))
         allocate(cpatch%dmean_psi_closed          (                    ncohorts))
         allocate(cpatch%dmean_water_supply        (                    ncohorts))
         allocate(cpatch%dmean_light_level         (                    ncohorts))
         allocate(cpatch%dmean_light_level_beam    (                    ncohorts))
         allocate(cpatch%dmean_light_level_diff    (                    ncohorts))
         
         allocate(cpatch%dmean_par_level_beam      (                    ncohorts))
         allocate(cpatch%dmean_par_level_diffu     (                    ncohorts))
         allocate(cpatch%dmean_par_level_diffd     (                    ncohorts))

         allocate(cpatch%dmean_par_l               (                    ncohorts))
         allocate(cpatch%dmean_par_l_beam          (                    ncohorts))
         allocate(cpatch%dmean_par_l_diff          (                    ncohorts))
         allocate(cpatch%dmean_rshort_l            (                    ncohorts))
         allocate(cpatch%dmean_rlong_l             (                    ncohorts))
         allocate(cpatch%dmean_sensible_lc         (                    ncohorts))
         allocate(cpatch%dmean_vapor_lc            (                    ncohorts))
         allocate(cpatch%dmean_transp              (                    ncohorts))
         allocate(cpatch%dmean_intercepted_al      (                    ncohorts))
         allocate(cpatch%dmean_wshed_lg            (                    ncohorts))
         allocate(cpatch%dmean_rshort_w            (                    ncohorts))
         allocate(cpatch%dmean_rlong_w             (                    ncohorts))
         allocate(cpatch%dmean_rad_profile         (          n_radprof,ncohorts))
         allocate(cpatch%dmean_sensible_wc         (                    ncohorts))
         allocate(cpatch%dmean_vapor_wc            (                    ncohorts))
         allocate(cpatch%dmean_intercepted_aw      (                    ncohorts))
         allocate(cpatch%dmean_wshed_wg            (                    ncohorts))
      end if

      if (writing_eorq) then
         allocate(cpatch%mmean_lai                 (                    ncohorts))
         allocate(cpatch%mmean_bleaf               (                    ncohorts))
         allocate(cpatch%mmean_broot               (                    ncohorts))
         allocate(cpatch%mmean_bstorage            (                    ncohorts))
         allocate(cpatch%mmean_mort_rate           (             n_mort,ncohorts))
         allocate(cpatch%mmean_leaf_maintenance    (                    ncohorts))
         allocate(cpatch%mmean_root_maintenance    (                    ncohorts))
         allocate(cpatch%mmean_leaf_drop           (                    ncohorts))
         allocate(cpatch%mmean_cb                  (                    ncohorts))
         allocate(cpatch%mmean_gpp                 (                    ncohorts))
         allocate(cpatch%mmean_npp                 (                    ncohorts))
         allocate(cpatch%mmean_leaf_resp           (                    ncohorts))
         allocate(cpatch%mmean_root_resp           (                    ncohorts))
         allocate(cpatch%mmean_leaf_growth_resp    (                    ncohorts))
         allocate(cpatch%mmean_root_growth_resp    (                    ncohorts))
         allocate(cpatch%mmean_sapa_growth_resp    (                    ncohorts))
         allocate(cpatch%mmean_sapb_growth_resp    (                    ncohorts))
         allocate(cpatch%mmean_leaf_storage_resp   (                    ncohorts))
         allocate(cpatch%mmean_root_storage_resp   (                    ncohorts))
         allocate(cpatch%mmean_sapa_storage_resp   (                    ncohorts))
         allocate(cpatch%mmean_sapb_storage_resp   (                    ncohorts))
         allocate(cpatch%mmean_plresp              (                    ncohorts))
         allocate(cpatch%mmean_leaf_energy         (                    ncohorts))
         allocate(cpatch%mmean_leaf_water          (                    ncohorts))
         allocate(cpatch%mmean_leaf_hcap           (                    ncohorts))
         allocate(cpatch%mmean_leaf_vpdef          (                    ncohorts))
         allocate(cpatch%mmean_leaf_temp           (                    ncohorts))
         allocate(cpatch%mmean_leaf_fliq           (                    ncohorts))
         allocate(cpatch%mmean_leaf_gsw            (                    ncohorts))
         allocate(cpatch%mmean_leaf_gbw            (                    ncohorts))
         allocate(cpatch%mmean_wood_energy         (                    ncohorts))
         allocate(cpatch%mmean_wood_water          (                    ncohorts))
         allocate(cpatch%mmean_wood_hcap           (                    ncohorts))
         allocate(cpatch%mmean_wood_temp           (                    ncohorts))
         allocate(cpatch%mmean_wood_fliq           (                    ncohorts))
         allocate(cpatch%mmean_wood_gbw            (                    ncohorts))
         allocate(cpatch%mmean_fs_open             (                    ncohorts))
         allocate(cpatch%mmean_fsw                 (                    ncohorts))
         allocate(cpatch%mmean_fsn                 (                    ncohorts))
         allocate(cpatch%mmean_a_open              (                    ncohorts))
         allocate(cpatch%mmean_a_closed            (                    ncohorts))
         allocate(cpatch%mmean_a_net               (                    ncohorts))
         allocate(cpatch%mmean_A_light             (                    ncohorts))
         allocate(cpatch%mmean_A_rubp              (                    ncohorts))
         allocate(cpatch%mmean_A_co2               (                    ncohorts))
         allocate(cpatch%mmean_psi_open            (                    ncohorts))
         allocate(cpatch%mmean_psi_closed          (                    ncohorts))
         allocate(cpatch%mmean_water_supply        (                    ncohorts))
         allocate(cpatch%mmean_light_level         (                    ncohorts))
         allocate(cpatch%mmean_light_level_beam    (                    ncohorts))
         allocate(cpatch%mmean_light_level_diff    (                    ncohorts))

         allocate(cpatch%mmean_par_level_beam      (                    ncohorts))
         allocate(cpatch%mmean_par_level_diffu     (                    ncohorts))
         allocate(cpatch%mmean_par_level_diffd     (                    ncohorts))

         allocate(cpatch%mmean_par_l               (                    ncohorts))
         allocate(cpatch%mmean_par_l_beam          (                    ncohorts))
         allocate(cpatch%mmean_par_l_diff          (                    ncohorts))
         allocate(cpatch%mmean_rshort_l            (                    ncohorts))
         allocate(cpatch%mmean_rlong_l             (                    ncohorts))
         allocate(cpatch%mmean_sensible_lc         (                    ncohorts))
         allocate(cpatch%mmean_vapor_lc            (                    ncohorts))
         allocate(cpatch%mmean_transp              (                    ncohorts))
         allocate(cpatch%mmean_intercepted_al      (                    ncohorts))
         allocate(cpatch%mmean_wshed_lg            (                    ncohorts))
         allocate(cpatch%mmean_rshort_w            (                    ncohorts))
         allocate(cpatch%mmean_rlong_w             (                    ncohorts))
         allocate(cpatch%mmean_rad_profile         (          n_radprof,ncohorts))
         allocate(cpatch%mmean_sensible_wc         (                    ncohorts))
         allocate(cpatch%mmean_vapor_wc            (                    ncohorts))
         allocate(cpatch%mmean_intercepted_aw      (                    ncohorts))
         allocate(cpatch%mmean_wshed_wg            (                    ncohorts))
         allocate(cpatch%mmean_nppleaf             (                    ncohorts))
         allocate(cpatch%mmean_nppfroot            (                    ncohorts))
         allocate(cpatch%mmean_nppsapwood          (                    ncohorts))
         allocate(cpatch%mmean_nppcroot            (                    ncohorts))
         allocate(cpatch%mmean_nppseeds            (                    ncohorts))
         allocate(cpatch%mmean_nppwood             (                    ncohorts))
         allocate(cpatch%mmean_nppdaily            (                    ncohorts))
         allocate(cpatch%mmsqu_gpp                 (                    ncohorts))
         allocate(cpatch%mmsqu_npp                 (                    ncohorts))
         allocate(cpatch%mmsqu_plresp              (                    ncohorts))
         allocate(cpatch%mmsqu_sensible_lc         (                    ncohorts))
         allocate(cpatch%mmsqu_vapor_lc            (                    ncohorts))
         allocate(cpatch%mmsqu_transp              (                    ncohorts))
         allocate(cpatch%mmsqu_sensible_wc         (                    ncohorts))
         allocate(cpatch%mmsqu_vapor_wc            (                    ncohorts))
      end if
      
      if (writing_dcyc) then
         allocate(cpatch%qmean_gpp                 (            ndcycle,ncohorts))
         allocate(cpatch%qmean_npp                 (            ndcycle,ncohorts))
         allocate(cpatch%qmean_leaf_resp           (            ndcycle,ncohorts))
         allocate(cpatch%qmean_root_resp           (            ndcycle,ncohorts))
         allocate(cpatch%qmean_leaf_growth_resp    (            ndcycle,ncohorts))
         allocate(cpatch%qmean_root_growth_resp    (            ndcycle,ncohorts))
         allocate(cpatch%qmean_sapa_growth_resp    (            ndcycle,ncohorts))
         allocate(cpatch%qmean_sapb_growth_resp    (            ndcycle,ncohorts))
         allocate(cpatch%qmean_leaf_storage_resp   (            ndcycle,ncohorts))
         allocate(cpatch%qmean_root_storage_resp   (            ndcycle,ncohorts))
         allocate(cpatch%qmean_sapa_storage_resp   (            ndcycle,ncohorts))
         allocate(cpatch%qmean_sapb_storage_resp   (            ndcycle,ncohorts))
         allocate(cpatch%qmean_plresp              (            ndcycle,ncohorts))
         allocate(cpatch%qmean_leaf_energy         (            ndcycle,ncohorts))
         allocate(cpatch%qmean_leaf_water          (            ndcycle,ncohorts))
         allocate(cpatch%qmean_leaf_hcap           (            ndcycle,ncohorts))
         allocate(cpatch%qmean_leaf_vpdef          (            ndcycle,ncohorts))
         allocate(cpatch%qmean_leaf_temp           (            ndcycle,ncohorts))
         allocate(cpatch%qmean_leaf_fliq           (            ndcycle,ncohorts))
         allocate(cpatch%qmean_leaf_gsw            (            ndcycle,ncohorts))
         allocate(cpatch%qmean_leaf_gbw            (            ndcycle,ncohorts))
         allocate(cpatch%qmean_wood_energy         (            ndcycle,ncohorts))
         allocate(cpatch%qmean_wood_water          (            ndcycle,ncohorts))
         allocate(cpatch%qmean_wood_hcap           (            ndcycle,ncohorts))
         allocate(cpatch%qmean_wood_temp           (            ndcycle,ncohorts))
         allocate(cpatch%qmean_wood_fliq           (            ndcycle,ncohorts))
         allocate(cpatch%qmean_wood_gbw            (            ndcycle,ncohorts))
         allocate(cpatch%qmean_fs_open             (            ndcycle,ncohorts))
         allocate(cpatch%qmean_fsw                 (            ndcycle,ncohorts))
         allocate(cpatch%qmean_fsn                 (            ndcycle,ncohorts))
         allocate(cpatch%qmean_a_open              (            ndcycle,ncohorts))
         allocate(cpatch%qmean_a_closed            (            ndcycle,ncohorts))
         allocate(cpatch%qmean_a_net               (            ndcycle,ncohorts))
         allocate(cpatch%qmean_A_light             (            ndcycle,ncohorts))
         allocate(cpatch%qmean_A_rubp              (            ndcycle,ncohorts))
         allocate(cpatch%qmean_A_co2               (            ndcycle,ncohorts))
         allocate(cpatch%qmean_psi_open            (            ndcycle,ncohorts))
         allocate(cpatch%qmean_psi_closed          (            ndcycle,ncohorts))
         allocate(cpatch%qmean_water_supply        (            ndcycle,ncohorts))
         allocate(cpatch%qmean_light_level         (            ndcycle,ncohorts))
         allocate(cpatch%qmean_light_level_beam    (            ndcycle,ncohorts))
         allocate(cpatch%qmean_light_level_diff    (            ndcycle,ncohorts))

         ! ADDED RGK 11-2014
         allocate(cpatch%qmean_par_level_beam      (            ndcycle,ncohorts))
         allocate(cpatch%qmean_par_level_diffu     (            ndcycle,ncohorts))
         allocate(cpatch%qmean_par_level_diffd     (            ndcycle,ncohorts))        


         allocate(cpatch%qmean_par_l               (            ndcycle,ncohorts))
         allocate(cpatch%qmean_par_l_beam          (            ndcycle,ncohorts))
         allocate(cpatch%qmean_par_l_diff          (            ndcycle,ncohorts))
         allocate(cpatch%qmean_rshort_l            (            ndcycle,ncohorts))
         allocate(cpatch%qmean_rlong_l             (            ndcycle,ncohorts))
         allocate(cpatch%qmean_sensible_lc         (            ndcycle,ncohorts))
         allocate(cpatch%qmean_vapor_lc            (            ndcycle,ncohorts))
         allocate(cpatch%qmean_transp              (            ndcycle,ncohorts))
         allocate(cpatch%qmean_intercepted_al      (            ndcycle,ncohorts))
         allocate(cpatch%qmean_wshed_lg            (            ndcycle,ncohorts))
         allocate(cpatch%qmean_rshort_w            (            ndcycle,ncohorts))
         allocate(cpatch%qmean_rlong_w             (            ndcycle,ncohorts))
         allocate(cpatch%qmean_rad_profile         (n_radprof,  ndcycle,ncohorts))
         allocate(cpatch%qmean_sensible_wc         (            ndcycle,ncohorts))
         allocate(cpatch%qmean_vapor_wc            (            ndcycle,ncohorts))
         allocate(cpatch%qmean_intercepted_aw      (            ndcycle,ncohorts))
         allocate(cpatch%qmean_wshed_wg            (            ndcycle,ncohorts))
         allocate(cpatch%qmsqu_gpp                 (            ndcycle,ncohorts))
         allocate(cpatch%qmsqu_npp                 (            ndcycle,ncohorts))
         allocate(cpatch%qmsqu_plresp              (            ndcycle,ncohorts))
         allocate(cpatch%qmsqu_sensible_lc         (            ndcycle,ncohorts))
         allocate(cpatch%qmsqu_vapor_lc            (            ndcycle,ncohorts))
         allocate(cpatch%qmsqu_transp              (            ndcycle,ncohorts))
         allocate(cpatch%qmsqu_sensible_wc         (            ndcycle,ncohorts))
         allocate(cpatch%qmsqu_vapor_wc            (            ndcycle,ncohorts))
      end if

      return
   end subroutine allocate_patchtype
   !=======================================================================================!
   !=======================================================================================!





   !=======================================================================================!
   !=======================================================================================!
   !     This sub-routine nullifies all polygon pointers.                                  !
   !---------------------------------------------------------------------------------------!
   subroutine nullify_edtype(cgrid)

      implicit none

      !----- Arguments. -------------------------------------------------------------------!
      type(edtype), target :: cgrid
      !------------------------------------------------------------------------------------!

      nullify(cgrid%polygon                 )

      nullify(cgrid%pysi_id                 )
      nullify(cgrid%pysi_n                  )
      nullify(cgrid%walltime_py             )
      nullify(cgrid%lon                     )
      nullify(cgrid%lat                     )
      nullify(cgrid%xatm                    )
      nullify(cgrid%yatm                    )
      nullify(cgrid%site_adjacency          )
      nullify(cgrid%wbar                    )
      nullify(cgrid%Te                      )
      nullify(cgrid%zbar                    )
      nullify(cgrid%sheat                   )
      nullify(cgrid%baseflow                )
      nullify(cgrid%runoff                  )
      nullify(cgrid%qrunoff                 )
      nullify(cgrid%swliq                   )
      nullify(cgrid%ilon                    )
      nullify(cgrid%ilat                    )
      nullify(cgrid%workload                )
      nullify(cgrid%total_agb               )
      nullify(cgrid%total_basal_area        )
      nullify(cgrid%total_agb_growth        )
      nullify(cgrid%total_agb_mort          )
      nullify(cgrid%total_agb_recruit       )
      nullify(cgrid%total_basal_area_growth )
      nullify(cgrid%total_basal_area_mort   )
      nullify(cgrid%total_basal_area_recruit)
      nullify(cgrid%nsites                  )
      nullify(cgrid%sitenums                )
      nullify(cgrid%load_adjacency          )
      nullify(cgrid%metinput                )
      nullify(cgrid%met                     )
      nullify(cgrid%lapse                   )
      nullify(cgrid%cosz                    )
      nullify(cgrid%cbudget_initialstorage  )
      nullify(cgrid%cbudget_nep             )
      nullify(cgrid%nbudget_initialstorage  )
      nullify(cgrid%max_leaf_temp           )
      nullify(cgrid%min_leaf_temp           )
      nullify(cgrid%max_wood_temp           )
      nullify(cgrid%min_wood_temp           )
      nullify(cgrid%max_soil_temp           )
      nullify(cgrid%min_soil_temp           )
      nullify(cgrid%nplant                  )
      nullify(cgrid%agb                     )
      nullify(cgrid%lai                     )
      nullify(cgrid%wai                     )
      nullify(cgrid%basal_area              )
      nullify(cgrid%bdead                   )
      nullify(cgrid%balive                  )
      nullify(cgrid%bleaf                   )
      nullify(cgrid%broot                   )
      nullify(cgrid%bsapwooda               )
      nullify(cgrid%bsapwoodb               )
      nullify(cgrid%bseeds                  )
      nullify(cgrid%bstorage                )
      nullify(cgrid%bdead_n                 )
      nullify(cgrid%balive_n                )
      nullify(cgrid%bleaf_n                 )
      nullify(cgrid%broot_n                 )
      nullify(cgrid%bsapwooda_n             )
      nullify(cgrid%bsapwoodb_n             )
      nullify(cgrid%bseeds_n                )
      nullify(cgrid%bstorage_n              )
      nullify(cgrid%leaf_maintenance        )
      nullify(cgrid%root_maintenance        )
      nullify(cgrid%leaf_drop               )
      nullify(cgrid%fast_soil_c             )
      nullify(cgrid%slow_soil_c             )
      nullify(cgrid%struct_soil_c           )
      nullify(cgrid%struct_soil_l           )
      nullify(cgrid%cwd_c                   )
      nullify(cgrid%fast_soil_n             )
      nullify(cgrid%mineral_soil_n          )
      nullify(cgrid%cwd_n                   )
      nullify(cgrid%Cleaf_grow              )
      nullify(cgrid%Croot_grow              )
      nullify(cgrid%Cdead_grow              )
      nullify(cgrid%Cstore_grow             )
      nullify(cgrid%Cleaf_litter_flux       )
      nullify(cgrid%Croot_litter_flux       )
      nullify(cgrid%Ccwd_flux               )
      nullify(cgrid%Nleaf_grow              )
      nullify(cgrid%Ndead_grow              )
      nullify(cgrid%Nroot_grow              )
      nullify(cgrid%Nstore_grow             )
      nullify(cgrid%Nleaf_litter_flux       )
      nullify(cgrid%Nroot_litter_flux       )
      nullify(cgrid%Ncwd_flux               )
      nullify(cgrid%Nbiomass_uptake         )
      nullify(cgrid%Ngross_min              )
      nullify(cgrid%Nnet_min                )
      nullify(cgrid%avg_lai_ebalvars        )
      nullify(cgrid%fmean_gpp               )
      nullify(cgrid%fmean_npp               )
      nullify(cgrid%fmean_leaf_resp         )
      nullify(cgrid%fmean_root_resp         )
      nullify(cgrid%fmean_leaf_growth_resp  )
      nullify(cgrid%fmean_root_growth_resp  )
      nullify(cgrid%fmean_sapa_growth_resp  )
      nullify(cgrid%fmean_sapb_growth_resp  )
      nullify(cgrid%fmean_leaf_storage_resp )
      nullify(cgrid%fmean_root_storage_resp )
      nullify(cgrid%fmean_sapa_storage_resp )
      nullify(cgrid%fmean_sapb_storage_resp )
      nullify(cgrid%fmean_plresp            )
      nullify(cgrid%fmean_leaf_energy       )
      nullify(cgrid%fmean_leaf_water        )
      nullify(cgrid%fmean_leaf_hcap         )
      nullify(cgrid%fmean_leaf_vpdef        )
      nullify(cgrid%fmean_leaf_temp         )
      nullify(cgrid%fmean_leaf_fliq         )
      nullify(cgrid%fmean_leaf_gsw          )
      nullify(cgrid%fmean_leaf_gbw          )
      nullify(cgrid%fmean_wood_energy       )
      nullify(cgrid%fmean_wood_water        )
      nullify(cgrid%fmean_wood_hcap         )
      nullify(cgrid%fmean_wood_temp         )
      nullify(cgrid%fmean_wood_fliq         )
      nullify(cgrid%fmean_wood_gbw          )
      nullify(cgrid%fmean_fs_open           )
      nullify(cgrid%fmean_fsw               )
      nullify(cgrid%fmean_fsn               )
      nullify(cgrid%fmean_a_open            )
      nullify(cgrid%fmean_a_closed          )
      nullify(cgrid%fmean_a_net             )
      nullify(cgrid%fmean_A_light           )
      nullify(cgrid%fmean_A_rubp            )
      nullify(cgrid%fmean_A_co2             )
      nullify(cgrid%fmean_psi_open          )
      nullify(cgrid%fmean_psi_closed        )
      nullify(cgrid%fmean_water_supply      )
      nullify(cgrid%fmean_par_l             )
      nullify(cgrid%fmean_par_l_beam        )
      nullify(cgrid%fmean_par_l_diff        )
      nullify(cgrid%fmean_rshort_l          )
      nullify(cgrid%fmean_rlong_l           )
      nullify(cgrid%fmean_sensible_lc       )
      nullify(cgrid%fmean_vapor_lc          )
      nullify(cgrid%fmean_transp            )
      nullify(cgrid%fmean_intercepted_al    )
      nullify(cgrid%fmean_wshed_lg          )
      nullify(cgrid%fmean_rshort_w          )
      nullify(cgrid%fmean_rlong_w           )
      nullify(cgrid%fmean_sensible_wc       )
      nullify(cgrid%fmean_vapor_wc          )
      nullify(cgrid%fmean_intercepted_aw    )
      nullify(cgrid%fmean_wshed_wg          )
      nullify(cgrid%fmean_rh                )
      nullify(cgrid%fmean_cwd_rh            )
      nullify(cgrid%fmean_nep               )
      nullify(cgrid%fmean_rk4step           )
      nullify(cgrid%fmean_available_water   )
      nullify(cgrid%fmean_can_theiv         )
      nullify(cgrid%fmean_can_theta         )
      nullify(cgrid%fmean_can_vpdef         )
      nullify(cgrid%fmean_can_temp          )
      nullify(cgrid%fmean_can_shv           )
      nullify(cgrid%fmean_can_co2           )
      nullify(cgrid%fmean_can_rhos          )
      nullify(cgrid%fmean_can_prss          )
      nullify(cgrid%fmean_gnd_temp          )
      nullify(cgrid%fmean_gnd_shv           )
      nullify(cgrid%fmean_can_ggnd          )
      nullify(cgrid%fmean_sfcw_depth        )
      nullify(cgrid%fmean_sfcw_energy       )
      nullify(cgrid%fmean_sfcw_mass         )
      nullify(cgrid%fmean_sfcw_temp         )
      nullify(cgrid%fmean_sfcw_fliq         )
      nullify(cgrid%fmean_soil_energy       )
      nullify(cgrid%fmean_soil_mstpot       )
      nullify(cgrid%fmean_soil_water        )
      nullify(cgrid%fmean_soil_temp         )
      nullify(cgrid%fmean_soil_fliq         )
      nullify(cgrid%fmean_rshort_gnd        )
      nullify(cgrid%fmean_par_gnd           )
      nullify(cgrid%fmean_rlong_gnd         )
      nullify(cgrid%fmean_rlongup           )
      nullify(cgrid%fmean_parup             )
      nullify(cgrid%fmean_nirup             )
      nullify(cgrid%fmean_rshortup          )
      nullify(cgrid%fmean_rnet              )
      nullify(cgrid%fmean_albedo            )
      nullify(cgrid%fmean_albedo_par        )
      nullify(cgrid%fmean_albedo_nir        )
      nullify(cgrid%fmean_rlong_albedo      )
      nullify(cgrid%fmean_ustar             )
      nullify(cgrid%fmean_tstar             )
      nullify(cgrid%fmean_qstar             )
      nullify(cgrid%fmean_cstar             )
      nullify(cgrid%fmean_carbon_ac         )
      nullify(cgrid%fmean_carbon_st         )
      nullify(cgrid%fmean_vapor_gc          )
      nullify(cgrid%fmean_vapor_ac          )
      nullify(cgrid%fmean_smoist_gg         )
      nullify(cgrid%fmean_throughfall       )
      nullify(cgrid%fmean_transloss         )
      nullify(cgrid%fmean_runoff            )
      nullify(cgrid%fmean_drainage          )
      nullify(cgrid%fmean_sensible_gc       )
      nullify(cgrid%fmean_sensible_ac       )
      nullify(cgrid%fmean_sensible_gg       )
      nullify(cgrid%fmean_qthroughfall      )
      nullify(cgrid%fmean_qrunoff           )
      nullify(cgrid%fmean_qdrainage         )
      nullify(cgrid%fmean_atm_theiv         )
      nullify(cgrid%fmean_atm_theta         )
      nullify(cgrid%fmean_atm_temp          )
      nullify(cgrid%fmean_atm_vpdef         )
      nullify(cgrid%fmean_atm_shv           )
      nullify(cgrid%fmean_atm_rshort        )
      nullify(cgrid%fmean_atm_rshort_diff   )
      nullify(cgrid%fmean_atm_par           )
      nullify(cgrid%fmean_atm_par_diff      )
      nullify(cgrid%fmean_atm_rlong         )
      nullify(cgrid%fmean_atm_vels          )
      nullify(cgrid%fmean_atm_rhos          )
      nullify(cgrid%fmean_atm_prss          )
      nullify(cgrid%fmean_atm_co2           )
      nullify(cgrid%fmean_pcpg              )
      nullify(cgrid%fmean_qpcpg             )
      nullify(cgrid%fmean_dpcpg             )
      nullify(cgrid%fmean_soil_wetness      )
      nullify(cgrid%fmean_skin_temp         )
      nullify(cgrid%fmean_lai		    )
      nullify(cgrid%fmean_bdead		    )
      nullify(cgrid%dmean_nppleaf           )
      nullify(cgrid%dmean_nppfroot          )
      nullify(cgrid%dmean_nppsapwood        )
      nullify(cgrid%dmean_nppcroot          )
      nullify(cgrid%dmean_nppseeds          )
      nullify(cgrid%dmean_nppwood           )
      nullify(cgrid%dmean_nppdaily          )
      nullify(cgrid%dmean_A_decomp          )
      nullify(cgrid%dmean_Af_decomp         )
      nullify(cgrid%dmean_co2_residual      )
      nullify(cgrid%dmean_energy_residual   )
      nullify(cgrid%dmean_water_residual    )
      nullify(cgrid%dmean_gpp               )
      nullify(cgrid%dmean_npp               )
      nullify(cgrid%dmean_leaf_resp         )
      nullify(cgrid%dmean_root_resp         )
      nullify(cgrid%dmean_leaf_growth_resp  )
      nullify(cgrid%dmean_root_growth_resp  )
      nullify(cgrid%dmean_sapa_growth_resp  )
      nullify(cgrid%dmean_sapb_growth_resp  )
      nullify(cgrid%dmean_leaf_storage_resp )
      nullify(cgrid%dmean_root_storage_resp )
      nullify(cgrid%dmean_sapa_storage_resp )
      nullify(cgrid%dmean_sapb_storage_resp )
      nullify(cgrid%dmean_plresp            )
      nullify(cgrid%dmean_leaf_energy       )
      nullify(cgrid%dmean_leaf_water        )
      nullify(cgrid%dmean_leaf_hcap         )
      nullify(cgrid%dmean_leaf_vpdef        )
      nullify(cgrid%dmean_leaf_temp         )
      nullify(cgrid%dmean_leaf_fliq         )
      nullify(cgrid%dmean_leaf_gsw          )
      nullify(cgrid%dmean_leaf_gbw          )
      nullify(cgrid%dmean_wood_energy       )
      nullify(cgrid%dmean_wood_water        )
      nullify(cgrid%dmean_wood_hcap         )
      nullify(cgrid%dmean_wood_temp         )
      nullify(cgrid%dmean_wood_fliq         )
      nullify(cgrid%dmean_wood_gbw          )
      nullify(cgrid%dmean_fs_open           )
      nullify(cgrid%dmean_fsw               )
      nullify(cgrid%dmean_fsn               )
      nullify(cgrid%dmean_a_open            )
      nullify(cgrid%dmean_a_closed          )
      nullify(cgrid%dmean_a_net             )
      nullify(cgrid%dmean_A_light           )
      nullify(cgrid%dmean_A_rubp            )
      nullify(cgrid%dmean_A_co2             )
      nullify(cgrid%dmean_psi_open          )
      nullify(cgrid%dmean_psi_closed        )
      nullify(cgrid%dmean_water_supply      )
      nullify(cgrid%dmean_par_l             )
      nullify(cgrid%dmean_par_l_beam        )
      nullify(cgrid%dmean_par_l_diff        )
      nullify(cgrid%dmean_rshort_l          )
      nullify(cgrid%dmean_rlong_l           )
      nullify(cgrid%dmean_sensible_lc       )
      nullify(cgrid%dmean_vapor_lc          )
      nullify(cgrid%dmean_transp            )
      nullify(cgrid%dmean_intercepted_al    )
      nullify(cgrid%dmean_wshed_lg          )
      nullify(cgrid%dmean_rshort_w          )
      nullify(cgrid%dmean_rlong_w           )
      nullify(cgrid%dmean_sensible_wc       )
      nullify(cgrid%dmean_vapor_wc          )
      nullify(cgrid%dmean_intercepted_aw    )
      nullify(cgrid%dmean_wshed_wg          )
      nullify(cgrid%dmean_rh                )
      nullify(cgrid%dmean_cwd_rh            )
      nullify(cgrid%dmean_nep               )
      nullify(cgrid%dmean_rk4step           )
      nullify(cgrid%dmean_available_water   )
      nullify(cgrid%dmean_can_theiv         )
      nullify(cgrid%dmean_can_theta         )
      nullify(cgrid%dmean_can_vpdef         )
      nullify(cgrid%dmean_can_temp          )
      nullify(cgrid%dmean_can_shv           )
      nullify(cgrid%dmean_can_co2           )
      nullify(cgrid%dmean_can_rhos          )
      nullify(cgrid%dmean_can_prss          )
      nullify(cgrid%dmean_gnd_temp          )
      nullify(cgrid%dmean_gnd_shv           )
      nullify(cgrid%dmean_can_ggnd          )
      nullify(cgrid%dmean_sfcw_depth        )
      nullify(cgrid%dmean_sfcw_energy       )
      nullify(cgrid%dmean_sfcw_mass         )
      nullify(cgrid%dmean_sfcw_temp         )
      nullify(cgrid%dmean_sfcw_fliq         )
      nullify(cgrid%dmean_soil_energy       )
      nullify(cgrid%dmean_soil_mstpot       )
      nullify(cgrid%dmean_soil_water        )
      nullify(cgrid%dmean_soil_temp         )
      nullify(cgrid%dmean_soil_fliq         )
      nullify(cgrid%dmean_rshort_gnd        )
      nullify(cgrid%dmean_par_gnd           )
      nullify(cgrid%dmean_rlong_gnd         )
      nullify(cgrid%dmean_rlongup           )
      nullify(cgrid%dmean_parup             )
      nullify(cgrid%dmean_nirup             )
      nullify(cgrid%dmean_rshortup          )
      nullify(cgrid%dmean_rnet              )
      nullify(cgrid%dmean_albedo            )
      nullify(cgrid%dmean_albedo_par        )
      nullify(cgrid%dmean_albedo_nir        )
      nullify(cgrid%dmean_rlong_albedo      )
      nullify(cgrid%dmean_ustar             )
      nullify(cgrid%dmean_tstar             )
      nullify(cgrid%dmean_qstar             )
      nullify(cgrid%dmean_cstar             )
      nullify(cgrid%dmean_carbon_ac         )
      nullify(cgrid%dmean_carbon_st         )
      nullify(cgrid%dmean_vapor_gc          )
      nullify(cgrid%dmean_vapor_ac          )
      nullify(cgrid%dmean_smoist_gg         )
      nullify(cgrid%dmean_throughfall       )
      nullify(cgrid%dmean_transloss         )
      nullify(cgrid%dmean_runoff            )
      nullify(cgrid%dmean_drainage          )
      nullify(cgrid%dmean_sensible_gc       )
      nullify(cgrid%dmean_sensible_ac       )
      nullify(cgrid%dmean_sensible_gg       )
      nullify(cgrid%dmean_qthroughfall      )
      nullify(cgrid%dmean_qrunoff           )
      nullify(cgrid%dmean_qdrainage         )
      nullify(cgrid%dmean_atm_theiv         )
      nullify(cgrid%dmean_atm_theta         )
      nullify(cgrid%dmean_atm_temp          )
      nullify(cgrid%dmean_atm_vpdef         )
      nullify(cgrid%dmean_atm_shv           )
      nullify(cgrid%dmean_atm_rshort        )
      nullify(cgrid%dmean_atm_rshort_diff   )
      nullify(cgrid%dmean_atm_par           )
      nullify(cgrid%dmean_atm_par_diff      )
      nullify(cgrid%dmean_atm_rlong         )
      nullify(cgrid%dmean_atm_vels          )
      nullify(cgrid%dmean_atm_rhos          )
      nullify(cgrid%dmean_atm_prss          )
      nullify(cgrid%dmean_atm_co2           )
      nullify(cgrid%dmean_pcpg              )
      nullify(cgrid%dmean_qpcpg             )
      nullify(cgrid%dmean_dpcpg             )
      nullify(cgrid%mmean_lai               )
      nullify(cgrid%mmean_bleaf             )
      nullify(cgrid%mmean_broot             )
      nullify(cgrid%mmean_bstorage          )
      nullify(cgrid%mmean_bleaf_n           )
      nullify(cgrid%mmean_broot_n           )
      nullify(cgrid%mmean_bstorage_n        )
      nullify(cgrid%mmean_leaf_maintenance  )
      nullify(cgrid%mmean_root_maintenance  )
      nullify(cgrid%mmean_leaf_drop         )
      nullify(cgrid%mmean_fast_soil_c       )
      nullify(cgrid%mmean_slow_soil_c       )
      nullify(cgrid%mmean_struct_soil_c     )
      nullify(cgrid%mmean_struct_soil_l     )
      nullify(cgrid%mmean_cwd_c             )
      nullify(cgrid%mmean_fast_soil_n       )
      nullify(cgrid%mmean_mineral_soil_n    )
      nullify(cgrid%mmean_cwd_n             )
      nullify(cgrid%mmean_gpp               )
      nullify(cgrid%mmean_npp               )
      nullify(cgrid%mmean_leaf_resp         )
      nullify(cgrid%mmean_root_resp         )
      nullify(cgrid%mmean_leaf_growth_resp  )
      nullify(cgrid%mmean_root_growth_resp  )
      nullify(cgrid%mmean_sapa_growth_resp  )
      nullify(cgrid%mmean_sapb_growth_resp  )
      nullify(cgrid%mmean_leaf_storage_resp )
      nullify(cgrid%mmean_root_storage_resp )
      nullify(cgrid%mmean_sapa_storage_resp )
      nullify(cgrid%mmean_sapb_storage_resp )
      nullify(cgrid%mmean_plresp            )
      nullify(cgrid%mmean_leaf_energy       )
      nullify(cgrid%mmean_leaf_water        )
      nullify(cgrid%mmean_leaf_hcap         )
      nullify(cgrid%mmean_leaf_vpdef        )
      nullify(cgrid%mmean_leaf_temp         )
      nullify(cgrid%mmean_leaf_fliq         )
      nullify(cgrid%mmean_leaf_gsw          )
      nullify(cgrid%mmean_leaf_gbw          )
      nullify(cgrid%mmean_wood_energy       )
      nullify(cgrid%mmean_wood_water        )
      nullify(cgrid%mmean_wood_hcap         )
      nullify(cgrid%mmean_wood_temp         )
      nullify(cgrid%mmean_wood_fliq         )
      nullify(cgrid%mmean_wood_gbw          )
      nullify(cgrid%mmean_fs_open           )
      nullify(cgrid%mmean_fsw               )
      nullify(cgrid%mmean_fsn               )
      nullify(cgrid%mmean_a_open            )
      nullify(cgrid%mmean_a_closed          )
      nullify(cgrid%mmean_a_net             )
      nullify(cgrid%mmean_A_light           )
      nullify(cgrid%mmean_A_rubp            )
      nullify(cgrid%mmean_A_co2             )
      nullify(cgrid%mmean_psi_open          )
      nullify(cgrid%mmean_psi_closed        )
      nullify(cgrid%mmean_water_supply      )
      nullify(cgrid%mmean_par_l             )
      nullify(cgrid%mmean_par_l_beam        )
      nullify(cgrid%mmean_par_l_diff        )
      nullify(cgrid%mmean_rshort_l          )
      nullify(cgrid%mmean_rlong_l           )
      nullify(cgrid%mmean_sensible_lc       )
      nullify(cgrid%mmean_vapor_lc          )
      nullify(cgrid%mmean_transp            )
      nullify(cgrid%mmean_intercepted_al    )
      nullify(cgrid%mmean_wshed_lg          )
      nullify(cgrid%mmean_rshort_w          )
      nullify(cgrid%mmean_rlong_w           )
      nullify(cgrid%mmean_sensible_wc       )
      nullify(cgrid%mmean_vapor_wc          )
      nullify(cgrid%mmean_intercepted_aw    )
      nullify(cgrid%mmean_wshed_wg          )
      nullify(cgrid%mmean_rh                )
      nullify(cgrid%mmean_cwd_rh            )
      nullify(cgrid%mmean_nep               )
      nullify(cgrid%mmean_rk4step           )
      nullify(cgrid%mmean_available_water   )
      nullify(cgrid%mmean_can_theiv         )
      nullify(cgrid%mmean_can_theta         )
      nullify(cgrid%mmean_can_vpdef         )
      nullify(cgrid%mmean_can_temp          )
      nullify(cgrid%mmean_can_shv           )
      nullify(cgrid%mmean_can_co2           )
      nullify(cgrid%mmean_can_rhos          )
      nullify(cgrid%mmean_can_prss          )
      nullify(cgrid%mmean_gnd_temp          )
      nullify(cgrid%mmean_gnd_shv           )
      nullify(cgrid%mmean_can_ggnd          )
      nullify(cgrid%mmean_sfcw_depth        )
      nullify(cgrid%mmean_sfcw_energy       )
      nullify(cgrid%mmean_sfcw_mass         )
      nullify(cgrid%mmean_sfcw_temp         )
      nullify(cgrid%mmean_sfcw_fliq         )
      nullify(cgrid%mmean_soil_energy       )
      nullify(cgrid%mmean_soil_mstpot       )
      nullify(cgrid%mmean_soil_water        )
      nullify(cgrid%mmean_soil_temp         )
      nullify(cgrid%mmean_soil_fliq         )
      nullify(cgrid%mmean_rshort_gnd        )
      nullify(cgrid%mmean_par_gnd           )
      nullify(cgrid%mmean_rlong_gnd         )
      nullify(cgrid%mmean_rlongup           )
      nullify(cgrid%mmean_parup             )
      nullify(cgrid%mmean_nirup             )
      nullify(cgrid%mmean_rshortup          )
      nullify(cgrid%mmean_rnet              )
      nullify(cgrid%mmean_albedo            )
      nullify(cgrid%mmean_albedo_par        )
      nullify(cgrid%mmean_albedo_nir        )
      nullify(cgrid%mmean_rlong_albedo      )
      nullify(cgrid%mmean_ustar             )
      nullify(cgrid%mmean_tstar             )
      nullify(cgrid%mmean_qstar             )
      nullify(cgrid%mmean_cstar             )
      nullify(cgrid%mmean_carbon_ac         )
      nullify(cgrid%mmean_carbon_st         )
      nullify(cgrid%mmean_vapor_gc          )
      nullify(cgrid%mmean_vapor_ac          )
      nullify(cgrid%mmean_smoist_gg         )
      nullify(cgrid%mmean_throughfall       )
      nullify(cgrid%mmean_transloss         )
      nullify(cgrid%mmean_runoff            )
      nullify(cgrid%mmean_drainage          )
      nullify(cgrid%mmean_sensible_gc       )
      nullify(cgrid%mmean_sensible_ac       )
      nullify(cgrid%mmean_sensible_gg       )
      nullify(cgrid%mmean_qthroughfall      )
      nullify(cgrid%mmean_qrunoff           )
      nullify(cgrid%mmean_qdrainage         )
      nullify(cgrid%mmean_nppleaf           )
      nullify(cgrid%mmean_nppfroot          )
      nullify(cgrid%mmean_nppsapwood        )
      nullify(cgrid%mmean_nppcroot          )
      nullify(cgrid%mmean_nppseeds          )
      nullify(cgrid%mmean_nppwood           )
      nullify(cgrid%mmean_nppdaily          )
      nullify(cgrid%mmean_A_decomp          )
      nullify(cgrid%mmean_Af_decomp         )
      nullify(cgrid%mmean_co2_residual      )
      nullify(cgrid%mmean_energy_residual   )
      nullify(cgrid%mmean_water_residual    )
      nullify(cgrid%mmean_atm_theiv         )
      nullify(cgrid%mmean_atm_theta         )
      nullify(cgrid%mmean_atm_temp          )
      nullify(cgrid%mmean_atm_vpdef         )
      nullify(cgrid%mmean_atm_shv           )
      nullify(cgrid%mmean_atm_rshort        )
      nullify(cgrid%mmean_atm_rshort_diff   )
      nullify(cgrid%mmean_atm_par           )
      nullify(cgrid%mmean_atm_par_diff      )
      nullify(cgrid%mmean_atm_rlong         )
      nullify(cgrid%mmean_atm_vels          )
      nullify(cgrid%mmean_atm_rhos          )
      nullify(cgrid%mmean_atm_prss          )
      nullify(cgrid%mmean_atm_co2           )
      nullify(cgrid%mmean_pcpg              )
      nullify(cgrid%mmean_qpcpg             )
      nullify(cgrid%mmean_dpcpg             )
      nullify(cgrid%mmsqu_gpp               )
      nullify(cgrid%mmsqu_npp               )
      nullify(cgrid%mmsqu_plresp            )
      nullify(cgrid%mmsqu_sensible_lc       )
      nullify(cgrid%mmsqu_vapor_lc          )
      nullify(cgrid%mmsqu_transp            )
      nullify(cgrid%mmsqu_sensible_wc       )
      nullify(cgrid%mmsqu_vapor_wc          )
      nullify(cgrid%mmsqu_rh                )
      nullify(cgrid%mmsqu_cwd_rh            )
      nullify(cgrid%mmsqu_nep               )
      nullify(cgrid%mmsqu_rlongup           )
      nullify(cgrid%mmsqu_parup             )
      nullify(cgrid%mmsqu_nirup             )
      nullify(cgrid%mmsqu_rshortup          )
      nullify(cgrid%mmsqu_rnet              )
      nullify(cgrid%mmsqu_albedo            )
      nullify(cgrid%mmsqu_ustar             )
      nullify(cgrid%mmsqu_carbon_ac         )
      nullify(cgrid%mmsqu_carbon_st         )
      nullify(cgrid%mmsqu_vapor_gc          )
      nullify(cgrid%mmsqu_vapor_ac          )
      nullify(cgrid%mmsqu_sensible_gc       )
      nullify(cgrid%mmsqu_sensible_ac       )
      nullify(cgrid%qmean_gpp               )
      nullify(cgrid%qmean_npp               )
      nullify(cgrid%qmean_leaf_resp         )
      nullify(cgrid%qmean_root_resp         )
      nullify(cgrid%qmean_leaf_growth_resp  )
      nullify(cgrid%qmean_root_growth_resp  )
      nullify(cgrid%qmean_sapa_growth_resp  )
      nullify(cgrid%qmean_sapb_growth_resp  )
      nullify(cgrid%qmean_leaf_storage_resp )
      nullify(cgrid%qmean_root_storage_resp )
      nullify(cgrid%qmean_sapa_storage_resp )
      nullify(cgrid%qmean_sapb_storage_resp )
      nullify(cgrid%qmean_plresp            )
      nullify(cgrid%qmean_leaf_energy       )
      nullify(cgrid%qmean_leaf_water        )
      nullify(cgrid%qmean_leaf_hcap         )
      nullify(cgrid%qmean_leaf_vpdef        )
      nullify(cgrid%qmean_leaf_temp         )
      nullify(cgrid%qmean_leaf_fliq         )
      nullify(cgrid%qmean_leaf_gsw          )
      nullify(cgrid%qmean_leaf_gbw          )
      nullify(cgrid%qmean_wood_energy       )
      nullify(cgrid%qmean_wood_water        )
      nullify(cgrid%qmean_wood_hcap         )
      nullify(cgrid%qmean_wood_temp         )
      nullify(cgrid%qmean_wood_fliq         )
      nullify(cgrid%qmean_wood_gbw          )
      nullify(cgrid%qmean_fs_open           )
      nullify(cgrid%qmean_fsw               )
      nullify(cgrid%qmean_fsn               )
      nullify(cgrid%qmean_a_open            )
      nullify(cgrid%qmean_a_closed          )
      nullify(cgrid%qmean_a_net             )
      nullify(cgrid%qmean_A_light           )
      nullify(cgrid%qmean_A_rubp            )
      nullify(cgrid%qmean_A_co2             )
      nullify(cgrid%qmean_psi_open          )
      nullify(cgrid%qmean_psi_closed        )
      nullify(cgrid%qmean_water_supply      )
      nullify(cgrid%qmean_par_l             )
      nullify(cgrid%qmean_par_l_beam        )
      nullify(cgrid%qmean_par_l_diff        )
      nullify(cgrid%qmean_rshort_l          )
      nullify(cgrid%qmean_rlong_l           )
      nullify(cgrid%qmean_sensible_lc       )
      nullify(cgrid%qmean_vapor_lc          )
      nullify(cgrid%qmean_transp            )
      nullify(cgrid%qmean_intercepted_al    )
      nullify(cgrid%qmean_wshed_lg          )
      nullify(cgrid%qmean_rshort_w          )
      nullify(cgrid%qmean_rlong_w           )
      nullify(cgrid%qmean_sensible_wc       )
      nullify(cgrid%qmean_vapor_wc          )
      nullify(cgrid%qmean_intercepted_aw    )
      nullify(cgrid%qmean_wshed_wg          )
      nullify(cgrid%qmean_rh                )
      nullify(cgrid%qmean_cwd_rh            )
      nullify(cgrid%qmean_nep               )
      nullify(cgrid%qmean_rk4step           )
      nullify(cgrid%qmean_available_water   )
      nullify(cgrid%qmean_can_theiv         )
      nullify(cgrid%qmean_can_theta         )
      nullify(cgrid%qmean_can_vpdef         )
      nullify(cgrid%qmean_can_temp          )
      nullify(cgrid%qmean_can_shv           )
      nullify(cgrid%qmean_can_co2           )
      nullify(cgrid%qmean_can_rhos          )
      nullify(cgrid%qmean_can_prss          )
      nullify(cgrid%qmean_gnd_temp          )
      nullify(cgrid%qmean_gnd_shv           )
      nullify(cgrid%qmean_can_ggnd          )
      nullify(cgrid%qmean_sfcw_depth        )
      nullify(cgrid%qmean_sfcw_energy       )
      nullify(cgrid%qmean_sfcw_mass         )
      nullify(cgrid%qmean_sfcw_temp         )
      nullify(cgrid%qmean_sfcw_fliq         )
      nullify(cgrid%qmean_soil_energy       )
      nullify(cgrid%qmean_soil_mstpot       )
      nullify(cgrid%qmean_soil_water        )
      nullify(cgrid%qmean_soil_temp         )
      nullify(cgrid%qmean_soil_fliq         )
      nullify(cgrid%qmean_rshort_gnd        )
      nullify(cgrid%qmean_par_gnd           )
      nullify(cgrid%qmean_rlong_gnd         )
      nullify(cgrid%qmean_rlongup           )
      nullify(cgrid%qmean_parup             )
      nullify(cgrid%qmean_nirup             )
      nullify(cgrid%qmean_rshortup          )
      nullify(cgrid%qmean_rnet              )
      nullify(cgrid%qmean_albedo            )
      nullify(cgrid%qmean_albedo_par        )
      nullify(cgrid%qmean_albedo_nir        )
      nullify(cgrid%qmean_rlong_albedo      )
      nullify(cgrid%qmean_ustar             )
      nullify(cgrid%qmean_tstar             )
      nullify(cgrid%qmean_qstar             )
      nullify(cgrid%qmean_cstar             )
      nullify(cgrid%qmean_carbon_ac         )
      nullify(cgrid%qmean_carbon_st         )
      nullify(cgrid%qmean_vapor_gc          )
      nullify(cgrid%qmean_vapor_ac          )
      nullify(cgrid%qmean_smoist_gg         )
      nullify(cgrid%qmean_throughfall       )
      nullify(cgrid%qmean_transloss         )
      nullify(cgrid%qmean_runoff            )
      nullify(cgrid%qmean_drainage          )
      nullify(cgrid%qmean_sensible_gc       )
      nullify(cgrid%qmean_sensible_ac       )
      nullify(cgrid%qmean_sensible_gg       )
      nullify(cgrid%qmean_qthroughfall      )
      nullify(cgrid%qmean_qrunoff           )
      nullify(cgrid%qmean_qdrainage         )
      nullify(cgrid%qmean_atm_theiv         )
      nullify(cgrid%qmean_atm_theta         )
      nullify(cgrid%qmean_atm_temp          )
      nullify(cgrid%qmean_atm_vpdef         )
      nullify(cgrid%qmean_atm_shv           )
      nullify(cgrid%qmean_atm_rshort        )
      nullify(cgrid%qmean_atm_rshort_diff   )
      nullify(cgrid%qmean_atm_par           )
      nullify(cgrid%qmean_atm_par_diff      )
      nullify(cgrid%qmean_atm_rlong         )
      nullify(cgrid%qmean_atm_vels          )
      nullify(cgrid%qmean_atm_rhos          )
      nullify(cgrid%qmean_atm_prss          )
      nullify(cgrid%qmean_atm_co2           )
      nullify(cgrid%qmean_pcpg              )
      nullify(cgrid%qmean_qpcpg             )
      nullify(cgrid%qmean_dpcpg             )
      nullify(cgrid%qmsqu_gpp               )
      nullify(cgrid%qmsqu_npp               )
      nullify(cgrid%qmsqu_plresp            )
      nullify(cgrid%qmsqu_sensible_lc       )
      nullify(cgrid%qmsqu_vapor_lc          )
      nullify(cgrid%qmsqu_transp            )
      nullify(cgrid%qmsqu_sensible_wc       )
      nullify(cgrid%qmsqu_vapor_wc          )
      nullify(cgrid%qmsqu_rh                )
      nullify(cgrid%qmsqu_cwd_rh            )
      nullify(cgrid%qmsqu_nep               )
      nullify(cgrid%qmsqu_rlongup           )
      nullify(cgrid%qmsqu_parup             )
      nullify(cgrid%qmsqu_nirup             )
      nullify(cgrid%qmsqu_rshortup          )
      nullify(cgrid%qmsqu_rnet              )
      nullify(cgrid%qmsqu_albedo            )
      nullify(cgrid%qmsqu_ustar             )
      nullify(cgrid%qmsqu_carbon_ac         )
      nullify(cgrid%qmsqu_carbon_st         )
      nullify(cgrid%qmsqu_vapor_gc          )
      nullify(cgrid%qmsqu_vapor_ac          )
      nullify(cgrid%qmsqu_sensible_gc       )
      nullify(cgrid%qmsqu_sensible_ac       )

      return
   end subroutine nullify_edtype
   !=======================================================================================!
   !=======================================================================================!





   !=======================================================================================!
   !=======================================================================================!
   !     This sub-routine nullifies all site pointers.                                     !
   !---------------------------------------------------------------------------------------!
   subroutine nullify_polygontype(cpoly)

      implicit none

      !----- Arguments. -------------------------------------------------------------------!
      type(polygontype), target :: cpoly
      !------------------------------------------------------------------------------------!

      nullify(cpoly%site                       )

      nullify(cpoly%sipa_id                    )
      nullify(cpoly%sipa_n                     )
      nullify(cpoly%patch_count                )
      nullify(cpoly%sitenum                    )
      nullify(cpoly%met                        )
      nullify(cpoly%area                       )
      nullify(cpoly%patch_area                 )
      nullify(cpoly%elevation                  )
      nullify(cpoly%slope                      )
      nullify(cpoly%aspect                     )
      nullify(cpoly%lsl                        )
      nullify(cpoly%ncol_soil                  )
      nullify(cpoly%ntext_soil                 )
      nullify(cpoly%TCI                        )
      nullify(cpoly%pptweight                  )
      nullify(cpoly%hydro_next                 )
      nullify(cpoly%hydro_prev                 )
      nullify(cpoly%moist_W                    )
      nullify(cpoly%moist_f                    )
      nullify(cpoly%moist_tau                  )
      nullify(cpoly%moist_zi                   )
      nullify(cpoly%baseflow                   )
      nullify(cpoly%runoff                     )
      nullify(cpoly%qrunoff                    )
      nullify(cpoly%min_monthly_temp           )
      nullify(cpoly%num_landuse_years          )
      nullify(cpoly%mindbh_primary             )
      nullify(cpoly%probharv_primary           )
      nullify(cpoly%mindbh_secondary           )
      nullify(cpoly%probharv_secondary         )
      nullify(cpoly%plantation                 )
      nullify(cpoly%agri_stocking_pft          )
      nullify(cpoly%agri_stocking_density      )
      nullify(cpoly%plantation_stocking_pft    )
      nullify(cpoly%plantation_stocking_density)
      nullify(cpoly%primary_harvest_target     )
      nullify(cpoly%secondary_harvest_target   )
      nullify(cpoly%primary_harvest_memory     )
      nullify(cpoly%secondary_harvest_memory   )
      nullify(cpoly%ignition_rate              )
      nullify(cpoly%lambda_fire                )
      nullify(cpoly%avg_monthly_pcpg           )
      nullify(cpoly%phen_pars                  )
      nullify(cpoly%disturbance_memory         )
      nullify(cpoly%disturbance_rates          )
      nullify(cpoly%green_leaf_factor          )
      nullify(cpoly%leaf_aging_factor          )
      nullify(cpoly%basal_area                 )
      nullify(cpoly%basal_area_growth          )
      nullify(cpoly%agb                        )
      nullify(cpoly%agb_growth                 )
      nullify(cpoly%basal_area_mort            )
      nullify(cpoly%basal_area_cut             )
      nullify(cpoly%agb_mort                   )
      nullify(cpoly%agb_cut                    )
      nullify(cpoly%cosaoi                     )
      nullify(cpoly%daylight                   )
      nullify(cpoly%nighttime                  )
      nullify(cpoly%rad_avg                    )
      nullify(cpoly%fmean_atm_theiv            )
      nullify(cpoly%fmean_atm_theta            )
      nullify(cpoly%fmean_atm_temp             )
      nullify(cpoly%fmean_atm_vpdef            )
      nullify(cpoly%fmean_atm_shv              )
      nullify(cpoly%fmean_atm_rshort           )
      nullify(cpoly%fmean_atm_rshort_diff      )
      nullify(cpoly%fmean_atm_par              )
      nullify(cpoly%fmean_atm_par_diff         )
      nullify(cpoly%fmean_atm_rlong            )
      nullify(cpoly%fmean_atm_vels             )
      nullify(cpoly%fmean_atm_rhos             )
      nullify(cpoly%fmean_atm_prss             )
      nullify(cpoly%fmean_atm_co2              )
      nullify(cpoly%fmean_pcpg                 )
      nullify(cpoly%fmean_qpcpg                )
      nullify(cpoly%fmean_dpcpg                )
      nullify(cpoly%dmean_atm_theiv            )
      nullify(cpoly%dmean_atm_theta            )
      nullify(cpoly%dmean_atm_temp             )
      nullify(cpoly%dmean_atm_vpdef            )
      nullify(cpoly%dmean_atm_shv              )
      nullify(cpoly%dmean_atm_rshort           )
      nullify(cpoly%dmean_atm_rshort_diff      )
      nullify(cpoly%dmean_atm_par              )
      nullify(cpoly%dmean_atm_par_diff         )
      nullify(cpoly%dmean_atm_rlong            )
      nullify(cpoly%dmean_atm_vels             )
      nullify(cpoly%dmean_atm_rhos             )
      nullify(cpoly%dmean_atm_prss             )
      nullify(cpoly%dmean_atm_co2              )
      nullify(cpoly%dmean_pcpg                 )
      nullify(cpoly%dmean_qpcpg                )
      nullify(cpoly%dmean_dpcpg                )
      nullify(cpoly%mmean_atm_theiv            )
      nullify(cpoly%mmean_atm_theta            )
      nullify(cpoly%mmean_atm_temp             )
      nullify(cpoly%mmean_atm_vpdef            )
      nullify(cpoly%mmean_atm_shv              )
      nullify(cpoly%mmean_atm_rshort           )
      nullify(cpoly%mmean_atm_rshort_diff      )
      nullify(cpoly%mmean_atm_par              )
      nullify(cpoly%mmean_atm_par_diff         )
      nullify(cpoly%mmean_atm_rlong            )
      nullify(cpoly%mmean_atm_vels             )
      nullify(cpoly%mmean_atm_rhos             )
      nullify(cpoly%mmean_atm_prss             )
      nullify(cpoly%mmean_atm_co2              )
      nullify(cpoly%mmean_pcpg                 )
      nullify(cpoly%mmean_qpcpg                )
      nullify(cpoly%mmean_dpcpg                )
      nullify(cpoly%qmean_atm_theiv            )
      nullify(cpoly%qmean_atm_theta            )
      nullify(cpoly%qmean_atm_temp             )
      nullify(cpoly%qmean_atm_vpdef            )
      nullify(cpoly%qmean_atm_shv              )
      nullify(cpoly%qmean_atm_rshort           )
      nullify(cpoly%qmean_atm_rshort_diff      )
      nullify(cpoly%qmean_atm_par              )
      nullify(cpoly%qmean_atm_par_diff         )
      nullify(cpoly%qmean_atm_rlong            )
      nullify(cpoly%qmean_atm_vels             )
      nullify(cpoly%qmean_atm_rhos             )
      nullify(cpoly%qmean_atm_prss             )
      nullify(cpoly%qmean_atm_co2              )
      nullify(cpoly%qmean_pcpg                 )
      nullify(cpoly%qmean_qpcpg                )
      nullify(cpoly%qmean_dpcpg                )

      return
   end subroutine nullify_polygontype
   !=======================================================================================!
   !=======================================================================================!





   !=======================================================================================!
   !=======================================================================================!
   !     This sub-routine nullifies all patch pointers.                                    !
   !---------------------------------------------------------------------------------------!
   subroutine nullify_sitetype(csite)

      implicit none

      !----- Arguments. -------------------------------------------------------------------!
      type(sitetype), target :: csite
      !------------------------------------------------------------------------------------!

      nullify(csite%paco_id                    )
      nullify(csite%paco_n                     )
      nullify(csite%patch                      )
      nullify(csite%cohort_count               )
      nullify(csite%pname                      )
      nullify(csite%area                       )
      nullify(csite%age                        )
      nullify(csite%dist_type                  )
      nullify(csite%fast_soil_C                )
      nullify(csite%slow_soil_C                )
      nullify(csite%structural_soil_C          )
      nullify(csite%structural_soil_L          )
      nullify(csite%mineralized_soil_N         )
      nullify(csite%fast_soil_N                )
      nullify(csite%sum_dgd                    )
      nullify(csite%sum_chd                    )
      nullify(csite%can_theiv                  )
      nullify(csite%can_vpdef                  )
      nullify(csite%can_temp                   )
      nullify(csite%can_temp_pv                )
      nullify(csite%can_shv                    )
      nullify(csite%can_co2                    )
      nullify(csite%can_rhos                   )
      nullify(csite%can_prss                   )
      nullify(csite%can_theta                  )
      nullify(csite%can_depth                  )
      nullify(csite%opencan_frac               )
      nullify(csite%ggbare                     )
      nullify(csite%ggveg                      )
      nullify(csite%ggnet                      )
      nullify(csite%ggsoil                     )
      nullify(csite%nlev_sfcwater              )
      nullify(csite%sfcwater_mass              )
      nullify(csite%sfcwater_energy            )
      nullify(csite%sfcwater_depth             )
      nullify(csite%sfcwater_tempk             )
      nullify(csite%sfcwater_fracliq           )
      nullify(csite%rshort_s                   )
      nullify(csite%rshort_s_beam              )
      nullify(csite%rshort_s_diffuse           )
      nullify(csite%par_s                      )
      nullify(csite%par_s_beam                 )
      nullify(csite%par_s_diffuse              )
      nullify(csite%soil_energy                )
      nullify(csite%soil_mstpot                )
      nullify(csite%soil_water                 )
      nullify(csite%soil_tempk                 )
      nullify(csite%soil_fracliq               )
      nullify(csite%rootdense                  )
      nullify(csite%ground_shv                 )
      nullify(csite%ground_ssh                 )
      nullify(csite%ground_temp                )
      nullify(csite%ground_fliq                )
      nullify(csite%rough                      )
      nullify(csite%par_l_max                  )
      nullify(csite%par_l_beam_max             )
      nullify(csite%par_l_diffuse_max          )
      nullify(csite%A_o_max                    )
      nullify(csite%A_c_max                    )
      nullify(csite%avg_daily_temp             )
      nullify(csite%avg_monthly_gndwater       )
      nullify(csite%avg_monthly_waterdef       )
      nullify(csite%wbudget_loss2atm           )
      nullify(csite%wbudget_denseffect         )
      nullify(csite%wbudget_precipgain         )
      nullify(csite%wbudget_loss2runoff        )
      nullify(csite%wbudget_loss2drainage      )
      nullify(csite%wbudget_initialstorage     )
      nullify(csite%wbudget_residual           )
      nullify(csite%ebudget_loss2atm           )
      nullify(csite%ebudget_denseffect         )
      nullify(csite%ebudget_prsseffect         )
      nullify(csite%ebudget_loss2runoff        )
      nullify(csite%ebudget_loss2drainage      )
      nullify(csite%ebudget_netrad             )
      nullify(csite%ebudget_precipgain         )
      nullify(csite%ebudget_initialstorage     )
      nullify(csite%ebudget_residual           )
      nullify(csite%co2budget_initialstorage   )
      nullify(csite%co2budget_residual         )
      nullify(csite%co2budget_loss2atm         )
      nullify(csite%co2budget_denseffect       )
      nullify(csite%co2budget_gpp              )
      nullify(csite%co2budget_plresp           )
      nullify(csite%co2budget_rh               )
      nullify(csite%today_A_decomp             )
      nullify(csite%today_Af_decomp            )
      nullify(csite%repro                      )
      nullify(csite%veg_rough                  )
      nullify(csite%veg_height                 )
      nullify(csite%veg_displace               )
      nullify(csite%fsc_in                     )
      nullify(csite%ssc_in                     )
      nullify(csite%ssl_in                     )
      nullify(csite%fsn_in                     )
      nullify(csite%total_plant_nitrogen_uptake)
      nullify(csite%mineralized_N_loss         )
      nullify(csite%mineralized_N_input        )
      nullify(csite%rshort_g                   )
      nullify(csite%rshort_g_beam              )
      nullify(csite%rshort_g_diffuse           )
      nullify(csite%par_g                      )
      nullify(csite%par_g_beam                 )
      nullify(csite%par_g_diffuse              )
      nullify(csite%par_b                      )
      nullify(csite%par_b_beam                 )
      nullify(csite%par_b_diffuse              )
      nullify(csite%nir_b                      )
      nullify(csite%nir_b_beam                 )
      nullify(csite%nir_b_diffuse              )
      nullify(csite%rlong_g                    )
      nullify(csite%rlong_s                    )
      nullify(csite%albedo                     )
      nullify(csite%albedo_par                 )
      nullify(csite%albedo_nir                 )
      nullify(csite%rlong_albedo               )
      nullify(csite%rnet                       )
      nullify(csite%rlongup                    )
      nullify(csite%parup                      )
      nullify(csite%nirup                      )
      nullify(csite%rshortup                   )
      nullify(csite%total_sfcw_depth           )
      nullify(csite%snowfac                    )
      nullify(csite%A_decomp                   )
      nullify(csite%f_decomp                   )
      nullify(csite%rh                         )
      nullify(csite%cwd_rh                     )
      nullify(csite%cumlai_profile             )
      nullify(csite%plant_ag_biomass           )
      nullify(csite%htry                       )
      nullify(csite%hprev                      )
      nullify(csite%ustar                      )
      nullify(csite%tstar                      )
      nullify(csite%qstar                      )
      nullify(csite%cstar                      )
      nullify(csite%zeta                       )
      nullify(csite%ribulk                     )
      nullify(csite%upwp                       )
      nullify(csite%qpwp                       )
      nullify(csite%cpwp                       )
      nullify(csite%tpwp                       )
      nullify(csite%wpwp                       )
      nullify(csite%watertable                 )
      nullify(csite%moist_dz                   )
      nullify(csite%ksat                       )
      nullify(csite%soil_sat_energy            )
      nullify(csite%soil_sat_water             )
      nullify(csite%soil_sat_heat              )
      nullify(csite%runoff_A                   )
      nullify(csite%runoff_rate                )
      nullify(csite%runoff                     )
      nullify(csite%qrunoff                    )
      nullify(csite%fmean_rh                   )
      nullify(csite%fmean_cwd_rh               )
      nullify(csite%fmean_nep                  )
      nullify(csite%fmean_rk4step              )
      nullify(csite%fmean_available_water      )
      nullify(csite%fmean_can_theiv            )
      nullify(csite%fmean_can_theta            )
      nullify(csite%fmean_can_vpdef            )
      nullify(csite%fmean_can_temp             )
      nullify(csite%fmean_can_shv              )
      nullify(csite%fmean_can_co2              )
      nullify(csite%fmean_can_rhos             )
      nullify(csite%fmean_can_prss             )
      nullify(csite%fmean_gnd_temp             )
      nullify(csite%fmean_gnd_shv              )
      nullify(csite%fmean_can_ggnd             )
      nullify(csite%fmean_sfcw_depth           )
      nullify(csite%fmean_sfcw_energy          )
      nullify(csite%fmean_sfcw_mass            )
      nullify(csite%fmean_sfcw_temp            )
      nullify(csite%fmean_sfcw_fliq            )
      nullify(csite%fmean_soil_energy          )
      nullify(csite%fmean_soil_mstpot          )
      nullify(csite%fmean_soil_water           )
      nullify(csite%fmean_soil_temp            )
      nullify(csite%fmean_soil_fliq            )
      nullify(csite%fmean_rshort_gnd           )
      nullify(csite%fmean_par_gnd              )
      nullify(csite%fmean_rlong_gnd            )
      nullify(csite%fmean_rlongup              )
      nullify(csite%fmean_parup                )
      nullify(csite%fmean_nirup                )
      nullify(csite%fmean_rshortup             )
      nullify(csite%fmean_rnet                 )
      nullify(csite%fmean_albedo               )
      nullify(csite%fmean_albedo_par           )
      nullify(csite%fmean_albedo_nir           )
      nullify(csite%fmean_rlong_albedo         )
      nullify(csite%fmean_ustar                )
      nullify(csite%fmean_tstar                )
      nullify(csite%fmean_qstar                )
      nullify(csite%fmean_cstar                )
      nullify(csite%fmean_carbon_ac            )
      nullify(csite%fmean_carbon_st            )
      nullify(csite%fmean_vapor_gc             )
      nullify(csite%fmean_vapor_ac             )
      nullify(csite%fmean_smoist_gg            )
      nullify(csite%fmean_throughfall          )
      nullify(csite%fmean_transloss            )
      nullify(csite%fmean_runoff               )
      nullify(csite%fmean_drainage             )
      nullify(csite%fmean_sensible_gc          )
      nullify(csite%fmean_sensible_ac          )
      nullify(csite%fmean_sensible_gg          )
      nullify(csite%fmean_qthroughfall         )
      nullify(csite%fmean_qrunoff              )
      nullify(csite%fmean_qdrainage            )
      nullify(csite%dmean_A_decomp             )
      nullify(csite%dmean_Af_decomp            )
      nullify(csite%dmean_co2_residual         )
      nullify(csite%dmean_energy_residual      )
      nullify(csite%dmean_water_residual       )
      nullify(csite%dmean_rh                   )
      nullify(csite%dmean_cwd_rh               )
      nullify(csite%dmean_nep                  )
      nullify(csite%dmean_rk4step              )
      nullify(csite%dmean_available_water      )
      nullify(csite%dmean_can_theiv            )
      nullify(csite%dmean_can_theta            )
      nullify(csite%dmean_can_vpdef            )
      nullify(csite%dmean_can_temp             )
      nullify(csite%dmean_can_shv              )
      nullify(csite%dmean_can_co2              )
      nullify(csite%dmean_can_rhos             )
      nullify(csite%dmean_can_prss             )
      nullify(csite%dmean_gnd_temp             )
      nullify(csite%dmean_gnd_shv              )
      nullify(csite%dmean_can_ggnd             )
      nullify(csite%dmean_sfcw_depth           )
      nullify(csite%dmean_sfcw_energy          )
      nullify(csite%dmean_sfcw_mass            )
      nullify(csite%dmean_sfcw_temp            )
      nullify(csite%dmean_sfcw_fliq            )
      nullify(csite%dmean_soil_energy          )
      nullify(csite%dmean_soil_mstpot          )
      nullify(csite%dmean_soil_water           )
      nullify(csite%dmean_soil_temp            )
      nullify(csite%dmean_soil_fliq            )
      nullify(csite%dmean_rshort_gnd           )
      nullify(csite%dmean_par_gnd              )
      nullify(csite%dmean_rlong_gnd            )
      nullify(csite%dmean_rlongup              )
      nullify(csite%dmean_parup                )
      nullify(csite%dmean_nirup                )
      nullify(csite%dmean_rshortup             )
      nullify(csite%dmean_rnet                 )
      nullify(csite%dmean_albedo               )
      nullify(csite%dmean_albedo_par           )
      nullify(csite%dmean_albedo_nir           )
      nullify(csite%dmean_rlong_albedo         )
      nullify(csite%dmean_ustar                )
      nullify(csite%dmean_tstar                )
      nullify(csite%dmean_qstar                )
      nullify(csite%dmean_cstar                )
      nullify(csite%dmean_carbon_ac            )
      nullify(csite%dmean_carbon_st            )
      nullify(csite%dmean_vapor_gc             )
      nullify(csite%dmean_vapor_ac             )
      nullify(csite%dmean_smoist_gg            )
      nullify(csite%dmean_throughfall          )
      nullify(csite%dmean_transloss            )
      nullify(csite%dmean_runoff               )
      nullify(csite%dmean_drainage             )
      nullify(csite%dmean_sensible_gc          )
      nullify(csite%dmean_sensible_ac          )
      nullify(csite%dmean_sensible_gg          )
      nullify(csite%dmean_qthroughfall         )
      nullify(csite%dmean_qrunoff              )
      nullify(csite%dmean_qdrainage            )
      nullify(csite%mmean_fast_soil_c          )
      nullify(csite%mmean_slow_soil_c          )
      nullify(csite%mmean_struct_soil_c        )
      nullify(csite%mmean_struct_soil_l        )
      nullify(csite%mmean_fast_soil_n          )
      nullify(csite%mmean_mineral_soil_n       )
      nullify(csite%mmean_co2_residual         )
      nullify(csite%mmean_energy_residual      )
      nullify(csite%mmean_water_residual       )
      nullify(csite%mmean_rh                   )
      nullify(csite%mmean_cwd_rh               )
      nullify(csite%mmean_nep                  )
      nullify(csite%mmean_A_decomp             )
      nullify(csite%mmean_Af_decomp            )
      nullify(csite%mmean_rk4step              )
      nullify(csite%mmean_available_water      )
      nullify(csite%mmean_can_theiv            )
      nullify(csite%mmean_can_theta            )
      nullify(csite%mmean_can_vpdef            )
      nullify(csite%mmean_can_temp             )
      nullify(csite%mmean_can_shv              )
      nullify(csite%mmean_can_co2              )
      nullify(csite%mmean_can_rhos             )
      nullify(csite%mmean_can_prss             )
      nullify(csite%mmean_gnd_temp             )
      nullify(csite%mmean_gnd_shv              )
      nullify(csite%mmean_can_ggnd             )
      nullify(csite%mmean_sfcw_depth           )
      nullify(csite%mmean_sfcw_energy          )
      nullify(csite%mmean_sfcw_mass            )
      nullify(csite%mmean_sfcw_temp            )
      nullify(csite%mmean_sfcw_fliq            )
      nullify(csite%mmean_soil_energy          )
      nullify(csite%mmean_soil_mstpot          )
      nullify(csite%mmean_soil_water           )
      nullify(csite%mmean_soil_temp            )
      nullify(csite%mmean_soil_fliq            )
      nullify(csite%mmean_rshort_gnd           )
      nullify(csite%mmean_par_gnd              )
      nullify(csite%mmean_rlong_gnd            )
      nullify(csite%mmean_rlongup              )
      nullify(csite%mmean_parup                )
      nullify(csite%mmean_nirup                )
      nullify(csite%mmean_rshortup             )
      nullify(csite%mmean_rnet                 )
      nullify(csite%mmean_albedo               )
      nullify(csite%mmean_albedo_par           )
      nullify(csite%mmean_albedo_nir           )
      nullify(csite%mmean_rlong_albedo         )
      nullify(csite%mmean_ustar                )
      nullify(csite%mmean_tstar                )
      nullify(csite%mmean_qstar                )
      nullify(csite%mmean_cstar                )
      nullify(csite%mmean_carbon_ac            )
      nullify(csite%mmean_carbon_st            )
      nullify(csite%mmean_vapor_gc             )
      nullify(csite%mmean_vapor_ac             )
      nullify(csite%mmean_smoist_gg            )
      nullify(csite%mmean_throughfall          )
      nullify(csite%mmean_transloss            )
      nullify(csite%mmean_runoff               )
      nullify(csite%mmean_drainage             )
      nullify(csite%mmean_sensible_gc          )
      nullify(csite%mmean_sensible_ac          )
      nullify(csite%mmean_sensible_gg          )
      nullify(csite%mmean_qthroughfall         )
      nullify(csite%mmean_qrunoff              )
      nullify(csite%mmean_qdrainage            )
      nullify(csite%mmean_A_decomp             )
      nullify(csite%mmean_Af_decomp            )
      nullify(csite%mmean_co2_residual         )
      nullify(csite%mmean_energy_residual      )
      nullify(csite%mmean_water_residual       )
      nullify(csite%mmsqu_rh                   )
      nullify(csite%mmsqu_cwd_rh               )
      nullify(csite%mmsqu_nep                  )
      nullify(csite%mmsqu_rlongup              )
      nullify(csite%mmsqu_parup                )
      nullify(csite%mmsqu_nirup                )
      nullify(csite%mmsqu_rshortup             )
      nullify(csite%mmsqu_rnet                 )
      nullify(csite%mmsqu_albedo               )
      nullify(csite%mmsqu_ustar                )
      nullify(csite%mmsqu_carbon_ac            )
      nullify(csite%mmsqu_carbon_st            )
      nullify(csite%mmsqu_vapor_gc             )
      nullify(csite%mmsqu_vapor_ac             )
      nullify(csite%mmsqu_sensible_gc          )
      nullify(csite%mmsqu_sensible_ac          )
      nullify(csite%qmean_rh                   )
      nullify(csite%qmean_cwd_rh               )
      nullify(csite%qmean_nep                  )
      nullify(csite%qmean_rk4step              )
      nullify(csite%qmean_available_water      )
      nullify(csite%qmean_can_theiv            )
      nullify(csite%qmean_can_theta            )
      nullify(csite%qmean_can_vpdef            )
      nullify(csite%qmean_can_temp             )
      nullify(csite%qmean_can_shv              )
      nullify(csite%qmean_can_co2              )
      nullify(csite%qmean_can_rhos             )
      nullify(csite%qmean_can_prss             )
      nullify(csite%qmean_gnd_temp             )
      nullify(csite%qmean_gnd_shv              )
      nullify(csite%qmean_can_ggnd             )
      nullify(csite%qmean_sfcw_depth           )
      nullify(csite%qmean_sfcw_energy          )
      nullify(csite%qmean_sfcw_mass            )
      nullify(csite%qmean_sfcw_temp            )
      nullify(csite%qmean_sfcw_fliq            )
      nullify(csite%qmean_soil_energy          )
      nullify(csite%qmean_soil_mstpot          )
      nullify(csite%qmean_soil_water           )
      nullify(csite%qmean_soil_temp            )
      nullify(csite%qmean_soil_fliq            )
      nullify(csite%qmean_rshort_gnd           )
      nullify(csite%qmean_par_gnd              )
      nullify(csite%qmean_rlong_gnd            )
      nullify(csite%qmean_rlongup              )
      nullify(csite%qmean_parup                )
      nullify(csite%qmean_nirup                )
      nullify(csite%qmean_rshortup             )
      nullify(csite%qmean_rnet                 )
      nullify(csite%qmean_albedo               )
      nullify(csite%qmean_albedo_par           )
      nullify(csite%qmean_albedo_nir           )
      nullify(csite%qmean_rlong_albedo         )
      nullify(csite%qmean_ustar                )
      nullify(csite%qmean_tstar                )
      nullify(csite%qmean_qstar                )
      nullify(csite%qmean_cstar                )
      nullify(csite%qmean_carbon_ac            )
      nullify(csite%qmean_carbon_st            )
      nullify(csite%qmean_vapor_gc             )
      nullify(csite%qmean_vapor_ac             )
      nullify(csite%qmean_smoist_gg            )
      nullify(csite%qmean_throughfall          )
      nullify(csite%qmean_transloss            )
      nullify(csite%qmean_runoff               )
      nullify(csite%qmean_drainage             )
      nullify(csite%qmean_sensible_gc          )
      nullify(csite%qmean_sensible_ac          )
      nullify(csite%qmean_sensible_gg          )
      nullify(csite%qmean_qthroughfall         )
      nullify(csite%qmean_qrunoff              )
      nullify(csite%qmean_qdrainage            )
      nullify(csite%qmsqu_rh                   )
      nullify(csite%qmsqu_cwd_rh               )
      nullify(csite%qmsqu_nep                  )
      nullify(csite%qmsqu_rlongup              )
      nullify(csite%qmsqu_parup                )
      nullify(csite%qmsqu_nirup                )
      nullify(csite%qmsqu_rshortup             )
      nullify(csite%qmsqu_rnet                 )
      nullify(csite%qmsqu_albedo               )
      nullify(csite%qmsqu_ustar                )
      nullify(csite%qmsqu_carbon_ac            )
      nullify(csite%qmsqu_carbon_st            )
      nullify(csite%qmsqu_vapor_gc             )
      nullify(csite%qmsqu_vapor_ac             )
      nullify(csite%qmsqu_sensible_gc          )
      nullify(csite%qmsqu_sensible_ac          )

      return
   end subroutine nullify_sitetype
   !=======================================================================================!
   !=======================================================================================!





   !=======================================================================================!
   !=======================================================================================!
   !     This sub-routine nullifies all cohort pointers.                                   !
   !---------------------------------------------------------------------------------------!
   subroutine nullify_patchtype(cpatch)

      implicit none

      !----- Arguments. -------------------------------------------------------------------!
      type(patchtype), target :: cpatch
      !------------------------------------------------------------------------------------!

      nullify(cpatch%pft                   )
      nullify(cpatch%nplant                )
      nullify(cpatch%phenology_status      )
      nullify(cpatch%recruit_dbh           )
      nullify(cpatch%census_status         )
      nullify(cpatch%hite                  )
      nullify(cpatch%agb                   )
      nullify(cpatch%basarea               )
      nullify(cpatch%dagb_dt               )
      nullify(cpatch%dlnagb_dt             )
      nullify(cpatch%dba_dt                )
      nullify(cpatch%dlnba_dt              )
      nullify(cpatch%ddbh_dt               )
      nullify(cpatch%dlndbh_dt             )
      nullify(cpatch%dbh                   )
      nullify(cpatch%bdead                 )
      nullify(cpatch%bleaf                 )
      nullify(cpatch%balive                )
      nullify(cpatch%broot                 )
      nullify(cpatch%bsapwooda             )
      nullify(cpatch%bsapwoodb             )
      nullify(cpatch%bstorage              )
      nullify(cpatch%bseeds                )
      nullify(cpatch%lai                   )
      nullify(cpatch%wai                   )
      nullify(cpatch%crown_area            )
      nullify(cpatch%leaf_resolvable       )
      nullify(cpatch%wood_resolvable       )
      nullify(cpatch%cb                    )
      nullify(cpatch%cb_lightmax           )
      nullify(cpatch%cb_moistmax           )
      nullify(cpatch%cb_mlmax              )
      nullify(cpatch%cbr_bar               )
      nullify(cpatch%leaf_energy           )
      nullify(cpatch%leaf_temp             )
      nullify(cpatch%leaf_vpdef            )
      nullify(cpatch%leaf_temp_pv          )
      nullify(cpatch%leaf_hcap             )
      nullify(cpatch%leaf_fliq             )
      nullify(cpatch%leaf_water            )
      nullify(cpatch%wood_energy           )
      nullify(cpatch%wood_temp             )
      nullify(cpatch%wood_temp_pv          )
      nullify(cpatch%wood_hcap             )
      nullify(cpatch%wood_fliq             )
      nullify(cpatch%wood_water            )
      nullify(cpatch%veg_wind              )
      nullify(cpatch%lsfc_shv_open         )
      nullify(cpatch%lsfc_shv_closed       )
      nullify(cpatch%lsfc_co2_open         )
      nullify(cpatch%lsfc_co2_closed       )
      nullify(cpatch%lint_shv              )
      nullify(cpatch%lint_co2_open         )
      nullify(cpatch%lint_co2_closed       )
      nullify(cpatch%today_leaf_resp       )
      nullify(cpatch%today_root_resp       )
      nullify(cpatch%today_gpp             )
      nullify(cpatch%today_gpp_pot         )
      nullify(cpatch%today_gpp_lightmax    )
      nullify(cpatch%today_gpp_moistmax    )
      nullify(cpatch%today_gpp_mlmax       )
      nullify(cpatch%today_nppleaf         )
      nullify(cpatch%today_nppfroot        )
      nullify(cpatch%today_nppsapwood      )
      nullify(cpatch%today_nppcroot        )
      nullify(cpatch%today_nppseeds        )
      nullify(cpatch%today_nppwood         )
      nullify(cpatch%today_nppdaily        )
      nullify(cpatch%leaf_growth_resp      )
      nullify(cpatch%root_growth_resp      )
      nullify(cpatch%sapa_growth_resp      )
      nullify(cpatch%sapb_growth_resp      )
      nullify(cpatch%leaf_storage_resp     )
      nullify(cpatch%root_storage_resp     )
      nullify(cpatch%sapa_storage_resp     )
      nullify(cpatch%sapb_storage_resp     )
      nullify(cpatch%monthly_dndt          )
      nullify(cpatch%monthly_dlnndt        )
      nullify(cpatch%mort_rate             )
      nullify(cpatch%krdepth               )
      nullify(cpatch%first_census          )
      nullify(cpatch%new_recruit_flag      )
      nullify(cpatch%light_level           )
      nullify(cpatch%light_level_beam      )
      nullify(cpatch%light_level_diff      )

      nullify(cpatch%par_level_beam        )
      nullify(cpatch%par_level_diffu       )
      nullify(cpatch%par_level_diffd       )

      nullify(cpatch%par_l                 )
      nullify(cpatch%par_l_beam            )
      nullify(cpatch%par_l_diffuse         )
      nullify(cpatch%rshort_l              )
      nullify(cpatch%rshort_l_beam         )
      nullify(cpatch%rshort_l_diffuse      )
      nullify(cpatch%rlong_l               )
      nullify(cpatch%rshort_w              )
      nullify(cpatch%rshort_w_beam         )
      nullify(cpatch%rshort_w_diffuse      )
      nullify(cpatch%rlong_w               )
      nullify(cpatch%rad_profile           )
      nullify(cpatch%leaf_gbh              )
      nullify(cpatch%leaf_gbw              )
      nullify(cpatch%wood_gbh              )
      nullify(cpatch%wood_gbw              )
      nullify(cpatch%A_open                )
      nullify(cpatch%A_closed              )
      nullify(cpatch%A_light               )
      nullify(cpatch%A_rubp                )
      nullify(cpatch%A_co2                 )
      nullify(cpatch%psi_open              )
      nullify(cpatch%psi_closed            )
      nullify(cpatch%gsw_open              )
      nullify(cpatch%gsw_closed            )
      nullify(cpatch%leaf_gsw              )
      nullify(cpatch%fsw                   )
      nullify(cpatch%fsn                   )
      nullify(cpatch%fs_open               )
      nullify(cpatch%water_supply          )
      nullify(cpatch%leaf_maintenance      )
      nullify(cpatch%root_maintenance      )
      nullify(cpatch%leaf_drop             )
      nullify(cpatch%leaf_respiration      )
      nullify(cpatch%root_respiration      )
      nullify(cpatch%gpp                   )
      nullify(cpatch%paw_avg               )
      nullify(cpatch%elongf                )
      nullify(cpatch%turnover_amp          )
      nullify(cpatch%llspan                )
      nullify(cpatch%vm_bar                )
      nullify(cpatch%sla                   )
      nullify(cpatch%fmean_gpp             )
      nullify(cpatch%fmean_npp             )
      nullify(cpatch%fmean_leaf_resp       )
      nullify(cpatch%fmean_root_resp       )
      nullify(cpatch%fmean_leaf_growth_resp)
      nullify(cpatch%fmean_root_growth_resp)
      nullify(cpatch%fmean_sapa_growth_resp)
      nullify(cpatch%fmean_sapb_growth_resp)
      nullify(cpatch%fmean_leaf_storage_resp)
      nullify(cpatch%fmean_root_storage_resp)
      nullify(cpatch%fmean_sapa_storage_resp)
      nullify(cpatch%fmean_sapb_storage_resp)
      nullify(cpatch%fmean_plresp          )
      nullify(cpatch%fmean_leaf_energy     )
      nullify(cpatch%fmean_leaf_water      )
      nullify(cpatch%fmean_leaf_hcap       )
      nullify(cpatch%fmean_leaf_vpdef      )
      nullify(cpatch%fmean_leaf_temp       )
      nullify(cpatch%fmean_leaf_fliq       )
      nullify(cpatch%fmean_leaf_gsw        )
      nullify(cpatch%fmean_leaf_gbw        )
      nullify(cpatch%fmean_wood_energy     )
      nullify(cpatch%fmean_wood_water      )
      nullify(cpatch%fmean_wood_hcap       )
      nullify(cpatch%fmean_wood_temp       )
      nullify(cpatch%fmean_wood_fliq       )
      nullify(cpatch%fmean_wood_gbw        )
      nullify(cpatch%fmean_fs_open         )
      nullify(cpatch%fmean_fsw             )
      nullify(cpatch%fmean_fsn             )
      nullify(cpatch%fmean_a_open          )
      nullify(cpatch%fmean_a_closed        )
      nullify(cpatch%fmean_a_net           )
      nullify(cpatch%fmean_A_light         )
      nullify(cpatch%fmean_A_rubp          )
      nullify(cpatch%fmean_A_co2           )
      nullify(cpatch%fmean_psi_open        )
      nullify(cpatch%fmean_psi_closed      )
      nullify(cpatch%fmean_water_supply    )
      nullify(cpatch%fmean_light_level     )
      nullify(cpatch%fmean_light_level_beam)
      nullify(cpatch%fmean_light_level_diff)
      
      nullify(cpatch%fmean_par_level_beam  )
      nullify(cpatch%fmean_par_level_diffu )
      nullify(cpatch%fmean_par_level_diffd )

      nullify(cpatch%fmean_par_l           )
      nullify(cpatch%fmean_par_l_beam      )
      nullify(cpatch%fmean_par_l_diff      )
      nullify(cpatch%fmean_rshort_l        )
      nullify(cpatch%fmean_rlong_l         )
      nullify(cpatch%fmean_sensible_lc     )
      nullify(cpatch%fmean_vapor_lc        )
      nullify(cpatch%fmean_transp          )
      nullify(cpatch%fmean_intercepted_al  )
      nullify(cpatch%fmean_wshed_lg        )
      nullify(cpatch%fmean_rshort_w        )
      nullify(cpatch%fmean_rlong_w         )
      nullify(cpatch%fmean_rad_profile     )
      nullify(cpatch%fmean_sensible_wc     )
      nullify(cpatch%fmean_vapor_wc        )
      nullify(cpatch%fmean_intercepted_aw  )
      nullify(cpatch%fmean_wshed_wg        )
      nullify(cpatch%fmean_lai             )
      nullify(cpatch%fmean_bdead           )
      nullify(cpatch%dmean_nppleaf         )
      nullify(cpatch%dmean_nppfroot        )
      nullify(cpatch%dmean_nppsapwood      )
      nullify(cpatch%dmean_nppcroot        )
      nullify(cpatch%dmean_nppseeds        )
      nullify(cpatch%dmean_nppwood         )
      nullify(cpatch%dmean_nppdaily        )
      nullify(cpatch%dmean_gpp             )
      nullify(cpatch%dmean_npp             )
      nullify(cpatch%dmean_leaf_resp       )
      nullify(cpatch%dmean_root_resp       )
      nullify(cpatch%dmean_leaf_growth_resp)
      nullify(cpatch%dmean_root_growth_resp)
      nullify(cpatch%dmean_sapa_growth_resp)
      nullify(cpatch%dmean_sapb_growth_resp)
      nullify(cpatch%dmean_leaf_storage_resp)
      nullify(cpatch%dmean_root_storage_resp)
      nullify(cpatch%dmean_sapa_storage_resp)
      nullify(cpatch%dmean_sapb_storage_resp)
      nullify(cpatch%dmean_plresp          )
      nullify(cpatch%dmean_leaf_energy     )
      nullify(cpatch%dmean_leaf_water      )
      nullify(cpatch%dmean_leaf_hcap       )
      nullify(cpatch%dmean_leaf_vpdef      )
      nullify(cpatch%dmean_leaf_temp       )
      nullify(cpatch%dmean_leaf_fliq       )
      nullify(cpatch%dmean_leaf_gsw        )
      nullify(cpatch%dmean_leaf_gbw        )
      nullify(cpatch%dmean_wood_energy     )
      nullify(cpatch%dmean_wood_water      )
      nullify(cpatch%dmean_wood_hcap       )
      nullify(cpatch%dmean_wood_temp       )
      nullify(cpatch%dmean_wood_fliq       )
      nullify(cpatch%dmean_wood_gbw        )
      nullify(cpatch%dmean_fs_open         )
      nullify(cpatch%dmean_fsw             )
      nullify(cpatch%dmean_fsn             )
      nullify(cpatch%dmean_a_open          )
      nullify(cpatch%dmean_a_closed        )
      nullify(cpatch%dmean_a_net           )
      nullify(cpatch%dmean_A_light         )
      nullify(cpatch%dmean_A_rubp          )
      nullify(cpatch%dmean_A_co2           )
      nullify(cpatch%dmean_psi_open        )
      nullify(cpatch%dmean_psi_closed      )
      nullify(cpatch%dmean_water_supply    )
      nullify(cpatch%dmean_light_level     )
      nullify(cpatch%dmean_light_level_beam)
      nullify(cpatch%dmean_light_level_diff)

      nullify(cpatch%dmean_par_level_beam  )
      nullify(cpatch%dmean_par_level_diffu )
      nullify(cpatch%dmean_par_level_diffd )

      nullify(cpatch%dmean_par_l           )
      nullify(cpatch%dmean_par_l_beam      )
      nullify(cpatch%dmean_par_l_diff      )
      nullify(cpatch%dmean_rshort_l        )
      nullify(cpatch%dmean_rlong_l         )
      nullify(cpatch%dmean_sensible_lc     )
      nullify(cpatch%dmean_vapor_lc        )
      nullify(cpatch%dmean_transp          )
      nullify(cpatch%dmean_intercepted_al  )
      nullify(cpatch%dmean_wshed_lg        )
      nullify(cpatch%dmean_rshort_w        )
      nullify(cpatch%dmean_rlong_w         )
      nullify(cpatch%dmean_rad_profile     )
      nullify(cpatch%dmean_sensible_wc     )
      nullify(cpatch%dmean_vapor_wc        )
      nullify(cpatch%dmean_intercepted_aw  )
      nullify(cpatch%dmean_wshed_wg        )
      nullify(cpatch%mmean_lai             )
      nullify(cpatch%mmean_bleaf           )
      nullify(cpatch%mmean_broot           )
      nullify(cpatch%mmean_bstorage        )
      nullify(cpatch%mmean_mort_rate       )
      nullify(cpatch%mmean_leaf_maintenance)
      nullify(cpatch%mmean_root_maintenance)
      nullify(cpatch%mmean_leaf_drop       )
      nullify(cpatch%mmean_cb              )
      nullify(cpatch%mmean_gpp             )
      nullify(cpatch%mmean_npp             )
      nullify(cpatch%mmean_leaf_resp       )
      nullify(cpatch%mmean_root_resp       )
      nullify(cpatch%mmean_leaf_growth_resp)
      nullify(cpatch%mmean_root_growth_resp)
      nullify(cpatch%mmean_sapa_growth_resp)
      nullify(cpatch%mmean_sapb_growth_resp)
      nullify(cpatch%mmean_leaf_storage_resp)
      nullify(cpatch%mmean_root_storage_resp)
      nullify(cpatch%mmean_sapa_storage_resp)
      nullify(cpatch%mmean_sapb_storage_resp)
      nullify(cpatch%mmean_plresp          )
      nullify(cpatch%mmean_leaf_energy     )
      nullify(cpatch%mmean_leaf_water      )
      nullify(cpatch%mmean_leaf_hcap       )
      nullify(cpatch%mmean_leaf_vpdef      )
      nullify(cpatch%mmean_leaf_temp       )
      nullify(cpatch%mmean_leaf_fliq       )
      nullify(cpatch%mmean_leaf_gsw        )
      nullify(cpatch%mmean_leaf_gbw        )
      nullify(cpatch%mmean_wood_energy     )
      nullify(cpatch%mmean_wood_water      )
      nullify(cpatch%mmean_wood_hcap       )
      nullify(cpatch%mmean_wood_temp       )
      nullify(cpatch%mmean_wood_fliq       )
      nullify(cpatch%mmean_wood_gbw        )
      nullify(cpatch%mmean_fs_open         )
      nullify(cpatch%mmean_fsw             )
      nullify(cpatch%mmean_fsn             )
      nullify(cpatch%mmean_a_open          )
      nullify(cpatch%mmean_a_closed        )
      nullify(cpatch%mmean_a_net           )
      nullify(cpatch%mmean_A_light         )
      nullify(cpatch%mmean_A_rubp          )
      nullify(cpatch%mmean_A_co2           )
      nullify(cpatch%mmean_psi_open        )
      nullify(cpatch%mmean_psi_closed      )
      nullify(cpatch%mmean_water_supply    )
      nullify(cpatch%mmean_light_level     )
      nullify(cpatch%mmean_light_level_beam)
      nullify(cpatch%mmean_light_level_diff)

      nullify(cpatch%mmean_par_level_beam  )
      nullify(cpatch%mmean_par_level_diffu )
      nullify(cpatch%mmean_par_level_diffd )

      nullify(cpatch%mmean_par_l           )
      nullify(cpatch%mmean_par_l_beam      )
      nullify(cpatch%mmean_par_l_diff      )
      nullify(cpatch%mmean_rshort_l        )
      nullify(cpatch%mmean_rlong_l         )
      nullify(cpatch%mmean_sensible_lc     )
      nullify(cpatch%mmean_vapor_lc        )
      nullify(cpatch%mmean_transp          )
      nullify(cpatch%mmean_intercepted_al  )
      nullify(cpatch%mmean_wshed_lg        )
      nullify(cpatch%mmean_rshort_w        )
      nullify(cpatch%mmean_rlong_w         )
      nullify(cpatch%mmean_rad_profile     )
      nullify(cpatch%mmean_sensible_wc     )
      nullify(cpatch%mmean_vapor_wc        )
      nullify(cpatch%mmean_intercepted_aw  )
      nullify(cpatch%mmean_wshed_wg        )
      nullify(cpatch%mmean_nppleaf         )
      nullify(cpatch%mmean_nppfroot        )
      nullify(cpatch%mmean_nppsapwood      )
      nullify(cpatch%mmean_nppcroot        )
      nullify(cpatch%mmean_nppseeds        )
      nullify(cpatch%mmean_nppwood         )
      nullify(cpatch%mmean_nppdaily        )
      nullify(cpatch%mmsqu_gpp             )
      nullify(cpatch%mmsqu_npp             )
      nullify(cpatch%mmsqu_plresp          )
      nullify(cpatch%mmsqu_sensible_lc     )
      nullify(cpatch%mmsqu_vapor_lc        )
      nullify(cpatch%mmsqu_transp          )
      nullify(cpatch%mmsqu_sensible_wc     )
      nullify(cpatch%mmsqu_vapor_wc        )
      nullify(cpatch%qmean_gpp             )
      nullify(cpatch%qmean_npp             )
      nullify(cpatch%qmean_leaf_resp       )
      nullify(cpatch%qmean_root_resp       )
      nullify(cpatch%qmean_leaf_growth_resp)
      nullify(cpatch%qmean_root_growth_resp)
      nullify(cpatch%qmean_sapa_growth_resp)
      nullify(cpatch%qmean_sapb_growth_resp)
      nullify(cpatch%qmean_leaf_storage_resp)
      nullify(cpatch%qmean_root_storage_resp)
      nullify(cpatch%qmean_sapa_storage_resp)
      nullify(cpatch%qmean_sapb_storage_resp)
      nullify(cpatch%qmean_plresp          )
      nullify(cpatch%qmean_leaf_energy     )
      nullify(cpatch%qmean_leaf_water      )
      nullify(cpatch%qmean_leaf_hcap       )
      nullify(cpatch%qmean_leaf_vpdef      )
      nullify(cpatch%qmean_leaf_temp       )
      nullify(cpatch%qmean_leaf_fliq       )
      nullify(cpatch%qmean_leaf_gsw        )
      nullify(cpatch%qmean_leaf_gbw        )
      nullify(cpatch%qmean_wood_energy     )
      nullify(cpatch%qmean_wood_water      )
      nullify(cpatch%qmean_wood_hcap       )
      nullify(cpatch%qmean_wood_temp       )
      nullify(cpatch%qmean_wood_fliq       )
      nullify(cpatch%qmean_wood_gbw        )
      nullify(cpatch%qmean_fs_open         )
      nullify(cpatch%qmean_fsw             )
      nullify(cpatch%qmean_fsn             )
      nullify(cpatch%qmean_a_open          )
      nullify(cpatch%qmean_a_closed        )
      nullify(cpatch%qmean_a_net           )
      nullify(cpatch%qmean_A_light         )
      nullify(cpatch%qmean_A_rubp          )
      nullify(cpatch%qmean_A_co2           )
      nullify(cpatch%qmean_psi_open        )
      nullify(cpatch%qmean_psi_closed      )
      nullify(cpatch%qmean_water_supply    )
      nullify(cpatch%qmean_light_level     )
      nullify(cpatch%qmean_light_level_beam)
      nullify(cpatch%qmean_light_level_diff)

      nullify(cpatch%qmean_par_level_beam  )
      nullify(cpatch%qmean_par_level_diffu )
      nullify(cpatch%qmean_par_level_diffd )

      nullify(cpatch%qmean_par_l           )
      nullify(cpatch%qmean_par_l_beam      )
      nullify(cpatch%qmean_par_l_diff      )
      nullify(cpatch%qmean_rshort_l        )
      nullify(cpatch%qmean_rlong_l         )
      nullify(cpatch%qmean_sensible_lc     )
      nullify(cpatch%qmean_vapor_lc        )
      nullify(cpatch%qmean_transp          )
      nullify(cpatch%qmean_intercepted_al  )
      nullify(cpatch%qmean_wshed_lg        )
      nullify(cpatch%qmean_rshort_w        )
      nullify(cpatch%qmean_rlong_w         )
      nullify(cpatch%qmean_rad_profile     )
      nullify(cpatch%qmean_sensible_wc     )
      nullify(cpatch%qmean_vapor_wc        )
      nullify(cpatch%qmean_intercepted_aw  )
      nullify(cpatch%qmean_wshed_wg        )
      nullify(cpatch%qmsqu_gpp             )
      nullify(cpatch%qmsqu_npp             )
      nullify(cpatch%qmsqu_plresp          )
      nullify(cpatch%qmsqu_sensible_lc     )
      nullify(cpatch%qmsqu_vapor_lc        )
      nullify(cpatch%qmsqu_transp          )
      nullify(cpatch%qmsqu_sensible_wc     )
      nullify(cpatch%qmsqu_vapor_wc        )

      return
   end subroutine nullify_patchtype
   !=======================================================================================!
   !=======================================================================================!





   !=======================================================================================!
   !=======================================================================================!
   !     This sub-routine de-allocates all polygon pointers.                               !
   !---------------------------------------------------------------------------------------!
   subroutine deallocate_edtype(cgrid)
      implicit none

      !----- Arguments. -------------------------------------------------------------------!
      type(edtype)     , target :: cgrid
      !----- Local variables. -------------------------------------------------------------!
      integer                   :: ipy
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      !     First thing we must do is to deallocate the nested dimensions.                 !
      !------------------------------------------------------------------------------------!
      do ipy=1,cgrid%npolygons
         call deallocate_polygontype(cgrid%polygon(ipy))
      end do
      if (associated(cgrid%polygon)) deallocate(cgrid%polygon)
      !------------------------------------------------------------------------------------!

      if(associated(cgrid%pysi_id               )) deallocate(cgrid%pysi_id               )
      if(associated(cgrid%pysi_n                )) deallocate(cgrid%pysi_n                )
      if(associated(cgrid%walltime_py           )) deallocate(cgrid%walltime_py           )
      if(associated(cgrid%lon                   )) deallocate(cgrid%lon                   )
      if(associated(cgrid%lat                   )) deallocate(cgrid%lat                   )
      if(associated(cgrid%xatm                  )) deallocate(cgrid%xatm                  )
      if(associated(cgrid%yatm                  )) deallocate(cgrid%yatm                  )
      if(associated(cgrid%site_adjacency        )) deallocate(cgrid%site_adjacency        )
      if(associated(cgrid%wbar                  )) deallocate(cgrid%wbar                  )
      if(associated(cgrid%Te                    )) deallocate(cgrid%Te                    )
      if(associated(cgrid%zbar                  )) deallocate(cgrid%zbar                  )
      if(associated(cgrid%sheat                 )) deallocate(cgrid%sheat                 )
      if(associated(cgrid%baseflow              )) deallocate(cgrid%baseflow              )
      if(associated(cgrid%runoff                )) deallocate(cgrid%runoff                )
      if(associated(cgrid%qrunoff               )) deallocate(cgrid%qrunoff               )
      if(associated(cgrid%swliq                 )) deallocate(cgrid%swliq                 )
      if(associated(cgrid%ilon                  )) deallocate(cgrid%ilon                  )
      if(associated(cgrid%ilat                  )) deallocate(cgrid%ilat                  )
      if(associated(cgrid%workload              )) deallocate(cgrid%workload              )
      if(associated(cgrid%total_agb             )) deallocate(cgrid%total_agb             )
      if(associated(cgrid%total_basal_area      )) deallocate(cgrid%total_basal_area      )
      if(associated(cgrid%total_agb_growth      )) deallocate(cgrid%total_agb_growth      )
      if(associated(cgrid%total_agb_mort        )) deallocate(cgrid%total_agb_mort        )
      if(associated(cgrid%total_agb_recruit     )) deallocate(cgrid%total_agb_recruit     )
      if(associated(cgrid%total_basal_area_growth ))                                       &
                                                 deallocate(cgrid%total_basal_area_growth )
      if(associated(cgrid%total_basal_area_mort   ))                                       &
                                                 deallocate(cgrid%total_basal_area_mort   )
      if(associated(cgrid%total_basal_area_recruit))                                       &
                                                 deallocate(cgrid%total_basal_area_recruit)
      if(associated(cgrid%nsites                )) deallocate(cgrid%nsites                )
      if(associated(cgrid%sitenums              )) deallocate(cgrid%sitenums              )
      if(associated(cgrid%load_adjacency        )) deallocate(cgrid%load_adjacency        )
      if(associated(cgrid%metinput              )) deallocate(cgrid%metinput              )
      if(associated(cgrid%met                   )) deallocate(cgrid%met                   )
      if(associated(cgrid%lapse                 )) deallocate(cgrid%lapse                 )
      if(associated(cgrid%cosz                  )) deallocate(cgrid%cosz                  )
      if(associated(cgrid%cbudget_initialstorage)) deallocate(cgrid%cbudget_initialstorage)
      if(associated(cgrid%cbudget_nep           )) deallocate(cgrid%cbudget_nep           )
      if(associated(cgrid%nbudget_initialstorage)) deallocate(cgrid%nbudget_initialstorage)
      if(associated(cgrid%max_leaf_temp         )) deallocate(cgrid%max_leaf_temp         )
      if(associated(cgrid%min_leaf_temp         )) deallocate(cgrid%min_leaf_temp         )
      if(associated(cgrid%max_wood_temp         )) deallocate(cgrid%max_wood_temp         )
      if(associated(cgrid%min_wood_temp         )) deallocate(cgrid%min_wood_temp         )
      if(associated(cgrid%max_soil_temp         )) deallocate(cgrid%max_soil_temp         )
      if(associated(cgrid%min_soil_temp         )) deallocate(cgrid%min_soil_temp         )
      if(associated(cgrid%nplant                )) deallocate(cgrid%nplant                )
      if(associated(cgrid%agb                   )) deallocate(cgrid%agb                   )
      if(associated(cgrid%lai                   )) deallocate(cgrid%lai                   )
      if(associated(cgrid%wai                   )) deallocate(cgrid%wai                   )
      if(associated(cgrid%basal_area            )) deallocate(cgrid%basal_area            )
      if(associated(cgrid%bdead                 )) deallocate(cgrid%bdead                 )
      if(associated(cgrid%balive                )) deallocate(cgrid%balive                )
      if(associated(cgrid%bleaf                 )) deallocate(cgrid%bleaf                 )
      if(associated(cgrid%broot                 )) deallocate(cgrid%broot                 )
      if(associated(cgrid%bsapwooda             )) deallocate(cgrid%bsapwooda             )
      if(associated(cgrid%bsapwoodb             )) deallocate(cgrid%bsapwoodb             )
      if(associated(cgrid%bseeds                )) deallocate(cgrid%bseeds                )
      if(associated(cgrid%bstorage              )) deallocate(cgrid%bstorage              )
      if(associated(cgrid%bdead_n               )) deallocate(cgrid%bdead_n               )
      if(associated(cgrid%balive_n              )) deallocate(cgrid%balive_n              )
      if(associated(cgrid%bleaf_n               )) deallocate(cgrid%bleaf_n               )
      if(associated(cgrid%broot_n               )) deallocate(cgrid%broot_n               )
      if(associated(cgrid%bsapwooda_n           )) deallocate(cgrid%bsapwooda_n           )
      if(associated(cgrid%bsapwoodb_n           )) deallocate(cgrid%bsapwoodb_n           )
      if(associated(cgrid%bseeds_n              )) deallocate(cgrid%bseeds_n              )
      if(associated(cgrid%bstorage_n            )) deallocate(cgrid%bstorage_n            )
      if(associated(cgrid%leaf_maintenance      )) deallocate(cgrid%leaf_maintenance      )
      if(associated(cgrid%root_maintenance      )) deallocate(cgrid%root_maintenance      )
      if(associated(cgrid%leaf_drop             )) deallocate(cgrid%leaf_drop             )
      if(associated(cgrid%fast_soil_c           )) deallocate(cgrid%fast_soil_c           )
      if(associated(cgrid%slow_soil_c           )) deallocate(cgrid%slow_soil_c           )
      if(associated(cgrid%struct_soil_c         )) deallocate(cgrid%struct_soil_c         )
      if(associated(cgrid%struct_soil_l         )) deallocate(cgrid%struct_soil_l         )
      if(associated(cgrid%cwd_c                 )) deallocate(cgrid%cwd_c                 )
      if(associated(cgrid%fast_soil_n           )) deallocate(cgrid%fast_soil_n           )
      if(associated(cgrid%mineral_soil_n        )) deallocate(cgrid%mineral_soil_n        )
      if(associated(cgrid%cwd_n                 )) deallocate(cgrid%cwd_n                 )
      if(associated(cgrid%Cleaf_grow            )) deallocate(cgrid%Cleaf_grow            )
      if(associated(cgrid%Croot_grow            )) deallocate(cgrid%Croot_grow            )
      if(associated(cgrid%Cdead_grow            )) deallocate(cgrid%Cdead_grow            )
      if(associated(cgrid%Cstore_grow           )) deallocate(cgrid%Cstore_grow           )
      if(associated(cgrid%Cleaf_litter_flux     )) deallocate(cgrid%Cleaf_litter_flux     )
      if(associated(cgrid%Croot_litter_flux     )) deallocate(cgrid%Croot_litter_flux     )
      if(associated(cgrid%Ccwd_flux             )) deallocate(cgrid%Ccwd_flux             )
      if(associated(cgrid%Nleaf_grow            )) deallocate(cgrid%Nleaf_grow            )
      if(associated(cgrid%Ndead_grow            )) deallocate(cgrid%Ndead_grow            )
      if(associated(cgrid%Nroot_grow            )) deallocate(cgrid%Nroot_grow            )
      if(associated(cgrid%Nstore_grow           )) deallocate(cgrid%Nstore_grow           )
      if(associated(cgrid%Nleaf_litter_flux     )) deallocate(cgrid%Nleaf_litter_flux     )
      if(associated(cgrid%Nroot_litter_flux     )) deallocate(cgrid%Nroot_litter_flux     )
      if(associated(cgrid%Ncwd_flux             )) deallocate(cgrid%Ncwd_flux             )
      if(associated(cgrid%Nbiomass_uptake       )) deallocate(cgrid%Nbiomass_uptake       )
      if(associated(cgrid%Ngross_min            )) deallocate(cgrid%Ngross_min            )
      if(associated(cgrid%Nnet_min              )) deallocate(cgrid%Nnet_min              )
      if(associated(cgrid%avg_lai_ebalvars      )) deallocate(cgrid%avg_lai_ebalvars      )
      if(associated(cgrid%fmean_gpp             )) deallocate(cgrid%fmean_gpp             )
      if(associated(cgrid%fmean_npp             )) deallocate(cgrid%fmean_npp             )
      if(associated(cgrid%fmean_leaf_resp       )) deallocate(cgrid%fmean_leaf_resp       )
      if(associated(cgrid%fmean_root_resp       )) deallocate(cgrid%fmean_root_resp       )
      if(associated(cgrid%fmean_leaf_growth_resp)) deallocate(cgrid%fmean_leaf_growth_resp)
      if(associated(cgrid%fmean_root_growth_resp)) deallocate(cgrid%fmean_root_growth_resp)
      if(associated(cgrid%fmean_sapa_growth_resp)) deallocate(cgrid%fmean_sapa_growth_resp)
      if(associated(cgrid%fmean_sapb_growth_resp)) deallocate(cgrid%fmean_sapb_growth_resp)
      if(associated(cgrid%fmean_leaf_storage_resp)) deallocate(cgrid%fmean_leaf_storage_resp)
      if(associated(cgrid%fmean_root_storage_resp)) deallocate(cgrid%fmean_root_storage_resp)
      if(associated(cgrid%fmean_sapa_storage_resp)) deallocate(cgrid%fmean_sapa_storage_resp)
      if(associated(cgrid%fmean_sapb_storage_resp)) deallocate(cgrid%fmean_sapb_storage_resp)
      if(associated(cgrid%fmean_plresp          )) deallocate(cgrid%fmean_plresp          )
      if(associated(cgrid%fmean_leaf_energy     )) deallocate(cgrid%fmean_leaf_energy     )
      if(associated(cgrid%fmean_leaf_water      )) deallocate(cgrid%fmean_leaf_water      )
      if(associated(cgrid%fmean_leaf_hcap       )) deallocate(cgrid%fmean_leaf_hcap       )
      if(associated(cgrid%fmean_leaf_vpdef      )) deallocate(cgrid%fmean_leaf_vpdef      )
      if(associated(cgrid%fmean_leaf_temp       )) deallocate(cgrid%fmean_leaf_temp       )
      if(associated(cgrid%fmean_leaf_fliq       )) deallocate(cgrid%fmean_leaf_fliq       )
      if(associated(cgrid%fmean_leaf_gsw        )) deallocate(cgrid%fmean_leaf_gsw        )
      if(associated(cgrid%fmean_leaf_gbw        )) deallocate(cgrid%fmean_leaf_gbw        )
      if(associated(cgrid%fmean_wood_energy     )) deallocate(cgrid%fmean_wood_energy     )
      if(associated(cgrid%fmean_wood_water      )) deallocate(cgrid%fmean_wood_water      )
      if(associated(cgrid%fmean_wood_hcap       )) deallocate(cgrid%fmean_wood_hcap       )
      if(associated(cgrid%fmean_wood_temp       )) deallocate(cgrid%fmean_wood_temp       )
      if(associated(cgrid%fmean_wood_fliq       )) deallocate(cgrid%fmean_wood_fliq       )
      if(associated(cgrid%fmean_wood_gbw        )) deallocate(cgrid%fmean_wood_gbw        )
      if(associated(cgrid%fmean_fs_open         )) deallocate(cgrid%fmean_fs_open         )
      if(associated(cgrid%fmean_fsw             )) deallocate(cgrid%fmean_fsw             )
      if(associated(cgrid%fmean_fsn             )) deallocate(cgrid%fmean_fsn             )
      if(associated(cgrid%fmean_a_open          )) deallocate(cgrid%fmean_a_open          )
      if(associated(cgrid%fmean_a_closed        )) deallocate(cgrid%fmean_a_closed        )
      if(associated(cgrid%fmean_a_net           )) deallocate(cgrid%fmean_a_net           )
      if(associated(cgrid%fmean_A_light         )) deallocate(cgrid%fmean_A_light         )
      if(associated(cgrid%fmean_A_rubp          )) deallocate(cgrid%fmean_A_rubp          )
      if(associated(cgrid%fmean_A_co2           )) deallocate(cgrid%fmean_A_co2           )
      if(associated(cgrid%fmean_psi_open        )) deallocate(cgrid%fmean_psi_open        )
      if(associated(cgrid%fmean_psi_closed      )) deallocate(cgrid%fmean_psi_closed      )
      if(associated(cgrid%fmean_water_supply    )) deallocate(cgrid%fmean_water_supply    )
      if(associated(cgrid%fmean_par_l           )) deallocate(cgrid%fmean_par_l           )
      if(associated(cgrid%fmean_par_l_beam      )) deallocate(cgrid%fmean_par_l_beam      )
      if(associated(cgrid%fmean_par_l_diff      )) deallocate(cgrid%fmean_par_l_diff      )
      if(associated(cgrid%fmean_rshort_l        )) deallocate(cgrid%fmean_rshort_l        )
      if(associated(cgrid%fmean_rlong_l         )) deallocate(cgrid%fmean_rlong_l         )
      if(associated(cgrid%fmean_sensible_lc     )) deallocate(cgrid%fmean_sensible_lc     )
      if(associated(cgrid%fmean_vapor_lc        )) deallocate(cgrid%fmean_vapor_lc        )
      if(associated(cgrid%fmean_transp          )) deallocate(cgrid%fmean_transp          )
      if(associated(cgrid%fmean_intercepted_al  )) deallocate(cgrid%fmean_intercepted_al  )
      if(associated(cgrid%fmean_wshed_lg        )) deallocate(cgrid%fmean_wshed_lg        )
      if(associated(cgrid%fmean_rshort_w        )) deallocate(cgrid%fmean_rshort_w        )
      if(associated(cgrid%fmean_rlong_w         )) deallocate(cgrid%fmean_rlong_w         )
      if(associated(cgrid%fmean_sensible_wc     )) deallocate(cgrid%fmean_sensible_wc     )
      if(associated(cgrid%fmean_vapor_wc        )) deallocate(cgrid%fmean_vapor_wc        )
      if(associated(cgrid%fmean_intercepted_aw  )) deallocate(cgrid%fmean_intercepted_aw  )
      if(associated(cgrid%fmean_wshed_wg        )) deallocate(cgrid%fmean_wshed_wg        )
      if(associated(cgrid%fmean_rh              )) deallocate(cgrid%fmean_rh              )
      if(associated(cgrid%fmean_cwd_rh          )) deallocate(cgrid%fmean_cwd_rh          )
      if(associated(cgrid%fmean_nep             )) deallocate(cgrid%fmean_nep             )
      if(associated(cgrid%fmean_rk4step         )) deallocate(cgrid%fmean_rk4step         )
      if(associated(cgrid%fmean_available_water )) deallocate(cgrid%fmean_available_water )
      if(associated(cgrid%fmean_can_theiv       )) deallocate(cgrid%fmean_can_theiv       )
      if(associated(cgrid%fmean_can_theta       )) deallocate(cgrid%fmean_can_theta       )
      if(associated(cgrid%fmean_can_vpdef       )) deallocate(cgrid%fmean_can_vpdef       )
      if(associated(cgrid%fmean_can_temp        )) deallocate(cgrid%fmean_can_temp        )
      if(associated(cgrid%fmean_can_shv         )) deallocate(cgrid%fmean_can_shv         )
      if(associated(cgrid%fmean_can_co2         )) deallocate(cgrid%fmean_can_co2         )
      if(associated(cgrid%fmean_can_rhos        )) deallocate(cgrid%fmean_can_rhos        )
      if(associated(cgrid%fmean_can_prss        )) deallocate(cgrid%fmean_can_prss        )
      if(associated(cgrid%fmean_gnd_temp        )) deallocate(cgrid%fmean_gnd_temp        )
      if(associated(cgrid%fmean_gnd_shv         )) deallocate(cgrid%fmean_gnd_shv         )
      if(associated(cgrid%fmean_can_ggnd        )) deallocate(cgrid%fmean_can_ggnd        )
      if(associated(cgrid%fmean_sfcw_depth      )) deallocate(cgrid%fmean_sfcw_depth      )
      if(associated(cgrid%fmean_sfcw_energy     )) deallocate(cgrid%fmean_sfcw_energy     )
      if(associated(cgrid%fmean_sfcw_mass       )) deallocate(cgrid%fmean_sfcw_mass       )
      if(associated(cgrid%fmean_sfcw_temp       )) deallocate(cgrid%fmean_sfcw_temp       )
      if(associated(cgrid%fmean_sfcw_fliq       )) deallocate(cgrid%fmean_sfcw_fliq       )
      if(associated(cgrid%fmean_soil_energy     )) deallocate(cgrid%fmean_soil_energy     )
      if(associated(cgrid%fmean_soil_mstpot     )) deallocate(cgrid%fmean_soil_mstpot     )
      if(associated(cgrid%fmean_soil_water      )) deallocate(cgrid%fmean_soil_water      )
      if(associated(cgrid%fmean_soil_temp       )) deallocate(cgrid%fmean_soil_temp       )
      if(associated(cgrid%fmean_soil_fliq       )) deallocate(cgrid%fmean_soil_fliq       )
      if(associated(cgrid%fmean_rshort_gnd      )) deallocate(cgrid%fmean_rshort_gnd      )
      if(associated(cgrid%fmean_par_gnd         )) deallocate(cgrid%fmean_par_gnd         )
      if(associated(cgrid%fmean_rlong_gnd       )) deallocate(cgrid%fmean_rlong_gnd       )
      if(associated(cgrid%fmean_rlongup         )) deallocate(cgrid%fmean_rlongup         )
      if(associated(cgrid%fmean_parup           )) deallocate(cgrid%fmean_parup           )
      if(associated(cgrid%fmean_nirup           )) deallocate(cgrid%fmean_nirup           )
      if(associated(cgrid%fmean_rshortup        )) deallocate(cgrid%fmean_rshortup        )
      if(associated(cgrid%fmean_rnet            )) deallocate(cgrid%fmean_rnet            )
      if(associated(cgrid%fmean_albedo          )) deallocate(cgrid%fmean_albedo          )
      if(associated(cgrid%fmean_albedo_par      )) deallocate(cgrid%fmean_albedo_par      )
      if(associated(cgrid%fmean_albedo_nir      )) deallocate(cgrid%fmean_albedo_nir      )
      if(associated(cgrid%fmean_rlong_albedo    )) deallocate(cgrid%fmean_rlong_albedo    )
      if(associated(cgrid%fmean_ustar           )) deallocate(cgrid%fmean_ustar           )
      if(associated(cgrid%fmean_tstar           )) deallocate(cgrid%fmean_tstar           )
      if(associated(cgrid%fmean_qstar           )) deallocate(cgrid%fmean_qstar           )
      if(associated(cgrid%fmean_cstar           )) deallocate(cgrid%fmean_cstar           )
      if(associated(cgrid%fmean_carbon_ac       )) deallocate(cgrid%fmean_carbon_ac       )
      if(associated(cgrid%fmean_carbon_st       )) deallocate(cgrid%fmean_carbon_st       )
      if(associated(cgrid%fmean_vapor_gc        )) deallocate(cgrid%fmean_vapor_gc        )
      if(associated(cgrid%fmean_vapor_ac        )) deallocate(cgrid%fmean_vapor_ac        )
      if(associated(cgrid%fmean_smoist_gg       )) deallocate(cgrid%fmean_smoist_gg       )
      if(associated(cgrid%fmean_throughfall     )) deallocate(cgrid%fmean_throughfall     )
      if(associated(cgrid%fmean_transloss       )) deallocate(cgrid%fmean_transloss       )
      if(associated(cgrid%fmean_runoff          )) deallocate(cgrid%fmean_runoff          )
      if(associated(cgrid%fmean_drainage        )) deallocate(cgrid%fmean_drainage        )
      if(associated(cgrid%fmean_sensible_gc     )) deallocate(cgrid%fmean_sensible_gc     )
      if(associated(cgrid%fmean_sensible_ac     )) deallocate(cgrid%fmean_sensible_ac     )
      if(associated(cgrid%fmean_sensible_gg     )) deallocate(cgrid%fmean_sensible_gg     )
      if(associated(cgrid%fmean_qthroughfall    )) deallocate(cgrid%fmean_qthroughfall    )
      if(associated(cgrid%fmean_qrunoff         )) deallocate(cgrid%fmean_qrunoff         )
      if(associated(cgrid%fmean_qdrainage       )) deallocate(cgrid%fmean_qdrainage       )
      if(associated(cgrid%fmean_atm_theiv       )) deallocate(cgrid%fmean_atm_theiv       )
      if(associated(cgrid%fmean_atm_theta       )) deallocate(cgrid%fmean_atm_theta       )
      if(associated(cgrid%fmean_atm_temp        )) deallocate(cgrid%fmean_atm_temp        )
      if(associated(cgrid%fmean_atm_vpdef       )) deallocate(cgrid%fmean_atm_vpdef       )
      if(associated(cgrid%fmean_atm_shv         )) deallocate(cgrid%fmean_atm_shv         )
      if(associated(cgrid%fmean_atm_rshort      )) deallocate(cgrid%fmean_atm_rshort      )
      if(associated(cgrid%fmean_atm_rshort_diff )) deallocate(cgrid%fmean_atm_rshort_diff )
      if(associated(cgrid%fmean_atm_par         )) deallocate(cgrid%fmean_atm_par         )
      if(associated(cgrid%fmean_atm_par_diff    )) deallocate(cgrid%fmean_atm_par_diff    )
      if(associated(cgrid%fmean_atm_rlong       )) deallocate(cgrid%fmean_atm_rlong       )
      if(associated(cgrid%fmean_atm_vels        )) deallocate(cgrid%fmean_atm_vels        )
      if(associated(cgrid%fmean_atm_rhos        )) deallocate(cgrid%fmean_atm_rhos        )
      if(associated(cgrid%fmean_atm_prss        )) deallocate(cgrid%fmean_atm_prss        )
      if(associated(cgrid%fmean_atm_co2         )) deallocate(cgrid%fmean_atm_co2         )
      if(associated(cgrid%fmean_pcpg            )) deallocate(cgrid%fmean_pcpg            )
      if(associated(cgrid%fmean_qpcpg           )) deallocate(cgrid%fmean_qpcpg           )
      if(associated(cgrid%fmean_dpcpg           )) deallocate(cgrid%fmean_dpcpg           )
      if(associated(cgrid%fmean_soil_wetness    )) deallocate(cgrid%fmean_soil_wetness    )
      if(associated(cgrid%fmean_skin_temp       )) deallocate(cgrid%fmean_skin_temp       )
      if(associated(cgrid%fmean_lai             )) deallocate(cgrid%fmean_lai             )
      if(associated(cgrid%fmean_bdead           )) deallocate(cgrid%fmean_bdead           )
      if(associated(cgrid%dmean_nppleaf         )) deallocate(cgrid%dmean_nppleaf         )
      if(associated(cgrid%dmean_nppfroot        )) deallocate(cgrid%dmean_nppfroot        )
      if(associated(cgrid%dmean_nppsapwood      )) deallocate(cgrid%dmean_nppsapwood      )
      if(associated(cgrid%dmean_nppcroot        )) deallocate(cgrid%dmean_nppcroot        )
      if(associated(cgrid%dmean_nppseeds        )) deallocate(cgrid%dmean_nppseeds        )
      if(associated(cgrid%dmean_nppwood         )) deallocate(cgrid%dmean_nppwood         )
      if(associated(cgrid%dmean_nppdaily        )) deallocate(cgrid%dmean_nppdaily        )
      if(associated(cgrid%dmean_A_decomp        )) deallocate(cgrid%dmean_A_decomp        )
      if(associated(cgrid%dmean_Af_decomp       )) deallocate(cgrid%dmean_Af_decomp       )
      if(associated(cgrid%dmean_co2_residual    )) deallocate(cgrid%dmean_co2_residual    )
      if(associated(cgrid%dmean_energy_residual )) deallocate(cgrid%dmean_energy_residual )
      if(associated(cgrid%dmean_water_residual  )) deallocate(cgrid%dmean_water_residual  )
      if(associated(cgrid%dmean_gpp             )) deallocate(cgrid%dmean_gpp             )
      if(associated(cgrid%dmean_npp             )) deallocate(cgrid%dmean_npp             )
      if(associated(cgrid%dmean_leaf_resp       )) deallocate(cgrid%dmean_leaf_resp       )
      if(associated(cgrid%dmean_root_resp       )) deallocate(cgrid%dmean_root_resp       )
      if(associated(cgrid%dmean_leaf_growth_resp)) deallocate(cgrid%dmean_leaf_growth_resp)
      if(associated(cgrid%dmean_root_growth_resp)) deallocate(cgrid%dmean_root_growth_resp)
      if(associated(cgrid%dmean_sapa_growth_resp)) deallocate(cgrid%dmean_sapa_growth_resp)
      if(associated(cgrid%dmean_sapb_growth_resp)) deallocate(cgrid%dmean_sapb_growth_resp)
      if(associated(cgrid%dmean_leaf_storage_resp)) deallocate(cgrid%dmean_leaf_storage_resp)
      if(associated(cgrid%dmean_root_storage_resp)) deallocate(cgrid%dmean_root_storage_resp)
      if(associated(cgrid%dmean_sapa_storage_resp)) deallocate(cgrid%dmean_sapa_storage_resp)
      if(associated(cgrid%dmean_sapb_storage_resp)) deallocate(cgrid%dmean_sapb_storage_resp)
      if(associated(cgrid%dmean_plresp          )) deallocate(cgrid%dmean_plresp          )
      if(associated(cgrid%dmean_leaf_energy     )) deallocate(cgrid%dmean_leaf_energy     )
      if(associated(cgrid%dmean_leaf_water      )) deallocate(cgrid%dmean_leaf_water      )
      if(associated(cgrid%dmean_leaf_hcap       )) deallocate(cgrid%dmean_leaf_hcap       )
      if(associated(cgrid%dmean_leaf_vpdef      )) deallocate(cgrid%dmean_leaf_vpdef      )
      if(associated(cgrid%dmean_leaf_temp       )) deallocate(cgrid%dmean_leaf_temp       )
      if(associated(cgrid%dmean_leaf_fliq       )) deallocate(cgrid%dmean_leaf_fliq       )
      if(associated(cgrid%dmean_leaf_gsw        )) deallocate(cgrid%dmean_leaf_gsw        )
      if(associated(cgrid%dmean_leaf_gbw        )) deallocate(cgrid%dmean_leaf_gbw        )
      if(associated(cgrid%dmean_wood_energy     )) deallocate(cgrid%dmean_wood_energy     )
      if(associated(cgrid%dmean_wood_water      )) deallocate(cgrid%dmean_wood_water      )
      if(associated(cgrid%dmean_wood_hcap       )) deallocate(cgrid%dmean_wood_hcap       )
      if(associated(cgrid%dmean_wood_temp       )) deallocate(cgrid%dmean_wood_temp       )
      if(associated(cgrid%dmean_wood_fliq       )) deallocate(cgrid%dmean_wood_fliq       )
      if(associated(cgrid%dmean_wood_gbw        )) deallocate(cgrid%dmean_wood_gbw        )
      if(associated(cgrid%dmean_fs_open         )) deallocate(cgrid%dmean_fs_open         )
      if(associated(cgrid%dmean_fsw             )) deallocate(cgrid%dmean_fsw             )
      if(associated(cgrid%dmean_fsn             )) deallocate(cgrid%dmean_fsn             )
      if(associated(cgrid%dmean_a_open          )) deallocate(cgrid%dmean_a_open          )
      if(associated(cgrid%dmean_a_closed        )) deallocate(cgrid%dmean_a_closed        )
      if(associated(cgrid%dmean_a_net           )) deallocate(cgrid%dmean_a_net           )
      if(associated(cgrid%dmean_A_light         )) deallocate(cgrid%dmean_A_light         )
      if(associated(cgrid%dmean_A_rubp          )) deallocate(cgrid%dmean_A_rubp          )
      if(associated(cgrid%dmean_A_co2           )) deallocate(cgrid%dmean_A_co2           )
      if(associated(cgrid%dmean_psi_open        )) deallocate(cgrid%dmean_psi_open        )
      if(associated(cgrid%dmean_psi_closed      )) deallocate(cgrid%dmean_psi_closed      )
      if(associated(cgrid%dmean_water_supply    )) deallocate(cgrid%dmean_water_supply    )
      if(associated(cgrid%dmean_par_l           )) deallocate(cgrid%dmean_par_l           )
      if(associated(cgrid%dmean_par_l_beam      )) deallocate(cgrid%dmean_par_l_beam      )
      if(associated(cgrid%dmean_par_l_diff      )) deallocate(cgrid%dmean_par_l_diff      )
      if(associated(cgrid%dmean_rshort_l        )) deallocate(cgrid%dmean_rshort_l        )
      if(associated(cgrid%dmean_rlong_l         )) deallocate(cgrid%dmean_rlong_l         )
      if(associated(cgrid%dmean_sensible_lc     )) deallocate(cgrid%dmean_sensible_lc     )
      if(associated(cgrid%dmean_vapor_lc        )) deallocate(cgrid%dmean_vapor_lc        )
      if(associated(cgrid%dmean_transp          )) deallocate(cgrid%dmean_transp          )
      if(associated(cgrid%dmean_intercepted_al  )) deallocate(cgrid%dmean_intercepted_al  )
      if(associated(cgrid%dmean_wshed_lg        )) deallocate(cgrid%dmean_wshed_lg        )
      if(associated(cgrid%dmean_rshort_w        )) deallocate(cgrid%dmean_rshort_w        )
      if(associated(cgrid%dmean_rlong_w         )) deallocate(cgrid%dmean_rlong_w         )
      if(associated(cgrid%dmean_sensible_wc     )) deallocate(cgrid%dmean_sensible_wc     )
      if(associated(cgrid%dmean_vapor_wc        )) deallocate(cgrid%dmean_vapor_wc        )
      if(associated(cgrid%dmean_intercepted_aw  )) deallocate(cgrid%dmean_intercepted_aw  )
      if(associated(cgrid%dmean_wshed_wg        )) deallocate(cgrid%dmean_wshed_wg        )
      if(associated(cgrid%dmean_rh              )) deallocate(cgrid%dmean_rh              )
      if(associated(cgrid%dmean_cwd_rh          )) deallocate(cgrid%dmean_cwd_rh          )
      if(associated(cgrid%dmean_nep             )) deallocate(cgrid%dmean_nep             )
      if(associated(cgrid%dmean_rk4step         )) deallocate(cgrid%dmean_rk4step         )
      if(associated(cgrid%dmean_available_water )) deallocate(cgrid%dmean_available_water )
      if(associated(cgrid%dmean_can_theiv       )) deallocate(cgrid%dmean_can_theiv       )
      if(associated(cgrid%dmean_can_theta       )) deallocate(cgrid%dmean_can_theta       )
      if(associated(cgrid%dmean_can_vpdef       )) deallocate(cgrid%dmean_can_vpdef       )
      if(associated(cgrid%dmean_can_temp        )) deallocate(cgrid%dmean_can_temp        )
      if(associated(cgrid%dmean_can_shv         )) deallocate(cgrid%dmean_can_shv         )
      if(associated(cgrid%dmean_can_co2         )) deallocate(cgrid%dmean_can_co2         )
      if(associated(cgrid%dmean_can_rhos        )) deallocate(cgrid%dmean_can_rhos        )
      if(associated(cgrid%dmean_can_prss        )) deallocate(cgrid%dmean_can_prss        )
      if(associated(cgrid%dmean_gnd_temp        )) deallocate(cgrid%dmean_gnd_temp        )
      if(associated(cgrid%dmean_gnd_shv         )) deallocate(cgrid%dmean_gnd_shv         )
      if(associated(cgrid%dmean_can_ggnd        )) deallocate(cgrid%dmean_can_ggnd        )
      if(associated(cgrid%dmean_sfcw_depth      )) deallocate(cgrid%dmean_sfcw_depth      )
      if(associated(cgrid%dmean_sfcw_energy     )) deallocate(cgrid%dmean_sfcw_energy     )
      if(associated(cgrid%dmean_sfcw_mass       )) deallocate(cgrid%dmean_sfcw_mass       )
      if(associated(cgrid%dmean_sfcw_temp       )) deallocate(cgrid%dmean_sfcw_temp       )
      if(associated(cgrid%dmean_sfcw_fliq       )) deallocate(cgrid%dmean_sfcw_fliq       )
      if(associated(cgrid%dmean_soil_energy     )) deallocate(cgrid%dmean_soil_energy     )
      if(associated(cgrid%dmean_soil_mstpot     )) deallocate(cgrid%dmean_soil_mstpot     )
      if(associated(cgrid%dmean_soil_water      )) deallocate(cgrid%dmean_soil_water      )
      if(associated(cgrid%dmean_soil_temp       )) deallocate(cgrid%dmean_soil_temp       )
      if(associated(cgrid%dmean_soil_fliq       )) deallocate(cgrid%dmean_soil_fliq       )
      if(associated(cgrid%dmean_rshort_gnd      )) deallocate(cgrid%dmean_rshort_gnd      )
      if(associated(cgrid%dmean_par_gnd         )) deallocate(cgrid%dmean_par_gnd         )
      if(associated(cgrid%dmean_rlong_gnd       )) deallocate(cgrid%dmean_rlong_gnd       )
      if(associated(cgrid%dmean_rlongup         )) deallocate(cgrid%dmean_rlongup         )
      if(associated(cgrid%dmean_parup           )) deallocate(cgrid%dmean_parup           )
      if(associated(cgrid%dmean_nirup           )) deallocate(cgrid%dmean_nirup           )
      if(associated(cgrid%dmean_rshortup        )) deallocate(cgrid%dmean_rshortup        )
      if(associated(cgrid%dmean_rnet            )) deallocate(cgrid%dmean_rnet            )
      if(associated(cgrid%dmean_albedo          )) deallocate(cgrid%dmean_albedo          )
      if(associated(cgrid%dmean_albedo_par      )) deallocate(cgrid%dmean_albedo_par      )
      if(associated(cgrid%dmean_albedo_nir      )) deallocate(cgrid%dmean_albedo_nir      )
      if(associated(cgrid%dmean_rlong_albedo    )) deallocate(cgrid%dmean_rlong_albedo    )
      if(associated(cgrid%dmean_ustar           )) deallocate(cgrid%dmean_ustar           )
      if(associated(cgrid%dmean_tstar           )) deallocate(cgrid%dmean_tstar           )
      if(associated(cgrid%dmean_qstar           )) deallocate(cgrid%dmean_qstar           )
      if(associated(cgrid%dmean_cstar           )) deallocate(cgrid%dmean_cstar           )
      if(associated(cgrid%dmean_carbon_ac       )) deallocate(cgrid%dmean_carbon_ac       )
      if(associated(cgrid%dmean_carbon_st       )) deallocate(cgrid%dmean_carbon_st       )
      if(associated(cgrid%dmean_vapor_gc        )) deallocate(cgrid%dmean_vapor_gc        )
      if(associated(cgrid%dmean_vapor_ac        )) deallocate(cgrid%dmean_vapor_ac        )
      if(associated(cgrid%dmean_smoist_gg       )) deallocate(cgrid%dmean_smoist_gg       )
      if(associated(cgrid%dmean_throughfall     )) deallocate(cgrid%dmean_throughfall     )
      if(associated(cgrid%dmean_transloss       )) deallocate(cgrid%dmean_transloss       )
      if(associated(cgrid%dmean_runoff          )) deallocate(cgrid%dmean_runoff          )
      if(associated(cgrid%dmean_drainage        )) deallocate(cgrid%dmean_drainage        )
      if(associated(cgrid%dmean_sensible_gc     )) deallocate(cgrid%dmean_sensible_gc     )
      if(associated(cgrid%dmean_sensible_ac     )) deallocate(cgrid%dmean_sensible_ac     )
      if(associated(cgrid%dmean_sensible_gg     )) deallocate(cgrid%dmean_sensible_gg     )
      if(associated(cgrid%dmean_qthroughfall    )) deallocate(cgrid%dmean_qthroughfall    )
      if(associated(cgrid%dmean_qrunoff         )) deallocate(cgrid%dmean_qrunoff         )
      if(associated(cgrid%dmean_qdrainage       )) deallocate(cgrid%dmean_qdrainage       )
      if(associated(cgrid%dmean_atm_theiv       )) deallocate(cgrid%dmean_atm_theiv       )
      if(associated(cgrid%dmean_atm_theta       )) deallocate(cgrid%dmean_atm_theta       )
      if(associated(cgrid%dmean_atm_temp        )) deallocate(cgrid%dmean_atm_temp        )
      if(associated(cgrid%dmean_atm_vpdef       )) deallocate(cgrid%dmean_atm_vpdef       )
      if(associated(cgrid%dmean_atm_shv         )) deallocate(cgrid%dmean_atm_shv         )
      if(associated(cgrid%dmean_atm_rshort      )) deallocate(cgrid%dmean_atm_rshort      )
      if(associated(cgrid%dmean_atm_rshort_diff )) deallocate(cgrid%dmean_atm_rshort_diff )
      if(associated(cgrid%dmean_atm_par         )) deallocate(cgrid%dmean_atm_par         )
      if(associated(cgrid%dmean_atm_par_diff    )) deallocate(cgrid%dmean_atm_par_diff    )
      if(associated(cgrid%dmean_atm_rlong       )) deallocate(cgrid%dmean_atm_rlong       )
      if(associated(cgrid%dmean_atm_vels        )) deallocate(cgrid%dmean_atm_vels        )
      if(associated(cgrid%dmean_atm_rhos        )) deallocate(cgrid%dmean_atm_rhos        )
      if(associated(cgrid%dmean_atm_prss        )) deallocate(cgrid%dmean_atm_prss        )
      if(associated(cgrid%dmean_atm_co2         )) deallocate(cgrid%dmean_atm_co2         )
      if(associated(cgrid%dmean_pcpg            )) deallocate(cgrid%dmean_pcpg            )
      if(associated(cgrid%dmean_qpcpg           )) deallocate(cgrid%dmean_qpcpg           )
      if(associated(cgrid%dmean_dpcpg           )) deallocate(cgrid%dmean_dpcpg           )
      if(associated(cgrid%mmean_lai             )) deallocate(cgrid%mmean_lai             )
      if(associated(cgrid%mmean_bleaf           )) deallocate(cgrid%mmean_bleaf           )
      if(associated(cgrid%mmean_broot           )) deallocate(cgrid%mmean_broot           )
      if(associated(cgrid%mmean_bstorage        )) deallocate(cgrid%mmean_bstorage        )
      if(associated(cgrid%mmean_bleaf_n         )) deallocate(cgrid%mmean_bleaf_n         )
      if(associated(cgrid%mmean_broot_n         )) deallocate(cgrid%mmean_broot_n         )
      if(associated(cgrid%mmean_bstorage_n      )) deallocate(cgrid%mmean_bstorage_n      )
      if(associated(cgrid%mmean_leaf_maintenance)) deallocate(cgrid%mmean_leaf_maintenance)
      if(associated(cgrid%mmean_root_maintenance)) deallocate(cgrid%mmean_root_maintenance)
      if(associated(cgrid%mmean_leaf_drop       )) deallocate(cgrid%mmean_leaf_drop       )
      if(associated(cgrid%mmean_fast_soil_c     )) deallocate(cgrid%mmean_fast_soil_c     )
      if(associated(cgrid%mmean_slow_soil_c     )) deallocate(cgrid%mmean_slow_soil_c     )
      if(associated(cgrid%mmean_struct_soil_c   )) deallocate(cgrid%mmean_struct_soil_c   )
      if(associated(cgrid%mmean_struct_soil_l   )) deallocate(cgrid%mmean_struct_soil_l   )
      if(associated(cgrid%mmean_cwd_c           )) deallocate(cgrid%mmean_cwd_c           )
      if(associated(cgrid%mmean_fast_soil_n     )) deallocate(cgrid%mmean_fast_soil_n     )
      if(associated(cgrid%mmean_mineral_soil_n  )) deallocate(cgrid%mmean_mineral_soil_n  )
      if(associated(cgrid%mmean_cwd_n           )) deallocate(cgrid%mmean_cwd_n           )
      if(associated(cgrid%mmean_gpp             )) deallocate(cgrid%mmean_gpp             )
      if(associated(cgrid%mmean_npp             )) deallocate(cgrid%mmean_npp             )
      if(associated(cgrid%mmean_leaf_resp       )) deallocate(cgrid%mmean_leaf_resp       )
      if(associated(cgrid%mmean_root_resp       )) deallocate(cgrid%mmean_root_resp       )
      if(associated(cgrid%mmean_leaf_growth_resp)) deallocate(cgrid%mmean_leaf_growth_resp)
      if(associated(cgrid%mmean_root_growth_resp)) deallocate(cgrid%mmean_root_growth_resp)
      if(associated(cgrid%mmean_sapa_growth_resp)) deallocate(cgrid%mmean_sapa_growth_resp)
      if(associated(cgrid%mmean_sapb_growth_resp)) deallocate(cgrid%mmean_sapb_growth_resp)
      if(associated(cgrid%mmean_leaf_storage_resp)) deallocate(cgrid%mmean_leaf_storage_resp)
      if(associated(cgrid%mmean_root_storage_resp)) deallocate(cgrid%mmean_root_storage_resp)
      if(associated(cgrid%mmean_sapa_storage_resp)) deallocate(cgrid%mmean_sapa_storage_resp)
      if(associated(cgrid%mmean_sapb_storage_resp)) deallocate(cgrid%mmean_sapb_storage_resp)
      if(associated(cgrid%mmean_plresp          )) deallocate(cgrid%mmean_plresp          )
      if(associated(cgrid%mmean_leaf_energy     )) deallocate(cgrid%mmean_leaf_energy     )
      if(associated(cgrid%mmean_leaf_water      )) deallocate(cgrid%mmean_leaf_water      )
      if(associated(cgrid%mmean_leaf_hcap       )) deallocate(cgrid%mmean_leaf_hcap       )
      if(associated(cgrid%mmean_leaf_vpdef      )) deallocate(cgrid%mmean_leaf_vpdef      )
      if(associated(cgrid%mmean_leaf_temp       )) deallocate(cgrid%mmean_leaf_temp       )
      if(associated(cgrid%mmean_leaf_fliq       )) deallocate(cgrid%mmean_leaf_fliq       )
      if(associated(cgrid%mmean_leaf_gsw        )) deallocate(cgrid%mmean_leaf_gsw        )
      if(associated(cgrid%mmean_leaf_gbw        )) deallocate(cgrid%mmean_leaf_gbw        )
      if(associated(cgrid%mmean_wood_energy     )) deallocate(cgrid%mmean_wood_energy     )
      if(associated(cgrid%mmean_wood_water      )) deallocate(cgrid%mmean_wood_water      )
      if(associated(cgrid%mmean_wood_hcap       )) deallocate(cgrid%mmean_wood_hcap       )
      if(associated(cgrid%mmean_wood_temp       )) deallocate(cgrid%mmean_wood_temp       )
      if(associated(cgrid%mmean_wood_fliq       )) deallocate(cgrid%mmean_wood_fliq       )
      if(associated(cgrid%mmean_wood_gbw        )) deallocate(cgrid%mmean_wood_gbw        )
      if(associated(cgrid%mmean_fs_open         )) deallocate(cgrid%mmean_fs_open         )
      if(associated(cgrid%mmean_fsw             )) deallocate(cgrid%mmean_fsw             )
      if(associated(cgrid%mmean_fsn             )) deallocate(cgrid%mmean_fsn             )
      if(associated(cgrid%mmean_a_open          )) deallocate(cgrid%mmean_a_open          )
      if(associated(cgrid%mmean_a_closed        )) deallocate(cgrid%mmean_a_closed        )
      if(associated(cgrid%mmean_a_net           )) deallocate(cgrid%mmean_a_net           )
      if(associated(cgrid%mmean_A_light         )) deallocate(cgrid%mmean_A_light         )
      if(associated(cgrid%mmean_A_rubp          )) deallocate(cgrid%mmean_A_rubp          )
      if(associated(cgrid%mmean_A_co2           )) deallocate(cgrid%mmean_A_co2           )
      if(associated(cgrid%mmean_psi_open        )) deallocate(cgrid%mmean_psi_open        )
      if(associated(cgrid%mmean_psi_closed      )) deallocate(cgrid%mmean_psi_closed      )
      if(associated(cgrid%mmean_water_supply    )) deallocate(cgrid%mmean_water_supply    )
      if(associated(cgrid%mmean_par_l           )) deallocate(cgrid%mmean_par_l           )
      if(associated(cgrid%mmean_par_l_beam      )) deallocate(cgrid%mmean_par_l_beam      )
      if(associated(cgrid%mmean_par_l_diff      )) deallocate(cgrid%mmean_par_l_diff      )
      if(associated(cgrid%mmean_rshort_l        )) deallocate(cgrid%mmean_rshort_l        )
      if(associated(cgrid%mmean_rlong_l         )) deallocate(cgrid%mmean_rlong_l         )
      if(associated(cgrid%mmean_sensible_lc     )) deallocate(cgrid%mmean_sensible_lc     )
      if(associated(cgrid%mmean_vapor_lc        )) deallocate(cgrid%mmean_vapor_lc        )
      if(associated(cgrid%mmean_transp          )) deallocate(cgrid%mmean_transp          )
      if(associated(cgrid%mmean_intercepted_al  )) deallocate(cgrid%mmean_intercepted_al  )
      if(associated(cgrid%mmean_wshed_lg        )) deallocate(cgrid%mmean_wshed_lg        )
      if(associated(cgrid%mmean_rshort_w        )) deallocate(cgrid%mmean_rshort_w        )
      if(associated(cgrid%mmean_rlong_w         )) deallocate(cgrid%mmean_rlong_w         )
      if(associated(cgrid%mmean_sensible_wc     )) deallocate(cgrid%mmean_sensible_wc     )
      if(associated(cgrid%mmean_vapor_wc        )) deallocate(cgrid%mmean_vapor_wc        )
      if(associated(cgrid%mmean_intercepted_aw  )) deallocate(cgrid%mmean_intercepted_aw  )
      if(associated(cgrid%mmean_wshed_wg        )) deallocate(cgrid%mmean_wshed_wg        )
      if(associated(cgrid%mmean_rh              )) deallocate(cgrid%mmean_rh              )
      if(associated(cgrid%mmean_cwd_rh          )) deallocate(cgrid%mmean_cwd_rh          )
      if(associated(cgrid%mmean_nep             )) deallocate(cgrid%mmean_nep             )
      if(associated(cgrid%mmean_rk4step         )) deallocate(cgrid%mmean_rk4step         )
      if(associated(cgrid%mmean_available_water )) deallocate(cgrid%mmean_available_water )
      if(associated(cgrid%mmean_can_theiv       )) deallocate(cgrid%mmean_can_theiv       )
      if(associated(cgrid%mmean_can_theta       )) deallocate(cgrid%mmean_can_theta       )
      if(associated(cgrid%mmean_can_vpdef       )) deallocate(cgrid%mmean_can_vpdef       )
      if(associated(cgrid%mmean_can_temp        )) deallocate(cgrid%mmean_can_temp        )
      if(associated(cgrid%mmean_can_shv         )) deallocate(cgrid%mmean_can_shv         )
      if(associated(cgrid%mmean_can_co2         )) deallocate(cgrid%mmean_can_co2         )
      if(associated(cgrid%mmean_can_rhos        )) deallocate(cgrid%mmean_can_rhos        )
      if(associated(cgrid%mmean_can_prss        )) deallocate(cgrid%mmean_can_prss        )
      if(associated(cgrid%mmean_gnd_temp        )) deallocate(cgrid%mmean_gnd_temp        )
      if(associated(cgrid%mmean_gnd_shv         )) deallocate(cgrid%mmean_gnd_shv         )
      if(associated(cgrid%mmean_can_ggnd        )) deallocate(cgrid%mmean_can_ggnd        )
      if(associated(cgrid%mmean_sfcw_depth      )) deallocate(cgrid%mmean_sfcw_depth      )
      if(associated(cgrid%mmean_sfcw_energy     )) deallocate(cgrid%mmean_sfcw_energy     )
      if(associated(cgrid%mmean_sfcw_mass       )) deallocate(cgrid%mmean_sfcw_mass       )
      if(associated(cgrid%mmean_sfcw_temp       )) deallocate(cgrid%mmean_sfcw_temp       )
      if(associated(cgrid%mmean_sfcw_fliq       )) deallocate(cgrid%mmean_sfcw_fliq       )
      if(associated(cgrid%mmean_soil_energy     )) deallocate(cgrid%mmean_soil_energy     )
      if(associated(cgrid%mmean_soil_mstpot     )) deallocate(cgrid%mmean_soil_mstpot     )
      if(associated(cgrid%mmean_soil_water      )) deallocate(cgrid%mmean_soil_water      )
      if(associated(cgrid%mmean_soil_temp       )) deallocate(cgrid%mmean_soil_temp       )
      if(associated(cgrid%mmean_soil_fliq       )) deallocate(cgrid%mmean_soil_fliq       )
      if(associated(cgrid%mmean_rshort_gnd      )) deallocate(cgrid%mmean_rshort_gnd      )
      if(associated(cgrid%mmean_par_gnd         )) deallocate(cgrid%mmean_par_gnd         )
      if(associated(cgrid%mmean_rlong_gnd       )) deallocate(cgrid%mmean_rlong_gnd       )
      if(associated(cgrid%mmean_rlongup         )) deallocate(cgrid%mmean_rlongup         )
      if(associated(cgrid%mmean_parup           )) deallocate(cgrid%mmean_parup           )
      if(associated(cgrid%mmean_nirup           )) deallocate(cgrid%mmean_nirup           )
      if(associated(cgrid%mmean_rshortup        )) deallocate(cgrid%mmean_rshortup        )
      if(associated(cgrid%mmean_rnet            )) deallocate(cgrid%mmean_rnet            )
      if(associated(cgrid%mmean_albedo          )) deallocate(cgrid%mmean_albedo          )
      if(associated(cgrid%mmean_albedo_par      )) deallocate(cgrid%mmean_albedo_par      )
      if(associated(cgrid%mmean_albedo_nir      )) deallocate(cgrid%mmean_albedo_nir      )
      if(associated(cgrid%mmean_rlong_albedo    )) deallocate(cgrid%mmean_rlong_albedo    )
      if(associated(cgrid%mmean_ustar           )) deallocate(cgrid%mmean_ustar           )
      if(associated(cgrid%mmean_tstar           )) deallocate(cgrid%mmean_tstar           )
      if(associated(cgrid%mmean_qstar           )) deallocate(cgrid%mmean_qstar           )
      if(associated(cgrid%mmean_cstar           )) deallocate(cgrid%mmean_cstar           )
      if(associated(cgrid%mmean_carbon_ac       )) deallocate(cgrid%mmean_carbon_ac       )
      if(associated(cgrid%mmean_carbon_st       )) deallocate(cgrid%mmean_carbon_st       )
      if(associated(cgrid%mmean_vapor_gc        )) deallocate(cgrid%mmean_vapor_gc        )
      if(associated(cgrid%mmean_vapor_ac        )) deallocate(cgrid%mmean_vapor_ac        )
      if(associated(cgrid%mmean_smoist_gg       )) deallocate(cgrid%mmean_smoist_gg       )
      if(associated(cgrid%mmean_throughfall     )) deallocate(cgrid%mmean_throughfall     )
      if(associated(cgrid%mmean_transloss       )) deallocate(cgrid%mmean_transloss       )
      if(associated(cgrid%mmean_runoff          )) deallocate(cgrid%mmean_runoff          )
      if(associated(cgrid%mmean_drainage        )) deallocate(cgrid%mmean_drainage        )
      if(associated(cgrid%mmean_sensible_gc     )) deallocate(cgrid%mmean_sensible_gc     )
      if(associated(cgrid%mmean_sensible_ac     )) deallocate(cgrid%mmean_sensible_ac     )
      if(associated(cgrid%mmean_sensible_gg     )) deallocate(cgrid%mmean_sensible_gg     )
      if(associated(cgrid%mmean_qthroughfall    )) deallocate(cgrid%mmean_qthroughfall    )
      if(associated(cgrid%mmean_qrunoff         )) deallocate(cgrid%mmean_qrunoff         )
      if(associated(cgrid%mmean_qdrainage       )) deallocate(cgrid%mmean_qdrainage       )
      if(associated(cgrid%mmean_nppleaf         )) deallocate(cgrid%mmean_nppleaf         )
      if(associated(cgrid%mmean_nppfroot        )) deallocate(cgrid%mmean_nppfroot        )
      if(associated(cgrid%mmean_nppsapwood      )) deallocate(cgrid%mmean_nppsapwood      )
      if(associated(cgrid%mmean_nppcroot        )) deallocate(cgrid%mmean_nppcroot        )
      if(associated(cgrid%mmean_nppseeds        )) deallocate(cgrid%mmean_nppseeds        )
      if(associated(cgrid%mmean_nppwood         )) deallocate(cgrid%mmean_nppwood         )
      if(associated(cgrid%mmean_nppdaily        )) deallocate(cgrid%mmean_nppdaily        )
      if(associated(cgrid%mmean_A_decomp        )) deallocate(cgrid%mmean_A_decomp        )
      if(associated(cgrid%mmean_Af_decomp       )) deallocate(cgrid%mmean_Af_decomp       )
      if(associated(cgrid%mmean_co2_residual    )) deallocate(cgrid%mmean_co2_residual    )
      if(associated(cgrid%mmean_energy_residual )) deallocate(cgrid%mmean_energy_residual )
      if(associated(cgrid%mmean_water_residual  )) deallocate(cgrid%mmean_water_residual  )
      if(associated(cgrid%mmean_atm_theiv       )) deallocate(cgrid%mmean_atm_theiv       )
      if(associated(cgrid%mmean_atm_theta       )) deallocate(cgrid%mmean_atm_theta       )
      if(associated(cgrid%mmean_atm_temp        )) deallocate(cgrid%mmean_atm_temp        )
      if(associated(cgrid%mmean_atm_vpdef       )) deallocate(cgrid%mmean_atm_vpdef       )
      if(associated(cgrid%mmean_atm_shv         )) deallocate(cgrid%mmean_atm_shv         )
      if(associated(cgrid%mmean_atm_rshort      )) deallocate(cgrid%mmean_atm_rshort      )
      if(associated(cgrid%mmean_atm_rshort_diff )) deallocate(cgrid%mmean_atm_rshort_diff )
      if(associated(cgrid%mmean_atm_par         )) deallocate(cgrid%mmean_atm_par         )
      if(associated(cgrid%mmean_atm_par_diff    )) deallocate(cgrid%mmean_atm_par_diff    )
      if(associated(cgrid%mmean_atm_rlong       )) deallocate(cgrid%mmean_atm_rlong       )
      if(associated(cgrid%mmean_atm_vels        )) deallocate(cgrid%mmean_atm_vels        )
      if(associated(cgrid%mmean_atm_rhos        )) deallocate(cgrid%mmean_atm_rhos        )
      if(associated(cgrid%mmean_atm_prss        )) deallocate(cgrid%mmean_atm_prss        )
      if(associated(cgrid%mmean_atm_co2         )) deallocate(cgrid%mmean_atm_co2         )
      if(associated(cgrid%mmean_pcpg            )) deallocate(cgrid%mmean_pcpg            )
      if(associated(cgrid%mmean_qpcpg           )) deallocate(cgrid%mmean_qpcpg           )
      if(associated(cgrid%mmean_dpcpg           )) deallocate(cgrid%mmean_dpcpg           )
      if(associated(cgrid%mmsqu_gpp             )) deallocate(cgrid%mmsqu_gpp             )
      if(associated(cgrid%mmsqu_npp             )) deallocate(cgrid%mmsqu_npp             )
      if(associated(cgrid%mmsqu_plresp          )) deallocate(cgrid%mmsqu_plresp          )
      if(associated(cgrid%mmsqu_sensible_lc     )) deallocate(cgrid%mmsqu_sensible_lc     )
      if(associated(cgrid%mmsqu_vapor_lc        )) deallocate(cgrid%mmsqu_vapor_lc        )
      if(associated(cgrid%mmsqu_transp          )) deallocate(cgrid%mmsqu_transp          )
      if(associated(cgrid%mmsqu_sensible_wc     )) deallocate(cgrid%mmsqu_sensible_wc     )
      if(associated(cgrid%mmsqu_vapor_wc        )) deallocate(cgrid%mmsqu_vapor_wc        )
      if(associated(cgrid%mmsqu_rh              )) deallocate(cgrid%mmsqu_rh              )
      if(associated(cgrid%mmsqu_cwd_rh          )) deallocate(cgrid%mmsqu_cwd_rh          )
      if(associated(cgrid%mmsqu_nep             )) deallocate(cgrid%mmsqu_nep             )
      if(associated(cgrid%mmsqu_rlongup         )) deallocate(cgrid%mmsqu_rlongup         )
      if(associated(cgrid%mmsqu_parup           )) deallocate(cgrid%mmsqu_parup           )
      if(associated(cgrid%mmsqu_nirup           )) deallocate(cgrid%mmsqu_nirup           )
      if(associated(cgrid%mmsqu_rshortup        )) deallocate(cgrid%mmsqu_rshortup        )
      if(associated(cgrid%mmsqu_rnet            )) deallocate(cgrid%mmsqu_rnet            )
      if(associated(cgrid%mmsqu_albedo          )) deallocate(cgrid%mmsqu_albedo          )
      if(associated(cgrid%mmsqu_ustar           )) deallocate(cgrid%mmsqu_ustar           )
      if(associated(cgrid%mmsqu_carbon_ac       )) deallocate(cgrid%mmsqu_carbon_ac       )
      if(associated(cgrid%mmsqu_carbon_st       )) deallocate(cgrid%mmsqu_carbon_st       )
      if(associated(cgrid%mmsqu_vapor_gc        )) deallocate(cgrid%mmsqu_vapor_gc        )
      if(associated(cgrid%mmsqu_vapor_ac        )) deallocate(cgrid%mmsqu_vapor_ac        )
      if(associated(cgrid%mmsqu_sensible_gc     )) deallocate(cgrid%mmsqu_sensible_gc     )
      if(associated(cgrid%mmsqu_sensible_ac     )) deallocate(cgrid%mmsqu_sensible_ac     )
      if(associated(cgrid%qmean_gpp             )) deallocate(cgrid%qmean_gpp             )
      if(associated(cgrid%qmean_npp             )) deallocate(cgrid%qmean_npp             )
      if(associated(cgrid%qmean_leaf_resp       )) deallocate(cgrid%qmean_leaf_resp       )
      if(associated(cgrid%qmean_root_resp       )) deallocate(cgrid%qmean_root_resp       )
      if(associated(cgrid%qmean_leaf_growth_resp)) deallocate(cgrid%qmean_leaf_growth_resp)
      if(associated(cgrid%qmean_root_growth_resp)) deallocate(cgrid%qmean_root_growth_resp)
      if(associated(cgrid%qmean_sapa_growth_resp)) deallocate(cgrid%qmean_sapa_growth_resp)
      if(associated(cgrid%qmean_sapb_growth_resp)) deallocate(cgrid%qmean_sapb_growth_resp)
      if(associated(cgrid%qmean_leaf_storage_resp)) deallocate(cgrid%qmean_leaf_storage_resp)
      if(associated(cgrid%qmean_root_storage_resp)) deallocate(cgrid%qmean_root_storage_resp)
      if(associated(cgrid%qmean_sapa_storage_resp)) deallocate(cgrid%qmean_sapa_storage_resp)
      if(associated(cgrid%qmean_sapb_storage_resp)) deallocate(cgrid%qmean_sapb_storage_resp)
      if(associated(cgrid%qmean_plresp          )) deallocate(cgrid%qmean_plresp          )
      if(associated(cgrid%qmean_leaf_energy     )) deallocate(cgrid%qmean_leaf_energy     )
      if(associated(cgrid%qmean_leaf_water      )) deallocate(cgrid%qmean_leaf_water      )
      if(associated(cgrid%qmean_leaf_hcap       )) deallocate(cgrid%qmean_leaf_hcap       )
      if(associated(cgrid%qmean_leaf_vpdef      )) deallocate(cgrid%qmean_leaf_vpdef      )
      if(associated(cgrid%qmean_leaf_temp       )) deallocate(cgrid%qmean_leaf_temp       )
      if(associated(cgrid%qmean_leaf_fliq       )) deallocate(cgrid%qmean_leaf_fliq       )
      if(associated(cgrid%qmean_leaf_gsw        )) deallocate(cgrid%qmean_leaf_gsw        )
      if(associated(cgrid%qmean_leaf_gbw        )) deallocate(cgrid%qmean_leaf_gbw        )
      if(associated(cgrid%qmean_wood_energy     )) deallocate(cgrid%qmean_wood_energy     )
      if(associated(cgrid%qmean_wood_water      )) deallocate(cgrid%qmean_wood_water      )
      if(associated(cgrid%qmean_wood_hcap       )) deallocate(cgrid%qmean_wood_hcap       )
      if(associated(cgrid%qmean_wood_temp       )) deallocate(cgrid%qmean_wood_temp       )
      if(associated(cgrid%qmean_wood_fliq       )) deallocate(cgrid%qmean_wood_fliq       )
      if(associated(cgrid%qmean_wood_gbw        )) deallocate(cgrid%qmean_wood_gbw        )
      if(associated(cgrid%qmean_fs_open         )) deallocate(cgrid%qmean_fs_open         )
      if(associated(cgrid%qmean_fsw             )) deallocate(cgrid%qmean_fsw             )
      if(associated(cgrid%qmean_fsn             )) deallocate(cgrid%qmean_fsn             )
      if(associated(cgrid%qmean_a_open          )) deallocate(cgrid%qmean_a_open          )
      if(associated(cgrid%qmean_a_closed        )) deallocate(cgrid%qmean_a_closed        )
      if(associated(cgrid%qmean_a_net           )) deallocate(cgrid%qmean_a_net           )
      if(associated(cgrid%qmean_A_light         )) deallocate(cgrid%qmean_A_light         )
      if(associated(cgrid%qmean_A_rubp          )) deallocate(cgrid%qmean_A_rubp          )
      if(associated(cgrid%qmean_A_co2           )) deallocate(cgrid%qmean_A_co2           )
      if(associated(cgrid%qmean_psi_open        )) deallocate(cgrid%qmean_psi_open        )
      if(associated(cgrid%qmean_psi_closed      )) deallocate(cgrid%qmean_psi_closed      )
      if(associated(cgrid%qmean_water_supply    )) deallocate(cgrid%qmean_water_supply    )
      if(associated(cgrid%qmean_par_l           )) deallocate(cgrid%qmean_par_l           )
      if(associated(cgrid%qmean_par_l_beam      )) deallocate(cgrid%qmean_par_l_beam      )
      if(associated(cgrid%qmean_par_l_diff      )) deallocate(cgrid%qmean_par_l_diff      )
      if(associated(cgrid%qmean_rshort_l        )) deallocate(cgrid%qmean_rshort_l        )
      if(associated(cgrid%qmean_rlong_l         )) deallocate(cgrid%qmean_rlong_l         )
      if(associated(cgrid%qmean_sensible_lc     )) deallocate(cgrid%qmean_sensible_lc     )
      if(associated(cgrid%qmean_vapor_lc        )) deallocate(cgrid%qmean_vapor_lc        )
      if(associated(cgrid%qmean_transp          )) deallocate(cgrid%qmean_transp          )
      if(associated(cgrid%qmean_intercepted_al  )) deallocate(cgrid%qmean_intercepted_al  )
      if(associated(cgrid%qmean_wshed_lg        )) deallocate(cgrid%qmean_wshed_lg        )
      if(associated(cgrid%qmean_rshort_w        )) deallocate(cgrid%qmean_rshort_w        )
      if(associated(cgrid%qmean_rlong_w         )) deallocate(cgrid%qmean_rlong_w         )
      if(associated(cgrid%qmean_sensible_wc     )) deallocate(cgrid%qmean_sensible_wc     )
      if(associated(cgrid%qmean_vapor_wc        )) deallocate(cgrid%qmean_vapor_wc        )
      if(associated(cgrid%qmean_intercepted_aw  )) deallocate(cgrid%qmean_intercepted_aw  )
      if(associated(cgrid%qmean_wshed_wg        )) deallocate(cgrid%qmean_wshed_wg        )
      if(associated(cgrid%qmean_rh              )) deallocate(cgrid%qmean_rh              )
      if(associated(cgrid%qmean_cwd_rh          )) deallocate(cgrid%qmean_cwd_rh          )
      if(associated(cgrid%qmean_nep             )) deallocate(cgrid%qmean_nep             )
      if(associated(cgrid%qmean_rk4step         )) deallocate(cgrid%qmean_rk4step         )
      if(associated(cgrid%qmean_available_water )) deallocate(cgrid%qmean_available_water )
      if(associated(cgrid%qmean_can_theiv       )) deallocate(cgrid%qmean_can_theiv       )
      if(associated(cgrid%qmean_can_theta       )) deallocate(cgrid%qmean_can_theta       )
      if(associated(cgrid%qmean_can_vpdef       )) deallocate(cgrid%qmean_can_vpdef       )
      if(associated(cgrid%qmean_can_temp        )) deallocate(cgrid%qmean_can_temp        )
      if(associated(cgrid%qmean_can_shv         )) deallocate(cgrid%qmean_can_shv         )
      if(associated(cgrid%qmean_can_co2         )) deallocate(cgrid%qmean_can_co2         )
      if(associated(cgrid%qmean_can_rhos        )) deallocate(cgrid%qmean_can_rhos        )
      if(associated(cgrid%qmean_can_prss        )) deallocate(cgrid%qmean_can_prss        )
      if(associated(cgrid%qmean_gnd_temp        )) deallocate(cgrid%qmean_gnd_temp        )
      if(associated(cgrid%qmean_gnd_shv         )) deallocate(cgrid%qmean_gnd_shv         )
      if(associated(cgrid%qmean_can_ggnd        )) deallocate(cgrid%qmean_can_ggnd        )
      if(associated(cgrid%qmean_sfcw_depth      )) deallocate(cgrid%qmean_sfcw_depth      )
      if(associated(cgrid%qmean_sfcw_energy     )) deallocate(cgrid%qmean_sfcw_energy     )
      if(associated(cgrid%qmean_sfcw_mass       )) deallocate(cgrid%qmean_sfcw_mass       )
      if(associated(cgrid%qmean_sfcw_temp       )) deallocate(cgrid%qmean_sfcw_temp       )
      if(associated(cgrid%qmean_sfcw_fliq       )) deallocate(cgrid%qmean_sfcw_fliq       )
      if(associated(cgrid%qmean_soil_energy     )) deallocate(cgrid%qmean_soil_energy     )
      if(associated(cgrid%qmean_soil_mstpot     )) deallocate(cgrid%qmean_soil_mstpot     )
      if(associated(cgrid%qmean_soil_water      )) deallocate(cgrid%qmean_soil_water      )
      if(associated(cgrid%qmean_soil_temp       )) deallocate(cgrid%qmean_soil_temp       )
      if(associated(cgrid%qmean_soil_fliq       )) deallocate(cgrid%qmean_soil_fliq       )
      if(associated(cgrid%qmean_rshort_gnd      )) deallocate(cgrid%qmean_rshort_gnd      )
      if(associated(cgrid%qmean_par_gnd         )) deallocate(cgrid%qmean_par_gnd         )
      if(associated(cgrid%qmean_rlong_gnd       )) deallocate(cgrid%qmean_rlong_gnd       )
      if(associated(cgrid%qmean_rlongup         )) deallocate(cgrid%qmean_rlongup         )
      if(associated(cgrid%qmean_parup           )) deallocate(cgrid%qmean_parup           )
      if(associated(cgrid%qmean_nirup           )) deallocate(cgrid%qmean_nirup           )
      if(associated(cgrid%qmean_rshortup        )) deallocate(cgrid%qmean_rshortup        )
      if(associated(cgrid%qmean_rnet            )) deallocate(cgrid%qmean_rnet            )
      if(associated(cgrid%qmean_albedo          )) deallocate(cgrid%qmean_albedo          )
      if(associated(cgrid%qmean_albedo_par      )) deallocate(cgrid%qmean_albedo_par      )
      if(associated(cgrid%qmean_albedo_nir      )) deallocate(cgrid%qmean_albedo_nir      )
      if(associated(cgrid%qmean_rlong_albedo    )) deallocate(cgrid%qmean_rlong_albedo    )
      if(associated(cgrid%qmean_ustar           )) deallocate(cgrid%qmean_ustar           )
      if(associated(cgrid%qmean_tstar           )) deallocate(cgrid%qmean_tstar           )
      if(associated(cgrid%qmean_qstar           )) deallocate(cgrid%qmean_qstar           )
      if(associated(cgrid%qmean_cstar           )) deallocate(cgrid%qmean_cstar           )
      if(associated(cgrid%qmean_carbon_ac       )) deallocate(cgrid%qmean_carbon_ac       )
      if(associated(cgrid%qmean_carbon_st       )) deallocate(cgrid%qmean_carbon_st       )
      if(associated(cgrid%qmean_vapor_gc        )) deallocate(cgrid%qmean_vapor_gc        )
      if(associated(cgrid%qmean_vapor_ac        )) deallocate(cgrid%qmean_vapor_ac        )
      if(associated(cgrid%qmean_smoist_gg       )) deallocate(cgrid%qmean_smoist_gg       )
      if(associated(cgrid%qmean_throughfall     )) deallocate(cgrid%qmean_throughfall     )
      if(associated(cgrid%qmean_transloss       )) deallocate(cgrid%qmean_transloss       )
      if(associated(cgrid%qmean_runoff          )) deallocate(cgrid%qmean_runoff          )
      if(associated(cgrid%qmean_drainage        )) deallocate(cgrid%qmean_drainage        )
      if(associated(cgrid%qmean_sensible_gc     )) deallocate(cgrid%qmean_sensible_gc     )
      if(associated(cgrid%qmean_sensible_ac     )) deallocate(cgrid%qmean_sensible_ac     )
      if(associated(cgrid%qmean_sensible_gg     )) deallocate(cgrid%qmean_sensible_gg     )
      if(associated(cgrid%qmean_qthroughfall    )) deallocate(cgrid%qmean_qthroughfall    )
      if(associated(cgrid%qmean_qrunoff         )) deallocate(cgrid%qmean_qrunoff         )
      if(associated(cgrid%qmean_qdrainage       )) deallocate(cgrid%qmean_qdrainage       )
      if(associated(cgrid%qmean_atm_theiv       )) deallocate(cgrid%qmean_atm_theiv       )
      if(associated(cgrid%qmean_atm_theta       )) deallocate(cgrid%qmean_atm_theta       )
      if(associated(cgrid%qmean_atm_temp        )) deallocate(cgrid%qmean_atm_temp        )
      if(associated(cgrid%qmean_atm_vpdef       )) deallocate(cgrid%qmean_atm_vpdef       )
      if(associated(cgrid%qmean_atm_shv         )) deallocate(cgrid%qmean_atm_shv         )
      if(associated(cgrid%qmean_atm_rshort      )) deallocate(cgrid%qmean_atm_rshort      )
      if(associated(cgrid%qmean_atm_rshort_diff )) deallocate(cgrid%qmean_atm_rshort_diff )
      if(associated(cgrid%qmean_atm_par         )) deallocate(cgrid%qmean_atm_par         )
      if(associated(cgrid%qmean_atm_par_diff    )) deallocate(cgrid%qmean_atm_par_diff    )
      if(associated(cgrid%qmean_atm_rlong       )) deallocate(cgrid%qmean_atm_rlong       )
      if(associated(cgrid%qmean_atm_vels        )) deallocate(cgrid%qmean_atm_vels        )
      if(associated(cgrid%qmean_atm_rhos        )) deallocate(cgrid%qmean_atm_rhos        )
      if(associated(cgrid%qmean_atm_prss        )) deallocate(cgrid%qmean_atm_prss        )
      if(associated(cgrid%qmean_atm_co2         )) deallocate(cgrid%qmean_atm_co2         )
      if(associated(cgrid%qmean_pcpg            )) deallocate(cgrid%qmean_pcpg            )
      if(associated(cgrid%qmean_qpcpg           )) deallocate(cgrid%qmean_qpcpg           )
      if(associated(cgrid%qmean_dpcpg           )) deallocate(cgrid%qmean_dpcpg           )
      if(associated(cgrid%qmsqu_gpp             )) deallocate(cgrid%qmsqu_gpp             )
      if(associated(cgrid%qmsqu_npp             )) deallocate(cgrid%qmsqu_npp             )
      if(associated(cgrid%qmsqu_plresp          )) deallocate(cgrid%qmsqu_plresp          )
      if(associated(cgrid%qmsqu_sensible_lc     )) deallocate(cgrid%qmsqu_sensible_lc     )
      if(associated(cgrid%qmsqu_vapor_lc        )) deallocate(cgrid%qmsqu_vapor_lc        )
      if(associated(cgrid%qmsqu_transp          )) deallocate(cgrid%qmsqu_transp          )
      if(associated(cgrid%qmsqu_sensible_wc     )) deallocate(cgrid%qmsqu_sensible_wc     )
      if(associated(cgrid%qmsqu_vapor_wc        )) deallocate(cgrid%qmsqu_vapor_wc        )
      if(associated(cgrid%qmsqu_rh              )) deallocate(cgrid%qmsqu_rh              )
      if(associated(cgrid%qmsqu_cwd_rh          )) deallocate(cgrid%qmsqu_cwd_rh          )
      if(associated(cgrid%qmsqu_nep             )) deallocate(cgrid%qmsqu_nep             )
      if(associated(cgrid%qmsqu_rlongup         )) deallocate(cgrid%qmsqu_rlongup         )
      if(associated(cgrid%qmsqu_parup           )) deallocate(cgrid%qmsqu_parup           )
      if(associated(cgrid%qmsqu_nirup           )) deallocate(cgrid%qmsqu_nirup           )
      if(associated(cgrid%qmsqu_rshortup        )) deallocate(cgrid%qmsqu_rshortup        )
      if(associated(cgrid%qmsqu_rnet            )) deallocate(cgrid%qmsqu_rnet            )
      if(associated(cgrid%qmsqu_albedo          )) deallocate(cgrid%qmsqu_albedo          )
      if(associated(cgrid%qmsqu_ustar           )) deallocate(cgrid%qmsqu_ustar           )
      if(associated(cgrid%qmsqu_carbon_ac       )) deallocate(cgrid%qmsqu_carbon_ac       )
      if(associated(cgrid%qmsqu_carbon_st       )) deallocate(cgrid%qmsqu_carbon_st       )
      if(associated(cgrid%qmsqu_vapor_gc        )) deallocate(cgrid%qmsqu_vapor_gc        )
      if(associated(cgrid%qmsqu_vapor_ac        )) deallocate(cgrid%qmsqu_vapor_ac        )
      if(associated(cgrid%qmsqu_sensible_gc     )) deallocate(cgrid%qmsqu_sensible_gc     )
      if(associated(cgrid%qmsqu_sensible_ac     )) deallocate(cgrid%qmsqu_sensible_ac     )


      return
   end subroutine deallocate_edtype
   !=======================================================================================!
   !=======================================================================================!





   !=======================================================================================!
   !=======================================================================================!
   !     This sub-routine de-allocates all site pointers.                                  !
   !---------------------------------------------------------------------------------------!
   subroutine deallocate_polygontype(cpoly)
      implicit none

      !----- Arguments. -------------------------------------------------------------------!
      type(polygontype), target :: cpoly
      !----- Local variables. -------------------------------------------------------------!
      integer                   :: isi
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      !     First thing we must do is to deallocate the nested dimensions.                 !
      !------------------------------------------------------------------------------------!
      do isi = 1, cpoly%nsites
         call deallocate_sitetype(cpoly%site(isi))
      end do
      if(associated(cpoly%site)) deallocate(cpoly%site)
      !------------------------------------------------------------------------------------!


      if(associated(cpoly%sipa_id               )) deallocate(cpoly%sipa_id               )
      if(associated(cpoly%sipa_n                )) deallocate(cpoly%sipa_n                )
      if(associated(cpoly%patch_count           )) deallocate(cpoly%patch_count           )
      if(associated(cpoly%sitenum               )) deallocate(cpoly%sitenum               )
      if(associated(cpoly%met                   )) deallocate(cpoly%met                   )
      if(associated(cpoly%area                  )) deallocate(cpoly%area                  )
      if(associated(cpoly%patch_area            )) deallocate(cpoly%patch_area            )
      if(associated(cpoly%elevation             )) deallocate(cpoly%elevation             )
      if(associated(cpoly%slope                 )) deallocate(cpoly%slope                 )
      if(associated(cpoly%aspect                )) deallocate(cpoly%aspect                )
      if(associated(cpoly%lsl                   )) deallocate(cpoly%lsl                   )
      if(associated(cpoly%ncol_soil             )) deallocate(cpoly%ncol_soil             )
      if(associated(cpoly%ntext_soil            )) deallocate(cpoly%ntext_soil            )
      if(associated(cpoly%TCI                   )) deallocate(cpoly%TCI                   )
      if(associated(cpoly%pptweight             )) deallocate(cpoly%pptweight             )
      if(associated(cpoly%hydro_next            )) deallocate(cpoly%hydro_next            )
      if(associated(cpoly%hydro_prev            )) deallocate(cpoly%hydro_prev            )
      if(associated(cpoly%moist_W               )) deallocate(cpoly%moist_W               )
      if(associated(cpoly%moist_f               )) deallocate(cpoly%moist_f               )
      if(associated(cpoly%moist_tau             )) deallocate(cpoly%moist_tau             )
      if(associated(cpoly%moist_zi              )) deallocate(cpoly%moist_zi              )
      if(associated(cpoly%baseflow              )) deallocate(cpoly%baseflow              )
      if(associated(cpoly%runoff                )) deallocate(cpoly%runoff                )
      if(associated(cpoly%qrunoff               )) deallocate(cpoly%qrunoff               )
      if(associated(cpoly%min_monthly_temp      )) deallocate(cpoly%min_monthly_temp      )
      if(associated(cpoly%num_landuse_years     )) deallocate(cpoly%num_landuse_years     )
      if(associated(cpoly%mindbh_primary        )) deallocate(cpoly%mindbh_primary        )
      if(associated(cpoly%probharv_primary      )) deallocate(cpoly%probharv_primary      )
      if(associated(cpoly%mindbh_secondary      )) deallocate(cpoly%mindbh_secondary      )
      if(associated(cpoly%probharv_secondary    )) deallocate(cpoly%probharv_secondary    )
      if(associated(cpoly%plantation            )) deallocate(cpoly%plantation            )
      if(associated(cpoly%agri_stocking_pft     )) deallocate(cpoly%agri_stocking_pft     )
      if(associated(cpoly%agri_stocking_density )) deallocate(cpoly%agri_stocking_density )
      if(associated(cpoly%plantation_stocking_pft    ))                                    &
                                              deallocate(cpoly%plantation_stocking_pft    )
      if(associated(cpoly%plantation_stocking_density))                                    &
                                              deallocate(cpoly%plantation_stocking_density)
      if(associated(cpoly%primary_harvest_target     ))                                    &
                                              deallocate(cpoly%primary_harvest_target     )
      if(associated(cpoly%secondary_harvest_target   ))                                    &
                                              deallocate(cpoly%secondary_harvest_target   )
      if(associated(cpoly%primary_harvest_memory     ))                                    &
                                              deallocate(cpoly%primary_harvest_memory     )
      if(associated(cpoly%secondary_harvest_memory   ))                                    &
                                              deallocate(cpoly%secondary_harvest_memory   )
      if(associated(cpoly%ignition_rate         )) deallocate(cpoly%ignition_rate         )
      if(associated(cpoly%lambda_fire           )) deallocate(cpoly%lambda_fire           )
      if(associated(cpoly%avg_monthly_pcpg      )) deallocate(cpoly%avg_monthly_pcpg      )
      if(associated(cpoly%phen_pars             )) deallocate(cpoly%phen_pars             )
      if(associated(cpoly%disturbance_memory    )) deallocate(cpoly%disturbance_memory    )
      if(associated(cpoly%disturbance_rates     )) deallocate(cpoly%disturbance_rates     )
      if(associated(cpoly%green_leaf_factor     )) deallocate(cpoly%green_leaf_factor     )
      if(associated(cpoly%leaf_aging_factor     )) deallocate(cpoly%leaf_aging_factor     )
      if(associated(cpoly%basal_area            )) deallocate(cpoly%basal_area            )
      if(associated(cpoly%basal_area_growth     )) deallocate(cpoly%basal_area_growth     )
      if(associated(cpoly%agb                   )) deallocate(cpoly%agb                   )
      if(associated(cpoly%agb_growth            )) deallocate(cpoly%agb_growth            )
      if(associated(cpoly%basal_area_mort       )) deallocate(cpoly%basal_area_mort       )
      if(associated(cpoly%basal_area_cut        )) deallocate(cpoly%basal_area_cut        )
      if(associated(cpoly%agb_mort              )) deallocate(cpoly%agb_mort              )
      if(associated(cpoly%agb_cut               )) deallocate(cpoly%agb_cut               )
      if(associated(cpoly%cosaoi                )) deallocate(cpoly%cosaoi                )
      if(associated(cpoly%daylight              )) deallocate(cpoly%daylight              )
      if(associated(cpoly%nighttime             )) deallocate(cpoly%nighttime             )
      if(associated(cpoly%rad_avg               )) deallocate(cpoly%rad_avg               )
      if(associated(cpoly%fmean_atm_theiv       )) deallocate(cpoly%fmean_atm_theiv       )
      if(associated(cpoly%fmean_atm_theta       )) deallocate(cpoly%fmean_atm_theta       )
      if(associated(cpoly%fmean_atm_temp        )) deallocate(cpoly%fmean_atm_temp        )
      if(associated(cpoly%fmean_atm_vpdef       )) deallocate(cpoly%fmean_atm_vpdef       )
      if(associated(cpoly%fmean_atm_shv         )) deallocate(cpoly%fmean_atm_shv         )
      if(associated(cpoly%fmean_atm_rshort      )) deallocate(cpoly%fmean_atm_rshort      )
      if(associated(cpoly%fmean_atm_rshort_diff )) deallocate(cpoly%fmean_atm_rshort_diff )
      if(associated(cpoly%fmean_atm_par         )) deallocate(cpoly%fmean_atm_par         )
      if(associated(cpoly%fmean_atm_par_diff    )) deallocate(cpoly%fmean_atm_par_diff    )
      if(associated(cpoly%fmean_atm_rlong       )) deallocate(cpoly%fmean_atm_rlong       )
      if(associated(cpoly%fmean_atm_vels        )) deallocate(cpoly%fmean_atm_vels        )
      if(associated(cpoly%fmean_atm_rhos        )) deallocate(cpoly%fmean_atm_rhos        )
      if(associated(cpoly%fmean_atm_prss        )) deallocate(cpoly%fmean_atm_prss        )
      if(associated(cpoly%fmean_atm_co2         )) deallocate(cpoly%fmean_atm_co2         )
      if(associated(cpoly%fmean_pcpg            )) deallocate(cpoly%fmean_pcpg            )
      if(associated(cpoly%fmean_qpcpg           )) deallocate(cpoly%fmean_qpcpg           )
      if(associated(cpoly%fmean_dpcpg           )) deallocate(cpoly%fmean_dpcpg           )
      if(associated(cpoly%dmean_atm_theiv       )) deallocate(cpoly%dmean_atm_theiv       )
      if(associated(cpoly%dmean_atm_theta       )) deallocate(cpoly%dmean_atm_theta       )
      if(associated(cpoly%dmean_atm_temp        )) deallocate(cpoly%dmean_atm_temp        )
      if(associated(cpoly%dmean_atm_vpdef       )) deallocate(cpoly%dmean_atm_vpdef       )
      if(associated(cpoly%dmean_atm_shv         )) deallocate(cpoly%dmean_atm_shv         )
      if(associated(cpoly%dmean_atm_rshort      )) deallocate(cpoly%dmean_atm_rshort      )
      if(associated(cpoly%dmean_atm_rshort_diff )) deallocate(cpoly%dmean_atm_rshort_diff )
      if(associated(cpoly%dmean_atm_par         )) deallocate(cpoly%dmean_atm_par         )
      if(associated(cpoly%dmean_atm_par_diff    )) deallocate(cpoly%dmean_atm_par_diff    )
      if(associated(cpoly%dmean_atm_rlong       )) deallocate(cpoly%dmean_atm_rlong       )
      if(associated(cpoly%dmean_atm_vels        )) deallocate(cpoly%dmean_atm_vels        )
      if(associated(cpoly%dmean_atm_rhos        )) deallocate(cpoly%dmean_atm_rhos        )
      if(associated(cpoly%dmean_atm_prss        )) deallocate(cpoly%dmean_atm_prss        )
      if(associated(cpoly%dmean_atm_co2         )) deallocate(cpoly%dmean_atm_co2         )
      if(associated(cpoly%dmean_pcpg            )) deallocate(cpoly%dmean_pcpg            )
      if(associated(cpoly%dmean_qpcpg           )) deallocate(cpoly%dmean_qpcpg           )
      if(associated(cpoly%dmean_dpcpg           )) deallocate(cpoly%dmean_dpcpg           )
      if(associated(cpoly%mmean_atm_theiv       )) deallocate(cpoly%mmean_atm_theiv       )
      if(associated(cpoly%mmean_atm_theta       )) deallocate(cpoly%mmean_atm_theta       )
      if(associated(cpoly%mmean_atm_temp        )) deallocate(cpoly%mmean_atm_temp        )
      if(associated(cpoly%mmean_atm_vpdef       )) deallocate(cpoly%mmean_atm_vpdef       )
      if(associated(cpoly%mmean_atm_shv         )) deallocate(cpoly%mmean_atm_shv         )
      if(associated(cpoly%mmean_atm_rshort      )) deallocate(cpoly%mmean_atm_rshort      )
      if(associated(cpoly%mmean_atm_rshort_diff )) deallocate(cpoly%mmean_atm_rshort_diff )
      if(associated(cpoly%mmean_atm_par         )) deallocate(cpoly%mmean_atm_par         )
      if(associated(cpoly%mmean_atm_par_diff    )) deallocate(cpoly%mmean_atm_par_diff    )
      if(associated(cpoly%mmean_atm_rlong       )) deallocate(cpoly%mmean_atm_rlong       )
      if(associated(cpoly%mmean_atm_vels        )) deallocate(cpoly%mmean_atm_vels        )
      if(associated(cpoly%mmean_atm_rhos        )) deallocate(cpoly%mmean_atm_rhos        )
      if(associated(cpoly%mmean_atm_prss        )) deallocate(cpoly%mmean_atm_prss        )
      if(associated(cpoly%mmean_atm_co2         )) deallocate(cpoly%mmean_atm_co2         )
      if(associated(cpoly%mmean_pcpg            )) deallocate(cpoly%mmean_pcpg            )
      if(associated(cpoly%mmean_qpcpg           )) deallocate(cpoly%mmean_qpcpg           )
      if(associated(cpoly%mmean_dpcpg           )) deallocate(cpoly%mmean_dpcpg           )
      if(associated(cpoly%qmean_atm_theiv       )) deallocate(cpoly%qmean_atm_theiv       )
      if(associated(cpoly%qmean_atm_theta       )) deallocate(cpoly%qmean_atm_theta       )
      if(associated(cpoly%qmean_atm_temp        )) deallocate(cpoly%qmean_atm_temp        )
      if(associated(cpoly%qmean_atm_vpdef       )) deallocate(cpoly%qmean_atm_vpdef       )
      if(associated(cpoly%qmean_atm_shv         )) deallocate(cpoly%qmean_atm_shv         )
      if(associated(cpoly%qmean_atm_rshort      )) deallocate(cpoly%qmean_atm_rshort      )
      if(associated(cpoly%qmean_atm_rshort_diff )) deallocate(cpoly%qmean_atm_rshort_diff )
      if(associated(cpoly%qmean_atm_par         )) deallocate(cpoly%qmean_atm_par         )
      if(associated(cpoly%qmean_atm_par_diff    )) deallocate(cpoly%qmean_atm_par_diff    )
      if(associated(cpoly%qmean_atm_rlong       )) deallocate(cpoly%qmean_atm_rlong       )
      if(associated(cpoly%qmean_atm_vels        )) deallocate(cpoly%qmean_atm_vels        )
      if(associated(cpoly%qmean_atm_rhos        )) deallocate(cpoly%qmean_atm_rhos        )
      if(associated(cpoly%qmean_atm_prss        )) deallocate(cpoly%qmean_atm_prss        )
      if(associated(cpoly%qmean_atm_co2         )) deallocate(cpoly%qmean_atm_co2         )
      if(associated(cpoly%qmean_pcpg            )) deallocate(cpoly%qmean_pcpg            )
      if(associated(cpoly%qmean_qpcpg           )) deallocate(cpoly%qmean_qpcpg           )
      if(associated(cpoly%qmean_dpcpg           )) deallocate(cpoly%qmean_dpcpg           )
      return
   end subroutine deallocate_polygontype
   !=======================================================================================!
   !=======================================================================================!





   !=======================================================================================!
   !=======================================================================================!
   !     This sub-routine de-allocates all patch pointers.                                 !
   !---------------------------------------------------------------------------------------!
   subroutine deallocate_sitetype(csite)
      implicit none

      !----- Arguments. -------------------------------------------------------------------!
      type(sitetype)   , target :: csite
      !----- Local variables. -------------------------------------------------------------!
      integer                   :: ipa
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      !     First thing we must do is to deallocate the nested dimensions.                 !
      !------------------------------------------------------------------------------------!
      do ipa=1,csite%npatches
          call deallocate_patchtype(csite%patch(ipa))
      end do
      if (associated(csite%patch)) deallocate(csite%patch)
      !------------------------------------------------------------------------------------!

      if(associated(csite%paco_id               )) deallocate(csite%paco_id               )
      if(associated(csite%paco_n                )) deallocate(csite%paco_n                )
      if(associated(csite%patch                 )) deallocate(csite%patch                 )
      if(associated(csite%cohort_count          )) deallocate(csite%cohort_count          )
      if(associated(csite%pname                 )) deallocate(csite%pname                 )
      if(associated(csite%area                  )) deallocate(csite%area                  )
      if(associated(csite%age                   )) deallocate(csite%age                   )
      if(associated(csite%dist_type             )) deallocate(csite%dist_type             )
      if(associated(csite%fast_soil_C           )) deallocate(csite%fast_soil_C           )
      if(associated(csite%slow_soil_C           )) deallocate(csite%slow_soil_C           )
      if(associated(csite%structural_soil_C     )) deallocate(csite%structural_soil_C     )
      if(associated(csite%structural_soil_L     )) deallocate(csite%structural_soil_L     )
      if(associated(csite%mineralized_soil_N    )) deallocate(csite%mineralized_soil_N    )
      if(associated(csite%fast_soil_N           )) deallocate(csite%fast_soil_N           )
      if(associated(csite%sum_dgd               )) deallocate(csite%sum_dgd               )
      if(associated(csite%sum_chd               )) deallocate(csite%sum_chd               )
      if(associated(csite%can_theiv             )) deallocate(csite%can_theiv             )
      if(associated(csite%can_vpdef             )) deallocate(csite%can_vpdef             )
      if(associated(csite%can_temp              )) deallocate(csite%can_temp              )
      if(associated(csite%can_temp_pv           )) deallocate(csite%can_temp_pv           )
      if(associated(csite%can_shv               )) deallocate(csite%can_shv               )
      if(associated(csite%can_co2               )) deallocate(csite%can_co2               )
      if(associated(csite%can_rhos              )) deallocate(csite%can_rhos              )
      if(associated(csite%can_prss              )) deallocate(csite%can_prss              )
      if(associated(csite%can_theta             )) deallocate(csite%can_theta             )
      if(associated(csite%can_depth             )) deallocate(csite%can_depth             )
      if(associated(csite%opencan_frac          )) deallocate(csite%opencan_frac          )
      if(associated(csite%ggbare                )) deallocate(csite%ggbare                )
      if(associated(csite%ggveg                 )) deallocate(csite%ggveg                 )
      if(associated(csite%ggnet                 )) deallocate(csite%ggnet                 )
      if(associated(csite%ggsoil                )) deallocate(csite%ggsoil                )
      if(associated(csite%nlev_sfcwater         )) deallocate(csite%nlev_sfcwater         )
      if(associated(csite%sfcwater_mass         )) deallocate(csite%sfcwater_mass         )
      if(associated(csite%sfcwater_energy       )) deallocate(csite%sfcwater_energy       )
      if(associated(csite%sfcwater_depth        )) deallocate(csite%sfcwater_depth        )
      if(associated(csite%sfcwater_tempk        )) deallocate(csite%sfcwater_tempk        )
      if(associated(csite%sfcwater_fracliq      )) deallocate(csite%sfcwater_fracliq      )
      if(associated(csite%rshort_s              )) deallocate(csite%rshort_s              )
      if(associated(csite%rshort_s_beam         )) deallocate(csite%rshort_s_beam         )
      if(associated(csite%rshort_s_diffuse      )) deallocate(csite%rshort_s_diffuse      )
      if(associated(csite%par_s                 )) deallocate(csite%par_s                 )
      if(associated(csite%par_s_beam            )) deallocate(csite%par_s_beam            )
      if(associated(csite%par_s_diffuse         )) deallocate(csite%par_s_diffuse         )
      if(associated(csite%soil_energy           )) deallocate(csite%soil_energy           )
      if(associated(csite%soil_mstpot           )) deallocate(csite%soil_mstpot           )
      if(associated(csite%soil_water            )) deallocate(csite%soil_water            )
      if(associated(csite%soil_tempk            )) deallocate(csite%soil_tempk            )
      if(associated(csite%soil_fracliq          )) deallocate(csite%soil_fracliq          )
      if(associated(csite%rootdense             )) deallocate(csite%rootdense             )
      if(associated(csite%ground_shv            )) deallocate(csite%ground_shv            )
      if(associated(csite%ground_ssh            )) deallocate(csite%ground_ssh            )
      if(associated(csite%ground_temp           )) deallocate(csite%ground_temp           )
      if(associated(csite%ground_fliq           )) deallocate(csite%ground_fliq           )
      if(associated(csite%rough                 )) deallocate(csite%rough                 )
      if(associated(csite%par_l_max             )) deallocate(csite%par_l_max             )
      if(associated(csite%par_l_beam_max        )) deallocate(csite%par_l_beam_max        )
      if(associated(csite%par_l_diffuse_max     )) deallocate(csite%par_l_diffuse_max     )
      if(associated(csite%A_o_max               )) deallocate(csite%A_o_max               )
      if(associated(csite%A_c_max               )) deallocate(csite%A_c_max               )
      if(associated(csite%avg_daily_temp        )) deallocate(csite%avg_daily_temp        )
      if(associated(csite%avg_monthly_gndwater  )) deallocate(csite%avg_monthly_gndwater  )
      if(associated(csite%avg_monthly_waterdef  )) deallocate(csite%avg_monthly_waterdef  )
      if(associated(csite%wbudget_loss2atm      )) deallocate(csite%wbudget_loss2atm      )
      if(associated(csite%wbudget_denseffect    )) deallocate(csite%wbudget_denseffect    )
      if(associated(csite%wbudget_precipgain    )) deallocate(csite%wbudget_precipgain    )
      if(associated(csite%wbudget_loss2runoff   )) deallocate(csite%wbudget_loss2runoff   )
      if(associated(csite%wbudget_loss2drainage )) deallocate(csite%wbudget_loss2drainage )
      if(associated(csite%wbudget_initialstorage)) deallocate(csite%wbudget_initialstorage)
      if(associated(csite%wbudget_residual      )) deallocate(csite%wbudget_residual      )
      if(associated(csite%ebudget_loss2atm      )) deallocate(csite%ebudget_loss2atm      )
      if(associated(csite%ebudget_denseffect    )) deallocate(csite%ebudget_denseffect    )
      if(associated(csite%ebudget_prsseffect    )) deallocate(csite%ebudget_prsseffect    )
      if(associated(csite%ebudget_loss2runoff   )) deallocate(csite%ebudget_loss2runoff   )
      if(associated(csite%ebudget_loss2drainage )) deallocate(csite%ebudget_loss2drainage )
      if(associated(csite%ebudget_netrad        )) deallocate(csite%ebudget_netrad        )
      if(associated(csite%ebudget_precipgain    )) deallocate(csite%ebudget_precipgain    )
      if(associated(csite%ebudget_initialstorage)) deallocate(csite%ebudget_initialstorage)
      if(associated(csite%ebudget_residual      )) deallocate(csite%ebudget_residual      )
      if(associated(csite%co2budget_initialstorage   ))                                    &
                                              deallocate(csite%co2budget_initialstorage   )
      if(associated(csite%co2budget_residual    )) deallocate(csite%co2budget_residual    )
      if(associated(csite%co2budget_loss2atm    )) deallocate(csite%co2budget_loss2atm    )
      if(associated(csite%co2budget_denseffect  )) deallocate(csite%co2budget_denseffect  )
      if(associated(csite%co2budget_gpp         )) deallocate(csite%co2budget_gpp         )
      if(associated(csite%co2budget_plresp      )) deallocate(csite%co2budget_plresp      )
      if(associated(csite%co2budget_rh          )) deallocate(csite%co2budget_rh          )
      if(associated(csite%today_A_decomp        )) deallocate(csite%today_A_decomp        )
      if(associated(csite%today_Af_decomp       )) deallocate(csite%today_Af_decomp       )
      if(associated(csite%repro                 )) deallocate(csite%repro                 )
      if(associated(csite%veg_rough             )) deallocate(csite%veg_rough             )
      if(associated(csite%veg_height            )) deallocate(csite%veg_height            )
      if(associated(csite%veg_displace          )) deallocate(csite%veg_displace          )
      if(associated(csite%fsc_in                )) deallocate(csite%fsc_in                )
      if(associated(csite%ssc_in                )) deallocate(csite%ssc_in                )
      if(associated(csite%ssl_in                )) deallocate(csite%ssl_in                )
      if(associated(csite%fsn_in                )) deallocate(csite%fsn_in                )
      if(associated(csite%total_plant_nitrogen_uptake))                                    &
                                              deallocate(csite%total_plant_nitrogen_uptake)
      if(associated(csite%mineralized_N_loss    )) deallocate(csite%mineralized_N_loss    )
      if(associated(csite%mineralized_N_input   )) deallocate(csite%mineralized_N_input   )
      if(associated(csite%rshort_g              )) deallocate(csite%rshort_g              )
      if(associated(csite%rshort_g_beam         )) deallocate(csite%rshort_g_beam         )
      if(associated(csite%rshort_g_diffuse      )) deallocate(csite%rshort_g_diffuse      )
      if(associated(csite%par_g                 )) deallocate(csite%par_g                 )
      if(associated(csite%par_g_beam            )) deallocate(csite%par_g_beam            )
      if(associated(csite%par_g_diffuse         )) deallocate(csite%par_g_diffuse         )
      if(associated(csite%par_b                 )) deallocate(csite%par_b                 )
      if(associated(csite%par_b_beam            )) deallocate(csite%par_b_beam            )
      if(associated(csite%par_b_diffuse         )) deallocate(csite%par_b_diffuse         )
      if(associated(csite%nir_b                 )) deallocate(csite%nir_b                 )
      if(associated(csite%nir_b_beam            )) deallocate(csite%nir_b_beam            )
      if(associated(csite%nir_b_diffuse         )) deallocate(csite%nir_b_diffuse         )
      if(associated(csite%rlong_g               )) deallocate(csite%rlong_g               )
      if(associated(csite%rlong_s               )) deallocate(csite%rlong_s               )
      if(associated(csite%albedo                )) deallocate(csite%albedo                )
      if(associated(csite%albedo_par            )) deallocate(csite%albedo_par            )
      if(associated(csite%albedo_nir            )) deallocate(csite%albedo_nir            )
      if(associated(csite%rlong_albedo          )) deallocate(csite%rlong_albedo          )
      if(associated(csite%rnet                  )) deallocate(csite%rnet                  )
      if(associated(csite%rlongup               )) deallocate(csite%rlongup               )
      if(associated(csite%parup                 )) deallocate(csite%parup                 )
      if(associated(csite%nirup                 )) deallocate(csite%nirup                 )
      if(associated(csite%rshortup              )) deallocate(csite%rshortup              )
      if(associated(csite%total_sfcw_depth      )) deallocate(csite%total_sfcw_depth      )
      if(associated(csite%snowfac               )) deallocate(csite%snowfac               )
      if(associated(csite%A_decomp              )) deallocate(csite%A_decomp              )
      if(associated(csite%f_decomp              )) deallocate(csite%f_decomp              )
      if(associated(csite%rh                    )) deallocate(csite%rh                    )
      if(associated(csite%cwd_rh                )) deallocate(csite%cwd_rh                )
      if(associated(csite%cumlai_profile        )) deallocate(csite%cumlai_profile        )
      if(associated(csite%plant_ag_biomass      )) deallocate(csite%plant_ag_biomass      )
      if(associated(csite%htry                  )) deallocate(csite%htry                  )
      if(associated(csite%hprev                 )) deallocate(csite%hprev                 )
      if(associated(csite%ustar                 )) deallocate(csite%ustar                 )
      if(associated(csite%tstar                 )) deallocate(csite%tstar                 )
      if(associated(csite%qstar                 )) deallocate(csite%qstar                 )
      if(associated(csite%cstar                 )) deallocate(csite%cstar                 )
      if(associated(csite%zeta                  )) deallocate(csite%zeta                  )
      if(associated(csite%ribulk                )) deallocate(csite%ribulk                )
      if(associated(csite%upwp                  )) deallocate(csite%upwp                  )
      if(associated(csite%qpwp                  )) deallocate(csite%qpwp                  )
      if(associated(csite%cpwp                  )) deallocate(csite%cpwp                  )
      if(associated(csite%tpwp                  )) deallocate(csite%tpwp                  )
      if(associated(csite%wpwp                  )) deallocate(csite%wpwp                  )
      if(associated(csite%watertable            )) deallocate(csite%watertable            )
      if(associated(csite%moist_dz              )) deallocate(csite%moist_dz              )
      if(associated(csite%ksat                  )) deallocate(csite%ksat                  )
      if(associated(csite%soil_sat_energy       )) deallocate(csite%soil_sat_energy       )
      if(associated(csite%soil_sat_water        )) deallocate(csite%soil_sat_water        )
      if(associated(csite%soil_sat_heat         )) deallocate(csite%soil_sat_heat         )
      if(associated(csite%runoff_A              )) deallocate(csite%runoff_A              )
      if(associated(csite%runoff_rate           )) deallocate(csite%runoff_rate           )
      if(associated(csite%runoff                )) deallocate(csite%runoff                )
      if(associated(csite%qrunoff               )) deallocate(csite%qrunoff               )
      if(associated(csite%fmean_rh              )) deallocate(csite%fmean_rh              )
      if(associated(csite%fmean_cwd_rh          )) deallocate(csite%fmean_cwd_rh          )
      if(associated(csite%fmean_nep             )) deallocate(csite%fmean_nep             )
      if(associated(csite%fmean_rk4step         )) deallocate(csite%fmean_rk4step         )
      if(associated(csite%fmean_available_water )) deallocate(csite%fmean_available_water )
      if(associated(csite%fmean_can_theiv       )) deallocate(csite%fmean_can_theiv       )
      if(associated(csite%fmean_can_theta       )) deallocate(csite%fmean_can_theta       )
      if(associated(csite%fmean_can_vpdef       )) deallocate(csite%fmean_can_vpdef       )
      if(associated(csite%fmean_can_temp        )) deallocate(csite%fmean_can_temp        )
      if(associated(csite%fmean_can_shv         )) deallocate(csite%fmean_can_shv         )
      if(associated(csite%fmean_can_co2         )) deallocate(csite%fmean_can_co2         )
      if(associated(csite%fmean_can_rhos        )) deallocate(csite%fmean_can_rhos        )
      if(associated(csite%fmean_can_prss        )) deallocate(csite%fmean_can_prss        )
      if(associated(csite%fmean_gnd_temp        )) deallocate(csite%fmean_gnd_temp        )
      if(associated(csite%fmean_gnd_shv         )) deallocate(csite%fmean_gnd_shv         )
      if(associated(csite%fmean_can_ggnd        )) deallocate(csite%fmean_can_ggnd        )
      if(associated(csite%fmean_sfcw_depth      )) deallocate(csite%fmean_sfcw_depth      )
      if(associated(csite%fmean_sfcw_energy     )) deallocate(csite%fmean_sfcw_energy     )
      if(associated(csite%fmean_sfcw_mass       )) deallocate(csite%fmean_sfcw_mass       )
      if(associated(csite%fmean_sfcw_temp       )) deallocate(csite%fmean_sfcw_temp       )
      if(associated(csite%fmean_sfcw_fliq       )) deallocate(csite%fmean_sfcw_fliq       )
      if(associated(csite%fmean_soil_energy     )) deallocate(csite%fmean_soil_energy     )
      if(associated(csite%fmean_soil_mstpot     )) deallocate(csite%fmean_soil_mstpot     )
      if(associated(csite%fmean_soil_water      )) deallocate(csite%fmean_soil_water      )
      if(associated(csite%fmean_soil_temp       )) deallocate(csite%fmean_soil_temp       )
      if(associated(csite%fmean_soil_fliq       )) deallocate(csite%fmean_soil_fliq       )
      if(associated(csite%fmean_rshort_gnd      )) deallocate(csite%fmean_rshort_gnd      )
      if(associated(csite%fmean_par_gnd         )) deallocate(csite%fmean_par_gnd         )
      if(associated(csite%fmean_rlong_gnd       )) deallocate(csite%fmean_rlong_gnd       )
      if(associated(csite%fmean_rlongup         )) deallocate(csite%fmean_rlongup         )
      if(associated(csite%fmean_parup           )) deallocate(csite%fmean_parup           )
      if(associated(csite%fmean_nirup           )) deallocate(csite%fmean_nirup           )
      if(associated(csite%fmean_rshortup        )) deallocate(csite%fmean_rshortup        )
      if(associated(csite%fmean_rnet            )) deallocate(csite%fmean_rnet            )
      if(associated(csite%fmean_albedo          )) deallocate(csite%fmean_albedo          )
      if(associated(csite%fmean_albedo_par      )) deallocate(csite%fmean_albedo_par      )
      if(associated(csite%fmean_albedo_nir      )) deallocate(csite%fmean_albedo_nir      )
      if(associated(csite%fmean_rlong_albedo    )) deallocate(csite%fmean_rlong_albedo    )
      if(associated(csite%fmean_ustar           )) deallocate(csite%fmean_ustar           )
      if(associated(csite%fmean_tstar           )) deallocate(csite%fmean_tstar           )
      if(associated(csite%fmean_qstar           )) deallocate(csite%fmean_qstar           )
      if(associated(csite%fmean_cstar           )) deallocate(csite%fmean_cstar           )
      if(associated(csite%fmean_carbon_ac       )) deallocate(csite%fmean_carbon_ac       )
      if(associated(csite%fmean_carbon_st       )) deallocate(csite%fmean_carbon_st       )
      if(associated(csite%fmean_vapor_gc        )) deallocate(csite%fmean_vapor_gc        )
      if(associated(csite%fmean_vapor_ac        )) deallocate(csite%fmean_vapor_ac        )
      if(associated(csite%fmean_smoist_gg       )) deallocate(csite%fmean_smoist_gg       )
      if(associated(csite%fmean_throughfall     )) deallocate(csite%fmean_throughfall     )
      if(associated(csite%fmean_transloss       )) deallocate(csite%fmean_transloss       )
      if(associated(csite%fmean_runoff          )) deallocate(csite%fmean_runoff          )
      if(associated(csite%fmean_drainage        )) deallocate(csite%fmean_drainage        )
      if(associated(csite%fmean_sensible_gc     )) deallocate(csite%fmean_sensible_gc     )
      if(associated(csite%fmean_sensible_ac     )) deallocate(csite%fmean_sensible_ac     )
      if(associated(csite%fmean_sensible_gg     )) deallocate(csite%fmean_sensible_gg     )
      if(associated(csite%fmean_qthroughfall    )) deallocate(csite%fmean_qthroughfall    )
      if(associated(csite%fmean_qrunoff         )) deallocate(csite%fmean_qrunoff         )
      if(associated(csite%fmean_qdrainage       )) deallocate(csite%fmean_qdrainage       )
      if(associated(csite%dmean_A_decomp        )) deallocate(csite%dmean_A_decomp        )
      if(associated(csite%dmean_Af_decomp       )) deallocate(csite%dmean_Af_decomp       )
      if(associated(csite%dmean_co2_residual    )) deallocate(csite%dmean_co2_residual    )
      if(associated(csite%dmean_energy_residual )) deallocate(csite%dmean_energy_residual )
      if(associated(csite%dmean_water_residual  )) deallocate(csite%dmean_water_residual  )
      if(associated(csite%dmean_rh              )) deallocate(csite%dmean_rh              )
      if(associated(csite%dmean_cwd_rh          )) deallocate(csite%dmean_cwd_rh          )
      if(associated(csite%dmean_nep             )) deallocate(csite%dmean_nep             )
      if(associated(csite%dmean_rk4step         )) deallocate(csite%dmean_rk4step         )
      if(associated(csite%dmean_available_water )) deallocate(csite%dmean_available_water )
      if(associated(csite%dmean_can_theiv       )) deallocate(csite%dmean_can_theiv       )
      if(associated(csite%dmean_can_theta       )) deallocate(csite%dmean_can_theta       )
      if(associated(csite%dmean_can_vpdef       )) deallocate(csite%dmean_can_vpdef       )
      if(associated(csite%dmean_can_temp        )) deallocate(csite%dmean_can_temp        )
      if(associated(csite%dmean_can_shv         )) deallocate(csite%dmean_can_shv         )
      if(associated(csite%dmean_can_co2         )) deallocate(csite%dmean_can_co2         )
      if(associated(csite%dmean_can_rhos        )) deallocate(csite%dmean_can_rhos        )
      if(associated(csite%dmean_can_prss        )) deallocate(csite%dmean_can_prss        )
      if(associated(csite%dmean_gnd_temp        )) deallocate(csite%dmean_gnd_temp        )
      if(associated(csite%dmean_gnd_shv         )) deallocate(csite%dmean_gnd_shv         )
      if(associated(csite%dmean_can_ggnd        )) deallocate(csite%dmean_can_ggnd        )
      if(associated(csite%dmean_sfcw_depth      )) deallocate(csite%dmean_sfcw_depth      )
      if(associated(csite%dmean_sfcw_energy     )) deallocate(csite%dmean_sfcw_energy     )
      if(associated(csite%dmean_sfcw_mass       )) deallocate(csite%dmean_sfcw_mass       )
      if(associated(csite%dmean_sfcw_temp       )) deallocate(csite%dmean_sfcw_temp       )
      if(associated(csite%dmean_sfcw_fliq       )) deallocate(csite%dmean_sfcw_fliq       )
      if(associated(csite%dmean_soil_energy     )) deallocate(csite%dmean_soil_energy     )
      if(associated(csite%dmean_soil_mstpot     )) deallocate(csite%dmean_soil_mstpot     )
      if(associated(csite%dmean_soil_water      )) deallocate(csite%dmean_soil_water      )
      if(associated(csite%dmean_soil_temp       )) deallocate(csite%dmean_soil_temp       )
      if(associated(csite%dmean_soil_fliq       )) deallocate(csite%dmean_soil_fliq       )
      if(associated(csite%dmean_rshort_gnd      )) deallocate(csite%dmean_rshort_gnd      )
      if(associated(csite%dmean_par_gnd         )) deallocate(csite%dmean_par_gnd         )
      if(associated(csite%dmean_rlong_gnd       )) deallocate(csite%dmean_rlong_gnd       )
      if(associated(csite%dmean_rlongup         )) deallocate(csite%dmean_rlongup         )
      if(associated(csite%dmean_parup           )) deallocate(csite%dmean_parup           )
      if(associated(csite%dmean_nirup           )) deallocate(csite%dmean_nirup           )
      if(associated(csite%dmean_rshortup        )) deallocate(csite%dmean_rshortup        )
      if(associated(csite%dmean_rnet            )) deallocate(csite%dmean_rnet            )
      if(associated(csite%dmean_albedo          )) deallocate(csite%dmean_albedo          )
      if(associated(csite%dmean_albedo_par      )) deallocate(csite%dmean_albedo_par      )
      if(associated(csite%dmean_albedo_nir      )) deallocate(csite%dmean_albedo_nir      )
      if(associated(csite%dmean_rlong_albedo    )) deallocate(csite%dmean_rlong_albedo    )
      if(associated(csite%dmean_ustar           )) deallocate(csite%dmean_ustar           )
      if(associated(csite%dmean_tstar           )) deallocate(csite%dmean_tstar           )
      if(associated(csite%dmean_qstar           )) deallocate(csite%dmean_qstar           )
      if(associated(csite%dmean_cstar           )) deallocate(csite%dmean_cstar           )
      if(associated(csite%dmean_carbon_ac       )) deallocate(csite%dmean_carbon_ac       )
      if(associated(csite%dmean_carbon_st       )) deallocate(csite%dmean_carbon_st       )
      if(associated(csite%dmean_vapor_gc        )) deallocate(csite%dmean_vapor_gc        )
      if(associated(csite%dmean_vapor_ac        )) deallocate(csite%dmean_vapor_ac        )
      if(associated(csite%dmean_smoist_gg       )) deallocate(csite%dmean_smoist_gg       )
      if(associated(csite%dmean_throughfall     )) deallocate(csite%dmean_throughfall     )
      if(associated(csite%dmean_transloss       )) deallocate(csite%dmean_transloss       )
      if(associated(csite%dmean_runoff          )) deallocate(csite%dmean_runoff          )
      if(associated(csite%dmean_drainage        )) deallocate(csite%dmean_drainage        )
      if(associated(csite%dmean_sensible_gc     )) deallocate(csite%dmean_sensible_gc     )
      if(associated(csite%dmean_sensible_ac     )) deallocate(csite%dmean_sensible_ac     )
      if(associated(csite%dmean_sensible_gg     )) deallocate(csite%dmean_sensible_gg     )
      if(associated(csite%dmean_qthroughfall    )) deallocate(csite%dmean_qthroughfall    )
      if(associated(csite%dmean_qrunoff         )) deallocate(csite%dmean_qrunoff         )
      if(associated(csite%dmean_qdrainage       )) deallocate(csite%dmean_qdrainage       )
      if(associated(csite%mmean_fast_soil_c     )) deallocate(csite%mmean_fast_soil_c     )
      if(associated(csite%mmean_slow_soil_c     )) deallocate(csite%mmean_slow_soil_c     )
      if(associated(csite%mmean_struct_soil_c   )) deallocate(csite%mmean_struct_soil_c   )
      if(associated(csite%mmean_struct_soil_l   )) deallocate(csite%mmean_struct_soil_l   )
      if(associated(csite%mmean_fast_soil_n     )) deallocate(csite%mmean_fast_soil_n     )
      if(associated(csite%mmean_mineral_soil_n  )) deallocate(csite%mmean_mineral_soil_n  )
      if(associated(csite%mmean_co2_residual    )) deallocate(csite%mmean_co2_residual    )
      if(associated(csite%mmean_energy_residual )) deallocate(csite%mmean_energy_residual )
      if(associated(csite%mmean_water_residual  )) deallocate(csite%mmean_water_residual  )
      if(associated(csite%mmean_rh              )) deallocate(csite%mmean_rh              )
      if(associated(csite%mmean_cwd_rh          )) deallocate(csite%mmean_cwd_rh          )
      if(associated(csite%mmean_nep             )) deallocate(csite%mmean_nep             )
      if(associated(csite%mmean_A_decomp        )) deallocate(csite%mmean_A_decomp        )
      if(associated(csite%mmean_Af_decomp       )) deallocate(csite%mmean_Af_decomp       )
      if(associated(csite%mmean_rk4step         )) deallocate(csite%mmean_rk4step         )
      if(associated(csite%mmean_available_water )) deallocate(csite%mmean_available_water )
      if(associated(csite%mmean_can_theiv       )) deallocate(csite%mmean_can_theiv       )
      if(associated(csite%mmean_can_theta       )) deallocate(csite%mmean_can_theta       )
      if(associated(csite%mmean_can_vpdef       )) deallocate(csite%mmean_can_vpdef       )
      if(associated(csite%mmean_can_temp        )) deallocate(csite%mmean_can_temp        )
      if(associated(csite%mmean_can_shv         )) deallocate(csite%mmean_can_shv         )
      if(associated(csite%mmean_can_co2         )) deallocate(csite%mmean_can_co2         )
      if(associated(csite%mmean_can_rhos        )) deallocate(csite%mmean_can_rhos        )
      if(associated(csite%mmean_can_prss        )) deallocate(csite%mmean_can_prss        )
      if(associated(csite%mmean_gnd_temp        )) deallocate(csite%mmean_gnd_temp        )
      if(associated(csite%mmean_gnd_shv         )) deallocate(csite%mmean_gnd_shv         )
      if(associated(csite%mmean_can_ggnd        )) deallocate(csite%mmean_can_ggnd        )
      if(associated(csite%mmean_sfcw_depth      )) deallocate(csite%mmean_sfcw_depth      )
      if(associated(csite%mmean_sfcw_energy     )) deallocate(csite%mmean_sfcw_energy     )
      if(associated(csite%mmean_sfcw_mass       )) deallocate(csite%mmean_sfcw_mass       )
      if(associated(csite%mmean_sfcw_temp       )) deallocate(csite%mmean_sfcw_temp       )
      if(associated(csite%mmean_sfcw_fliq       )) deallocate(csite%mmean_sfcw_fliq       )
      if(associated(csite%mmean_soil_energy     )) deallocate(csite%mmean_soil_energy     )
      if(associated(csite%mmean_soil_mstpot     )) deallocate(csite%mmean_soil_mstpot     )
      if(associated(csite%mmean_soil_water      )) deallocate(csite%mmean_soil_water      )
      if(associated(csite%mmean_soil_temp       )) deallocate(csite%mmean_soil_temp       )
      if(associated(csite%mmean_soil_fliq       )) deallocate(csite%mmean_soil_fliq       )
      if(associated(csite%mmean_rshort_gnd      )) deallocate(csite%mmean_rshort_gnd      )
      if(associated(csite%mmean_par_gnd         )) deallocate(csite%mmean_par_gnd         )
      if(associated(csite%mmean_rlong_gnd       )) deallocate(csite%mmean_rlong_gnd       )
      if(associated(csite%mmean_rlongup         )) deallocate(csite%mmean_rlongup         )
      if(associated(csite%mmean_parup           )) deallocate(csite%mmean_parup           )
      if(associated(csite%mmean_nirup           )) deallocate(csite%mmean_nirup           )
      if(associated(csite%mmean_rshortup        )) deallocate(csite%mmean_rshortup        )
      if(associated(csite%mmean_rnet            )) deallocate(csite%mmean_rnet            )
      if(associated(csite%mmean_albedo          )) deallocate(csite%mmean_albedo          )
      if(associated(csite%mmean_albedo_par      )) deallocate(csite%mmean_albedo_par      )
      if(associated(csite%mmean_albedo_nir      )) deallocate(csite%mmean_albedo_nir      )
      if(associated(csite%mmean_rlong_albedo    )) deallocate(csite%mmean_rlong_albedo    )
      if(associated(csite%mmean_ustar           )) deallocate(csite%mmean_ustar           )
      if(associated(csite%mmean_tstar           )) deallocate(csite%mmean_tstar           )
      if(associated(csite%mmean_qstar           )) deallocate(csite%mmean_qstar           )
      if(associated(csite%mmean_cstar           )) deallocate(csite%mmean_cstar           )
      if(associated(csite%mmean_carbon_ac       )) deallocate(csite%mmean_carbon_ac       )
      if(associated(csite%mmean_carbon_st       )) deallocate(csite%mmean_carbon_st       )
      if(associated(csite%mmean_vapor_gc        )) deallocate(csite%mmean_vapor_gc        )
      if(associated(csite%mmean_vapor_ac        )) deallocate(csite%mmean_vapor_ac        )
      if(associated(csite%mmean_smoist_gg       )) deallocate(csite%mmean_smoist_gg       )
      if(associated(csite%mmean_throughfall     )) deallocate(csite%mmean_throughfall     )
      if(associated(csite%mmean_transloss       )) deallocate(csite%mmean_transloss       )
      if(associated(csite%mmean_runoff          )) deallocate(csite%mmean_runoff          )
      if(associated(csite%mmean_drainage        )) deallocate(csite%mmean_drainage        )
      if(associated(csite%mmean_sensible_gc     )) deallocate(csite%mmean_sensible_gc     )
      if(associated(csite%mmean_sensible_ac     )) deallocate(csite%mmean_sensible_ac     )
      if(associated(csite%mmean_sensible_gg     )) deallocate(csite%mmean_sensible_gg     )
      if(associated(csite%mmean_qthroughfall    )) deallocate(csite%mmean_qthroughfall    )
      if(associated(csite%mmean_qrunoff         )) deallocate(csite%mmean_qrunoff         )
      if(associated(csite%mmean_qdrainage       )) deallocate(csite%mmean_qdrainage       )
      if(associated(csite%mmean_A_decomp        )) deallocate(csite%mmean_A_decomp        )
      if(associated(csite%mmean_Af_decomp       )) deallocate(csite%mmean_Af_decomp       )
      if(associated(csite%mmean_co2_residual    )) deallocate(csite%mmean_co2_residual    )
      if(associated(csite%mmean_energy_residual )) deallocate(csite%mmean_energy_residual )
      if(associated(csite%mmean_water_residual  )) deallocate(csite%mmean_water_residual  )
      if(associated(csite%mmsqu_rh              )) deallocate(csite%mmsqu_rh              )
      if(associated(csite%mmsqu_cwd_rh          )) deallocate(csite%mmsqu_cwd_rh          )
      if(associated(csite%mmsqu_nep             )) deallocate(csite%mmsqu_nep             )
      if(associated(csite%mmsqu_rlongup         )) deallocate(csite%mmsqu_rlongup         )
      if(associated(csite%mmsqu_parup           )) deallocate(csite%mmsqu_parup           )
      if(associated(csite%mmsqu_nirup           )) deallocate(csite%mmsqu_nirup           )
      if(associated(csite%mmsqu_rshortup        )) deallocate(csite%mmsqu_rshortup        )
      if(associated(csite%mmsqu_rnet            )) deallocate(csite%mmsqu_rnet            )
      if(associated(csite%mmsqu_albedo          )) deallocate(csite%mmsqu_albedo          )
      if(associated(csite%mmsqu_ustar           )) deallocate(csite%mmsqu_ustar           )
      if(associated(csite%mmsqu_carbon_ac       )) deallocate(csite%mmsqu_carbon_ac       )
      if(associated(csite%mmsqu_carbon_st       )) deallocate(csite%mmsqu_carbon_st       )
      if(associated(csite%mmsqu_vapor_gc        )) deallocate(csite%mmsqu_vapor_gc        )
      if(associated(csite%mmsqu_vapor_ac        )) deallocate(csite%mmsqu_vapor_ac        )
      if(associated(csite%mmsqu_sensible_gc     )) deallocate(csite%mmsqu_sensible_gc     )
      if(associated(csite%mmsqu_sensible_ac     )) deallocate(csite%mmsqu_sensible_ac     )
      if(associated(csite%qmean_rh              )) deallocate(csite%qmean_rh              )
      if(associated(csite%qmean_cwd_rh          )) deallocate(csite%qmean_cwd_rh          )
      if(associated(csite%qmean_nep             )) deallocate(csite%qmean_nep             )
      if(associated(csite%qmean_rk4step         )) deallocate(csite%qmean_rk4step         )
      if(associated(csite%qmean_available_water )) deallocate(csite%qmean_available_water )
      if(associated(csite%qmean_can_theiv       )) deallocate(csite%qmean_can_theiv       )
      if(associated(csite%qmean_can_theta       )) deallocate(csite%qmean_can_theta       )
      if(associated(csite%qmean_can_vpdef       )) deallocate(csite%qmean_can_vpdef       )
      if(associated(csite%qmean_can_temp        )) deallocate(csite%qmean_can_temp        )
      if(associated(csite%qmean_can_shv         )) deallocate(csite%qmean_can_shv         )
      if(associated(csite%qmean_can_co2         )) deallocate(csite%qmean_can_co2         )
      if(associated(csite%qmean_can_rhos        )) deallocate(csite%qmean_can_rhos        )
      if(associated(csite%qmean_can_prss        )) deallocate(csite%qmean_can_prss        )
      if(associated(csite%qmean_gnd_temp        )) deallocate(csite%qmean_gnd_temp        )
      if(associated(csite%qmean_gnd_shv         )) deallocate(csite%qmean_gnd_shv         )
      if(associated(csite%qmean_can_ggnd        )) deallocate(csite%qmean_can_ggnd        )
      if(associated(csite%qmean_sfcw_depth      )) deallocate(csite%qmean_sfcw_depth      )
      if(associated(csite%qmean_sfcw_energy     )) deallocate(csite%qmean_sfcw_energy     )
      if(associated(csite%qmean_sfcw_mass       )) deallocate(csite%qmean_sfcw_mass       )
      if(associated(csite%qmean_sfcw_temp       )) deallocate(csite%qmean_sfcw_temp       )
      if(associated(csite%qmean_sfcw_fliq       )) deallocate(csite%qmean_sfcw_fliq       )
      if(associated(csite%qmean_soil_energy     )) deallocate(csite%qmean_soil_energy     )
      if(associated(csite%qmean_soil_mstpot     )) deallocate(csite%qmean_soil_mstpot     )
      if(associated(csite%qmean_soil_water      )) deallocate(csite%qmean_soil_water      )
      if(associated(csite%qmean_soil_temp       )) deallocate(csite%qmean_soil_temp       )
      if(associated(csite%qmean_soil_fliq       )) deallocate(csite%qmean_soil_fliq       )
      if(associated(csite%qmean_rshort_gnd      )) deallocate(csite%qmean_rshort_gnd      )
      if(associated(csite%qmean_par_gnd         )) deallocate(csite%qmean_par_gnd         )
      if(associated(csite%qmean_rlong_gnd       )) deallocate(csite%qmean_rlong_gnd       )
      if(associated(csite%qmean_rlongup         )) deallocate(csite%qmean_rlongup         )
      if(associated(csite%qmean_parup           )) deallocate(csite%qmean_parup           )
      if(associated(csite%qmean_nirup           )) deallocate(csite%qmean_nirup           )
      if(associated(csite%qmean_rshortup        )) deallocate(csite%qmean_rshortup        )
      if(associated(csite%qmean_rnet            )) deallocate(csite%qmean_rnet            )
      if(associated(csite%qmean_albedo          )) deallocate(csite%qmean_albedo          )
      if(associated(csite%qmean_albedo_par      )) deallocate(csite%qmean_albedo_par      )
      if(associated(csite%qmean_albedo_nir      )) deallocate(csite%qmean_albedo_nir      )
      if(associated(csite%qmean_rlong_albedo    )) deallocate(csite%qmean_rlong_albedo    )
      if(associated(csite%qmean_ustar           )) deallocate(csite%qmean_ustar           )
      if(associated(csite%qmean_tstar           )) deallocate(csite%qmean_tstar           )
      if(associated(csite%qmean_qstar           )) deallocate(csite%qmean_qstar           )
      if(associated(csite%qmean_cstar           )) deallocate(csite%qmean_cstar           )
      if(associated(csite%qmean_carbon_ac       )) deallocate(csite%qmean_carbon_ac       )
      if(associated(csite%qmean_carbon_st       )) deallocate(csite%qmean_carbon_st       )
      if(associated(csite%qmean_vapor_gc        )) deallocate(csite%qmean_vapor_gc        )
      if(associated(csite%qmean_vapor_ac        )) deallocate(csite%qmean_vapor_ac        )
      if(associated(csite%qmean_smoist_gg       )) deallocate(csite%qmean_smoist_gg       )
      if(associated(csite%qmean_throughfall     )) deallocate(csite%qmean_throughfall     )
      if(associated(csite%qmean_transloss       )) deallocate(csite%qmean_transloss       )
      if(associated(csite%qmean_runoff          )) deallocate(csite%qmean_runoff          )
      if(associated(csite%qmean_drainage        )) deallocate(csite%qmean_drainage        )
      if(associated(csite%qmean_sensible_gc     )) deallocate(csite%qmean_sensible_gc     )
      if(associated(csite%qmean_sensible_ac     )) deallocate(csite%qmean_sensible_ac     )
      if(associated(csite%qmean_sensible_gg     )) deallocate(csite%qmean_sensible_gg     )
      if(associated(csite%qmean_qthroughfall    )) deallocate(csite%qmean_qthroughfall    )
      if(associated(csite%qmean_qrunoff         )) deallocate(csite%qmean_qrunoff         )
      if(associated(csite%qmean_qdrainage       )) deallocate(csite%qmean_qdrainage       )
      if(associated(csite%qmsqu_rh              )) deallocate(csite%qmsqu_rh              )
      if(associated(csite%qmsqu_cwd_rh          )) deallocate(csite%qmsqu_cwd_rh          )
      if(associated(csite%qmsqu_nep             )) deallocate(csite%qmsqu_nep             )
      if(associated(csite%qmsqu_rlongup         )) deallocate(csite%qmsqu_rlongup         )
      if(associated(csite%qmsqu_parup           )) deallocate(csite%qmsqu_parup           )
      if(associated(csite%qmsqu_nirup           )) deallocate(csite%qmsqu_nirup           )
      if(associated(csite%qmsqu_rshortup        )) deallocate(csite%qmsqu_rshortup        )
      if(associated(csite%qmsqu_rnet            )) deallocate(csite%qmsqu_rnet            )
      if(associated(csite%qmsqu_albedo          )) deallocate(csite%qmsqu_albedo          )
      if(associated(csite%qmsqu_ustar           )) deallocate(csite%qmsqu_ustar           )
      if(associated(csite%qmsqu_carbon_ac       )) deallocate(csite%qmsqu_carbon_ac       )
      if(associated(csite%qmsqu_carbon_st       )) deallocate(csite%qmsqu_carbon_st       )
      if(associated(csite%qmsqu_vapor_gc        )) deallocate(csite%qmsqu_vapor_gc        )
      if(associated(csite%qmsqu_vapor_ac        )) deallocate(csite%qmsqu_vapor_ac        )
      if(associated(csite%qmsqu_sensible_gc     )) deallocate(csite%qmsqu_sensible_gc     )
      if(associated(csite%qmsqu_sensible_ac     )) deallocate(csite%qmsqu_sensible_ac     )

      return
   end subroutine deallocate_sitetype
   !=======================================================================================!
   !=======================================================================================!





   !=======================================================================================!
   !=======================================================================================!
   !     This sub-routine de-allocates all patch pointers.                                 !
   !---------------------------------------------------------------------------------------!
   subroutine deallocate_patchtype(cpatch)

      implicit none

      !----- Arguments. -------------------------------------------------------------------!
      type(patchtype)  , target :: cpatch
      !----- Local variables. -------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
    
      if (cpatch%ncohorts == 0) return

      if(associated(cpatch%pft                 )) deallocate(cpatch%pft                 )
      if(associated(cpatch%nplant              )) deallocate(cpatch%nplant              )
      if(associated(cpatch%phenology_status    )) deallocate(cpatch%phenology_status    )
      if(associated(cpatch%recruit_dbh         )) deallocate(cpatch%recruit_dbh         )
      if(associated(cpatch%census_status       )) deallocate(cpatch%census_status       )
      if(associated(cpatch%hite                )) deallocate(cpatch%hite                )
      if(associated(cpatch%agb                 )) deallocate(cpatch%agb                 )
      if(associated(cpatch%basarea             )) deallocate(cpatch%basarea             )
      if(associated(cpatch%dagb_dt             )) deallocate(cpatch%dagb_dt             )
      if(associated(cpatch%dlnagb_dt           )) deallocate(cpatch%dlnagb_dt           )
      if(associated(cpatch%dba_dt              )) deallocate(cpatch%dba_dt              )
      if(associated(cpatch%dlnba_dt            )) deallocate(cpatch%dlnba_dt            )
      if(associated(cpatch%ddbh_dt             )) deallocate(cpatch%ddbh_dt             )
      if(associated(cpatch%dlndbh_dt           )) deallocate(cpatch%dlndbh_dt           )
      if(associated(cpatch%dbh                 )) deallocate(cpatch%dbh                 )
      if(associated(cpatch%bdead               )) deallocate(cpatch%bdead               )
      if(associated(cpatch%bleaf               )) deallocate(cpatch%bleaf               )
      if(associated(cpatch%balive              )) deallocate(cpatch%balive              )
      if(associated(cpatch%broot               )) deallocate(cpatch%broot               )
      if(associated(cpatch%bsapwooda           )) deallocate(cpatch%bsapwooda           )
      if(associated(cpatch%bsapwoodb           )) deallocate(cpatch%bsapwoodb           )
      if(associated(cpatch%bstorage            )) deallocate(cpatch%bstorage            )
      if(associated(cpatch%bseeds              )) deallocate(cpatch%bseeds              )
      if(associated(cpatch%lai                 )) deallocate(cpatch%lai                 )
      if(associated(cpatch%wai                 )) deallocate(cpatch%wai                 )
      if(associated(cpatch%crown_area          )) deallocate(cpatch%crown_area          )
      if(associated(cpatch%leaf_resolvable     )) deallocate(cpatch%leaf_resolvable     )
      if(associated(cpatch%wood_resolvable     )) deallocate(cpatch%wood_resolvable     )
      if(associated(cpatch%cb                  )) deallocate(cpatch%cb                  )
      if(associated(cpatch%cb_lightmax         )) deallocate(cpatch%cb_lightmax         )
      if(associated(cpatch%cb_moistmax         )) deallocate(cpatch%cb_moistmax         )
      if(associated(cpatch%cb_mlmax            )) deallocate(cpatch%cb_mlmax            )
      if(associated(cpatch%cbr_bar             )) deallocate(cpatch%cbr_bar             )
      if(associated(cpatch%leaf_energy         )) deallocate(cpatch%leaf_energy         )
      if(associated(cpatch%leaf_temp           )) deallocate(cpatch%leaf_temp           )
      if(associated(cpatch%leaf_vpdef          )) deallocate(cpatch%leaf_vpdef          )
      if(associated(cpatch%leaf_temp_pv        )) deallocate(cpatch%leaf_temp_pv        )
      if(associated(cpatch%leaf_hcap           )) deallocate(cpatch%leaf_hcap           )
      if(associated(cpatch%leaf_fliq           )) deallocate(cpatch%leaf_fliq           )
      if(associated(cpatch%leaf_water          )) deallocate(cpatch%leaf_water          )
      if(associated(cpatch%wood_energy         )) deallocate(cpatch%wood_energy         )
      if(associated(cpatch%wood_temp           )) deallocate(cpatch%wood_temp           )
      if(associated(cpatch%wood_temp_pv        )) deallocate(cpatch%wood_temp_pv        )
      if(associated(cpatch%wood_hcap           )) deallocate(cpatch%wood_hcap           )
      if(associated(cpatch%wood_fliq           )) deallocate(cpatch%wood_fliq           )
      if(associated(cpatch%wood_water          )) deallocate(cpatch%wood_water          )
      if(associated(cpatch%veg_wind            )) deallocate(cpatch%veg_wind            )
      if(associated(cpatch%lsfc_shv_open       )) deallocate(cpatch%lsfc_shv_open       )
      if(associated(cpatch%lsfc_shv_closed     )) deallocate(cpatch%lsfc_shv_closed     )
      if(associated(cpatch%lsfc_co2_open       )) deallocate(cpatch%lsfc_co2_open       )
      if(associated(cpatch%lsfc_co2_closed     )) deallocate(cpatch%lsfc_co2_closed     )
      if(associated(cpatch%lint_shv            )) deallocate(cpatch%lint_shv            )
      if(associated(cpatch%lint_co2_open       )) deallocate(cpatch%lint_co2_open       )
      if(associated(cpatch%lint_co2_closed     )) deallocate(cpatch%lint_co2_closed     )
      if(associated(cpatch%today_leaf_resp     )) deallocate(cpatch%today_leaf_resp     )
      if(associated(cpatch%today_root_resp     )) deallocate(cpatch%today_root_resp     )
      if(associated(cpatch%today_gpp           )) deallocate(cpatch%today_gpp           )
      if(associated(cpatch%today_gpp_pot       )) deallocate(cpatch%today_gpp_pot       )
      if(associated(cpatch%today_gpp_lightmax  )) deallocate(cpatch%today_gpp_lightmax  )
      if(associated(cpatch%today_gpp_moistmax  )) deallocate(cpatch%today_gpp_moistmax  )
      if(associated(cpatch%today_gpp_mlmax     )) deallocate(cpatch%today_gpp_mlmax     )
      if(associated(cpatch%today_nppleaf       )) deallocate(cpatch%today_nppleaf       )
      if(associated(cpatch%today_nppfroot      )) deallocate(cpatch%today_nppfroot      )
      if(associated(cpatch%today_nppsapwood    )) deallocate(cpatch%today_nppsapwood    )
      if(associated(cpatch%today_nppcroot      )) deallocate(cpatch%today_nppcroot      )
      if(associated(cpatch%today_nppseeds      )) deallocate(cpatch%today_nppseeds      )
      if(associated(cpatch%today_nppwood       )) deallocate(cpatch%today_nppwood       )
      if(associated(cpatch%today_nppdaily      )) deallocate(cpatch%today_nppdaily      )
      if(associated(cpatch%leaf_growth_resp    )) deallocate(cpatch%leaf_growth_resp    )
      if(associated(cpatch%root_growth_resp    )) deallocate(cpatch%root_growth_resp    )
      if(associated(cpatch%sapa_growth_resp    )) deallocate(cpatch%sapa_growth_resp    )
      if(associated(cpatch%sapb_growth_resp    )) deallocate(cpatch%sapb_growth_resp    )
      if(associated(cpatch%leaf_storage_resp   )) deallocate(cpatch%leaf_storage_resp   )
      if(associated(cpatch%root_storage_resp   )) deallocate(cpatch%root_storage_resp   )
      if(associated(cpatch%sapa_storage_resp   )) deallocate(cpatch%sapa_storage_resp   )
      if(associated(cpatch%sapb_storage_resp   )) deallocate(cpatch%sapb_storage_resp   )
      if(associated(cpatch%monthly_dndt        )) deallocate(cpatch%monthly_dndt        )
      if(associated(cpatch%monthly_dlnndt      )) deallocate(cpatch%monthly_dlnndt      )
      if(associated(cpatch%mort_rate           )) deallocate(cpatch%mort_rate           )
      if(associated(cpatch%krdepth             )) deallocate(cpatch%krdepth             )
      if(associated(cpatch%first_census        )) deallocate(cpatch%first_census        )
      if(associated(cpatch%new_recruit_flag    )) deallocate(cpatch%new_recruit_flag    )
      if(associated(cpatch%light_level         )) deallocate(cpatch%light_level         )
      if(associated(cpatch%light_level_beam    )) deallocate(cpatch%light_level_beam    )
      if(associated(cpatch%light_level_diff    )) deallocate(cpatch%light_level_diff    )

      if(associated(cpatch%par_level_beam      )) deallocate(cpatch%par_level_beam      )
      if(associated(cpatch%par_level_diffu     )) deallocate(cpatch%par_level_diffu     )
      if(associated(cpatch%par_level_diffd     )) deallocate(cpatch%par_level_diffd     )

      if(associated(cpatch%par_l               )) deallocate(cpatch%par_l               )
      if(associated(cpatch%par_l_beam          )) deallocate(cpatch%par_l_beam          )
      if(associated(cpatch%par_l_diffuse       )) deallocate(cpatch%par_l_diffuse       )
      if(associated(cpatch%rshort_l            )) deallocate(cpatch%rshort_l            )
      if(associated(cpatch%rshort_l_beam       )) deallocate(cpatch%rshort_l_beam       )
      if(associated(cpatch%rshort_l_diffuse    )) deallocate(cpatch%rshort_l_diffuse    )
      if(associated(cpatch%rlong_l             )) deallocate(cpatch%rlong_l             )
      if(associated(cpatch%rshort_w            )) deallocate(cpatch%rshort_w            )
      if(associated(cpatch%rshort_w_beam       )) deallocate(cpatch%rshort_w_beam       )
      if(associated(cpatch%rshort_w_diffuse    )) deallocate(cpatch%rshort_w_diffuse    )
      if(associated(cpatch%rlong_w             )) deallocate(cpatch%rlong_w             )
      if(associated(cpatch%rad_profile         )) deallocate(cpatch%rad_profile         )
      if(associated(cpatch%leaf_gbh            )) deallocate(cpatch%leaf_gbh            )
      if(associated(cpatch%leaf_gbw            )) deallocate(cpatch%leaf_gbw            )
      if(associated(cpatch%wood_gbh            )) deallocate(cpatch%wood_gbh            )
      if(associated(cpatch%wood_gbw            )) deallocate(cpatch%wood_gbw            )
      if(associated(cpatch%A_open              )) deallocate(cpatch%A_open              )
      if(associated(cpatch%A_closed            )) deallocate(cpatch%A_closed            )
      if(associated(cpatch%A_light             )) deallocate(cpatch%A_light             )
      if(associated(cpatch%A_rubp              )) deallocate(cpatch%A_rubp              )
      if(associated(cpatch%A_co2               )) deallocate(cpatch%A_co2               )
      if(associated(cpatch%psi_open            )) deallocate(cpatch%psi_open            )
      if(associated(cpatch%psi_closed          )) deallocate(cpatch%psi_closed          )
      if(associated(cpatch%gsw_open            )) deallocate(cpatch%gsw_open            )
      if(associated(cpatch%gsw_closed          )) deallocate(cpatch%gsw_closed          )
      if(associated(cpatch%leaf_gsw            )) deallocate(cpatch%leaf_gsw            )
      if(associated(cpatch%fsw                 )) deallocate(cpatch%fsw                 )
      if(associated(cpatch%fsn                 )) deallocate(cpatch%fsn                 )
      if(associated(cpatch%fs_open             )) deallocate(cpatch%fs_open             )
      if(associated(cpatch%water_supply        )) deallocate(cpatch%water_supply        )
      if(associated(cpatch%leaf_maintenance    )) deallocate(cpatch%leaf_maintenance    )
      if(associated(cpatch%root_maintenance    )) deallocate(cpatch%root_maintenance    )
      if(associated(cpatch%leaf_drop           )) deallocate(cpatch%leaf_drop           )
      if(associated(cpatch%leaf_respiration    )) deallocate(cpatch%leaf_respiration    )
      if(associated(cpatch%root_respiration    )) deallocate(cpatch%root_respiration    )
      if(associated(cpatch%gpp                 )) deallocate(cpatch%gpp                 )
      if(associated(cpatch%paw_avg             )) deallocate(cpatch%paw_avg             )
      if(associated(cpatch%elongf              )) deallocate(cpatch%elongf              )
      if(associated(cpatch%turnover_amp        )) deallocate(cpatch%turnover_amp        )
      if(associated(cpatch%llspan              )) deallocate(cpatch%llspan              )
      if(associated(cpatch%vm_bar              )) deallocate(cpatch%vm_bar              )
      if(associated(cpatch%sla                 )) deallocate(cpatch%sla                 )
      if(associated(cpatch%fmean_gpp           )) deallocate(cpatch%fmean_gpp           )
      if(associated(cpatch%fmean_npp           )) deallocate(cpatch%fmean_npp           )
      if(associated(cpatch%fmean_leaf_resp     )) deallocate(cpatch%fmean_leaf_resp     )
      if(associated(cpatch%fmean_root_resp     )) deallocate(cpatch%fmean_root_resp     )
      if(associated(cpatch%fmean_leaf_growth_resp)) deallocate(cpatch%fmean_leaf_growth_resp)
      if(associated(cpatch%fmean_root_growth_resp)) deallocate(cpatch%fmean_root_growth_resp)
      if(associated(cpatch%fmean_sapa_growth_resp)) deallocate(cpatch%fmean_sapa_growth_resp)
      if(associated(cpatch%fmean_sapb_growth_resp)) deallocate(cpatch%fmean_sapb_growth_resp)
      if(associated(cpatch%fmean_leaf_storage_resp)) deallocate(cpatch%fmean_leaf_storage_resp)
      if(associated(cpatch%fmean_root_storage_resp)) deallocate(cpatch%fmean_root_storage_resp)
      if(associated(cpatch%fmean_sapa_storage_resp)) deallocate(cpatch%fmean_sapa_storage_resp)
      if(associated(cpatch%fmean_sapb_storage_resp)) deallocate(cpatch%fmean_sapb_storage_resp)
      if(associated(cpatch%fmean_plresp        )) deallocate(cpatch%fmean_plresp        )
      if(associated(cpatch%fmean_leaf_energy   )) deallocate(cpatch%fmean_leaf_energy   )
      if(associated(cpatch%fmean_leaf_water    )) deallocate(cpatch%fmean_leaf_water    )
      if(associated(cpatch%fmean_leaf_hcap     )) deallocate(cpatch%fmean_leaf_hcap     )
      if(associated(cpatch%fmean_leaf_vpdef    )) deallocate(cpatch%fmean_leaf_vpdef    )
      if(associated(cpatch%fmean_leaf_temp     )) deallocate(cpatch%fmean_leaf_temp     )
      if(associated(cpatch%fmean_leaf_fliq     )) deallocate(cpatch%fmean_leaf_fliq     )
      if(associated(cpatch%fmean_leaf_gsw      )) deallocate(cpatch%fmean_leaf_gsw      )
      if(associated(cpatch%fmean_leaf_gbw      )) deallocate(cpatch%fmean_leaf_gbw      )
      if(associated(cpatch%fmean_wood_energy   )) deallocate(cpatch%fmean_wood_energy   )
      if(associated(cpatch%fmean_wood_water    )) deallocate(cpatch%fmean_wood_water    )
      if(associated(cpatch%fmean_wood_hcap     )) deallocate(cpatch%fmean_wood_hcap     )
      if(associated(cpatch%fmean_wood_temp     )) deallocate(cpatch%fmean_wood_temp     )
      if(associated(cpatch%fmean_wood_fliq     )) deallocate(cpatch%fmean_wood_fliq     )
      if(associated(cpatch%fmean_wood_gbw      )) deallocate(cpatch%fmean_wood_gbw      )
      if(associated(cpatch%fmean_fs_open       )) deallocate(cpatch%fmean_fs_open       )
      if(associated(cpatch%fmean_fsw           )) deallocate(cpatch%fmean_fsw           )
      if(associated(cpatch%fmean_fsn           )) deallocate(cpatch%fmean_fsn           )
      if(associated(cpatch%fmean_a_open        )) deallocate(cpatch%fmean_a_open        )
      if(associated(cpatch%fmean_a_closed      )) deallocate(cpatch%fmean_a_closed      )
      if(associated(cpatch%fmean_a_net         )) deallocate(cpatch%fmean_a_net         )
      if(associated(cpatch%fmean_A_light       )) deallocate(cpatch%fmean_A_light       )
      if(associated(cpatch%fmean_A_rubp        )) deallocate(cpatch%fmean_A_rubp        )
      if(associated(cpatch%fmean_A_co2         )) deallocate(cpatch%fmean_A_co2         )
      if(associated(cpatch%fmean_psi_open      )) deallocate(cpatch%fmean_psi_open      )
      if(associated(cpatch%fmean_psi_closed    )) deallocate(cpatch%fmean_psi_closed    )
      if(associated(cpatch%fmean_water_supply  )) deallocate(cpatch%fmean_water_supply  )
      if(associated(cpatch%fmean_light_level   )) deallocate(cpatch%fmean_light_level   )
      if(associated(cpatch%fmean_light_level_beam))                                        &
                                                deallocate(cpatch%fmean_light_level_beam)
      if(associated(cpatch%fmean_light_level_diff))                                        &
                                                deallocate(cpatch%fmean_light_level_diff)

      if(associated(cpatch%fmean_par_level_beam ))deallocate(cpatch%fmean_par_level_beam )
      if(associated(cpatch%fmean_par_level_diffu))deallocate(cpatch%fmean_par_level_diffu)
      if(associated(cpatch%fmean_par_level_diffd))deallocate(cpatch%fmean_par_level_diffd)

      if(associated(cpatch%fmean_par_l         )) deallocate(cpatch%fmean_par_l         )
      if(associated(cpatch%fmean_par_l_beam    )) deallocate(cpatch%fmean_par_l_beam    )
      if(associated(cpatch%fmean_par_l_diff    )) deallocate(cpatch%fmean_par_l_diff    )
      if(associated(cpatch%fmean_rshort_l      )) deallocate(cpatch%fmean_rshort_l      )
      if(associated(cpatch%fmean_rlong_l       )) deallocate(cpatch%fmean_rlong_l       )
      if(associated(cpatch%fmean_sensible_lc   )) deallocate(cpatch%fmean_sensible_lc   )
      if(associated(cpatch%fmean_vapor_lc      )) deallocate(cpatch%fmean_vapor_lc      )
      if(associated(cpatch%fmean_transp        )) deallocate(cpatch%fmean_transp        )
      if(associated(cpatch%fmean_intercepted_al)) deallocate(cpatch%fmean_intercepted_al)
      if(associated(cpatch%fmean_wshed_lg      )) deallocate(cpatch%fmean_wshed_lg      )
      if(associated(cpatch%fmean_rshort_w      )) deallocate(cpatch%fmean_rshort_w      )
      if(associated(cpatch%fmean_rlong_w       )) deallocate(cpatch%fmean_rlong_w       )
      if(associated(cpatch%fmean_rad_profile   )) deallocate(cpatch%fmean_rad_profile   )
      if(associated(cpatch%fmean_sensible_wc   )) deallocate(cpatch%fmean_sensible_wc   )
      if(associated(cpatch%fmean_vapor_wc      )) deallocate(cpatch%fmean_vapor_wc      )
      if(associated(cpatch%fmean_intercepted_aw)) deallocate(cpatch%fmean_intercepted_aw)
      if(associated(cpatch%fmean_wshed_wg      )) deallocate(cpatch%fmean_wshed_wg      )
      if(associated(cpatch%fmean_lai           )) deallocate(cpatch%fmean_lai           )
      if(associated(cpatch%fmean_bdead         )) deallocate(cpatch%fmean_bdead         )
      if(associated(cpatch%dmean_nppleaf       )) deallocate(cpatch%dmean_nppleaf       )
      if(associated(cpatch%dmean_nppfroot      )) deallocate(cpatch%dmean_nppfroot      )
      if(associated(cpatch%dmean_nppsapwood    )) deallocate(cpatch%dmean_nppsapwood    )
      if(associated(cpatch%dmean_nppcroot      )) deallocate(cpatch%dmean_nppcroot      )
      if(associated(cpatch%dmean_nppseeds      )) deallocate(cpatch%dmean_nppseeds      )
      if(associated(cpatch%dmean_nppwood       )) deallocate(cpatch%dmean_nppwood       )
      if(associated(cpatch%dmean_nppdaily      )) deallocate(cpatch%dmean_nppdaily      )
      if(associated(cpatch%dmean_gpp           )) deallocate(cpatch%dmean_gpp           )
      if(associated(cpatch%dmean_npp           )) deallocate(cpatch%dmean_npp           )
      if(associated(cpatch%dmean_leaf_resp     )) deallocate(cpatch%dmean_leaf_resp     )
      if(associated(cpatch%dmean_root_resp     )) deallocate(cpatch%dmean_root_resp     )
      if(associated(cpatch%dmean_leaf_growth_resp)) deallocate(cpatch%dmean_leaf_growth_resp)
      if(associated(cpatch%dmean_root_growth_resp)) deallocate(cpatch%dmean_root_growth_resp)
      if(associated(cpatch%dmean_sapa_growth_resp)) deallocate(cpatch%dmean_sapa_growth_resp)
      if(associated(cpatch%dmean_sapb_growth_resp)) deallocate(cpatch%dmean_sapb_growth_resp)
      if(associated(cpatch%dmean_leaf_storage_resp)) deallocate(cpatch%dmean_leaf_storage_resp)
      if(associated(cpatch%dmean_root_storage_resp)) deallocate(cpatch%dmean_root_storage_resp)
      if(associated(cpatch%dmean_sapa_storage_resp)) deallocate(cpatch%dmean_sapa_storage_resp)
      if(associated(cpatch%dmean_sapb_storage_resp)) deallocate(cpatch%dmean_sapb_storage_resp)
      if(associated(cpatch%dmean_plresp        )) deallocate(cpatch%dmean_plresp        )
      if(associated(cpatch%dmean_leaf_energy   )) deallocate(cpatch%dmean_leaf_energy   )
      if(associated(cpatch%dmean_leaf_water    )) deallocate(cpatch%dmean_leaf_water    )
      if(associated(cpatch%dmean_leaf_hcap     )) deallocate(cpatch%dmean_leaf_hcap     )
      if(associated(cpatch%dmean_leaf_vpdef    )) deallocate(cpatch%dmean_leaf_vpdef    )
      if(associated(cpatch%dmean_leaf_temp     )) deallocate(cpatch%dmean_leaf_temp     )
      if(associated(cpatch%dmean_leaf_fliq     )) deallocate(cpatch%dmean_leaf_fliq     )
      if(associated(cpatch%dmean_leaf_gsw      )) deallocate(cpatch%dmean_leaf_gsw      )
      if(associated(cpatch%dmean_leaf_gbw      )) deallocate(cpatch%dmean_leaf_gbw      )
      if(associated(cpatch%dmean_wood_energy   )) deallocate(cpatch%dmean_wood_energy   )
      if(associated(cpatch%dmean_wood_water    )) deallocate(cpatch%dmean_wood_water    )
      if(associated(cpatch%dmean_wood_hcap     )) deallocate(cpatch%dmean_wood_hcap     )
      if(associated(cpatch%dmean_wood_temp     )) deallocate(cpatch%dmean_wood_temp     )
      if(associated(cpatch%dmean_wood_fliq     )) deallocate(cpatch%dmean_wood_fliq     )
      if(associated(cpatch%dmean_wood_gbw      )) deallocate(cpatch%dmean_wood_gbw      )
      if(associated(cpatch%dmean_fs_open       )) deallocate(cpatch%dmean_fs_open       )
      if(associated(cpatch%dmean_fsw           )) deallocate(cpatch%dmean_fsw           )
      if(associated(cpatch%dmean_fsn           )) deallocate(cpatch%dmean_fsn           )
      if(associated(cpatch%dmean_a_open        )) deallocate(cpatch%dmean_a_open        )
      if(associated(cpatch%dmean_a_closed      )) deallocate(cpatch%dmean_a_closed      )
      if(associated(cpatch%dmean_a_net         )) deallocate(cpatch%dmean_a_net         )
      if(associated(cpatch%dmean_A_light       )) deallocate(cpatch%dmean_A_light       )
      if(associated(cpatch%dmean_A_rubp        )) deallocate(cpatch%dmean_A_rubp        )
      if(associated(cpatch%dmean_A_co2         )) deallocate(cpatch%dmean_A_co2         )
      if(associated(cpatch%dmean_psi_open      )) deallocate(cpatch%dmean_psi_open      )
      if(associated(cpatch%dmean_psi_closed    )) deallocate(cpatch%dmean_psi_closed    )
      if(associated(cpatch%dmean_water_supply  )) deallocate(cpatch%dmean_water_supply  )
      if(associated(cpatch%dmean_light_level   )) deallocate(cpatch%dmean_light_level   )
      if(associated(cpatch%dmean_light_level_beam))                                        &
                                                deallocate(cpatch%dmean_light_level_beam)
      if(associated(cpatch%dmean_light_level_diff))                                        &
                                                deallocate(cpatch%dmean_light_level_diff)

      if(associated(cpatch%dmean_par_level_beam ))deallocate(cpatch%dmean_par_level_beam )
      if(associated(cpatch%dmean_par_level_diffu))deallocate(cpatch%dmean_par_level_diffu)
      if(associated(cpatch%dmean_par_level_diffd))deallocate(cpatch%dmean_par_level_diffd)

      if(associated(cpatch%dmean_par_l         )) deallocate(cpatch%dmean_par_l         )
      if(associated(cpatch%dmean_par_l_beam    )) deallocate(cpatch%dmean_par_l_beam    )
      if(associated(cpatch%dmean_par_l_diff    )) deallocate(cpatch%dmean_par_l_diff    )
      if(associated(cpatch%dmean_rshort_l      )) deallocate(cpatch%dmean_rshort_l      )
      if(associated(cpatch%dmean_rlong_l       )) deallocate(cpatch%dmean_rlong_l       )
      if(associated(cpatch%dmean_sensible_lc   )) deallocate(cpatch%dmean_sensible_lc   )
      if(associated(cpatch%dmean_vapor_lc      )) deallocate(cpatch%dmean_vapor_lc      )
      if(associated(cpatch%dmean_transp        )) deallocate(cpatch%dmean_transp        )
      if(associated(cpatch%dmean_intercepted_al)) deallocate(cpatch%dmean_intercepted_al)
      if(associated(cpatch%dmean_wshed_lg      )) deallocate(cpatch%dmean_wshed_lg      )
      if(associated(cpatch%dmean_rshort_w      )) deallocate(cpatch%dmean_rshort_w      )
      if(associated(cpatch%dmean_rlong_w       )) deallocate(cpatch%dmean_rlong_w       )
      if(associated(cpatch%dmean_rad_profile   )) deallocate(cpatch%dmean_rad_profile   )
      if(associated(cpatch%dmean_sensible_wc   )) deallocate(cpatch%dmean_sensible_wc   )
      if(associated(cpatch%dmean_vapor_wc      )) deallocate(cpatch%dmean_vapor_wc      )
      if(associated(cpatch%dmean_intercepted_aw)) deallocate(cpatch%dmean_intercepted_aw)
      if(associated(cpatch%dmean_wshed_wg      )) deallocate(cpatch%dmean_wshed_wg      )
      if(associated(cpatch%mmean_lai           )) deallocate(cpatch%mmean_lai           )
      if(associated(cpatch%mmean_bleaf         )) deallocate(cpatch%mmean_bleaf         )
      if(associated(cpatch%mmean_broot         )) deallocate(cpatch%mmean_broot         )
      if(associated(cpatch%mmean_bstorage      )) deallocate(cpatch%mmean_bstorage      )
      if(associated(cpatch%mmean_mort_rate     )) deallocate(cpatch%mmean_mort_rate     )
      if(associated(cpatch%mmean_leaf_maintenance))                                        &
                                                deallocate(cpatch%mmean_leaf_maintenance)
      if(associated(cpatch%mmean_root_maintenance))                                        &
                                                deallocate(cpatch%mmean_root_maintenance)
      if(associated(cpatch%mmean_leaf_drop     )) deallocate(cpatch%mmean_leaf_drop     )
      if(associated(cpatch%mmean_cb            )) deallocate(cpatch%mmean_cb            )
      if(associated(cpatch%mmean_gpp           )) deallocate(cpatch%mmean_gpp           )
      if(associated(cpatch%mmean_npp           )) deallocate(cpatch%mmean_npp           )
      if(associated(cpatch%mmean_leaf_resp     )) deallocate(cpatch%mmean_leaf_resp     )
      if(associated(cpatch%mmean_root_resp     )) deallocate(cpatch%mmean_root_resp     )
      if(associated(cpatch%mmean_leaf_growth_resp)) deallocate(cpatch%mmean_leaf_growth_resp)
      if(associated(cpatch%mmean_root_growth_resp)) deallocate(cpatch%mmean_root_growth_resp)
      if(associated(cpatch%mmean_sapa_growth_resp)) deallocate(cpatch%mmean_sapa_growth_resp)
      if(associated(cpatch%mmean_sapb_growth_resp)) deallocate(cpatch%mmean_sapb_growth_resp)
      if(associated(cpatch%mmean_leaf_storage_resp)) deallocate(cpatch%mmean_leaf_storage_resp)
      if(associated(cpatch%mmean_root_storage_resp)) deallocate(cpatch%mmean_root_storage_resp)
      if(associated(cpatch%mmean_sapa_storage_resp)) deallocate(cpatch%mmean_sapa_storage_resp)
      if(associated(cpatch%mmean_sapb_storage_resp)) deallocate(cpatch%mmean_sapb_storage_resp)
      if(associated(cpatch%mmean_plresp        )) deallocate(cpatch%mmean_plresp        )
      if(associated(cpatch%mmean_leaf_energy   )) deallocate(cpatch%mmean_leaf_energy   )
      if(associated(cpatch%mmean_leaf_water    )) deallocate(cpatch%mmean_leaf_water    )
      if(associated(cpatch%mmean_leaf_hcap     )) deallocate(cpatch%mmean_leaf_hcap     )
      if(associated(cpatch%mmean_leaf_vpdef    )) deallocate(cpatch%mmean_leaf_vpdef    )
      if(associated(cpatch%mmean_leaf_temp     )) deallocate(cpatch%mmean_leaf_temp     )
      if(associated(cpatch%mmean_leaf_fliq     )) deallocate(cpatch%mmean_leaf_fliq     )
      if(associated(cpatch%mmean_leaf_gsw      )) deallocate(cpatch%mmean_leaf_gsw      )
      if(associated(cpatch%mmean_leaf_gbw      )) deallocate(cpatch%mmean_leaf_gbw      )
      if(associated(cpatch%mmean_wood_energy   )) deallocate(cpatch%mmean_wood_energy   )
      if(associated(cpatch%mmean_wood_water    )) deallocate(cpatch%mmean_wood_water    )
      if(associated(cpatch%mmean_wood_hcap     )) deallocate(cpatch%mmean_wood_hcap     )
      if(associated(cpatch%mmean_wood_temp     )) deallocate(cpatch%mmean_wood_temp     )
      if(associated(cpatch%mmean_wood_fliq     )) deallocate(cpatch%mmean_wood_fliq     )
      if(associated(cpatch%mmean_wood_gbw      )) deallocate(cpatch%mmean_wood_gbw      )
      if(associated(cpatch%mmean_fs_open       )) deallocate(cpatch%mmean_fs_open       )
      if(associated(cpatch%mmean_fsw           )) deallocate(cpatch%mmean_fsw           )
      if(associated(cpatch%mmean_fsn           )) deallocate(cpatch%mmean_fsn           )
      if(associated(cpatch%mmean_a_open        )) deallocate(cpatch%mmean_a_open        )
      if(associated(cpatch%mmean_a_closed      )) deallocate(cpatch%mmean_a_closed      )
      if(associated(cpatch%mmean_a_net         )) deallocate(cpatch%mmean_a_net         )
      if(associated(cpatch%mmean_A_light       )) deallocate(cpatch%mmean_A_light       )
      if(associated(cpatch%mmean_A_rubp        )) deallocate(cpatch%mmean_A_rubp        )
      if(associated(cpatch%mmean_A_co2         )) deallocate(cpatch%mmean_A_co2         )
      if(associated(cpatch%mmean_psi_open      )) deallocate(cpatch%mmean_psi_open      )
      if(associated(cpatch%mmean_psi_closed    )) deallocate(cpatch%mmean_psi_closed    )
      if(associated(cpatch%mmean_water_supply  )) deallocate(cpatch%mmean_water_supply  )
      if(associated(cpatch%mmean_light_level   )) deallocate(cpatch%mmean_light_level   )
      if(associated(cpatch%mmean_light_level_beam))                                        &
                                                deallocate(cpatch%mmean_light_level_beam)
      if(associated(cpatch%mmean_light_level_diff))                                        &
                                                deallocate(cpatch%mmean_light_level_diff)
      
      if(associated(cpatch%mmean_par_level_beam ))deallocate(cpatch%mmean_par_level_beam )
      if(associated(cpatch%mmean_par_level_diffu))deallocate(cpatch%mmean_par_level_diffu)
      if(associated(cpatch%mmean_par_level_diffd))deallocate(cpatch%mmean_par_level_diffd)

      if(associated(cpatch%mmean_par_l         )) deallocate(cpatch%mmean_par_l         )
      if(associated(cpatch%mmean_par_l_beam    )) deallocate(cpatch%mmean_par_l_beam    )
      if(associated(cpatch%mmean_par_l_diff    )) deallocate(cpatch%mmean_par_l_diff    )
      if(associated(cpatch%mmean_rshort_l      )) deallocate(cpatch%mmean_rshort_l      )
      if(associated(cpatch%mmean_rlong_l       )) deallocate(cpatch%mmean_rlong_l       )
      if(associated(cpatch%mmean_sensible_lc   )) deallocate(cpatch%mmean_sensible_lc   )
      if(associated(cpatch%mmean_vapor_lc      )) deallocate(cpatch%mmean_vapor_lc      )
      if(associated(cpatch%mmean_transp        )) deallocate(cpatch%mmean_transp        )
      if(associated(cpatch%mmean_intercepted_al)) deallocate(cpatch%mmean_intercepted_al)
      if(associated(cpatch%mmean_wshed_lg      )) deallocate(cpatch%mmean_wshed_lg      )
      if(associated(cpatch%mmean_rshort_w      )) deallocate(cpatch%mmean_rshort_w      )
      if(associated(cpatch%mmean_rlong_w       )) deallocate(cpatch%mmean_rlong_w       )
      if(associated(cpatch%mmean_rad_profile   )) deallocate(cpatch%mmean_rad_profile   )
      if(associated(cpatch%mmean_sensible_wc   )) deallocate(cpatch%mmean_sensible_wc   )
      if(associated(cpatch%mmean_vapor_wc      )) deallocate(cpatch%mmean_vapor_wc      )
      if(associated(cpatch%mmean_intercepted_aw)) deallocate(cpatch%mmean_intercepted_aw)
      if(associated(cpatch%mmean_wshed_wg      )) deallocate(cpatch%mmean_wshed_wg      )
      if(associated(cpatch%mmean_nppleaf       )) deallocate(cpatch%mmean_nppleaf       )
      if(associated(cpatch%mmean_nppfroot      )) deallocate(cpatch%mmean_nppfroot      )
      if(associated(cpatch%mmean_nppsapwood    )) deallocate(cpatch%mmean_nppsapwood    )
      if(associated(cpatch%mmean_nppcroot      )) deallocate(cpatch%mmean_nppcroot      )
      if(associated(cpatch%mmean_nppseeds      )) deallocate(cpatch%mmean_nppseeds      )
      if(associated(cpatch%mmean_nppwood       )) deallocate(cpatch%mmean_nppwood       )
      if(associated(cpatch%mmean_nppdaily      )) deallocate(cpatch%mmean_nppdaily      )
      if(associated(cpatch%mmsqu_gpp           )) deallocate(cpatch%mmsqu_gpp           )
      if(associated(cpatch%mmsqu_npp           )) deallocate(cpatch%mmsqu_npp           )
      if(associated(cpatch%mmsqu_plresp        )) deallocate(cpatch%mmsqu_plresp        )
      if(associated(cpatch%mmsqu_sensible_lc   )) deallocate(cpatch%mmsqu_sensible_lc   )
      if(associated(cpatch%mmsqu_vapor_lc      )) deallocate(cpatch%mmsqu_vapor_lc      )
      if(associated(cpatch%mmsqu_transp        )) deallocate(cpatch%mmsqu_transp        )
      if(associated(cpatch%mmsqu_sensible_wc   )) deallocate(cpatch%mmsqu_sensible_wc   )
      if(associated(cpatch%mmsqu_vapor_wc      )) deallocate(cpatch%mmsqu_vapor_wc      )
      if(associated(cpatch%qmean_gpp           )) deallocate(cpatch%qmean_gpp           )
      if(associated(cpatch%qmean_npp           )) deallocate(cpatch%qmean_npp           )
      if(associated(cpatch%qmean_leaf_resp     )) deallocate(cpatch%qmean_leaf_resp     )
      if(associated(cpatch%qmean_root_resp     )) deallocate(cpatch%qmean_root_resp     )
      if(associated(cpatch%qmean_leaf_growth_resp)) deallocate(cpatch%qmean_leaf_growth_resp)
      if(associated(cpatch%qmean_root_growth_resp)) deallocate(cpatch%qmean_root_growth_resp)
      if(associated(cpatch%qmean_sapa_growth_resp)) deallocate(cpatch%qmean_sapa_growth_resp)
      if(associated(cpatch%qmean_sapb_growth_resp)) deallocate(cpatch%qmean_sapb_growth_resp)
      if(associated(cpatch%qmean_leaf_storage_resp)) deallocate(cpatch%qmean_leaf_storage_resp)
      if(associated(cpatch%qmean_root_storage_resp)) deallocate(cpatch%qmean_root_storage_resp)
      if(associated(cpatch%qmean_sapa_storage_resp)) deallocate(cpatch%qmean_sapa_storage_resp)
      if(associated(cpatch%qmean_sapb_storage_resp)) deallocate(cpatch%qmean_sapb_storage_resp)
      if(associated(cpatch%qmean_plresp        )) deallocate(cpatch%qmean_plresp        )
      if(associated(cpatch%qmean_leaf_energy   )) deallocate(cpatch%qmean_leaf_energy   )
      if(associated(cpatch%qmean_leaf_water    )) deallocate(cpatch%qmean_leaf_water    )
      if(associated(cpatch%qmean_leaf_hcap     )) deallocate(cpatch%qmean_leaf_hcap     )
      if(associated(cpatch%qmean_leaf_vpdef    )) deallocate(cpatch%qmean_leaf_vpdef    )
      if(associated(cpatch%qmean_leaf_temp     )) deallocate(cpatch%qmean_leaf_temp     )
      if(associated(cpatch%qmean_leaf_fliq     )) deallocate(cpatch%qmean_leaf_fliq     )
      if(associated(cpatch%qmean_leaf_gsw      )) deallocate(cpatch%qmean_leaf_gsw      )
      if(associated(cpatch%qmean_leaf_gbw      )) deallocate(cpatch%qmean_leaf_gbw      )
      if(associated(cpatch%qmean_wood_energy   )) deallocate(cpatch%qmean_wood_energy   )
      if(associated(cpatch%qmean_wood_water    )) deallocate(cpatch%qmean_wood_water    )
      if(associated(cpatch%qmean_wood_hcap     )) deallocate(cpatch%qmean_wood_hcap     )
      if(associated(cpatch%qmean_wood_temp     )) deallocate(cpatch%qmean_wood_temp     )
      if(associated(cpatch%qmean_wood_fliq     )) deallocate(cpatch%qmean_wood_fliq     )
      if(associated(cpatch%qmean_wood_gbw      )) deallocate(cpatch%qmean_wood_gbw      )
      if(associated(cpatch%qmean_fs_open       )) deallocate(cpatch%qmean_fs_open       )
      if(associated(cpatch%qmean_fsw           )) deallocate(cpatch%qmean_fsw           )
      if(associated(cpatch%qmean_fsn           )) deallocate(cpatch%qmean_fsn           )
      if(associated(cpatch%qmean_a_open        )) deallocate(cpatch%qmean_a_open        )
      if(associated(cpatch%qmean_a_closed      )) deallocate(cpatch%qmean_a_closed      )
      if(associated(cpatch%qmean_a_net         )) deallocate(cpatch%qmean_a_net         )
      if(associated(cpatch%qmean_A_light       )) deallocate(cpatch%qmean_A_light       )
      if(associated(cpatch%qmean_A_rubp        )) deallocate(cpatch%qmean_A_rubp        )
      if(associated(cpatch%qmean_A_co2         )) deallocate(cpatch%qmean_A_co2         )
      if(associated(cpatch%qmean_psi_open      )) deallocate(cpatch%qmean_psi_open      )
      if(associated(cpatch%qmean_psi_closed    )) deallocate(cpatch%qmean_psi_closed    )
      if(associated(cpatch%qmean_water_supply  )) deallocate(cpatch%qmean_water_supply  )
      if(associated(cpatch%qmean_light_level   )) deallocate(cpatch%qmean_light_level   )
      if(associated(cpatch%qmean_light_level_beam))                                        &
                                                deallocate(cpatch%qmean_light_level_beam)
      if(associated(cpatch%qmean_light_level_diff))                                        &
                                                deallocate(cpatch%qmean_light_level_diff)

      if(associated(cpatch%qmean_par_level_beam ))deallocate(cpatch%qmean_par_level_beam )
      if(associated(cpatch%qmean_par_level_diffu))deallocate(cpatch%qmean_par_level_diffu)
      if(associated(cpatch%qmean_par_level_diffd))deallocate(cpatch%qmean_par_level_diffd)

      if(associated(cpatch%qmean_par_l         )) deallocate(cpatch%qmean_par_l         )
      if(associated(cpatch%qmean_par_l_beam    )) deallocate(cpatch%qmean_par_l_beam    )
      if(associated(cpatch%qmean_par_l_diff    )) deallocate(cpatch%qmean_par_l_diff    )
      if(associated(cpatch%qmean_rshort_l      )) deallocate(cpatch%qmean_rshort_l      )
      if(associated(cpatch%qmean_rlong_l       )) deallocate(cpatch%qmean_rlong_l       )
      if(associated(cpatch%qmean_sensible_lc   )) deallocate(cpatch%qmean_sensible_lc   )
      if(associated(cpatch%qmean_vapor_lc      )) deallocate(cpatch%qmean_vapor_lc      )
      if(associated(cpatch%qmean_transp        )) deallocate(cpatch%qmean_transp        )
      if(associated(cpatch%qmean_intercepted_al)) deallocate(cpatch%qmean_intercepted_al)
      if(associated(cpatch%qmean_wshed_lg      )) deallocate(cpatch%qmean_wshed_lg      )
      if(associated(cpatch%qmean_rshort_w      )) deallocate(cpatch%qmean_rshort_w      )
      if(associated(cpatch%qmean_rlong_w       )) deallocate(cpatch%qmean_rlong_w       )
      if(associated(cpatch%qmean_rad_profile   )) deallocate(cpatch%qmean_rad_profile   )
      if(associated(cpatch%qmean_sensible_wc   )) deallocate(cpatch%qmean_sensible_wc   )
      if(associated(cpatch%qmean_vapor_wc      )) deallocate(cpatch%qmean_vapor_wc      )
      if(associated(cpatch%qmean_intercepted_aw)) deallocate(cpatch%qmean_intercepted_aw)
      if(associated(cpatch%qmean_wshed_wg      )) deallocate(cpatch%qmean_wshed_wg      )
      if(associated(cpatch%qmsqu_gpp           )) deallocate(cpatch%qmsqu_gpp           )
      if(associated(cpatch%qmsqu_npp           )) deallocate(cpatch%qmsqu_npp           )
      if(associated(cpatch%qmsqu_plresp        )) deallocate(cpatch%qmsqu_plresp        )
      if(associated(cpatch%qmsqu_sensible_lc   )) deallocate(cpatch%qmsqu_sensible_lc   )
      if(associated(cpatch%qmsqu_vapor_lc      )) deallocate(cpatch%qmsqu_vapor_lc      )
      if(associated(cpatch%qmsqu_transp        )) deallocate(cpatch%qmsqu_transp        )
      if(associated(cpatch%qmsqu_sensible_wc   )) deallocate(cpatch%qmsqu_sensible_wc   )
      if(associated(cpatch%qmsqu_vapor_wc      )) deallocate(cpatch%qmsqu_vapor_wc      )

      return
   end subroutine deallocate_patchtype
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This subroutine copies a continuous chunk of patches from one place to another.   !
   ! The number of patches to be copied must match between the input and output, and every !
   ! single variable, including the cohorts, will be copied.                               !
   ! IMPORTANT.  This subroutine assumes that the output patches still don't have cohorts  !
   !             allocated, so this should be never used in a previously allocated patch.  !
   !---------------------------------------------------------------------------------------!
   subroutine copy_sitetype(isite,osite,ipaa,ipaz,opaa,opaz)
      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(sitetype)  , target     :: isite ! Input  (donor) site
      type(sitetype)  , target     :: osite ! Output (receptor) site
      integer         , intent(in) :: ipaa  ! First input  patch index
      integer         , intent(in) :: ipaz  ! Last  input  patch index
      integer         , intent(in) :: opaa  ! First output patch index
      integer         , intent(in) :: opaz  ! Last  output patch index
      !----- Local variables. -------------------------------------------------------------!
      integer                      :: ipa   ! Counter for the input  site patches
      integer                      :: opa   ! Counter for the output site patches
      integer                      :: m     ! First counter
      integer                      :: n     ! Second counter
      !------------------------------------------------------------------------------------!


      !------ Check whether this patch copying makes sense. -------------------------------!
      if (ipaz - ipaa /= opaz - opaa) then
         write (unit=*,fmt='(a)')       '------------------------------------------------'
         write (unit=*,fmt='(a)')       ' - Input site:'
         write (unit=*,fmt='(a,1x,i6)') '   * First patch: ',ipaa
         write (unit=*,fmt='(a,1x,i6)') '   * Last  patch: ',ipaz
         write (unit=*,fmt='(a,1x,i6)') '   * Patch count: ',ipaz-ipaa+1
         write (unit=*,fmt='(a)')       ' - Output site:'
         write (unit=*,fmt='(a,1x,i6)') '   * First patch: ',opaa
         write (unit=*,fmt='(a,1x,i6)') '   * Last  patch: ',opaz
         write (unit=*,fmt='(a,1x,i6)') '   * Patch count: ',opaz-opaa+1
         write (unit=*,fmt='(a)'      ) '------------------------------------------------'
         call fatal_error ('Patch count of input and output sites don''t match'            &
                          ,'copy_sitetype','ed_state_vars.f90')
      end if
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !     First we copy the cohorts.                                                     !
      !------------------------------------------------------------------------------------!
      ipa = ipaa - 1
      opaloop: do opa = opaa, opaz
         ipa = ipa + 1
         !----- Copy all cohorts. ---------------------------------------------------------!
         call allocate_patchtype(osite%patch(opa),isite%patch(ipa)%ncohorts)
         call copy_patchtype    (isite%patch(ipa),osite%patch(opa)                         &
                                ,1,isite%patch(ipa)%ncohorts,1,isite%patch(ipa)%ncohorts)
         !---------------------------------------------------------------------------------!


         !----- Scalars. ------------------------------------------------------------------!
         osite%paco_id                    (opa) = isite%paco_id                    (ipa)
         osite%paco_n                     (opa) = isite%paco_n                     (ipa)
         osite%cohort_count               (opa) = isite%cohort_count               (ipa)
         osite%pname                      (opa) = isite%pname                      (ipa)
         osite%area                       (opa) = isite%area                       (ipa)
         osite%age                        (opa) = isite%age                        (ipa)
         osite%dist_type                  (opa) = isite%dist_type                  (ipa)
         osite%fast_soil_C                (opa) = isite%fast_soil_C                (ipa)
         osite%slow_soil_C                (opa) = isite%slow_soil_C                (ipa)
         osite%structural_soil_C          (opa) = isite%structural_soil_C          (ipa)
         osite%structural_soil_L          (opa) = isite%structural_soil_L          (ipa)
         osite%mineralized_soil_N         (opa) = isite%mineralized_soil_N         (ipa)
         osite%fast_soil_N                (opa) = isite%fast_soil_N                (ipa)
         osite%sum_dgd                    (opa) = isite%sum_dgd                    (ipa)
         osite%sum_chd                    (opa) = isite%sum_chd                    (ipa)
         osite%can_theiv                  (opa) = isite%can_theiv                  (ipa)
         osite%can_vpdef                  (opa) = isite%can_vpdef                  (ipa)
         osite%can_temp                   (opa) = isite%can_temp                   (ipa)
         osite%can_temp_pv                (opa) = isite%can_temp_pv                (ipa)
         osite%can_shv                    (opa) = isite%can_shv                    (ipa)
         osite%can_co2                    (opa) = isite%can_co2                    (ipa)
         osite%can_rhos                   (opa) = isite%can_rhos                   (ipa)
         osite%can_prss                   (opa) = isite%can_prss                   (ipa)
         osite%can_theta                  (opa) = isite%can_theta                  (ipa)
         osite%can_depth                  (opa) = isite%can_depth                  (ipa)
         osite%opencan_frac               (opa) = isite%opencan_frac               (ipa)
         osite%ggbare                     (opa) = isite%ggbare                     (ipa)
         osite%ggveg                      (opa) = isite%ggveg                      (ipa)
         osite%ggnet                      (opa) = isite%ggnet                      (ipa)
         osite%ggsoil                     (opa) = isite%ggsoil                     (ipa)
         osite%nlev_sfcwater              (opa) = isite%nlev_sfcwater              (ipa)
         osite%ground_shv                 (opa) = isite%ground_shv                 (ipa)
         osite%ground_ssh                 (opa) = isite%ground_ssh                 (ipa)
         osite%ground_temp                (opa) = isite%ground_temp                (ipa)
         osite%ground_fliq                (opa) = isite%ground_fliq                (ipa)
         osite%rough                      (opa) = isite%rough                      (ipa)
         osite%par_l_max                  (opa) = isite%par_l_max                  (ipa)
         osite%par_l_beam_max             (opa) = isite%par_l_beam_max             (ipa)
         osite%par_l_diffuse_max          (opa) = isite%par_l_diffuse_max          (ipa)
         osite%avg_daily_temp             (opa) = isite%avg_daily_temp             (ipa)
         osite%avg_monthly_gndwater       (opa) = isite%avg_monthly_gndwater       (ipa)
         osite%avg_monthly_waterdef       (opa) = isite%avg_monthly_waterdef       (ipa)
         osite%wbudget_loss2atm           (opa) = isite%wbudget_loss2atm           (ipa)
         osite%wbudget_denseffect         (opa) = isite%wbudget_denseffect         (ipa)
         osite%wbudget_precipgain         (opa) = isite%wbudget_precipgain         (ipa)
         osite%wbudget_loss2runoff        (opa) = isite%wbudget_loss2runoff        (ipa)
         osite%wbudget_loss2drainage      (opa) = isite%wbudget_loss2drainage      (ipa)
         osite%wbudget_initialstorage     (opa) = isite%wbudget_initialstorage     (ipa)
         osite%wbudget_residual           (opa) = isite%wbudget_residual           (ipa)
         osite%ebudget_loss2atm           (opa) = isite%ebudget_loss2atm           (ipa)
         osite%ebudget_denseffect         (opa) = isite%ebudget_denseffect         (ipa)
         osite%ebudget_prsseffect         (opa) = isite%ebudget_prsseffect         (ipa)
         osite%ebudget_loss2runoff        (opa) = isite%ebudget_loss2runoff        (ipa)
         osite%ebudget_loss2drainage      (opa) = isite%ebudget_loss2drainage      (ipa)
         osite%ebudget_netrad             (opa) = isite%ebudget_netrad             (ipa)
         osite%ebudget_precipgain         (opa) = isite%ebudget_precipgain         (ipa)
         osite%ebudget_initialstorage     (opa) = isite%ebudget_initialstorage     (ipa)
         osite%ebudget_residual           (opa) = isite%ebudget_residual           (ipa)
         osite%co2budget_initialstorage   (opa) = isite%co2budget_initialstorage   (ipa)
         osite%co2budget_residual         (opa) = isite%co2budget_residual         (ipa)
         osite%co2budget_loss2atm         (opa) = isite%co2budget_loss2atm         (ipa)
         osite%co2budget_denseffect       (opa) = isite%co2budget_denseffect       (ipa)
         osite%co2budget_gpp              (opa) = isite%co2budget_gpp              (ipa)
         osite%co2budget_plresp           (opa) = isite%co2budget_plresp           (ipa)
         osite%co2budget_rh               (opa) = isite%co2budget_rh               (ipa)
         osite%today_A_decomp             (opa) = isite%today_A_decomp             (ipa)
         osite%today_Af_decomp            (opa) = isite%today_Af_decomp            (ipa)
         osite%veg_rough                  (opa) = isite%veg_rough                  (ipa)
         osite%veg_height                 (opa) = isite%veg_height                 (ipa)
         osite%veg_displace               (opa) = isite%veg_displace               (ipa)
         osite%fsc_in                     (opa) = isite%fsc_in                     (ipa)
         osite%ssc_in                     (opa) = isite%ssc_in                     (ipa)
         osite%ssl_in                     (opa) = isite%ssl_in                     (ipa)
         osite%fsn_in                     (opa) = isite%fsn_in                     (ipa)
         osite%total_plant_nitrogen_uptake(opa) = isite%total_plant_nitrogen_uptake(ipa)
         osite%mineralized_N_loss         (opa) = isite%mineralized_N_loss         (ipa)
         osite%mineralized_N_input        (opa) = isite%mineralized_N_input        (ipa)
         osite%rshort_g                   (opa) = isite%rshort_g                   (ipa)
         osite%rshort_g_beam              (opa) = isite%rshort_g_beam              (ipa)
         osite%rshort_g_diffuse           (opa) = isite%rshort_g_diffuse           (ipa)
         osite%par_g                      (opa) = isite%par_g                      (ipa)
         osite%par_g_beam                 (opa) = isite%par_g_beam                 (ipa)
         osite%par_g_diffuse              (opa) = isite%par_g_diffuse              (ipa)
         osite%par_b                      (opa) = isite%par_b                      (ipa)
         osite%par_b_beam                 (opa) = isite%par_b_beam                 (ipa)
         osite%par_b_diffuse              (opa) = isite%par_b_diffuse              (ipa)
         osite%nir_b                      (opa) = isite%nir_b                      (ipa)
         osite%nir_b_beam                 (opa) = isite%nir_b_beam                 (ipa)
         osite%nir_b_diffuse              (opa) = isite%nir_b_diffuse              (ipa)
         osite%rlong_g                    (opa) = isite%rlong_g                    (ipa)
         osite%rlong_s                    (opa) = isite%rlong_s                    (ipa)
         osite%albedo                     (opa) = isite%albedo                     (ipa)
         osite%albedo_par                 (opa) = isite%albedo_par                 (ipa)
         osite%albedo_nir                 (opa) = isite%albedo_nir                 (ipa)
         osite%rlong_albedo               (opa) = isite%rlong_albedo               (ipa)
         osite%rnet                       (opa) = isite%rnet                       (ipa)
         osite%rlongup                    (opa) = isite%rlongup                    (ipa)
         osite%parup                      (opa) = isite%parup                      (ipa)
         osite%nirup                      (opa) = isite%nirup                      (ipa)
         osite%rshortup                   (opa) = isite%rshortup                   (ipa)
         osite%total_sfcw_depth           (opa) = isite%total_sfcw_depth           (ipa)
         osite%snowfac                    (opa) = isite%snowfac                    (ipa)
         osite%A_decomp                   (opa) = isite%A_decomp                   (ipa)
         osite%f_decomp                   (opa) = isite%f_decomp                   (ipa)
         osite%rh                         (opa) = isite%rh                         (ipa)
         osite%cwd_rh                     (opa) = isite%cwd_rh                     (ipa)
         osite%plant_ag_biomass           (opa) = isite%plant_ag_biomass           (ipa)
         osite%htry                       (opa) = isite%htry                       (ipa)
         osite%hprev                      (opa) = isite%hprev                      (ipa)
         osite%ustar                      (opa) = isite%ustar                      (ipa)
         osite%tstar                      (opa) = isite%tstar                      (ipa)
         osite%qstar                      (opa) = isite%qstar                      (ipa)
         osite%cstar                      (opa) = isite%cstar                      (ipa)
         osite%zeta                       (opa) = isite%zeta                       (ipa)
         osite%ribulk                     (opa) = isite%ribulk                     (ipa)
         osite%upwp                       (opa) = isite%upwp                       (ipa)
         osite%qpwp                       (opa) = isite%qpwp                       (ipa)
         osite%cpwp                       (opa) = isite%cpwp                       (ipa)
         osite%tpwp                       (opa) = isite%tpwp                       (ipa)
         osite%wpwp                       (opa) = isite%wpwp                       (ipa)
         osite%watertable                 (opa) = isite%watertable                 (ipa)
         osite%moist_dz                   (opa) = isite%moist_dz                   (ipa)
         osite%ksat                       (opa) = isite%ksat                       (ipa)
         osite%soil_sat_energy            (opa) = isite%soil_sat_energy            (ipa)
         osite%soil_sat_water             (opa) = isite%soil_sat_water             (ipa)
         osite%soil_sat_heat              (opa) = isite%soil_sat_heat              (ipa)
         osite%runoff_rate                (opa) = isite%runoff_rate                (ipa)
         osite%runoff                     (opa) = isite%runoff                     (ipa)
         osite%qrunoff                    (opa) = isite%qrunoff                    (ipa)
         osite%fmean_rh                   (opa) = isite%fmean_rh                   (ipa)
         osite%fmean_cwd_rh               (opa) = isite%fmean_cwd_rh               (ipa)
         osite%fmean_nep                  (opa) = isite%fmean_nep                  (ipa)
         osite%fmean_rk4step              (opa) = isite%fmean_rk4step              (ipa)
         osite%fmean_available_water      (opa) = isite%fmean_available_water      (ipa)
         osite%fmean_can_theiv            (opa) = isite%fmean_can_theiv            (ipa)
         osite%fmean_can_theta            (opa) = isite%fmean_can_theta            (ipa)
         osite%fmean_can_vpdef            (opa) = isite%fmean_can_vpdef            (ipa)
         osite%fmean_can_temp             (opa) = isite%fmean_can_temp             (ipa)
         osite%fmean_can_shv              (opa) = isite%fmean_can_shv              (ipa)
         osite%fmean_can_co2              (opa) = isite%fmean_can_co2              (ipa)
         osite%fmean_can_rhos             (opa) = isite%fmean_can_rhos             (ipa)
         osite%fmean_can_prss             (opa) = isite%fmean_can_prss             (ipa)
         osite%fmean_gnd_temp             (opa) = isite%fmean_gnd_temp             (ipa)
         osite%fmean_gnd_shv              (opa) = isite%fmean_gnd_shv              (ipa)
         osite%fmean_can_ggnd             (opa) = isite%fmean_can_ggnd             (ipa)
         osite%fmean_sfcw_depth           (opa) = isite%fmean_sfcw_depth           (ipa)
         osite%fmean_sfcw_energy          (opa) = isite%fmean_sfcw_energy          (ipa)
         osite%fmean_sfcw_mass            (opa) = isite%fmean_sfcw_mass            (ipa)
         osite%fmean_sfcw_temp            (opa) = isite%fmean_sfcw_temp            (ipa)
         osite%fmean_sfcw_fliq            (opa) = isite%fmean_sfcw_fliq            (ipa)
         osite%fmean_rshort_gnd           (opa) = isite%fmean_rshort_gnd           (ipa)
         osite%fmean_par_gnd              (opa) = isite%fmean_par_gnd              (ipa)
         osite%fmean_rlong_gnd            (opa) = isite%fmean_rlong_gnd            (ipa)
         osite%fmean_rlongup              (opa) = isite%fmean_rlongup              (ipa)
         osite%fmean_parup                (opa) = isite%fmean_parup                (ipa)
         osite%fmean_nirup                (opa) = isite%fmean_nirup                (ipa)
         osite%fmean_rshortup             (opa) = isite%fmean_rshortup             (ipa)
         osite%fmean_rnet                 (opa) = isite%fmean_rnet                 (ipa)
         osite%fmean_albedo               (opa) = isite%fmean_albedo               (ipa)
         osite%fmean_albedo_par           (opa) = isite%fmean_albedo_par           (ipa)
         osite%fmean_albedo_nir           (opa) = isite%fmean_albedo_nir           (ipa)
         osite%fmean_rlong_albedo         (opa) = isite%fmean_rlong_albedo         (ipa)
         osite%fmean_ustar                (opa) = isite%fmean_ustar                (ipa)
         osite%fmean_tstar                (opa) = isite%fmean_tstar                (ipa)
         osite%fmean_qstar                (opa) = isite%fmean_qstar                (ipa)
         osite%fmean_cstar                (opa) = isite%fmean_cstar                (ipa)
         osite%fmean_carbon_ac            (opa) = isite%fmean_carbon_ac            (ipa)
         osite%fmean_carbon_st            (opa) = isite%fmean_carbon_st            (ipa)
         osite%fmean_vapor_gc             (opa) = isite%fmean_vapor_gc             (ipa)
         osite%fmean_vapor_ac             (opa) = isite%fmean_vapor_ac             (ipa)
         osite%fmean_throughfall          (opa) = isite%fmean_throughfall          (ipa)
         osite%fmean_runoff               (opa) = isite%fmean_runoff               (ipa)
         osite%fmean_drainage             (opa) = isite%fmean_drainage             (ipa)
         osite%fmean_sensible_gc          (opa) = isite%fmean_sensible_gc          (ipa)
         osite%fmean_sensible_ac          (opa) = isite%fmean_sensible_ac          (ipa)
         osite%fmean_qthroughfall         (opa) = isite%fmean_qthroughfall         (ipa)
         osite%fmean_qrunoff              (opa) = isite%fmean_qrunoff              (ipa)
         osite%fmean_qdrainage            (opa) = isite%fmean_qdrainage            (ipa)

         !----- Temporary pounding/snow layer variables. ----------------------------------!
         do m=1,nzs
            osite%sfcwater_mass   (m,opa) = isite%sfcwater_mass   (m,ipa)
            osite%sfcwater_energy (m,opa) = isite%sfcwater_energy (m,ipa)
            osite%sfcwater_depth  (m,opa) = isite%sfcwater_depth  (m,ipa)
            osite%sfcwater_tempk  (m,opa) = isite%sfcwater_tempk  (m,ipa)
            osite%sfcwater_fracliq(m,opa) = isite%sfcwater_fracliq(m,ipa)
            osite%rshort_s        (m,opa) = isite%rshort_s        (m,ipa)
            osite%rshort_s_beam   (m,opa) = isite%rshort_s_beam   (m,ipa)
            osite%rshort_s_diffuse(m,opa) = isite%rshort_s_diffuse(m,ipa)
            osite%par_s           (m,opa) = isite%par_s           (m,ipa)
            osite%par_s_beam      (m,opa) = isite%par_s_beam      (m,ipa)
            osite%par_s_diffuse   (m,opa) = isite%par_s_diffuse   (m,ipa)
         end do
         !---------------------------------------------------------------------------------!


         !----- Soil variables. -----------------------------------------------------------!
         do m=1,nzg
            osite%soil_energy      (m,opa) = isite%soil_energy      (m,ipa)
            osite%soil_mstpot      (m,opa) = isite%soil_mstpot      (m,ipa)
            osite%soil_water       (m,opa) = isite%soil_water       (m,ipa)
            osite%soil_tempk       (m,opa) = isite%soil_tempk       (m,ipa)
            osite%soil_fracliq     (m,opa) = isite%soil_fracliq     (m,ipa)
            osite%rootdense        (m,opa) = isite%rootdense        (m,ipa)
            osite%fmean_soil_energy(m,opa) = isite%fmean_soil_energy(m,ipa)
            osite%fmean_soil_mstpot(m,opa) = isite%fmean_soil_mstpot(m,ipa)
            osite%fmean_soil_water (m,opa) = isite%fmean_soil_water (m,ipa)
            osite%fmean_soil_temp  (m,opa) = isite%fmean_soil_temp  (m,ipa)
            osite%fmean_soil_fliq  (m,opa) = isite%fmean_soil_fliq  (m,ipa)
            osite%fmean_smoist_gg  (m,opa) = isite%fmean_smoist_gg  (m,ipa)
            osite%fmean_transloss  (m,opa) = isite%fmean_transloss  (m,ipa)
            osite%fmean_sensible_gg(m,opa) = isite%fmean_sensible_gg(m,ipa)
         end do
         !---------------------------------------------------------------------------------!


         !----- PFT variables. ------------------------------------------------------------!
         do m=1,n_pft
            osite%A_o_max          (  m,opa) = isite%A_o_max       (  m,ipa)
            osite%A_c_max          (  m,opa) = isite%A_c_max       (  m,ipa)
            osite%repro            (  m,opa) = isite%repro         (  m,ipa)
            do n=1,ff_nhgt
               osite%cumlai_profile(m,n,opa) = isite%cumlai_profile(m,n,ipa)
            end do
         end do
         !---------------------------------------------------------------------------------!


         !----- "3" variables. ------------------------------------------------------------!
         do m=1,3
            osite%runoff_A(m,opa) = isite%runoff_A(m,ipa)
         end do
         !---------------------------------------------------------------------------------!



         !---------------------------------------------------------------------------------!
         !     Daily means.                                                                !
         !---------------------------------------------------------------------------------!
         if (writing_long) then
            !----- Scalars. ---------------------------------------------------------------!
            osite%dmean_A_decomp       (opa) = isite%dmean_A_decomp       (ipa)
            osite%dmean_Af_decomp      (opa) = isite%dmean_Af_decomp      (ipa)
            osite%dmean_co2_residual   (opa) = isite%dmean_co2_residual   (ipa)
            osite%dmean_energy_residual(opa) = isite%dmean_energy_residual(ipa)
            osite%dmean_water_residual (opa) = isite%dmean_water_residual (ipa)
            osite%dmean_rh             (opa) = isite%dmean_rh             (ipa)
            osite%dmean_cwd_rh         (opa) = isite%dmean_cwd_rh         (ipa)
            osite%dmean_nep            (opa) = isite%dmean_nep            (ipa)
            osite%dmean_rk4step        (opa) = isite%dmean_rk4step        (ipa)
            osite%dmean_available_water(opa) = isite%dmean_available_water(ipa)
            osite%dmean_can_theiv      (opa) = isite%dmean_can_theiv      (ipa)
            osite%dmean_can_theta      (opa) = isite%dmean_can_theta      (ipa)
            osite%dmean_can_vpdef      (opa) = isite%dmean_can_vpdef      (ipa)
            osite%dmean_can_temp       (opa) = isite%dmean_can_temp       (ipa)
            osite%dmean_can_shv        (opa) = isite%dmean_can_shv        (ipa)
            osite%dmean_can_co2        (opa) = isite%dmean_can_co2        (ipa)
            osite%dmean_can_rhos       (opa) = isite%dmean_can_rhos       (ipa)
            osite%dmean_can_prss       (opa) = isite%dmean_can_prss       (ipa)
            osite%dmean_gnd_temp       (opa) = isite%dmean_gnd_temp       (ipa)
            osite%dmean_gnd_shv        (opa) = isite%dmean_gnd_shv        (ipa)
            osite%dmean_can_ggnd       (opa) = isite%dmean_can_ggnd       (ipa)
            osite%dmean_sfcw_depth     (opa) = isite%dmean_sfcw_depth     (ipa)
            osite%dmean_sfcw_energy    (opa) = isite%dmean_sfcw_energy    (ipa)
            osite%dmean_sfcw_mass      (opa) = isite%dmean_sfcw_mass      (ipa)
            osite%dmean_sfcw_temp      (opa) = isite%dmean_sfcw_temp      (ipa)
            osite%dmean_sfcw_fliq      (opa) = isite%dmean_sfcw_fliq      (ipa)
            osite%dmean_rshort_gnd     (opa) = isite%dmean_rshort_gnd     (ipa)
            osite%dmean_par_gnd        (opa) = isite%dmean_par_gnd        (ipa)
            osite%dmean_rlong_gnd      (opa) = isite%dmean_rlong_gnd      (ipa)
            osite%dmean_rlongup        (opa) = isite%dmean_rlongup        (ipa)
            osite%dmean_parup          (opa) = isite%dmean_parup          (ipa)
            osite%dmean_nirup          (opa) = isite%dmean_nirup          (ipa)
            osite%dmean_rshortup       (opa) = isite%dmean_rshortup       (ipa)
            osite%dmean_rnet           (opa) = isite%dmean_rnet           (ipa)
            osite%dmean_albedo         (opa) = isite%dmean_albedo         (ipa)
            osite%dmean_albedo_par     (opa) = isite%dmean_albedo_par     (ipa)
            osite%dmean_albedo_nir     (opa) = isite%dmean_albedo_nir     (ipa)
            osite%dmean_rlong_albedo   (opa) = isite%dmean_rlong_albedo   (ipa)
            osite%dmean_ustar          (opa) = isite%dmean_ustar          (ipa)
            osite%dmean_tstar          (opa) = isite%dmean_tstar          (ipa)
            osite%dmean_qstar          (opa) = isite%dmean_qstar          (ipa)
            osite%dmean_cstar          (opa) = isite%dmean_cstar          (ipa)
            osite%dmean_carbon_ac      (opa) = isite%dmean_carbon_ac      (ipa)
            osite%dmean_carbon_st      (opa) = isite%dmean_carbon_st      (ipa)
            osite%dmean_vapor_gc       (opa) = isite%dmean_vapor_gc       (ipa)
            osite%dmean_vapor_ac       (opa) = isite%dmean_vapor_ac       (ipa)
            osite%dmean_throughfall    (opa) = isite%dmean_throughfall    (ipa)
            osite%dmean_runoff         (opa) = isite%dmean_runoff         (ipa)
            osite%dmean_drainage       (opa) = isite%dmean_drainage       (ipa)
            osite%dmean_sensible_gc    (opa) = isite%dmean_sensible_gc    (ipa)
            osite%dmean_sensible_ac    (opa) = isite%dmean_sensible_ac    (ipa)
            osite%dmean_qthroughfall   (opa) = isite%dmean_qthroughfall   (ipa)
            osite%dmean_qrunoff        (opa) = isite%dmean_qrunoff        (ipa)
            osite%dmean_qdrainage      (opa) = isite%dmean_qdrainage      (ipa)
            !------------------------------------------------------------------------------!


            !----- Soil variables. --------------------------------------------------------!
            do m=1,nzg
               osite%dmean_soil_energy(m,opa) = isite%dmean_soil_energy(m,ipa)
               osite%dmean_soil_mstpot(m,opa) = isite%dmean_soil_mstpot(m,ipa)
               osite%dmean_soil_water (m,opa) = isite%dmean_soil_water (m,ipa)
               osite%dmean_soil_temp  (m,opa) = isite%dmean_soil_temp  (m,ipa)
               osite%dmean_soil_fliq  (m,opa) = isite%dmean_soil_fliq  (m,ipa)
               osite%dmean_smoist_gg  (m,opa) = isite%dmean_smoist_gg  (m,ipa)
               osite%dmean_transloss  (m,opa) = isite%dmean_transloss  (m,ipa)
               osite%dmean_sensible_gg(m,opa) = isite%dmean_sensible_gg(m,ipa)
            end do
            !------------------------------------------------------------------------------!
         end if
         !---------------------------------------------------------------------------------!



         !---------------------------------------------------------------------------------!
         !     Monthly means.                                                              !
         !---------------------------------------------------------------------------------!
         if (writing_eorq) then
            !----- Scalars. ---------------------------------------------------------------!
            osite%mmean_fast_soil_c    (opa) = isite%mmean_fast_soil_c    (ipa)
            osite%mmean_slow_soil_c    (opa) = isite%mmean_slow_soil_c    (ipa)
            osite%mmean_struct_soil_c  (opa) = isite%mmean_struct_soil_c  (ipa)
            osite%mmean_struct_soil_l  (opa) = isite%mmean_struct_soil_l  (ipa)
            osite%mmean_fast_soil_n    (opa) = isite%mmean_fast_soil_n    (ipa)
            osite%mmean_mineral_soil_n (opa) = isite%mmean_mineral_soil_n (ipa)
            osite%mmean_co2_residual   (opa) = isite%mmean_co2_residual   (ipa)
            osite%mmean_energy_residual(opa) = isite%mmean_energy_residual(ipa)
            osite%mmean_water_residual (opa) = isite%mmean_water_residual (ipa)
            osite%mmean_rh             (opa) = isite%mmean_rh             (ipa)
            osite%mmean_cwd_rh         (opa) = isite%mmean_cwd_rh         (ipa)
            osite%mmean_nep            (opa) = isite%mmean_nep            (ipa)
            osite%mmean_A_decomp       (opa) = isite%mmean_A_decomp       (ipa)
            osite%mmean_Af_decomp      (opa) = isite%mmean_Af_decomp      (ipa)
            osite%mmean_rk4step        (opa) = isite%mmean_rk4step        (ipa)
            osite%mmean_available_water(opa) = isite%mmean_available_water(ipa)
            osite%mmean_can_theiv      (opa) = isite%mmean_can_theiv      (ipa)
            osite%mmean_can_theta      (opa) = isite%mmean_can_theta      (ipa)
            osite%mmean_can_vpdef      (opa) = isite%mmean_can_vpdef      (ipa)
            osite%mmean_can_temp       (opa) = isite%mmean_can_temp       (ipa)
            osite%mmean_can_shv        (opa) = isite%mmean_can_shv        (ipa)
            osite%mmean_can_co2        (opa) = isite%mmean_can_co2        (ipa)
            osite%mmean_can_rhos       (opa) = isite%mmean_can_rhos       (ipa)
            osite%mmean_can_prss       (opa) = isite%mmean_can_prss       (ipa)
            osite%mmean_gnd_temp       (opa) = isite%mmean_gnd_temp       (ipa)
            osite%mmean_gnd_shv        (opa) = isite%mmean_gnd_shv        (ipa)
            osite%mmean_can_ggnd       (opa) = isite%mmean_can_ggnd       (ipa)
            osite%mmean_sfcw_depth     (opa) = isite%mmean_sfcw_depth     (ipa)
            osite%mmean_sfcw_energy    (opa) = isite%mmean_sfcw_energy    (ipa)
            osite%mmean_sfcw_mass      (opa) = isite%mmean_sfcw_mass      (ipa)
            osite%mmean_sfcw_temp      (opa) = isite%mmean_sfcw_temp      (ipa)
            osite%mmean_sfcw_fliq      (opa) = isite%mmean_sfcw_fliq      (ipa)
            osite%mmean_rshort_gnd     (opa) = isite%mmean_rshort_gnd     (ipa)
            osite%mmean_par_gnd        (opa) = isite%mmean_par_gnd        (ipa)
            osite%mmean_rlong_gnd      (opa) = isite%mmean_rlong_gnd      (ipa)
            osite%mmean_rlongup        (opa) = isite%mmean_rlongup        (ipa)
            osite%mmean_parup          (opa) = isite%mmean_parup          (ipa)
            osite%mmean_nirup          (opa) = isite%mmean_nirup          (ipa)
            osite%mmean_rshortup       (opa) = isite%mmean_rshortup       (ipa)
            osite%mmean_rnet           (opa) = isite%mmean_rnet           (ipa)
            osite%mmean_albedo         (opa) = isite%mmean_albedo         (ipa)
            osite%mmean_albedo_par     (opa) = isite%mmean_albedo_par     (ipa)
            osite%mmean_albedo_nir     (opa) = isite%mmean_albedo_nir     (ipa)
            osite%mmean_rlong_albedo   (opa) = isite%mmean_rlong_albedo   (ipa)
            osite%mmean_ustar          (opa) = isite%mmean_ustar          (ipa)
            osite%mmean_tstar          (opa) = isite%mmean_tstar          (ipa)
            osite%mmean_qstar          (opa) = isite%mmean_qstar          (ipa)
            osite%mmean_cstar          (opa) = isite%mmean_cstar          (ipa)
            osite%mmean_carbon_ac      (opa) = isite%mmean_carbon_ac      (ipa)
            osite%mmean_carbon_st      (opa) = isite%mmean_carbon_st      (ipa)
            osite%mmean_vapor_gc       (opa) = isite%mmean_vapor_gc       (ipa)
            osite%mmean_vapor_ac       (opa) = isite%mmean_vapor_ac       (ipa)
            osite%mmean_throughfall    (opa) = isite%mmean_throughfall    (ipa)
            osite%mmean_runoff         (opa) = isite%mmean_runoff         (ipa)
            osite%mmean_drainage       (opa) = isite%mmean_drainage       (ipa)
            osite%mmean_sensible_gc    (opa) = isite%mmean_sensible_gc    (ipa)
            osite%mmean_sensible_ac    (opa) = isite%mmean_sensible_ac    (ipa)
            osite%mmean_qthroughfall   (opa) = isite%mmean_qthroughfall   (ipa)
            osite%mmean_qrunoff        (opa) = isite%mmean_qrunoff        (ipa)
            osite%mmean_qdrainage      (opa) = isite%mmean_qdrainage      (ipa)
            osite%mmean_A_decomp       (opa) = isite%mmean_A_decomp       (ipa)
            osite%mmean_Af_decomp      (opa) = isite%mmean_Af_decomp      (ipa)
            osite%mmean_co2_residual   (opa) = isite%mmean_co2_residual   (ipa)
            osite%mmean_energy_residual(opa) = isite%mmean_energy_residual(ipa)
            osite%mmean_water_residual (opa) = isite%mmean_water_residual (ipa)
            osite%mmsqu_rh             (opa) = isite%mmsqu_rh             (ipa)
            osite%mmsqu_cwd_rh         (opa) = isite%mmsqu_cwd_rh         (ipa)
            osite%mmsqu_nep            (opa) = isite%mmsqu_nep            (ipa)
            osite%mmsqu_rlongup        (opa) = isite%mmsqu_rlongup        (ipa)
            osite%mmsqu_parup          (opa) = isite%mmsqu_parup          (ipa)
            osite%mmsqu_nirup          (opa) = isite%mmsqu_nirup          (ipa)
            osite%mmsqu_rshortup       (opa) = isite%mmsqu_rshortup       (ipa)
            osite%mmsqu_rnet           (opa) = isite%mmsqu_rnet           (ipa)
            osite%mmsqu_albedo         (opa) = isite%mmsqu_albedo         (ipa)
            osite%mmsqu_ustar          (opa) = isite%mmsqu_ustar          (ipa)
            osite%mmsqu_carbon_ac      (opa) = isite%mmsqu_carbon_ac      (ipa)
            osite%mmsqu_carbon_st      (opa) = isite%mmsqu_carbon_st      (ipa)
            osite%mmsqu_vapor_gc       (opa) = isite%mmsqu_vapor_gc       (ipa)
            osite%mmsqu_vapor_ac       (opa) = isite%mmsqu_vapor_ac       (ipa)
            osite%mmsqu_sensible_gc    (opa) = isite%mmsqu_sensible_gc    (ipa)
            osite%mmsqu_sensible_ac    (opa) = isite%mmsqu_sensible_ac    (ipa)
            !------------------------------------------------------------------------------!


            !----- Soil variables. --------------------------------------------------------!
            do m=1,nzg
               osite%mmean_soil_energy(m,opa) = isite%mmean_soil_energy(m,ipa)
               osite%mmean_soil_mstpot(m,opa) = isite%mmean_soil_mstpot(m,ipa)
               osite%mmean_soil_water (m,opa) = isite%mmean_soil_water (m,ipa)
               osite%mmean_soil_temp  (m,opa) = isite%mmean_soil_temp  (m,ipa)
               osite%mmean_soil_fliq  (m,opa) = isite%mmean_soil_fliq  (m,ipa)
               osite%mmean_smoist_gg  (m,opa) = isite%mmean_smoist_gg  (m,ipa)
               osite%mmean_transloss  (m,opa) = isite%mmean_transloss  (m,ipa)
               osite%mmean_sensible_gg(m,opa) = isite%mmean_sensible_gg(m,ipa)
            end do
            !------------------------------------------------------------------------------!
         end if
         !---------------------------------------------------------------------------------!



         !---------------------------------------------------------------------------------!
         !     Mean diel.                                                                  !
         !---------------------------------------------------------------------------------!
         if (writing_dcyc) then
            do n=1,ndcycle
               !----- Scalars. ------------------------------------------------------------!
               osite%qmean_rh             (n,opa) = isite%qmean_rh             (n,ipa)
               osite%qmean_cwd_rh         (n,opa) = isite%qmean_cwd_rh         (n,ipa)
               osite%qmean_nep            (n,opa) = isite%qmean_nep            (n,ipa)
               osite%qmean_rk4step        (n,opa) = isite%qmean_rk4step        (n,ipa)
               osite%qmean_available_water(n,opa) = isite%qmean_available_water(n,ipa)
               osite%qmean_can_theiv      (n,opa) = isite%qmean_can_theiv      (n,ipa)
               osite%qmean_can_theta      (n,opa) = isite%qmean_can_theta      (n,ipa)
               osite%qmean_can_vpdef      (n,opa) = isite%qmean_can_vpdef      (n,ipa)
               osite%qmean_can_temp       (n,opa) = isite%qmean_can_temp       (n,ipa)
               osite%qmean_can_shv        (n,opa) = isite%qmean_can_shv        (n,ipa)
               osite%qmean_can_co2        (n,opa) = isite%qmean_can_co2        (n,ipa)
               osite%qmean_can_rhos       (n,opa) = isite%qmean_can_rhos       (n,ipa)
               osite%qmean_can_prss       (n,opa) = isite%qmean_can_prss       (n,ipa)
               osite%qmean_gnd_temp       (n,opa) = isite%qmean_gnd_temp       (n,ipa)
               osite%qmean_gnd_shv        (n,opa) = isite%qmean_gnd_shv        (n,ipa)
               osite%qmean_can_ggnd       (n,opa) = isite%qmean_can_ggnd       (n,ipa)
               osite%qmean_sfcw_depth     (n,opa) = isite%qmean_sfcw_depth     (n,ipa)
               osite%qmean_sfcw_energy    (n,opa) = isite%qmean_sfcw_energy    (n,ipa)
               osite%qmean_sfcw_mass      (n,opa) = isite%qmean_sfcw_mass      (n,ipa)
               osite%qmean_sfcw_temp      (n,opa) = isite%qmean_sfcw_temp      (n,ipa)
               osite%qmean_sfcw_fliq      (n,opa) = isite%qmean_sfcw_fliq      (n,ipa)
               osite%qmean_rshort_gnd     (n,opa) = isite%qmean_rshort_gnd     (n,ipa)
               osite%qmean_par_gnd        (n,opa) = isite%qmean_par_gnd        (n,ipa)
               osite%qmean_rlong_gnd      (n,opa) = isite%qmean_rlong_gnd      (n,ipa)
               osite%qmean_rlongup        (n,opa) = isite%qmean_rlongup        (n,ipa)
               osite%qmean_parup          (n,opa) = isite%qmean_parup          (n,ipa)
               osite%qmean_nirup          (n,opa) = isite%qmean_nirup          (n,ipa)
               osite%qmean_rshortup       (n,opa) = isite%qmean_rshortup       (n,ipa)
               osite%qmean_rnet           (n,opa) = isite%qmean_rnet           (n,ipa)
               osite%qmean_albedo         (n,opa) = isite%qmean_albedo         (n,ipa)
               osite%qmean_albedo_par     (n,opa) = isite%qmean_albedo_par     (n,ipa)
               osite%qmean_albedo_nir     (n,opa) = isite%qmean_albedo_nir     (n,ipa)
               osite%qmean_rlong_albedo   (n,opa) = isite%qmean_rlong_albedo   (n,ipa)
               osite%qmean_ustar          (n,opa) = isite%qmean_ustar          (n,ipa)
               osite%qmean_tstar          (n,opa) = isite%qmean_tstar          (n,ipa)
               osite%qmean_qstar          (n,opa) = isite%qmean_qstar          (n,ipa)
               osite%qmean_cstar          (n,opa) = isite%qmean_cstar          (n,ipa)
               osite%qmean_carbon_ac      (n,opa) = isite%qmean_carbon_ac      (n,ipa)
               osite%qmean_carbon_st      (n,opa) = isite%qmean_carbon_st      (n,ipa)
               osite%qmean_vapor_gc       (n,opa) = isite%qmean_vapor_gc       (n,ipa)
               osite%qmean_vapor_ac       (n,opa) = isite%qmean_vapor_ac       (n,ipa)
               osite%qmean_throughfall    (n,opa) = isite%qmean_throughfall    (n,ipa)
               osite%qmean_runoff         (n,opa) = isite%qmean_runoff         (n,ipa)
               osite%qmean_drainage       (n,opa) = isite%qmean_drainage       (n,ipa)
               osite%qmean_sensible_gc    (n,opa) = isite%qmean_sensible_gc    (n,ipa)
               osite%qmean_sensible_ac    (n,opa) = isite%qmean_sensible_ac    (n,ipa)
               osite%qmean_qthroughfall   (n,opa) = isite%qmean_qthroughfall   (n,ipa)
               osite%qmean_qrunoff        (n,opa) = isite%qmean_qrunoff        (n,ipa)
               osite%qmean_qdrainage      (n,opa) = isite%qmean_qdrainage      (n,ipa)
               osite%qmsqu_rh             (n,opa) = isite%qmsqu_rh             (n,ipa)
               osite%qmsqu_cwd_rh         (n,opa) = isite%qmsqu_cwd_rh         (n,ipa)
               osite%qmsqu_nep            (n,opa) = isite%qmsqu_nep            (n,ipa)
               osite%qmsqu_rlongup        (n,opa) = isite%qmsqu_rlongup        (n,ipa)
               osite%qmsqu_parup          (n,opa) = isite%qmsqu_parup          (n,ipa)
               osite%qmsqu_nirup          (n,opa) = isite%qmsqu_nirup          (n,ipa)
               osite%qmsqu_rshortup       (n,opa) = isite%qmsqu_rshortup       (n,ipa)
               osite%qmsqu_rnet           (n,opa) = isite%qmsqu_rnet           (n,ipa)
               osite%qmsqu_albedo         (n,opa) = isite%qmsqu_albedo         (n,ipa)
               osite%qmsqu_ustar          (n,opa) = isite%qmsqu_ustar          (n,ipa)
               osite%qmsqu_carbon_ac      (n,opa) = isite%qmsqu_carbon_ac      (n,ipa)
               osite%qmsqu_carbon_st      (n,opa) = isite%qmsqu_carbon_st      (n,ipa)
               osite%qmsqu_vapor_gc       (n,opa) = isite%qmsqu_vapor_gc       (n,ipa)
               osite%qmsqu_vapor_ac       (n,opa) = isite%qmsqu_vapor_ac       (n,ipa)
               osite%qmsqu_sensible_gc    (n,opa) = isite%qmsqu_sensible_gc    (n,ipa)
               osite%qmsqu_sensible_ac    (n,opa) = isite%qmsqu_sensible_ac    (n,ipa)
               !---------------------------------------------------------------------------!


               !----- Soil variables. -----------------------------------------------------!
               do m=1,nzg
                  osite%qmean_soil_energy(m,n,opa) = isite%qmean_soil_energy(m,n,ipa)
                  osite%qmean_soil_mstpot(m,n,opa) = isite%qmean_soil_mstpot(m,n,ipa)
                  osite%qmean_soil_water (m,n,opa) = isite%qmean_soil_water (m,n,ipa)
                  osite%qmean_soil_temp  (m,n,opa) = isite%qmean_soil_temp  (m,n,ipa)
                  osite%qmean_soil_fliq  (m,n,opa) = isite%qmean_soil_fliq  (m,n,ipa)
                  osite%qmean_smoist_gg  (m,n,opa) = isite%qmean_smoist_gg  (m,n,ipa)
                  osite%qmean_transloss  (m,n,opa) = isite%qmean_transloss  (m,n,ipa)
                  osite%qmean_sensible_gg(m,n,opa) = isite%qmean_sensible_gg(m,n,ipa)
               end do
               !---------------------------------------------------------------------------!
            end do
            !------------------------------------------------------------------------------!
         end if
         !---------------------------------------------------------------------------------!
      end do opaloop
      !------------------------------------------------------------------------------------!

      return
   end subroutine copy_sitetype
   !=======================================================================================!
   !=======================================================================================!





   !=======================================================================================!
   !=======================================================================================!
   !      This subroutine copies part of one patch to another (only the ones where mask    !
   ! is .true.).  Please notice the following requirements.                                !
   !                                                                                       !
   !      1. ISITE and LMASK must have dimension MASKSZ                                    !
   !      2. OSITE must have dimension NEWSZ                                               !
   !      3. The number of .true. elements of LMASK must be NEWSZ                          !
   !      This subroutine assumes that the size of vectors in osite is the number of true  !
   ! elements in mask, whilst the size of the vectors in isite must be the size of the     !
   ! mask itself.                                                                          !
   !                                                                                       !
   ! IMPORTANT:  This routine currently assumes that the output site has not allocated     !
   !             any cohort yet, because the length of these nested vectors depend on the  !
   !             donor patch vector sizes.  Never call this sub-routine if osite has been  !
   !             allocated before.                                                         !
   !---------------------------------------------------------------------------------------!
   subroutine copy_sitetype_mask(isite,osite,lmask,isize,osize)


      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(sitetype)                  , target     :: isite   ! Input  (donor) site
      type(sitetype)                  , target     :: osite   ! Output (receptor) site
      integer                         , intent(in) :: isize   ! Input size 
      integer                         , intent(in) :: osize   ! Output size
      logical       , dimension(isize), intent(in) :: lmask   ! Mask
      !----- Local variables. -------------------------------------------------------------!
      integer       , dimension(isize)             :: allind
      integer       , dimension(osize)             :: iindex
      integer                                      :: i
      integer                                      :: o
      integer                                      :: z
      integer                                      :: m
      integer                                      :: n
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      !     Map the indices between the input and the output vectors.                      !
      !------------------------------------------------------------------------------------!
      z = count(lmask)
      !----- Crash if the dimensions don't match. -----------------------------------------!
      if (z /= osize) then
         write (unit=*,fmt='(a,1x,i12)') ' Z     = ',z
         write (unit=*,fmt='(a,1x,i12)') ' OSIZE = ',osize
         call fatal_error('Mask doesn''t have OSIZE .true. elements'                       &
                         ,'copy_sitetype_mask','ed_state_vars.f90')
      end if

      !----- Save the indices from where we are going to copy. ----------------------------!
      allind = (/ (i,i=1,isize) /)
      iindex = pack(allind,lmask)
      !------------------------------------------------------------------------------------!

      !------------------------------------------------------------------------------------!
      !      Copy the cohorts.                                                             !
      !------------------------------------------------------------------------------------!
      do o=1,osize
         i = iindex(o)
         call allocate_patchtype(osite%patch(o),isite%patch(i)%ncohorts)
         call copy_patchtype    (isite%patch(i),osite%patch(o)                             &
                                ,1,isite%patch(i)%ncohorts,1,osite%patch(o)%ncohorts)
      end do
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      !      We break the subroutines into smaller pieces so Fortran doesn't complain...   !
      !------------------------------------------------------------------------------------!
      call copy_sitetype_mask_inst (isite,osite,z,lmask,isize,osize)
      call copy_sitetype_mask_fmean(isite,osite,z,lmask,isize,osize)
      if (writing_long) call copy_sitetype_mask_dmean(isite,osite,z,lmask,isize,osize)
      if (writing_eorq) call copy_sitetype_mask_mmean(isite,osite,z,lmask,isize,osize)
      if (writing_dcyc) call copy_sitetype_mask_qmean(isite,osite,z,lmask,isize,osize)
      !------------------------------------------------------------------------------------!

      return
   end subroutine copy_sitetype_mask
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This sub-routine copies the variables that are not fmean, dmean, mmean, mmsqu,    !
   ! qmean, and qmsqu.  This sub-routine should be called only from copy_sitetype_mask!!!  !
   !---------------------------------------------------------------------------------------!
   subroutine copy_sitetype_mask_inst (isite,osite,z,lmask,isize,osize)


      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(sitetype)                  , target     :: isite   ! Input  (donor) site
      type(sitetype)                  , target     :: osite   ! Output (receptor) site
      integer                         , intent(in) :: z       ! Number of elements
      integer                         , intent(in) :: isize   ! Input size 
      integer                         , intent(in) :: osize   ! Output size
      logical       , dimension(isize), intent(in) :: lmask   ! Mask
      !----- Local variables. -------------------------------------------------------------!
      integer                                      :: i
      integer                                      :: o
      integer                                      :: m
      integer                                      :: n
      !------------------------------------------------------------------------------------!


      !----- Scalars. ---------------------------------------------------------------------!
      osite%paco_id                   (1:z) = pack(isite%paco_id                   ,lmask)
      osite%paco_n                    (1:z) = pack(isite%paco_n                    ,lmask)
      osite%cohort_count              (1:z) = pack(isite%cohort_count              ,lmask)
      osite%pname                     (1:z) = pack(isite%pname                     ,lmask)
      osite%area                      (1:z) = pack(isite%area                      ,lmask)
      osite%age                       (1:z) = pack(isite%age                       ,lmask)
      osite%dist_type                 (1:z) = pack(isite%dist_type                 ,lmask)
      osite%fast_soil_C               (1:z) = pack(isite%fast_soil_C               ,lmask)
      osite%slow_soil_C               (1:z) = pack(isite%slow_soil_C               ,lmask)
      osite%structural_soil_C         (1:z) = pack(isite%structural_soil_C         ,lmask)
      osite%structural_soil_L         (1:z) = pack(isite%structural_soil_L         ,lmask)
      osite%mineralized_soil_N        (1:z) = pack(isite%mineralized_soil_N        ,lmask)
      osite%fast_soil_N               (1:z) = pack(isite%fast_soil_N               ,lmask)
      osite%sum_dgd                   (1:z) = pack(isite%sum_dgd                   ,lmask)
      osite%sum_chd                   (1:z) = pack(isite%sum_chd                   ,lmask)
      osite%can_theiv                 (1:z) = pack(isite%can_theiv                 ,lmask)
      osite%can_vpdef                 (1:z) = pack(isite%can_vpdef                 ,lmask)
      osite%can_temp                  (1:z) = pack(isite%can_temp                  ,lmask)
      osite%can_temp_pv               (1:z) = pack(isite%can_temp_pv               ,lmask)
      osite%can_shv                   (1:z) = pack(isite%can_shv                   ,lmask)
      osite%can_co2                   (1:z) = pack(isite%can_co2                   ,lmask)
      osite%can_rhos                  (1:z) = pack(isite%can_rhos                  ,lmask)
      osite%can_prss                  (1:z) = pack(isite%can_prss                  ,lmask)
      osite%can_theta                 (1:z) = pack(isite%can_theta                 ,lmask)
      osite%can_depth                 (1:z) = pack(isite%can_depth                 ,lmask)
      osite%opencan_frac              (1:z) = pack(isite%opencan_frac              ,lmask)
      osite%ggbare                    (1:z) = pack(isite%ggbare                    ,lmask)
      osite%ggveg                     (1:z) = pack(isite%ggveg                     ,lmask)
      osite%ggnet                     (1:z) = pack(isite%ggnet                     ,lmask)
      osite%ggsoil                    (1:z) = pack(isite%ggsoil                    ,lmask)
      osite%nlev_sfcwater             (1:z) = pack(isite%nlev_sfcwater             ,lmask)
      osite%ground_shv                (1:z) = pack(isite%ground_shv                ,lmask)
      osite%ground_ssh                (1:z) = pack(isite%ground_ssh                ,lmask)
      osite%ground_temp               (1:z) = pack(isite%ground_temp               ,lmask)
      osite%ground_fliq               (1:z) = pack(isite%ground_fliq               ,lmask)
      osite%rough                     (1:z) = pack(isite%rough                     ,lmask)
      osite%par_l_max                 (1:z) = pack(isite%par_l_max                 ,lmask)
      osite%par_l_beam_max            (1:z) = pack(isite%par_l_beam_max            ,lmask)
      osite%par_l_diffuse_max         (1:z) = pack(isite%par_l_diffuse_max         ,lmask)
      osite%avg_daily_temp            (1:z) = pack(isite%avg_daily_temp            ,lmask)
      osite%avg_monthly_gndwater      (1:z) = pack(isite%avg_monthly_gndwater      ,lmask)
      osite%avg_monthly_waterdef      (1:z) = pack(isite%avg_monthly_waterdef      ,lmask)
      osite%wbudget_loss2atm          (1:z) = pack(isite%wbudget_loss2atm          ,lmask)
      osite%wbudget_denseffect        (1:z) = pack(isite%wbudget_denseffect        ,lmask)
      osite%wbudget_precipgain        (1:z) = pack(isite%wbudget_precipgain        ,lmask)
      osite%wbudget_loss2runoff       (1:z) = pack(isite%wbudget_loss2runoff       ,lmask)
      osite%wbudget_loss2drainage     (1:z) = pack(isite%wbudget_loss2drainage     ,lmask)
      osite%wbudget_initialstorage    (1:z) = pack(isite%wbudget_initialstorage    ,lmask)
      osite%wbudget_residual          (1:z) = pack(isite%wbudget_residual          ,lmask)
      osite%ebudget_loss2atm          (1:z) = pack(isite%ebudget_loss2atm          ,lmask)
      osite%ebudget_denseffect        (1:z) = pack(isite%ebudget_denseffect        ,lmask)
      osite%ebudget_prsseffect        (1:z) = pack(isite%ebudget_prsseffect        ,lmask)
      osite%ebudget_loss2runoff       (1:z) = pack(isite%ebudget_loss2runoff       ,lmask)
      osite%ebudget_loss2drainage     (1:z) = pack(isite%ebudget_loss2drainage     ,lmask)
      osite%ebudget_netrad            (1:z) = pack(isite%ebudget_netrad            ,lmask)
      osite%ebudget_precipgain        (1:z) = pack(isite%ebudget_precipgain        ,lmask)
      osite%ebudget_initialstorage    (1:z) = pack(isite%ebudget_initialstorage    ,lmask)
      osite%ebudget_residual          (1:z) = pack(isite%ebudget_residual          ,lmask)
      osite%co2budget_initialstorage  (1:z) = pack(isite%co2budget_initialstorage  ,lmask)
      osite%co2budget_residual        (1:z) = pack(isite%co2budget_residual        ,lmask)
      osite%co2budget_loss2atm        (1:z) = pack(isite%co2budget_loss2atm        ,lmask)
      osite%co2budget_denseffect      (1:z) = pack(isite%co2budget_denseffect      ,lmask)
      osite%co2budget_gpp             (1:z) = pack(isite%co2budget_gpp             ,lmask)
      osite%co2budget_plresp          (1:z) = pack(isite%co2budget_plresp          ,lmask)
      osite%co2budget_rh              (1:z) = pack(isite%co2budget_rh              ,lmask)
      osite%today_A_decomp            (1:z) = pack(isite%today_A_decomp            ,lmask)
      osite%today_Af_decomp           (1:z) = pack(isite%today_Af_decomp           ,lmask)
      osite%veg_rough                 (1:z) = pack(isite%veg_rough                 ,lmask)
      osite%veg_height                (1:z) = pack(isite%veg_height                ,lmask)
      osite%veg_displace              (1:z) = pack(isite%veg_displace              ,lmask)
      osite%fsc_in                    (1:z) = pack(isite%fsc_in                    ,lmask)
      osite%ssc_in                    (1:z) = pack(isite%ssc_in                    ,lmask)
      osite%ssl_in                    (1:z) = pack(isite%ssl_in                    ,lmask)
      osite%fsn_in                    (1:z) = pack(isite%fsn_in                    ,lmask)
      osite%total_plant_nitrogen_uptake(1:z) =                                             &
                                             pack(isite%total_plant_nitrogen_uptake,lmask)
      osite%mineralized_N_loss        (1:z) = pack(isite%mineralized_N_loss        ,lmask)
      osite%mineralized_N_input       (1:z) = pack(isite%mineralized_N_input       ,lmask)
      osite%rshort_g                  (1:z) = pack(isite%rshort_g                  ,lmask)
      osite%rshort_g_beam             (1:z) = pack(isite%rshort_g_beam             ,lmask)
      osite%rshort_g_diffuse          (1:z) = pack(isite%rshort_g_diffuse          ,lmask)
      osite%par_g                     (1:z) = pack(isite%par_g                     ,lmask)
      osite%par_g_beam                (1:z) = pack(isite%par_g_beam                ,lmask)
      osite%par_g_diffuse             (1:z) = pack(isite%par_g_diffuse             ,lmask)
      osite%par_b                     (1:z) = pack(isite%par_b                     ,lmask)
      osite%par_b_beam                (1:z) = pack(isite%par_b_beam                ,lmask)
      osite%par_b_diffuse             (1:z) = pack(isite%par_b_diffuse             ,lmask)
      osite%nir_b                     (1:z) = pack(isite%nir_b                     ,lmask)
      osite%nir_b_beam                (1:z) = pack(isite%nir_b_beam                ,lmask)
      osite%nir_b_diffuse             (1:z) = pack(isite%nir_b_diffuse             ,lmask)
      osite%rlong_g                   (1:z) = pack(isite%rlong_g                   ,lmask)
      osite%rlong_s                   (1:z) = pack(isite%rlong_s                   ,lmask)
      osite%albedo                    (1:z) = pack(isite%albedo                    ,lmask)
      osite%albedo_par                (1:z) = pack(isite%albedo_par                ,lmask)
      osite%albedo_nir                (1:z) = pack(isite%albedo_nir                ,lmask)
      osite%rlong_albedo              (1:z) = pack(isite%rlong_albedo              ,lmask)
      osite%rnet                      (1:z) = pack(isite%rnet                      ,lmask)
      osite%rlongup                   (1:z) = pack(isite%rlongup                   ,lmask)
      osite%parup                     (1:z) = pack(isite%parup                     ,lmask)
      osite%nirup                     (1:z) = pack(isite%nirup                     ,lmask)
      osite%rshortup                  (1:z) = pack(isite%rshortup                  ,lmask)
      osite%total_sfcw_depth          (1:z) = pack(isite%total_sfcw_depth          ,lmask)
      osite%snowfac                   (1:z) = pack(isite%snowfac                   ,lmask)
      osite%A_decomp                  (1:z) = pack(isite%A_decomp                  ,lmask)
      osite%f_decomp                  (1:z) = pack(isite%f_decomp                  ,lmask)
      osite%rh                        (1:z) = pack(isite%rh                        ,lmask)
      osite%cwd_rh                    (1:z) = pack(isite%cwd_rh                    ,lmask)
      osite%plant_ag_biomass          (1:z) = pack(isite%plant_ag_biomass          ,lmask)
      osite%htry                      (1:z) = pack(isite%htry                      ,lmask)
      osite%hprev                     (1:z) = pack(isite%hprev                     ,lmask)
      osite%ustar                     (1:z) = pack(isite%ustar                     ,lmask)
      osite%tstar                     (1:z) = pack(isite%tstar                     ,lmask)
      osite%qstar                     (1:z) = pack(isite%qstar                     ,lmask)
      osite%cstar                     (1:z) = pack(isite%cstar                     ,lmask)
      osite%zeta                      (1:z) = pack(isite%zeta                      ,lmask)
      osite%ribulk                    (1:z) = pack(isite%ribulk                    ,lmask)
      osite%upwp                      (1:z) = pack(isite%upwp                      ,lmask)
      osite%qpwp                      (1:z) = pack(isite%qpwp                      ,lmask)
      osite%cpwp                      (1:z) = pack(isite%cpwp                      ,lmask)
      osite%tpwp                      (1:z) = pack(isite%tpwp                      ,lmask)
      osite%wpwp                      (1:z) = pack(isite%wpwp                      ,lmask)
      osite%watertable                (1:z) = pack(isite%watertable                ,lmask)
      osite%moist_dz                  (1:z) = pack(isite%moist_dz                  ,lmask)
      osite%ksat                      (1:z) = pack(isite%ksat                      ,lmask)
      osite%soil_sat_energy           (1:z) = pack(isite%soil_sat_energy           ,lmask)
      osite%soil_sat_water            (1:z) = pack(isite%soil_sat_water            ,lmask)
      osite%soil_sat_heat             (1:z) = pack(isite%soil_sat_heat             ,lmask)
      osite%runoff_rate               (1:z) = pack(isite%runoff_rate               ,lmask)
      osite%runoff                    (1:z) = pack(isite%runoff                    ,lmask)
      osite%qrunoff                   (1:z) = pack(isite%qrunoff                   ,lmask)
      !------------------------------------------------------------------------------------!

      !----- Temporary pounding/snow layer variables. -------------------------------------!
      do m=1,nzs
         osite%sfcwater_mass   (m,1:z) = pack(isite%sfcwater_mass   (m,:),lmask)
         osite%sfcwater_energy (m,1:z) = pack(isite%sfcwater_energy (m,:),lmask)
         osite%sfcwater_depth  (m,1:z) = pack(isite%sfcwater_depth  (m,:),lmask)
         osite%sfcwater_tempk  (m,1:z) = pack(isite%sfcwater_tempk  (m,:),lmask)
         osite%sfcwater_fracliq(m,1:z) = pack(isite%sfcwater_fracliq(m,:),lmask)
         osite%rshort_s        (m,1:z) = pack(isite%rshort_s        (m,:),lmask)
         osite%rshort_s_beam   (m,1:z) = pack(isite%rshort_s_beam   (m,:),lmask)
         osite%rshort_s_diffuse(m,1:z) = pack(isite%rshort_s_diffuse(m,:),lmask)
         osite%par_s           (m,1:z) = pack(isite%par_s           (m,:),lmask)
         osite%par_s_beam      (m,1:z) = pack(isite%par_s_beam      (m,:),lmask)
         osite%par_s_diffuse   (m,1:z) = pack(isite%par_s_diffuse   (m,:),lmask)
      end do
      !------------------------------------------------------------------------------------!


      !----- Soil variables. --------------------------------------------------------------!
      do m=1,nzg
         osite%soil_energy      (m,1:z) = pack(isite%soil_energy      (m,:),lmask)
         osite%soil_mstpot      (m,1:z) = pack(isite%soil_mstpot      (m,:),lmask)
         osite%soil_water       (m,1:z) = pack(isite%soil_water       (m,:),lmask)
         osite%soil_tempk       (m,1:z) = pack(isite%soil_tempk       (m,:),lmask)
         osite%soil_fracliq     (m,1:z) = pack(isite%soil_fracliq     (m,:),lmask)
         osite%rootdense        (m,1:z) = pack(isite%rootdense        (m,:),lmask)
      end do
      !------------------------------------------------------------------------------------!


      !----- PFT variables. ---------------------------------------------------------------!
      do m=1,n_pft
         osite%A_o_max          (  m,1:z) = pack(isite%A_o_max       (  m,:),lmask)
         osite%A_c_max          (  m,1:z) = pack(isite%A_c_max       (  m,:),lmask)
         osite%repro            (  m,1:z) = pack(isite%repro         (  m,:),lmask)
         do n=1,ff_nhgt
            osite%cumlai_profile(m,n,1:z) = pack(isite%cumlai_profile(m,n,:),lmask)
         end do
      end do
      !------------------------------------------------------------------------------------!


      !----- "3" variables. ---------------------------------------------------------------!
      do m=1,3
         osite%runoff_A(m,1:z) = pack(isite%runoff_A(m,:),lmask)
      end do
      !------------------------------------------------------------------------------------!



      return
   end subroutine copy_sitetype_mask_inst
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This sub-routine copies the variables that are fmean.  This sub-routine should be !
   ! called only from copy_sitetype_mask!!!                                                !
   !---------------------------------------------------------------------------------------!
   subroutine copy_sitetype_mask_fmean(isite,osite,z,lmask,isize,osize)


      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(sitetype)                  , target     :: isite   ! Input  (donor) site
      type(sitetype)                  , target     :: osite   ! Output (receptor) site
      integer                         , intent(in) :: z       ! Number of elements
      integer                         , intent(in) :: isize   ! Input size 
      integer                         , intent(in) :: osize   ! Output size
      logical       , dimension(isize), intent(in) :: lmask   ! Mask
      !----- Local variables. -------------------------------------------------------------!
      integer                                      :: i
      integer                                      :: o
      integer                                      :: m
      integer                                      :: n
      !------------------------------------------------------------------------------------!

      osite%fmean_rh                  (1:z) = pack(isite%fmean_rh                  ,lmask)
      osite%fmean_cwd_rh              (1:z) = pack(isite%fmean_cwd_rh              ,lmask)
      osite%fmean_nep                 (1:z) = pack(isite%fmean_nep                 ,lmask)
      osite%fmean_rk4step             (1:z) = pack(isite%fmean_rk4step             ,lmask)
      osite%fmean_available_water     (1:z) = pack(isite%fmean_available_water     ,lmask)
      osite%fmean_can_theiv           (1:z) = pack(isite%fmean_can_theiv           ,lmask)
      osite%fmean_can_theta           (1:z) = pack(isite%fmean_can_theta           ,lmask)
      osite%fmean_can_vpdef           (1:z) = pack(isite%fmean_can_vpdef           ,lmask)
      osite%fmean_can_temp            (1:z) = pack(isite%fmean_can_temp            ,lmask)
      osite%fmean_can_shv             (1:z) = pack(isite%fmean_can_shv             ,lmask)
      osite%fmean_can_co2             (1:z) = pack(isite%fmean_can_co2             ,lmask)
      osite%fmean_can_rhos            (1:z) = pack(isite%fmean_can_rhos            ,lmask)
      osite%fmean_can_prss            (1:z) = pack(isite%fmean_can_prss            ,lmask)
      osite%fmean_gnd_temp            (1:z) = pack(isite%fmean_gnd_temp            ,lmask)
      osite%fmean_gnd_shv             (1:z) = pack(isite%fmean_gnd_shv             ,lmask)
      osite%fmean_can_ggnd            (1:z) = pack(isite%fmean_can_ggnd            ,lmask)
      osite%fmean_sfcw_depth          (1:z) = pack(isite%fmean_sfcw_depth          ,lmask)
      osite%fmean_sfcw_energy         (1:z) = pack(isite%fmean_sfcw_energy         ,lmask)
      osite%fmean_sfcw_mass           (1:z) = pack(isite%fmean_sfcw_mass           ,lmask)
      osite%fmean_sfcw_temp           (1:z) = pack(isite%fmean_sfcw_temp           ,lmask)
      osite%fmean_sfcw_fliq           (1:z) = pack(isite%fmean_sfcw_fliq           ,lmask)
      osite%fmean_rshort_gnd          (1:z) = pack(isite%fmean_rshort_gnd          ,lmask)
      osite%fmean_par_gnd             (1:z) = pack(isite%fmean_par_gnd             ,lmask)
      osite%fmean_rlong_gnd           (1:z) = pack(isite%fmean_rlong_gnd           ,lmask)
      osite%fmean_rlongup             (1:z) = pack(isite%fmean_rlongup             ,lmask)
      osite%fmean_parup               (1:z) = pack(isite%fmean_parup               ,lmask)
      osite%fmean_nirup               (1:z) = pack(isite%fmean_nirup               ,lmask)
      osite%fmean_rshortup            (1:z) = pack(isite%fmean_rshortup            ,lmask)
      osite%fmean_rnet                (1:z) = pack(isite%fmean_rnet                ,lmask)
      osite%fmean_albedo              (1:z) = pack(isite%fmean_albedo              ,lmask)
      osite%fmean_albedo_par          (1:z) = pack(isite%fmean_albedo_par          ,lmask)
      osite%fmean_albedo_nir          (1:z) = pack(isite%fmean_albedo_nir          ,lmask)
      osite%fmean_rlong_albedo        (1:z) = pack(isite%fmean_rlong_albedo        ,lmask)
      osite%fmean_ustar               (1:z) = pack(isite%fmean_ustar               ,lmask)
      osite%fmean_tstar               (1:z) = pack(isite%fmean_tstar               ,lmask)
      osite%fmean_qstar               (1:z) = pack(isite%fmean_qstar               ,lmask)
      osite%fmean_cstar               (1:z) = pack(isite%fmean_cstar               ,lmask)
      osite%fmean_carbon_ac           (1:z) = pack(isite%fmean_carbon_ac           ,lmask)
      osite%fmean_carbon_st           (1:z) = pack(isite%fmean_carbon_st           ,lmask)
      osite%fmean_vapor_gc            (1:z) = pack(isite%fmean_vapor_gc            ,lmask)
      osite%fmean_vapor_ac            (1:z) = pack(isite%fmean_vapor_ac            ,lmask)
      osite%fmean_throughfall         (1:z) = pack(isite%fmean_throughfall         ,lmask)
      osite%fmean_runoff              (1:z) = pack(isite%fmean_runoff              ,lmask)
      osite%fmean_drainage            (1:z) = pack(isite%fmean_drainage            ,lmask)
      osite%fmean_sensible_gc         (1:z) = pack(isite%fmean_sensible_gc         ,lmask)
      osite%fmean_sensible_ac         (1:z) = pack(isite%fmean_sensible_ac         ,lmask)
      osite%fmean_qthroughfall        (1:z) = pack(isite%fmean_qthroughfall        ,lmask)
      osite%fmean_qrunoff             (1:z) = pack(isite%fmean_qrunoff             ,lmask)
      osite%fmean_qdrainage           (1:z) = pack(isite%fmean_qdrainage           ,lmask)


      !----- Soil variables. --------------------------------------------------------------!
      do m=1,nzg
         osite%fmean_soil_energy(m,1:z) = pack(isite%fmean_soil_energy(m,:),lmask)
         osite%fmean_soil_mstpot(m,1:z) = pack(isite%fmean_soil_mstpot(m,:),lmask)
         osite%fmean_soil_water (m,1:z) = pack(isite%fmean_soil_water (m,:),lmask)
         osite%fmean_soil_temp  (m,1:z) = pack(isite%fmean_soil_temp  (m,:),lmask)
         osite%fmean_soil_fliq  (m,1:z) = pack(isite%fmean_soil_fliq  (m,:),lmask)
         osite%fmean_smoist_gg  (m,1:z) = pack(isite%fmean_smoist_gg  (m,:),lmask)
         osite%fmean_transloss  (m,1:z) = pack(isite%fmean_transloss  (m,:),lmask)
         osite%fmean_sensible_gg(m,1:z) = pack(isite%fmean_sensible_gg(m,:),lmask)
      end do
      !------------------------------------------------------------------------------------!



      return
   end subroutine copy_sitetype_mask_fmean
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This sub-routine copies the variables that are dmean.  This sub-routine should be !
   ! called only from copy_sitetype_mask!!!                                                !
   !---------------------------------------------------------------------------------------!
   subroutine copy_sitetype_mask_dmean(isite,osite,z,lmask,isize,osize)


      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(sitetype)                  , target     :: isite   ! Input  (donor) site
      type(sitetype)                  , target     :: osite   ! Output (receptor) site
      integer                         , intent(in) :: z       ! Number of elements
      integer                         , intent(in) :: isize   ! Input size 
      integer                         , intent(in) :: osize   ! Output size
      logical       , dimension(isize), intent(in) :: lmask   ! Mask
      !----- Local variables. -------------------------------------------------------------!
      integer                                      :: i
      integer                                      :: o
      integer                                      :: m
      integer                                      :: n
      !------------------------------------------------------------------------------------!



      !----- Scalars. ---------------------------------------------------------------------!
      osite%dmean_A_decomp       (1:z) = pack(isite%dmean_A_decomp       (:),lmask)
      osite%dmean_Af_decomp      (1:z) = pack(isite%dmean_Af_decomp      (:),lmask)
      osite%dmean_co2_residual   (1:z) = pack(isite%dmean_co2_residual   (:),lmask)
      osite%dmean_energy_residual(1:z) = pack(isite%dmean_energy_residual(:),lmask)
      osite%dmean_water_residual (1:z) = pack(isite%dmean_water_residual (:),lmask)
      osite%dmean_rh             (1:z) = pack(isite%dmean_rh             (:),lmask)
      osite%dmean_cwd_rh         (1:z) = pack(isite%dmean_cwd_rh         (:),lmask)
      osite%dmean_nep            (1:z) = pack(isite%dmean_nep            (:),lmask)
      osite%dmean_rk4step        (1:z) = pack(isite%dmean_rk4step        (:),lmask)
      osite%dmean_available_water(1:z) = pack(isite%dmean_available_water(:),lmask)
      osite%dmean_can_theiv      (1:z) = pack(isite%dmean_can_theiv      (:),lmask)
      osite%dmean_can_theta      (1:z) = pack(isite%dmean_can_theta      (:),lmask)
      osite%dmean_can_vpdef      (1:z) = pack(isite%dmean_can_vpdef      (:),lmask)
      osite%dmean_can_temp       (1:z) = pack(isite%dmean_can_temp       (:),lmask)
      osite%dmean_can_shv        (1:z) = pack(isite%dmean_can_shv        (:),lmask)
      osite%dmean_can_co2        (1:z) = pack(isite%dmean_can_co2        (:),lmask)
      osite%dmean_can_rhos       (1:z) = pack(isite%dmean_can_rhos       (:),lmask)
      osite%dmean_can_prss       (1:z) = pack(isite%dmean_can_prss       (:),lmask)
      osite%dmean_gnd_temp       (1:z) = pack(isite%dmean_gnd_temp       (:),lmask)
      osite%dmean_gnd_shv        (1:z) = pack(isite%dmean_gnd_shv        (:),lmask)
      osite%dmean_can_ggnd       (1:z) = pack(isite%dmean_can_ggnd       (:),lmask)
      osite%dmean_sfcw_depth     (1:z) = pack(isite%dmean_sfcw_depth     (:),lmask)
      osite%dmean_sfcw_energy    (1:z) = pack(isite%dmean_sfcw_energy    (:),lmask)
      osite%dmean_sfcw_mass      (1:z) = pack(isite%dmean_sfcw_mass      (:),lmask)
      osite%dmean_sfcw_temp      (1:z) = pack(isite%dmean_sfcw_temp      (:),lmask)
      osite%dmean_sfcw_fliq      (1:z) = pack(isite%dmean_sfcw_fliq      (:),lmask)
      osite%dmean_rshort_gnd     (1:z) = pack(isite%dmean_rshort_gnd     (:),lmask)
      osite%dmean_par_gnd        (1:z) = pack(isite%dmean_par_gnd        (:),lmask)
      osite%dmean_rlong_gnd      (1:z) = pack(isite%dmean_rlong_gnd      (:),lmask)
      osite%dmean_rlongup        (1:z) = pack(isite%dmean_rlongup        (:),lmask)
      osite%dmean_parup          (1:z) = pack(isite%dmean_parup          (:),lmask)
      osite%dmean_nirup          (1:z) = pack(isite%dmean_nirup          (:),lmask)
      osite%dmean_rshortup       (1:z) = pack(isite%dmean_rshortup       (:),lmask)
      osite%dmean_rnet           (1:z) = pack(isite%dmean_rnet           (:),lmask)
      osite%dmean_albedo         (1:z) = pack(isite%dmean_albedo         (:),lmask)
      osite%dmean_albedo_par     (1:z) = pack(isite%dmean_albedo_par     (:),lmask)
      osite%dmean_albedo_nir     (1:z) = pack(isite%dmean_albedo_nir     (:),lmask)
      osite%dmean_rlong_albedo   (1:z) = pack(isite%dmean_rlong_albedo   (:),lmask)
      osite%dmean_ustar          (1:z) = pack(isite%dmean_ustar          (:),lmask)
      osite%dmean_tstar          (1:z) = pack(isite%dmean_tstar          (:),lmask)
      osite%dmean_qstar          (1:z) = pack(isite%dmean_qstar          (:),lmask)
      osite%dmean_cstar          (1:z) = pack(isite%dmean_cstar          (:),lmask)
      osite%dmean_carbon_ac      (1:z) = pack(isite%dmean_carbon_ac      (:),lmask)
      osite%dmean_carbon_st      (1:z) = pack(isite%dmean_carbon_st      (:),lmask)
      osite%dmean_vapor_gc       (1:z) = pack(isite%dmean_vapor_gc       (:),lmask)
      osite%dmean_vapor_ac       (1:z) = pack(isite%dmean_vapor_ac       (:),lmask)
      osite%dmean_throughfall    (1:z) = pack(isite%dmean_throughfall    (:),lmask)
      osite%dmean_runoff         (1:z) = pack(isite%dmean_runoff         (:),lmask)
      osite%dmean_drainage       (1:z) = pack(isite%dmean_drainage       (:),lmask)
      osite%dmean_sensible_gc    (1:z) = pack(isite%dmean_sensible_gc    (:),lmask)
      osite%dmean_sensible_ac    (1:z) = pack(isite%dmean_sensible_ac    (:),lmask)
      osite%dmean_qthroughfall   (1:z) = pack(isite%dmean_qthroughfall   (:),lmask)
      osite%dmean_qrunoff        (1:z) = pack(isite%dmean_qrunoff        (:),lmask)
      osite%dmean_qdrainage      (1:z) = pack(isite%dmean_qdrainage      (:),lmask)
      !------------------------------------------------------------------------------------!


      !----- Soil variables. --------------------------------------------------------------!
      do m=1,nzg
         osite%dmean_soil_energy(m,1:z) = pack(isite%dmean_soil_energy(m,:),lmask)
         osite%dmean_soil_mstpot(m,1:z) = pack(isite%dmean_soil_mstpot(m,:),lmask)
         osite%dmean_soil_water (m,1:z) = pack(isite%dmean_soil_water (m,:),lmask)
         osite%dmean_soil_temp  (m,1:z) = pack(isite%dmean_soil_temp  (m,:),lmask)
         osite%dmean_soil_fliq  (m,1:z) = pack(isite%dmean_soil_fliq  (m,:),lmask)
         osite%dmean_smoist_gg  (m,1:z) = pack(isite%dmean_smoist_gg  (m,:),lmask)
         osite%dmean_transloss  (m,1:z) = pack(isite%dmean_transloss  (m,:),lmask)
         osite%dmean_sensible_gg(m,1:z) = pack(isite%dmean_sensible_gg(m,:),lmask)
      end do
      !------------------------------------------------------------------------------------!



      return
   end subroutine copy_sitetype_mask_dmean
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This sub-routine copies the variables that are mmean.  This sub-routine should be !
   ! called only from copy_sitetype_mask!!!                                                !
   !---------------------------------------------------------------------------------------!
   subroutine copy_sitetype_mask_mmean(isite,osite,z,lmask,isize,osize)


      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(sitetype)                  , target     :: isite   ! Input  (donor) site
      type(sitetype)                  , target     :: osite   ! Output (receptor) site
      integer                         , intent(in) :: z       ! Number of elements
      integer                         , intent(in) :: isize   ! Input size 
      integer                         , intent(in) :: osize   ! Output size
      logical       , dimension(isize), intent(in) :: lmask   ! Mask
      !----- Local variables. -------------------------------------------------------------!
      integer                                      :: i
      integer                                      :: o
      integer                                      :: m
      integer                                      :: n
      !------------------------------------------------------------------------------------!




      !----- Scalars. ---------------------------------------------------------------------!
      osite%mmean_fast_soil_c    (1:z) = pack(isite%mmean_fast_soil_c    (:),lmask)
      osite%mmean_slow_soil_c    (1:z) = pack(isite%mmean_slow_soil_c    (:),lmask)
      osite%mmean_struct_soil_c  (1:z) = pack(isite%mmean_struct_soil_c  (:),lmask)
      osite%mmean_struct_soil_l  (1:z) = pack(isite%mmean_struct_soil_l  (:),lmask)
      osite%mmean_fast_soil_n    (1:z) = pack(isite%mmean_fast_soil_n    (:),lmask)
      osite%mmean_mineral_soil_n (1:z) = pack(isite%mmean_mineral_soil_n (:),lmask)
      osite%mmean_co2_residual   (1:z) = pack(isite%mmean_co2_residual   (:),lmask)
      osite%mmean_energy_residual(1:z) = pack(isite%mmean_energy_residual(:),lmask)
      osite%mmean_water_residual (1:z) = pack(isite%mmean_water_residual (:),lmask)
      osite%mmean_rh             (1:z) = pack(isite%mmean_rh             (:),lmask)
      osite%mmean_cwd_rh         (1:z) = pack(isite%mmean_cwd_rh         (:),lmask)
      osite%mmean_nep            (1:z) = pack(isite%mmean_nep            (:),lmask)
      osite%mmean_A_decomp       (1:z) = pack(isite%mmean_A_decomp       (:),lmask)
      osite%mmean_Af_decomp      (1:z) = pack(isite%mmean_Af_decomp      (:),lmask)
      osite%mmean_rk4step        (1:z) = pack(isite%mmean_rk4step        (:),lmask)
      osite%mmean_available_water(1:z) = pack(isite%mmean_available_water(:),lmask)
      osite%mmean_can_theiv      (1:z) = pack(isite%mmean_can_theiv      (:),lmask)
      osite%mmean_can_theta      (1:z) = pack(isite%mmean_can_theta      (:),lmask)
      osite%mmean_can_vpdef      (1:z) = pack(isite%mmean_can_vpdef      (:),lmask)
      osite%mmean_can_temp       (1:z) = pack(isite%mmean_can_temp       (:),lmask)
      osite%mmean_can_shv        (1:z) = pack(isite%mmean_can_shv        (:),lmask)
      osite%mmean_can_co2        (1:z) = pack(isite%mmean_can_co2        (:),lmask)
      osite%mmean_can_rhos       (1:z) = pack(isite%mmean_can_rhos       (:),lmask)
      osite%mmean_can_prss       (1:z) = pack(isite%mmean_can_prss       (:),lmask)
      osite%mmean_gnd_temp       (1:z) = pack(isite%mmean_gnd_temp       (:),lmask)
      osite%mmean_gnd_shv        (1:z) = pack(isite%mmean_gnd_shv        (:),lmask)
      osite%mmean_can_ggnd       (1:z) = pack(isite%mmean_can_ggnd       (:),lmask)
      osite%mmean_sfcw_depth     (1:z) = pack(isite%mmean_sfcw_depth     (:),lmask)
      osite%mmean_sfcw_energy    (1:z) = pack(isite%mmean_sfcw_energy    (:),lmask)
      osite%mmean_sfcw_mass      (1:z) = pack(isite%mmean_sfcw_mass      (:),lmask)
      osite%mmean_sfcw_temp      (1:z) = pack(isite%mmean_sfcw_temp      (:),lmask)
      osite%mmean_sfcw_fliq      (1:z) = pack(isite%mmean_sfcw_fliq      (:),lmask)
      osite%mmean_rshort_gnd     (1:z) = pack(isite%mmean_rshort_gnd     (:),lmask)
      osite%mmean_par_gnd        (1:z) = pack(isite%mmean_par_gnd        (:),lmask)
      osite%mmean_rlong_gnd      (1:z) = pack(isite%mmean_rlong_gnd      (:),lmask)
      osite%mmean_rlongup        (1:z) = pack(isite%mmean_rlongup        (:),lmask)
      osite%mmean_parup          (1:z) = pack(isite%mmean_parup          (:),lmask)
      osite%mmean_nirup          (1:z) = pack(isite%mmean_nirup          (:),lmask)
      osite%mmean_rshortup       (1:z) = pack(isite%mmean_rshortup       (:),lmask)
      osite%mmean_rnet           (1:z) = pack(isite%mmean_rnet           (:),lmask)
      osite%mmean_albedo         (1:z) = pack(isite%mmean_albedo         (:),lmask)
      osite%mmean_albedo_par     (1:z) = pack(isite%mmean_albedo_par     (:),lmask)
      osite%mmean_albedo_nir     (1:z) = pack(isite%mmean_albedo_nir     (:),lmask)
      osite%mmean_rlong_albedo   (1:z) = pack(isite%mmean_rlong_albedo   (:),lmask)
      osite%mmean_ustar          (1:z) = pack(isite%mmean_ustar          (:),lmask)
      osite%mmean_tstar          (1:z) = pack(isite%mmean_tstar          (:),lmask)
      osite%mmean_qstar          (1:z) = pack(isite%mmean_qstar          (:),lmask)
      osite%mmean_cstar          (1:z) = pack(isite%mmean_cstar          (:),lmask)
      osite%mmean_carbon_ac      (1:z) = pack(isite%mmean_carbon_ac      (:),lmask)
      osite%mmean_carbon_st      (1:z) = pack(isite%mmean_carbon_st      (:),lmask)
      osite%mmean_vapor_gc       (1:z) = pack(isite%mmean_vapor_gc       (:),lmask)
      osite%mmean_vapor_ac       (1:z) = pack(isite%mmean_vapor_ac       (:),lmask)
      osite%mmean_throughfall    (1:z) = pack(isite%mmean_throughfall    (:),lmask)
      osite%mmean_runoff         (1:z) = pack(isite%mmean_runoff         (:),lmask)
      osite%mmean_drainage       (1:z) = pack(isite%mmean_drainage       (:),lmask)
      osite%mmean_sensible_gc    (1:z) = pack(isite%mmean_sensible_gc    (:),lmask)
      osite%mmean_sensible_ac    (1:z) = pack(isite%mmean_sensible_ac    (:),lmask)
      osite%mmean_qthroughfall   (1:z) = pack(isite%mmean_qthroughfall   (:),lmask)
      osite%mmean_qrunoff        (1:z) = pack(isite%mmean_qrunoff        (:),lmask)
      osite%mmean_qdrainage      (1:z) = pack(isite%mmean_qdrainage      (:),lmask)
      osite%mmean_A_decomp       (1:z) = pack(isite%mmean_A_decomp       (:),lmask)
      osite%mmean_Af_decomp      (1:z) = pack(isite%mmean_Af_decomp      (:),lmask)
      osite%mmean_co2_residual   (1:z) = pack(isite%mmean_co2_residual   (:),lmask)
      osite%mmean_energy_residual(1:z) = pack(isite%mmean_energy_residual(:),lmask)
      osite%mmean_water_residual (1:z) = pack(isite%mmean_water_residual (:),lmask)
      osite%mmsqu_rh             (1:z) = pack(isite%mmsqu_rh             (:),lmask)
      osite%mmsqu_cwd_rh         (1:z) = pack(isite%mmsqu_cwd_rh         (:),lmask)
      osite%mmsqu_nep            (1:z) = pack(isite%mmsqu_nep            (:),lmask)
      osite%mmsqu_rlongup        (1:z) = pack(isite%mmsqu_rlongup        (:),lmask)
      osite%mmsqu_parup          (1:z) = pack(isite%mmsqu_parup          (:),lmask)
      osite%mmsqu_nirup          (1:z) = pack(isite%mmsqu_nirup          (:),lmask)
      osite%mmsqu_rshortup       (1:z) = pack(isite%mmsqu_rshortup       (:),lmask)
      osite%mmsqu_rnet           (1:z) = pack(isite%mmsqu_rnet           (:),lmask)
      osite%mmsqu_albedo         (1:z) = pack(isite%mmsqu_albedo         (:),lmask)
      osite%mmsqu_ustar          (1:z) = pack(isite%mmsqu_ustar          (:),lmask)
      osite%mmsqu_carbon_ac      (1:z) = pack(isite%mmsqu_carbon_ac      (:),lmask)
      osite%mmsqu_carbon_st      (1:z) = pack(isite%mmsqu_carbon_st      (:),lmask)
      osite%mmsqu_vapor_gc       (1:z) = pack(isite%mmsqu_vapor_gc       (:),lmask)
      osite%mmsqu_vapor_ac       (1:z) = pack(isite%mmsqu_vapor_ac       (:),lmask)
      osite%mmsqu_sensible_gc    (1:z) = pack(isite%mmsqu_sensible_gc    (:),lmask)
      osite%mmsqu_sensible_ac    (1:z) = pack(isite%mmsqu_sensible_ac    (:),lmask)
      !------------------------------------------------------------------------------------!


      !----- Soil variables. --------------------------------------------------------------!
      do m=1,nzg
         osite%mmean_soil_energy(m,1:z) = pack(isite%mmean_soil_energy(m,:),lmask)
         osite%mmean_soil_mstpot(m,1:z) = pack(isite%mmean_soil_mstpot(m,:),lmask)
         osite%mmean_soil_water (m,1:z) = pack(isite%mmean_soil_water (m,:),lmask)
         osite%mmean_soil_temp  (m,1:z) = pack(isite%mmean_soil_temp  (m,:),lmask)
         osite%mmean_soil_fliq  (m,1:z) = pack(isite%mmean_soil_fliq  (m,:),lmask)
         osite%mmean_smoist_gg  (m,1:z) = pack(isite%mmean_smoist_gg  (m,:),lmask)
         osite%mmean_transloss  (m,1:z) = pack(isite%mmean_transloss  (m,:),lmask)
         osite%mmean_sensible_gg(m,1:z) = pack(isite%mmean_sensible_gg(m,:),lmask)
      end do
      !------------------------------------------------------------------------------------!


      return
   end subroutine copy_sitetype_mask_mmean
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This sub-routine copies the variables that are qmean.  This sub-routine should be !
   ! called only from copy_sitetype_mask!!!                                                !
   !---------------------------------------------------------------------------------------!
   subroutine copy_sitetype_mask_qmean(isite,osite,z,lmask,isize,osize)


      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(sitetype)                  , target     :: isite   ! Input  (donor) site
      type(sitetype)                  , target     :: osite   ! Output (receptor) site
      integer                         , intent(in) :: z       ! Number of elements
      integer                         , intent(in) :: isize   ! Input size 
      integer                         , intent(in) :: osize   ! Output size
      logical       , dimension(isize), intent(in) :: lmask   ! Mask
      !----- Local variables. -------------------------------------------------------------!
      integer                                      :: i
      integer                                      :: o
      integer                                      :: m
      integer                                      :: n
      !------------------------------------------------------------------------------------!



      do n=1,ndcycle
         !----- Scalars. ------------------------------------------------------------------!
         osite%qmean_rh            (n,1:z)  = pack(isite%qmean_rh             (n,:),lmask)
         osite%qmean_cwd_rh        (n,1:z)  = pack(isite%qmean_cwd_rh         (n,:),lmask)
         osite%qmean_nep           (n,1:z)  = pack(isite%qmean_nep            (n,:),lmask)
         osite%qmean_rk4step       (n,1:z)  = pack(isite%qmean_rk4step        (n,:),lmask)
         osite%qmean_available_water(n,1:z) = pack(isite%qmean_available_water(n,:),lmask)
         osite%qmean_can_theiv     (n,1:z)  = pack(isite%qmean_can_theiv      (n,:),lmask)
         osite%qmean_can_theta     (n,1:z)  = pack(isite%qmean_can_theta      (n,:),lmask)
         osite%qmean_can_vpdef     (n,1:z)  = pack(isite%qmean_can_vpdef      (n,:),lmask)
         osite%qmean_can_temp      (n,1:z)  = pack(isite%qmean_can_temp       (n,:),lmask)
         osite%qmean_can_shv       (n,1:z)  = pack(isite%qmean_can_shv        (n,:),lmask)
         osite%qmean_can_co2       (n,1:z)  = pack(isite%qmean_can_co2        (n,:),lmask)
         osite%qmean_can_rhos      (n,1:z)  = pack(isite%qmean_can_rhos       (n,:),lmask)
         osite%qmean_can_prss      (n,1:z)  = pack(isite%qmean_can_prss       (n,:),lmask)
         osite%qmean_gnd_temp      (n,1:z)  = pack(isite%qmean_gnd_temp       (n,:),lmask)
         osite%qmean_gnd_shv       (n,1:z)  = pack(isite%qmean_gnd_shv        (n,:),lmask)
         osite%qmean_can_ggnd      (n,1:z)  = pack(isite%qmean_can_ggnd       (n,:),lmask)
         osite%qmean_sfcw_depth    (n,1:z)  = pack(isite%qmean_sfcw_depth     (n,:),lmask)
         osite%qmean_sfcw_energy   (n,1:z)  = pack(isite%qmean_sfcw_energy    (n,:),lmask)
         osite%qmean_sfcw_mass     (n,1:z)  = pack(isite%qmean_sfcw_mass      (n,:),lmask)
         osite%qmean_sfcw_temp     (n,1:z)  = pack(isite%qmean_sfcw_temp      (n,:),lmask)
         osite%qmean_sfcw_fliq     (n,1:z)  = pack(isite%qmean_sfcw_fliq      (n,:),lmask)
         osite%qmean_rshort_gnd    (n,1:z)  = pack(isite%qmean_rshort_gnd     (n,:),lmask)
         osite%qmean_par_gnd       (n,1:z)  = pack(isite%qmean_par_gnd        (n,:),lmask)
         osite%qmean_rlong_gnd     (n,1:z)  = pack(isite%qmean_rlong_gnd      (n,:),lmask)
         osite%qmean_rlongup       (n,1:z)  = pack(isite%qmean_rlongup        (n,:),lmask)
         osite%qmean_parup         (n,1:z)  = pack(isite%qmean_parup          (n,:),lmask)
         osite%qmean_nirup         (n,1:z)  = pack(isite%qmean_nirup          (n,:),lmask)
         osite%qmean_rshortup      (n,1:z)  = pack(isite%qmean_rshortup       (n,:),lmask)
         osite%qmean_rnet          (n,1:z)  = pack(isite%qmean_rnet           (n,:),lmask)
         osite%qmean_albedo        (n,1:z)  = pack(isite%qmean_albedo         (n,:),lmask)
         osite%qmean_albedo_par    (n,1:z)  = pack(isite%qmean_albedo_par     (n,:),lmask)
         osite%qmean_albedo_nir    (n,1:z)  = pack(isite%qmean_albedo_nir     (n,:),lmask)
         osite%qmean_rlong_albedo  (n,1:z)  = pack(isite%qmean_rlong_albedo   (n,:),lmask)
         osite%qmean_ustar         (n,1:z)  = pack(isite%qmean_ustar          (n,:),lmask)
         osite%qmean_tstar         (n,1:z)  = pack(isite%qmean_tstar          (n,:),lmask)
         osite%qmean_qstar         (n,1:z)  = pack(isite%qmean_qstar          (n,:),lmask)
         osite%qmean_cstar         (n,1:z)  = pack(isite%qmean_cstar          (n,:),lmask)
         osite%qmean_carbon_ac     (n,1:z)  = pack(isite%qmean_carbon_ac      (n,:),lmask)
         osite%qmean_carbon_st     (n,1:z)  = pack(isite%qmean_carbon_st      (n,:),lmask)
         osite%qmean_vapor_gc      (n,1:z)  = pack(isite%qmean_vapor_gc       (n,:),lmask)
         osite%qmean_vapor_ac      (n,1:z)  = pack(isite%qmean_vapor_ac       (n,:),lmask)
         osite%qmean_throughfall   (n,1:z)  = pack(isite%qmean_throughfall    (n,:),lmask)
         osite%qmean_runoff        (n,1:z)  = pack(isite%qmean_runoff         (n,:),lmask)
         osite%qmean_drainage      (n,1:z)  = pack(isite%qmean_drainage       (n,:),lmask)
         osite%qmean_sensible_gc   (n,1:z)  = pack(isite%qmean_sensible_gc    (n,:),lmask)
         osite%qmean_sensible_ac   (n,1:z)  = pack(isite%qmean_sensible_ac    (n,:),lmask)
         osite%qmean_qthroughfall  (n,1:z)  = pack(isite%qmean_qthroughfall   (n,:),lmask)
         osite%qmean_qrunoff       (n,1:z)  = pack(isite%qmean_qrunoff        (n,:),lmask)
         osite%qmean_qdrainage     (n,1:z)  = pack(isite%qmean_qdrainage      (n,:),lmask)
         osite%qmsqu_rh            (n,1:z)  = pack(isite%qmsqu_rh             (n,:),lmask)
         osite%qmsqu_cwd_rh        (n,1:z)  = pack(isite%qmsqu_cwd_rh         (n,:),lmask)
         osite%qmsqu_nep           (n,1:z)  = pack(isite%qmsqu_nep            (n,:),lmask)
         osite%qmsqu_rlongup       (n,1:z)  = pack(isite%qmsqu_rlongup        (n,:),lmask)
         osite%qmsqu_parup         (n,1:z)  = pack(isite%qmsqu_parup          (n,:),lmask)
         osite%qmsqu_nirup         (n,1:z)  = pack(isite%qmsqu_nirup          (n,:),lmask)
         osite%qmsqu_rshortup      (n,1:z)  = pack(isite%qmsqu_rshortup       (n,:),lmask)
         osite%qmsqu_rnet          (n,1:z)  = pack(isite%qmsqu_rnet           (n,:),lmask)
         osite%qmsqu_albedo        (n,1:z)  = pack(isite%qmsqu_albedo         (n,:),lmask)
         osite%qmsqu_ustar         (n,1:z)  = pack(isite%qmsqu_ustar          (n,:),lmask)
         osite%qmsqu_carbon_ac     (n,1:z)  = pack(isite%qmsqu_carbon_ac      (n,:),lmask)
         osite%qmsqu_carbon_st     (n,1:z)  = pack(isite%qmsqu_carbon_st      (n,:),lmask)
         osite%qmsqu_vapor_gc      (n,1:z)  = pack(isite%qmsqu_vapor_gc       (n,:),lmask)
         osite%qmsqu_vapor_ac      (n,1:z)  = pack(isite%qmsqu_vapor_ac       (n,:),lmask)
         osite%qmsqu_sensible_gc   (n,1:z)  = pack(isite%qmsqu_sensible_gc    (n,:),lmask)
         osite%qmsqu_sensible_ac   (n,1:z)  = pack(isite%qmsqu_sensible_ac    (n,:),lmask)
         !---------------------------------------------------------------------------------!


         !----- Soil variables. -----------------------------------------------------------!
         do m=1,nzg
            osite%qmean_soil_energy(m,n,1:z) = pack(isite%qmean_soil_energy(m,n,:),lmask)
            osite%qmean_soil_mstpot(m,n,1:z) = pack(isite%qmean_soil_mstpot(m,n,:),lmask)
            osite%qmean_soil_water (m,n,1:z) = pack(isite%qmean_soil_water (m,n,:),lmask)
            osite%qmean_soil_temp  (m,n,1:z) = pack(isite%qmean_soil_temp  (m,n,:),lmask)
            osite%qmean_soil_fliq  (m,n,1:z) = pack(isite%qmean_soil_fliq  (m,n,:),lmask)
            osite%qmean_smoist_gg  (m,n,1:z) = pack(isite%qmean_smoist_gg  (m,n,:),lmask)
            osite%qmean_transloss  (m,n,1:z) = pack(isite%qmean_transloss  (m,n,:),lmask)
            osite%qmean_sensible_gg(m,n,1:z) = pack(isite%qmean_sensible_gg(m,n,:),lmask)
         end do
         !---------------------------------------------------------------------------------!
      end do
      !------------------------------------------------------------------------------------!

      return
   end subroutine copy_sitetype_mask_qmean
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This subroutine copies a continuous chunk of patches from one place to another.   !
   ! The number of patches to be copied must match between the input and output, and every !
   ! single variable, including the cohorts, will be copied.                               !
   ! IMPORTANT.  This subroutine assumes that the output patches still don't have cohorts  !
   !             allocated, so this should be never used in a previously allocated patch.  !
   !---------------------------------------------------------------------------------------!
   subroutine copy_patchtype(ipatch,opatch,icoa,icoz,ocoa,ocoz)
      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(patchtype)         , target     :: ipatch
      type(patchtype)         , target     :: opatch
      integer                 , intent(in) :: icoa
      integer                 , intent(in) :: icoz
      integer                 , intent(in) :: ocoa
      integer                 , intent(in) :: ocoz
      !----- Local variables. -------------------------------------------------------------!
      integer                              :: ico
      integer                              :: oco
      integer                              :: m
      integer                              :: n
      !------------------------------------------------------------------------------------!


      !------ Check whether this cohort copying makes sense. ------------------------------!
      if (icoz - icoa /= ocoz - ocoa) then
         write (unit=*,fmt='(a)')       '------------------------------------------------'
         write (unit=*,fmt='(a)')       ' - Input patch:'
         write (unit=*,fmt='(a,1x,i6)') '   * First cohort: ',icoa
         write (unit=*,fmt='(a,1x,i6)') '   * Last  cohort: ',icoz
         write (unit=*,fmt='(a,1x,i6)') '   * Cohort count: ',icoz-icoa+1
         write (unit=*,fmt='(a)')       ' - Output site:'
         write (unit=*,fmt='(a,1x,i6)') '   * First cohort: ',ocoa
         write (unit=*,fmt='(a,1x,i6)') '   * Last  cohort: ',ocoz
         write (unit=*,fmt='(a,1x,i6)') '   * Cohort count: ',ocoz-ocoa+1
         write (unit=*,fmt='(a)'      ) '------------------------------------------------'
         call fatal_error ('Cohort count of input and output patches don''t match'         &
                          ,'copy_patchtype','ed_state_vars.f90')
      end if
      !------------------------------------------------------------------------------------!

      ico = icoa - 1
      cohortloop: do oco = ocoa,ocoz
         ico = ico + 1

         !----- Scalars. ------------------------------------------------------------------!
         opatch%pft                   (oco) = ipatch%pft                   (ico)
         opatch%nplant                (oco) = ipatch%nplant                (ico)
         opatch%phenology_status      (oco) = ipatch%phenology_status      (ico)
         opatch%recruit_dbh           (oco) = ipatch%recruit_dbh           (ico)
         opatch%census_status         (oco) = ipatch%census_status         (ico)
         opatch%hite                  (oco) = ipatch%hite                  (ico)
         opatch%agb                   (oco) = ipatch%agb                   (ico)
         opatch%basarea               (oco) = ipatch%basarea               (ico)
         opatch%dagb_dt               (oco) = ipatch%dagb_dt               (ico)
         opatch%dlnagb_dt             (oco) = ipatch%dlnagb_dt             (ico)
         opatch%dba_dt                (oco) = ipatch%dba_dt                (ico)
         opatch%dlnba_dt              (oco) = ipatch%dlnba_dt              (ico)
         opatch%ddbh_dt               (oco) = ipatch%ddbh_dt               (ico)
         opatch%dlndbh_dt             (oco) = ipatch%dlndbh_dt             (ico)
         opatch%dbh                   (oco) = ipatch%dbh                   (ico)
         opatch%bdead                 (oco) = ipatch%bdead                 (ico)
         opatch%bleaf                 (oco) = ipatch%bleaf                 (ico)
         opatch%balive                (oco) = ipatch%balive                (ico)
         opatch%broot                 (oco) = ipatch%broot                 (ico)
         opatch%bsapwooda             (oco) = ipatch%bsapwooda             (ico)
         opatch%bsapwoodb             (oco) = ipatch%bsapwoodb             (ico)
         opatch%bstorage              (oco) = ipatch%bstorage              (ico)
         opatch%bseeds                (oco) = ipatch%bseeds                (ico)
         opatch%lai                   (oco) = ipatch%lai                   (ico)
         opatch%wai                   (oco) = ipatch%wai                   (ico)
         opatch%crown_area            (oco) = ipatch%crown_area            (ico)
         opatch%leaf_resolvable       (oco) = ipatch%leaf_resolvable       (ico)
         opatch%wood_resolvable       (oco) = ipatch%wood_resolvable       (ico)
         opatch%cbr_bar               (oco) = ipatch%cbr_bar               (ico)
         opatch%leaf_energy           (oco) = ipatch%leaf_energy           (ico)
         opatch%leaf_temp             (oco) = ipatch%leaf_temp             (ico)
         opatch%leaf_vpdef            (oco) = ipatch%leaf_vpdef            (ico)
         opatch%leaf_temp_pv          (oco) = ipatch%leaf_temp_pv          (ico)
         opatch%leaf_hcap             (oco) = ipatch%leaf_hcap             (ico)
         opatch%leaf_fliq             (oco) = ipatch%leaf_fliq             (ico)
         opatch%leaf_water            (oco) = ipatch%leaf_water            (ico)
         opatch%wood_energy           (oco) = ipatch%wood_energy           (ico)
         opatch%wood_temp             (oco) = ipatch%wood_temp             (ico)
         opatch%wood_temp_pv          (oco) = ipatch%wood_temp_pv          (ico)
         opatch%wood_hcap             (oco) = ipatch%wood_hcap             (ico)
         opatch%wood_fliq             (oco) = ipatch%wood_fliq             (ico)
         opatch%wood_water            (oco) = ipatch%wood_water            (ico)
         opatch%veg_wind              (oco) = ipatch%veg_wind              (ico)
         opatch%lsfc_shv_open         (oco) = ipatch%lsfc_shv_open         (ico)
         opatch%lsfc_shv_closed       (oco) = ipatch%lsfc_shv_closed       (ico)
         opatch%lsfc_co2_open         (oco) = ipatch%lsfc_co2_open         (ico)
         opatch%lsfc_co2_closed       (oco) = ipatch%lsfc_co2_closed       (ico)
         opatch%lint_shv              (oco) = ipatch%lint_shv              (ico)
         opatch%lint_co2_open         (oco) = ipatch%lint_co2_open         (ico)
         opatch%lint_co2_closed       (oco) = ipatch%lint_co2_closed       (ico)
         opatch%today_leaf_resp       (oco) = ipatch%today_leaf_resp       (ico)
         opatch%today_root_resp       (oco) = ipatch%today_root_resp       (ico)
         opatch%today_gpp             (oco) = ipatch%today_gpp             (ico)
         opatch%today_gpp_pot         (oco) = ipatch%today_gpp_pot         (ico)
         opatch%today_gpp_lightmax    (oco) = ipatch%today_gpp_lightmax    (ico)
         opatch%today_gpp_moistmax    (oco) = ipatch%today_gpp_moistmax    (ico)
         opatch%today_gpp_mlmax       (oco) = ipatch%today_gpp_mlmax       (ico)
         opatch%today_nppleaf         (oco) = ipatch%today_nppleaf         (ico)
         opatch%today_nppfroot        (oco) = ipatch%today_nppfroot        (ico)
         opatch%today_nppsapwood      (oco) = ipatch%today_nppsapwood      (ico)
         opatch%today_nppcroot        (oco) = ipatch%today_nppcroot        (ico)
         opatch%today_nppseeds        (oco) = ipatch%today_nppseeds        (ico)
         opatch%today_nppwood         (oco) = ipatch%today_nppwood         (ico)
         opatch%today_nppdaily        (oco) = ipatch%today_nppdaily        (ico)
         opatch%leaf_growth_resp      (oco) = ipatch%leaf_growth_resp      (ico)
         opatch%root_growth_resp      (oco) = ipatch%root_growth_resp      (ico)
         opatch%sapa_growth_resp      (oco) = ipatch%sapa_growth_resp      (ico)
         opatch%sapb_growth_resp      (oco) = ipatch%sapb_growth_resp      (ico)
         opatch%leaf_storage_resp     (oco) = ipatch%leaf_storage_resp     (ico)
         opatch%root_storage_resp     (oco) = ipatch%root_storage_resp     (ico)
         opatch%sapa_storage_resp     (oco) = ipatch%sapa_storage_resp     (ico)
         opatch%sapb_storage_resp     (oco) = ipatch%sapb_storage_resp     (ico)
         opatch%monthly_dndt          (oco) = ipatch%monthly_dndt          (ico)
         opatch%monthly_dlnndt        (oco) = ipatch%monthly_dlnndt        (ico)
         opatch%krdepth               (oco) = ipatch%krdepth               (ico)
         opatch%first_census          (oco) = ipatch%first_census          (ico)
         opatch%new_recruit_flag      (oco) = ipatch%new_recruit_flag      (ico)
         opatch%light_level           (oco) = ipatch%light_level           (ico)
         opatch%light_level_beam      (oco) = ipatch%light_level_beam      (ico)
         opatch%light_level_diff      (oco) = ipatch%light_level_diff      (ico)
         
         opatch%par_level_beam        (oco) = ipatch%par_level_beam        (ico)
         opatch%par_level_diffu       (oco) = ipatch%par_level_diffu       (ico)
         opatch%par_level_diffd       (oco) = ipatch%par_level_diffd       (ico)

         opatch%par_l                 (oco) = ipatch%par_l                 (ico)
         opatch%par_l_beam            (oco) = ipatch%par_l_beam            (ico)
         opatch%par_l_diffuse         (oco) = ipatch%par_l_diffuse         (ico)
         opatch%rshort_l              (oco) = ipatch%rshort_l              (ico)
         opatch%rshort_l_beam         (oco) = ipatch%rshort_l_beam         (ico)
         opatch%rshort_l_diffuse      (oco) = ipatch%rshort_l_diffuse      (ico)
         opatch%rlong_l               (oco) = ipatch%rlong_l               (ico)
         opatch%rshort_w              (oco) = ipatch%rshort_w              (ico)
         opatch%rshort_w_beam         (oco) = ipatch%rshort_w_beam         (ico)
         opatch%rshort_w_diffuse      (oco) = ipatch%rshort_w_diffuse      (ico)
         opatch%rlong_w               (oco) = ipatch%rlong_w               (ico)
         opatch%leaf_gbh              (oco) = ipatch%leaf_gbh              (ico)
         opatch%leaf_gbw              (oco) = ipatch%leaf_gbw              (ico)
         opatch%wood_gbh              (oco) = ipatch%wood_gbh              (ico)
         opatch%wood_gbw              (oco) = ipatch%wood_gbw              (ico)
         opatch%A_open                (oco) = ipatch%A_open                (ico)
         opatch%A_closed              (oco) = ipatch%A_closed              (ico)
         opatch%A_light               (oco) = ipatch%A_light               (ico)
         opatch%A_rubp                (oco) = ipatch%A_rubp                (ico)
         opatch%A_co2                 (oco) = ipatch%A_co2                 (ico)
         opatch%psi_open              (oco) = ipatch%psi_open              (ico)
         opatch%psi_closed            (oco) = ipatch%psi_closed            (ico)
         opatch%gsw_open              (oco) = ipatch%gsw_open              (ico)
         opatch%gsw_closed            (oco) = ipatch%gsw_closed            (ico)
         opatch%leaf_gsw              (oco) = ipatch%leaf_gsw              (ico)
         opatch%fsw                   (oco) = ipatch%fsw                   (ico)
         opatch%fsn                   (oco) = ipatch%fsn                   (ico)
         opatch%fs_open               (oco) = ipatch%fs_open               (ico)
         opatch%water_supply          (oco) = ipatch%water_supply          (ico)
         opatch%leaf_maintenance      (oco) = ipatch%leaf_maintenance      (ico)
         opatch%root_maintenance      (oco) = ipatch%root_maintenance      (ico)
         opatch%leaf_drop             (oco) = ipatch%leaf_drop             (ico)
         opatch%leaf_respiration      (oco) = ipatch%leaf_respiration      (ico)
         opatch%root_respiration      (oco) = ipatch%root_respiration      (ico)
         opatch%gpp                   (oco) = ipatch%gpp                   (ico)
         opatch%paw_avg               (oco) = ipatch%paw_avg               (ico)
         opatch%elongf                (oco) = ipatch%elongf                (ico)
         opatch%turnover_amp          (oco) = ipatch%turnover_amp          (ico)
         opatch%llspan                (oco) = ipatch%llspan                (ico)
         opatch%vm_bar                (oco) = ipatch%vm_bar                (ico)
         opatch%sla                   (oco) = ipatch%sla                   (ico)
         opatch%fmean_gpp             (oco) = ipatch%fmean_gpp             (ico)
         opatch%fmean_npp             (oco) = ipatch%fmean_npp             (ico)
         opatch%fmean_leaf_resp       (oco) = ipatch%fmean_leaf_resp       (ico)
         opatch%fmean_root_resp       (oco) = ipatch%fmean_root_resp       (ico)
         opatch%fmean_leaf_growth_resp(oco) = ipatch%fmean_leaf_growth_resp(ico)
         opatch%fmean_root_growth_resp(oco) = ipatch%fmean_root_growth_resp(ico)
         opatch%fmean_sapa_growth_resp(oco) = ipatch%fmean_sapa_growth_resp(ico)
         opatch%fmean_sapb_growth_resp(oco) = ipatch%fmean_sapb_growth_resp(ico)
         opatch%fmean_leaf_storage_resp(oco) = ipatch%fmean_leaf_storage_resp(ico)
         opatch%fmean_root_storage_resp(oco) = ipatch%fmean_root_storage_resp(ico)
         opatch%fmean_sapa_storage_resp(oco) = ipatch%fmean_sapa_storage_resp(ico)
         opatch%fmean_sapb_storage_resp(oco) = ipatch%fmean_sapb_storage_resp(ico)
         opatch%fmean_plresp          (oco) = ipatch%fmean_plresp          (ico)
         opatch%fmean_leaf_energy     (oco) = ipatch%fmean_leaf_energy     (ico)
         opatch%fmean_leaf_water      (oco) = ipatch%fmean_leaf_water      (ico)
         opatch%fmean_leaf_hcap       (oco) = ipatch%fmean_leaf_hcap       (ico)
         opatch%fmean_leaf_vpdef      (oco) = ipatch%fmean_leaf_vpdef      (ico)
         opatch%fmean_leaf_temp       (oco) = ipatch%fmean_leaf_temp       (ico)
         opatch%fmean_leaf_fliq       (oco) = ipatch%fmean_leaf_fliq       (ico)
         opatch%fmean_leaf_gsw        (oco) = ipatch%fmean_leaf_gsw        (ico)
         opatch%fmean_leaf_gbw        (oco) = ipatch%fmean_leaf_gbw        (ico)
         opatch%fmean_wood_energy     (oco) = ipatch%fmean_wood_energy     (ico)
         opatch%fmean_wood_water      (oco) = ipatch%fmean_wood_water      (ico)
         opatch%fmean_wood_hcap       (oco) = ipatch%fmean_wood_hcap       (ico)
         opatch%fmean_wood_temp       (oco) = ipatch%fmean_wood_temp       (ico)
         opatch%fmean_wood_fliq       (oco) = ipatch%fmean_wood_fliq       (ico)
         opatch%fmean_wood_gbw        (oco) = ipatch%fmean_wood_gbw        (ico)
         opatch%fmean_fs_open         (oco) = ipatch%fmean_fs_open         (ico)
         opatch%fmean_fsw             (oco) = ipatch%fmean_fsw             (ico)
         opatch%fmean_fsn             (oco) = ipatch%fmean_fsn             (ico)
         opatch%fmean_a_open          (oco) = ipatch%fmean_a_open          (ico)
         opatch%fmean_a_closed        (oco) = ipatch%fmean_a_closed        (ico)
         opatch%fmean_a_net           (oco) = ipatch%fmean_a_net           (ico)
         opatch%fmean_A_light         (oco) = ipatch%fmean_A_light         (ico)
         opatch%fmean_A_rubp          (oco) = ipatch%fmean_A_rubp          (ico)
         opatch%fmean_A_co2           (oco) = ipatch%fmean_A_co2           (ico)
         opatch%fmean_psi_open        (oco) = ipatch%fmean_psi_open        (ico)
         opatch%fmean_psi_closed      (oco) = ipatch%fmean_psi_closed      (ico)
         opatch%fmean_water_supply    (oco) = ipatch%fmean_water_supply    (ico)
         opatch%fmean_light_level     (oco) = ipatch%fmean_light_level     (ico)
         opatch%fmean_light_level_beam(oco) = ipatch%fmean_light_level_beam(ico)
         opatch%fmean_light_level_diff(oco) = ipatch%fmean_light_level_diff(ico)
         
         opatch%fmean_par_level_beam  (oco) = ipatch%fmean_par_level_beam  (ico)
         opatch%fmean_par_level_diffu (oco) = ipatch%fmean_par_level_diffu (ico)
         opatch%fmean_par_level_diffd (oco) = ipatch%fmean_par_level_diffd (ico)

         opatch%fmean_par_l           (oco) = ipatch%fmean_par_l           (ico)
         opatch%fmean_par_l_beam      (oco) = ipatch%fmean_par_l_beam      (ico)
         opatch%fmean_par_l_diff      (oco) = ipatch%fmean_par_l_diff      (ico)
         opatch%fmean_rshort_l        (oco) = ipatch%fmean_rshort_l        (ico)
         opatch%fmean_rlong_l         (oco) = ipatch%fmean_rlong_l         (ico)
         opatch%fmean_sensible_lc     (oco) = ipatch%fmean_sensible_lc     (ico)
         opatch%fmean_vapor_lc        (oco) = ipatch%fmean_vapor_lc        (ico)
         opatch%fmean_transp          (oco) = ipatch%fmean_transp          (ico)
         opatch%fmean_intercepted_al  (oco) = ipatch%fmean_intercepted_al  (ico)
         opatch%fmean_wshed_lg        (oco) = ipatch%fmean_wshed_lg        (ico)
         opatch%fmean_rshort_w        (oco) = ipatch%fmean_rshort_w        (ico)
         opatch%fmean_rlong_w         (oco) = ipatch%fmean_rlong_w         (ico)
         opatch%fmean_sensible_wc     (oco) = ipatch%fmean_sensible_wc     (ico)
         opatch%fmean_vapor_wc        (oco) = ipatch%fmean_vapor_wc        (ico)
         opatch%fmean_intercepted_aw  (oco) = ipatch%fmean_intercepted_aw  (ico)
         opatch%fmean_wshed_wg        (oco) = ipatch%fmean_wshed_wg        (ico)
         opatch%fmean_lai             (oco) = ipatch%fmean_lai             (ico)
         opatch%fmean_bdead           (oco) = ipatch%fmean_bdead           (ico)
         !---------------------------------------------------------------------------------!


         !------ Carbon balance variables. ------------------------------------------------!
         do m=1,13
            opatch%cb         (m,oco) = ipatch%cb         (m,ico)
            opatch%cb_lightmax(m,oco) = ipatch%cb_lightmax(m,ico)
            opatch%cb_moistmax(m,oco) = ipatch%cb_moistmax(m,ico)
            opatch%cb_mlmax   (m,oco) = ipatch%cb_mlmax   (m,ico)
         end do
         !---------------------------------------------------------------------------------!


         !------ Mortality variables. -----------------------------------------------------!
         do m=1,n_mort
            opatch%mort_rate(m,oco) = ipatch%mort_rate(m,ico)
         end do
         !---------------------------------------------------------------------------------!


         !------ Radiation profile variables. ---------------------------------------------!
         do m=1,n_radprof
            opatch%rad_profile      (m,oco) = ipatch%rad_profile      (m,ico)
            opatch%fmean_rad_profile(m,oco) = ipatch%fmean_rad_profile(m,ico)
         end do
         !---------------------------------------------------------------------------------!



         !---------------------------------------------------------------------------------!
         !     Daily means.                                                                !
         !---------------------------------------------------------------------------------!
         if (writing_long) then
            !----- Scalars. ---------------------------------------------------------------!
            opatch%dmean_nppleaf         (oco) = ipatch%dmean_nppleaf         (ico)
            opatch%dmean_nppfroot        (oco) = ipatch%dmean_nppfroot        (ico)
            opatch%dmean_nppsapwood      (oco) = ipatch%dmean_nppsapwood      (ico)
            opatch%dmean_nppcroot        (oco) = ipatch%dmean_nppcroot        (ico)
            opatch%dmean_nppseeds        (oco) = ipatch%dmean_nppseeds        (ico)
            opatch%dmean_nppwood         (oco) = ipatch%dmean_nppwood         (ico)
            opatch%dmean_nppdaily        (oco) = ipatch%dmean_nppdaily        (ico)
            opatch%dmean_gpp             (oco) = ipatch%dmean_gpp             (ico)
            opatch%dmean_npp             (oco) = ipatch%dmean_npp             (ico)
            opatch%dmean_leaf_resp       (oco) = ipatch%dmean_leaf_resp       (ico)
            opatch%dmean_root_resp       (oco) = ipatch%dmean_root_resp       (ico)
            opatch%dmean_leaf_growth_resp(oco) = ipatch%dmean_leaf_growth_resp(ico)
            opatch%dmean_root_growth_resp(oco) = ipatch%dmean_root_growth_resp(ico)
            opatch%dmean_sapa_growth_resp(oco) = ipatch%dmean_sapa_growth_resp(ico)
            opatch%dmean_sapb_growth_resp(oco) = ipatch%dmean_sapb_growth_resp(ico)
            opatch%dmean_leaf_storage_resp(oco) = ipatch%dmean_leaf_storage_resp(ico)
            opatch%dmean_root_storage_resp(oco) = ipatch%dmean_root_storage_resp(ico)
            opatch%dmean_sapa_storage_resp(oco) = ipatch%dmean_sapa_storage_resp(ico)
            opatch%dmean_sapb_storage_resp(oco) = ipatch%dmean_sapb_storage_resp(ico)
            opatch%dmean_plresp          (oco) = ipatch%dmean_plresp          (ico)
            opatch%dmean_leaf_energy     (oco) = ipatch%dmean_leaf_energy     (ico)
            opatch%dmean_leaf_water      (oco) = ipatch%dmean_leaf_water      (ico)
            opatch%dmean_leaf_hcap       (oco) = ipatch%dmean_leaf_hcap       (ico)
            opatch%dmean_leaf_vpdef      (oco) = ipatch%dmean_leaf_vpdef      (ico)
            opatch%dmean_leaf_temp       (oco) = ipatch%dmean_leaf_temp       (ico)
            opatch%dmean_leaf_fliq       (oco) = ipatch%dmean_leaf_fliq       (ico)
            opatch%dmean_leaf_gsw        (oco) = ipatch%dmean_leaf_gsw        (ico)
            opatch%dmean_leaf_gbw        (oco) = ipatch%dmean_leaf_gbw        (ico)
            opatch%dmean_wood_energy     (oco) = ipatch%dmean_wood_energy     (ico)
            opatch%dmean_wood_water      (oco) = ipatch%dmean_wood_water      (ico)
            opatch%dmean_wood_hcap       (oco) = ipatch%dmean_wood_hcap       (ico)
            opatch%dmean_wood_temp       (oco) = ipatch%dmean_wood_temp       (ico)
            opatch%dmean_wood_fliq       (oco) = ipatch%dmean_wood_fliq       (ico)
            opatch%dmean_wood_gbw        (oco) = ipatch%dmean_wood_gbw        (ico)
            opatch%dmean_fs_open         (oco) = ipatch%dmean_fs_open         (ico)
            opatch%dmean_fsw             (oco) = ipatch%dmean_fsw             (ico)
            opatch%dmean_fsn             (oco) = ipatch%dmean_fsn             (ico)
            opatch%dmean_a_open          (oco) = ipatch%dmean_a_open          (ico)
            opatch%dmean_a_closed        (oco) = ipatch%dmean_a_closed        (ico)
            opatch%dmean_a_net           (oco) = ipatch%dmean_a_net           (ico)
            opatch%dmean_A_light         (oco) = ipatch%dmean_A_light         (ico)
            opatch%dmean_A_rubp          (oco) = ipatch%dmean_A_rubp          (ico)
            opatch%dmean_A_co2           (oco) = ipatch%dmean_A_co2           (ico)
            opatch%dmean_psi_open        (oco) = ipatch%dmean_psi_open        (ico)
            opatch%dmean_psi_closed      (oco) = ipatch%dmean_psi_closed      (ico)
            opatch%dmean_water_supply    (oco) = ipatch%dmean_water_supply    (ico)
            opatch%dmean_light_level     (oco) = ipatch%dmean_light_level     (ico)
            opatch%dmean_light_level_beam(oco) = ipatch%dmean_light_level_beam(ico)
            opatch%dmean_light_level_diff(oco) = ipatch%dmean_light_level_diff(ico)

            opatch%dmean_par_level_beam  (oco) = ipatch%dmean_par_level_beam  (ico)
            opatch%dmean_par_level_diffu (oco) = ipatch%dmean_par_level_diffu (ico)
            opatch%dmean_par_level_diffd (oco) = ipatch%dmean_par_level_diffd (ico)

            opatch%dmean_par_l           (oco) = ipatch%dmean_par_l           (ico)
            opatch%dmean_par_l_beam      (oco) = ipatch%dmean_par_l_beam      (ico)
            opatch%dmean_par_l_diff      (oco) = ipatch%dmean_par_l_diff      (ico)
            opatch%dmean_rshort_l        (oco) = ipatch%dmean_rshort_l        (ico)
            opatch%dmean_rlong_l         (oco) = ipatch%dmean_rlong_l         (ico)
            opatch%dmean_sensible_lc     (oco) = ipatch%dmean_sensible_lc     (ico)
            opatch%dmean_vapor_lc        (oco) = ipatch%dmean_vapor_lc        (ico)
            opatch%dmean_transp          (oco) = ipatch%dmean_transp          (ico)
            opatch%dmean_intercepted_al  (oco) = ipatch%dmean_intercepted_al  (ico)
            opatch%dmean_wshed_lg        (oco) = ipatch%dmean_wshed_lg        (ico)
            opatch%dmean_rshort_w        (oco) = ipatch%dmean_rshort_w        (ico)
            opatch%dmean_rlong_w         (oco) = ipatch%dmean_rlong_w         (ico)
            opatch%dmean_sensible_wc     (oco) = ipatch%dmean_sensible_wc     (ico)
            opatch%dmean_vapor_wc        (oco) = ipatch%dmean_vapor_wc        (ico)
            opatch%dmean_intercepted_aw  (oco) = ipatch%dmean_intercepted_aw  (ico)
            opatch%dmean_wshed_wg        (oco) = ipatch%dmean_wshed_wg        (ico)


            !------ Radiation profile variables. ------------------------------------------!
            do m=1,n_radprof
               opatch%dmean_rad_profile(m,oco) = ipatch%dmean_rad_profile(m,ico)
            end do
            !------------------------------------------------------------------------------!

         end if
         !---------------------------------------------------------------------------------!



         !---------------------------------------------------------------------------------!
         !     Monthly means.                                                              !
         !---------------------------------------------------------------------------------!
         if (writing_eorq) then
            !----- Scalars. ---------------------------------------------------------------!
            opatch%mmean_lai             (oco) = ipatch%mmean_lai             (ico)
            opatch%mmean_bleaf           (oco) = ipatch%mmean_bleaf           (ico)
            opatch%mmean_broot           (oco) = ipatch%mmean_broot           (ico)
            opatch%mmean_bstorage        (oco) = ipatch%mmean_bstorage        (ico)
            opatch%mmean_leaf_maintenance(oco) = ipatch%mmean_leaf_maintenance(ico)
            opatch%mmean_root_maintenance(oco) = ipatch%mmean_root_maintenance(ico)
            opatch%mmean_leaf_drop       (oco) = ipatch%mmean_leaf_drop       (ico)
            opatch%mmean_cb              (oco) = ipatch%mmean_cb              (ico)
            opatch%mmean_gpp             (oco) = ipatch%mmean_gpp             (ico)
            opatch%mmean_npp             (oco) = ipatch%mmean_npp             (ico)
            opatch%mmean_leaf_resp       (oco) = ipatch%mmean_leaf_resp       (ico)
            opatch%mmean_root_resp       (oco) = ipatch%mmean_root_resp       (ico)
            opatch%mmean_leaf_growth_resp(oco) = ipatch%mmean_leaf_growth_resp(ico)
            opatch%mmean_root_growth_resp(oco) = ipatch%mmean_root_growth_resp(ico)
            opatch%mmean_sapa_growth_resp(oco) = ipatch%mmean_sapa_growth_resp(ico)
            opatch%mmean_sapb_growth_resp(oco) = ipatch%mmean_sapb_growth_resp(ico)
            opatch%mmean_leaf_storage_resp(oco) = ipatch%mmean_leaf_storage_resp(ico)
            opatch%mmean_root_storage_resp(oco) = ipatch%mmean_root_storage_resp(ico)
            opatch%mmean_sapa_storage_resp(oco) = ipatch%mmean_sapa_storage_resp(ico)
            opatch%mmean_sapb_storage_resp(oco) = ipatch%mmean_sapb_storage_resp(ico)
            opatch%mmean_plresp          (oco) = ipatch%mmean_plresp          (ico)
            opatch%mmean_leaf_energy     (oco) = ipatch%mmean_leaf_energy     (ico)
            opatch%mmean_leaf_water      (oco) = ipatch%mmean_leaf_water      (ico)
            opatch%mmean_leaf_hcap       (oco) = ipatch%mmean_leaf_hcap       (ico)
            opatch%mmean_leaf_vpdef      (oco) = ipatch%mmean_leaf_vpdef      (ico)
            opatch%mmean_leaf_temp       (oco) = ipatch%mmean_leaf_temp       (ico)
            opatch%mmean_leaf_fliq       (oco) = ipatch%mmean_leaf_fliq       (ico)
            opatch%mmean_leaf_gsw        (oco) = ipatch%mmean_leaf_gsw        (ico)
            opatch%mmean_leaf_gbw        (oco) = ipatch%mmean_leaf_gbw        (ico)
            opatch%mmean_wood_energy     (oco) = ipatch%mmean_wood_energy     (ico)
            opatch%mmean_wood_water      (oco) = ipatch%mmean_wood_water      (ico)
            opatch%mmean_wood_hcap       (oco) = ipatch%mmean_wood_hcap       (ico)
            opatch%mmean_wood_temp       (oco) = ipatch%mmean_wood_temp       (ico)
            opatch%mmean_wood_fliq       (oco) = ipatch%mmean_wood_fliq       (ico)
            opatch%mmean_wood_gbw        (oco) = ipatch%mmean_wood_gbw        (ico)
            opatch%mmean_fs_open         (oco) = ipatch%mmean_fs_open         (ico)
            opatch%mmean_fsw             (oco) = ipatch%mmean_fsw             (ico)
            opatch%mmean_fsn             (oco) = ipatch%mmean_fsn             (ico)
            opatch%mmean_a_open          (oco) = ipatch%mmean_a_open          (ico)
            opatch%mmean_a_closed        (oco) = ipatch%mmean_a_closed        (ico)
            opatch%mmean_a_net           (oco) = ipatch%mmean_a_net           (ico)
            opatch%mmean_A_light         (oco) = ipatch%mmean_A_light         (ico)
            opatch%mmean_A_rubp          (oco) = ipatch%mmean_A_rubp          (ico)
            opatch%mmean_A_co2           (oco) = ipatch%mmean_A_co2           (ico)
            opatch%mmean_psi_open        (oco) = ipatch%mmean_psi_open        (ico)
            opatch%mmean_psi_closed      (oco) = ipatch%mmean_psi_closed      (ico)
            opatch%mmean_water_supply    (oco) = ipatch%mmean_water_supply    (ico)
            opatch%mmean_light_level     (oco) = ipatch%mmean_light_level     (ico)
            opatch%mmean_light_level_beam(oco) = ipatch%mmean_light_level_beam(ico)
            opatch%mmean_light_level_diff(oco) = ipatch%mmean_light_level_diff(ico)

            opatch%mmean_par_level_beam  (oco) = ipatch%mmean_par_level_beam  (ico)
            opatch%mmean_par_level_diffu (oco) = ipatch%mmean_par_level_diffu (ico)
            opatch%mmean_par_level_diffd (oco) = ipatch%mmean_par_level_diffd (ico)

            opatch%mmean_par_l           (oco) = ipatch%mmean_par_l           (ico)
            opatch%mmean_par_l_beam      (oco) = ipatch%mmean_par_l_beam      (ico)
            opatch%mmean_par_l_diff      (oco) = ipatch%mmean_par_l_diff      (ico)
            opatch%mmean_rshort_l        (oco) = ipatch%mmean_rshort_l        (ico)
            opatch%mmean_rlong_l         (oco) = ipatch%mmean_rlong_l         (ico)
            opatch%mmean_sensible_lc     (oco) = ipatch%mmean_sensible_lc     (ico)
            opatch%mmean_vapor_lc        (oco) = ipatch%mmean_vapor_lc        (ico)
            opatch%mmean_transp          (oco) = ipatch%mmean_transp          (ico)
            opatch%mmean_intercepted_al  (oco) = ipatch%mmean_intercepted_al  (ico)
            opatch%mmean_wshed_lg        (oco) = ipatch%mmean_wshed_lg        (ico)
            opatch%mmean_rshort_w        (oco) = ipatch%mmean_rshort_w        (ico)
            opatch%mmean_rlong_w         (oco) = ipatch%mmean_rlong_w         (ico)
            opatch%mmean_sensible_wc     (oco) = ipatch%mmean_sensible_wc     (ico)
            opatch%mmean_vapor_wc        (oco) = ipatch%mmean_vapor_wc        (ico)
            opatch%mmean_intercepted_aw  (oco) = ipatch%mmean_intercepted_aw  (ico)
            opatch%mmean_wshed_wg        (oco) = ipatch%mmean_wshed_wg        (ico)
            opatch%mmean_nppleaf         (oco) = ipatch%mmean_nppleaf         (ico)
            opatch%mmean_nppfroot        (oco) = ipatch%mmean_nppfroot        (ico)
            opatch%mmean_nppsapwood      (oco) = ipatch%mmean_nppsapwood      (ico)
            opatch%mmean_nppcroot        (oco) = ipatch%mmean_nppcroot        (ico)
            opatch%mmean_nppseeds        (oco) = ipatch%mmean_nppseeds        (ico)
            opatch%mmean_nppwood         (oco) = ipatch%mmean_nppwood         (ico)
            opatch%mmean_nppdaily        (oco) = ipatch%mmean_nppdaily        (ico)
            opatch%mmsqu_gpp             (oco) = ipatch%mmsqu_gpp             (ico)
            opatch%mmsqu_npp             (oco) = ipatch%mmsqu_npp             (ico)
            opatch%mmsqu_plresp          (oco) = ipatch%mmsqu_plresp          (ico)
            opatch%mmsqu_sensible_lc     (oco) = ipatch%mmsqu_sensible_lc     (ico)
            opatch%mmsqu_vapor_lc        (oco) = ipatch%mmsqu_vapor_lc        (ico)
            opatch%mmsqu_transp          (oco) = ipatch%mmsqu_transp          (ico)
            opatch%mmsqu_sensible_wc     (oco) = ipatch%mmsqu_sensible_wc     (ico)
            opatch%mmsqu_vapor_wc        (oco) = ipatch%mmsqu_vapor_wc        (ico)
            !----- Mortality variables. ---------------------------------------------------!
            do m=1,n_mort
               opatch%mmean_mort_rate(m,oco) = ipatch%mmean_mort_rate(m,ico)
            end do
            !------------------------------------------------------------------------------!

            !------ Radiation profile variables. ------------------------------------------!
            do m=1,n_radprof
               opatch%mmean_rad_profile(m,oco) = ipatch%mmean_rad_profile(m,ico)
            end do
            !------------------------------------------------------------------------------!
         end if
         !---------------------------------------------------------------------------------!



         !---------------------------------------------------------------------------------!
         !     Mean diel.                                                                  !
         !---------------------------------------------------------------------------------!

         if (writing_dcyc) then
            do n=1,ndcycle
               opatch%qmean_gpp             (n,oco) = ipatch%qmean_gpp             (n,ico)
               opatch%qmean_npp             (n,oco) = ipatch%qmean_npp             (n,ico)
               opatch%qmean_leaf_resp       (n,oco) = ipatch%qmean_leaf_resp       (n,ico)
               opatch%qmean_root_resp       (n,oco) = ipatch%qmean_root_resp       (n,ico)
               opatch%qmean_leaf_growth_resp(n,oco) = ipatch%qmean_leaf_growth_resp(n,ico)
               opatch%qmean_root_growth_resp(n,oco) = ipatch%qmean_root_growth_resp(n,ico)
               opatch%qmean_sapa_growth_resp(n,oco) = ipatch%qmean_sapa_growth_resp(n,ico)
               opatch%qmean_sapb_growth_resp(n,oco) = ipatch%qmean_sapb_growth_resp(n,ico)
               opatch%qmean_leaf_storage_resp(n,oco) = ipatch%qmean_leaf_storage_resp(n,ico)
               opatch%qmean_root_storage_resp(n,oco) = ipatch%qmean_root_storage_resp(n,ico)
               opatch%qmean_sapa_storage_resp(n,oco) = ipatch%qmean_sapa_storage_resp(n,ico)
               opatch%qmean_sapb_storage_resp(n,oco) = ipatch%qmean_sapb_storage_resp(n,ico)
               opatch%qmean_plresp          (n,oco) = ipatch%qmean_plresp          (n,ico)
               opatch%qmean_leaf_energy     (n,oco) = ipatch%qmean_leaf_energy     (n,ico)
               opatch%qmean_leaf_water      (n,oco) = ipatch%qmean_leaf_water      (n,ico)
               opatch%qmean_leaf_hcap       (n,oco) = ipatch%qmean_leaf_hcap       (n,ico)
               opatch%qmean_leaf_vpdef      (n,oco) = ipatch%qmean_leaf_vpdef      (n,ico)
               opatch%qmean_leaf_temp       (n,oco) = ipatch%qmean_leaf_temp       (n,ico)
               opatch%qmean_leaf_fliq       (n,oco) = ipatch%qmean_leaf_fliq       (n,ico)
               opatch%qmean_leaf_gsw        (n,oco) = ipatch%qmean_leaf_gsw        (n,ico)
               opatch%qmean_leaf_gbw        (n,oco) = ipatch%qmean_leaf_gbw        (n,ico)
               opatch%qmean_wood_energy     (n,oco) = ipatch%qmean_wood_energy     (n,ico)
               opatch%qmean_wood_water      (n,oco) = ipatch%qmean_wood_water      (n,ico)
               opatch%qmean_wood_hcap       (n,oco) = ipatch%qmean_wood_hcap       (n,ico)
               opatch%qmean_wood_temp       (n,oco) = ipatch%qmean_wood_temp       (n,ico)
               opatch%qmean_wood_fliq       (n,oco) = ipatch%qmean_wood_fliq       (n,ico)
               opatch%qmean_wood_gbw        (n,oco) = ipatch%qmean_wood_gbw        (n,ico)
               opatch%qmean_fs_open         (n,oco) = ipatch%qmean_fs_open         (n,ico)
               opatch%qmean_fsw             (n,oco) = ipatch%qmean_fsw             (n,ico)
               opatch%qmean_fsn             (n,oco) = ipatch%qmean_fsn             (n,ico)
               opatch%qmean_a_open          (n,oco) = ipatch%qmean_a_open          (n,ico)
               opatch%qmean_a_closed        (n,oco) = ipatch%qmean_a_closed        (n,ico)
               opatch%qmean_a_net           (n,oco) = ipatch%qmean_a_net           (n,ico)
               opatch%qmean_A_light         (n,oco) = ipatch%qmean_A_light         (n,ico)
               opatch%qmean_A_rubp          (n,oco) = ipatch%qmean_A_rubp          (n,ico)
               opatch%qmean_A_co2           (n,oco) = ipatch%qmean_A_co2           (n,ico)
               opatch%qmean_psi_open        (n,oco) = ipatch%qmean_psi_open        (n,ico)
               opatch%qmean_psi_closed      (n,oco) = ipatch%qmean_psi_closed      (n,ico)
               opatch%qmean_water_supply    (n,oco) = ipatch%qmean_water_supply    (n,ico)
               opatch%qmean_light_level     (n,oco) = ipatch%qmean_light_level     (n,ico)
               opatch%qmean_light_level_beam(n,oco) = ipatch%qmean_light_level_beam(n,ico)
               opatch%qmean_light_level_diff(n,oco) = ipatch%qmean_light_level_diff(n,ico)

               opatch%qmean_par_level_beam  (n,oco) = ipatch%qmean_par_level_beam  (n,ico)
               opatch%qmean_par_level_diffu (n,oco) = ipatch%qmean_par_level_diffu (n,ico)
               opatch%qmean_par_level_diffd (n,oco) = ipatch%qmean_par_level_diffd (n,ico)

               opatch%qmean_par_l           (n,oco) = ipatch%qmean_par_l           (n,ico)
               opatch%qmean_par_l_beam      (n,oco) = ipatch%qmean_par_l_beam      (n,ico)
               opatch%qmean_par_l_diff      (n,oco) = ipatch%qmean_par_l_diff      (n,ico)
               opatch%qmean_rshort_l        (n,oco) = ipatch%qmean_rshort_l        (n,ico)
               opatch%qmean_rlong_l         (n,oco) = ipatch%qmean_rlong_l         (n,ico)
               opatch%qmean_sensible_lc     (n,oco) = ipatch%qmean_sensible_lc     (n,ico)
               opatch%qmean_vapor_lc        (n,oco) = ipatch%qmean_vapor_lc        (n,ico)
               opatch%qmean_transp          (n,oco) = ipatch%qmean_transp          (n,ico)
               opatch%qmean_intercepted_al  (n,oco) = ipatch%qmean_intercepted_al  (n,ico)
               opatch%qmean_wshed_lg        (n,oco) = ipatch%qmean_wshed_lg        (n,ico)
               opatch%qmean_rshort_w        (n,oco) = ipatch%qmean_rshort_w        (n,ico)
               opatch%qmean_rlong_w         (n,oco) = ipatch%qmean_rlong_w         (n,ico)
               opatch%qmean_sensible_wc     (n,oco) = ipatch%qmean_sensible_wc     (n,ico)
               opatch%qmean_vapor_wc        (n,oco) = ipatch%qmean_vapor_wc        (n,ico)
               opatch%qmean_intercepted_aw  (n,oco) = ipatch%qmean_intercepted_aw  (n,ico)
               opatch%qmean_wshed_wg        (n,oco) = ipatch%qmean_wshed_wg        (n,ico)
               opatch%qmsqu_gpp             (n,oco) = ipatch%qmsqu_gpp             (n,ico)
               opatch%qmsqu_npp             (n,oco) = ipatch%qmsqu_npp             (n,ico)
               opatch%qmsqu_plresp          (n,oco) = ipatch%qmsqu_plresp          (n,ico)
               opatch%qmsqu_sensible_lc     (n,oco) = ipatch%qmsqu_sensible_lc     (n,ico)
               opatch%qmsqu_vapor_lc        (n,oco) = ipatch%qmsqu_vapor_lc        (n,ico)
               opatch%qmsqu_transp          (n,oco) = ipatch%qmsqu_transp          (n,ico)
               opatch%qmsqu_sensible_wc     (n,oco) = ipatch%qmsqu_sensible_wc     (n,ico)
               opatch%qmsqu_vapor_wc        (n,oco) = ipatch%qmsqu_vapor_wc        (n,ico)

               !------ Radiation profile variables. ---------------------------------------!
               do m=1,n_radprof
                  opatch%qmean_rad_profile(m,n,oco) = ipatch%qmean_rad_profile(m,n,ico)
               end do
               !---------------------------------------------------------------------------!
            end do
            !------------------------------------------------------------------------------!
         end if
         !---------------------------------------------------------------------------------!
      end do cohortloop

      return
   end subroutine copy_patchtype
   !=======================================================================================!
   !=======================================================================================!





   !=======================================================================================!
   !=======================================================================================!
   !      This subroutine copies part of one cohort to another (only the ones where mask   !
   ! is .true.).  Please notice the following requirements.                                !
   !                                                                                       !
   !      1. IPATCH and LMASK must have dimension MASKSZ                                   !
   !      2. OPATCH must have dimension NEWSZ                                              !
   !      3. The number of .true. elements of LMASK must be NEWSZ                          !
   !                                                                                       !
   !      In case these conditions aren't met, then the code will crash due to             !
   ! segmentation violation.                                                               !
   !                                                                                       !
   !      Also, NEVER use this sub-routine for a pre-allocated OPATCH.  This will try to   !
   ! allocate cohorts and you may experience horrible memory leaks.                        !
   !---------------------------------------------------------------------------------------!
   subroutine copy_patchtype_mask(ipatch,opatch,lmask,isize,osize)
      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(patchtype)         , target     :: ipatch
      type(patchtype)         , target     :: opatch
      integer                 , intent(in) :: isize
      integer                 , intent(in) :: osize
      logical,dimension(isize), intent(in) :: lmask
      !----- Local variables. -------------------------------------------------------------!
      integer                              :: i
      integer                              :: m
      integer                              :: n
      integer                              :: z
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      !     Map the indices between the input and the output vectors.                      !
      !------------------------------------------------------------------------------------!
      z = count(lmask)
      !----- Crash if the dimensions don't match. -----------------------------------------!
      if (z /= osize) then
         write (unit=*,fmt='(a,1x,i12)') ' Z     = ',z
         write (unit=*,fmt='(a,1x,i12)') ' OSIZE = ',osize
         call fatal_error('Mask doesn''t have OSIZE .true. elements'                       &
                         ,'copy_sitetype_mask','ed_state_vars.f90')
      end if
      !----- In case nothing should be copied, we return. ---------------------------------!
      if (z == 0) return
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !      Fortran doesn't like sub-routines that call too many sub-routines or          !
      ! functions.  We split the packing procedure to reduce the number of calls per       !
      ! procedure and make the compilation more efficient.                                 !
      !------------------------------------------------------------------------------------!
      call copy_patchtype_mask_inst (ipatch,opatch,z,lmask,isize,osize)
      call copy_patchtype_mask_fmean(ipatch,opatch,z,lmask,isize,osize)
      if (writing_long) call copy_patchtype_mask_dmean(ipatch,opatch,z,lmask,isize,osize)
      if (writing_eorq) call copy_patchtype_mask_mmean(ipatch,opatch,z,lmask,isize,osize)
      if (writing_dcyc) call copy_patchtype_mask_qmean(ipatch,opatch,z,lmask,isize,osize)
      !------------------------------------------------------------------------------------!

      return
   end subroutine copy_patchtype_mask
   !=======================================================================================!
   !=======================================================================================!





   !=======================================================================================!
   !=======================================================================================!
   !     This sub-routine copies the cohort-level variables that are not fmean, mmean,     !
   ! mmsqu, qmean, qmsqu.  The only sub-routine that should ever call this one is          !
   ! copy_patchtype_mask.                                                                  !
   !---------------------------------------------------------------------------------------!
   subroutine copy_patchtype_mask_inst(ipatch,opatch,z,lmask,isize,osize)
      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(patchtype)         , target     :: ipatch
      type(patchtype)         , target     :: opatch
      integer                 , intent(in) :: z
      integer                 , intent(in) :: isize
      integer                 , intent(in) :: osize
      logical,dimension(isize), intent(in) :: lmask
      !----- Local variables. -------------------------------------------------------------!
      integer                              :: i
      integer                              :: m
      integer                              :: n
      !------------------------------------------------------------------------------------!



      !----- Scalars. ---------------------------------------------------------------------!
      opatch%pft                   (1:z) = pack(ipatch%pft                       ,lmask)
      opatch%nplant                (1:z) = pack(ipatch%nplant                    ,lmask)
      opatch%phenology_status      (1:z) = pack(ipatch%phenology_status          ,lmask)
      opatch%recruit_dbh           (1:z) = pack(ipatch%recruit_dbh               ,lmask)
      opatch%census_status         (1:z) = pack(ipatch%census_status             ,lmask)
      opatch%hite                  (1:z) = pack(ipatch%hite                      ,lmask)
      opatch%agb                   (1:z) = pack(ipatch%agb                       ,lmask)
      opatch%basarea               (1:z) = pack(ipatch%basarea                   ,lmask)
      opatch%dagb_dt               (1:z) = pack(ipatch%dagb_dt                   ,lmask)
      opatch%dlnagb_dt             (1:z) = pack(ipatch%dlnagb_dt                 ,lmask)
      opatch%dba_dt                (1:z) = pack(ipatch%dba_dt                    ,lmask)
      opatch%dlnba_dt              (1:z) = pack(ipatch%dlnba_dt                  ,lmask)
      opatch%ddbh_dt               (1:z) = pack(ipatch%ddbh_dt                   ,lmask)
      opatch%dlndbh_dt             (1:z) = pack(ipatch%dlndbh_dt                 ,lmask)
      opatch%dbh                   (1:z) = pack(ipatch%dbh                       ,lmask)
      opatch%bdead                 (1:z) = pack(ipatch%bdead                     ,lmask)
      opatch%bleaf                 (1:z) = pack(ipatch%bleaf                     ,lmask)
      opatch%balive                (1:z) = pack(ipatch%balive                    ,lmask)
      opatch%broot                 (1:z) = pack(ipatch%broot                     ,lmask)
      opatch%bsapwooda             (1:z) = pack(ipatch%bsapwooda                 ,lmask)
      opatch%bsapwoodb             (1:z) = pack(ipatch%bsapwoodb                 ,lmask)
      opatch%bstorage              (1:z) = pack(ipatch%bstorage                  ,lmask)
      opatch%bseeds                (1:z) = pack(ipatch%bseeds                    ,lmask)
      opatch%lai                   (1:z) = pack(ipatch%lai                       ,lmask)
      opatch%wai                   (1:z) = pack(ipatch%wai                       ,lmask)
      opatch%crown_area            (1:z) = pack(ipatch%crown_area                ,lmask)
      opatch%leaf_resolvable       (1:z) = pack(ipatch%leaf_resolvable           ,lmask)
      opatch%wood_resolvable       (1:z) = pack(ipatch%wood_resolvable           ,lmask)
      opatch%cbr_bar               (1:z) = pack(ipatch%cbr_bar                   ,lmask)
      opatch%leaf_energy           (1:z) = pack(ipatch%leaf_energy               ,lmask)
      opatch%leaf_temp             (1:z) = pack(ipatch%leaf_temp                 ,lmask)
      opatch%leaf_vpdef            (1:z) = pack(ipatch%leaf_vpdef                ,lmask)
      opatch%leaf_temp_pv          (1:z) = pack(ipatch%leaf_temp_pv              ,lmask)
      opatch%leaf_hcap             (1:z) = pack(ipatch%leaf_hcap                 ,lmask)
      opatch%leaf_fliq             (1:z) = pack(ipatch%leaf_fliq                 ,lmask)
      opatch%leaf_water            (1:z) = pack(ipatch%leaf_water                ,lmask)
      opatch%wood_energy           (1:z) = pack(ipatch%wood_energy               ,lmask)
      opatch%wood_temp             (1:z) = pack(ipatch%wood_temp                 ,lmask)
      opatch%wood_temp_pv          (1:z) = pack(ipatch%wood_temp_pv              ,lmask)
      opatch%wood_hcap             (1:z) = pack(ipatch%wood_hcap                 ,lmask)
      opatch%wood_fliq             (1:z) = pack(ipatch%wood_fliq                 ,lmask)
      opatch%wood_water            (1:z) = pack(ipatch%wood_water                ,lmask)
      opatch%veg_wind              (1:z) = pack(ipatch%veg_wind                  ,lmask)
      opatch%lsfc_shv_open         (1:z) = pack(ipatch%lsfc_shv_open             ,lmask)
      opatch%lsfc_shv_closed       (1:z) = pack(ipatch%lsfc_shv_closed           ,lmask)
      opatch%lsfc_co2_open         (1:z) = pack(ipatch%lsfc_co2_open             ,lmask)
      opatch%lsfc_co2_closed       (1:z) = pack(ipatch%lsfc_co2_closed           ,lmask)
      opatch%lint_shv              (1:z) = pack(ipatch%lint_shv                  ,lmask)
      opatch%lint_co2_open         (1:z) = pack(ipatch%lint_co2_open             ,lmask)
      opatch%lint_co2_closed       (1:z) = pack(ipatch%lint_co2_closed           ,lmask)
      opatch%today_leaf_resp       (1:z) = pack(ipatch%today_leaf_resp           ,lmask)
      opatch%today_root_resp       (1:z) = pack(ipatch%today_root_resp           ,lmask)
      opatch%today_gpp             (1:z) = pack(ipatch%today_gpp                 ,lmask)
      opatch%today_gpp_pot         (1:z) = pack(ipatch%today_gpp_pot             ,lmask)
      opatch%today_gpp_lightmax    (1:z) = pack(ipatch%today_gpp_lightmax        ,lmask)
      opatch%today_gpp_moistmax    (1:z) = pack(ipatch%today_gpp_moistmax        ,lmask)
      opatch%today_gpp_mlmax       (1:z) = pack(ipatch%today_gpp_mlmax           ,lmask)
      opatch%today_nppleaf         (1:z) = pack(ipatch%today_nppleaf             ,lmask)
      opatch%today_nppfroot        (1:z) = pack(ipatch%today_nppfroot            ,lmask)
      opatch%today_nppsapwood      (1:z) = pack(ipatch%today_nppsapwood          ,lmask)
      opatch%today_nppcroot        (1:z) = pack(ipatch%today_nppcroot            ,lmask)
      opatch%today_nppseeds        (1:z) = pack(ipatch%today_nppseeds            ,lmask)
      opatch%today_nppwood         (1:z) = pack(ipatch%today_nppwood             ,lmask)
      opatch%today_nppdaily        (1:z) = pack(ipatch%today_nppdaily            ,lmask)
      opatch%leaf_growth_resp      (1:z) = pack(ipatch%leaf_growth_resp          ,lmask)
      opatch%root_growth_resp      (1:z) = pack(ipatch%root_growth_resp          ,lmask)
      opatch%sapa_growth_resp      (1:z) = pack(ipatch%sapa_growth_resp          ,lmask)
      opatch%sapb_growth_resp      (1:z) = pack(ipatch%sapb_growth_resp          ,lmask)
      opatch%leaf_storage_resp     (1:z) = pack(ipatch%leaf_storage_resp         ,lmask)
      opatch%root_storage_resp     (1:z) = pack(ipatch%root_storage_resp         ,lmask)
      opatch%sapa_storage_resp     (1:z) = pack(ipatch%sapa_storage_resp         ,lmask)
      opatch%sapb_storage_resp     (1:z) = pack(ipatch%sapb_storage_resp         ,lmask)
      opatch%monthly_dndt          (1:z) = pack(ipatch%monthly_dndt              ,lmask)
      opatch%monthly_dlnndt        (1:z) = pack(ipatch%monthly_dlnndt            ,lmask)
      opatch%krdepth               (1:z) = pack(ipatch%krdepth                   ,lmask)
      opatch%first_census          (1:z) = pack(ipatch%first_census              ,lmask)
      opatch%new_recruit_flag      (1:z) = pack(ipatch%new_recruit_flag          ,lmask)
      opatch%light_level           (1:z) = pack(ipatch%light_level               ,lmask)
      opatch%light_level_beam      (1:z) = pack(ipatch%light_level_beam          ,lmask)
      opatch%light_level_diff      (1:z) = pack(ipatch%light_level_diff          ,lmask)

      opatch%par_level_beam        (1:z) = pack(ipatch%par_level_beam            ,lmask)
      opatch%par_level_diffu       (1:z) = pack(ipatch%par_level_diffu           ,lmask)
      opatch%par_level_diffd       (1:z) = pack(ipatch%par_level_diffd           ,lmask)

      opatch%par_l                 (1:z) = pack(ipatch%par_l                     ,lmask)
      opatch%par_l_beam            (1:z) = pack(ipatch%par_l_beam                ,lmask)
      opatch%par_l_diffuse         (1:z) = pack(ipatch%par_l_diffuse             ,lmask)
      opatch%rshort_l              (1:z) = pack(ipatch%rshort_l                  ,lmask)
      opatch%rshort_l_beam         (1:z) = pack(ipatch%rshort_l_beam             ,lmask)
      opatch%rshort_l_diffuse      (1:z) = pack(ipatch%rshort_l_diffuse          ,lmask)
      opatch%rlong_l               (1:z) = pack(ipatch%rlong_l                   ,lmask)
      opatch%rshort_w              (1:z) = pack(ipatch%rshort_w                  ,lmask)
      opatch%rshort_w_beam         (1:z) = pack(ipatch%rshort_w_beam             ,lmask)
      opatch%rshort_w_diffuse      (1:z) = pack(ipatch%rshort_w_diffuse          ,lmask)
      opatch%rlong_w               (1:z) = pack(ipatch%rlong_w                   ,lmask)
      opatch%leaf_gbh              (1:z) = pack(ipatch%leaf_gbh                  ,lmask)
      opatch%leaf_gbw              (1:z) = pack(ipatch%leaf_gbw                  ,lmask)
      opatch%wood_gbh              (1:z) = pack(ipatch%wood_gbh                  ,lmask)
      opatch%wood_gbw              (1:z) = pack(ipatch%wood_gbw                  ,lmask)
      opatch%A_open                (1:z) = pack(ipatch%A_open                    ,lmask)
      opatch%A_closed              (1:z) = pack(ipatch%A_closed                  ,lmask)
      opatch%A_light               (1:z) = pack(ipatch%A_light                   ,lmask)
      opatch%A_rubp                (1:z) = pack(ipatch%A_rubp                    ,lmask)
      opatch%A_co2                 (1:z) = pack(ipatch%A_co2                     ,lmask)
      opatch%psi_open              (1:z) = pack(ipatch%psi_open                  ,lmask)
      opatch%psi_closed            (1:z) = pack(ipatch%psi_closed                ,lmask)
      opatch%gsw_open              (1:z) = pack(ipatch%gsw_open                  ,lmask)
      opatch%gsw_closed            (1:z) = pack(ipatch%gsw_closed                ,lmask)
      opatch%leaf_gsw              (1:z) = pack(ipatch%leaf_gsw                  ,lmask)
      opatch%fsw                   (1:z) = pack(ipatch%fsw                       ,lmask)
      opatch%fsn                   (1:z) = pack(ipatch%fsn                       ,lmask)
      opatch%fs_open               (1:z) = pack(ipatch%fs_open                   ,lmask)
      opatch%water_supply          (1:z) = pack(ipatch%water_supply              ,lmask)
      opatch%leaf_maintenance      (1:z) = pack(ipatch%leaf_maintenance          ,lmask)
      opatch%root_maintenance      (1:z) = pack(ipatch%root_maintenance          ,lmask)
      opatch%leaf_drop             (1:z) = pack(ipatch%leaf_drop                 ,lmask)
      opatch%leaf_respiration      (1:z) = pack(ipatch%leaf_respiration          ,lmask)
      opatch%root_respiration      (1:z) = pack(ipatch%root_respiration          ,lmask)
      opatch%gpp                   (1:z) = pack(ipatch%gpp                       ,lmask)
      opatch%paw_avg               (1:z) = pack(ipatch%paw_avg                   ,lmask)
      opatch%elongf                (1:z) = pack(ipatch%elongf                    ,lmask)
      opatch%turnover_amp          (1:z) = pack(ipatch%turnover_amp              ,lmask)
      opatch%llspan                (1:z) = pack(ipatch%llspan                    ,lmask)
      opatch%vm_bar                (1:z) = pack(ipatch%vm_bar                    ,lmask)
      opatch%sla                   (1:z) = pack(ipatch%sla                       ,lmask)
      !------------------------------------------------------------------------------------!


      !------ Carbon balance variables. ---------------------------------------------------!
      do m=1,13
         opatch%cb         (m,1:z) = pack(ipatch%cb           (m,:),lmask)
         opatch%cb_lightmax(m,1:z) = pack(ipatch%cb_lightmax  (m,:),lmask)
         opatch%cb_moistmax(m,1:z) = pack(ipatch%cb_moistmax  (m,:),lmask)
         opatch%cb_mlmax   (m,1:z) = pack(ipatch%cb_mlmax     (m,:),lmask)
      end do
      !------------------------------------------------------------------------------------!


      !------ Mortality variables. --------------------------------------------------------!
      do m=1,n_mort
         opatch%mort_rate(m,1:z) = pack(ipatch%mort_rate  (m,:),lmask)
      end do
      !------------------------------------------------------------------------------------!


      !------ Radiation profile variables. ------------------------------------------------!
      do m=1,n_radprof
         opatch%rad_profile(m,1:z) = pack(ipatch%rad_profile  (m,:),lmask)
      end do
      !------------------------------------------------------------------------------------!

      return
   end subroutine copy_patchtype_mask_inst
   !=======================================================================================!
   !=======================================================================================!





   !=======================================================================================!
   !=======================================================================================!
   !     This sub-routine copies the cohort-level variables that are fmean.                !
   ! The only sub-routine that should ever call this one is copy_patchtype_mask.           !
   !---------------------------------------------------------------------------------------!
   subroutine copy_patchtype_mask_fmean(ipatch,opatch,z,lmask,isize,osize)
      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(patchtype)         , target     :: ipatch
      type(patchtype)         , target     :: opatch
      integer                 , intent(in) :: z
      integer                 , intent(in) :: isize
      integer                 , intent(in) :: osize
      logical,dimension(isize), intent(in) :: lmask
      !----- Local variables. -------------------------------------------------------------!
      integer                              :: i
      integer                              :: m
      integer                              :: n
      !------------------------------------------------------------------------------------!



      opatch%fmean_gpp             (1:z) = pack(ipatch%fmean_gpp                 ,lmask)
      opatch%fmean_npp             (1:z) = pack(ipatch%fmean_npp                 ,lmask)
      opatch%fmean_leaf_resp       (1:z) = pack(ipatch%fmean_leaf_resp           ,lmask)
      opatch%fmean_root_resp       (1:z) = pack(ipatch%fmean_root_resp           ,lmask)
      opatch%fmean_leaf_growth_resp(1:z) = pack(ipatch%fmean_leaf_growth_resp    ,lmask)
      opatch%fmean_root_growth_resp(1:z) = pack(ipatch%fmean_root_growth_resp    ,lmask)
      opatch%fmean_sapa_growth_resp(1:z) = pack(ipatch%fmean_sapa_growth_resp    ,lmask)
      opatch%fmean_sapb_growth_resp(1:z) = pack(ipatch%fmean_sapb_growth_resp    ,lmask)
      opatch%fmean_leaf_storage_resp(1:z) = pack(ipatch%fmean_leaf_storage_resp  ,lmask)
      opatch%fmean_root_storage_resp(1:z) = pack(ipatch%fmean_root_storage_resp  ,lmask)
      opatch%fmean_sapa_storage_resp(1:z) = pack(ipatch%fmean_sapa_storage_resp  ,lmask)
      opatch%fmean_sapb_storage_resp(1:z) = pack(ipatch%fmean_sapb_storage_resp  ,lmask)
      opatch%fmean_plresp          (1:z) = pack(ipatch%fmean_plresp              ,lmask)
      opatch%fmean_leaf_energy     (1:z) = pack(ipatch%fmean_leaf_energy         ,lmask)
      opatch%fmean_leaf_water      (1:z) = pack(ipatch%fmean_leaf_water          ,lmask)
      opatch%fmean_leaf_hcap       (1:z) = pack(ipatch%fmean_leaf_hcap           ,lmask)
      opatch%fmean_leaf_vpdef      (1:z) = pack(ipatch%fmean_leaf_vpdef          ,lmask)
      opatch%fmean_leaf_temp       (1:z) = pack(ipatch%fmean_leaf_temp           ,lmask)
      opatch%fmean_leaf_fliq       (1:z) = pack(ipatch%fmean_leaf_fliq           ,lmask)
      opatch%fmean_leaf_gsw        (1:z) = pack(ipatch%fmean_leaf_gsw            ,lmask)
      opatch%fmean_leaf_gbw        (1:z) = pack(ipatch%fmean_leaf_gbw            ,lmask)
      opatch%fmean_wood_energy     (1:z) = pack(ipatch%fmean_wood_energy         ,lmask)
      opatch%fmean_wood_water      (1:z) = pack(ipatch%fmean_wood_water          ,lmask)
      opatch%fmean_wood_hcap       (1:z) = pack(ipatch%fmean_wood_hcap           ,lmask)
      opatch%fmean_wood_temp       (1:z) = pack(ipatch%fmean_wood_temp           ,lmask)
      opatch%fmean_wood_fliq       (1:z) = pack(ipatch%fmean_wood_fliq           ,lmask)
      opatch%fmean_wood_gbw        (1:z) = pack(ipatch%fmean_wood_gbw            ,lmask)
      opatch%fmean_fs_open         (1:z) = pack(ipatch%fmean_fs_open             ,lmask)
      opatch%fmean_fsw             (1:z) = pack(ipatch%fmean_fsw                 ,lmask)
      opatch%fmean_fsn             (1:z) = pack(ipatch%fmean_fsn                 ,lmask)
      opatch%fmean_a_open          (1:z) = pack(ipatch%fmean_a_open              ,lmask)
      opatch%fmean_a_closed        (1:z) = pack(ipatch%fmean_a_closed            ,lmask)
      opatch%fmean_a_net           (1:z) = pack(ipatch%fmean_a_net               ,lmask)
      opatch%fmean_A_light         (1:z) = pack(ipatch%fmean_A_light             ,lmask)
      opatch%fmean_A_rubp          (1:z) = pack(ipatch%fmean_A_rubp              ,lmask)
      opatch%fmean_A_co2           (1:z) = pack(ipatch%fmean_A_co2               ,lmask)
      opatch%fmean_psi_open        (1:z) = pack(ipatch%fmean_psi_open            ,lmask)
      opatch%fmean_psi_closed      (1:z) = pack(ipatch%fmean_psi_closed          ,lmask)
      opatch%fmean_water_supply    (1:z) = pack(ipatch%fmean_water_supply        ,lmask)
      opatch%fmean_light_level     (1:z) = pack(ipatch%fmean_light_level         ,lmask)
      opatch%fmean_light_level_beam(1:z) = pack(ipatch%fmean_light_level_beam    ,lmask)
      opatch%fmean_light_level_diff(1:z) = pack(ipatch%fmean_light_level_diff    ,lmask)

      opatch%fmean_par_level_beam  (1:z) = pack(ipatch%fmean_par_level_beam      ,lmask)
      opatch%fmean_par_level_diffu (1:z) = pack(ipatch%fmean_par_level_diffu     ,lmask)
      opatch%fmean_par_level_diffd (1:z) = pack(ipatch%fmean_par_level_diffd     ,lmask)

      opatch%fmean_par_l           (1:z) = pack(ipatch%fmean_par_l               ,lmask)
      opatch%fmean_par_l_beam      (1:z) = pack(ipatch%fmean_par_l_beam          ,lmask)
      opatch%fmean_par_l_diff      (1:z) = pack(ipatch%fmean_par_l_diff          ,lmask)
      opatch%fmean_rshort_l        (1:z) = pack(ipatch%fmean_rshort_l            ,lmask)
      opatch%fmean_rlong_l         (1:z) = pack(ipatch%fmean_rlong_l             ,lmask)
      opatch%fmean_sensible_lc     (1:z) = pack(ipatch%fmean_sensible_lc         ,lmask)
      opatch%fmean_vapor_lc        (1:z) = pack(ipatch%fmean_vapor_lc            ,lmask)
      opatch%fmean_transp          (1:z) = pack(ipatch%fmean_transp              ,lmask)
      opatch%fmean_intercepted_al  (1:z) = pack(ipatch%fmean_intercepted_al      ,lmask)
      opatch%fmean_wshed_lg        (1:z) = pack(ipatch%fmean_wshed_lg            ,lmask)
      opatch%fmean_rshort_w        (1:z) = pack(ipatch%fmean_rshort_w            ,lmask)
      opatch%fmean_rlong_w         (1:z) = pack(ipatch%fmean_rlong_w             ,lmask)
      opatch%fmean_sensible_wc     (1:z) = pack(ipatch%fmean_sensible_wc         ,lmask)
      opatch%fmean_vapor_wc        (1:z) = pack(ipatch%fmean_vapor_wc            ,lmask)
      opatch%fmean_intercepted_aw  (1:z) = pack(ipatch%fmean_intercepted_aw      ,lmask)
      opatch%fmean_wshed_wg        (1:z) = pack(ipatch%fmean_wshed_wg            ,lmask)
      opatch%fmean_lai             (1:z) = pack(ipatch%fmean_lai                 ,lmask)
      opatch%fmean_bdead           (1:z) = pack(ipatch%fmean_bdead               ,lmask)
      !------------------------------------------------------------------------------------!


      !------ Radiation profile variables. ------------------------------------------------!
      do m=1,n_radprof
         opatch%fmean_rad_profile(m,1:z) = pack(ipatch%fmean_rad_profile  (m,:),lmask)
      end do
      !------------------------------------------------------------------------------------!

      return
   end subroutine copy_patchtype_mask_fmean
   !=======================================================================================!
   !=======================================================================================!





   !=======================================================================================!
   !=======================================================================================!
   !     This sub-routine copies the cohort-level variables that are dmean.                !
   ! The only sub-routine that should ever call this one is copy_patchtype_mask.           !
   !---------------------------------------------------------------------------------------!
   subroutine copy_patchtype_mask_dmean(ipatch,opatch,z,lmask,isize,osize)
      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(patchtype)         , target     :: ipatch
      type(patchtype)         , target     :: opatch
      integer                 , intent(in) :: z
      integer                 , intent(in) :: isize
      integer                 , intent(in) :: osize
      logical,dimension(isize), intent(in) :: lmask
      !----- Local variables. -------------------------------------------------------------!
      integer                              :: i
      integer                              :: m
      integer                              :: n
      !------------------------------------------------------------------------------------!



      !----- Scalars. ------------------------------------------------------------------!
      opatch%dmean_nppleaf         (1:z) = pack(ipatch%dmean_nppleaf             ,lmask)
      opatch%dmean_nppfroot        (1:z) = pack(ipatch%dmean_nppfroot            ,lmask)
      opatch%dmean_nppsapwood      (1:z) = pack(ipatch%dmean_nppsapwood          ,lmask)
      opatch%dmean_nppcroot        (1:z) = pack(ipatch%dmean_nppcroot            ,lmask)
      opatch%dmean_nppseeds        (1:z) = pack(ipatch%dmean_nppseeds            ,lmask)
      opatch%dmean_nppwood         (1:z) = pack(ipatch%dmean_nppwood             ,lmask)
      opatch%dmean_nppdaily        (1:z) = pack(ipatch%dmean_nppdaily            ,lmask)
      opatch%dmean_gpp             (1:z) = pack(ipatch%dmean_gpp                 ,lmask)
      opatch%dmean_npp             (1:z) = pack(ipatch%dmean_npp                 ,lmask)
      opatch%dmean_leaf_resp       (1:z) = pack(ipatch%dmean_leaf_resp           ,lmask)
      opatch%dmean_root_resp       (1:z) = pack(ipatch%dmean_root_resp           ,lmask)
      opatch%dmean_leaf_growth_resp(1:z) = pack(ipatch%dmean_leaf_growth_resp    ,lmask)
      opatch%dmean_root_growth_resp(1:z) = pack(ipatch%dmean_root_growth_resp    ,lmask)
      opatch%dmean_sapa_growth_resp(1:z) = pack(ipatch%dmean_sapa_growth_resp    ,lmask)
      opatch%dmean_sapb_growth_resp(1:z) = pack(ipatch%dmean_sapb_growth_resp    ,lmask)
      opatch%dmean_leaf_storage_resp(1:z) = pack(ipatch%dmean_leaf_storage_resp  ,lmask)
      opatch%dmean_root_storage_resp(1:z) = pack(ipatch%dmean_root_storage_resp  ,lmask)
      opatch%dmean_sapa_storage_resp(1:z) = pack(ipatch%dmean_sapa_storage_resp  ,lmask)
      opatch%dmean_sapb_storage_resp(1:z) = pack(ipatch%dmean_sapb_storage_resp  ,lmask)
      opatch%dmean_plresp          (1:z) = pack(ipatch%dmean_plresp              ,lmask)
      opatch%dmean_leaf_energy     (1:z) = pack(ipatch%dmean_leaf_energy         ,lmask)
      opatch%dmean_leaf_water      (1:z) = pack(ipatch%dmean_leaf_water          ,lmask)
      opatch%dmean_leaf_hcap       (1:z) = pack(ipatch%dmean_leaf_hcap           ,lmask)
      opatch%dmean_leaf_vpdef      (1:z) = pack(ipatch%dmean_leaf_vpdef          ,lmask)
      opatch%dmean_leaf_temp       (1:z) = pack(ipatch%dmean_leaf_temp           ,lmask)
      opatch%dmean_leaf_fliq       (1:z) = pack(ipatch%dmean_leaf_fliq           ,lmask)
      opatch%dmean_leaf_gsw        (1:z) = pack(ipatch%dmean_leaf_gsw            ,lmask)
      opatch%dmean_leaf_gbw        (1:z) = pack(ipatch%dmean_leaf_gbw            ,lmask)
      opatch%dmean_wood_energy     (1:z) = pack(ipatch%dmean_wood_energy         ,lmask)
      opatch%dmean_wood_water      (1:z) = pack(ipatch%dmean_wood_water          ,lmask)
      opatch%dmean_wood_hcap       (1:z) = pack(ipatch%dmean_wood_hcap           ,lmask)
      opatch%dmean_wood_temp       (1:z) = pack(ipatch%dmean_wood_temp           ,lmask)
      opatch%dmean_wood_fliq       (1:z) = pack(ipatch%dmean_wood_fliq           ,lmask)
      opatch%dmean_wood_gbw        (1:z) = pack(ipatch%dmean_wood_gbw            ,lmask)
      opatch%dmean_fs_open         (1:z) = pack(ipatch%dmean_fs_open             ,lmask)
      opatch%dmean_fsw             (1:z) = pack(ipatch%dmean_fsw                 ,lmask)
      opatch%dmean_fsn             (1:z) = pack(ipatch%dmean_fsn                 ,lmask)
      opatch%dmean_a_open          (1:z) = pack(ipatch%dmean_a_open              ,lmask)
      opatch%dmean_a_closed        (1:z) = pack(ipatch%dmean_a_closed            ,lmask)
      opatch%dmean_a_net           (1:z) = pack(ipatch%dmean_a_net               ,lmask)
      opatch%dmean_A_light         (1:z) = pack(ipatch%dmean_A_light             ,lmask)
      opatch%dmean_A_rubp          (1:z) = pack(ipatch%dmean_A_rubp              ,lmask)
      opatch%dmean_A_co2           (1:z) = pack(ipatch%dmean_A_co2               ,lmask)
      opatch%dmean_psi_open        (1:z) = pack(ipatch%dmean_psi_open            ,lmask)
      opatch%dmean_psi_closed      (1:z) = pack(ipatch%dmean_psi_closed          ,lmask)
      opatch%dmean_water_supply    (1:z) = pack(ipatch%dmean_water_supply        ,lmask)
      opatch%dmean_light_level     (1:z) = pack(ipatch%dmean_light_level         ,lmask)
      opatch%dmean_light_level_beam(1:z) = pack(ipatch%dmean_light_level_beam    ,lmask)
      opatch%dmean_light_level_diff(1:z) = pack(ipatch%dmean_light_level_diff    ,lmask)

      opatch%dmean_par_level_beam  (1:z) = pack(ipatch%dmean_par_level_beam      ,lmask)
      opatch%dmean_par_level_diffu (1:z) = pack(ipatch%dmean_par_level_diffu     ,lmask)
      opatch%dmean_par_level_diffd (1:z) = pack(ipatch%dmean_par_level_diffd     ,lmask)

      opatch%dmean_par_l           (1:z) = pack(ipatch%dmean_par_l               ,lmask)
      opatch%dmean_par_l_beam      (1:z) = pack(ipatch%dmean_par_l_beam          ,lmask)
      opatch%dmean_par_l_diff      (1:z) = pack(ipatch%dmean_par_l_diff          ,lmask)
      opatch%dmean_rshort_l        (1:z) = pack(ipatch%dmean_rshort_l            ,lmask)
      opatch%dmean_rlong_l         (1:z) = pack(ipatch%dmean_rlong_l             ,lmask)
      opatch%dmean_sensible_lc     (1:z) = pack(ipatch%dmean_sensible_lc         ,lmask)
      opatch%dmean_vapor_lc        (1:z) = pack(ipatch%dmean_vapor_lc            ,lmask)
      opatch%dmean_transp          (1:z) = pack(ipatch%dmean_transp              ,lmask)
      opatch%dmean_intercepted_al  (1:z) = pack(ipatch%dmean_intercepted_al      ,lmask)
      opatch%dmean_wshed_lg        (1:z) = pack(ipatch%dmean_wshed_lg            ,lmask)
      opatch%dmean_rshort_w        (1:z) = pack(ipatch%dmean_rshort_w            ,lmask)
      opatch%dmean_rlong_w         (1:z) = pack(ipatch%dmean_rlong_w             ,lmask)
      opatch%dmean_sensible_wc     (1:z) = pack(ipatch%dmean_sensible_wc         ,lmask)
      opatch%dmean_vapor_wc        (1:z) = pack(ipatch%dmean_vapor_wc            ,lmask)
      opatch%dmean_intercepted_aw  (1:z) = pack(ipatch%dmean_intercepted_aw      ,lmask)
      opatch%dmean_wshed_wg        (1:z) = pack(ipatch%dmean_wshed_wg            ,lmask)
      !------------------------------------------------------------------------------------!


      !------ Radiation profile variables. ------------------------------------------------!
      do m=1,n_radprof
         opatch%dmean_rad_profile(m,1:z) = pack(ipatch%dmean_rad_profile  (m,:),lmask)
      end do
      !------------------------------------------------------------------------------------!

      return
   end subroutine copy_patchtype_mask_dmean
   !=======================================================================================!
   !=======================================================================================!





   !=======================================================================================!
   !=======================================================================================!
   !     This sub-routine copies the cohort-level variables that are mmean or mmsqu.       !
   ! The only sub-routine that should ever call this one is copy_patchtype_mask.           !
   !---------------------------------------------------------------------------------------!
   subroutine copy_patchtype_mask_mmean(ipatch,opatch,z,lmask,isize,osize)
      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(patchtype)         , target     :: ipatch
      type(patchtype)         , target     :: opatch
      integer                 , intent(in) :: z
      integer                 , intent(in) :: isize
      integer                 , intent(in) :: osize
      logical,dimension(isize), intent(in) :: lmask
      !----- Local variables. -------------------------------------------------------------!
      integer                              :: i
      integer                              :: m
      integer                              :: n
      !------------------------------------------------------------------------------------!



      !----- Scalars. ---------------------------------------------------------------------!
      opatch%mmean_lai             (1:z) = pack(ipatch%mmean_lai                 ,lmask)
      opatch%mmean_bleaf           (1:z) = pack(ipatch%mmean_bleaf               ,lmask)
      opatch%mmean_broot           (1:z) = pack(ipatch%mmean_broot               ,lmask)
      opatch%mmean_bstorage        (1:z) = pack(ipatch%mmean_bstorage            ,lmask)
      opatch%mmean_leaf_maintenance(1:z) = pack(ipatch%mmean_leaf_maintenance    ,lmask)
      opatch%mmean_root_maintenance(1:z) = pack(ipatch%mmean_root_maintenance    ,lmask)
      opatch%mmean_leaf_drop       (1:z) = pack(ipatch%mmean_leaf_drop           ,lmask)
      opatch%mmean_cb              (1:z) = pack(ipatch%mmean_cb                  ,lmask)
      opatch%mmean_gpp             (1:z) = pack(ipatch%mmean_gpp                 ,lmask)
      opatch%mmean_npp             (1:z) = pack(ipatch%mmean_npp                 ,lmask)
      opatch%mmean_leaf_resp       (1:z) = pack(ipatch%mmean_leaf_resp           ,lmask)
      opatch%mmean_root_resp       (1:z) = pack(ipatch%mmean_root_resp           ,lmask)
      opatch%mmean_leaf_growth_resp(1:z) = pack(ipatch%mmean_leaf_growth_resp    ,lmask)
      opatch%mmean_root_growth_resp(1:z) = pack(ipatch%mmean_root_growth_resp    ,lmask)
      opatch%mmean_sapa_growth_resp(1:z) = pack(ipatch%mmean_sapa_growth_resp    ,lmask)
      opatch%mmean_sapb_growth_resp(1:z) = pack(ipatch%mmean_sapb_growth_resp    ,lmask)
      opatch%mmean_leaf_storage_resp(1:z) = pack(ipatch%mmean_leaf_storage_resp  ,lmask)
      opatch%mmean_root_storage_resp(1:z) = pack(ipatch%mmean_root_storage_resp  ,lmask)
      opatch%mmean_sapa_storage_resp(1:z) = pack(ipatch%mmean_sapa_storage_resp  ,lmask)
      opatch%mmean_sapb_storage_resp(1:z) = pack(ipatch%mmean_sapb_storage_resp  ,lmask)
      opatch%mmean_plresp          (1:z) = pack(ipatch%mmean_plresp              ,lmask)
      opatch%mmean_leaf_energy     (1:z) = pack(ipatch%mmean_leaf_energy         ,lmask)
      opatch%mmean_leaf_water      (1:z) = pack(ipatch%mmean_leaf_water          ,lmask)
      opatch%mmean_leaf_hcap       (1:z) = pack(ipatch%mmean_leaf_hcap           ,lmask)
      opatch%mmean_leaf_vpdef      (1:z) = pack(ipatch%mmean_leaf_vpdef          ,lmask)
      opatch%mmean_leaf_temp       (1:z) = pack(ipatch%mmean_leaf_temp           ,lmask)
      opatch%mmean_leaf_fliq       (1:z) = pack(ipatch%mmean_leaf_fliq           ,lmask)
      opatch%mmean_leaf_gsw        (1:z) = pack(ipatch%mmean_leaf_gsw            ,lmask)
      opatch%mmean_leaf_gbw        (1:z) = pack(ipatch%mmean_leaf_gbw            ,lmask)
      opatch%mmean_wood_energy     (1:z) = pack(ipatch%mmean_wood_energy         ,lmask)
      opatch%mmean_wood_water      (1:z) = pack(ipatch%mmean_wood_water          ,lmask)
      opatch%mmean_wood_hcap       (1:z) = pack(ipatch%mmean_wood_hcap           ,lmask)
      opatch%mmean_wood_temp       (1:z) = pack(ipatch%mmean_wood_temp           ,lmask)
      opatch%mmean_wood_fliq       (1:z) = pack(ipatch%mmean_wood_fliq           ,lmask)
      opatch%mmean_wood_gbw        (1:z) = pack(ipatch%mmean_wood_gbw            ,lmask)
      opatch%mmean_fs_open         (1:z) = pack(ipatch%mmean_fs_open             ,lmask)
      opatch%mmean_fsw             (1:z) = pack(ipatch%mmean_fsw                 ,lmask)
      opatch%mmean_fsn             (1:z) = pack(ipatch%mmean_fsn                 ,lmask)
      opatch%mmean_a_open          (1:z) = pack(ipatch%mmean_a_open              ,lmask)
      opatch%mmean_a_closed        (1:z) = pack(ipatch%mmean_a_closed            ,lmask)
      opatch%mmean_a_net           (1:z) = pack(ipatch%mmean_a_net               ,lmask)
      opatch%mmean_A_light         (1:z) = pack(ipatch%mmean_A_light             ,lmask)
      opatch%mmean_A_rubp          (1:z) = pack(ipatch%mmean_A_rubp              ,lmask)
      opatch%mmean_A_co2           (1:z) = pack(ipatch%mmean_A_co2               ,lmask)
      opatch%mmean_psi_open        (1:z) = pack(ipatch%mmean_psi_open            ,lmask)
      opatch%mmean_psi_closed      (1:z) = pack(ipatch%mmean_psi_closed          ,lmask)
      opatch%mmean_water_supply    (1:z) = pack(ipatch%mmean_water_supply        ,lmask)
      opatch%mmean_light_level     (1:z) = pack(ipatch%mmean_light_level         ,lmask)
      opatch%mmean_light_level_beam(1:z) = pack(ipatch%mmean_light_level_beam    ,lmask)
      opatch%mmean_light_level_diff(1:z) = pack(ipatch%mmean_light_level_diff    ,lmask)

      opatch%mmean_par_level_beam  (1:z) = pack(ipatch%mmean_par_level_beam      ,lmask)
      opatch%mmean_par_level_diffu (1:z) = pack(ipatch%mmean_par_level_diffu     ,lmask)
      opatch%mmean_par_level_diffd (1:z) = pack(ipatch%mmean_par_level_diffd     ,lmask)

      opatch%mmean_par_l           (1:z) = pack(ipatch%mmean_par_l               ,lmask)
      opatch%mmean_par_l_beam      (1:z) = pack(ipatch%mmean_par_l_beam          ,lmask)
      opatch%mmean_par_l_diff      (1:z) = pack(ipatch%mmean_par_l_diff          ,lmask)
      opatch%mmean_rshort_l        (1:z) = pack(ipatch%mmean_rshort_l            ,lmask)
      opatch%mmean_rlong_l         (1:z) = pack(ipatch%mmean_rlong_l             ,lmask)
      opatch%mmean_sensible_lc     (1:z) = pack(ipatch%mmean_sensible_lc         ,lmask)
      opatch%mmean_vapor_lc        (1:z) = pack(ipatch%mmean_vapor_lc            ,lmask)
      opatch%mmean_transp          (1:z) = pack(ipatch%mmean_transp              ,lmask)
      opatch%mmean_intercepted_al  (1:z) = pack(ipatch%mmean_intercepted_al      ,lmask)
      opatch%mmean_wshed_lg        (1:z) = pack(ipatch%mmean_wshed_lg            ,lmask)
      opatch%mmean_rshort_w        (1:z) = pack(ipatch%mmean_rshort_w            ,lmask)
      opatch%mmean_rlong_w         (1:z) = pack(ipatch%mmean_rlong_w             ,lmask)
      opatch%mmean_sensible_wc     (1:z) = pack(ipatch%mmean_sensible_wc         ,lmask)
      opatch%mmean_vapor_wc        (1:z) = pack(ipatch%mmean_vapor_wc            ,lmask)
      opatch%mmean_intercepted_aw  (1:z) = pack(ipatch%mmean_intercepted_aw      ,lmask)
      opatch%mmean_wshed_wg        (1:z) = pack(ipatch%mmean_wshed_wg            ,lmask)
      opatch%mmean_nppleaf         (1:z) = pack(ipatch%mmean_nppleaf             ,lmask)
      opatch%mmean_nppfroot        (1:z) = pack(ipatch%mmean_nppfroot            ,lmask)
      opatch%mmean_nppsapwood      (1:z) = pack(ipatch%mmean_nppsapwood          ,lmask)
      opatch%mmean_nppcroot        (1:z) = pack(ipatch%mmean_nppcroot            ,lmask)
      opatch%mmean_nppseeds        (1:z) = pack(ipatch%mmean_nppseeds            ,lmask)
      opatch%mmean_nppwood         (1:z) = pack(ipatch%mmean_nppwood             ,lmask)
      opatch%mmean_nppdaily        (1:z) = pack(ipatch%mmean_nppdaily            ,lmask)
      opatch%mmsqu_gpp             (1:z) = pack(ipatch%mmsqu_gpp                 ,lmask)
      opatch%mmsqu_npp             (1:z) = pack(ipatch%mmsqu_npp                 ,lmask)
      opatch%mmsqu_plresp          (1:z) = pack(ipatch%mmsqu_plresp              ,lmask)
      opatch%mmsqu_sensible_lc     (1:z) = pack(ipatch%mmsqu_sensible_lc         ,lmask)
      opatch%mmsqu_vapor_lc        (1:z) = pack(ipatch%mmsqu_vapor_lc            ,lmask)
      opatch%mmsqu_transp          (1:z) = pack(ipatch%mmsqu_transp              ,lmask)
      opatch%mmsqu_sensible_wc     (1:z) = pack(ipatch%mmsqu_sensible_wc         ,lmask)
      opatch%mmsqu_vapor_wc        (1:z) = pack(ipatch%mmsqu_vapor_wc            ,lmask)
      !------------------------------------------------------------------------------------!


      !----- Mortality variables. ---------------------------------------------------------!
      do m=1,n_mort
         opatch%mmean_mort_rate(m,1:z) = pack(ipatch%mmean_mort_rate        (m,:),lmask)
      end do
      !------------------------------------------------------------------------------------!


      !------ Radiation profile variables. ------------------------------------------------!
      do m=1,n_radprof
         opatch%mmean_rad_profile(m,1:z) = pack(ipatch%mmean_rad_profile  (m,:),lmask)
      end do
      !------------------------------------------------------------------------------------!

      return
   end subroutine copy_patchtype_mask_mmean
   !=======================================================================================!
   !=======================================================================================!





   !=======================================================================================!
   !=======================================================================================!
   !     This sub-routine copies the cohort-level variables that are qmean or qmsqu.       !
   ! The only sub-routine that should ever call this one is copy_patchtype_mask.           !
   !---------------------------------------------------------------------------------------!
   subroutine copy_patchtype_mask_qmean(ipatch,opatch,z,lmask,isize,osize)
      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(patchtype)         , target     :: ipatch
      type(patchtype)         , target     :: opatch
      integer                 , intent(in) :: z
      integer                 , intent(in) :: isize
      integer                 , intent(in) :: osize
      logical,dimension(isize), intent(in) :: lmask
      !----- Local variables. -------------------------------------------------------------!
      integer                              :: i
      integer                              :: m
      integer                              :: n
      !------------------------------------------------------------------------------------!



      do n=1,ndcycle
         opatch%qmean_gpp           (n,1:z) = pack(ipatch%qmean_gpp           (n,:),lmask)
         opatch%qmean_npp           (n,1:z) = pack(ipatch%qmean_npp           (n,:),lmask)
         opatch%qmean_leaf_resp     (n,1:z) = pack(ipatch%qmean_leaf_resp     (n,:),lmask)
         opatch%qmean_root_resp     (n,1:z) = pack(ipatch%qmean_root_resp     (n,:),lmask)
         opatch%qmean_leaf_growth_resp(n,1:z) = pack(ipatch%qmean_leaf_growth_resp(n,:),lmask)
         opatch%qmean_root_growth_resp(n,1:z) = pack(ipatch%qmean_root_growth_resp(n,:),lmask)
         opatch%qmean_sapa_growth_resp(n,1:z) = pack(ipatch%qmean_sapa_growth_resp(n,:),lmask)
         opatch%qmean_sapb_growth_resp(n,1:z) = pack(ipatch%qmean_sapb_growth_resp(n,:),lmask)
         opatch%qmean_leaf_storage_resp(n,1:z) = pack(ipatch%qmean_leaf_storage_resp(n,:),lmask)
         opatch%qmean_root_storage_resp(n,1:z) = pack(ipatch%qmean_root_storage_resp(n,:),lmask)
         opatch%qmean_sapa_storage_resp(n,1:z) = pack(ipatch%qmean_sapa_storage_resp(n,:),lmask)
         opatch%qmean_sapb_storage_resp(n,1:z) = pack(ipatch%qmean_sapb_storage_resp(n,:),lmask)
         opatch%qmean_plresp        (n,1:z) = pack(ipatch%qmean_plresp        (n,:),lmask)
         opatch%qmean_leaf_energy   (n,1:z) = pack(ipatch%qmean_leaf_energy   (n,:),lmask)
         opatch%qmean_leaf_water    (n,1:z) = pack(ipatch%qmean_leaf_water    (n,:),lmask)
         opatch%qmean_leaf_hcap     (n,1:z) = pack(ipatch%qmean_leaf_hcap     (n,:),lmask)
         opatch%qmean_leaf_vpdef    (n,1:z) = pack(ipatch%qmean_leaf_vpdef    (n,:),lmask)
         opatch%qmean_leaf_temp     (n,1:z) = pack(ipatch%qmean_leaf_temp     (n,:),lmask)
         opatch%qmean_leaf_fliq     (n,1:z) = pack(ipatch%qmean_leaf_fliq     (n,:),lmask)
         opatch%qmean_leaf_gsw      (n,1:z) = pack(ipatch%qmean_leaf_gsw      (n,:),lmask)
         opatch%qmean_leaf_gbw      (n,1:z) = pack(ipatch%qmean_leaf_gbw      (n,:),lmask)
         opatch%qmean_wood_energy   (n,1:z) = pack(ipatch%qmean_wood_energy   (n,:),lmask)
         opatch%qmean_wood_water    (n,1:z) = pack(ipatch%qmean_wood_water    (n,:),lmask)
         opatch%qmean_wood_hcap     (n,1:z) = pack(ipatch%qmean_wood_hcap     (n,:),lmask)
         opatch%qmean_wood_temp     (n,1:z) = pack(ipatch%qmean_wood_temp     (n,:),lmask)
         opatch%qmean_wood_fliq     (n,1:z) = pack(ipatch%qmean_wood_fliq     (n,:),lmask)
         opatch%qmean_wood_gbw      (n,1:z) = pack(ipatch%qmean_wood_gbw      (n,:),lmask)
         opatch%qmean_fs_open       (n,1:z) = pack(ipatch%qmean_fs_open       (n,:),lmask)
         opatch%qmean_fsw           (n,1:z) = pack(ipatch%qmean_fsw           (n,:),lmask)
         opatch%qmean_fsn           (n,1:z) = pack(ipatch%qmean_fsn           (n,:),lmask)
         opatch%qmean_a_open        (n,1:z) = pack(ipatch%qmean_a_open        (n,:),lmask)
         opatch%qmean_a_closed      (n,1:z) = pack(ipatch%qmean_a_closed      (n,:),lmask)
         opatch%qmean_a_net         (n,1:z) = pack(ipatch%qmean_a_net         (n,:),lmask)
         opatch%qmean_A_light       (n,1:z) = pack(ipatch%qmean_A_light       (n,:),lmask)
         opatch%qmean_A_rubp        (n,1:z) = pack(ipatch%qmean_A_rubp        (n,:),lmask)
         opatch%qmean_A_co2         (n,1:z) = pack(ipatch%qmean_A_co2         (n,:),lmask)
         opatch%qmean_psi_open      (n,1:z) = pack(ipatch%qmean_psi_open      (n,:),lmask)
         opatch%qmean_psi_closed    (n,1:z) = pack(ipatch%qmean_psi_closed    (n,:),lmask)
         opatch%qmean_water_supply  (n,1:z) = pack(ipatch%qmean_water_supply  (n,:),lmask)
         opatch%qmean_light_level   (n,1:z) = pack(ipatch%qmean_light_level   (n,:),lmask)
         opatch%qmean_light_level_beam(n,1:z) =                                            &
                                            pack(ipatch%qmean_light_level_beam(n,:),lmask)
         opatch%qmean_light_level_diff(n,1:z) =                                            &
                                            pack(ipatch%qmean_light_level_diff(n,:),lmask)

         opatch%qmean_par_level_beam  (n,1:z) = pack(ipatch%qmean_par_level_beam(n,:)     ,lmask)
         opatch%qmean_par_level_diffu (n,1:z) = pack(ipatch%qmean_par_level_diffu(n,:)    ,lmask)
         opatch%qmean_par_level_diffd (n,1:z) = pack(ipatch%qmean_par_level_diffd(n,:)    ,lmask)

         opatch%qmean_par_l         (n,1:z) = pack(ipatch%qmean_par_l         (n,:),lmask)
         opatch%qmean_par_l_beam    (n,1:z) = pack(ipatch%qmean_par_l_beam    (n,:),lmask)
         opatch%qmean_par_l_diff    (n,1:z) = pack(ipatch%qmean_par_l_diff    (n,:),lmask)
         opatch%qmean_rshort_l      (n,1:z) = pack(ipatch%qmean_rshort_l      (n,:),lmask)
         opatch%qmean_rlong_l       (n,1:z) = pack(ipatch%qmean_rlong_l       (n,:),lmask)
         opatch%qmean_sensible_lc   (n,1:z) = pack(ipatch%qmean_sensible_lc   (n,:),lmask)
         opatch%qmean_vapor_lc      (n,1:z) = pack(ipatch%qmean_vapor_lc      (n,:),lmask)
         opatch%qmean_transp        (n,1:z) = pack(ipatch%qmean_transp        (n,:),lmask)
         opatch%qmean_intercepted_al(n,1:z) = pack(ipatch%qmean_intercepted_al(n,:),lmask)
         opatch%qmean_wshed_lg      (n,1:z) = pack(ipatch%qmean_wshed_lg      (n,:),lmask)
         opatch%qmean_rshort_w      (n,1:z) = pack(ipatch%qmean_rshort_w      (n,:),lmask)
         opatch%qmean_rlong_w       (n,1:z) = pack(ipatch%qmean_rlong_w       (n,:),lmask)
         opatch%qmean_sensible_wc   (n,1:z) = pack(ipatch%qmean_sensible_wc   (n,:),lmask)
         opatch%qmean_vapor_wc      (n,1:z) = pack(ipatch%qmean_vapor_wc      (n,:),lmask)
         opatch%qmean_intercepted_aw(n,1:z) = pack(ipatch%qmean_intercepted_aw(n,:),lmask)
         opatch%qmean_wshed_wg      (n,1:z) = pack(ipatch%qmean_wshed_wg      (n,:),lmask)
         opatch%qmsqu_gpp           (n,1:z) = pack(ipatch%qmsqu_gpp           (n,:),lmask)
         opatch%qmsqu_npp           (n,1:z) = pack(ipatch%qmsqu_npp           (n,:),lmask)
         opatch%qmsqu_plresp        (n,1:z) = pack(ipatch%qmsqu_plresp        (n,:),lmask)
         opatch%qmsqu_sensible_lc   (n,1:z) = pack(ipatch%qmsqu_sensible_lc   (n,:),lmask)
         opatch%qmsqu_vapor_lc      (n,1:z) = pack(ipatch%qmsqu_vapor_lc      (n,:),lmask)
         opatch%qmsqu_transp        (n,1:z) = pack(ipatch%qmsqu_transp        (n,:),lmask)
         opatch%qmsqu_sensible_wc   (n,1:z) = pack(ipatch%qmsqu_sensible_wc   (n,:),lmask)
         opatch%qmsqu_vapor_wc      (n,1:z) = pack(ipatch%qmsqu_vapor_wc      (n,:),lmask)



         !------ Radiation profile variables. ---------------------------------------------!
         do m=1,n_radprof
            opatch%qmean_rad_profile(m,n,1:z) = pack(ipatch%qmean_rad_profile(m,n,:),lmask)
         end do
         !---------------------------------------------------------------------------------!

      end do
      !------------------------------------------------------------------------------------!


      return
   end subroutine copy_patchtype_mask_qmean
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !      This subroutine is the main driver for filling the variable table (var_table) of !
   ! ED variables.  On a serial computing environment, this routine should be called near  !
   ! the end of the initialization process after the hierarchical tree structure has been  !
   ! trimmed via fusion/splitting/termination.  Similiarly, this routine should be called  !
   ! after any fusion/splitting/termination process, assuming that the major vtable        !
   ! structures have been deallocated prior to reallocation.                               !
   !                                                                                       !
   !      In a parallel environment, this routine should operate in a similiar fashion on  !
   ! each of the compute nodes.  It is designed such that the compute nodes will write     !
   ! hyperslabs of data in parallel to a joining HDF5 dataset as "collective-chunked"      !
   ! data.  The modifications that must be made after running this subroutine, are that    !
   ! the indexes should account for the offset of the current compute node.                !
   !                                                                                       !!
   !    The various state scalars, vectors and arrays are now populate the vtable.  The    !
   ! vtable indexes the array gives it a name, records its dimensions, when it is to be    !
   ! used as output and how (averaging and such) and most importantly saves a pointer to   !
   ! its starting position.  If this routine is being called as a compute node in          !
   ! parallel, the first position is not necessarily the first position of the whole       !
   ! datavector, but will only be the first position of that nodes hyperslab chunk         !
   ! within the given continuous dataset.                                                  !
   !                                                                                       !
   ! The first number correspond to the data level:                                        !
   !    1. Gridtype     (polygon level)                                                    !
   !    2. Polygontype  (site    level)                                                    !
   !    3. Sitetype     (patch   level)                                                    !
   !    4. Patchtype    (cohort  level)                                                    !
   !    9. Scalar                                                                          !
   !                                                                                       !
   ! The other numbers correspond to the kind of dimension and variable.                   !
   !    0. Main vector ordinate only, integer.                                             !
   !    1. Main vector ordinate only, real.                                                !
   !    2. Soil layer                                                                      !
   !    3. Surface water layer                                                             !
   !    4. PFT                                                                             !
   !    5. Disturbance Type                                                                !
   !    6. DBH class                                                                       !
   !    7. FF_DBH class                                                                    !
   !    8. Mortality                                                                       !
   !    9. Month/13 months                                                                 !
   !                                                                                       !
   ! The negative sign means that the variable has an extra dimension, for the diurnal     !
   !    cycle output.                                                                      !
   !                                                                                       !
   ! Of these possible dimensions (2-9), they may be used concurrently to partition the    !
   !    data into multi-dimensional spaces, but all seven will not be used simultaneously. !
   !    Currently the highest ranks in use is 3, and you should think twice before making  !
   !    new types with higher dimensionality: these arrays may become huge.                !
   !                                                                                       !
   ! Each unique combination will have a call number associated with it.                   !
   !                                                                                       !
   !  10    : rank 1 : polygon, integer                                                    !
   !  11    : rank 1 : polygon                                                             !
   ! -11    : rank 2 : polygon, diurnal cycle                                              !
   !  12    : rank 2 : polygon, s-layer                                                    !
   !  120   : rank 2 : polygon, s-layer, integer                                           !
   ! -12    : rank 3 : polygon, diurnal cycle, s-layer                                     !
   !  13    : rank 2 : polygon, w-layer                                                    !
   !  14    : rank 2 : polygon, pft                                                        !
   !  146   : rank 3 : polygon, pft, dbh                                                   !
   !  15    : rank 2 : polygon, disturbance                                                !
   !  155   : rank 3 : polygon, disturbance, disturbance                                   !
   !  157   : rank 3 : polygon, disturbance, age                                           !
   !  16    : rank 2 : polygon, dbh                                                        !
   !  17    : rank 2 : polygon, age                                                        !
   !  18    : rank 2 : polygon, mort                                                       !
   !  19    : rank 2 : polygon, month+1                                                    !
   !                                                                                       !
   !  20    : rank 1 : site, integer                                                       !
   !  21    : rank 1 : site                                                                !
   !  22    : rank 2 : site, s-layer                                                       !
   !  23    : rank 2 : site, w-layer                                                       !
   !  24    : rank 2 : site, pft                                                           !
   !  246   : rank 3 : site, pft, dbh                                                      !
   !  25    : rank 2 : site, disturbance                                                   !
   !  255   : rank 3 : site, disturbance, disturbance                                      !
   !  26    : rank 2 : site, dbh                                                           !
   !  27    : rank 2 : site, age                                                           !
   !  28    : rank 2 : site, mort                                                          !
   !  29    : rank 2 : site, month                                                         !
   !                                                                                       !
   !  30    : rank 1 : patch, integer                                                      !
   !  31    : rank 1 : patch                                                               !
   ! -31    : rank 2 : patch, diurnal cycle                                                !
   !  32    : rank 2 : patch, s-layer                                                      !
   !  33    : rank 2 : patch, w-layer                                                      !
   !  34    : rank 2 : patch, pft                                                          !
   !  346   : rank 3 : patch, pft, ff_dbh                                                  !
   !  35    : rank 2 : patch, disturbance                                                  !
   !  36    : rank 2 : patch, dbh                                                          !
   !  37    : rank 2 : patch, age                                                          !
   !  38    : rank 2 : patch, mort                                                         !
   !                                                                                       !
   !  40    : rank 1 : cohort, integer                                                     !
   !  41    : rank 1 : cohort                                                              !
   ! -41    : rank 2 : cohort, diurnal cycle                                               !
   !  44    : rank 2 : cohort, pft                                                         !
   !  46    : rank 2 : cohort, dbh                                                         !
   !  47    : rank 2 : cohort, age                                                         !
   !  48    : rank 2 : cohort, mort                                                        !
   !  49    : rank 2 : cohort, month+1                                                     !
   !  411   : rank 2 : cohort, radiation                                                   !
   ! -411   : rank 2 : cohort, diurnal cycle , radiation                                   !
   !  90    : rank 0 : integer scalar                                                      !
   !  92    : rank 1 : s-layer                                                             !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_alltypes

      use ed_var_tables, only : num_var                  & ! intent(in)
                              , vt_info                  & ! intent(inout)
                              , var_table                & ! intent(inout)
                              , reset_vt_vector_pointers ! ! sub-routine
      use ed_node_coms , only : mynum                    & ! intent(in)
                              , mchnum                   & ! intent(in)
                              , machs                    & ! intent(in)
                              , nmachs                   & ! intent(in)
                              , nnodetot                 & ! intent(in)
                              , sendnum                  & ! intent(in)
                              , recvnum                  & ! intent(in)
                              , master_num               ! ! intent(in)
      use ed_max_dims  , only : maxgrds                  & ! intent(in)
                              , maxmach                  ! ! intent(in)
      implicit none

#if defined(RAMS_MPI)
      include 'mpif.h'
#endif
      !----- Local variables. -------------------------------------------------------------!
      type(edtype)                                 , pointer   :: cgrid
      type(polygontype)                            , pointer   :: cpoly
      type(sitetype)                               , pointer   :: csite
      type(patchtype)                              , pointer   :: cpatch
      integer                                                  :: ncohorts_g
      integer                                                  :: npatches_g
      integer                                                  :: nsites_g
      integer                                                  :: igr
      integer                                                  :: ipy
      integer                                                  :: isi
      integer                                                  :: ipa
      integer                                                  :: nv
      integer                                                  :: ierr
      integer                                                  :: nm
      integer                                                  :: iptr
#if defined(RAMS_MPI)
      integer          , dimension(MPI_STATUS_SIZE)            :: status
#endif
      integer                                                  :: ping
      integer                                                  :: uniqueid
      !----- Local constants. -------------------------------------------------------------!
      logical                                      , parameter :: verbose = .false.    
      !----- Local saved variables. -------------------------------------------------------!
      logical                                      , save      :: model_start = .true.
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      if (mynum == 1 .and. verbose) then
         write(unit=*,fmt='(a)') '--- Re-hashing the IO pointer tables and mapping arrays'
      end if
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !      The first loop through populates the info tables.                             !
      !------------------------------------------------------------------------------------!
      do igr = 1,ngrids
         cgrid => edgrid_g(igr)
         if (num_var(igr) > 0) then
            do nv=1,num_var(igr)
               call reset_vt_vector_pointers(vt_info(nv,igr))
            end do
         end if

         num_var(igr)           = 0

         cgrid%npolygons_global = cgrid%npolygons
         cgrid%nsites_global    = get_nsites(cgrid)
         cgrid%npatches_global  = get_npatches(cgrid)
         cgrid%ncohorts_global  = get_ncohorts(cgrid)
         
         cgrid%mach_cohort_offset_index = 0
         cgrid%mach_patch_offset_index  = 0
         cgrid%mach_site_offset_index   = 0
         cgrid%mach_polygon_offset_index= 0

         if (nnodetot /= 1) then

            !----- Send all them sizes to root (CHANGED, NODE 1). -------------------------!
            if (mynum == 1) then
            
               gdpy(1,igr) = cgrid%npolygons_global
               gdsi(1,igr) = cgrid%nsites_global
               gdpa(1,igr) = cgrid%npatches_global
               gdco(1,igr) = cgrid%ncohorts_global

#if defined(RAMS_MPI)
               call MPI_Send(ping,1,MPI_INTEGER,sendnum,94,MPI_COMM_WORLD,ierr)
               
               !----- Have node 1 recieve the info. ---------------------------------------!
               do nm=2,nnodetot
                  uniqueid = ((igr-1)*maxmach)+nm
                  call MPI_Recv(gdpy(nm,igr),1,MPI_INTEGER,machs(nm)                       &
                               ,500000+uniqueid,MPI_COMM_WORLD,status,ierr)
                  call MPI_Recv(gdsi(nm,igr),1,MPI_INTEGER,machs(nm)                       &
                               ,600000+uniqueid,MPI_COMM_WORLD,status,ierr)
                  call MPI_Recv(gdpa(nm,igr),1,MPI_INTEGER,machs(nm)                       &
                               ,700000+uniqueid,MPI_COMM_WORLD,status,ierr)
                  call MPI_Recv(gdco(nm,igr),1,MPI_INTEGER,machs(nm)                       &
                               ,800000+uniqueid,MPI_COMM_WORLD,status,ierr)
               end do
               !---------------------------------------------------------------------------!



               !----- Broadcast all this info to the nodes. -------------------------------!
               do nm=2,nnodetot
                  uniqueid=((igr-1)*maxmach)+nm
                  call MPI_Send(gdpy,maxmach*maxgrds,MPI_INTEGER,machs(nm)                 &
                               , 900000+uniqueid,MPI_COMM_WORLD,ierr)
                  call MPI_Send(gdsi,maxmach*maxgrds,MPI_INTEGER,machs(nm)                 &
                               ,1000000+uniqueid,MPI_COMM_WORLD,ierr)
                  call MPI_Send(gdpa,maxmach*maxgrds,MPI_INTEGER,machs(nm)                 &
                               ,1100000+uniqueid,MPI_COMM_WORLD,ierr)
                  call MPI_Send(gdco,maxmach*maxgrds,MPI_INTEGER,machs(nm)                 &
                               ,1200000+uniqueid,MPI_COMM_WORLD,ierr)
               end do
               !---------------------------------------------------------------------------!
#endif
            else

#if defined(RAMS_MPI)
               !----- Set the blocking receive to allow ordering, start with machine 1. ---!
               call MPI_Recv(ping,1,MPI_INTEGER,recvnum,94,MPI_COMM_WORLD,status,ierr)
               !---------------------------------------------------------------------------!

               uniqueid=((igr-1)*maxmach)+mynum
               !----- Send the information to node (1). -----------------------------------!
               call MPI_Send(cgrid%npolygons_global, 1,MPI_INTEGER,machs(1)                &
                            ,500000+uniqueid,MPI_COMM_WORLD,ierr)
               call MPI_Send(cgrid%nsites_global   , 1,MPI_INTEGER,machs(1)                &
                            ,600000+uniqueid,MPI_COMM_WORLD,ierr)
               call MPI_Send(cgrid%npatches_global , 1,MPI_INTEGER,machs(1)                &
                            ,700000+uniqueid,MPI_COMM_WORLD,ierr)
               call MPI_Send(cgrid%ncohorts_global , 1,MPI_INTEGER,machs(1)                &
                            ,800000+uniqueid,MPI_COMM_WORLD,ierr)

               !---------------------------------------------------------------------------!
               !     When this node is finished, send the blocking MPI_Send to the next    !
               ! machine.                                                                  !
               !---------------------------------------------------------------------------!
               if (mynum /= nnodetot) call MPI_Send(ping,1,MPI_INTEGER,sendnum             &
                                                   ,94,MPI_COMM_WORLD,ierr)
               !---------------------------------------------------------------------------!


               !------ Receive the data from the main node. -------------------------------!
               uniqueid=((igr-1)*maxmach)+mynum
               call MPI_Recv(gdpy,maxmach*maxgrds,MPI_INTEGER,machs(1)                     &
                            , 900000+uniqueid,MPI_COMM_WORLD,status,ierr)
               call MPI_Recv(gdsi,maxmach*maxgrds,MPI_INTEGER,machs(1)                     &
                            ,1000000+uniqueid,MPI_COMM_WORLD,status,ierr)
               call MPI_Recv(gdpa,maxmach*maxgrds,MPI_INTEGER,machs(1)                     &
                            ,1100000+uniqueid,MPI_COMM_WORLD,status,ierr)
               call MPI_Recv(gdco,maxmach*maxgrds,MPI_INTEGER,machs(1)                     &
                            ,1200000+uniqueid,MPI_COMM_WORLD,status,ierr)
               !---------------------------------------------------------------------------!
#else
               continue
#endif
            end if
            !------------------------------------------------------------------------------!


            !------------------------------------------------------------------------------!
            !     Display the information.                                                 !
            !------------------------------------------------------------------------------!
            if (mynum == 1 .and. model_start .and. verbose) then
               
               print*,"Global Polygons: ",gdpy(1:nnodetot,igr)
               print*,"Global Site: "    ,gdsi(1:nnodetot,igr)
               print*,"Global Patches: " ,gdpa(1:nnodetot,igr)
               print*,"Global Cohorts: " ,gdco(1:nnodetot,igr)
            end if
            !------------------------------------------------------------------------------!




            !----- Calculate the offsets that each machine has. ---------------------------!
            py_off(1,igr) = 0
            si_off(1,igr) = 0
            pa_off(1,igr) = 0
            co_off(1,igr) = 0
            do nm=2,nnodetot
               py_off(nm,igr) = py_off(nm-1,igr) + gdpy(nm-1,igr)
               si_off(nm,igr) = si_off(nm-1,igr) + gdsi(nm-1,igr)
               pa_off(nm,igr) = pa_off(nm-1,igr) + gdpa(nm-1,igr)
               co_off(nm,igr) = co_off(nm-1,igr) + gdco(nm-1,igr)
            end do
            !------------------------------------------------------------------------------!


            !----- Calculate the total sizes of the arrays. -------------------------------!
            cgrid%npolygons_global = sum(gdpy(1:nnodetot,igr))
            cgrid%nsites_global    = sum(gdsi(1:nnodetot,igr))
            cgrid%npatches_global  = sum(gdpa(1:nnodetot,igr))
            cgrid%ncohorts_global  = sum(gdco(1:nnodetot,igr))
            !------------------------------------------------------------------------------!



            !----- Calculate the local offsets. -------------------------------------------!
            cgrid%mach_polygon_offset_index = py_off(mynum,igr)
            cgrid%mach_site_offset_index    = si_off(mynum,igr)
            cgrid%mach_patch_offset_index   = pa_off(mynum,igr)
            cgrid%mach_cohort_offset_index  = co_off(mynum,igr)
            !------------------------------------------------------------------------------!
         end if
         !---------------------------------------------------------------------------------!




         !---------------------------------------------------------------------------------!
         !      Call the routines that will fill the variable table for the first time.    !
         !---------------------------------------------------------------------------------!
         call filltab_globtype(igr)
         call filltab_edtype(igr,0)
         if (gdpy(mynum,igr) > 0) then
            call filltab_polygontype(igr,1,0)
            call filltab_sitetype   (igr,1,1,0)
            call filltab_patchtype  (igr,1,1,1,0)
         end if
         !---------------------------------------------------------------------------------!
      end do
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      !     Loop over all grids again.                                                     !
      !------------------------------------------------------------------------------------!
      do igr = 1,ngrids
         !---------------------------------------------------------------------------------!
         !    Test to see if the var_table has been initialized.  If it has, then          !
         ! de-allocate its pointers and reset its first flag.  These will be reallocated   !
         ! on the first pass of the filltab_ subroutines.                                  !
         !---------------------------------------------------------------------------------!
         cgrid => edgrid_g(igr)

         !----- Find the global index. ----------------------------------------------------!
         cgrid%pyglob_id = 0 + cgrid%mach_polygon_offset_index
         !---------------------------------------------------------------------------------!


         !---------------------------------------------------------------------------------!
         !      Determine the total number of variables for each grid.  These will         !
         ! determine the length of the vt_vector.                                          !
         !---------------------------------------------------------------------------------!
         call filltab_edtype(igr,1)
         !---------------------------------------------------------------------------------!

         ncohorts_g = 0 + cgrid%mach_cohort_offset_index
         npatches_g = 0 + cgrid%mach_patch_offset_index
         nsites_g   = 0 + cgrid%mach_site_offset_index


         do ipy = 1,cgrid%npolygons
            cpoly => cgrid%polygon(ipy)
            !----- This is the offset for the vtable write. -------------------------------!
            cpoly%siglob_id = nsites_g + 0
            !------------------------------------------------------------------------------!

            !----- This is the index written in the file for the user to reference. -------!
            cgrid%pysi_id(ipy) = nsites_g + 1 ! 
            !------------------------------------------------------------------------------!


            !----- This is the dimension of the sites that belong to this polygon. --------!
            cgrid%pysi_n(ipy) = cpoly%nsites
            !------------------------------------------------------------------------------!

            nsites_g = nsites_g + cpoly%nsites
            call filltab_polygontype(igr,ipy,1)

            do isi = 1,cpoly%nsites
               csite => cpoly%site(isi)

               csite%paglob_id    = npatches_g + 0
               cpoly%sipa_id(isi) = npatches_g + 1
               cpoly%sipa_n(isi)  = csite%npatches
               npatches_g         = npatches_g + csite%npatches
               call filltab_sitetype(igr,ipy,isi,1)

               do ipa = 1,csite%npatches
                  cpatch => csite%patch(ipa)

                  cpatch%coglob_id   = ncohorts_g + 0
                  csite%paco_id(ipa) = ncohorts_g + 1
                  csite%paco_n(ipa)  = cpatch%ncohorts
                  ncohorts_g         = ncohorts_g + cpatch%ncohorts

                  if (cpatch%ncohorts > 0) then
                     call filltab_patchtype(igr,ipy,isi,ipa,1)
                  end if
               end do
            end do
         end do
         if (mynum == 1 .and. verbose) then
            write(*,"(a)")'--- Mapping Completed'
         end if

         if (mynum == 1 .and. model_start .and. verbose) then
            model_start = .false.
            do nv=1,num_var(igr)
               write(unit=*,fmt='(a,i4,a,i4,a,a)') 'Registering: ',nv,' of',num_var(igr)   &
                                                             ,'  ',vt_info(nv,igr)%name
            end do
         end if 
      end do


      return
   end subroutine filltab_alltypes
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   subroutine filltab_globtype(igr)

      use ed_var_tables, only : vtable_edio_r       & ! sub-routine
                              , vtable_edio_r_sca   & ! sub-rouitne
                              , vtable_edio_i_sca   ! ! sub-rouitne
      use soil_coms    , only : slz                 & ! intent(in)
                              , slxclay             & ! intent(in)
                              , slxsand             & ! intent(in)
                              , isoilflg            ! ! intent(in)


      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      integer     , intent(in) :: igr
      !----- Local variables. -------------------------------------------------------------!
      integer                  :: var_len
      integer                  :: max_ptrs
      integer                  :: var_len_global
      integer                  :: nvar
      type(edtype), pointer    :: cgrid
      !------------------------------------------------------------------------------------!


      cgrid => edgrid_g(igr)


      !------------------------------------------------------------------------------------!
      !     Single values (scalars).                                                       !
      !------------------------------------------------------------------------------------!
      var_len        = 1
      var_len_global = 1
      max_ptrs       = 1

      nvar=1
      call vtable_edio_i_sca(cgrid%npolygons_global,nvar,igr,0,0                           &
                            ,var_len,var_len_global,max_ptrs                               &
                            ,'NPOLYGONS_GLOBAL :90:hist:anal:dail:mont:dcyc:year')
      call vtable_edio_i_sca(cgrid%npolygons_global,nvar,igr,1,0                           &
                            ,var_len,var_len_global,max_ptrs                               &
                            ,'NPOLYGONS_GLOBAL :90:hist:anal:dail:mont:dcyc:year')
      
      nvar=nvar+1
      call vtable_edio_i_sca(cgrid%nsites_global,nvar,igr,0,0                              &
                            ,var_len,var_len_global,max_ptrs                               &
                            ,'NSITES_GLOBAL :90:hist:anal:dail:mont:dcyc:year')
      call vtable_edio_i_sca(cgrid%nsites_global,nvar,igr,1,0                              &
                            ,var_len,var_len_global,max_ptrs                               &
                            ,'NSITES_GLOBAL :90:hist:anal:dail:mont:dcyc:year')
      
      nvar=nvar+1
      call vtable_edio_i_sca(cgrid%npatches_global,nvar,igr,0,0                            &
                            ,var_len,var_len_global,max_ptrs                               &
                            ,'NPATCHES_GLOBAL :90:hist:anal:dail:mont:dcyc:year')
      call vtable_edio_i_sca(cgrid%npatches_global,nvar,igr,1,0                            &
                            ,var_len,var_len_global,max_ptrs                               &
                            ,'NPATCHES_GLOBAL :90:hist:anal:dail:mont:dcyc:year')

      nvar=nvar+1
      call vtable_edio_i_sca(cgrid%ncohorts_global,nvar,igr,0,0                            &
                            ,var_len,var_len_global,max_ptrs                               &
                            ,'NCOHORTS_GLOBAL :90:hist:anal:dail:mont:dcyc:year')
      call vtable_edio_i_sca(cgrid%ncohorts_global,nvar,igr,1,0                            &
                            ,var_len,var_len_global,max_ptrs                               &
                            ,'NCOHORTS_GLOBAL :90:hist:anal:dail:mont:dcyc:year')

      nvar=nvar+1
      call vtable_edio_i_sca(nzg,nvar,igr,0,0                                              &
                            ,var_len,var_len_global,max_ptrs                               &
                            ,'NZG :90:hist:anal:dail:mont:dcyc:year')
      call vtable_edio_i_sca(nzg,nvar,igr,1,0                                              &
                            ,var_len,var_len_global,max_ptrs                               &
                            ,'NZG :90:hist:anal:dail:mont:dcyc:year')

      nvar=nvar+1
      call vtable_edio_i_sca(nzs,nvar,igr,0,0                                              &
                            ,var_len,var_len_global,max_ptrs                               &
                            ,'NZS :90:hist:anal:dail:mont:dcyc:year')
      call vtable_edio_i_sca(nzs,nvar,igr,1,0                                              &
                            ,var_len,var_len_global,max_ptrs                               &
                            ,'NZS :90:hist:anal:dail:mont:dcyc:year')

      nvar=nvar+1
      call vtable_edio_i_sca(ff_nhgt,nvar,igr,0,0                                              &
                            ,var_len,var_len_global,max_ptrs                               &
                            ,'FF_NHGT :90:hist:anal:dail:mont:dcyc:year')
      call vtable_edio_i_sca(ff_nhgt,nvar,igr,1,0                                          &
                            ,var_len,var_len_global,max_ptrs                               &
                            ,'FF_NHGT :90:hist:anal:dail:mont:dcyc:year')

      nvar=nvar+1
      call vtable_edio_i_sca(ndcycle,nvar,igr,0,0                                              &
                            ,var_len,var_len_global,max_ptrs                               &
                            ,'NDCYCLE :90:hist:anal:dail:mont:dcyc:year')
      call vtable_edio_i_sca(ndcycle,nvar,igr,1,0                                              &
                            ,var_len,var_len_global,max_ptrs                               &
                            ,'NDCYCLE :90:hist:anal:dail:mont:dcyc:year')

      nvar=nvar+1
      call vtable_edio_i_sca(isoilflg(igr),nvar,igr,0,0                                    &
                            ,var_len,var_len_global,max_ptrs                               &
                            ,'ISOILFLG :90:hist:anal:dail:mont:dcyc:year')
      call vtable_edio_i_sca(isoilflg(igr),nvar,igr,1,0                                    &
                            ,var_len,var_len_global,max_ptrs                               &
                            ,'ISOILFLG :90:hist:anal:dail:mont:dcyc:year')

      nvar=nvar+1
      call vtable_edio_r_sca(slxsand,nvar,igr,0,0                                          &
                            ,var_len,var_len_global,max_ptrs                               &
                            ,'SLXSAND :90:hist:anal:dail:mont:dcyc:year')
      call vtable_edio_r_sca(slxsand,nvar,igr,1,0                                          &
                            ,var_len,var_len_global,max_ptrs                               &
                            ,'SLXSAND :90:hist:anal:dail:mont:dcyc:year')

      nvar=nvar+1
      call vtable_edio_r_sca(slxclay,nvar,igr,0,0                                          &
                            ,var_len,var_len_global,max_ptrs                               &
                            ,'SLXCLAY :90:hist:anal:dail:mont:dcyc:year')
      call vtable_edio_r_sca(slxclay,nvar,igr,1,0                                          &
                            ,var_len,var_len_global,max_ptrs                               &
                            ,'SLXCLAY :90:hist:anal:dail:mont:dcyc:year')


      !------------------------------------------------------------------------------------!
      !    1-D variables, soil layers.                                                     !
      !------------------------------------------------------------------------------------!
      var_len        = nzg
      var_len_global = nzg



      nvar=nvar+1
      call vtable_edio_r(nzg,slz,nvar,igr,0,0                                              &
                        ,var_len,var_len_global,max_ptrs                                   &
                        ,'SLZ :92:hist:anal:dail:mont:dcyc:year')
      call vtable_edio_r(nzg,slz,nvar,igr,1,0                                              &
                        ,var_len,var_len_global,max_ptrs                                   &
                        ,'SLZ :92:hist:anal:dail:mont:dcyc:year')
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      !    1-D variables, height classes.                                                  !
      !------------------------------------------------------------------------------------!
      var_len        = ff_nhgt
      var_len_global = ff_nhgt



      nvar=nvar+1
      call vtable_edio_r(ff_nhgt,hgt_class,nvar,igr,0,0                                    &
                        ,var_len,var_len_global,max_ptrs                                   &
                        ,'HGT_CLASS :96:hist:anal:dail:mont:dcyc:year')
      call vtable_edio_r(ff_nhgt,hgt_class,nvar,igr,1,0                                    &
                        ,var_len,var_len_global,max_ptrs                                   &
                        ,'HGT_CLASS :96:hist:anal:dail:mont:dcyc:year')
      !------------------------------------------------------------------------------------!


   


      !----- Save the number of global-level variables that go to the output. -------------!
      nioglobal=nvar
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_globtype
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the polygon-level variables         !
   ! (edtype).                                                                             !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_edtype(igr,init)

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      integer     , intent(in) :: init
      integer     , intent(in) :: igr
      !----- Local variables. -------------------------------------------------------------!
      type(edtype), pointer    :: cgrid
      integer                  :: var_len
      integer                  :: max_ptrs
      integer                  :: var_len_global
      integer                  :: nvar
      integer                  :: npts
      !------------------------------------------------------------------------------------!

      cgrid => edgrid_g(igr)

      !------ Define the global dimensions. -----------------------------------------------!
      var_len        = cgrid%npolygons
      var_len_global = cgrid%npolygons_global
      max_ptrs       = 1

      !------ Continue the counting. ------------------------------------------------------!
      nvar = nioglobal

      call filltab_edtype_p10     (cgrid,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_edtype_p11inst (cgrid,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_edtype_p11fmean(cgrid,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_edtype_p11dmean(cgrid,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_edtype_p11mmean(cgrid,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_edtype_m11     (cgrid,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_edtype_p12     (cgrid,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_edtype_m12     (cgrid,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_edtype_p19     (cgrid,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_edtype_p146    (cgrid,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_edtype_p199    (cgrid,igr,init,var_len,var_len_global,max_ptrs,nvar)
      !----- Save the number of polygon-level (edtype) variables that go to the output. ---!
      if (init == 0) niogrid=nvar-nioglobal
      !------------------------------------------------------------------------------------!

      return
     
   end subroutine filltab_edtype
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the polygon-level variables         !
   ! (edtype) that have one dimension and are integer (type 10).                           !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_edtype_p10(cgrid,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_i  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(edtype), target        :: cgrid
      integer     , intent(in)    :: init
      integer     , intent(in)    :: igr
      integer     , intent(in)    :: var_len
      integer     , intent(in)    :: max_ptrs
      integer     , intent(in)    :: var_len_global
      integer     , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                     :: npts
      !------------------------------------------------------------------------------------!
      
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !     This is the 1-D block.  All variables must have the number of points defined   !
      ! by npts.                                                                           !
      !------------------------------------------------------------------------------------!

      npts = cgrid%npolygons

      if (associated(cgrid%pysi_id)) then
         nvar = nvar + 1
         call vtable_edio_i(npts,cgrid%pysi_id,nvar,igr,init,cgrid%pyglob_id               &
                           ,var_len,var_len_global,max_ptrs                                &
                           ,'PYSI_ID :10:hist:anal:dail:mont:dcyc:year')
         call metadata_edio(nvar,igr,'Polygons first site indices','[--]','(ipoly)')
      end if

      
      if (associated(cgrid%pysi_n)) then
         nvar=nvar+1
         call vtable_edio_i(npts,cgrid%pysi_n,nvar,igr,init,cgrid%pyglob_id                &
                           ,var_len,var_len_global,max_ptrs                                &
                           ,'PYSI_N :10:hist:anal:dail:mont:dcyc:year')
         call metadata_edio(nvar,igr,'Number of sites per polygon','[--]','(ipoly)')
      end if

      if (associated(cgrid%xatm)) then
         nvar=nvar+1
         call vtable_edio_i(npts,cgrid%xatm,nvar,igr,init,cgrid%pyglob_id                  &
                           ,var_len,var_len_global,max_ptrs                                &
                           ,'XATM :10:hist:anal:dail:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'Atm. cell x-indices of polygon','NA','ipoly')
      end if
      
      if (associated(cgrid%yatm)) then
         nvar=nvar+1
         call vtable_edio_i(npts,cgrid%yatm,nvar,igr,init,cgrid%pyglob_id                  &
                           ,var_len,var_len_global,max_ptrs                                &
                           ,'YATM :10:hist:anal:dail:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'Atm cell y-indices of polygon','NA','ipoly')
      end if
      
      if (associated(cgrid%load_adjacency)) then
         nvar=nvar+1
         call vtable_edio_i(npts,cgrid%load_adjacency,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'LOAD_ADJACENCY :10:hist') 
         call metadata_edio(nvar,igr,'Load Adjacency','[NA]','ipoly')
      end if
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_edtype_p10
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the polygon-level variables         !
   ! (edtype) that have one dimension and are real (type 11), and are instantaneous.       !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_edtype_p11inst(cgrid,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(edtype), target        :: cgrid
      integer     , intent(in)    :: init
      integer     , intent(in)    :: igr
      integer     , intent(in)    :: var_len
      integer     , intent(in)    :: max_ptrs
      integer     , intent(in)    :: var_len_global
      integer     , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                     :: npts
      !------------------------------------------------------------------------------------!


      
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !     This is the 1-D block.  All variables must have the number of points defined   !
      ! by npts.                                                                           !
      !------------------------------------------------------------------------------------!
      npts = cgrid%npolygons

      if (associated(cgrid%lon)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%lon,nvar,igr,init,cgrid%pyglob_id                   &
                           ,var_len,var_len_global,max_ptrs                                &
                           ,'LONGITUDE :11:hist:anal:dail:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'Longitude of Polygon','[deg]','(ipoly)')
      end if

      if (associated(cgrid%lat)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%lat,nvar,igr,init,cgrid%pyglob_id                   &
                           ,var_len,var_len_global,max_ptrs                                &
                           ,'LATITUDE :11:hist:anal:dail:mont:dcyc:year')
         call metadata_edio(nvar,igr,'Latitude of Polygon','[deg]','(ipoly)')
      end if

      
      if (associated(cgrid%wbar)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%wbar,nvar,igr,init,cgrid%pyglob_id                  &
                           ,var_len,var_len_global,max_ptrs                                &
                           ,'WBAR :11:hist') 
         call metadata_edio(nvar,igr                                                       &
                           ,'Polygon average topographic moisture index'                   &
                           ,'NA','(ipoly)')
      end if
      
      if (associated(cgrid%Te)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%Te                                                  &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'TE :11:hist') 
         call metadata_edio(nvar,igr,'No metadata available','NA','(ipoly)')
      end if
      
      if (associated(cgrid%zbar)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%zbar                                                &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'ZBAR :11:hist:anal:opti') 
         call metadata_edio(nvar,igr,'Polygon average water table depth','[m]','(ipoly)')
      end if
      
      if (associated(cgrid%sheat)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%sheat                                               &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'SHEAT :11:hist') 
         call metadata_edio(nvar,igr,'Soil heat pool for lateral hydrology','NA','(ipoly)')
      end if
      
      if (associated(cgrid%baseflow)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%baseflow                                            &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'BASEFLOW :11:hist:anal:opti') 
         call metadata_edio(nvar,igr                                                       &
                           ,'Loss of water from site to watershed discharge'               &
                           ,'[kg/m2/s]','(ipoly)')
      end if
      
      if (associated(cgrid%runoff)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%runoff                                              &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'RUNOFF :11:hist:anal:opti') 
         call metadata_edio(nvar,igr,'NA','NA','ipoly')
      end if
      
      if (associated(cgrid%qrunoff)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%qrunoff                                             &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QRUNOFF :11:hist:anal:opti') 
         call metadata_edio(nvar,igr,'NA','NA','ipoly')
      end if
      
      if (associated(cgrid%swliq)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%swliq,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'SWLIQ :11:hist:anal') 
         call metadata_edio(nvar,igr,'NA','NA','ipoly')
      end if

      if (associated(cgrid%total_agb)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%total_agb,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'TOTAL_AGB :11:hist:anal:year:opti') 
         call metadata_edio(nvar,igr,'Polygon Total Above Ground Biomass','[kgC/m2]','ipoly')
      end if
      
      if (associated(cgrid%total_basal_area)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%total_basal_area,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'TOTAL_BASAL_AREA :11:hist:anal:year') 
         call metadata_edio(nvar,igr,'Polygon Total Basal Area','[cm2/m2]','ipoly')
         
      end if

      if (associated(cgrid%total_agb_growth)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%total_agb_growth,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'TOTAL_AGB_GROWTH:11:hist:anal:year') 
          call metadata_edio(nvar,igr,'Polygon AGB gain through growth','[kgC/m2/yr]','ipoly')
      end if
      
      if (associated(cgrid%total_agb_mort)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%total_agb_mort,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'TOTAL_AGB_MORT :11:hist:anal:year') 
         call metadata_edio(nvar,igr,'Polygon AGB lost due to mortality','[kgC/m2/yr]','ipoly')
      end if
      
      if (associated(cgrid%total_agb_recruit)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%total_agb_recruit,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'TOTAL_AGB_RECRUIT :11:hist:anal:year') 
         call metadata_edio(nvar,igr,'Polygon AGB used to generate recruits','[kgC/m2/yr]','ipoly')
      end if
      
      if (associated(cgrid%total_basal_area_growth)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%total_basal_area_growth,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'TOTAL_BASAL_AREA_GROWTH :11:hist:anal:year') 
         call metadata_edio(nvar,igr,'Polygon basal area gained through growth ','[kgC/m2/yr]','ipoly')
      end if
      
      if (associated(cgrid%total_basal_area_mort)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%total_basal_area_mort,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'TOTAL_BASAL_AREA_MORT :11:hist:anal:year') 
         call metadata_edio(nvar,igr,'Polygon basal area lost through growth ','[cm2/m2/yr]','ipoly')
      end if
      
      if (associated(cgrid%total_basal_area_recruit)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%total_basal_area_recruit,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'TOTAL_BASAL_AREA_RECRUIT :11:hist:anal:year') 
         call metadata_edio(nvar,igr,'Polygon basal area gained by recruits','[cm2/m2/yr]','ipoly')
      end if
      
      if (associated(cgrid%cosz)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%cosz,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'COSZ :11:hist') 
         call metadata_edio(nvar,igr,'Cosine of the zenith angle','[a/h]','ipoly')
      end if
      
      if (associated(cgrid%cbudget_initialstorage)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%cbudget_initialstorage,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'CBUDGET_INITIALSTORAGE :11:hist') 
         call metadata_edio(nvar,igr,'Vegetation and soil carbon,at start of budget-averaging','[kgC/m2]','ipoly')
      end if
          
      if (associated(cgrid%cbudget_nep)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%cbudget_nep,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'CBUDGET_NEP :11:hist') 
         call metadata_edio(nvar,igr,'Polygon average net ecosystem production','[kgC/m2/day]','ipoly')
      end if
      
      if (associated(cgrid%nbudget_initialstorage)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%nbudget_initialstorage,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'NBUDGET_INITIALSTORAGE :11:hist') 
         call metadata_edio(nvar,igr,'Veg and soil nitrogen, at start of budget-averaging','[kgN/m2]','ipoly')
      end if

       !-------- TOTAL CARBON AND NITROGEN POOLS  ---------------
       ! Added by MCD for NCEAS/FACE intercomparison (Apr 7 2009)
       ! if (associated(cgrid%Cleaf)) then
       !    nvar=nvar+1
       !    call vtable_edio_r(npts,cgrid%Cleaf,nvar,igr,init,cgrid%pyglob_id, &
       !         var_len,var_len_global,max_ptrs,'CLEAF :11:dail') 
       !    call metadata_edio(nvar,igr,'Poly Avg. Leaf Carbon','?','ipoly') 
       ! end if
       ! if (associated(cgrid%Croot)) then
       !    nvar=nvar+1
       !    call vtable_edio_r(npts,cgrid%Croot,nvar,igr,init,cgrid%pyglob_id, &
       !         var_len,var_len_global,max_ptrs,'CROOT :11:dail') 
       !    call metadata_edio(nvar,igr,'Poly Avg. Fine Root Carbon','?','ipoly') 
       ! end if
       ! if (associated(cgrid%Cstore)) then
       !    nvar=nvar+1
       !    call vtable_edio_r(npts,cgrid%cstore,nvar,igr,init,cgrid%pyglob_id, &
       !         var_len,var_len_global,max_ptrs,'CSTORE :11:dail') 
       !    call metadata_edio(nvar,igr,'Poly Avg. Storage/TNC Carbon','?','ipoly') 
       ! end if
       ! if (associated(cgrid%Ccwd)) then
       !    nvar=nvar+1
       !    call vtable_edio_r(npts,cgrid%Ccwd,nvar,igr,init,cgrid%pyglob_id, &
       !         var_len,var_len_global,max_ptrs,'CCWD :11:dail') 
       !    call metadata_edio(nvar,igr,'Poly Avg. Coarse Woody Debris Carbon','?','ipoly') 
       ! end if
       ! if (associated(cgrid%Nleaf)) then
       !    nvar=nvar+1
       !    call vtable_edio_r(npts,cgrid%Nleaf,nvar,igr,init,cgrid%pyglob_id, &
       !         var_len,var_len_global,max_ptrs,'NLEAF :11:dail') 
       !    call metadata_edio(nvar,igr,'Poly Avg. Leaf Nitrogen','?','ipoly') 
       ! end if
       ! if (associated(cgrid%Ndead)) then
       !    nvar=nvar+1
       !    call vtable_edio_r(npts,cgrid%Ndead,nvar,igr,init,cgrid%pyglob_id, &
       !         var_len,var_len_global,max_ptrs,'NDEAD :11:dail') 
       !    call metadata_edio(nvar,igr,'Poly Avg. Wood Nitrogen','?','ipoly') 
       ! end if
       ! if (associated(cgrid%Nroot)) then
       !    nvar=nvar+1
       !    call vtable_edio_r(npts,cgrid%Nroot,nvar,igr,init,cgrid%pyglob_id, &
       !         var_len,var_len_global,max_ptrs,'NROOT :11:dail') 
       !    call metadata_edio(nvar,igr,'Poly Avg. Fine Root Nitrogen','?','ipoly') 
       ! end if
       ! if (associated(cgrid%Nstore)) then
       !    nvar=nvar+1
       !    call vtable_edio_r(npts,cgrid%Nstore,nvar,igr,init,cgrid%pyglob_id, &
       !         var_len,var_len_global,max_ptrs,'NSTORE :11:dail') 
       !    call metadata_edio(nvar,igr,'Poly Avg. Storage Nitrogen','?','ipoly') 
       ! end if
       ! if (associated(cgrid%Ncwd)) then
       !    nvar=nvar+1
       !    call vtable_edio_r(npts,cgrid%Ncwd,nvar,igr,init,cgrid%pyglob_id, &
       !         var_len,var_len_global,max_ptrs,'NCWD :11:dail') 
       !    call metadata_edio(nvar,igr,'Poly Avg. Leaf Carbon','?','ipoly') 
       ! end if

       !-------- TOTAL CARBON AND NITROGEN FLUX  ---------------
       ! Added by MCD for NCEAS/FACE intercomparison (Apr 7 2009)
      if (associated(cgrid%Cleaf_grow)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%Cleaf_grow,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'CLEAF_GROW :11:hist:anal:dail') 
         call metadata_edio(nvar,igr,'Poly Avg. Leaf Carbon Growth','?','ipoly') 
      end if
      if (associated(cgrid%Croot_grow)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%Croot_grow,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'CROOT_GROW :11:hist:anal:dail') 
         call metadata_edio(nvar,igr,'Poly Avg. Fine Root Carbon Growth','?','ipoly') 
      end if
      if (associated(cgrid%Cdead_grow)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%Cdead_grow,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'CDEAD_GROW :11:hist:anal:dail') 
         call metadata_edio(nvar,igr,'Poly Avg. Wood Carbon Growth','?','ipoly') 
      end if
      if (associated(cgrid%Cstore_grow)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%Cstore_Grow,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'CSTORE_GROW :11:hist:anal:dail') 
         call metadata_edio(nvar,igr,'Poly Avg. Storage Carbon growth','?','ipoly') 
      end if
      if (associated(cgrid%Cleaf_litter_flux)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%Cleaf_litter_flux,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'CLEAF_LITTER_FLUX :11:hist:anal:dail') 
         call metadata_edio(nvar,igr,'Poly Avg. Leaf Litter Carbon Flux','?','ipoly') 
      end if
      if (associated(cgrid%Croot_litter_flux)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%Croot_litter_flux,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'CROOT_LITTER_FLUX :11:hist:anal:dail') 
         call metadata_edio(nvar,igr,'Poly Avg. Fine Root Litter Carbon Flux','?','ipoly') 
      end if
      if (associated(cgrid%Ccwd_flux)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%Ccwd_flux,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'CCWD_FLUX :11:hist:anal:dail') 
         call metadata_edio(nvar,igr,'Poly Avg. Coarse Woody Debris Carbon FLUX','?','ipoly') 
      end if
      if (associated(cgrid%Nleaf_grow)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%Nleaf_grow,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'NLEAF_GROW :11:hist:anal:dail') 
         call metadata_edio(nvar,igr,'Poly Avg. Leaf Nitrogen growth','?','ipoly') 
      end if
      if (associated(cgrid%Nroot_grow)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%Nroot_grow,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'NROOT_GROW :11:hist:anal:dail') 
         call metadata_edio(nvar,igr,'Poly Avg. Fine Root Nitrogen Growth','?','ipoly') 
      end if
      if (associated(cgrid%Ndead_grow)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%Ndead_grow,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'NDEAD_GROW :11:hist:anal:dail') 
         call metadata_edio(nvar,igr,'Poly Avg. Wood Nitrogen growth','?','ipoly') 
      end if
      if (associated(cgrid%Nstore_grow)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%Nstore_grow,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'NSTORE_GROW :11:hist:anal:dail') 
         call metadata_edio(nvar,igr,'Poly Avg. Storage Nitrogen Growth','?','ipoly') 
      end if
      if (associated(cgrid%Nleaf_litter_flux)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%Nleaf_litter_flux,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'NLEAF_LITTER_FLUX :11:hist:anal:dail') 
         call metadata_edio(nvar,igr,'Poly Avg. Leaf litter Nitrogen flux','?','ipoly') 
      end if
      if (associated(cgrid%Nroot_litter_flux)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%Nroot_litter_flux,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'NROOT_LITTER_FLUX :11:hist:anal:dail') 
         call metadata_edio(nvar,igr,'Poly Avg. fine root litter Nitrogen flux','?','ipoly') 
      end if
      if (associated(cgrid%Ncwd_flux)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%Ncwd_flux,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'NCWD_FLUX :11:hist:anal:dail') 
         call metadata_edio(nvar,igr,'Poly Avg. CWD Nitrogen flux','?','ipoly') 
      end if
      if (associated(cgrid%Nbiomass_uptake)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%Nbiomass_uptake,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'NBIOMASS_UPTAKE :11:hist:anal:dail') 
         call metadata_edio(nvar,igr,'Poly Avg. Nitrogen Uptake','?','ipoly') 
      end if
      if (associated(cgrid%Ngross_min)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%Ngross_min,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'NGROSS_MIN :11:hist:anal:dail') 
         call metadata_edio(nvar,igr,'Poly Avg. Gross Nitrogen mineralization','?','ipoly') 
      end if
      if (associated(cgrid%Nnet_min)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%Nnet_min,nvar,igr,init,cgrid%pyglob_id, &
              var_len,var_len_global,max_ptrs,'NNET_MIN :11:hist:anal:dail') 
         call metadata_edio(nvar,igr,'Poly Avg. Net Nitrogen mineralization','?','ipoly') 
      end if


      !------ Polygon average of soil C and N pools. --------------------------------------!
      if(associated(cgrid%fast_soil_c    )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%fast_soil_c                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FAST_SOIL_C_PY     :11:hist:anal:dail:opti')
         call metadata_edio(nvar,igr,'Soil Carbon (Fast pool)'                             &
                           ,'[kgC/m2]','(ipoly)')
      end if
      if(associated(cgrid%slow_soil_c    )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%slow_soil_c                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'SLOW_SOIL_C_PY     :11:hist:anal:dail:opti')
         call metadata_edio(nvar,igr,'Soil Carbon (Slow pool)'                             &
                           ,'[kgC/m2]','(ipoly)')
      end if
      if(associated(cgrid%struct_soil_c  )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%struct_soil_c                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'STRUCT_SOIL_C_PY   :11:hist:anal:dail:opti')
         call metadata_edio(nvar,igr,'Soil Carbon (Structural pool)'                       &
                           ,'[kgC/m2]','(ipoly)')
      end if
      if(associated(cgrid%struct_soil_l  )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%struct_soil_l                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'STRUCT_SOIL_L_PY   :11:hist:anal:dail')
         call metadata_edio(nvar,igr,'Soil Lignin (Structural pool)'                       &
                           ,'[kgC/m2]','(ipoly)')
      end if
      if(associated(cgrid%cwd_c     )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%cwd_c                                               &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'CWD_C_PY    :11:hist:anal:dail')
         call metadata_edio(nvar,igr,'Coarse woody debris'                                 &
                           ,'[kgC/m2]','(ipoly)')
      end if
      if(associated(cgrid%fast_soil_n    )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%fast_soil_n                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FAST_SOIL_N_PY     :11:hist:anal:dail')
         call metadata_edio(nvar,igr,'Soil Nitrogen (Fast pool)'                           &
                           ,'[kgN/m2]','(ipoly)')
      end if
      if(associated(cgrid%mineral_soil_n )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%mineral_soil_n                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MINERAL_SOIL_N_PY  :11:hist:anal:dail')
         call metadata_edio(nvar,igr,'Soil Nitrogen (Mineralised pool)'                    &
                           ,'[kgN/m2]','(ipoly)')
      end if
      if(associated(cgrid%cwd_n     )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%cwd_n                                               &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'CWD_N_PY    :11:hist:anal:dail')
         call metadata_edio(nvar,igr,'Coarse woody debris nitrogen'                        &
                           ,'[kgN/m2]','(ipoly)')
      end if
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_edtype_p11inst
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the polygon-level variables         !
   ! (edtype) that have one dimension and are real (type 11), and are fast mean variables. !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_edtype_p11fmean(cgrid,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(edtype)          , target        :: cgrid
      integer               , intent(in)    :: init
      integer               , intent(in)    :: igr
      integer               , intent(in)    :: var_len
      integer               , intent(in)    :: max_ptrs
      integer               , intent(in)    :: var_len_global
      integer               , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                               :: npts
      character(len=str_len)                :: fast_keys
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !      Decide whether to write the sub-daily means to the history file.              !
      !------------------------------------------------------------------------------------!
      if (history_fast) then
         fast_keys = 'hist:anal:opti'
      else
         fast_keys = 'anal:opti'
      end if
      !------------------------------------------------------------------------------------!


      
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !     This is the 1-D block.  All variables must have the number of points defined   !
      ! by npts.                                                                           !
      !------------------------------------------------------------------------------------!
      npts = cgrid%npolygons

      if (associated(cgrid%fmean_gpp             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_gpp                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_GPP_PY               :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Gross primary productivity'                  &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_npp             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_npp                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_NPP_PY               :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Net primary productivity'                    &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_leaf_resp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_leaf_resp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_LEAF_RESP_PY         :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf respiration'                            &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_root_resp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_root_resp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ROOT_RESP_PY         :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Root respiration'                            &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_leaf_growth_resp )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_leaf_growth_resp                              &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_LEAF_GROWTH_RESP_PY   :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf growth respiration'                     &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_root_growth_resp )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_root_growth_resp                              &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ROOT_GROWTH_RESP_PY   :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Root growth respiration'                     &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_sapa_growth_resp )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_sapa_growth_resp                              &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SAPA_GROWTH_RESP_PY   :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Aboveground sapwood growth respiration'      &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_sapb_growth_resp )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_sapb_growth_resp                              &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SAPB_GROWTH_RESP_PY   :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Belowground sapwood growth respiration'      &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_leaf_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_leaf_storage_resp                             &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_LEAF_STORAGE_RESP_PY      :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf storage respiration'                    &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_root_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_root_storage_resp                             &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ROOT_STORAGE_RESP_PY      :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Root storage respiration'                    &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_sapa_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_sapa_storage_resp                             &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SAPA_STORAGE_RESP_PY      :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Aboveground Sapwood storage respiration'     &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_sapb_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_sapb_storage_resp                             &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SAPB_STORAGE_RESP_PY      :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Belowground Sapwood storage respiration'     &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_plresp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_plresp                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_PLRESP_PY            :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Plant respiration'                           &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_leaf_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_leaf_energy                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_LEAF_ENERGY_PY       :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf internal energy'                        &
                           ,'[       J/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_leaf_water      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_leaf_water                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_LEAF_WATER_PY        :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf surface water'                          &
                           ,'[      kg/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_leaf_hcap       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_leaf_hcap                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_LEAF_HCAP_PY         :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf heat capacity'                          &
                           ,'[     J/m2/K]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_leaf_vpdef      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_leaf_vpdef                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_LEAF_VPDEF_PY        :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf vapour pressure deficit'                &
                           ,'[         Pa]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_leaf_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_leaf_temp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_LEAF_TEMP_PY         :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf temperature'                            &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_leaf_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_leaf_fliq                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_LEAF_FLIQ_PY         :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Liquid fraction'                             &
                           ,'[         --]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_leaf_gsw        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_leaf_gsw                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_LEAF_GSW_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Stomatal conductance'                        &
                           ,'[kg/m2leaf/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_leaf_gbw        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_leaf_gbw                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_LEAF_GBW_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf boundary layer conductance'             &
                           ,'[kg/m2leaf/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_wood_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_wood_energy                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_WOOD_ENERGY_PY       :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Wood internal energy'                        &
                           ,'[       J/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_wood_water      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_wood_water                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_WOOD_WATER_PY        :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Wood surface water'                          &
                           ,'[      kg/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_wood_hcap       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_wood_hcap                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_WOOD_HCAP_PY         :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Wood heat capacity'                          &
                           ,'[     J/m2/K]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_wood_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_wood_temp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_WOOD_TEMP_PY         :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Wood temperature'                            &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_wood_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_wood_fliq                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_WOOD_FLIQ_PY         :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Liquid fraction'                             &
                           ,'[         --]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_wood_gbw        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_wood_gbw                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_WOOD_GBW_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Wood boundary layer conductance'             &
                           ,'[kg/m2wood/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_fs_open         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_fs_open                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_FS_OPEN_PY           :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Net stress factor'                           &
                           ,'[         --]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_fsw             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_fsw                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_FSW_PY               :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Moisture stress'                             &
                           ,'[         --]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_fsn             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_fsn                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_FSN_PY               :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Nitrogen stress'                             &
                           ,'[         --]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_A_open          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_A_open                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_A_OPEN_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Assimilation rate (no soil moist. stress)'   &
                           ,'[umol/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_A_closed        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_A_closed                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_A_CLOSED_PY        :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Minimum assimilation rate'                   &
                           ,'[umol/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_A_net           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_A_net                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_A_NET_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Actual assimilation rate'                    &
                           ,'[umol/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_A_light         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_A_light                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_A_LIGHT_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Light-limited assimilation rate'             &
                           ,'[umol/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_A_rubp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_A_rubp                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_A_RUBP_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - RuBP-limited assimilation rate'              &
                           ,'[umol/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_A_co2           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_A_co2                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_A_CO2_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - CO2-limited assimilation rate'               &
                           ,'[umol/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_psi_open        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_psi_open                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_PSI_OPEN_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Transpiration with no stress'                &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_psi_closed      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_psi_closed                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_PSI_CLOSED_PY        :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Transpiration at maximum stress'             &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_water_supply    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_water_supply                                  &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_WATER_SUPPLY_PY      :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Water supply'                                &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_par_l           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_par_l                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_PAR_L_PY             :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - PAR absorbed by leaves'                      &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_par_l_beam      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_par_l_beam                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_PAR_L_BEAM_PY        :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Direct PAR absorbed by leaves'               &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_par_l_diff      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_par_l_diff                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_PAR_L_DIFF_PY        :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Diffuse PAR absorbed by leaves'              &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_rshort_l        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_rshort_l                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_RSHORT_L_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Shortwave radiation absorbed by leaves'      &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_rlong_l         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_rlong_l                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_RLONG_L_PY           :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Longwave radiation absorbed by leaves'       &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_sensible_lc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_sensible_lc                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SENSIBLE_LC_PY       :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Sensible heat'                               &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_vapor_lc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_vapor_lc                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_VAPOR_LC_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf evaporation'                            &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_transp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_transp                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_TRANSP_PY            :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf transpiration'                          &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_intercepted_al  )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_intercepted_al                                &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_INTERCEPTED_AL_PY    :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf interception'                           &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_wshed_lg        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_wshed_lg                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_WSHED_LG_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf shedding'                               &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_rshort_w        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_rshort_w                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_RSHORT_W_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Shortwave radiation absorbed by wood'        &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_rlong_w         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_rlong_w                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_RLONG_W_PY           :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Longwave radiation absorbed by wood'         &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_sensible_wc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_sensible_wc                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SENSIBLE_WC_PY       :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Sensible heat'                               &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_vapor_wc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_vapor_wc                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_VAPOR_WC_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Wood evaporation'                            &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_intercepted_aw  )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_intercepted_aw                                &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_INTERCEPTED_AW_PY    :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Wood interception'                           &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_wshed_wg        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_wshed_wg                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_WSHED_WG_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Wood shedding'                               &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_rh              )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_rh                                            &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_RH_PY                :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Heterotrophic respiration'                   &
                           ,'[   kg/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_cwd_rh          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_cwd_rh                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CWD_RH_PY            :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Coarse woody debris respiration'             &
                           ,'[   kg/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_nep             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_nep                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_NEP_PY               :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Net Ecosystem productivity'                  &
                           ,'[   kg/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_rk4step         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_rk4step                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_RK4STEP_PY           :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - RK4 time step'                               &
                           ,'[          s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_available_water )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_available_water                               &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_AVAILABLE_WATER_PY   :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Available water'                             &
                           ,'[      kg/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_can_theiv       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_can_theiv                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CAN_THEIV_PY         :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Ice-vapour equiv. pot. temperature - CAS'    &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_can_theta       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_can_theta                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CAN_THETA_PY         :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Potential temperature - CAS'                 &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_can_vpdef       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_can_vpdef                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CAN_VPDEF_PY         :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Vapour pressure deficit - CAS'               &
                           ,'[         Pa]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_can_temp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_can_temp                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CAN_TEMP_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Temperature - CAS'                           &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_can_shv         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_can_shv                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CAN_SHV_PY           :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Specific humidity - CAS'                     &
                           ,'[      kg/kg]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_can_co2         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_can_co2                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CAN_CO2_PY           :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - CO2 mixing ratio - CAS'                      &
                           ,'[   umol/mol]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_can_rhos        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_can_rhos                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CAN_RHOS_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Air density - CAS'                           &
                           ,'[      kg/m3]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_can_prss        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_can_prss                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CAN_PRSS_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Pressure - CAS'                              &
                           ,'[         Pa]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_gnd_temp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_gnd_temp                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_GND_TEMP_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Ground temperature'                          &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_gnd_shv         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_gnd_shv                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_GND_SHV_PY           :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Ground specific humidity'                    &
                           ,'[      kg/kg]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_can_ggnd        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_can_ggnd                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CAN_GGND_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Net canopy conductance'                      &
                           ,'[        m/2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_sfcw_depth      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_sfcw_depth                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SFCW_DEPTH_PY        :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Depth - temporary water layer'               &
                           ,'[          m]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_sfcw_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_sfcw_energy                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SFCW_ENERGY_PY       :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Internal energy - temporary water layer'     &
                           ,'[       J/kg]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_sfcw_mass       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_sfcw_mass                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SFCW_MASS_PY         :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Water mass - temporary water layer'          &
                           ,'[      kg/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_sfcw_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_sfcw_temp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SFCW_TEMP_PY         :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Temperature - temporary water layer'         &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_sfcw_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_sfcw_fliq                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SFCW_FLIQ_PY         :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Liquid fraction - temporary water layer'     &
                           ,'[         --]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_rshort_gnd      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_rshort_gnd                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_RSHORT_GND_PY        :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Shortwave radiation absorbed by ground'      &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_par_gnd         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_par_gnd                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_PAR_GND_PY           :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - PAR absorbed by ground'                      &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_rlong_gnd       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_rlong_gnd                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_RLONG_GND_PY         :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Longwave radiation absorbed by ground'       &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_rlongup         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_rlongup                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_RLONGUP_PY           :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Outgoing longwave radiation'                 &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_parup           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_parup                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_PARUP_PY             :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Outgoing PAR'                                &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_nirup           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_nirup                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_NIRUP_PY             :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Outgoing near infra-red radiation'           &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_rshortup        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_rshortup                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_RSHORTUP_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Outgoing shortwave radiation'                &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_rnet            )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_rnet                                          &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_RNET_PY              :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Net radiation at top of canopy'              &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_albedo          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_albedo                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ALBEDO_PY            :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Albedo'                                      &
                           ,'[       ----]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_albedo_par      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_albedo_par                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ALBEDO_PAR_PY        :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Albedo - direct radiation'                   &
                           ,'[       ----]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_albedo_nir      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_albedo_nir                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ALBEDO_NIR_PY        :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Albedo - diffuse radiation'                  &
                           ,'[       ----]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_rlong_albedo    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_rlong_albedo                                  &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_RLONG_ALBEDO_PY      :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Kind of albedo - longwave radiation'         &
                           ,'[       ----]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_ustar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_ustar                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_USTAR_PY             :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Friction velocity'                           &
                           ,'[        m/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_tstar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_tstar                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_TSTAR_PY             :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Gradient scale for potential temperature'    &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_qstar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_qstar                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_QSTAR_PY             :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Gradient scale for specific humidity'        &
                           ,'[      kg/kg]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_cstar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_cstar                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CSTAR_PY             :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Gradient scale for CO2 mixing ratio'         &
                           ,'[   umol/mol]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_carbon_ac       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_carbon_ac                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CARBON_AC_PY         :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - CO2 flux: atmosphere -> CAS'                 &
                           ,'[  umol/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_carbon_st       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_carbon_st                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CARBON_ST_PY         :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - CO2 storage at CAS'                          &
                           ,'[  umol/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_vapor_gc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_vapor_gc                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_VAPOR_GC_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Water vapour flux: ground -> CAS'            &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_vapor_ac        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_vapor_ac                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_VAPOR_AC_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Water vapour flux: atmosphere -> CAS'        &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_throughfall     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_throughfall                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_THROUGHFALL_PY       :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Throughfall rate'                            &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_runoff          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_runoff                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_RUNOFF_PY            :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Water runoff'                                &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_drainage        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_drainage                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_DRAINAGE_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Water drainage'                              &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_sensible_gc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_sensible_gc                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SENSIBLE_GC_PY       :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Sensible heat flux: ground -> CAS'           &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_sensible_ac     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_sensible_ac                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SENSIBLE_AC_PY       :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Sensible heat flux: atmosphere -> CAS'       &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_qthroughfall    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_qthroughfall                                  &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_QTHROUGHFALL_PY      :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Throughfall rate'                            &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_qrunoff         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_qrunoff                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_QRUNOFF_PY           :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Surface runoff'                              &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_qdrainage       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_qdrainage                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_QDRAINAGE_PY         :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Drainage energy loss'                        &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_atm_theiv       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_atm_theiv                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_THEIV_PY         :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Ice-vapour equiv. pot. temp.: Atmosphere'    &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_atm_theta       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_atm_theta                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_THETA_PY         :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Potential temperature: Atmosphere'           &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_atm_temp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_atm_temp                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_TEMP_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Temperature: Atmosphere'                     &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_atm_vpdef       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_atm_vpdef                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_VPDEF_PY         :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Vapour pressure deficit: Atmosphere'         &
                           ,'[         Pa]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_atm_shv         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_atm_shv                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_SHV_PY           :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Specific humidity: Atmosphere'               &
                           ,'[      kg/kg]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_atm_rshort      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_atm_rshort                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_RSHORT_PY        :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Shortwave radiation: Atmosphere'             &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_atm_rshort_diff )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_atm_rshort_diff                               &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_RSHORT_DIFF_PY   :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Diffuse SW radiation: Atmosphere'            &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_atm_par         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_atm_par                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_PAR_PY           :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - PAR: Atmosphere'                             &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_atm_par_diff    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_atm_par_diff                                  &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_PAR_DIFF_PY      :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Diffuse PAR: Atmosphere'                     &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_atm_rlong       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_atm_rlong                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_RLONG_PY         :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Longwave radiation: Atmosphere'              &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_atm_vels        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_atm_vels                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_VELS_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Wind speed: atmosphere'                      &
                           ,'[        m/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_atm_rhos        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_atm_rhos                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_RHOS_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Air density: Atmosphere'                     &
                           ,'[      kg/m3]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_atm_prss        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_atm_prss                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_PRSS_PY          :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Air pressure: Atmosphere'                    &
                           ,'[         Pa]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_atm_co2         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_atm_co2                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_CO2_PY           :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - CO2 mixing ratio: Atmosphere'                &
                           ,'[   umol/mol]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_pcpg            )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_pcpg                                          &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_PCPG_PY              :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Precipitation rate'                          &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_qpcpg           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_qpcpg                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_QPCPG_PY             :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Energy gain - rain'                          &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_dpcpg           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_dpcpg                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_DPCPG_PY             :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Precipitation depth'                         &
                           ,'[          m]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_soil_wetness    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_soil_wetness                                  &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SOIL_WETNESS_PY      :11:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Soil wetness index'                          &
                           ,'[         --]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_skin_temp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_skin_temp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SKIN_TEMP_PY  :11:'//trim(fast_keys)  )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Skin temperature'                            &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_lai)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_lai                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_LAI_PY  :11:'//trim(fast_keys)  )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - LAI'					   &
                           ,'[           m2/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%fmean_skin_temp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_bdead                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_BDEAD_PY  :11:'//trim(fast_keys)  )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Dead biomass'                                &
                           ,'[          kg/plant]','(ipoly)'            )
      end if

      return
   end subroutine filltab_edtype_p11fmean
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the polygon-level variables         !
   ! (edtype) that have one dimension and are real (type 11), and are daily mean           !
   ! variables.                                                                            !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_edtype_p11dmean(cgrid,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(edtype)          , target        :: cgrid
      integer               , intent(in)    :: init
      integer               , intent(in)    :: igr
      integer               , intent(in)    :: var_len
      integer               , intent(in)    :: max_ptrs
      integer               , intent(in)    :: var_len_global
      integer               , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                               :: npts
      character(len=str_len)                :: dail_keys
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      !     Decide whether to save daily mean to history or not.                           !
      !------------------------------------------------------------------------------------!
      if (history_dail) then
         dail_keys = 'hist:dail'
      else
         dail_keys = 'dail'
      end if
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !     This is the 1-D block.  All variables must have the number of points defined   !
      ! by npts.                                                                           !
      !------------------------------------------------------------------------------------!
      npts = cgrid%npolygons


      if (associated(cgrid%dmean_nppleaf         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_nppleaf                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_NPPLEAF_PY           :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Net primary productivity - Leaf'                 &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_nppfroot        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_nppfroot                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_NPPFROOT_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Net primary productivity - Fine root'            &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_nppsapwood      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_nppsapwood                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_NPPSAPWOOD_PY        :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Net primary productivity - Sap wood'             &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_nppcroot        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_nppcroot                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_NPPCROOT_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Net primary productivity - Coarse root'          &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_nppseeds        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_nppseeds                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_NPPSEEDS_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Net primary productivity - seeds'                &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_nppwood         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_nppwood                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_NPPWOOD_PY           :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Net primary productivity - heart wood'           &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_nppdaily        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_nppdaily                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_NPPDAILY_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Net primary productivity - total'                &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_A_decomp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_A_decomp                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_A_DECOMP_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Decomposition scale'                             &
                           ,'[         --]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_Af_decomp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_Af_decomp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_AF_DECOMP_PY         :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Decomposition scale with N immobilisation'       &
                           ,'[         --]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_co2_residual    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_co2_residual                                  &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CO2_RESIDUAL_PY      :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Budget check - CO2 residual'                     &
                           ,'[    umol/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_energy_residual )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_energy_residual                               &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ENERGY_RESIDUAL_PY   :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Budget check - Enthalpy residual'                &
                           ,'[       J/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_water_residual  )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_water_residual                                &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_WATER_RESIDUAL_PY    :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Budget check - Water residual'                   &
                           ,'[      kg/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_gpp             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_gpp                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_GPP_PY               :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Gross primary productivity'                      &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_npp             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_npp                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_NPP_PY               :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Net primary productivity'                        &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_leaf_resp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_leaf_resp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_LEAF_RESP_PY         :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf respiration'                                &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_root_resp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_root_resp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ROOT_RESP_PY         :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Root respiration'                                &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_leaf_growth_resp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_leaf_growth_resp                              &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_LEAF_GROWTH_RESP_PY  :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf growth respiration'                         &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_root_growth_resp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_root_growth_resp                              &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ROOT_GROWTH_RESP_PY  :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Root growth respiration'                         &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_sapa_growth_resp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_sapa_growth_resp                              &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SAPA_GROWTH_RESP_PY  :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Aboveground sapwood growth respiration'          &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_sapb_growth_resp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_sapb_growth_resp                              &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SAPB_GROWTH_RESP_PY  :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Belowground sapwood growth respiration'          &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_leaf_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_leaf_storage_resp                             &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_LEAF_STORAGE_RESP_PY      :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf storage respiration'                        &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_root_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_root_storage_resp                             &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ROOT_STORAGE_RESP_PY      :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Root Storage respiration'                        &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_sapa_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_sapa_storage_resp                             &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SAPB_STORAGE_RESP_PY      :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Aboveground Sapwood Storage respiration'         &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_sapb_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_sapb_storage_resp                             &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SAPB_STORAGE_RESP_PY      :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Belowground Sapwood Storage respiration'         &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_plresp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_plresp                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_PLRESP_PY            :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Plant respiration'                               &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_leaf_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_leaf_energy                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_LEAF_ENERGY_PY       :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf internal energy'                            &
                           ,'[       J/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_leaf_water      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_leaf_water                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_LEAF_WATER_PY        :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf surface water'                              &
                           ,'[      kg/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_leaf_hcap       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_leaf_hcap                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_LEAF_HCAP_PY         :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf heat capacity'                              &
                           ,'[     J/m2/K]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_leaf_vpdef      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_leaf_vpdef                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_LEAF_VPDEF_PY        :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf vapour pressure deficit'                    &
                           ,'[         Pa]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_leaf_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_leaf_temp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_LEAF_TEMP_PY         :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf temperature'                                &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_leaf_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_leaf_fliq                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_LEAF_FLIQ_PY         :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Liquid fraction'                                 &
                           ,'[         --]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_leaf_gsw        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_leaf_gsw                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_LEAF_GSW_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Stomatal conductance'                            &
                           ,'[kg/m2leaf/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_leaf_gbw        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_leaf_gbw                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_LEAF_GBW_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf boundary layer conductance'                 &
                           ,'[kg/m2leaf/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_wood_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_wood_energy                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_WOOD_ENERGY_PY       :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Wood internal energy'                            &
                           ,'[       J/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_wood_water      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_wood_water                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_WOOD_WATER_PY        :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Wood surface water'                              &
                           ,'[      kg/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_wood_hcap       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_wood_hcap                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_WOOD_HCAP_PY         :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Wood heat capacity'                              &
                           ,'[     J/m2/K]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_wood_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_wood_temp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_WOOD_TEMP_PY         :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Wood temperature'                                &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_wood_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_wood_fliq                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_WOOD_FLIQ_PY         :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Liquid fraction'                                 &
                           ,'[         --]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_wood_gbw        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_wood_gbw                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_WOOD_GBW_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Wood boundary layer conductance'                 &
                           ,'[kg/m2wood/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_fs_open         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_fs_open                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_FS_OPEN_PY           :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Net stress factor'                               &
                           ,'[         --]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_fsw             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_fsw                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_FSW_PY               :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Moisture stress'                                 &
                           ,'[         --]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_fsn             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_fsn                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_FSN_PY               :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Nitrogen stress'                                 &
                           ,'[         --]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_A_open          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_A_open                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_A_OPEN_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Assimilation rate (no soil moist. stress)'       &
                           ,'[umol/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_A_closed        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_A_closed                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_A_CLOSED_PY        :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Minimum assimilation rate'                       &
                           ,'[umol/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_A_net           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_A_net                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_A_NET_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Actual assimilation rate'                        &
                           ,'[umol/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_A_light         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_A_light                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_A_LIGHT_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Light-limited assimilation rate'                 &
                           ,'[umol/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_A_rubp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_A_rubp                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_A_RUBP_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - RuBP-limited assimilation rate'                  &
                           ,'[umol/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_A_co2           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_A_co2                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_A_CO2_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - CO2-limited assimilation rate'                   &
                           ,'[umol/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_psi_open        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_psi_open                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_PSI_OPEN_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Transpiration with no stress'                    &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_psi_closed      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_psi_closed                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_PSI_CLOSED_PY        :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Transpiration at maximum stress'                 &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_water_supply    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_water_supply                                  &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_WATER_SUPPLY_PY      :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Water supply'                                    &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_par_l           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_par_l                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_PAR_L_PY             :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - PAR absorbed by leaves'                          &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_par_l_beam      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_par_l_beam                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_PAR_L_BEAM_PY        :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Direct PAR absorbed by leaves'                   &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_par_l_diff      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_par_l_diff                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_PAR_L_DIFF_PY        :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Diffuse PAR absorbed by leaves'                  &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_rshort_l        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_rshort_l                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_RSHORT_L_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Shortwave radiation absorbed by leaves'          &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_rlong_l         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_rlong_l                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_RLONG_L_PY           :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Longwave radiation absorbed by leaves'           &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_sensible_lc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_sensible_lc                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SENSIBLE_LC_PY       :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Sensible heat'                                   &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_vapor_lc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_vapor_lc                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_VAPOR_LC_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf evaporation'                                &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_transp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_transp                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_TRANSP_PY            :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf transpiration'                              &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_intercepted_al  )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_intercepted_al                                &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_INTERCEPTED_AL_PY    :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf interception'                               &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_wshed_lg        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_wshed_lg                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_WSHED_LG_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf shedding'                                   &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_rshort_w        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_rshort_w                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_RSHORT_W_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Shortwave radiation absorbed by wood'            &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_rlong_w         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_rlong_w                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_RLONG_W_PY           :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Longwave radiation absorbed by wood'             &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_sensible_wc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_sensible_wc                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SENSIBLE_WC_PY       :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Sensible heat'                                   &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_vapor_wc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_vapor_wc                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_VAPOR_WC_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Wood evaporation'                                &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_intercepted_aw  )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_intercepted_aw                                &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_INTERCEPTED_AW_PY    :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Wood interception'                               &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_wshed_wg        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_wshed_wg                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_WSHED_WG_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Wood shedding'                                   &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_rh              )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_rh                                            &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_RH_PY                :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Heterotrophic respiration'                       &
                           ,'[   kg/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_cwd_rh          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_cwd_rh                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CWD_RH_PY            :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Coarse woody debris respiration'                 &
                           ,'[   kg/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_nep             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_nep                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_NEP_PY               :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Net Ecosystem productivity'                      &
                           ,'[   kg/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_rk4step         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_rk4step                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_RK4STEP_PY           :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - RK4 time step'                                   &
                           ,'[          s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_available_water )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_available_water                               &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_AVAILABLE_WATER_PY   :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Available water'                                 &
                           ,'[      kg/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_can_theiv       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_can_theiv                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CAN_THEIV_PY         :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Ice-vapour equiv. pot. temperature - CAS'        &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_can_theta       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_can_theta                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CAN_THETA_PY         :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Potential temperature - CAS'                     &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_can_vpdef       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_can_vpdef                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CAN_VPDEF_PY         :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Vapour pressure deficit - CAS'                   &
                           ,'[         Pa]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_can_temp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_can_temp                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CAN_TEMP_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Temperature - CAS'                               &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_can_shv         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_can_shv                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CAN_SHV_PY           :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Specific humidity - CAS'                         &
                           ,'[      kg/kg]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_can_co2         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_can_co2                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CAN_CO2_PY           :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - CO2 mixing ratio - CAS'                          &
                           ,'[   umol/mol]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_can_rhos        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_can_rhos                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CAN_RHOS_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Air density - CAS'                               &
                           ,'[      kg/m3]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_can_prss        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_can_prss                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CAN_PRSS_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Pressure - CAS'                                  &
                           ,'[         Pa]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_gnd_temp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_gnd_temp                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_GND_TEMP_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Ground temperature'                              &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_gnd_shv         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_gnd_shv                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_GND_SHV_PY           :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Ground specific humidity'                        &
                           ,'[      kg/kg]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_can_ggnd        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_can_ggnd                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CAN_GGND_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Net canopy conductance'                          &
                           ,'[        m/2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_sfcw_depth      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_sfcw_depth                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SFCW_DEPTH_PY        :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Depth - temporary water layer'                   &
                           ,'[          m]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_sfcw_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_sfcw_energy                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SFCW_ENERGY_PY       :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Internal energy - temporary water layer'         &
                           ,'[       J/kg]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_sfcw_mass       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_sfcw_mass                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SFCW_MASS_PY         :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Water mass - temporary water layer'              &
                           ,'[      kg/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_sfcw_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_sfcw_temp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SFCW_TEMP_PY         :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Temperature - temporary water layer'             &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_sfcw_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_sfcw_fliq                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SFCW_FLIQ_PY         :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Liquid fraction - temporary water layer'         &
                           ,'[         --]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_rshort_gnd      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_rshort_gnd                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_RSHORT_GND_PY        :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Shortwave radiation absorbed by ground'          &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_par_gnd         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_par_gnd                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_PAR_GND_PY           :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - PAR absorbed by ground'                          &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_rlong_gnd       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_rlong_gnd                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_RLONG_GND_PY         :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Longwave radiation absorbed by ground'           &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_rlongup         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_rlongup                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_RLONGUP_PY           :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Outgoing longwave radiation'                     &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_parup           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_parup                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_PARUP_PY             :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Outgoing PAR'                                    &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_nirup           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_nirup                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_NIRUP_PY             :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Outgoing near infra-red radiation'               &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_rshortup        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_rshortup                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_RSHORTUP_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Outgoing shortwave radiation'                    &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_rnet            )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_rnet                                          &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_RNET_PY              :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Net radiation at top of canopy'                  &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_albedo          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_albedo                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ALBEDO_PY            :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Albedo'                                          &
                           ,'[       ----]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_albedo_par      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_albedo_par                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ALBEDO_PAR_PY        :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Albedo - direct radiation'                       &
                           ,'[       ----]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_albedo_nir      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_albedo_nir                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ALBEDO_NIR_PY        :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Albedo - diffuse radiation'                      &
                           ,'[       ----]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_rlong_albedo    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_rlong_albedo                                  &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_RLONG_ALBEDO_PY      :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Kind of albedo - longwave radiation'             &
                           ,'[       ----]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_ustar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_ustar                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_USTAR_PY             :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Friction velocity'                               &
                           ,'[        m/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_tstar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_tstar                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_TSTAR_PY             :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Gradient scale for potential temperature'        &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_qstar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_qstar                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_QSTAR_PY             :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Gradient scale for specific humidity'            &
                           ,'[      kg/kg]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_cstar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_cstar                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CSTAR_PY             :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Gradient scale for CO2 mixing ratio'             &
                           ,'[   umol/mol]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_carbon_ac       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_carbon_ac                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CARBON_AC_PY         :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - CO2 flux: atmosphere -> CAS'                     &
                           ,'[  umol/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_carbon_st       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_carbon_st                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CARBON_ST_PY         :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - CO2 storage at CAS'                              &
                           ,'[  umol/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_vapor_gc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_vapor_gc                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_VAPOR_GC_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Water vapour flux: ground -> CAS'                &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_vapor_ac        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_vapor_ac                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_VAPOR_AC_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Water vapour flux: atmosphere -> CAS'            &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_throughfall     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_throughfall                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_THROUGHFALL_PY       :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Throughfall rate'                                &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_runoff          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_runoff                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_RUNOFF_PY            :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Water runoff'                                    &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_drainage        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_drainage                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_DRAINAGE_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Water drainage'                                  &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_sensible_gc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_sensible_gc                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SENSIBLE_GC_PY       :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Sensible heat flux: ground -> CAS'               &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_sensible_ac     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_sensible_ac                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SENSIBLE_AC_PY       :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Sensible heat flux: atmosphere -> CAS'           &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_qthroughfall    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_qthroughfall                                  &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_QTHROUGHFALL_PY      :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Throughfall rate'                                &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_qrunoff         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_qrunoff                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_QRUNOFF_PY           :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Surface runoff'                                  &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_qdrainage       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_qdrainage                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_QDRAINAGE_PY         :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Drainage energy loss'                            &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_atm_theiv       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_atm_theiv                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_THEIV_PY         :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Ice-vapour equiv. pot. temp.: Atmosphere'        &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_atm_theta       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_atm_theta                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_THETA_PY         :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Potential temperature: Atmosphere'               &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_atm_temp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_atm_temp                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_TEMP_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Temperature: Atmosphere'                         &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_atm_vpdef       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_atm_vpdef                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_VPDEF_PY         :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Vapour pressure deficit: Atmosphere'             &
                           ,'[         Pa]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_atm_shv         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_atm_shv                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_SHV_PY           :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Specific humidity: Atmosphere'                   &
                           ,'[      kg/kg]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_atm_rshort      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_atm_rshort                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_RSHORT_PY        :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Shortwave radiation: Atmosphere'                 &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_atm_rshort_diff )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_atm_rshort_diff                               &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_RSHORT_DIFF_PY   :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Diffuse SW radiation: Atmosphere'                &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_atm_par         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_atm_par                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_PAR_PY           :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - PAR: Atmosphere'                                 &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_atm_par_diff    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_atm_par_diff                                  &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_PAR_DIFF_PY      :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Diffuse PAR: Atmosphere'                         &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_atm_rlong       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_atm_rlong                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_RLONG_PY         :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Longwave radiation: Atmosphere'                  &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_atm_vels        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_atm_vels                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_VELS_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Wind speed: atmosphere'                          &
                           ,'[        m/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_atm_rhos        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_atm_rhos                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_RHOS_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Air density: Atmosphere'                         &
                           ,'[      kg/m3]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_atm_prss        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_atm_prss                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_PRSS_PY          :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Air pressure: Atmosphere'                        &
                           ,'[         Pa]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_atm_co2         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_atm_co2                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_CO2_PY           :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - CO2 mixing ratio: Atmosphere'                    &
                           ,'[   umol/mol]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_pcpg            )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_pcpg                                          &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_PCPG_PY              :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Precipitation rate'                              &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_qpcpg           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_qpcpg                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_QPCPG_PY             :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Energy gain - rain'                              &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%dmean_dpcpg           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_dpcpg                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_DPCPG_PY             :11:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Precipitation depth'                             &
                           ,'[          m]','(ipoly)'            )
      end if
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_edtype_p11dmean
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the polygon-level variables         !
   ! (edtype) that have one dimension and are real (type 11), and are monthly mean         !
   ! variables.                                                                            !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_edtype_p11mmean(cgrid,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(edtype)          , target        :: cgrid
      integer               , intent(in)    :: init
      integer               , intent(in)    :: igr
      integer               , intent(in)    :: var_len
      integer               , intent(in)    :: max_ptrs
      integer               , intent(in)    :: var_len_global
      integer               , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                               :: npts
      character(len=str_len)                :: eorq_keys
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !      Check whether we need to save monthly means to history or not.                !
      !------------------------------------------------------------------------------------!
      if (history_eorq) then
         eorq_keys = 'hist:mont:dcyc'
      else
         eorq_keys = 'mont:dcyc'
      end if
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !     This is the 1-D block.  All variables must have the number of points defined   !
      ! by npts.                                                                           !
      !------------------------------------------------------------------------------------!
      npts = cgrid%npolygons

      if (associated(cgrid%mmean_gpp             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_gpp                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_GPP_PY               :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Gross primary productivity'                    &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_npp             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_npp                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_NPP_PY               :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Net primary productivity'                      &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_leaf_resp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_leaf_resp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_LEAF_RESP_PY         :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf respiration'                              &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_root_resp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_root_resp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ROOT_RESP_PY         :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Root respiration'                              &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_leaf_growth_resp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_leaf_growth_resp                              &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_LEAF_GROWTH_RESP_PY  :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf growth respiration'                       &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_root_growth_resp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_root_growth_resp                              &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ROOT_GROWTH_RESP_PY  :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Root growth respiration'                       &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_sapa_growth_resp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_sapa_growth_resp                              &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SAPA_GROWTH_RESP_PY  :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Aboveground sapwood growth respiration'        &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_sapb_growth_resp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_sapb_growth_resp                              &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SAPB_GROWTH_RESP_PY  :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Belowground sapwood growth respiration'        &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_leaf_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_leaf_storage_resp                             &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_LEAF_STORAGE_RESP_PY      :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf Storage respiration'                      &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_root_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_root_storage_resp                             &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ROOT_STORAGE_RESP_PY      :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Root storage respiration'                      &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_sapa_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_sapa_storage_resp                             &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SAPA_STORAGE_RESP_PY      :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Aboveground Sapwood storage respiration'       &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_sapb_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_sapb_storage_resp                             &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SAPB_STORAGE_RESP_PY      :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Belowground Sapwood Storage respiration'       &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_plresp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_plresp                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_PLRESP_PY            :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Plant respiration'                             &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_leaf_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_leaf_energy                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_LEAF_ENERGY_PY       :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf internal energy'                          &
                           ,'[       J/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_leaf_water      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_leaf_water                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_LEAF_WATER_PY        :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf surface water'                            &
                           ,'[      kg/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_leaf_hcap       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_leaf_hcap                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_LEAF_HCAP_PY         :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf heat capacity'                            &
                           ,'[     J/m2/K]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_leaf_vpdef      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_leaf_vpdef                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_LEAF_VPDEF_PY        :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf vapour pressure deficit'                  &
                           ,'[         Pa]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_leaf_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_leaf_temp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_LEAF_TEMP_PY         :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf temperature'                              &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_leaf_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_leaf_fliq                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_LEAF_FLIQ_PY         :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Liquid fraction'                               &
                           ,'[         --]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_leaf_gsw        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_leaf_gsw                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_LEAF_GSW_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Stomatal conductance'                          &
                           ,'[kg/m2leaf/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_leaf_gbw        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_leaf_gbw                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_LEAF_GBW_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf boundary layer conductance'               &
                           ,'[kg/m2leaf/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_wood_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_wood_energy                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_WOOD_ENERGY_PY       :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Wood internal energy'                          &
                           ,'[       J/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_wood_water      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_wood_water                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_WOOD_WATER_PY        :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Wood surface water'                            &
                           ,'[      kg/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_wood_hcap       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_wood_hcap                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_WOOD_HCAP_PY         :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Wood heat capacity'                            &
                           ,'[     J/m2/K]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_wood_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_wood_temp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_WOOD_TEMP_PY         :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Wood temperature'                              &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_wood_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_wood_fliq                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_WOOD_FLIQ_PY         :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Liquid fraction'                               &
                           ,'[         --]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_wood_gbw        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_wood_gbw                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_WOOD_GBW_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Wood boundary layer conductance'               &
                           ,'[kg/m2wood/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_fs_open         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_fs_open                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_FS_OPEN_PY           :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Net stress factor'                             &
                           ,'[         --]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_fsw             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_fsw                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_FSW_PY               :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Moisture stress'                               &
                           ,'[         --]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_fsn             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_fsn                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_FSN_PY               :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Nitrogen stress'                               &
                           ,'[         --]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_A_open          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_A_open                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_A_OPEN_PY          :11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Assimilation rate (no soil moist. stress)'     &
                           ,'[umol/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_A_closed        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_A_closed                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_A_CLOSED_PY        :11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Minimum assimilation rate'                     &
                           ,'[umol/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_A_net           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_A_net                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_A_NET_PY          :11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Actual assimilation rate'                      &
                           ,'[umol/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_A_light         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_A_light                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_A_LIGHT_PY          :11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Light-limited assimilation rate'               &
                           ,'[umol/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_A_rubp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_A_rubp                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_A_RUBP_PY          :11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - RuBP-limited assimilation rate'                &
                           ,'[umol/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_A_co2           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_A_co2                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_A_CO2_PY          :11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - CO2-limited assimilation rate'                 &
                           ,'[umol/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_psi_open        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_psi_open                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_PSI_OPEN_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Transpiration with no stress'                  &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_psi_closed      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_psi_closed                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_PSI_CLOSED_PY        :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Transpiration at maximum stress'               &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_water_supply    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_water_supply                                  &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_WATER_SUPPLY_PY      :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Water supply'                                  &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_par_l           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_par_l                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_PAR_L_PY             :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - PAR absorbed by leaves'                        &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_par_l_beam      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_par_l_beam                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_PAR_L_BEAM_PY        :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Direct PAR absorbed by leaves'                 &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_par_l_diff      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_par_l_diff                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_PAR_L_DIFF_PY        :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Diffuse PAR absorbed by leaves'                &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_rshort_l        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_rshort_l                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_RSHORT_L_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Shortwave radiation absorbed by leaves'        &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_rlong_l         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_rlong_l                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_RLONG_L_PY           :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Longwave radiation absorbed by leaves'         &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_sensible_lc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_sensible_lc                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SENSIBLE_LC_PY       :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Sensible heat'                                 &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_vapor_lc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_vapor_lc                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_VAPOR_LC_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf evaporation'                              &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_transp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_transp                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_TRANSP_PY            :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf transpiration'                            &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_intercepted_al  )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_intercepted_al                                &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_INTERCEPTED_AL_PY    :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf interception'                             &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_wshed_lg        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_wshed_lg                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_WSHED_LG_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf shedding'                                 &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_rshort_w        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_rshort_w                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_RSHORT_W_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Shortwave radiation absorbed by wood'          &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_rlong_w         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_rlong_w                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_RLONG_W_PY           :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Longwave radiation absorbed by wood'           &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_sensible_wc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_sensible_wc                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SENSIBLE_WC_PY       :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Sensible heat'                                 &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_vapor_wc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_vapor_wc                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_VAPOR_WC_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Wood evaporation'                              &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_intercepted_aw  )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_intercepted_aw                                &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_INTERCEPTED_AW_PY    :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Wood interception'                             &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_wshed_wg        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_wshed_wg                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_WSHED_WG_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Wood shedding'                                 &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_rh              )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_rh                                            &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_RH_PY                :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Heterotrophic respiration'                     &
                           ,'[   kg/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_cwd_rh          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_cwd_rh                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CWD_RH_PY            :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Coarse woody debris respiration'               &
                           ,'[   kg/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_nep             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_nep                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_NEP_PY               :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Net Ecosystem productivity'                    &
                           ,'[   kg/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_rk4step         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_rk4step                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_RK4STEP_PY           :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - RK4 time step'                                 &
                           ,'[          s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_available_water )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_available_water                               &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_AVAILABLE_WATER_PY   :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Available water'                               &
                           ,'[      kg/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_can_theiv       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_can_theiv                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CAN_THEIV_PY         :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Ice-vapour equiv. pot. temperature - CAS'      &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_can_theta       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_can_theta                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CAN_THETA_PY         :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Potential temperature - CAS'                   &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_can_vpdef       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_can_vpdef                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CAN_VPDEF_PY         :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Vapour pressure deficit - CAS'                 &
                           ,'[         Pa]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_can_temp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_can_temp                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CAN_TEMP_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Temperature - CAS'                             &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_can_shv         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_can_shv                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CAN_SHV_PY           :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Specific humidity - CAS'                       &
                           ,'[      kg/kg]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_can_co2         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_can_co2                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CAN_CO2_PY           :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - CO2 mixing ratio - CAS'                        &
                           ,'[   umol/mol]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_can_rhos        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_can_rhos                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CAN_RHOS_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Air density - CAS'                             &
                           ,'[      kg/m3]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_can_prss        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_can_prss                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CAN_PRSS_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Pressure - CAS'                                &
                           ,'[         Pa]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_gnd_temp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_gnd_temp                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_GND_TEMP_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Ground temperature'                            &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_gnd_shv         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_gnd_shv                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_GND_SHV_PY           :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Ground specific humidity'                      &
                           ,'[      kg/kg]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_can_ggnd        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_can_ggnd                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CAN_GGND_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Net canopy conductance'                        &
                           ,'[        m/2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_sfcw_depth      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_sfcw_depth                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SFCW_DEPTH_PY        :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Depth - temporary water layer'                 &
                           ,'[          m]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_sfcw_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_sfcw_energy                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SFCW_ENERGY_PY       :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Internal energy - temporary water layer'       &
                           ,'[       J/kg]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_sfcw_mass       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_sfcw_mass                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SFCW_MASS_PY         :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Water mass - temporary water layer'            &
                           ,'[      kg/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_sfcw_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_sfcw_temp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SFCW_TEMP_PY         :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Temperature - temporary water layer'           &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_sfcw_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_sfcw_fliq                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SFCW_FLIQ_PY         :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Liquid fraction - temporary water layer'       &
                           ,'[         --]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_rshort_gnd      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_rshort_gnd                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_RSHORT_GND_PY        :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Shortwave radiation absorbed by ground'        &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_par_gnd         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_par_gnd                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_PAR_GND_PY           :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - PAR absorbed by ground'                        &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_rlong_gnd       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_rlong_gnd                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_RLONG_GND_PY         :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Longwave radiation absorbed by ground'         &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_rlongup         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_rlongup                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_RLONGUP_PY           :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Outgoing longwave radiation'                   &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_parup           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_parup                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_PARUP_PY             :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Outgoing PAR'                                  &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_nirup           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_nirup                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_NIRUP_PY             :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Outgoing near infra-red radiation'             &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_rshortup        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_rshortup                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_RSHORTUP_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Outgoing shortwave radiation'                  &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_rnet            )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_rnet                                          &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_RNET_PY              :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Net radiation at top of canopy'                &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_albedo          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_albedo                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ALBEDO_PY            :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Albedo'                                        &
                           ,'[       ----]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_albedo_par      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_albedo_par                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ALBEDO_PAR_PY        :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Albedo - direct radiation'                     &
                           ,'[       ----]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_albedo_nir      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_albedo_nir                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ALBEDO_NIR_PY        :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Albedo - diffuse radiation'                    &
                           ,'[       ----]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_rlong_albedo    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_rlong_albedo                                  &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_RLONG_ALBEDO_PY      :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Kind of albedo - longwave radiation'           &
                           ,'[       ----]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_ustar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_ustar                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_USTAR_PY             :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Friction velocity'                             &
                           ,'[        m/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_tstar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_tstar                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_TSTAR_PY             :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Gradient scale for potential temperature'      &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_qstar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_qstar                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_QSTAR_PY             :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Gradient scale for specific humidity'          &
                           ,'[      kg/kg]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_cstar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_cstar                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CSTAR_PY             :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Gradient scale for CO2 mixing ratio'           &
                           ,'[   umol/mol]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_carbon_ac       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_carbon_ac                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CARBON_AC_PY         :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - CO2 flux: atmosphere -> CAS'                   &
                           ,'[  umol/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_carbon_st       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_carbon_st                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CARBON_ST_PY         :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - CO2 storage at CAS'                            &
                           ,'[  umol/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_vapor_gc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_vapor_gc                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_VAPOR_GC_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Water vapour flux: ground -> CAS'              &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_vapor_ac        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_vapor_ac                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_VAPOR_AC_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Water vapour flux: atmosphere -> CAS'          &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_throughfall     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_throughfall                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_THROUGHFALL_PY       :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Throughfall rate'                              &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_runoff          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_runoff                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_RUNOFF_PY            :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Water runoff'                                  &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_drainage        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_drainage                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_DRAINAGE_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Water drainage'                                &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_sensible_gc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_sensible_gc                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SENSIBLE_GC_PY       :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Sensible heat flux: ground -> CAS'             &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_sensible_ac     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_sensible_ac                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SENSIBLE_AC_PY       :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Sensible heat flux: atmosphere -> CAS'         &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_qthroughfall    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_qthroughfall                                  &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_QTHROUGHFALL_PY      :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Throughfall rate'                              &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_qrunoff         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_qrunoff                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_QRUNOFF_PY           :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Surface runoff'                                &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_qdrainage       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_qdrainage                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_QDRAINAGE_PY         :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Drainage energy loss'                          &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_nppleaf         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_nppleaf                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_NPPLEAF_PY           :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Net primary productivity - Leaf'               &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_nppfroot        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_nppfroot                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_NPPFROOT_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Net primary productivity - Fine root'          &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_nppsapwood      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_nppsapwood                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_NPPSAPWOOD_PY        :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Net primary productivity - Sap wood'           &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_nppcroot        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_nppcroot                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_NPPCROOT_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Net primary productivity - Coarse root'        &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_nppseeds        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_nppseeds                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_NPPSEEDS_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Net primary productivity - seeds'              &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_nppwood         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_nppwood                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_NPPWOOD_PY           :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Net primary productivity - heart wood'         &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_nppdaily        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_nppdaily                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_NPPDAILY_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Net primary productivity - total'              &
                           ,'[  kgC/m2/yr]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_A_decomp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_A_decomp                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_A_DECOMP_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Decomposition scale'                           &
                           ,'[         --]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_Af_decomp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_Af_decomp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_AF_DECOMP_PY         :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Decomposition scale with N immobilisation'     &
                           ,'[         --]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_co2_residual    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_co2_residual                                  &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CO2_RESIDUAL_PY      :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Budget check - CO2 residual'                   &
                           ,'[    umol/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_energy_residual )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_energy_residual                               &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ENERGY_RESIDUAL_PY   :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Budget check - Enthalpy residual'              &
                           ,'[       J/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_water_residual  )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_water_residual                                &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_WATER_RESIDUAL_PY    :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Budget check - Water residual'                 &
                           ,'[      kg/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_atm_theiv       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_atm_theiv                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_THEIV_PY         :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Ice-vapour equiv. pot. temp.: Atmosphere'      &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_atm_theta       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_atm_theta                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_THETA_PY         :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Potential temperature: Atmosphere'             &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_atm_temp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_atm_temp                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_TEMP_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Temperature: Atmosphere'                       &
                           ,'[          K]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_atm_vpdef       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_atm_vpdef                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_VPDEF_PY         :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Vapour pressure deficit: Atmosphere'           &
                           ,'[         Pa]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_atm_shv         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_atm_shv                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_SHV_PY           :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Specific humidity: Atmosphere'                 &
                           ,'[      kg/kg]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_atm_rshort      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_atm_rshort                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_RSHORT_PY        :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Shortwave radiation: Atmosphere'               &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_atm_rshort_diff )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_atm_rshort_diff                               &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_RSHORT_DIFF_PY   :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Diffuse SW radiation: Atmosphere'              &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_atm_par         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_atm_par                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_PAR_PY           :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - PAR: Atmosphere'                               &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_atm_par_diff    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_atm_par_diff                                  &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_PAR_DIFF_PY      :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Diffuse PAR: Atmosphere'                       &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_atm_rlong       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_atm_rlong                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_RLONG_PY         :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Longwave radiation: Atmosphere'                &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_atm_vels        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_atm_vels                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_VELS_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Wind speed: atmosphere'                        &
                           ,'[        m/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_atm_rhos        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_atm_rhos                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_RHOS_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Air density: Atmosphere'                       &
                           ,'[      kg/m3]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_atm_prss        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_atm_prss                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_PRSS_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Air pressure: Atmosphere'                      &
                           ,'[         Pa]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_atm_co2         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_atm_co2                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_CO2_PY           :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - CO2 mixing ratio: Atmosphere'                  &
                           ,'[   umol/mol]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_pcpg            )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_pcpg                                          &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_PCPG_PY              :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Precipitation rate'                            &
                           ,'[    kg/m2/s]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_qpcpg           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_qpcpg                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_QPCPG_PY             :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Energy gain - rain'                            &
                           ,'[       W/m2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmean_dpcpg           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_dpcpg                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_DPCPG_PY             :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Precipitation depth'                           &
                           ,'[          m]','(ipoly)'            )
      end if
      if(associated(cgrid%mmean_fast_soil_c    )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%mmean_fast_soil_c                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_FAST_SOIL_C_PY     :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr,'Monthly mean - Soil Carbon (Fast pool)'              &
                           ,'[kgC/m2]','(ipoly)')
      end if
      if(associated(cgrid%mmean_slow_soil_c    )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%mmean_slow_soil_c                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SLOW_SOIL_C_PY     :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr,'Monthly mean - Soil Carbon (Slow pool)'              &
                           ,'[kgC/m2]','(ipoly)')
      end if
      if(associated(cgrid%mmean_struct_soil_c  )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%mmean_struct_soil_c                                 &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_STRUCT_SOIL_C_PY   :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr,'Monthly mean - Soil Carbon (Structural pool)'        &
                           ,'[kgC/m2]','(ipoly)')
      end if
      if(associated(cgrid%mmean_struct_soil_l  )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%mmean_struct_soil_l                                 &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_STRUCT_SOIL_L_PY   :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr,'Monthly mean - Soil Lignin (Structural pool)'        &
                           ,'[kgC/m2]','(ipoly)')
      end if
      if(associated(cgrid%mmean_cwd_c  )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%mmean_cwd_c                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CWD_C_PY   :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr,'Monthly mean - Coarse woody debris'                  &
                           ,'[kgC/m2]','(ipoly)')
      end if
      if(associated(cgrid%mmean_fast_soil_n    )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%mmean_fast_soil_n                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_FAST_SOIL_N_PY     :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr,'Monthly mean - Soil Nitrogen (Fast pool)'            &
                           ,'[kgN/m2]','(ipoly)')
      end if
      if(associated(cgrid%mmean_mineral_soil_n )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%mmean_mineral_soil_n                                &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_MINERAL_SOIL_N_PY  :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr,'Monthly mean - Soil Nitrogen (Mineralised pool)'     &
                           ,'[kgN/m2]','(ipoly)')
      end if
      if(associated(cgrid%mmean_cwd_n  )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%mmean_cwd_n                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CWD_N_PY   :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr,'Monthly mean - Coarse woody debris nitrogen'         &
                           ,'[kgN/m2]','(ipoly)')
      end if
      if (associated(cgrid%mmsqu_gpp             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmsqu_gpp                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_GPP_PY               :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Gross primary productivity'                    &
                           ,'[kgC2/m4/yr2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmsqu_npp             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmsqu_npp                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_NPP_PY               :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Net primary productivity'                      &
                           ,'[kgC2/m4/yr2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmsqu_plresp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmsqu_plresp                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_PLRESP_PY            :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Plant respiration'                             &
                           ,'[kgC2/m4/yr2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmsqu_sensible_lc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmsqu_sensible_lc                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_SENSIBLE_LC_PY       :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Sensible heat'                                 &
                           ,'[      W2/m4]','(ipoly)'            )
      end if
      if (associated(cgrid%mmsqu_vapor_lc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmsqu_vapor_lc                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_VAPOR_LC_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Leaf evaporation'                              &
                           ,'[  kg2/m4/s2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmsqu_transp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmsqu_transp                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_TRANSP_PY            :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Leaf transpiration'                            &
                           ,'[  kg2/m4/s2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmsqu_sensible_wc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmsqu_sensible_wc                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_SENSIBLE_WC_PY       :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Sensible heat'                                 &
                           ,'[      W2/m4]','(ipoly)'            )
      end if
      if (associated(cgrid%mmsqu_vapor_wc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmsqu_vapor_wc                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_VAPOR_WC_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Wood evaporation'                              &
                           ,'[  kg2/m4/s2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmsqu_rh              )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmsqu_rh                                            &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_RH_PY                :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Heterotrophic respiration'                     &
                           ,'[ kg2/m4/yr2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmsqu_cwd_rh          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmsqu_cwd_rh                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_CWD_RH_PY            :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Coarse woody debris respiration'               &
                           ,'[ kg2/m4/yr2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmsqu_nep             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmsqu_nep                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_NEP_PY               :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Net Ecosystem productivity'                    &
                           ,'[ kg2/m4/yr2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmsqu_rlongup         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmsqu_rlongup                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_RLONGUP_PY           :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Outgoing longwave radiation'                   &
                           ,'[      W2/m4]','(ipoly)'            )
      end if
      if (associated(cgrid%mmsqu_parup           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmsqu_parup                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_PARUP_PY             :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Outgoing PAR'                                  &
                           ,'[      W2/m4]','(ipoly)'            )
      end if
      if (associated(cgrid%mmsqu_nirup           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmsqu_nirup                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_NIRUP_PY             :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Outgoing near infra-red radiation'             &
                           ,'[      W2/m4]','(ipoly)'            )
      end if
      if (associated(cgrid%mmsqu_rshortup        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmsqu_rshortup                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_RSHORTUP_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Outgoing shortwave radiation'                  &
                           ,'[      W2/m4]','(ipoly)'            )
      end if
      if (associated(cgrid%mmsqu_rnet            )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmsqu_rnet                                          &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_RNET_PY              :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Net radiation at top of canopy'                &
                           ,'[      W2/m4]','(ipoly)'            )
      end if
      if (associated(cgrid%mmsqu_albedo          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmsqu_albedo                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_ALBEDO_PY            :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Albedo'                                        &
                           ,'[       ----]','(ipoly)'            )
      end if
      if (associated(cgrid%mmsqu_ustar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmsqu_ustar                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_USTAR_PY             :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Friction velocity'                             &
                           ,'[      m2/s2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmsqu_carbon_ac       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmsqu_carbon_ac                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_CARBON_AC_PY         :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - CO2 flux: atmosphere -> CAS'                   &
                           ,'[umol2/m4/s2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmsqu_carbon_st       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmsqu_carbon_st                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_CARBON_ST_PY         :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - CO2 storage at CAS'                            &
                           ,'[umol2/m4/s2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmsqu_vapor_gc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmsqu_vapor_gc                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_VAPOR_GC_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Water vapour flux: ground -> CAS'              &
                           ,'[  kg2/m4/s2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmsqu_vapor_ac        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmsqu_vapor_ac                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_VAPOR_AC_PY          :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Water vapour flux: atmosphere -> CAS'          &
                           ,'[  kg2/m4/s2]','(ipoly)'            )
      end if
      if (associated(cgrid%mmsqu_sensible_gc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmsqu_sensible_gc                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_SENSIBLE_GC_PY       :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Sensible heat flux: ground -> CAS'             &
                           ,'[      W2/m4]','(ipoly)'            )
      end if
      if (associated(cgrid%mmsqu_sensible_ac     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmsqu_sensible_ac                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_SENSIBLE_AC_PY       :11:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Sensible heat flux: atmosphere -> CAS'         &
                           ,'[      W2/m4]','(ipoly)'            )
      end if
      !------------------------------------------------------------------------------------!


      return
   end subroutine filltab_edtype_p11mmean
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the polygon-level variables         !
   ! (edtype) that have two dimensions (ndcycle,npolygons) and are real (type -11).        !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_edtype_m11(cgrid,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(edtype)          , target        :: cgrid
      integer               , intent(in)    :: init
      integer               , intent(in)    :: igr
      integer               , intent(in)    :: var_len
      integer               , intent(in)    :: max_ptrs
      integer               , intent(in)    :: var_len_global
      integer               , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                               :: npts
      character(len=str_len)                :: eorq_keys
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      !     Decide whether to save the mean diel variables to history or not.              !
      !------------------------------------------------------------------------------------!
      if (history_eorq) then
         eorq_keys = 'hist:dcyc'
      else
         eorq_keys = 'dcyc'
      end if
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !     This is the 2-D block, with diurnal cycle.  All variables must have the number !
      ! of points defined by npts.                                                         !
      !------------------------------------------------------------------------------------!
      npts = cgrid%npolygons * ndcycle
      if (associated(cgrid%qmean_gpp             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_gpp                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_GPP_PY              :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Gross primary productivity'                       &
                           ,'[  kgC/m2/yr]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_npp             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_npp                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_NPP_PY              :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Net primary productivity'                         &
                           ,'[  kgC/m2/yr]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_leaf_resp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_leaf_resp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_LEAF_RESP_PY        :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf respiration'                                 &
                           ,'[  kgC/m2/yr]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_root_resp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_root_resp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ROOT_RESP_PY        :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Root respiration'                                 &
                           ,'[  kgC/m2/yr]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_leaf_growth_resp )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_leaf_growth_resp                              &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_LEAF_GROWTH_RESP_PY :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf Growth respiration'                          &
                           ,'[  kgC/m2/yr]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_root_growth_resp )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_root_growth_resp                              &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ROOT_GROWTH_RESP_PY :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Root Growth respiration'                          &
                           ,'[  kgC/m2/yr]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_sapa_growth_resp )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_sapa_growth_resp                              &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SAPA_GROWTH_RESP_PY :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Aboveground Sapwood Growth respiration'           &
                           ,'[  kgC/m2/yr]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_sapb_growth_resp )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_sapb_growth_resp                              &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SAPB_GROWTH_RESP_PY :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Belowground Sapwood Growth respiration'           &
                           ,'[  kgC/m2/yr]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_leaf_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_leaf_storage_resp                             &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_LEAF_STORAGE_RESP_PY     :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf storage respiration'                         &
                           ,'[  kgC/m2/yr]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_root_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_root_storage_resp                             &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ROOT_STORAGE_RESP_PY     :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Root Storage respiration'                         &
                           ,'[  kgC/m2/yr]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_sapa_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_sapa_storage_resp                             &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SAPA_STORAGE_RESP_PY     :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Aboveground Sapwood Storage respiration'          &
                           ,'[  kgC/m2/yr]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_sapb_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_sapb_storage_resp                             &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SAPB_STORAGE_RESP_PY     :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Belowground Sapwood Storage respiration'          &
                           ,'[  kgC/m2/yr]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_plresp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_plresp                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_PLRESP_PY           :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Plant respiration'                                &
                           ,'[  kgC/m2/yr]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_leaf_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_leaf_energy                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_LEAF_ENERGY_PY      :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf internal energy'                             &
                           ,'[       J/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_leaf_water      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_leaf_water                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_LEAF_WATER_PY       :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf surface water'                               &
                           ,'[      kg/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_leaf_hcap       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_leaf_hcap                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_LEAF_HCAP_PY        :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf heat capacity'                               &
                           ,'[     J/m2/K]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_leaf_vpdef      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_leaf_vpdef                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_LEAF_VPDEF_PY       :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf vapour pressure deficit'                     &
                           ,'[         Pa]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_leaf_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_leaf_temp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_LEAF_TEMP_PY        :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf temperature'                                 &
                           ,'[          K]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_leaf_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_leaf_fliq                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_LEAF_FLIQ_PY        :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Liquid fraction'                                  &
                           ,'[         --]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_leaf_gsw        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_leaf_gsw                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_LEAF_GSW_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Stomatal conductance'                             &
                           ,'[kg/m2leaf/s]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_leaf_gbw        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_leaf_gbw                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_LEAF_GBW_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf boundary layer conductance'                  &
                           ,'[kg/m2leaf/s]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_wood_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_wood_energy                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_WOOD_ENERGY_PY      :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Wood internal energy'                             &
                           ,'[       J/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_wood_water      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_wood_water                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_WOOD_WATER_PY       :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Wood surface water'                               &
                           ,'[      kg/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_wood_hcap       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_wood_hcap                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_WOOD_HCAP_PY        :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Wood heat capacity'                               &
                           ,'[     J/m2/K]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_wood_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_wood_temp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_WOOD_TEMP_PY        :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Wood temperature'                                 &
                           ,'[          K]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_wood_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_wood_fliq                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_WOOD_FLIQ_PY        :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Liquid fraction'                                  &
                           ,'[         --]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_wood_gbw        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_wood_gbw                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_WOOD_GBW_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Wood boundary layer conductance'                  &
                           ,'[kg/m2wood/s]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_fs_open         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_fs_open                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_FS_OPEN_PY          :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Net stress factor'                                &
                           ,'[         --]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_fsw             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_fsw                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_FSW_PY              :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Moisture stress'                                  &
                           ,'[         --]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_fsn             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_fsn                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_FSN_PY              :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Nitrogen stress'                                  &
                           ,'[         --]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_A_open          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_A_open                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_A_OPEN_PY          :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Assimilation rate (no soil moist. stress)'        &
                           ,'[umol/m2/s]','(ndcycle,ipoly)'      )
      end if
      if (associated(cgrid%qmean_A_closed        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_A_closed                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_A_CLOSED_PY        :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Minimum assimilation rate'                        &
                           ,'[umol/m2/s]','(ndcycle,ipoly)'      )
      end if
      if (associated(cgrid%qmean_A_net           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_A_net                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_A_NET_PY          :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Actual assimilation rate'                         &
                           ,'[umol/m2/s]','(ndcycle,ipoly)'      )
      end if
      if (associated(cgrid%qmean_A_light         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_A_light                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_A_LIGHT_PY          :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Light-limited assimilation rate'                  &
                           ,'[umol/m2/s]','(ndcycle,ipoly)'            )
      end if
      if (associated(cgrid%qmean_A_rubp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_A_rubp                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_A_RUBP_PY          :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - RuBP-limited assimilation rate'                   &
                           ,'[umol/m2/s]','(ndcycle,ipoly)'            )
      end if
      if (associated(cgrid%qmean_A_co2           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_A_co2                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_A_CO2_PY          :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - CO2-limited assimilation rate'                    &
                           ,'[umol/m2/s]','(ndcycle,ipoly)'            )
      end if
      if (associated(cgrid%qmean_psi_open        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_psi_open                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_PSI_OPEN_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Transpiration with no stress'                     &
                           ,'[    kg/m2/s]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_psi_closed      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_psi_closed                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_PSI_CLOSED_PY       :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Transpiration at maximum stress'                  &
                           ,'[    kg/m2/s]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_water_supply    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_water_supply                                  &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_WATER_SUPPLY_PY     :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Water supply'                                     &
                           ,'[    kg/m2/s]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_par_l           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_par_l                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_PAR_L_PY            :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - PAR absorbed by leaves'                           &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_par_l_beam      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_par_l_beam                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_PAR_L_BEAM_PY       :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Direct PAR absorbed by leaves'                    &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_par_l_diff      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_par_l_diff                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_PAR_L_DIFF_PY       :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Diffuse PAR absorbed by leaves'                   &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_rshort_l        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_rshort_l                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_RSHORT_L_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Shortwave radiation absorbed by leaves'           &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_rlong_l         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_rlong_l                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_RLONG_L_PY          :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Longwave radiation absorbed by leaves'            &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_sensible_lc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_sensible_lc                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SENSIBLE_LC_PY      :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Sensible heat'                                    &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_vapor_lc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_vapor_lc                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_VAPOR_LC_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf evaporation'                                 &
                           ,'[    kg/m2/s]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_transp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_transp                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_TRANSP_PY           :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf transpiration'                               &
                           ,'[    kg/m2/s]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_intercepted_al  )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_intercepted_al                                &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_INTERCEPTED_AL_PY   :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf interception'                                &
                           ,'[    kg/m2/s]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_wshed_lg        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_wshed_lg                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_WSHED_LG_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf shedding'                                    &
                           ,'[    kg/m2/s]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_rshort_w        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_rshort_w                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_RSHORT_W_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Shortwave radiation absorbed by wood'             &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_rlong_w         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_rlong_w                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_RLONG_W_PY          :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Longwave radiation absorbed by wood'              &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_sensible_wc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_sensible_wc                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SENSIBLE_WC_PY      :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Sensible heat'                                    &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_vapor_wc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_vapor_wc                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_VAPOR_WC_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Wood evaporation'                                 &
                           ,'[    kg/m2/s]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_intercepted_aw  )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_intercepted_aw                                &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_INTERCEPTED_AW_PY   :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Wood interception'                                &
                           ,'[    kg/m2/s]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_wshed_wg        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_wshed_wg                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_WSHED_WG_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Wood shedding'                                    &
                           ,'[    kg/m2/s]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_rh              )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_rh                                            &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_RH_PY               :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Heterotrophic respiration'                        &
                           ,'[   kg/m2/yr]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_cwd_rh          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_cwd_rh                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CWD_RH_PY           :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Coarse woody debris respiration'                  &
                           ,'[   kg/m2/yr]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_nep             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_nep                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_NEP_PY              :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Net Ecosystem productivity'                       &
                           ,'[   kg/m2/yr]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_rk4step         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_rk4step                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_RK4STEP_PY          :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - RK4 time step'                                    &
                           ,'[          s]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_available_water )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_available_water                               &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_AVAILABLE_WATER_PY  :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Available water'                                  &
                           ,'[      kg/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_can_theiv       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_can_theiv                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CAN_THEIV_PY        :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Ice-vapour equiv. pot. temperature - CAS'         &
                           ,'[          K]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_can_theta       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_can_theta                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CAN_THETA_PY        :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Potential temperature - CAS'                      &
                           ,'[          K]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_can_vpdef       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_can_vpdef                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CAN_VPDEF_PY        :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Vapour pressure deficit - CAS'                    &
                           ,'[         Pa]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_can_temp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_can_temp                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CAN_TEMP_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Temperature - CAS'                                &
                           ,'[          K]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_can_shv         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_can_shv                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CAN_SHV_PY          :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Specific humidity - CAS'                          &
                           ,'[      kg/kg]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_can_co2         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_can_co2                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CAN_CO2_PY          :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - CO2 mixing ratio - CAS'                           &
                           ,'[   umol/mol]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_can_rhos        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_can_rhos                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CAN_RHOS_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Air density - CAS'                                &
                           ,'[      kg/m3]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_can_prss        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_can_prss                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CAN_PRSS_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Pressure - CAS'                                   &
                           ,'[         Pa]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_gnd_temp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_gnd_temp                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_GND_TEMP_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Ground temperature'                               &
                           ,'[          K]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_gnd_shv         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_gnd_shv                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_GND_SHV_PY          :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Ground specific humidity'                         &
                           ,'[      kg/kg]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_can_ggnd        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_can_ggnd                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CAN_GGND_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Net canopy conductance'                           &
                           ,'[        m/2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_sfcw_depth      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_sfcw_depth                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SFCW_DEPTH_PY       :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Depth - temporary water layer'                    &
                           ,'[          m]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_sfcw_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_sfcw_energy                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SFCW_ENERGY_PY      :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Internal energy - temporary water layer'          &
                           ,'[       J/kg]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_sfcw_mass       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_sfcw_mass                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SFCW_MASS_PY        :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Water mass - temporary water layer'               &
                           ,'[      kg/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_sfcw_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_sfcw_temp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SFCW_TEMP_PY        :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Temperature - temporary water layer'              &
                           ,'[          K]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_sfcw_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_sfcw_fliq                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SFCW_FLIQ_PY        :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Liquid fraction - temporary water layer'          &
                           ,'[         --]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_rshort_gnd      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_rshort_gnd                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_RSHORT_GND_PY       :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Shortwave radiation absorbed by ground'           &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_par_gnd         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_par_gnd                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_PAR_GND_PY          :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - PAR absorbed by ground'                           &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_rlong_gnd       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_rlong_gnd                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_RLONG_GND_PY        :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Longwave radiation absorbed by ground'            &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_rlongup         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_rlongup                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_RLONGUP_PY          :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Outgoing longwave radiation'                      &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_parup           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_parup                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_PARUP_PY            :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Outgoing PAR'                                     &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_nirup           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_nirup                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_NIRUP_PY            :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Outgoing near infra-red radiation'                &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_rshortup        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_rshortup                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_RSHORTUP_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Outgoing shortwave radiation'                     &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_rnet            )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_rnet                                          &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_RNET_PY             :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Net radiation at top of canopy'                   &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_albedo          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_albedo                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ALBEDO_PY           :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Albedo'                                           &
                           ,'[       ----]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_albedo_par      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_albedo_par                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ALBEDO_PAR_PY       :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Albedo - direct radiation'                        &
                           ,'[       ----]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_albedo_nir      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_albedo_nir                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ALBEDO_NIR_PY       :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Albedo - diffuse radiation'                       &
                           ,'[       ----]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_rlong_albedo    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_rlong_albedo                                  &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_RLONG_ALBEDO_PY     :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Kind of albedo - longwave radiation'              &
                           ,'[       ----]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_ustar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_ustar                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_USTAR_PY            :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Friction velocity'                                &
                           ,'[        m/s]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_tstar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_tstar                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_TSTAR_PY            :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Gradient scale for potential temperature'         &
                           ,'[          K]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_qstar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_qstar                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_QSTAR_PY            :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Gradient scale for specific humidity'             &
                           ,'[      kg/kg]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_cstar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_cstar                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CSTAR_PY            :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Gradient scale for CO2 mixing ratio'              &
                           ,'[   umol/mol]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_carbon_ac       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_carbon_ac                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CARBON_AC_PY        :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - CO2 flux: atmosphere -> CAS'                      &
                           ,'[  umol/m2/s]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_carbon_st       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_carbon_st                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CARBON_ST_PY        :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - CO2 storage at CAS'                               &
                           ,'[  umol/m2/s]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_vapor_gc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_vapor_gc                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_VAPOR_GC_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Water vapour flux: ground -> CAS'                 &
                           ,'[    kg/m2/s]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_vapor_ac        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_vapor_ac                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_VAPOR_AC_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Water vapour flux: atmosphere -> CAS'             &
                           ,'[    kg/m2/s]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_throughfall     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_throughfall                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_THROUGHFALL_PY      :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Throughfall rate'                                 &
                           ,'[    kg/m2/s]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_runoff          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_runoff                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_RUNOFF_PY           :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Water runoff'                                     &
                           ,'[    kg/m2/s]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_drainage        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_drainage                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_DRAINAGE_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Water drainage'                                   &
                           ,'[    kg/m2/s]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_sensible_gc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_sensible_gc                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SENSIBLE_GC_PY      :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Sensible heat flux: ground -> CAS'                &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_sensible_ac     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_sensible_ac                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SENSIBLE_AC_PY      :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Sensible heat flux: atmosphere -> CAS'            &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_qthroughfall    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_qthroughfall                                  &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_QTHROUGHFALL_PY     :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Throughfall rate'                                 &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_qrunoff         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_qrunoff                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_QRUNOFF_PY          :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Surface runoff'                                   &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_qdrainage       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_qdrainage                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_QDRAINAGE_PY        :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Drainage energy loss'                             &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_atm_theiv       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_atm_theiv                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_THEIV_PY        :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Ice-vapour equiv. pot. temp.: Atmosphere'         &
                           ,'[          K]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_atm_theta       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_atm_theta                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_THETA_PY        :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Potential temperature: Atmosphere'                &
                           ,'[          K]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_atm_temp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_atm_temp                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_TEMP_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Temperature: Atmosphere'                          &
                           ,'[          K]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_atm_vpdef       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_atm_vpdef                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_VPDEF_PY        :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Vapour pressure deficit: Atmosphere'              &
                           ,'[         Pa]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_atm_shv         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_atm_shv                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_SHV_PY          :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Specific humidity: Atmosphere'                    &
                           ,'[      kg/kg]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_atm_rshort      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_atm_rshort                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_RSHORT_PY       :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Shortwave radiation: Atmosphere'                  &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_atm_rshort_diff )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_atm_rshort_diff                               &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_RSHORT_DIFF_PY  :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Diffuse SW radiation: Atmosphere'                 &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_atm_par         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_atm_par                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_PAR_PY          :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - PAR: Atmosphere'                                  &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_atm_par_diff    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_atm_par_diff                                  &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_PAR_DIFF_PY     :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Diffuse PAR: Atmosphere'                          &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_atm_rlong       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_atm_rlong                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_RLONG_PY        :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Longwave radiation: Atmosphere'                   &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_atm_vels        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_atm_vels                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_VELS_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Wind speed: atmosphere'                           &
                           ,'[        m/s]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_atm_rhos        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_atm_rhos                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_RHOS_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Air density: Atmosphere'                          &
                           ,'[      kg/m3]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_atm_prss        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_atm_prss                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_PRSS_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Air pressure: Atmosphere'                         &
                           ,'[         Pa]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_atm_co2         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_atm_co2                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_CO2_PY          :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - CO2 mixing ratio: Atmosphere'                     &
                           ,'[   umol/mol]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_pcpg            )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_pcpg                                          &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_PCPG_PY             :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Precipitation rate'                               &
                           ,'[    kg/m2/s]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_qpcpg           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_qpcpg                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_QPCPG_PY            :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Energy gain - rain'                               &
                           ,'[       W/m2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmean_dpcpg           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_dpcpg                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_DPCPG_PY            :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Precipitation depth'                              &
                           ,'[          m]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmsqu_gpp             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmsqu_gpp                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_GPP_PY              :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Gross primary productivity'                       &
                           ,'[kgC2/m4/yr2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmsqu_npp             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmsqu_npp                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_NPP_PY              :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Net primary productivity'                         &
                           ,'[kgC2/m4/yr2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmsqu_plresp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmsqu_plresp                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_PLRESP_PY           :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Plant respiration'                                &
                           ,'[kgC2/m4/yr2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmsqu_sensible_lc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmsqu_sensible_lc                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_SENSIBLE_LC_PY      :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Sensible heat'                                    &
                           ,'[      W2/m4]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmsqu_vapor_lc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmsqu_vapor_lc                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_VAPOR_LC_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Leaf evaporation'                                 &
                           ,'[  kg2/m4/s2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmsqu_transp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmsqu_transp                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_TRANSP_PY           :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Leaf transpiration'                               &
                           ,'[  kg2/m4/s2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmsqu_sensible_wc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmsqu_sensible_wc                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_SENSIBLE_WC_PY      :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Sensible heat'                                    &
                           ,'[      W2/m4]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmsqu_vapor_wc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmsqu_vapor_wc                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_VAPOR_WC_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Wood evaporation'                                 &
                           ,'[  kg2/m4/s2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmsqu_rh              )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmsqu_rh                                            &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_RH_PY               :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Heterotrophic respiration'                        &
                           ,'[ kg2/m4/yr2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmsqu_cwd_rh          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmsqu_cwd_rh                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_CWD_RH_PY           :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Coarse woody debris respiration'                  &
                           ,'[ kg2/m4/yr2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmsqu_nep             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmsqu_nep                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_NEP_PY              :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Net Ecosystem productivity'                       &
                           ,'[ kg2/m4/yr2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmsqu_rlongup         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmsqu_rlongup                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_RLONGUP_PY          :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Outgoing longwave radiation'                      &
                           ,'[      W2/m4]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmsqu_parup           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmsqu_parup                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_PARUP_PY            :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Outgoing PAR'                                     &
                           ,'[      W2/m4]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmsqu_nirup           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmsqu_nirup                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_NIRUP_PY            :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Outgoing near infra-red radiation'                &
                           ,'[      W2/m4]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmsqu_rshortup        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmsqu_rshortup                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_RSHORTUP_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Outgoing shortwave radiation'                     &
                           ,'[      W2/m4]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmsqu_rnet            )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmsqu_rnet                                          &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_RNET_PY             :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Net radiation at top of canopy'                   &
                           ,'[      W2/m4]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmsqu_albedo          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmsqu_albedo                                        &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_ALBEDO_PY           :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Albedo'                                           &
                           ,'[       ----]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmsqu_ustar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmsqu_ustar                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_USTAR_PY            :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Friction velocity'                                &
                           ,'[      m2/s2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmsqu_carbon_ac       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmsqu_carbon_ac                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_CARBON_AC_PY        :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - CO2 flux: atmosphere -> CAS'                      &
                           ,'[umol2/m4/s2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmsqu_carbon_st       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmsqu_carbon_st                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_CARBON_ST_PY        :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - CO2 storage at CAS'                               &
                           ,'[umol2/m4/s2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmsqu_vapor_gc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmsqu_vapor_gc                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_VAPOR_GC_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Water vapour flux: ground -> CAS'                 &
                           ,'[  kg2/m4/s2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmsqu_vapor_ac        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmsqu_vapor_ac                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_VAPOR_AC_PY         :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Water vapour flux: atmosphere -> CAS'             &
                           ,'[  kg2/m4/s2]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmsqu_sensible_gc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmsqu_sensible_gc                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_SENSIBLE_GC_PY      :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Sensible heat flux: ground -> CAS'                &
                           ,'[      W2/m4]','(ndcycle,ipoly)'    )
      end if
      if (associated(cgrid%qmsqu_sensible_ac     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmsqu_sensible_ac                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_SENSIBLE_AC_PY      :-11:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Sensible heat flux: atmosphere -> CAS'            &
                           ,'[      W2/m4]','(ndcycle,ipoly)'    )
      end if
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!


      return     
   end subroutine filltab_edtype_m11
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the polygon-level variables         !
   ! (edtype) that have two dimensions (ndcycle,npolygons) and are real (type 12).         !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_edtype_p12(cgrid,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(edtype)          , target        :: cgrid
      integer               , intent(in)    :: init
      integer               , intent(in)    :: igr
      integer               , intent(in)    :: var_len
      integer               , intent(in)    :: max_ptrs
      integer               , intent(in)    :: var_len_global
      integer               , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                               :: npts
      character(len=str_len)                :: fast_keys
      character(len=str_len)                :: dail_keys
      character(len=str_len)                :: eorq_keys
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !      Decide whether to write fast, daily, and monthly mean variables to history.   !
      !------------------------------------------------------------------------------------!
      if (history_fast) then
         fast_keys = 'hist:anal:opti'
      else
         fast_keys = 'anal:opti'
      end if
      if (history_dail) then
         dail_keys = 'hist:dail'
      else
         dail_keys = 'dail'
      end if
      if (history_eorq) then
         eorq_keys = 'hist:mont:dcyc'
      else
         eorq_keys = 'mont:dcyc'
      end if
      !------------------------------------------------------------------------------------!




      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !     This is the 2-D block, with soil layers.  All variables must have the number   !
      ! of points defined by npts.                                                         !
      !------------------------------------------------------------------------------------!
      npts = cgrid%npolygons * nzg
      if (associated(cgrid%fmean_soil_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_soil_energy                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SOIL_ENERGY_PY       :12:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Soil internal energy'                        &
                           ,'[       J/m3]','(nzg,ipoly)'        )
      end if
      if (associated(cgrid%fmean_soil_mstpot     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_soil_mstpot                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SOIL_MSTPOT_PY       :12:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Soil matric potential'                       &
                           ,'[          m]','(nzg,ipoly)'        )
      end if
      if (associated(cgrid%fmean_soil_water      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_soil_water                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SOIL_WATER_PY        :12:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Soil water content'                          &
                           ,'[      m3/m3]','(nzg,ipoly)'        )
      end if
      if (associated(cgrid%fmean_soil_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_soil_temp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SOIL_TEMP_PY         :12:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Soil temperature'                            &
                           ,'[          K]','(nzg,ipoly)'        )
      end if
      if (associated(cgrid%fmean_soil_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_soil_fliq                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SOIL_FLIQ_PY         :12:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Soil liquid fraction'                        &
                           ,'[         --]','(nzg,ipoly)'        )
      end if
      if (associated(cgrid%fmean_smoist_gg       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_smoist_gg                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SMOIST_GG_PY         :12:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Soil water flux'                             &
                           ,'[    kg/m2/s]','(nzg,ipoly)'        )
      end if
      if (associated(cgrid%fmean_transloss       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_transloss                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_TRANSLOSS_PY         :12:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Water loss through transpiration'            &
                           ,'[    kg/m2/s]','(nzg,ipoly)'        )
      end if
      if (associated(cgrid%fmean_sensible_gg     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%fmean_sensible_gg                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SENSIBLE_GG_PY       :12:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Soil heat flux'                              &
                           ,'[       W/m2]','(nzg,ipoly)'        )
      end if
      if (associated(cgrid%dmean_soil_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_soil_energy                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SOIL_ENERGY_PY       :12:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Soil internal energy'                            &
                           ,'[       J/m3]','(nzg,ipoly)'        )
      end if
      if (associated(cgrid%dmean_soil_mstpot     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_soil_mstpot                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SOIL_MSTPOT_PY       :12:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Soil matric potential'                           &
                           ,'[          m]','(nzg,ipoly)'        )
      end if
      if (associated(cgrid%dmean_soil_water      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_soil_water                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SOIL_WATER_PY        :12:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Soil water content'                              &
                           ,'[      m3/m3]','(nzg,ipoly)'        )
      end if
      if (associated(cgrid%dmean_soil_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_soil_temp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SOIL_TEMP_PY         :12:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Soil temperature'                                &
                           ,'[          K]','(nzg,ipoly)'        )
      end if
      if (associated(cgrid%dmean_soil_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_soil_fliq                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SOIL_FLIQ_PY         :12:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Soil liquid fraction'                            &
                           ,'[         --]','(nzg,ipoly)'        )
      end if
      if (associated(cgrid%dmean_smoist_gg       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_smoist_gg                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SMOIST_GG_PY         :12:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Soil water flux'                                 &
                           ,'[    kg/m2/s]','(nzg,ipoly)'        )
      end if
      if (associated(cgrid%dmean_transloss       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_transloss                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_TRANSLOSS_PY         :12:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Water loss through transpiration'                &
                           ,'[    kg/m2/s]','(nzg,ipoly)'        )
      end if
      if (associated(cgrid%dmean_sensible_gg     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%dmean_sensible_gg                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SENSIBLE_GG_PY       :12:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Soil heat flux'                                  &
                           ,'[       W/m2]','(nzg,ipoly)'        )
      end if
      if (associated(cgrid%mmean_soil_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_soil_energy                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SOIL_ENERGY_PY       :12:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Soil internal energy'                          &
                           ,'[       J/m3]','(nzg,ipoly)'        )
      end if
      if (associated(cgrid%mmean_soil_mstpot     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_soil_mstpot                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SOIL_MSTPOT_PY       :12:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Soil matric potential'                         &
                           ,'[          m]','(nzg,ipoly)'        )
      end if
      if (associated(cgrid%mmean_soil_water      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_soil_water                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SOIL_WATER_PY        :12:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Soil water content'                            &
                           ,'[      m3/m3]','(nzg,ipoly)'        )
      end if
      if (associated(cgrid%mmean_soil_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_soil_temp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SOIL_TEMP_PY         :12:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Soil temperature'                              &
                           ,'[          K]','(nzg,ipoly)'        )
      end if
      if (associated(cgrid%mmean_soil_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_soil_fliq                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SOIL_FLIQ_PY         :12:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Soil liquid fraction'                          &
                           ,'[         --]','(nzg,ipoly)'        )
      end if
      if (associated(cgrid%mmean_smoist_gg       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_smoist_gg                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SMOIST_GG_PY         :12:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Soil water flux'                               &
                           ,'[    kg/m2/s]','(nzg,ipoly)'        )
      end if
      if (associated(cgrid%mmean_transloss       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_transloss                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_TRANSLOSS_PY         :12:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Water loss through transpiration'              &
                           ,'[    kg/m2/s]','(nzg,ipoly)'        )
      end if
      if (associated(cgrid%mmean_sensible_gg     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%mmean_sensible_gg                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SENSIBLE_GG_PY       :12:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Soil heat flux'                                &
                           ,'[       W/m2]','(nzg,ipoly)'        )
      end if
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!



      return     
   end subroutine filltab_edtype_p12
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the polygon-level variables         !
   ! (edtype) that have three dimensions (nzg,ndcycle,npolygons) and are real (type -12).  !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_edtype_m12(cgrid,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(edtype)          , target        :: cgrid
      integer               , intent(in)    :: init
      integer               , intent(in)    :: igr
      integer               , intent(in)    :: var_len
      integer               , intent(in)    :: max_ptrs
      integer               , intent(in)    :: var_len_global
      integer               , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                               :: npts
      character(len=str_len)                :: eorq_keys
      !------------------------------------------------------------------------------------!




      !------------------------------------------------------------------------------------!
      !      Decide whether to write the mean diel variables to the history.               !
      !------------------------------------------------------------------------------------!
      if (history_eorq) then
         eorq_keys = 'hist:dcyc'
      else
         eorq_keys = 'dcyc'
      end if
      !------------------------------------------------------------------------------------!




      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !     This is the 3-D block, with soil depth class and diurnal cycle.  All vari-     !
      ! ables must have the number of points defined by npts.                              !
      !------------------------------------------------------------------------------------!
      npts = cgrid%npolygons *  nzg * ndcycle
      if (associated(cgrid%qmean_soil_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_soil_energy                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SOIL_ENERGY_PY      :-12:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Soil internal energy'                             &
                           ,'[       J/m3]','(nzg,ndcycle,ipoly)')
      end if
      if (associated(cgrid%qmean_soil_mstpot     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_soil_mstpot                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SOIL_MSTPOT_PY      :-12:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Soil matric potential'                            &
                           ,'[          m]','(nzg,ndcycle,ipoly)')
      end if
      if (associated(cgrid%qmean_soil_water      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_soil_water                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SOIL_WATER_PY       :-12:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Soil water content'                               &
                           ,'[      m3/m3]','(nzg,ndcycle,ipoly)')
      end if
      if (associated(cgrid%qmean_soil_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_soil_temp                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SOIL_TEMP_PY        :-12:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Soil temperature'                                 &
                           ,'[          K]','(nzg,ndcycle,ipoly)')
      end if
      if (associated(cgrid%qmean_soil_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_soil_fliq                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SOIL_FLIQ_PY        :-12:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Soil liquid fraction'                             &
                           ,'[         --]','(nzg,ndcycle,ipoly)')
      end if
      if (associated(cgrid%qmean_smoist_gg       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_smoist_gg                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SMOIST_GG_PY        :-12:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Soil water flux'                                  &
                           ,'[    kg/m2/s]','(nzg,ndcycle,ipoly)')
      end if
      if (associated(cgrid%qmean_transloss       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_transloss                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_TRANSLOSS_PY        :-12:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Water loss through transpiration'                 &
                           ,'[    kg/m2/s]','(nzg,ndcycle,ipoly)')
      end if
      if (associated(cgrid%qmean_sensible_gg     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cgrid%qmean_sensible_gg                                   &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SENSIBLE_GG_PY      :-12:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Soil heat flux'                                   &
                           ,'[       W/m2]','(nzg,ndcycle,ipoly)')
      end if
      return     
   end subroutine filltab_edtype_m12
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the polygon-level variables         !
   ! (edtype) that have two dimensions (13 months,npolygons) and are real (type 19).       !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_edtype_p19(cgrid,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(edtype), target        :: cgrid
      integer     , intent(in)    :: init
      integer     , intent(in)    :: igr
      integer     , intent(in)    :: var_len
      integer     , intent(in)    :: max_ptrs
      integer     , intent(in)    :: var_len_global
      integer     , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                     :: npts
      !------------------------------------------------------------------------------------!

      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !     This is the 2-D block, with 13 months.  All variables must have the            !
      ! number of points defined by npts.                                                  !
      !------------------------------------------------------------------------------------!
      npts = cgrid%npolygons * 13

      if(associated(cgrid%workload)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%workload                                            &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'WORKLOAD :19:hist:mont:dcyc') 
         call metadata_edio(nvar,igr,'Polygon computational workload'                      &
                           ,'[steps/month]','(13,ipoly)') 
      end if
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!


      return     
   end subroutine filltab_edtype_p19
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the polygon-level variables         !
   ! (edtype) that have three dimensions (n_dbh,n_pft,npolygons) and are real (type 146).  !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_edtype_p146(cgrid,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(edtype)          , target        :: cgrid
      integer               , intent(in)    :: init
      integer               , intent(in)    :: igr
      integer               , intent(in)    :: var_len
      integer               , intent(in)    :: max_ptrs
      integer               , intent(in)    :: var_len_global
      integer               , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                               :: npts
      character(len=str_len)                :: eorq_keys
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !     Decide whether to write monthly means to history or not.                       !
      !------------------------------------------------------------------------------------!
      if (history_eorq) then
         eorq_keys = 'hist:mont:dcyc'
      else
         eorq_keys = 'mont:dcyc'
      end if
      !------------------------------------------------------------------------------------!

      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !     This is the 3-D block, with PFT and size class.  All variables must have the   !
      ! number of points defined by npts.                                                  !
      !------------------------------------------------------------------------------------!
      npts = cgrid%npolygons * n_pft * n_dbh
      if (associated(cgrid%nplant     )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%nplant                                              &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'NPLANT_PY      :146:hist:anal:mont:dail:dcyc')
         call metadata_edio(nvar,igr,'Plant density'                                       &
                           ,'[plants/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%agb        )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%agb                                                 &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'AGB_PY         :146:hist:anal:mont:dail:dcyc')
         call metadata_edio(nvar,igr,'Above-ground biomass'                                &
                           ,   '[kgC/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%lai        )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%lai                                                 &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'LAI_PY         :146:hist:anal:dail:opti')
         call metadata_edio(nvar,igr,'Leaf area index'                                     &
                           ,'[m2leaf/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%wai        )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%wai                                                 &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'WAI_PY         :146:hist:anal:mont:dail:dcyc')
         call metadata_edio(nvar,igr,'Wood area index'                                     &
                           ,'[m2wood/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%basal_area )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%basal_area                                          &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'BASAL_AREA_PY  :146:hist:anal:mont:dail:dcyc')
         call metadata_edio(nvar,igr,'Basal area'                                          &
                           ,   '[cm2/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%bdead      )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%bdead                                               &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'BDEAD_PY       :146:hist:anal:mont:dail:dcyc')
         call metadata_edio(nvar,igr,'Biomass of structural (dead) tissues'                &
                           ,   '[kgC/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%balive     )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%balive                                              &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'BALIVE_PY      :146:hist:anal:mont:dail:dcyc')
         call metadata_edio(nvar,igr,'Biomass of active (living) tissues'                  &
                           ,   '[kgC/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%bleaf      )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%bleaf                                               &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'BLEAF_PY       :146:hist:anal:dail')
         call metadata_edio(nvar,igr,'Leaf biomass'                                        &
                           ,   '[kgC/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%broot      )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%broot                                               &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'BROOT_PY       :146:hist:anal:dail')
         call metadata_edio(nvar,igr,'Root biomass'                                        &
                           ,   '[kgC/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%bsapwooda  )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%bsapwooda                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'BSAPWOODA_PY   :146:hist:anal:mont:dail:dcyc')
         call metadata_edio(nvar,igr,'Above-ground sapwood biomass'                        &
                           ,   '[kgC/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%bsapwoodb  )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%bsapwoodb                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'BSAPWOODB_PY   :146:hist:anal:mont:dail:dcyc')
         call metadata_edio(nvar,igr,'Below-ground sapwood biomass'                        &
                           ,   '[kgC/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%bseeds     )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%bseeds                                              &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'BSEEDS_PY      :146:hist:anal:mont:dail:dcyc')
         call metadata_edio(nvar,igr,'Seed biomass'                                        &
                           ,   '[kgC/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%bstorage   )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%bstorage                                            &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'BSTORAGE_PY    :146:hist:anal:dail')
         call metadata_edio(nvar,igr,'Storage biomass'                                     &
                           ,   '[kgC/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%bdead_n      )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%bdead_n                                             &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'BDEAD_N_PY     :146:hist:anal:mont:dail:dcyc')
         call metadata_edio(nvar,igr,'Nitrogen mass of structural (dead) tissues'          &
                           ,   '[kgN/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%balive_n   )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%balive_n                                            &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'BALIVE_N_PY    :146:hist:anal:mont:dail:dcyc')
         call metadata_edio(nvar,igr,'Nitrogen mass of active (living) tissues'            &
                           ,   '[kgN/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%bleaf_n    )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%bleaf_n                                             &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'BLEAF_N_PY     :146:hist:anal:dail')
         call metadata_edio(nvar,igr,'Leaf nitrogen mass'                                  &
                           ,   '[kgN/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%broot_n    )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%broot_n                                             &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'BROOT_N_PY     :146:hist:anal:dail')
         call metadata_edio(nvar,igr,'Root nitrogen mass'                                  &
                           ,   '[kgN/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%bsapwooda_n)) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%bsapwooda_n                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'BSAPWOODA_N_PY :146:hist:anal:mont:dail:dcyc')
         call metadata_edio(nvar,igr,'Above-ground sapwood nitrogen mass'                  &
                           ,   '[kgN/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%bsapwoodb_n)) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%bsapwoodb_n                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'BSAPWOODB_N_PY :146:hist:anal:mont:dail:dcyc')
         call metadata_edio(nvar,igr,'Below-ground sapwood nitrogen mass'                  &
                           ,   '[kgN/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%bseeds_n   )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%bseeds_n                                            &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'BSEEDS_N_PY    :146:hist:anal:mont:dail:dcyc')
         call metadata_edio(nvar,igr,'Seed nitrogen mass'                                  &
                           ,   '[kgN/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%bstorage_n )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%bstorage_n                                          &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'BSTORAGE_N_PY  :146:hist:anal:dail')
         call metadata_edio(nvar,igr,'Storage nitrogen mass'                               &
                           ,   '[kgN/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%leaf_maintenance)) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%leaf_maintenance                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'LEAF_MAINTENANCE_PY    :146:hist:anal:dail')
         call metadata_edio(nvar,igr,'Leaf maintenance rate'                               &
                           ,   '[kgC/m2/yr]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%root_maintenance)) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%root_maintenance                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'ROOT_MAINTENANCE_PY    :146:hist:anal:dail')
         call metadata_edio(nvar,igr,'Root maintenance rate'                               &
                           ,   '[kgC/m2/yr]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%leaf_drop)) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%leaf_drop                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'LEAF_DROP_PY    :146:hist:anal:dail')
         call metadata_edio(nvar,igr,'Leaf drop'                                           &
                           ,   '[kgC/m2/yr]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%mmean_lai)) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%mmean_lai                                           &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_LAI_PY         :146:'//trim(eorq_keys))
         call metadata_edio(nvar,igr,'Monthly mean - Leaf area index'                      &
                           ,'[m2leaf/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%mmean_bleaf      )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%mmean_bleaf                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_BLEAF_PY       :146:'//trim(eorq_keys))
         call metadata_edio(nvar,igr,'Monthly mean - Leaf biomass'                         &
                           ,   '[kgC/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%mmean_broot      )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%mmean_broot                                         &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_BROOT_PY       :146:'//trim(eorq_keys))
         call metadata_edio(nvar,igr,'Monthly mean - Root biomass'                         &
                           ,   '[kgC/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%mmean_bstorage   )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%mmean_bstorage                                      &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_BSTORAGE_PY       :146:'//trim(eorq_keys))
         call metadata_edio(nvar,igr,'Monthly mean - Storage biomass'                      &
                           ,   '[kgC/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%mmean_bleaf_n    )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%mmean_bleaf_n                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_BLEAF_N_PY     :146:'//trim(eorq_keys))
         call metadata_edio(nvar,igr,'Monthly mean - Leaf nitrogen mass'                   &
                           ,   '[kgN/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%mmean_broot_n    )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%mmean_broot_n                                       &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_BROOT_N_PY     :146:'//trim(eorq_keys))
         call metadata_edio(nvar,igr,'Monthly mean - Root nitrogen mass'                   &
                           ,   '[kgN/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%mmean_bstorage_n )) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%mmean_bstorage_n                                    &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_BSTORAGE_N_PY     :146:'//trim(eorq_keys))
         call metadata_edio(nvar,igr,'Monthly mean - Storage nitrogen mass'                &
                           ,   '[kgN/m2]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%mmean_leaf_maintenance)) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%mmean_leaf_maintenance                              &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_LEAF_MAINTENANCE_PY       :146:'//trim(eorq_keys))
         call metadata_edio(nvar,igr,'Monthly mean - Leaf maintenance rate'                &
                           ,   '[kgC/m2/yr]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%mmean_root_maintenance)) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%mmean_root_maintenance                              &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ROOT_MAINTENANCE_PY       :146:'//trim(eorq_keys))
         call metadata_edio(nvar,igr,'Monthly mean - Root maintenance rate'                &
                           ,   '[kgC/m2/yr]','(n_pft,n_dbh,ipoly)')
      end if
      if (associated(cgrid%mmean_leaf_drop)) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cgrid%mmean_leaf_drop                                     &
                           ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_LEAF_DROP_PY       :146:'//trim(eorq_keys))
         call metadata_edio(nvar,igr,'Monthly mean - Leaf drop'                            &
                           ,   '[kgC/m2/yr]','(n_pft,n_dbh,ipoly)')
      end if
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!


      return     
   end subroutine filltab_edtype_p146
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the polygon-level variables         !
   ! (edtype) that have three dimensions (3,4,npolygons) and are real (type 199).          !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_edtype_p199(cgrid,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(edtype), target        :: cgrid
      integer     , intent(in)    :: init
      integer     , intent(in)    :: igr
      integer     , intent(in)    :: var_len
      integer     , intent(in)    :: max_ptrs
      integer     , intent(in)    :: var_len_global
      integer     , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                     :: npts
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !     This is the 3-D block, with LAI and 4 vars.  All variables must have the       !
      ! number of points defined by npts.                                                  !
      !------------------------------------------------------------------------------------!
      npts = cgrid%npolygons * 3 * 4

      if (associated(cgrid%avg_lai_ebalvars)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cgrid%avg_lai_ebalvars                                  &
                          ,nvar,igr,init,cgrid%pyglob_id,var_len,var_len_global,max_ptrs &
                           ,'AVG_LAI_EBALVARS :199:anal') 
        call metadata_edio(nvar,igr,'Polygon Average Energy Balance Variables'           &
                          ,'[variable]','(4,3,ipoly)') 
      end if
      
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!



      return     
   end subroutine filltab_edtype_p199
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This sub-routine fills in the variable table with site-level variables.           !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_polygontype(igr,ipy,init)
      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      integer, intent(in) :: init
      integer, intent(in) :: igr
      integer, intent(in) :: ipy
      !----- Local variables. -------------------------------------------------------------!
      type(polygontype), pointer :: cpoly
      integer                    :: var_len
      integer                    :: max_ptrs
      integer                    :: var_len_global
      integer                    :: nvar
      integer                    :: npts
      !------------------------------------------------------------------------------------!



      !----- Check whether this polygon is allocated. -------------------------------------!
      if (.not.associated(edgrid_g(igr)%polygon(ipy)%sipa_id)) then
         write (unit=*,fmt='(a)')       '------------------------------------------------'
         write (unit=*,fmt='(a)')       ' Returning from filltab_polygontype'
         write (unit=*,fmt='(a,1x,i5)') ' IGR  = ',igr
         write (unit=*,fmt='(a,1x,i5)') ' IPY  = ',ipy
         write (unit=*,fmt='(a,1x,i5)') ' INIT = ',init
         write (unit=*,fmt='(a)')       '------------------------------------------------'
         return
      end if
      !------------------------------------------------------------------------------------!



      !----- Assign a pointer to the current polygon. -------------------------------------!
      cpoly => edgrid_g(igr)%polygon(ipy)
      !------------------------------------------------------------------------------------!



      !----- Define some variables to be used by all variables. ---------------------------!
      var_len        = cpoly%nsites
      var_len_global = edgrid_g(igr)%nsites_global
      max_ptrs       = edgrid_g(igr)%npolygons
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      !     Initialise the variable number, which should be unique for each variable.      !
      ! nioglobal has the current position after all the parent structure variables have   !
      ! been positioned in the variable table.                                             !
      !------------------------------------------------------------------------------------!
      nvar=nioglobal+niogrid
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !      We split the routine into smaller ones to avoid excessive number of calls to  !
      ! sub-routines from the same routine (this allows more optimisation for ifort, and   !
      ! reduces chances of segmentation violation during the compilation).                 !
      !------------------------------------------------------------------------------------!
      call filltab_polygontype_p20     (cpoly,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_polygontype_p21inst (cpoly,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_polygontype_p21fmean(cpoly,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_polygontype_p21dmean(cpoly,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_polygontype_p21mmean(cpoly,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_polygontype_p220    (cpoly,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_polygontype_p24     (cpoly,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_polygontype_m21     (cpoly,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_polygontype_p29     (cpoly,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_polygontype_p246    (cpoly,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_polygontype_p255    (cpoly,igr,init,var_len,var_len_global,max_ptrs,nvar)
      !------------------------------------------------------------------------------------!


      !----- Save the number of site-level (polygontype) variables that go to the output. -!
      if (init == 0) niopoly=nvar-niogrid-nioglobal
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_polygontype
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the site-level variables            !
   ! (polygontype) that have one dimension and are integer (type 20).                      !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_polygontype_p20(cpoly,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_i  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(polygontype), target        :: cpoly
      integer          , intent(in)    :: init
      integer          , intent(in)    :: igr
      integer          , intent(in)    :: var_len
      integer          , intent(in)    :: max_ptrs
      integer          , intent(in)    :: var_len_global
      integer          , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                          :: npts
      !------------------------------------------------------------------------------------!




      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !     This is the 1-D block.  All variables must have the number of points defined   !
      ! by npts.                                                                           !
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 1-D vectors (nsites).  Notice that they all use   !
      ! npts = cpoly%nsites.  Add only variables of types 20, integer variables.           !
      !------------------------------------------------------------------------------------!
      npts = cpoly%nsites


      if (associated(cpoly%sipa_id)) then
         nvar=nvar+1
         call vtable_edio_i(npts,cpoly%sipa_id                                             &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'SIPA_ID :20:hist:anal:dail:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'Sites first patch indices','[--]','(isite)') 
      end if
      
      if (associated(cpoly%sipa_n)) then
         nvar=nvar+1
         call vtable_edio_i(npts,cpoly%sipa_n                                              &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'SIPA_N :20:hist:anal:dail:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'Number of patches per site','[--]','(isite)') 
      end if
      
      if (associated(cpoly%patch_count)) then
         nvar=nvar+1
         call vtable_edio_i(npts,cpoly%patch_count                                         &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'PATCH_COUNT :20:hist:anal:dail:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'Patch count for each site','[--]','(isite)') 
      end if
      
      if (associated(cpoly%sitenum)) then
         nvar=nvar+1
         call vtable_edio_i(npts,cpoly%sitenum                                             &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'SITENUM :20:hist:anal:year') 
         call metadata_edio(nvar,igr,'Site number','[--]','(isite)') 
      end if

      if (associated(cpoly%lsl)) then
         nvar=nvar+1
         call vtable_edio_i(npts,cpoly%lsl                                                 &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'LSL :20:hist:anal:dail:mont:dcyc') 
         call metadata_edio(nvar,igr,'Lowest soil layer','[--]','(isite)') 
      end if   

      if (associated(cpoly%ncol_soil)) then
         nvar=nvar+1
         call vtable_edio_i(npts,cpoly%ncol_soil                                           &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'NCOL_SOIL :20:hist:anal:dail:mont:dcyc') 
         call metadata_edio(nvar,igr,'Soil colour index','[--]','(isite)') 
      end if   
      
      if (associated(cpoly%num_landuse_years)) then
         nvar=nvar+1
         call vtable_edio_i(npts,cpoly%num_landuse_years                                   &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'NUM_LANDUSE_YEARS :20:hist') 
         call metadata_edio(nvar,igr,'Number of years with land use data','[--]','(isite)') 
      end if

      if (associated(cpoly%hydro_next)) then
         nvar=nvar+1
         call vtable_edio_i(npts,cpoly%hydro_next                                          &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'HYDRO_NEXT :20:hist') 
         call metadata_edio(nvar,igr,'Next site for hydrological routing'                  &
                           ,'[--]','(isite)') 
      end if

      if (associated(cpoly%hydro_prev)) then
         nvar=nvar+1
         call vtable_edio_i(npts,cpoly%hydro_prev                                          &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'HYDRO_PREV :20:hist') 
         call metadata_edio(nvar,igr,'Previous site for hydrological routing'              &
                           ,'[--]','(isite)') 
      end if

      if (associated(cpoly%plantation)) then
         nvar=nvar+1
           call vtable_edio_i(npts,cpoly%plantation,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'PLANTATION_SI :20:hist') 
         call metadata_edio(nvar,igr,'Plantation flag (0-no;1-yes)','[--]','(isite)') 
      end if 

      if (associated(cpoly%agri_stocking_pft)) then
         nvar=nvar+1
           call vtable_edio_i(npts,cpoly%agri_stocking_pft,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'AGRI_STOCKING_PFT :20:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpoly%plantation_stocking_pft)) then
         nvar=nvar+1
           call vtable_edio_i(npts,cpoly%plantation_stocking_pft,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'PLANTATION_STOCKING_PFT :20:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if
     !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_polygontype_p20
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the site-level variables            !
   ! (polygontype) that have one dimension and are real (type 21), and are not averaged    !
   ! variable (fmean, dmean, mmean, mmsqu, qmean, qmsqu).                                  !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_polygontype_p21inst(cpoly,igr,init,var_len,var_len_global,max_ptrs   &
                                         ,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(polygontype), target        :: cpoly
      integer          , intent(in)    :: init
      integer          , intent(in)    :: igr
      integer          , intent(in)    :: var_len
      integer          , intent(in)    :: max_ptrs
      integer          , intent(in)    :: var_len_global
      integer          , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                          :: npts
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 1-D vectors (nsites).  Notice that they all use   !
      ! npts = cpoly%nsites.  Add only variables of types 21, real variables, that are not !
      ! ?mean or ?msqu.                                                                    !
      !------------------------------------------------------------------------------------!
      npts = cpoly%nsites

      if (associated(cpoly%area)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpoly%area,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'AREA_SI:21:hist:dail:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpoly%patch_area)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpoly%patch_area,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'PATCH_AREA:21:hist:year') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpoly%elevation)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpoly%elevation,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'ELEVATION :21:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpoly%slope)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpoly%slope,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'SLOPE :21:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpoly%aspect)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpoly%aspect,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'ASPECT :21:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpoly%TCI)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpoly%TCI,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'TCI :21:hist') 
         call metadata_edio(nvar,igr,'Topographic convergence index','[NA]','NA') 
      end if      

      if (associated(cpoly%pptweight)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpoly%TCI,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'PPTWEIGHT :21:hist') 
         call metadata_edio(nvar,igr,'precip lapse weighting','[NA]','NA') 
      end if      

      if (associated(cpoly%moist_W)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpoly%moist_W,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'MOIST_W :21:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpoly%moist_f)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpoly%moist_f,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'MOIST_F :21:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if  

      if (associated(cpoly%moist_tau)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpoly%moist_tau,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'MOIST_TAU :21:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpoly%moist_zi)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpoly%moist_zi,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'MOIST_ZI :21:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if 

      if (associated(cpoly%baseflow)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpoly%baseflow  ,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'BASEFLOW_SI :21:hist') 
         call metadata_edio(nvar,igr,'loss of water from site to watershed discharge','[kg/m2/s]','NA') 
      end if 

      if (associated(cpoly%min_monthly_temp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpoly%min_monthly_temp,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'MIN_MONTHLY_TEMP :21:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpoly%agri_stocking_density)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpoly%agri_stocking_density,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'AGRI_STOCKING_DENSITY :21:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpoly%plantation_stocking_density)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpoly%plantation_stocking_density,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'PLANTATION_STOCKING_DENSITY :21:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpoly%primary_harvest_target)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpoly%primary_harvest_target,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'PRIMARY_HARVEST_TARGET :21:hist') 
         call metadata_edio(nvar,igr,'Target biomass to be harvest from primary vegetation','[kgC/m2]','(isite)') 
      end if

      if (associated(cpoly%secondary_harvest_target)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpoly%secondary_harvest_target,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'SECONDARY_HARVEST_TARGET:21:hist') 
         call metadata_edio(nvar,igr,'Target biomass to be harvest from secondary vegetation','[kgC/m2]','(isite)') 
      end if

      if (associated(cpoly%primary_harvest_memory)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpoly%primary_harvest_memory,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'PRIMARY_HARVEST_MEMORY :21:hist') 
         call metadata_edio(nvar,igr,'Harvest debt from primary vegetation','[kgC/m2]','(isite)') 
      end if

      if (associated(cpoly%secondary_harvest_memory)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpoly%secondary_harvest_memory,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'SECONDARY_HARVEST_MEMORY:21:hist') 
         call metadata_edio(nvar,igr,'Harvest debt from secondary vegetation','[kgC/m2]','(isite)') 
      end if

      if (associated(cpoly%ignition_rate)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpoly%ignition_rate,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'IGNITION_RATE :21:hist:mont:dcyc') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpoly%daylight)) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cpoly%daylight                                            &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DAYLIGHT :21:hist:dail') 
         call metadata_edio(nvar,igr,'Length of day time light','[s]','(isite)') 
      end if

      if (associated(cpoly%cosaoi)) then
         nvar = nvar + 1
         call vtable_edio_r(npts,cpoly%cosaoi                                              &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'COSAOI :21:hist') 
         call metadata_edio(nvar,igr,'Co-sine of the angle of incidence','[--]','(isite)') 
      end if

      if (associated(cpoly%rad_avg)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpoly%rad_avg,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'RAD_AVG :21:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_polygontype_p21inst
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the site-level variables            !
   ! (polygontype) that have one dimension and are real (type 21), and are sub-daily       !
   ! averages (fmean).                                                                     !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_polygontype_p21fmean(cpoly,igr,init,var_len,var_len_global,max_ptrs  &
                                         ,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(polygontype)     , target        :: cpoly
      integer               , intent(in)    :: init
      integer               , intent(in)    :: igr
      integer               , intent(in)    :: var_len
      integer               , intent(in)    :: max_ptrs
      integer               , intent(in)    :: var_len_global
      integer               , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                               :: npts
      character(len=str_len)                :: fast_keys
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !      Decide whether to write the sub-daily means to the history file.              !
      !------------------------------------------------------------------------------------!
      select case (iadd_site_means)
      case (0)
         return
      case (1)
         if (history_fast) then
            fast_keys = 'hist:anal'
         else
            fast_keys = 'anal'
         end if
      end select
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 1-D vectors (nsites).  Notice that they all use   !
      ! npts = cpoly%nsites.  Add only variables of types 21, real variables, that are     !
      ! fmean.                                                                             !
      !------------------------------------------------------------------------------------!
      npts = cpoly%nsites


      if (associated(cpoly%fmean_atm_theiv       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%fmean_atm_theiv                                     &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_THEIV_SI         :21:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Ice-vapour equiv. pot. temp.: Atmosphere'    &
                           ,'[          K]','(isite)'            )
      end if
      if (associated(cpoly%fmean_atm_theta       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%fmean_atm_theta                                     &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_THETA_SI         :21:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Potential temperature: Atmosphere'           &
                           ,'[          K]','(isite)'            )
      end if
      if (associated(cpoly%fmean_atm_temp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%fmean_atm_temp                                      &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_TEMP_SI          :21:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Temperature: Atmosphere'                     &
                           ,'[          K]','(isite)'            )
      end if
      if (associated(cpoly%fmean_atm_vpdef       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%fmean_atm_vpdef                                     &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_VPDEF_SI         :21:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Vapour pressure deficit: Atmosphere'         &
                           ,'[         Pa]','(isite)'            )
      end if
      if (associated(cpoly%fmean_atm_shv         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%fmean_atm_shv                                       &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_SHV_SI           :21:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Specific humidity: Atmosphere'               &
                           ,'[      kg/kg]','(isite)'            )
      end if
      if (associated(cpoly%fmean_atm_rshort      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%fmean_atm_rshort                                    &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_RSHORT_SI        :21:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Shortwave radiation: Atmosphere'             &
                           ,'[       W/m2]','(isite)'            )
      end if
      if (associated(cpoly%fmean_atm_rshort_diff )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%fmean_atm_rshort_diff                               &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_RSHORT_DIFF_SI   :21:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Diffuse SW radiation: Atmosphere'            &
                           ,'[       W/m2]','(isite)'            )
      end if
      if (associated(cpoly%fmean_atm_par         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%fmean_atm_par                                       &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_PAR_SI           :21:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - PAR: Atmosphere'                             &
                           ,'[       W/m2]','(isite)'            )
      end if
      if (associated(cpoly%fmean_atm_par_diff    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%fmean_atm_par_diff                                  &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_PAR_DIFF_SI      :21:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Diffuse PAR: Atmosphere'                     &
                           ,'[       W/m2]','(isite)'            )
      end if
      if (associated(cpoly%fmean_atm_rlong       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%fmean_atm_rlong                                     &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_RLONG_SI         :21:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Longwave radiation: Atmosphere'              &
                           ,'[       W/m2]','(isite)'            )
      end if
      if (associated(cpoly%fmean_atm_vels        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%fmean_atm_vels                                      &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_VELS_SI          :21:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Wind speed: atmosphere'                      &
                           ,'[        m/s]','(isite)'            )
      end if
      if (associated(cpoly%fmean_atm_rhos        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%fmean_atm_rhos                                      &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_RHOS_SI          :21:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Air density: Atmosphere'                     &
                           ,'[      kg/m3]','(isite)'            )
      end if
      if (associated(cpoly%fmean_atm_prss        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%fmean_atm_prss                                      &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_PRSS_SI          :21:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Air pressure: Atmosphere'                    &
                           ,'[         Pa]','(isite)'            )
      end if
      if (associated(cpoly%fmean_atm_co2         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%fmean_atm_co2                                       &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ATM_CO2_SI           :21:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - CO2 mixing ratio: Atmosphere'                &
                           ,'[   umol/mol]','(isite)'            )
      end if
      if (associated(cpoly%fmean_pcpg            )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%fmean_pcpg                                          &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_PCPG_SI              :21:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Precipitation rate'                          &
                           ,'[    kg/m2/s]','(isite)'            )
      end if
      if (associated(cpoly%fmean_qpcpg           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%fmean_qpcpg                                         &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_QPCPG_SI             :21:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Energy gain - rain'                          &
                           ,'[       W/m2]','(isite)'            )
      end if
      if (associated(cpoly%fmean_dpcpg           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%fmean_dpcpg                                         &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_DPCPG_SI             :21:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Precipitation depth'                         &
                           ,'[          m]','(isite)'            )
      end if
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_polygontype_p21fmean
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the site-level variables            !
   ! (polygontype) that have one dimension and are real (type 21), and are daily           !
   ! averages (dmean).                                                                     !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_polygontype_p21dmean(cpoly,igr,init,var_len,var_len_global,max_ptrs  &
                                         ,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(polygontype)     , target        :: cpoly
      integer               , intent(in)    :: init
      integer               , intent(in)    :: igr
      integer               , intent(in)    :: var_len
      integer               , intent(in)    :: max_ptrs
      integer               , intent(in)    :: var_len_global
      integer               , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                               :: npts
      character(len=str_len)                :: dail_keys
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !      Decide whether to write the daily means to the history file.                  !
      !------------------------------------------------------------------------------------!
      select case (iadd_site_means)
      case (0)
         return
      case (1)
         if (history_dail) then
            dail_keys = 'hist:dail'
         else
            dail_keys = 'dail'
         end if
      end select
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 1-D vectors (nsites).  Notice that they all use   !
      ! npts = cpoly%nsites.  Add only variables of types 21, real variables, that are     !
      ! dmean.                                                                             !
      !------------------------------------------------------------------------------------!
      npts = cpoly%nsites

      if (associated(cpoly%dmean_atm_theiv       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%dmean_atm_theiv                                     &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_THEIV_SI         :21:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Ice-vapour equiv. pot. temp.: Atmosphere'        &
                           ,'[          K]','(isite)'            )
      end if
      if (associated(cpoly%dmean_atm_theta       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%dmean_atm_theta                                     &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_THETA_SI         :21:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Potential temperature: Atmosphere'               &
                           ,'[          K]','(isite)'            )
      end if
      if (associated(cpoly%dmean_atm_temp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%dmean_atm_temp                                      &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_TEMP_SI          :21:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Temperature: Atmosphere'                         &
                           ,'[          K]','(isite)'            )
      end if
      if (associated(cpoly%dmean_atm_vpdef       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%dmean_atm_vpdef                                     &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_VPDEF_SI         :21:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Vapour pressure deficit: Atmosphere'             &
                           ,'[         Pa]','(isite)'            )
      end if
      if (associated(cpoly%dmean_atm_shv         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%dmean_atm_shv                                       &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_SHV_SI           :21:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Specific humidity: Atmosphere'                   &
                           ,'[      kg/kg]','(isite)'            )
      end if
      if (associated(cpoly%dmean_atm_rshort      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%dmean_atm_rshort                                    &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_RSHORT_SI        :21:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Shortwave radiation: Atmosphere'                 &
                           ,'[       W/m2]','(isite)'            )
      end if
      if (associated(cpoly%dmean_atm_rshort_diff )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%dmean_atm_rshort_diff                               &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_RSHORT_DIFF_SI   :21:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Diffuse SW radiation: Atmosphere'                &
                           ,'[       W/m2]','(isite)'            )
      end if
      if (associated(cpoly%dmean_atm_par         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%dmean_atm_par                                       &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_PAR_SI           :21:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - PAR: Atmosphere'                                 &
                           ,'[       W/m2]','(isite)'            )
      end if
      if (associated(cpoly%dmean_atm_par_diff    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%dmean_atm_par_diff                                  &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_PAR_DIFF_SI      :21:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Diffuse PAR: Atmosphere'                         &
                           ,'[       W/m2]','(isite)'            )
      end if
      if (associated(cpoly%dmean_atm_rlong       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%dmean_atm_rlong                                     &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_RLONG_SI         :21:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Longwave radiation: Atmosphere'                  &
                           ,'[       W/m2]','(isite)'            )
      end if
      if (associated(cpoly%dmean_atm_vels        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%dmean_atm_vels                                      &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_VELS_SI          :21:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Wind speed: atmosphere'                          &
                           ,'[        m/s]','(isite)'            )
      end if
      if (associated(cpoly%dmean_atm_rhos        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%dmean_atm_rhos                                      &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_RHOS_SI          :21:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Air density: Atmosphere'                         &
                           ,'[      kg/m3]','(isite)'            )
      end if
      if (associated(cpoly%dmean_atm_prss        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%dmean_atm_prss                                      &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_PRSS_SI          :21:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Air pressure: Atmosphere'                        &
                           ,'[         Pa]','(isite)'            )
      end if
      if (associated(cpoly%dmean_atm_co2         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%dmean_atm_co2                                       &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ATM_CO2_SI           :21:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - CO2 mixing ratio: Atmosphere'                    &
                           ,'[   umol/mol]','(isite)'            )
      end if
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_polygontype_p21dmean
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the site-level variables            !
   ! (polygontype) that have one dimension and are real (type 21), and are monthly         !
   ! averages (mmean or mmsqu).                                                            !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_polygontype_p21mmean(cpoly,igr,init,var_len,var_len_global,max_ptrs  &
                                         ,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(polygontype)     , target        :: cpoly
      integer               , intent(in)    :: init
      integer               , intent(in)    :: igr
      integer               , intent(in)    :: var_len
      integer               , intent(in)    :: max_ptrs
      integer               , intent(in)    :: var_len_global
      integer               , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                               :: npts
      character(len=str_len)                :: eorq_keys
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !      Decide whether to write the monthly means to the history file.                !
      !------------------------------------------------------------------------------------!
      select case (iadd_site_means)
      case (0)
         return
      case (1)
         if (history_eorq) then
            eorq_keys = 'hist:mont:dcyc'
         else
            eorq_keys = 'mont:dcyc'
         end if
      end select
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 1-D vectors (nsites).  Notice that they all use   !
      ! npts = cpoly%nsites.  Add only variables of types 21, real variables, that are     !
      ! mmean or mmsqu.                                                                    !
      !------------------------------------------------------------------------------------!
      npts = cpoly%nsites

      if (associated(cpoly%mmean_atm_theiv       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%mmean_atm_theiv                                     &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_THEIV_SI         :21:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Ice-vapour equiv. pot. temp.: Atmosphere'      &
                           ,'[          K]','(isite)'            )
      end if
      if (associated(cpoly%mmean_atm_theta       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%mmean_atm_theta                                     &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_THETA_SI         :21:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Potential temperature: Atmosphere'             &
                           ,'[          K]','(isite)'            )
      end if
      if (associated(cpoly%mmean_atm_temp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%mmean_atm_temp                                      &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_TEMP_SI          :21:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Temperature: Atmosphere'                       &
                           ,'[          K]','(isite)'            )
      end if
      if (associated(cpoly%mmean_atm_vpdef       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%mmean_atm_vpdef                                     &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_VPDEF_SI         :21:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Vapour pressure deficit: Atmosphere'           &
                           ,'[         Pa]','(isite)'            )
      end if
      if (associated(cpoly%mmean_atm_shv         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%mmean_atm_shv                                       &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_SHV_SI           :21:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Specific humidity: Atmosphere'                 &
                           ,'[      kg/kg]','(isite)'            )
      end if
      if (associated(cpoly%mmean_atm_rshort      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%mmean_atm_rshort                                    &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_RSHORT_SI        :21:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Shortwave radiation: Atmosphere'               &
                           ,'[       W/m2]','(isite)'            )
      end if
      if (associated(cpoly%mmean_atm_rshort_diff )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%mmean_atm_rshort_diff                               &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_RSHORT_DIFF_SI   :21:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Diffuse SW radiation: Atmosphere'              &
                           ,'[       W/m2]','(isite)'            )
      end if
      if (associated(cpoly%mmean_atm_par         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%mmean_atm_par                                       &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_PAR_SI           :21:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - PAR: Atmosphere'                               &
                           ,'[       W/m2]','(isite)'            )
      end if
      if (associated(cpoly%mmean_atm_par_diff    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%mmean_atm_par_diff                                  &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_PAR_DIFF_SI      :21:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Diffuse PAR: Atmosphere'                       &
                           ,'[       W/m2]','(isite)'            )
      end if
      if (associated(cpoly%mmean_atm_rlong       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%mmean_atm_rlong                                     &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_RLONG_SI         :21:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Longwave radiation: Atmosphere'                &
                           ,'[       W/m2]','(isite)'            )
      end if
      if (associated(cpoly%mmean_atm_vels        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%mmean_atm_vels                                      &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_VELS_SI          :21:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Wind speed: atmosphere'                        &
                           ,'[        m/s]','(isite)'            )
      end if
      if (associated(cpoly%mmean_atm_rhos        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%mmean_atm_rhos                                      &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_RHOS_SI          :21:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Air density: Atmosphere'                       &
                           ,'[      kg/m3]','(isite)'            )
      end if
      if (associated(cpoly%mmean_atm_prss        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%mmean_atm_prss                                      &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_PRSS_SI          :21:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Air pressure: Atmosphere'                      &
                           ,'[         Pa]','(isite)'            )
      end if
      if (associated(cpoly%mmean_atm_co2         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%mmean_atm_co2                                       &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ATM_CO2_SI           :21:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - CO2 mixing ratio: Atmosphere'                  &
                           ,'[   umol/mol]','(isite)'            )
      end if
      if (associated(cpoly%mmean_pcpg            )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%mmean_pcpg                                          &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_PCPG_SI              :21:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Precipitation rate'                            &
                           ,'[    kg/m2/s]','(isite)'            )
      end if
      if (associated(cpoly%mmean_qpcpg           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%mmean_qpcpg                                         &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_QPCPG_SI             :21:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Energy gain - rain'                            &
                           ,'[       W/m2]','(isite)'            )
      end if
      if (associated(cpoly%mmean_dpcpg           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%mmean_dpcpg                                         &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_DPCPG_SI             :21:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Precipitation depth'                           &
                           ,'[          m]','(isite)'            )
      end if
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_polygontype_p21mmean
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the site-level variables            !
   ! (polygontype) that have two dimensions (nzg,nsites) and are integer (type 220).       !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_polygontype_p220(cpoly,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_i  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(polygontype), target        :: cpoly
      integer          , intent(in)    :: init
      integer          , intent(in)    :: igr
      integer          , intent(in)    :: var_len
      integer          , intent(in)    :: max_ptrs
      integer          , intent(in)    :: var_len_global
      integer          , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                          :: npts
      !------------------------------------------------------------------------------------!





      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !     This is the 2-D block, with dimensions being nsites and nzg.  Make sure to     !
      ! include only variables of types 22 and 220 here, as they will all use the same     !
      ! npts.                                                                              !
      !------------------------------------------------------------------------------------!
      npts = cpoly%nsites * nzg

  
      if (associated(cpoly%ntext_soil)) then
         nvar=nvar+1
         call vtable_edio_i(npts,cpoly%ntext_soil                                          &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'NTEXT_SOIL :220:hist:anal:dail:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'Soil texture class','[--]','(isite)')
      end if

      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_polygontype_p220
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the site-level variables            !
   ! (polygontype) that have two dimensions (n_pft,nsites) and are real (type 24).         !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_polygontype_p24(cpoly,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(polygontype), target        :: cpoly
      integer          , intent(in)    :: init
      integer          , intent(in)    :: igr
      integer          , intent(in)    :: var_len
      integer          , intent(in)    :: max_ptrs
      integer          , intent(in)    :: var_len_global
      integer          , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                          :: npts
      !------------------------------------------------------------------------------------!





      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !     This is the 2-D block, with dimensions being nsites and n_pft.  Make sure to   !
      ! include only variables of type 24 here, as they will all use the same npts.        !
      !------------------------------------------------------------------------------------!
      npts = cpoly%nsites * n_pft

      if (associated(cpoly%green_leaf_factor)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpoly%green_leaf_factor,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'GREEN_LEAF_FACTOR :24:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpoly%leaf_aging_factor)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpoly%leaf_aging_factor,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'LEAF_AGING_FACTOR :24:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_polygontype_p24
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the site-level variables            !
   ! (polygontype) that have two dimensions (ndcycle,nsites) and are real (type -21).      !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_polygontype_m21(cpoly,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(polygontype)     , target        :: cpoly
      integer               , intent(in)    :: init
      integer               , intent(in)    :: igr
      integer               , intent(in)    :: var_len
      integer               , intent(in)    :: max_ptrs
      integer               , intent(in)    :: var_len_global
      integer               , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                               :: npts
      character(len=str_len)                :: eorq_keys
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !      Decide whether to write the mean diel to the history file.                    !
      !------------------------------------------------------------------------------------!
      select case (iadd_site_means)
      case (0)
         return
      case (1)
         if (history_eorq) then
            eorq_keys = 'hist:dcyc'
         else
            eorq_keys = 'dcyc'
         end if
      end select
      !------------------------------------------------------------------------------------!





      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !     This is the 2-D block, with dimensions being nsites and ndcycle.  Make sure    !
      ! to include only variables of type -21 here, as they will all use the same npts.    !
      !------------------------------------------------------------------------------------!
      npts = cpoly%nsites * ndcycle

      if (associated(cpoly%qmean_atm_theiv       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%qmean_atm_theiv                                     &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_THEIV_SI        :-21:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Ice-vapour equiv. pot. temp.: Atmosphere'         &
                           ,'[          K]','(ndcycle,isite)'    )
      end if
      if (associated(cpoly%qmean_atm_theta       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%qmean_atm_theta                                     &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_THETA_SI        :-21:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Potential temperature: Atmosphere'                &
                           ,'[          K]','(ndcycle,isite)'    )
      end if
      if (associated(cpoly%qmean_atm_temp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%qmean_atm_temp                                      &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_TEMP_SI         :-21:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Temperature: Atmosphere'                          &
                           ,'[          K]','(ndcycle,isite)'    )
      end if
      if (associated(cpoly%qmean_atm_vpdef       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%qmean_atm_vpdef                                     &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_VPDEF_SI        :-21:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Vapour pressure deficit: Atmosphere'              &
                           ,'[         Pa]','(ndcycle,isite)'    )
      end if
      if (associated(cpoly%qmean_atm_shv         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%qmean_atm_shv                                       &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_SHV_SI          :-21:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Specific humidity: Atmosphere'                    &
                           ,'[      kg/kg]','(ndcycle,isite)'    )
      end if
      if (associated(cpoly%qmean_atm_rshort      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%qmean_atm_rshort                                    &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_RSHORT_SI       :-21:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Shortwave radiation: Atmosphere'                  &
                           ,'[       W/m2]','(ndcycle,isite)'    )
      end if
      if (associated(cpoly%qmean_atm_rshort_diff )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%qmean_atm_rshort_diff                               &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_RSHORT_DIFF_SI  :-21:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Diffuse SW radiation: Atmosphere'                 &
                           ,'[       W/m2]','(ndcycle,isite)'    )
      end if
      if (associated(cpoly%qmean_atm_par         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%qmean_atm_par                                       &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_PAR_SI          :-21:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - PAR: Atmosphere'                                  &
                           ,'[       W/m2]','(ndcycle,isite)'    )
      end if
      if (associated(cpoly%qmean_atm_par_diff    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%qmean_atm_par_diff                                  &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_PAR_DIFF_SI     :-21:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Diffuse PAR: Atmosphere'                          &
                           ,'[       W/m2]','(ndcycle,isite)'    )
      end if
      if (associated(cpoly%qmean_atm_rlong       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%qmean_atm_rlong                                     &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_RLONG_SI        :-21:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Longwave radiation: Atmosphere'                   &
                           ,'[       W/m2]','(ndcycle,isite)'    )
      end if
      if (associated(cpoly%qmean_atm_vels        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%qmean_atm_vels                                      &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_VELS_SI         :-21:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Wind speed: atmosphere'                           &
                           ,'[        m/s]','(ndcycle,isite)'    )
      end if
      if (associated(cpoly%qmean_atm_rhos        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%qmean_atm_rhos                                      &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_RHOS_SI         :-21:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Air density: Atmosphere'                          &
                           ,'[      kg/m3]','(ndcycle,isite)'    )
      end if
      if (associated(cpoly%qmean_atm_prss        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%qmean_atm_prss                                      &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_PRSS_SI         :-21:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Air pressure: Atmosphere'                         &
                           ,'[         Pa]','(ndcycle,isite)'    )
      end if
      if (associated(cpoly%qmean_atm_co2         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%qmean_atm_co2                                       &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ATM_CO2_SI          :-21:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - CO2 mixing ratio: Atmosphere'                     &
                           ,'[   umol/mol]','(ndcycle,isite)'    )
      end if
      if (associated(cpoly%qmean_pcpg            )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%qmean_pcpg                                          &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_PCPG_SI             :-21:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Precipitation rate'                               &
                           ,'[    kg/m2/s]','(ndcycle,isite)'    )
      end if
      if (associated(cpoly%qmean_qpcpg           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%qmean_qpcpg                                         &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_QPCPG_SI            :-21:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Energy gain - rain'                               &
                           ,'[       W/m2]','(ndcycle,isite)'    )
      end if
      if (associated(cpoly%qmean_dpcpg           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpoly%qmean_dpcpg                                         &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_DPCPG_SI            :-21:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Precipitation depth'                              &
                           ,'[          m]','(ndcycle,isite)'    )
      end if
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      return
   end subroutine filltab_polygontype_m21
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the site-level variables            !
   ! (polygontype) that have two dimensions (12 months,nsites) and are real (type 29).     !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_polygontype_p29(cpoly,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(polygontype), target        :: cpoly
      integer          , intent(in)    :: init
      integer          , intent(in)    :: igr
      integer          , intent(in)    :: var_len
      integer          , intent(in)    :: max_ptrs
      integer          , intent(in)    :: var_len_global
      integer          , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                          :: npts
      !------------------------------------------------------------------------------------!






      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !     This is the 2-D block, with dimensions being nsites and 12 months.  Make sure  !
      ! to include only variables of type 29 here, as they will all use the same npts.     !
      !------------------------------------------------------------------------------------!
      npts = cpoly%nsites * 12

      if (associated(cpoly%lambda_fire)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpoly%lambda_fire,nvar,igr,init,cpoly%siglob_id           &
                           ,var_len,var_len_global,max_ptrs,'LAMBDA_FIRE :29:hist')
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpoly%avg_monthly_pcpg)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpoly%avg_monthly_pcpg,nvar,igr,init,cpoly%siglob_id      &
                           ,var_len,var_len_global,max_ptrs,'AVG_MONTHLY_PCPG :29:hist')     
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      return
   end subroutine filltab_polygontype_p29
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the site-level variables            !
   ! (polygontype) that have three dimensions (n_pft,n_dbh,nsites) and are real (type 246).!
   !---------------------------------------------------------------------------------------!
   subroutine filltab_polygontype_p246(cpoly,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(polygontype), target        :: cpoly
      integer          , intent(in)    :: init
      integer          , intent(in)    :: igr
      integer          , intent(in)    :: var_len
      integer          , intent(in)    :: max_ptrs
      integer          , intent(in)    :: var_len_global
      integer          , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                          :: npts
      !------------------------------------------------------------------------------------!





      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !     This is the 3-D block, with dimensions being n_pft and n_dbh.  Make sure to    !
      ! include only variables of type 246 here, as they will all use the same npts.       !
      !------------------------------------------------------------------------------------!
      npts = cpoly%nsites * n_pft * n_dbh

      if (associated(cpoly%basal_area)) then
         nvar=nvar+1
        call vtable_edio_r(npts,cpoly%basal_area                                           &
                          ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs   &
                          ,'BASAL_AREA_SI :246:hist')
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpoly%agb)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpoly%agb                                                 &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'AGB_SI :246:hist')
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpoly%basal_area_growth)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpoly%basal_area_growth                                   &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'BASAL_AREA_GROWTH :246:hist:year') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpoly%agb_growth)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpoly%agb_growth                                          &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'AGB_GROWTH :246:hist:year') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpoly%basal_area_mort)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpoly%basal_area_mort                                     &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'BASAL_AREA_MORT :246:hist:year') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpoly%basal_area_cut)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpoly%basal_area_cut                                      &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'BASAL_AREA_CUT :246:year:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpoly%agb_mort)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpoly%agb_mort                                            &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'AGB_MORT :246:hist:year') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpoly%agb_cut)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpoly%agb_cut                                             &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'AGB_CUT :246:hist:year') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      return
   end subroutine filltab_polygontype_p246
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the site-level variables            !
   ! (polygontype) that have three dimensions (n_dist_types,n_dist_types,nsites) and are   !
   ! real (type 255).                                                                      !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_polygontype_p255(cpoly,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(polygontype), target        :: cpoly
      integer          , intent(in)    :: init
      integer          , intent(in)    :: igr
      integer          , intent(in)    :: var_len
      integer          , intent(in)    :: max_ptrs
      integer          , intent(in)    :: var_len_global
      integer          , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                          :: npts
      !------------------------------------------------------------------------------------!





      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !     This is the 3-D block, with dimensions being nsites and n_dist_types,          !
      ! n_dist_types (from and to).  Make sure to include only variables of type 255 here, !
      ! as they will all use the same npts.                                                !
      !------------------------------------------------------------------------------------!
      npts = cpoly%nsites * n_dist_types * n_dist_types

      if (associated(cpoly%disturbance_memory)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpoly%disturbance_memory,nvar,igr,init,cpoly%siglob_id, &
           var_len,var_len_global,max_ptrs,'DISTURBANCE_MEMORY :255:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpoly%disturbance_rates)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpoly%disturbance_rates                                   &
                           ,nvar,igr,init,cpoly%siglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DISTURBANCE_RATES :255:hist:anal:dail:mont:dcyc') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_polygontype_p255
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This sub-routine will fill the variable table with the sitetype variables (patch- !
   ! -level).  Because of the sheer number of variables, we must split the subroutines     !
   ! into smaller routines.                                                                !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_sitetype(igr,ipy,isi,init)

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      integer       , intent(in) :: init
      integer       , intent(in) :: igr
      integer       , intent(in) :: ipy
      integer       , intent(in) :: isi
      !----- Local variables. -------------------------------------------------------------!
      type(sitetype), pointer    :: csite
      integer                    :: var_len
      integer                    :: max_ptrs
      integer                    :: var_len_global
      integer                    :: nvar
      integer                    :: npts
      !------------------------------------------------------------------------------------!



      !----- Assign a pointer to the current polygon. -------------------------------------!
      csite => edgrid_g(igr)%polygon(ipy)%site(isi)
      !------------------------------------------------------------------------------------!



      !----- Define some variables to be used by all variables. ---------------------------!
      var_len        = csite%npatches
      var_len_global = edgrid_g(igr)%npatches_global
      max_ptrs       = get_nsites(edgrid_g(igr))
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      !     Initialise the variable number, which should be unique for each variable.      !
      ! nioglobal has the current position after all the parent structure variables have   !
      ! been positioned in the variable table.                                             !
      !------------------------------------------------------------------------------------!
      nvar=nioglobal+niogrid+niopoly
      !------------------------------------------------------------------------------------!

      call filltab_sitetype_p30     (csite,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_sitetype_p31inst (csite,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_sitetype_p31fmean(csite,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_sitetype_p31dmean(csite,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_sitetype_p31mmean(csite,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_sitetype_m31     (csite,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_sitetype_p32     (csite,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_sitetype_p33     (csite,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_sitetype_p34     (csite,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_sitetype_p346    (csite,igr,init,var_len,var_len_global,max_ptrs,nvar)
      !------------------------------------------------------------------------------------!

      !----- Save the number of patch-level (sitetype) variables that go to the output. ---!
      if (init == 0) niosite=nvar-niopoly-niogrid-nioglobal
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_sitetype
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the patch-level variables           !
   !  (sitetype) that have one dimension and are integer (type 30).                        !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_sitetype_p30(csite,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_i  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(sitetype), target        :: csite
      integer       , intent(in)    :: init
      integer       , intent(in)    :: igr
      integer       , intent(in)    :: var_len
      integer       , intent(in)    :: max_ptrs
      integer       , intent(in)    :: var_len_global
      integer       , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                       :: npts
      !------------------------------------------------------------------------------------!




      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 1-D vectors (npatches).  Notice that they all use !
      ! npts = csite%npatches.  Add only variables of type 30.                             !
      !------------------------------------------------------------------------------------!
      npts = csite%npatches


      if (associated(csite%paco_id)) then
         nvar=nvar+1
         call vtable_edio_i(npts,csite%paco_id,nvar,igr,init,csite%paglob_id, &
              var_len,var_len_global,max_ptrs,'PACO_ID :30:hist:anal:dail:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%paco_n)) then
         nvar=nvar+1
           call vtable_edio_i(npts,csite%paco_n,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'PACO_N :30:hist:anal:dail:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if
      
      if (associated(csite%dist_type)) then
         nvar=nvar+1
           call vtable_edio_i(npts,csite%dist_type,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'DIST_TYPE :30:hist:anal:dail:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%nlev_sfcwater)) then
         nvar=nvar+1
           call vtable_edio_i(npts,csite%nlev_sfcwater,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'NLEV_SFCWATER :30:hist:year') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_sitetype_p30
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the patch-level variables           !
   !  (sitetype) that have one dimension and are real (type 31).  Because of the sheer     !
   ! amount of variables with type 31, we do the averages in separate sub-routines.        !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_sitetype_p31inst(csite,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(sitetype), target        :: csite
      integer       , intent(in)    :: init
      integer       , intent(in)    :: igr
      integer       , intent(in)    :: var_len
      integer       , intent(in)    :: max_ptrs
      integer       , intent(in)    :: var_len_global
      integer       , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                       :: npts
      !------------------------------------------------------------------------------------!




      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 1-D vectors (npatches).  Notice that they all use !
      ! npts = csite%npatches.  Add only variables of type 31 that are not fmean, dmean,   !
      ! mmean, or mmsqu.                                                                   !
      !------------------------------------------------------------------------------------!
      npts = csite%npatches

      if (associated(csite%age)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%age,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'AGE :31:hist:anal:dail:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%area)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%area,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'AREA :31:hist:anal:dail:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%fast_soil_C)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%fast_soil_C,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'FAST_SOIL_C :31:hist:year:anal') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%slow_soil_C)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%slow_soil_C,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'SLOW_SOIL_C :31:hist:year:anal') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%structural_soil_C)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%structural_soil_C,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'STRUCTURAL_SOIL_C :31:hist:year:anal') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%structural_soil_L)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%structural_soil_L,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'STRUCTURAL_SOIL_L :31:hist:year:anal') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%mineralized_soil_N)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%mineralized_soil_N,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'MINERALIZED_SOIL_N :31:hist:year:anal') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%fast_soil_N)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%fast_soil_N,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'FAST_SOIL_N :31:hist:year:anal') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if
      
      if (associated(csite%sum_dgd)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%sum_dgd,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'SUM_DGD :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%sum_chd)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%sum_chd,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'SUM_CHD :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%can_theiv)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%can_theiv,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'CAN_THEIV :31:hist') 
         call metadata_edio(nvar,igr,'Canopy air ice-vapour equivalent potential temperature','[K]','NA') 
      end if

      if (associated(csite%can_vpdef)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%can_vpdef,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'CAN_VPDEF :31:hist') 
         call metadata_edio(nvar,igr,'Canopy air ice-vapour equivalent potential temperature','[K]','NA') 
      end if

      if (associated(csite%can_temp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%can_temp,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'CAN_TEMP :31:hist') 
         call metadata_edio(nvar,igr,'Canopy air temperature','[K]','NA') 
      end if

      if (associated(csite%can_temp_pv)) then
         nvar=nvar+1
         call vtable_edio_r(npts,csite%can_temp_pv,nvar,igr,init,csite%paglob_id, &
              var_len,var_len_global,max_ptrs,'CAN_TEMP_PV :31:hist') 
         call metadata_edio(nvar,igr,'Canopy air temperature at previous step','[K]','NA') 
      end if

      if (associated(csite%can_shv)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%can_shv,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'CAN_SHV :31:hist') 
         call metadata_edio(nvar,igr,'Canopy air specific humidity','[kg_H2O/kg_Air]','NA') 
      end if

      if (associated(csite%can_co2)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%can_co2,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'CAN_CO2 :31:hist') 
         call metadata_edio(nvar,igr,'Canopy air CO2 mixing ratio','[umol/mol]','NA') 
      end if
   
      if (associated(csite%can_rhos)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%can_rhos,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'CAN_RHOS :31:hist') 
         call metadata_edio(nvar,igr,'Canopy air Density','[kg/m3]','NA') 
      end if
   
      if (associated(csite%can_prss)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%can_prss,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'CAN_PRSS :31:hist') 
         call metadata_edio(nvar,igr,'Canopy air Pressure','[Pa]','NA') 
      end if
   
      if (associated(csite%can_theta)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%can_theta,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'CAN_THETA :31:hist') 
         call metadata_edio(nvar,igr,'Canopy air Potential temperature','[K]','NA') 
      end if
     
      if (associated(csite%can_depth)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%can_depth,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'CAN_DEPTH :31:hist') 
         call metadata_edio(nvar,igr,'Canopy depth','[m]','NA') 
      end if
     
      if (associated(csite%opencan_frac)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%opencan_frac,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'OPENCAN_FRAC :31:hist')
           call metadata_edio(nvar,igr,'Fraction of open canopy','[---]','NA') 
      end if
     
      if (associated(csite%ggbare)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%ggbare,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'GGBARE :31:hist')
           call metadata_edio(nvar,igr,'Bare ground conductance','[m/s]','NA') 
      end if

      if (associated(csite%ggveg)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%ggveg,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'GGVEG :31:hist')
           call metadata_edio(nvar,igr,'Vegetated ground conductance','[m/s]','NA') 
      end if

      if (associated(csite%ggnet)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%ggnet,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'GGNET :31:hist')
           call metadata_edio(nvar,igr,'Net ground conductance','[m/s]','NA') 
      end if

      if (associated(csite%ggsoil)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%ggsoil,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'GGSOIL :31:hist')
           call metadata_edio(nvar,igr,'Soil conductance for evaporation','[m/s]','NA') 
      end if

      if (associated(csite%ground_shv)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%ground_shv,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'GROUND_SHV :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%ground_ssh)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%ground_ssh,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'GROUND_SSH :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%ground_temp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%ground_temp,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'GROUND_TEMP :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%ground_fliq)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%ground_fliq,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'GROUND_FLIQ :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%rough)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%rough,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'ROUGH :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%wbudget_loss2atm)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%wbudget_loss2atm,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'WBUDGET_LOSS2ATM :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%wbudget_denseffect)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%wbudget_denseffect,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'WBUDGET_DENSEFFECT :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%wbudget_precipgain)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%wbudget_precipgain,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'WBUDGET_PRECIPGAIN :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%wbudget_loss2runoff)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%wbudget_loss2runoff,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'WBUDGET_LOSS2RUNOFF :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%wbudget_loss2drainage)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%wbudget_loss2drainage,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'WBUDGET_LOSS2DRAINAGE :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%wbudget_initialstorage)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%wbudget_initialstorage,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'WBUDGET_INITIALSTORAGE :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%wbudget_residual)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%wbudget_residual,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'WBUDGET_RESIDUAL :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%ebudget_loss2atm)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%ebudget_loss2atm,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'EBUDGET_LOSS2ATM :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%ebudget_denseffect)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%ebudget_denseffect,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'EBUDGET_DENSEFFECT :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%ebudget_prsseffect)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%ebudget_prsseffect,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'EBUDGET_PRSSEFFECT :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%ebudget_loss2runoff)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%ebudget_loss2runoff,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'EBUDGET_LOSS2RUNOFF :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%ebudget_loss2drainage)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%ebudget_loss2drainage,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'EBUDGET_LOSS2DRAINAGE :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%ebudget_netrad)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%ebudget_netrad,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'EBUDGET_NETRAD :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%ebudget_precipgain)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%ebudget_precipgain,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'EBUDGET_PRECIPGAIN :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%ebudget_initialstorage)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%ebudget_initialstorage,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'EBUDGET_INITIALSTORAGE :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%ebudget_residual)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%ebudget_residual,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'EBUDGET_RESIDUAL :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%co2budget_initialstorage)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%co2budget_initialstorage,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'CO2BUDGET_INITIALSTORAGE :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%co2budget_residual)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%co2budget_residual,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'CO2BUDGET_RESIDUAL :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%co2budget_loss2atm)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%co2budget_loss2atm,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'CO2BUDGET_LOSS2ATM :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%co2budget_denseffect)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%co2budget_denseffect,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'CO2BUDGET_DENSEFFECT :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%co2budget_gpp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%co2budget_gpp,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'CO2BUDGET_GPP :31:hist') 
         call metadata_edio(nvar,igr,'Patch total gross primary productivity per timestep','[umol/m2/dtlsm]','NA') 
      end if


      if (associated(csite%avg_daily_temp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%avg_daily_temp,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'AVG_DAILY_TEMP :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if  

      if (associated(csite%avg_monthly_gndwater)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%avg_monthly_gndwater,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'AVG_MONTHLY_GNDWATER :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if  

      if (associated(csite%avg_monthly_waterdef)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%avg_monthly_waterdef,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'AVG_MONTHLY_WATERDEF :31:hist:anal:dail:mont:dcyc') 
         call metadata_edio(nvar,igr,'Running average of water deficit','[kg/m2/30days]','NA') 
      end if  

      if (associated(csite%co2budget_plresp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%co2budget_plresp,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'CO2BUDGET_PLRESP :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%co2budget_rh)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%co2budget_rh,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'CO2BUDGET_RH :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%today_A_decomp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%today_A_decomp,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'TODAY_A_DECOMP :31:hist') 
         call metadata_edio(nvar,igr,'NOT A DIAGNOSTIC-WILL ZERO AT END OF DAY','[NA]','NA') 
      end if

      if (associated(csite%today_Af_decomp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%today_Af_decomp,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'TODAY_AF_DECOMP :31:hist') 
         call metadata_edio(nvar,igr,'NOT A DIAGNOSTIC-WILL ZERO AT END OF DAY','[NA]','NA') 
      end if

      if (associated(csite%veg_rough)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%veg_rough,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'VEG_ROUGH :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%veg_height)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%veg_height ,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'VEG_HEIGHT :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%veg_displace)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%veg_displace ,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'VEG_DISPLACE :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%fsc_in)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%fsc_in,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'FSC_IN :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%ssc_in)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%ssc_in,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'SSC_IN :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%ssl_in)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%ssl_in,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'SSL_IN :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%fsn_in)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%fsn_in,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'FSN_IN :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%total_plant_nitrogen_uptake)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%total_plant_nitrogen_uptake,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'TOTAL_PLANT_NITROGEN_UPTAKE :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%mineralized_N_loss)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%mineralized_N_loss,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'NMIN_LOSS :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if


      if (associated(csite%mineralized_N_input)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%mineralized_N_input,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'NMIN_INPUT :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%rshort_g)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%rshort_g,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'RSHORT_G :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%rshort_g_beam)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%rshort_g_beam,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'RSHORT_G_BEAM :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%rshort_g_diffuse)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%rshort_g_diffuse,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'RSHORT_G_DIFFUSE :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%par_g)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%par_g,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'PAR_G :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%par_g_beam)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%par_g_beam,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'PAR_G_BEAM :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%par_g_diffuse)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%par_g_diffuse,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'PAR_G_DIFFUSE :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%par_b)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%par_b,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'PAR_B :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%par_b_beam)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%par_b_beam,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'PAR_B_BEAM :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%par_b_diffuse)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%par_b_diffuse,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'PAR_B_DIFFUSE :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%nir_b)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%nir_b,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'NIR_B :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%nir_b_beam)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%nir_b_beam,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'NIR_B_BEAM :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%nir_b_diffuse)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%nir_b_diffuse,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'NIR_B_DIFFUSE :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%rlong_g)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%rlong_g,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'RLONG_G :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%rlong_s)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%rlong_s,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'RLONG_S :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%albedo)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%albedo,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'ALBEDO :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%albedo_par )) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%albedo_par ,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'ALBEDO_PAR :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%albedo_nir)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%albedo_nir,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'ALBEDO_NIR :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%rnet)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%rnet,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'RNET :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%rlongup)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%rlongup,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'RLONGUP :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%parup)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%parup,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'PARUP :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%nirup)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%nirup,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'NIRUP :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%rshortup)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%rshortup,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'RSHORTUP :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%rlong_albedo)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%rlong_albedo,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'RLONG_ALBEDO :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%total_sfcw_depth)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%total_sfcw_depth,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'TOTAL_SFCW_DEPTH :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%snowfac)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%snowfac,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'SNOWFAC :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%A_decomp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%A_decomp,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'A_DECOMP :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%f_decomp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%f_decomp,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'F_DECOMP :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%rh)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%rh,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'RH :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%cwd_rh)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%cwd_rh,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'CWD_RH :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%plant_ag_biomass)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%plant_ag_biomass,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'PLANT_AG_BIOMASS :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if
      
      if (associated(csite%htry)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%htry,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'HTRY :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if
      
      if (associated(csite%hprev)) then
         nvar=nvar+1
         call vtable_edio_r(npts,csite%hprev,nvar,igr,init,csite%paglob_id, &
              var_len,var_len_global,max_ptrs,'HPREV :31:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%ustar)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%ustar,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'USTAR :31:hist') 
           call metadata_edio(nvar,igr,'Patch level friction velocity','[m/s]','ipatch') 
      end if

      if (associated(csite%tstar)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%tstar,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'TSTAR :31:hist') 
         call metadata_edio(nvar,igr,'patch level heat transfer atm->canopy','[K]','ipatch') 
      end if

      if (associated(csite%qstar)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%qstar,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'QSTAR :31:hist') 
         call metadata_edio(nvar,igr,'patch level vapor transfer atm->canopy','[kg/kg]','ipatch') 
      end if

      if (associated(csite%cstar)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%cstar,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'CSTAR :31:hist') 
         call metadata_edio(nvar,igr,'patch level co2 transfer atm->canopy','[ppm?]','ipatch') 
      end if

      if (associated(csite%zeta)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%zeta,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'ZETA :31:hist') 
         call metadata_edio(nvar,igr,'patch level height over Obukhov length','[---]','ipatch') 
      end if

      if (associated(csite%ribulk)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%ribulk,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'RIBULK :31:hist') 
         call metadata_edio(nvar,igr,'patch level bulk Richardson number','[---]','ipatch') 
      end if
      
      if (associated(csite%upwp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%upwp,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'UPWP :31:hist') 
         call metadata_edio(nvar,igr,'Vertical flux of U-direction momentum','[kg m^-1 s^-2]','ipatch') 
      end if

      if (associated(csite%tpwp)) then
         nvar=nvar+1
         call vtable_edio_r(npts,csite%tpwp,nvar,igr,init,csite%paglob_id, &
              var_len,var_len_global,max_ptrs,'TPWP :31:hist') 
         call metadata_edio(nvar,igr,'Vertical flux of Heat','[kg K m^-2 s^-1]','ipatch')
      end if

      if (associated(csite%qpwp)) then
         nvar=nvar+1
         call vtable_edio_r(npts,csite%qpwp,nvar,igr,init,csite%paglob_id, &
              var_len,var_len_global,max_ptrs,'QPWP :31:hist') 
         call metadata_edio(nvar,igr,'Vertical flux of Moisture','[kg m^-2 s% -1]','ipatch')
      end if

      if (associated(csite%cpwp)) then
         nvar=nvar+1
         call vtable_edio_r(npts,csite%cpwp,nvar,igr,init,csite%paglob_id, &
              var_len,var_len_global,max_ptrs,'CPWP :31:hist') 
         call metadata_edio(nvar,igr,'Vertical flux of Carbon','[kg umolC mol^-1 m^-2 s-1]','ipatch')
      end if
      
      if (associated(csite%wpwp)) then
         nvar=nvar+1
         call vtable_edio_r(npts,csite%wpwp,nvar,igr,init,csite%paglob_id, &
              var_len,var_len_global,max_ptrs,'WPWP :31:hist') 
         call metadata_edio(nvar,igr,'Vertical flux of W-direction momentum','[kg m^-1 s^-2]','ipatch')
      end if
      
      if (associated(csite%par_l_max)) then
         nvar=nvar+1
         call vtable_edio_r(npts,csite%par_l_max,nvar,igr,init,csite%paglob_id, &
              var_len,var_len_global,max_ptrs,'PAR_L_MAX :31:hist') 
         call metadata_edio(nvar,igr,'Maximum PAR - not an output variable','[W/m2]','ipatch')
      end if
      
      if (associated(csite%par_l_beam_max)) then
         nvar=nvar+1
         call vtable_edio_r(npts,csite%par_l_beam_max,nvar,igr,init,csite%paglob_id, &
              var_len,var_len_global,max_ptrs,'PAR_L_BEAM_MAX :31:hist') 
         call metadata_edio(nvar,igr,'Maximum direct PAR - not an output variable' &
                           ,'[W/m2]','ipatch')
      end if
      
      if (associated(csite%par_l_diffuse_max)) then
         nvar=nvar+1
         call vtable_edio_r(npts,csite%par_l_diffuse_max,nvar,igr,init,csite%paglob_id, &
              var_len,var_len_global,max_ptrs,'PAR_L_DIFFUSE_MAX :31:hist') 
         call metadata_edio(nvar,igr,'Maximum diffuse PAR - not an output variable' &
                           ,'[W/m2]','ipatch')
      end if
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_sitetype_p31inst
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the patch-level variables           !
   ! (sitetype) that have one dimension, are real (type 31), and are sub-daily averages.   !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_sitetype_p31fmean(csite,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(sitetype)        , target        :: csite
      integer               , intent(in)    :: init
      integer               , intent(in)    :: igr
      integer               , intent(in)    :: var_len
      integer               , intent(in)    :: max_ptrs
      integer               , intent(in)    :: var_len_global
      integer               , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                               :: npts
      character(len=str_len)                :: fast_keys
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !      Decide whether to write the sub-daily means to the history file.              !
      !------------------------------------------------------------------------------------!
      select case (iadd_patch_means)
      case (0)
         return
      case (1)
         if (history_fast) then
            fast_keys = 'hist:anal'
         else
            fast_keys = 'anal'
         end if
      end select
      !------------------------------------------------------------------------------------!




      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 1-D vectors (npatches).  Notice that they all use !
      ! npts = csite%npatches.  Add only variables of type 31 that are sub-daily means.    !
      !------------------------------------------------------------------------------------!
      npts = csite%npatches

      !------ Averaged properties. --------------------------------------------------------!
      if (associated(csite%fmean_rh              )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_rh                                            &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_RH_PA                :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Heterotrophic respiration'                   &
                           ,'[   kg/m2/yr]','(ipatch)'            )
      end if
      if (associated(csite%fmean_cwd_rh          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_cwd_rh                                        &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CWD_RH_PA            :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Coarse woody debris respiration'             &
                           ,'[   kg/m2/yr]','(ipatch)'            )
      end if
      if (associated(csite%fmean_nep             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_nep                                           &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_NEP_PA               :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Net Ecosystem productivity'                  &
                           ,'[   kg/m2/yr]','(ipatch)'            )
      end if
      if (associated(csite%fmean_rk4step         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_rk4step                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_RK4STEP_PA           :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - RK4 time step'                               &
                           ,'[          s]','(ipatch)'            )
      end if
      if (associated(csite%fmean_available_water )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_available_water                               &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_AVAILABLE_WATER_PA   :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Available water'                             &
                           ,'[      kg/m2]','(ipatch)'            )
      end if
      if (associated(csite%fmean_can_theiv       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_can_theiv                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CAN_THEIV_PA         :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Ice-vapour equiv. pot. temperature - CAS'    &
                           ,'[          K]','(ipatch)'            )
      end if
      if (associated(csite%fmean_can_theta       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_can_theta                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CAN_THETA_PA         :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Potential temperature - CAS'                 &
                           ,'[          K]','(ipatch)'            )
      end if
      if (associated(csite%fmean_can_vpdef       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_can_vpdef                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CAN_VPDEF_PA         :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Vapour pressure deficit - CAS'               &
                           ,'[         Pa]','(ipatch)'            )
      end if
      if (associated(csite%fmean_can_temp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_can_temp                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CAN_TEMP_PA          :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Temperature - CAS'                           &
                           ,'[          K]','(ipatch)'            )
      end if
      if (associated(csite%fmean_can_shv         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_can_shv                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CAN_SHV_PA           :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Specific humidity - CAS'                     &
                           ,'[      kg/kg]','(ipatch)'            )
      end if
      if (associated(csite%fmean_can_co2         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_can_co2                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CAN_CO2_PA           :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - CO2 mixing ratio - CAS'                      &
                           ,'[   umol/mol]','(ipatch)'            )
      end if
      if (associated(csite%fmean_can_rhos        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_can_rhos                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CAN_RHOS_PA          :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Air density - CAS'                           &
                           ,'[      kg/m3]','(ipatch)'            )
      end if
      if (associated(csite%fmean_can_prss        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_can_prss                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CAN_PRSS_PA          :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Pressure - CAS'                              &
                           ,'[         Pa]','(ipatch)'            )
      end if
      if (associated(csite%fmean_gnd_temp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_gnd_temp                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_GND_TEMP_PA          :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Ground temperature'                          &
                           ,'[          K]','(ipatch)'            )
      end if
      if (associated(csite%fmean_gnd_shv         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_gnd_shv                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_GND_SHV_PA           :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Ground specific humidity'                    &
                           ,'[      kg/kg]','(ipatch)'            )
      end if
      if (associated(csite%fmean_can_ggnd        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_can_ggnd                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CAN_GGND_PA          :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Net canopy conductance'                      &
                           ,'[        m/2]','(ipatch)'            )
      end if
      if (associated(csite%fmean_sfcw_depth      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_sfcw_depth                                    &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SFCW_DEPTH_PA        :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Depth - temporary water layer'               &
                           ,'[          m]','(ipatch)'            )
      end if
      if (associated(csite%fmean_sfcw_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_sfcw_energy                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SFCW_ENERGY_PA       :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Internal energy - temporary water layer'     &
                           ,'[       J/kg]','(ipatch)'            )
      end if
      if (associated(csite%fmean_sfcw_mass       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_sfcw_mass                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SFCW_MASS_PA         :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Water mass - temporary water layer'          &
                           ,'[      kg/m2]','(ipatch)'            )
      end if
      if (associated(csite%fmean_sfcw_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_sfcw_temp                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SFCW_TEMP_PA         :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Temperature - temporary water layer'         &
                           ,'[          K]','(ipatch)'            )
      end if
      if (associated(csite%fmean_sfcw_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_sfcw_fliq                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SFCW_FLIQ_PA         :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Liquid fraction - temporary water layer'     &
                           ,'[         --]','(ipatch)'            )
      end if
      if (associated(csite%fmean_rshort_gnd      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_rshort_gnd                                    &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_RSHORT_GND_PA        :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Shortwave radiation absorbed by ground'      &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%fmean_par_gnd         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_par_gnd                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_PAR_GND_PA           :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - PAR absorbed by ground'                      &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%fmean_rlong_gnd       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_rlong_gnd                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_RLONG_GND_PA         :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Longwave radiation absorbed by ground'       &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%fmean_rlongup         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_rlongup                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_RLONGUP_PA           :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Outgoing longwave radiation'                 &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%fmean_parup           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_parup                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_PARUP_PA             :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Outgoing PAR'                                &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%fmean_nirup           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_nirup                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_NIRUP_PA             :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Outgoing near infra-red radiation'           &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%fmean_rshortup        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_rshortup                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_RSHORTUP_PA          :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Outgoing shortwave radiation'                &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%fmean_rnet            )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_rnet                                          &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_RNET_PA              :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Net radiation at top of canopy'              &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%fmean_albedo          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_albedo                                        &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ALBEDO_PA            :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Albedo'                                      &
                           ,'[       ----]','(ipatch)'            )
      end if
      if (associated(csite%fmean_albedo_par      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_albedo_par                                    &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ALBEDO_PAR_PA        :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Albedo - direct radiation'                   &
                           ,'[       ----]','(ipatch)'            )
      end if
      if (associated(csite%fmean_albedo_nir      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_albedo_nir                                    &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_ALBEDO_NIR_PA        :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Albedo - diffuse radiation'                  &
                           ,'[       ----]','(ipatch)'            )
      end if
      if (associated(csite%fmean_rlong_albedo    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_rlong_albedo                                  &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_RLONG_ALBEDO_PA      :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Kind of albedo - longwave radiation'         &
                           ,'[       ----]','(ipatch)'            )
      end if
      if (associated(csite%fmean_ustar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_ustar                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_USTAR_PA             :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Friction velocity'                           &
                           ,'[        m/s]','(ipatch)'            )
      end if
      if (associated(csite%fmean_tstar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_tstar                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_TSTAR_PA             :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Gradient scale for potential temperature'    &
                           ,'[          K]','(ipatch)'            )
      end if
      if (associated(csite%fmean_qstar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_qstar                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_QSTAR_PA             :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Gradient scale for specific humidity'        &
                           ,'[      kg/kg]','(ipatch)'            )
      end if
      if (associated(csite%fmean_cstar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_cstar                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CSTAR_PA             :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Gradient scale for CO2 mixing ratio'         &
                           ,'[   umol/mol]','(ipatch)'            )
      end if
      if (associated(csite%fmean_carbon_ac       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_carbon_ac                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CARBON_AC_PA         :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - CO2 flux: atmosphere -> CAS'                 &
                           ,'[  umol/m2/s]','(ipatch)'            )
      end if
      if (associated(csite%fmean_carbon_st       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_carbon_st                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_CARBON_ST_PA         :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - CO2 storage at CAS'                          &
                           ,'[  umol/m2/s]','(ipatch)'            )
      end if
      if (associated(csite%fmean_vapor_gc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_vapor_gc                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_VAPOR_GC_PA          :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Water vapour flux: ground -> CAS'            &
                           ,'[    kg/m2/s]','(ipatch)'            )
      end if
      if (associated(csite%fmean_vapor_ac        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_vapor_ac                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_VAPOR_AC_PA          :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Water vapour flux: atmosphere -> CAS'        &
                           ,'[    kg/m2/s]','(ipatch)'            )
      end if
      if (associated(csite%fmean_throughfall     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_throughfall                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_THROUGHFALL_PA       :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Throughfall rate'                            &
                           ,'[    kg/m2/s]','(ipatch)'            )
      end if
      if (associated(csite%fmean_runoff          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_runoff                                        &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_RUNOFF_PA            :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Water runoff'                                &
                           ,'[    kg/m2/s]','(ipatch)'            )
      end if
      if (associated(csite%fmean_drainage        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_drainage                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_DRAINAGE_PA          :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Water drainage'                              &
                           ,'[    kg/m2/s]','(ipatch)'            )
      end if
      if (associated(csite%fmean_sensible_gc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_sensible_gc                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SENSIBLE_GC_PA       :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Sensible heat flux: ground -> CAS'           &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%fmean_sensible_ac     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_sensible_ac                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SENSIBLE_AC_PA       :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Sensible heat flux: atmosphere -> CAS'       &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%fmean_qthroughfall    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_qthroughfall                                  &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_QTHROUGHFALL_PA      :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Throughfall rate'                            &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%fmean_qrunoff         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_qrunoff                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_QRUNOFF_PA           :31:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Surface runoff'                              &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_sitetype_p31fmean
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the patch-level variables           !
   ! (sitetype) that have one dimension, are real (type 31), and are daily averages.       !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_sitetype_p31dmean(csite,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(sitetype)        , target        :: csite
      integer               , intent(in)    :: init
      integer               , intent(in)    :: igr
      integer               , intent(in)    :: var_len
      integer               , intent(in)    :: max_ptrs
      integer               , intent(in)    :: var_len_global
      integer               , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                               :: npts
      character(len=str_len)                :: dail_keys
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !      Decide whether to write the daily means to the history file.                  !
      !------------------------------------------------------------------------------------!
      select case (iadd_patch_means)
      case (0)
         return
      case (1)
         if (history_dail) then
            dail_keys = 'hist:dail'
         else
            dail_keys = 'dail'
         end if
      end select
      !------------------------------------------------------------------------------------!




      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 1-D vectors (npatches).  Notice that they all use !
      ! npts = csite%npatches.  Add only variables of type 31 that are daily means.        !
      !------------------------------------------------------------------------------------!
      npts = csite%npatches


      if (associated(csite%dmean_A_decomp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_A_decomp                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_A_DECOMP_PA          :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Decomposition scale'                             &
                           ,'[         --]','(ipatch)'            )
      end if
      if (associated(csite%dmean_Af_decomp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_Af_decomp                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_AF_DECOMP_PA         :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Decomposition scale with N immobilisation'       &
                           ,'[         --]','(ipatch)'            )
      end if
      if (associated(csite%dmean_co2_residual    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_co2_residual                                  &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CO2_RESIDUAL_PA      :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Budget check - CO2 residual'                     &
                           ,'[    umol/m2]','(ipatch)'            )
      end if
      if (associated(csite%dmean_energy_residual )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_energy_residual                               &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ENERGY_RESIDUAL_PA   :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Budget check - Enthalpy residual'                &
                           ,'[       J/m2]','(ipatch)'            )
      end if
      if (associated(csite%dmean_water_residual  )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_water_residual                                &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_WATER_RESIDUAL_PA    :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Budget check - Water residual'                   &
                           ,'[      kg/m2]','(ipatch)'            )
      end if

      if (associated(csite%dmean_rh              )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_rh                                            &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_RH_PA                :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Heterotrophic respiration'                       &
                           ,'[   kg/m2/yr]','(ipatch)'            )
      end if
      if (associated(csite%dmean_cwd_rh          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_cwd_rh                                        &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CWD_RH_PA            :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Coarse woody debris respiration'                 &
                           ,'[   kg/m2/yr]','(ipatch)'            )
      end if
      if (associated(csite%dmean_nep             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_nep                                           &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_NEP_PA               :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Net Ecosystem productivity'                      &
                           ,'[   kg/m2/yr]','(ipatch)'            )
      end if
      if (associated(csite%dmean_rk4step         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_rk4step                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_RK4STEP_PA           :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - RK4 time step'                                   &
                           ,'[          s]','(ipatch)'            )
      end if
      if (associated(csite%dmean_available_water )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_available_water                               &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_AVAILABLE_WATER_PA   :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Available water'                                 &
                           ,'[      kg/m2]','(ipatch)'            )
      end if
      if (associated(csite%dmean_can_theiv       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_can_theiv                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CAN_THEIV_PA         :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Ice-vapour equiv. pot. temperature - CAS'        &
                           ,'[          K]','(ipatch)'            )
      end if
      if (associated(csite%dmean_can_theta       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_can_theta                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CAN_THETA_PA         :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Potential temperature - CAS'                     &
                           ,'[          K]','(ipatch)'            )
      end if
      if (associated(csite%dmean_can_vpdef       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_can_vpdef                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CAN_VPDEF_PA         :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Vapour pressure deficit - CAS'                   &
                           ,'[         Pa]','(ipatch)'            )
      end if
      if (associated(csite%dmean_can_temp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_can_temp                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CAN_TEMP_PA          :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Temperature - CAS'                               &
                           ,'[          K]','(ipatch)'            )
      end if
      if (associated(csite%dmean_can_shv         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_can_shv                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CAN_SHV_PA           :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Specific humidity - CAS'                         &
                           ,'[      kg/kg]','(ipatch)'            )
      end if
      if (associated(csite%dmean_can_co2         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_can_co2                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CAN_CO2_PA           :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - CO2 mixing ratio - CAS'                          &
                           ,'[   umol/mol]','(ipatch)'            )
      end if
      if (associated(csite%dmean_can_rhos        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_can_rhos                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CAN_RHOS_PA          :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Air density - CAS'                               &
                           ,'[      kg/m3]','(ipatch)'            )
      end if
      if (associated(csite%dmean_can_prss        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_can_prss                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CAN_PRSS_PA          :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Pressure - CAS'                                  &
                           ,'[         Pa]','(ipatch)'            )
      end if
      if (associated(csite%dmean_gnd_temp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_gnd_temp                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_GND_TEMP_PA          :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Ground temperature'                              &
                           ,'[          K]','(ipatch)'            )
      end if
      if (associated(csite%dmean_gnd_shv         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_gnd_shv                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_GND_SHV_PA           :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Ground specific humidity'                        &
                           ,'[      kg/kg]','(ipatch)'            )
      end if
      if (associated(csite%dmean_can_ggnd        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_can_ggnd                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CAN_GGND_PA          :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Net canopy conductance'                          &
                           ,'[        m/2]','(ipatch)'            )
      end if
      if (associated(csite%dmean_sfcw_depth      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_sfcw_depth                                    &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SFCW_DEPTH_PA        :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Depth - temporary water layer'                   &
                           ,'[          m]','(ipatch)'            )
      end if
      if (associated(csite%dmean_sfcw_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_sfcw_energy                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SFCW_ENERGY_PA       :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Internal energy - temporary water layer'         &
                           ,'[       J/kg]','(ipatch)'            )
      end if
      if (associated(csite%dmean_sfcw_mass       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_sfcw_mass                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SFCW_MASS_PA         :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Water mass - temporary water layer'              &
                           ,'[      kg/m2]','(ipatch)'            )
      end if
      if (associated(csite%dmean_sfcw_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_sfcw_temp                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SFCW_TEMP_PA         :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Temperature - temporary water layer'             &
                           ,'[          K]','(ipatch)'            )
      end if
      if (associated(csite%dmean_sfcw_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_sfcw_fliq                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SFCW_FLIQ_PA         :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Liquid fraction - temporary water layer'         &
                           ,'[         --]','(ipatch)'            )
      end if
      if (associated(csite%dmean_rshort_gnd      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_rshort_gnd                                    &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_RSHORT_GND_PA        :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Shortwave radiation absorbed by ground'          &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%dmean_par_gnd         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_par_gnd                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_PAR_GND_PA           :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - PAR absorbed by ground'                          &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%dmean_rlong_gnd       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_rlong_gnd                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_RLONG_GND_PA         :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Longwave radiation absorbed by ground'           &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%dmean_rlongup         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_rlongup                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_RLONGUP_PA           :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Outgoing longwave radiation'                     &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%dmean_parup           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_parup                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_PARUP_PA             :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Outgoing PAR'                                    &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%dmean_nirup           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_nirup                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_NIRUP_PA             :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Outgoing near infra-red radiation'               &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%dmean_rshortup        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_rshortup                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_RSHORTUP_PA          :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Outgoing shortwave radiation'                    &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%dmean_rnet            )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_rnet                                          &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_RNET_PA              :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Net radiation at top of canopy'                  &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%dmean_albedo          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_albedo                                        &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ALBEDO_PA            :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Albedo'                                          &
                           ,'[       ----]','(ipatch)'            )
      end if
      if (associated(csite%dmean_albedo_par      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_albedo_par                                    &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ALBEDO_PAR_PA        :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Albedo - direct radiation'                       &
                           ,'[       ----]','(ipatch)'            )
      end if
      if (associated(csite%dmean_albedo_nir      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_albedo_nir                                    &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_ALBEDO_NIR_PA        :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Albedo - diffuse radiation'                      &
                           ,'[       ----]','(ipatch)'            )
      end if
      if (associated(csite%dmean_rlong_albedo    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_rlong_albedo                                  &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_RLONG_ALBEDO_PA      :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Kind of albedo - longwave radiation'             &
                           ,'[       ----]','(ipatch)'            )
      end if
      if (associated(csite%dmean_ustar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_ustar                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_USTAR_PA             :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Friction velocity'                               &
                           ,'[        m/s]','(ipatch)'            )
      end if
      if (associated(csite%dmean_tstar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_tstar                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_TSTAR_PA             :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Gradient scale for potential temperature'        &
                           ,'[          K]','(ipatch)'            )
      end if
      if (associated(csite%dmean_qstar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_qstar                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_QSTAR_PA             :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Gradient scale for specific humidity'            &
                           ,'[      kg/kg]','(ipatch)'            )
      end if
      if (associated(csite%dmean_cstar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_cstar                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CSTAR_PA             :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Gradient scale for CO2 mixing ratio'             &
                           ,'[   umol/mol]','(ipatch)'            )
      end if
      if (associated(csite%dmean_carbon_ac       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_carbon_ac                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CARBON_AC_PA         :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - CO2 flux: atmosphere -> CAS'                     &
                           ,'[  umol/m2/s]','(ipatch)'            )
      end if
      if (associated(csite%dmean_carbon_st       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_carbon_st                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_CARBON_ST_PA         :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - CO2 storage at CAS'                              &
                           ,'[  umol/m2/s]','(ipatch)'            )
      end if
      if (associated(csite%dmean_vapor_gc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_vapor_gc                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_VAPOR_GC_PA          :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Water vapour flux: ground -> CAS'                &
                           ,'[    kg/m2/s]','(ipatch)'            )
      end if
      if (associated(csite%dmean_vapor_ac        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_vapor_ac                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_VAPOR_AC_PA          :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Water vapour flux: atmosphere -> CAS'            &
                           ,'[    kg/m2/s]','(ipatch)'            )
      end if
      if (associated(csite%dmean_throughfall     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_throughfall                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_THROUGHFALL_PA       :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Throughfall rate'                                &
                           ,'[    kg/m2/s]','(ipatch)'            )
      end if
      if (associated(csite%dmean_runoff          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_runoff                                        &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_RUNOFF_PA            :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Water runoff'                                    &
                           ,'[    kg/m2/s]','(ipatch)'            )
      end if
      if (associated(csite%dmean_drainage        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_drainage                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_DRAINAGE_PA          :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Water drainage'                                  &
                           ,'[    kg/m2/s]','(ipatch)'            )
      end if
      if (associated(csite%dmean_sensible_gc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_sensible_gc                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SENSIBLE_GC_PA       :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Sensible heat flux: ground -> CAS'               &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%dmean_sensible_ac     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_sensible_ac                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SENSIBLE_AC_PA       :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Sensible heat flux: atmosphere -> CAS'           &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%dmean_qthroughfall    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_qthroughfall                                  &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_QTHROUGHFALL_PA      :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Throughfall rate'                                &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%dmean_qrunoff         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_qrunoff                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_QRUNOFF_PA           :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Surface runoff'                                  &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%dmean_qdrainage       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_qdrainage                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_QDRAINAGE_PA         :31:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Drainage energy loss'                            &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_sitetype_p31dmean
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the patch-level variables           !
   ! (sitetype) that have one dimension, are real (type 31), and are monthly means or mean !
   ! sum of squares.                                                                       !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_sitetype_p31mmean(csite,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(sitetype)        , target        :: csite
      integer               , intent(in)    :: init
      integer               , intent(in)    :: igr
      integer               , intent(in)    :: var_len
      integer               , intent(in)    :: max_ptrs
      integer               , intent(in)    :: var_len_global
      integer               , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                               :: npts
      character(len=str_len)                :: eorq_keys
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !      Decide whether to write the monthly means to the history file.                !
      !------------------------------------------------------------------------------------!
      select case (iadd_patch_means)
      case (0)
         return
      case (1)
         if (history_eorq) then
            eorq_keys = 'hist:mont:dcyc'
         else
            eorq_keys = 'mont:dcyc'
         end if
      end select
      !------------------------------------------------------------------------------------!




      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 1-D vectors (npatches).  Notice that they all use !
      ! npts = csite%npatches.  Add only variables of type 31 that are monthly means.      !
      !------------------------------------------------------------------------------------!
      npts = csite%npatches

      if (associated(csite%mmean_rh              )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_rh                                            &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_RH_PA                :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Heterotrophic respiration'                     &
                           ,'[   kg/m2/yr]','(ipatch)'            )
      end if
      if (associated(csite%mmean_cwd_rh          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_cwd_rh                                        &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CWD_RH_PA            :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Coarse woody debris respiration'               &
                           ,'[   kg/m2/yr]','(ipatch)'            )
      end if
      if (associated(csite%mmean_nep             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_nep                                           &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_NEP_PA               :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Net Ecosystem productivity'                    &
                           ,'[   kg/m2/yr]','(ipatch)'            )
      end if
      if (associated(csite%mmean_rk4step         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_rk4step                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_RK4STEP_PA           :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - RK4 time step'                                 &
                           ,'[          s]','(ipatch)'            )
      end if
      if (associated(csite%mmean_available_water )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_available_water                               &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_AVAILABLE_WATER_PA   :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Available water'                               &
                           ,'[      kg/m2]','(ipatch)'            )
      end if
      if (associated(csite%mmean_can_theiv       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_can_theiv                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CAN_THEIV_PA         :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Ice-vapour equiv. pot. temperature - CAS'      &
                           ,'[          K]','(ipatch)'            )
      end if
      if (associated(csite%mmean_can_theta       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_can_theta                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CAN_THETA_PA         :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Potential temperature - CAS'                   &
                           ,'[          K]','(ipatch)'            )
      end if
      if (associated(csite%mmean_can_vpdef       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_can_vpdef                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CAN_VPDEF_PA         :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Vapour pressure deficit - CAS'                 &
                           ,'[         Pa]','(ipatch)'            )
      end if
      if (associated(csite%mmean_can_temp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_can_temp                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CAN_TEMP_PA          :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Temperature - CAS'                             &
                           ,'[          K]','(ipatch)'            )
      end if
      if (associated(csite%mmean_can_shv         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_can_shv                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CAN_SHV_PA           :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Specific humidity - CAS'                       &
                           ,'[      kg/kg]','(ipatch)'            )
      end if
      if (associated(csite%mmean_can_co2         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_can_co2                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CAN_CO2_PA           :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - CO2 mixing ratio - CAS'                        &
                           ,'[   umol/mol]','(ipatch)'            )
      end if
      if (associated(csite%mmean_can_rhos        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_can_rhos                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CAN_RHOS_PA          :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Air density - CAS'                             &
                           ,'[      kg/m3]','(ipatch)'            )
      end if
      if (associated(csite%mmean_can_prss        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_can_prss                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CAN_PRSS_PA          :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Pressure - CAS'                                &
                           ,'[         Pa]','(ipatch)'            )
      end if
      if (associated(csite%mmean_gnd_temp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_gnd_temp                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_GND_TEMP_PA          :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Ground temperature'                            &
                           ,'[          K]','(ipatch)'            )
      end if
      if (associated(csite%mmean_gnd_shv         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_gnd_shv                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_GND_SHV_PA           :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Ground specific humidity'                      &
                           ,'[      kg/kg]','(ipatch)'            )
      end if
      if (associated(csite%mmean_can_ggnd        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_can_ggnd                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CAN_GGND_PA          :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Net canopy conductance'                        &
                           ,'[        m/2]','(ipatch)'            )
      end if
      if (associated(csite%mmean_sfcw_depth      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_sfcw_depth                                    &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SFCW_DEPTH_PA        :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Depth - temporary water layer'                 &
                           ,'[          m]','(ipatch)'            )
      end if
      if (associated(csite%mmean_sfcw_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_sfcw_energy                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SFCW_ENERGY_PA       :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Internal energy - temporary water layer'       &
                           ,'[       J/kg]','(ipatch)'            )
      end if
      if (associated(csite%mmean_sfcw_mass       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_sfcw_mass                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SFCW_MASS_PA         :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Water mass - temporary water layer'            &
                           ,'[      kg/m2]','(ipatch)'            )
      end if
      if (associated(csite%mmean_sfcw_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_sfcw_temp                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SFCW_TEMP_PA         :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Temperature - temporary water layer'           &
                           ,'[          K]','(ipatch)'            )
      end if
      if (associated(csite%mmean_sfcw_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_sfcw_fliq                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SFCW_FLIQ_PA         :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Liquid fraction - temporary water layer'       &
                           ,'[         --]','(ipatch)'            )
      end if
      if (associated(csite%mmean_rshort_gnd      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_rshort_gnd                                    &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_RSHORT_GND_PA        :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Shortwave radiation absorbed by ground'        &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%mmean_par_gnd         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_par_gnd                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_PAR_GND_PA           :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - PAR absorbed by ground'                        &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%mmean_rlong_gnd       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_rlong_gnd                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_RLONG_GND_PA         :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Longwave radiation absorbed by ground'         &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%mmean_rlongup         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_rlongup                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_RLONGUP_PA           :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Outgoing longwave radiation'                   &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%mmean_parup           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_parup                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_PARUP_PA             :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Outgoing PAR'                                  &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%mmean_nirup           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_nirup                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_NIRUP_PA             :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Outgoing near infra-red radiation'             &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%mmean_rshortup        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_rshortup                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_RSHORTUP_PA          :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Outgoing shortwave radiation'                  &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%mmean_rnet            )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_rnet                                          &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_RNET_PA              :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Net radiation at top of canopy'                &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%mmean_albedo          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_albedo                                        &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ALBEDO_PA            :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Albedo'                                        &
                           ,'[       ----]','(ipatch)'            )
      end if
      if (associated(csite%mmean_albedo_par      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_albedo_par                                    &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ALBEDO_PAR_PA        :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Albedo - direct radiation'                     &
                           ,'[       ----]','(ipatch)'            )
      end if
      if (associated(csite%mmean_albedo_nir      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_albedo_nir                                    &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ALBEDO_NIR_PA        :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Albedo - diffuse radiation'                    &
                           ,'[       ----]','(ipatch)'            )
      end if
      if (associated(csite%mmean_rlong_albedo    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_rlong_albedo                                  &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_RLONG_ALBEDO_PA      :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Kind of albedo - longwave radiation'           &
                           ,'[       ----]','(ipatch)'            )
      end if
      if (associated(csite%mmean_ustar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_ustar                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_USTAR_PA             :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Friction velocity'                             &
                           ,'[        m/s]','(ipatch)'            )
      end if
      if (associated(csite%mmean_tstar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_tstar                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_TSTAR_PA             :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Gradient scale for potential temperature'      &
                           ,'[          K]','(ipatch)'            )
      end if
      if (associated(csite%mmean_qstar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_qstar                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_QSTAR_PA             :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Gradient scale for specific humidity'          &
                           ,'[      kg/kg]','(ipatch)'            )
      end if
      if (associated(csite%mmean_cstar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_cstar                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CSTAR_PA             :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Gradient scale for CO2 mixing ratio'           &
                           ,'[   umol/mol]','(ipatch)'            )
      end if
      if (associated(csite%mmean_carbon_ac       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_carbon_ac                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CARBON_AC_PA         :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - CO2 flux: atmosphere -> CAS'                   &
                           ,'[  umol/m2/s]','(ipatch)'            )
      end if
      if (associated(csite%mmean_carbon_st       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_carbon_st                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CARBON_ST_PA         :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - CO2 storage at CAS'                            &
                           ,'[  umol/m2/s]','(ipatch)'            )
      end if
      if (associated(csite%mmean_vapor_gc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_vapor_gc                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_VAPOR_GC_PA          :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Water vapour flux: ground -> CAS'              &
                           ,'[    kg/m2/s]','(ipatch)'            )
      end if
      if (associated(csite%mmean_vapor_ac        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_vapor_ac                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_VAPOR_AC_PA          :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Water vapour flux: atmosphere -> CAS'          &
                           ,'[    kg/m2/s]','(ipatch)'            )
      end if
      if (associated(csite%mmean_throughfall     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_throughfall                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_THROUGHFALL_PA       :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Throughfall rate'                              &
                           ,'[    kg/m2/s]','(ipatch)'            )
      end if
      if (associated(csite%mmean_runoff          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_runoff                                        &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_RUNOFF_PA            :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Water runoff'                                  &
                           ,'[    kg/m2/s]','(ipatch)'            )
      end if
      if (associated(csite%mmean_drainage        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_drainage                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_DRAINAGE_PA          :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Water drainage'                                &
                           ,'[    kg/m2/s]','(ipatch)'            )
      end if
      if (associated(csite%mmean_sensible_gc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_sensible_gc                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SENSIBLE_GC_PA       :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Sensible heat flux: ground -> CAS'             &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%mmean_sensible_ac     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_sensible_ac                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SENSIBLE_AC_PA       :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Sensible heat flux: atmosphere -> CAS'         &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%mmean_qthroughfall    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_qthroughfall                                  &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_QTHROUGHFALL_PA      :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Throughfall rate'                              &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%mmean_qrunoff         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_qrunoff                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_QRUNOFF_PA           :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Surface runoff'                                &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%mmean_qdrainage       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_qdrainage                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_QDRAINAGE_PA         :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Drainage energy loss'                          &
                           ,'[       W/m2]','(ipatch)'            )
      end if
      if (associated(csite%mmean_fast_soil_c     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_fast_soil_c                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_FAST_SOIL_C          :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Soil Carbon (Fast pool)'                       &
                           ,'[     kgC/m2]','(ipatch)'            )
      end if
      if (associated(csite%mmean_slow_soil_c     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_slow_soil_c                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SLOW_SOIL_C          :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Soil Carbon (Slow pool)'                       &
                           ,'[     kgC/m2]','(ipatch)'            )
      end if
      if (associated(csite%mmean_struct_soil_c   )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_struct_soil_c                                 &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_STRUCT_SOIL_C        :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Soil Carbon (Structural pool)'                 &
                           ,'[     kgC/m2]','(ipatch)'            )
      end if
      if (associated(csite%mmean_struct_soil_l   )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_struct_soil_l                                 &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_STRUCT_SOIL_L        :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Soil Lignin (Structural pool)'                 &
                           ,'[     kgL/m2]','(ipatch)'            )
      end if
      if (associated(csite%mmean_fast_soil_n     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_fast_soil_n                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_FAST_SOIL_N          :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Soil Nitrogen (Fast pool)'                     &
                           ,'[     kgN/m2]','(ipatch)'            )
      end if
      if (associated(csite%mmean_mineral_soil_n  )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_mineral_soil_n                                &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_MINERAL_SOIL_N       :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Soil Nitrogen (Mineralised pool)'              &
                           ,'[     kgN/m2]','(ipatch)'            )
      end if
      if (associated(csite%mmean_A_decomp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_A_decomp                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_A_DECOMP_PA          :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Decomposition scale'                           &
                           ,'[         --]','(ipatch)'            )
      end if
      if (associated(csite%mmean_Af_decomp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_Af_decomp                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_AF_DECOMP_PA         :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Decomposition scale with N immobilisation'     &
                           ,'[         --]','(ipatch)'            )
      end if
      if (associated(csite%mmean_co2_residual    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_co2_residual                                  &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_CO2_RESIDUAL_PA      :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Budget check - CO2 residual'                   &
                           ,'[    umol/m2]','(ipatch)'            )
      end if
      if (associated(csite%mmean_energy_residual )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_energy_residual                               &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_ENERGY_RESIDUAL_PA   :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Budget check - Enthalpy residual'              &
                           ,'[       J/m2]','(ipatch)'            )
      end if
      if (associated(csite%mmean_water_residual  )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_water_residual                                &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_WATER_RESIDUAL_PA    :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Budget check - Water residual'                 &
                           ,'[      kg/m2]','(ipatch)'            )
      end if
      if (associated(csite%mmsqu_rh              )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmsqu_rh                                            &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_RH_PA                :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Heterotrophic respiration'                     &
                           ,'[ kg2/m4/yr2]','(ipatch)'            )
      end if
      if (associated(csite%mmsqu_cwd_rh          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmsqu_cwd_rh                                        &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_CWD_RH_PA            :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Coarse woody debris respiration'               &
                           ,'[ kg2/m4/yr2]','(ipatch)'            )
      end if
      if (associated(csite%mmsqu_nep             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmsqu_nep                                           &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_NEP_PA               :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Net Ecosystem productivity'                    &
                           ,'[ kg2/m4/yr2]','(ipatch)'            )
      end if
      if (associated(csite%mmsqu_rlongup         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmsqu_rlongup                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_RLONGUP_PA           :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Outgoing longwave radiation'                   &
                           ,'[      W2/m4]','(ipatch)'            )
      end if
      if (associated(csite%mmsqu_parup           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmsqu_parup                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_PARUP_PA             :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Outgoing PAR'                                  &
                           ,'[      W2/m4]','(ipatch)'            )
      end if
      if (associated(csite%mmsqu_nirup           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmsqu_nirup                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_NIRUP_PA             :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Outgoing near infra-red radiation'             &
                           ,'[      W2/m4]','(ipatch)'            )
      end if
      if (associated(csite%mmsqu_rshortup        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmsqu_rshortup                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_RSHORTUP_PA          :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Outgoing shortwave radiation'                  &
                           ,'[      W2/m4]','(ipatch)'            )
      end if
      if (associated(csite%mmsqu_rnet            )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmsqu_rnet                                          &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_RNET_PA              :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Net radiation at top of canopy'                &
                           ,'[      W2/m4]','(ipatch)'            )
      end if
      if (associated(csite%mmsqu_albedo          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmsqu_albedo                                        &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_ALBEDO_PA            :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Albedo'                                        &
                           ,'[       ----]','(ipatch)'            )
      end if
      if (associated(csite%mmsqu_ustar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmsqu_ustar                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_USTAR_PA             :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Friction velocity'                             &
                           ,'[      m2/s2]','(ipatch)'            )
      end if
      if (associated(csite%mmsqu_carbon_ac       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmsqu_carbon_ac                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_CARBON_AC_PA         :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - CO2 flux: atmosphere -> CAS'                   &
                           ,'[umol2/m4/s2]','(ipatch)'            )
      end if
      if (associated(csite%mmsqu_carbon_st       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmsqu_carbon_st                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_CARBON_ST_PA         :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - CO2 storage at CAS'                            &
                           ,'[umol2/m4/s2]','(ipatch)'            )
      end if
      if (associated(csite%mmsqu_vapor_gc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmsqu_vapor_gc                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_VAPOR_GC_PA          :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Water vapour flux: ground -> CAS'              &
                           ,'[  kg2/m4/s2]','(ipatch)'            )
      end if
      if (associated(csite%mmsqu_vapor_ac        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmsqu_vapor_ac                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_VAPOR_AC_PA          :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Water vapour flux: atmosphere -> CAS'          &
                           ,'[  kg2/m4/s2]','(ipatch)'            )
      end if
      if (associated(csite%mmsqu_sensible_gc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmsqu_sensible_gc                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_SENSIBLE_GC_PA       :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Sensible heat flux: ground -> CAS'             &
                           ,'[      W2/m4]','(ipatch)'            )
      end if
      if (associated(csite%mmsqu_sensible_ac     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmsqu_sensible_ac                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMSQU_SENSIBLE_AC_PA       :31:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Sensible heat flux: atmosphere -> CAS'         &
                           ,'[      W2/m4]','(ipatch)'            )
      end if
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_sitetype_p31mmean
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the patch-level variables           !
   ! (sitetype) that have two dimensions (ndcycle,npatches), and are of type -31 (qmean    !
   ! and qmsqu).                                                                           !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_sitetype_m31(csite,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(sitetype)        , target        :: csite
      integer               , intent(in)    :: init
      integer               , intent(in)    :: igr
      integer               , intent(in)    :: var_len
      integer               , intent(in)    :: max_ptrs
      integer               , intent(in)    :: var_len_global
      integer               , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                               :: npts
      character(len=str_len)                :: eorq_keys
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !      Decide whether to write the mean diel to the history file.                    !
      !------------------------------------------------------------------------------------!
      select case (iadd_patch_means)
      case (0)
         return
      case (1)
         if (history_eorq) then
            eorq_keys = 'hist:dcyc'
         else
            eorq_keys = 'dcyc'
         end if
      end select
      !------------------------------------------------------------------------------------!





      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 2-D vectors with dimensions npatches and ndcycle. !
      !  Notice that they all use the same npts.  Here you should only add variables of    !
      ! types -31.                                                                         !
      !------------------------------------------------------------------------------------!
      npts = csite%npatches * ndcycle


      if (associated(csite%qmean_rh              )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_rh                                            &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_RH_PA               :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Heterotrophic respiration'                        &
                           ,'[   kg/m2/yr]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_cwd_rh          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_cwd_rh                                        &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CWD_RH_PA           :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Coarse woody debris respiration'                  &
                           ,'[   kg/m2/yr]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_nep             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_nep                                           &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_NEP_PA              :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Net Ecosystem productivity'                       &
                           ,'[   kg/m2/yr]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_rk4step         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_rk4step                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_RK4STEP_PA          :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - RK4 time step'                                    &
                           ,'[          s]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_available_water )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_available_water                               &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_AVAILABLE_WATER_PA  :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Available water'                                  &
                           ,'[      kg/m2]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_can_theiv       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_can_theiv                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CAN_THEIV_PA        :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Ice-vapour equiv. pot. temperature - CAS'         &
                           ,'[          K]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_can_theta       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_can_theta                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CAN_THETA_PA        :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Potential temperature - CAS'                      &
                           ,'[          K]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_can_vpdef       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_can_vpdef                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CAN_VPDEF_PA        :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Vapour pressure deficit - CAS'                    &
                           ,'[         Pa]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_can_temp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_can_temp                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CAN_TEMP_PA         :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Temperature - CAS'                                &
                           ,'[          K]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_can_shv         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_can_shv                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CAN_SHV_PA          :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Specific humidity - CAS'                          &
                           ,'[      kg/kg]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_can_co2         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_can_co2                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CAN_CO2_PA          :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - CO2 mixing ratio - CAS'                           &
                           ,'[   umol/mol]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_can_rhos        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_can_rhos                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CAN_RHOS_PA         :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Air density - CAS'                                &
                           ,'[      kg/m3]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_can_prss        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_can_prss                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CAN_PRSS_PA         :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Pressure - CAS'                                   &
                           ,'[         Pa]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_gnd_temp        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_gnd_temp                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_GND_TEMP_PA         :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Ground temperature'                               &
                           ,'[          K]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_gnd_shv         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_gnd_shv                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_GND_SHV_PA          :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Ground specific humidity'                         &
                           ,'[      kg/kg]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_can_ggnd        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_can_ggnd                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CAN_GGND_PA         :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Net canopy conductance'                           &
                           ,'[        m/2]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_sfcw_depth      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_sfcw_depth                                    &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SFCW_DEPTH_PA       :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Depth - temporary water layer'                    &
                           ,'[          m]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_sfcw_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_sfcw_energy                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SFCW_ENERGY_PA      :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Internal energy - temporary water layer'          &
                           ,'[       J/kg]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_sfcw_mass       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_sfcw_mass                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SFCW_MASS_PA        :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Water mass - temporary water layer'               &
                           ,'[      kg/m2]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_sfcw_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_sfcw_temp                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SFCW_TEMP_PA        :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Temperature - temporary water layer'              &
                           ,'[          K]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_sfcw_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_sfcw_fliq                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SFCW_FLIQ_PA        :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Liquid fraction - temporary water layer'          &
                           ,'[         --]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_rshort_gnd      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_rshort_gnd                                    &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_RSHORT_GND_PA       :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Shortwave radiation absorbed by ground'           &
                           ,'[       W/m2]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_par_gnd         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_par_gnd                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_PAR_GND_PA          :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - PAR absorbed by ground'                           &
                           ,'[       W/m2]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_rlong_gnd       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_rlong_gnd                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_RLONG_GND_PA        :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Longwave radiation absorbed by ground'            &
                           ,'[       W/m2]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_rlongup         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_rlongup                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_RLONGUP_PA          :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Outgoing longwave radiation'                      &
                           ,'[       W/m2]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_parup           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_parup                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_PARUP_PA            :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Outgoing PAR'                                     &
                           ,'[       W/m2]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_nirup           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_nirup                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_NIRUP_PA            :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Outgoing near infra-red radiation'                &
                           ,'[       W/m2]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_rshortup        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_rshortup                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_RSHORTUP_PA         :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Outgoing shortwave radiation'                     &
                           ,'[       W/m2]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_rnet            )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_rnet                                          &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_RNET_PA             :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Net radiation at top of canopy'                   &
                           ,'[       W/m2]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_albedo          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_albedo                                        &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ALBEDO_PA           :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Albedo'                                           &
                           ,'[       ----]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_albedo_par      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_albedo_par                                    &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ALBEDO_PAR_PA       :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Albedo - direct radiation'                        &
                           ,'[       ----]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_albedo_nir      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_albedo_nir                                    &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_ALBEDO_NIR_PA       :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Albedo - diffuse radiation'                       &
                           ,'[       ----]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_rlong_albedo    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_rlong_albedo                                  &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_RLONG_ALBEDO_PA     :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Kind of albedo - longwave radiation'              &
                           ,'[       ----]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_ustar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_ustar                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_USTAR_PA            :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Friction velocity'                                &
                           ,'[        m/s]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_tstar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_tstar                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_TSTAR_PA            :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Gradient scale for potential temperature'         &
                           ,'[          K]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_qstar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_qstar                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_QSTAR_PA            :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Gradient scale for specific humidity'             &
                           ,'[      kg/kg]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_cstar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_cstar                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CSTAR_PA            :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Gradient scale for CO2 mixing ratio'              &
                           ,'[   umol/mol]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_carbon_ac       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_carbon_ac                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CARBON_AC_PA        :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - CO2 flux: atmosphere -> CAS'                      &
                           ,'[  umol/m2/s]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_carbon_st       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_carbon_st                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_CARBON_ST_PA        :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - CO2 storage at CAS'                               &
                           ,'[  umol/m2/s]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_vapor_gc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_vapor_gc                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_VAPOR_GC_PA         :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Water vapour flux: ground -> CAS'                 &
                           ,'[    kg/m2/s]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_vapor_ac        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_vapor_ac                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_VAPOR_AC_PA         :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Water vapour flux: atmosphere -> CAS'             &
                           ,'[    kg/m2/s]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_throughfall     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_throughfall                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_THROUGHFALL_PA      :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Throughfall rate'                                 &
                           ,'[    kg/m2/s]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_runoff          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_runoff                                        &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_RUNOFF_PA           :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Water runoff'                                     &
                           ,'[    kg/m2/s]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_drainage        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_drainage                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_DRAINAGE_PA         :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Water drainage'                                   &
                           ,'[    kg/m2/s]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_sensible_gc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_sensible_gc                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SENSIBLE_GC_PA      :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Sensible heat flux: ground -> CAS'                &
                           ,'[       W/m2]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_sensible_ac     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_sensible_ac                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SENSIBLE_AC_PA      :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Sensible heat flux: atmosphere -> CAS'            &
                           ,'[       W/m2]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_qthroughfall    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_qthroughfall                                  &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_QTHROUGHFALL_PA     :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Throughfall rate'                                 &
                           ,'[       W/m2]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_qrunoff         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_qrunoff                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_QRUNOFF_PA          :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Surface runoff'                                   &
                           ,'[       W/m2]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmean_qdrainage       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_qdrainage                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_QDRAINAGE_PA        :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Drainage energy loss'                             &
                           ,'[       W/m2]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmsqu_rh              )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmsqu_rh                                            &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_RH_PA               :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Heterotrophic respiration'                        &
                           ,'[ kg2/m4/yr2]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmsqu_cwd_rh          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmsqu_cwd_rh                                        &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_CWD_RH_PA           :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Coarse woody debris respiration'                  &
                           ,'[ kg2/m4/yr2]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmsqu_nep             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmsqu_nep                                           &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_NEP_PA              :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Net Ecosystem productivity'                       &
                           ,'[ kg2/m4/yr2]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmsqu_rlongup         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmsqu_rlongup                                       &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_RLONGUP_PA          :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Outgoing longwave radiation'                      &
                           ,'[      W2/m4]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmsqu_parup           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmsqu_parup                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_PARUP_PA            :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Outgoing PAR'                                     &
                           ,'[      W2/m4]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmsqu_nirup           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmsqu_nirup                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_NIRUP_PA            :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Outgoing near infra-red radiation'                &
                           ,'[      W2/m4]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmsqu_rshortup        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmsqu_rshortup                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_RSHORTUP_PA         :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Outgoing shortwave radiation'                     &
                           ,'[      W2/m4]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmsqu_rnet            )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmsqu_rnet                                          &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_RNET_PA             :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Net radiation at top of canopy'                   &
                           ,'[      W2/m4]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmsqu_albedo          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmsqu_albedo                                        &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_ALBEDO_PA           :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Albedo'                                           &
                           ,'[       ----]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmsqu_ustar           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmsqu_ustar                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_USTAR_PA            :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Friction velocity'                                &
                           ,'[      m2/s2]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmsqu_carbon_ac       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmsqu_carbon_ac                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_CARBON_AC_PA        :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - CO2 flux: atmosphere -> CAS'                      &
                           ,'[umol2/m4/s2]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmsqu_carbon_st       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmsqu_carbon_st                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_CARBON_ST_PA        :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - CO2 storage at CAS'                               &
                           ,'[umol2/m4/s2]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmsqu_vapor_gc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmsqu_vapor_gc                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_VAPOR_GC_PA         :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Water vapour flux: ground -> CAS'                 &
                           ,'[  kg2/m4/s2]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmsqu_vapor_ac        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmsqu_vapor_ac                                      &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_VAPOR_AC_PA         :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Water vapour flux: atmosphere -> CAS'             &
                           ,'[  kg2/m4/s2]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmsqu_sensible_gc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmsqu_sensible_gc                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_SENSIBLE_GC_PA      :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Sensible heat flux: ground -> CAS'                &
                           ,'[      W2/m4]','(ndcycle,ipatch)'    )
      end if
      if (associated(csite%qmsqu_sensible_ac     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmsqu_sensible_ac                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMSQU_SENSIBLE_AC_PA      :-31:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Sensible heat flux: atmosphere -> CAS'            &
                           ,'[      W2/m4]','(ndcycle,ipatch)'    )
      end if
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_sitetype_m31
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the patch-level variables           !
   ! (sitetype) that have two dimensions (nzg,npatches).                                   !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_sitetype_p32(csite,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(sitetype)        , target        :: csite
      integer               , intent(in)    :: init
      integer               , intent(in)    :: igr
      integer               , intent(in)    :: var_len
      integer               , intent(in)    :: max_ptrs
      integer               , intent(in)    :: var_len_global
      integer               , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                               :: npts
      character(len=str_len)                :: fast_keys
      character(len=str_len)                :: dail_keys
      character(len=str_len)                :: eorq_keys
      !------------------------------------------------------------------------------------!




      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 2-D vectors with dimensions npatches and nzg.     !
      !  Notice that they all use the same npts.  Here you should only add variables of    !
      ! types 32 and 320.                                                                  !
      !------------------------------------------------------------------------------------!
      npts = csite%npatches * nzg

      if (associated(csite%soil_energy)) then
         nvar=nvar+1
         call vtable_edio_r(npts,csite%soil_energy                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'SOIL_ENERGY_PA :32:hist') 
         call metadata_edio(nvar,igr,'Instantaneous - Soil internal energy'                &
                           ,'[J/m3]','(nzg,ipatch)') 
      end if

      if (associated(csite%soil_mstpot)) then
         nvar=nvar+1
         call vtable_edio_r(npts,csite%soil_mstpot                                         &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'SOIL_MSTPOT_PA :32:hist') 
         call metadata_edio(nvar,igr,'Instantaneous - Soil matric potential'               &
                           ,'[m]','(nzg,ipatch)') 
      end if

      if (associated(csite%soil_water)) then
         nvar=nvar+1
         call vtable_edio_r(npts,csite%soil_water                                          &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'SOIL_WATER_PA :32:hist')
         call metadata_edio(nvar,igr,'Soil moisture','[m3/m3]','(nzg,ipatch)') 
      end if

      if (associated(csite%soil_tempk)) then
         nvar=nvar+1
         call vtable_edio_r(npts,csite%soil_tempk                                          &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'SOIL_TEMPK_PA :32:hist') 
         call metadata_edio(nvar,igr,'Instantaneous - Soil temperature'                    &
                           ,'[K]','(nzg,ipatch)') 
      end if

      if (associated(csite%soil_fracliq)) then
         nvar=nvar+1
         call vtable_edio_r(npts,csite%soil_fracliq                                        &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'SOIL_FRACLIQ_PA :32:hist') 
         call metadata_edio(nvar,igr,'Instantaneous - Soil liquid water fraction'          &
                           ,'[--]','(nzg,ipatch)') 
      end if

      if (associated(csite%rootdense)) then
         nvar=nvar+1
         call vtable_edio_r(npts,csite%rootdense                                           &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'PATCH_ROOT_DENSITY :32:hist') 
         call metadata_edio(nvar,igr,'Patch level root density with depth'                 &
                           ,'[kg/m3]','(nzg,ipatch)') 
      end if
      !------------------------------------------------------------------------------------!






      !------------------------------------------------------------------------------------!
      !      Decide whether to write the fast, daily, and monthly means to the history.    !
      !------------------------------------------------------------------------------------!
      select case (iadd_patch_means)
      case (0)
         return
      case (1)
         if (history_fast) then
            fast_keys = 'hist:anal'
         else
            fast_keys = 'anal'
         end if
         if (history_dail) then
            dail_keys = 'hist:dail'
         else
            dail_keys = 'dail'
         end if
         if (history_eorq) then
            eorq_keys = 'hist:mont:dcyc'
         else
            eorq_keys = 'mont:dcyc'
         end if
      end select
      !------------------------------------------------------------------------------------!



      if (associated(csite%fmean_soil_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_soil_energy                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SOIL_ENERGY_PA       :32:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Soil internal energy'                        &
                           ,'[       J/m3]','(nzg,ipatch)'        )
      end if
      if (associated(csite%fmean_soil_mstpot     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_soil_mstpot                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SOIL_MSTPOT_PA       :32:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Soil matric potential'                       &
                           ,'[          m]','(nzg,ipatch)'        )
      end if
      if (associated(csite%fmean_soil_water      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_soil_water                                    &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SOIL_WATER_PA        :32:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Soil water content'                          &
                           ,'[      m3/m3]','(nzg,ipatch)'        )
      end if
      if (associated(csite%fmean_soil_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_soil_temp                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SOIL_TEMP_PA         :32:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Soil temperature'                            &
                           ,'[          K]','(nzg,ipatch)'        )
      end if
      if (associated(csite%fmean_soil_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_soil_fliq                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SOIL_FLIQ_PA         :32:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Soil liquid fraction'                        &
                           ,'[         --]','(nzg,ipatch)'        )
      end if
      if (associated(csite%fmean_smoist_gg       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_smoist_gg                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SMOIST_GG_PA         :32:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Soil water flux'                             &
                           ,'[    kg/m2/s]','(nzg,ipatch)'        )
      end if
      if (associated(csite%fmean_transloss       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_transloss                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_TRANSLOSS_PA         :32:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Water loss through transpiration'            &
                           ,'[    kg/m2/s]','(nzg,ipatch)'        )
      end if
      if (associated(csite%fmean_sensible_gg     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%fmean_sensible_gg                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'FMEAN_SENSIBLE_GG_PA       :32:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Soil heat flux'                              &
                           ,'[       W/m2]','(nzg,ipatch)'        )
      end if
      if (associated(csite%dmean_soil_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_soil_energy                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SOIL_ENERGY_PA       :32:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Soil internal energy'                            &
                           ,'[       J/m3]','(nzg,ipatch)'        )
      end if
      if (associated(csite%dmean_soil_mstpot     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_soil_mstpot                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SOIL_MSTPOT_PA       :32:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Soil matric potential'                           &
                           ,'[          m]','(nzg,ipatch)'        )
      end if
      if (associated(csite%dmean_soil_water      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_soil_water                                    &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SOIL_WATER_PA        :32:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Soil water content'                              &
                           ,'[      m3/m3]','(nzg,ipatch)'        )
      end if
      if (associated(csite%dmean_soil_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_soil_temp                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SOIL_TEMP_PA         :32:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Soil temperature'                                &
                           ,'[          K]','(nzg,ipatch)'        )
      end if
      if (associated(csite%dmean_soil_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_soil_fliq                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SOIL_FLIQ_PA         :32:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Soil liquid fraction'                            &
                           ,'[         --]','(nzg,ipatch)'        )
      end if
      if (associated(csite%dmean_smoist_gg       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_smoist_gg                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SMOIST_GG_PA         :32:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Soil water flux'                                 &
                           ,'[    kg/m2/s]','(nzg,ipatch)'        )
      end if
      if (associated(csite%dmean_transloss       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_transloss                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_TRANSLOSS_PA         :32:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Water loss through transpiration'                &
                           ,'[    kg/m2/s]','(nzg,ipatch)'        )
      end if
      if (associated(csite%dmean_sensible_gg     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%dmean_sensible_gg                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'DMEAN_SENSIBLE_GG_PA       :32:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Soil heat flux'                                  &
                           ,'[       W/m2]','(nzg,ipatch)'        )
      end if
      if (associated(csite%mmean_soil_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_soil_energy                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SOIL_ENERGY_PA       :32:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Soil internal energy'                          &
                           ,'[       J/m3]','(nzg,ipatch)'        )
      end if
      if (associated(csite%mmean_soil_mstpot     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_soil_mstpot                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SOIL_MSTPOT_PA       :32:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Soil matric potential'                         &
                           ,'[          m]','(nzg,ipatch)'        )
      end if
      if (associated(csite%mmean_soil_water      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_soil_water                                    &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SOIL_WATER_PA        :32:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Soil water content'                            &
                           ,'[      m3/m3]','(nzg,ipatch)'        )
      end if
      if (associated(csite%mmean_soil_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_soil_temp                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SOIL_TEMP_PA         :32:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Soil temperature'                              &
                           ,'[          K]','(nzg,ipatch)'        )
      end if
      if (associated(csite%mmean_soil_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_soil_fliq                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SOIL_FLIQ_PA         :32:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Soil liquid fraction'                          &
                           ,'[         --]','(nzg,ipatch)'        )
      end if
      if (associated(csite%mmean_smoist_gg       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_smoist_gg                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SMOIST_GG_PA         :32:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Soil water flux'                               &
                           ,'[    kg/m2/s]','(nzg,ipatch)'        )
      end if
      if (associated(csite%mmean_transloss       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_transloss                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_TRANSLOSS_PA         :32:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Water loss through transpiration'              &
                           ,'[    kg/m2/s]','(nzg,ipatch)'        )
      end if
      if (associated(csite%mmean_sensible_gg     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%mmean_sensible_gg                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'MMEAN_SENSIBLE_GG_PA       :32:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Soil heat flux'                                &
                           ,'[       W/m2]','(nzg,ipatch)'        )
      end if
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_sitetype_p32
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the patch-level variables           !
   ! (sitetype) that have three dimensions (nzg,ndcycle,npatches).                         !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_sitetype_m32(csite,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(sitetype)        , target        :: csite
      integer               , intent(in)    :: init
      integer               , intent(in)    :: igr
      integer               , intent(in)    :: var_len
      integer               , intent(in)    :: max_ptrs
      integer               , intent(in)    :: var_len_global
      integer               , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                               :: npts
      character(len=str_len)                :: eorq_keys
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !      Decide whether to write the mean diel to the history file.                    !
      !------------------------------------------------------------------------------------!
      select case (iadd_patch_means)
      case (0)
         return
      case (1)
         if (history_eorq) then
            eorq_keys = 'hist:dcyc'
         else
            eorq_keys = 'dcyc'
         end if
      end select
      !------------------------------------------------------------------------------------!







      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 3-D vectors with dimensions npatches, ndcycle,    !
      ! and nzg.  Notice that they all use the same npts.  Here you should only add vari-  !
      ! ables of type -32.                                                                 !
      !------------------------------------------------------------------------------------!
      npts = csite%npatches * ndcycle * nzg
      if (associated(csite%qmean_soil_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_soil_energy                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SOIL_ENERGY_PA      :-32:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Soil internal energy'                             &
                           ,'[       J/m3]','(nzg,ndcycle,ipatch)')
      end if
      if (associated(csite%qmean_soil_mstpot     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_soil_mstpot                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SOIL_MSTPOT_PA      :-32:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Soil matric potential'                            &
                           ,'[          m]','(nzg,ndcycle,ipatch)')
      end if
      if (associated(csite%qmean_soil_water      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_soil_water                                    &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SOIL_WATER_PA       :-32:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Soil water content'                               &
                           ,'[      m3/m3]','(nzg,ndcycle,ipatch)')
      end if
      if (associated(csite%qmean_soil_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_soil_temp                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SOIL_TEMP_PA        :-32:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Soil temperature'                                 &
                           ,'[          K]','(nzg,ndcycle,ipatch)')
      end if
      if (associated(csite%qmean_soil_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_soil_fliq                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SOIL_FLIQ_PA        :-32:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Soil liquid fraction'                             &
                           ,'[         --]','(nzg,ndcycle,ipatch)')
      end if
      if (associated(csite%qmean_smoist_gg       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_smoist_gg                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SMOIST_GG_PA        :-32:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Soil water flux'                                  &
                           ,'[    kg/m2/s]','(nzg,ndcycle,ipatch)')
      end if
      if (associated(csite%qmean_transloss       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_transloss                                     &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_TRANSLOSS_PA        :-32:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Water loss through transpiration'                 &
                           ,'[    kg/m2/s]','(nzg,ndcycle,ipatch)')
      end if
      if (associated(csite%qmean_sensible_gg     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,csite%qmean_sensible_gg                                   &
                           ,nvar,igr,init,csite%paglob_id,var_len,var_len_global,max_ptrs  &
                           ,'QMEAN_SENSIBLE_GG_PA      :-32:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Soil heat flux'                                   &
                           ,'[       W/m2]','(nzg,ndcycle,ipatch)')
      end if
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_sitetype_m32
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the patch-level variables           !
   ! (sitetype) that have two dimensions (nzs,npatches).                                   !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_sitetype_p33(csite,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(sitetype), target        :: csite
      integer       , intent(in)    :: init
      integer       , intent(in)    :: igr
      integer       , intent(in)    :: var_len
      integer       , intent(in)    :: max_ptrs
      integer       , intent(in)    :: var_len_global
      integer       , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                       :: npts
      !------------------------------------------------------------------------------------!






      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 2-D vectors with dimensions npatches and nzs.     !
      !  Notice that they all use the same npts.  Here you should only add variables of    !
      ! type 33.                                                                           !
      !------------------------------------------------------------------------------------!
      npts = csite%npatches * nzs


      if (associated(csite%sfcwater_mass)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%sfcwater_mass,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'SFCWATER_MASS :33:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%sfcwater_energy)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%sfcwater_energy,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'SFCWATER_ENERGY :33:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%sfcwater_depth)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%sfcwater_depth,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'SFCWATER_DEPTH :33:hist:opti') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','m') 
      end if

      if (associated(csite%rshort_s)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%rshort_s,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'RSHORT_S :33:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%rshort_s_beam)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%rshort_s_beam,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'RSHORT_S_BEAM :33:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%rshort_s_diffuse)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%rshort_s_diffuse,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'RSHORT_S_DIFFUSE :33:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%par_s)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%par_s,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'PAR_S :33:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%par_s_beam)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%par_s_beam,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'PAR_S_BEAM :33:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%par_s_diffuse)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%par_s_diffuse,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'PAR_S_DIFFUSE :33:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%sfcwater_tempk)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%sfcwater_tempk,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'SFCWATER_TEMPK :33:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%sfcwater_fracliq)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%sfcwater_fracliq,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'SFCWATER_FRACLIQ :33:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      return
   end subroutine filltab_sitetype_p33
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the patch-level variables           !
   ! (sitetype) that have two dimensions (n_pft,npatches).                                 !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_sitetype_p34(csite,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(sitetype), target        :: csite
      integer       , intent(in)    :: init
      integer       , intent(in)    :: igr
      integer       , intent(in)    :: var_len
      integer       , intent(in)    :: max_ptrs
      integer       , intent(in)    :: var_len_global
      integer       , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                       :: npts
      !------------------------------------------------------------------------------------!






      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 2-D vectors with dimensions npatches and n_pft.   !
      !  Notice that they all use the same npts.  Here you should only add variables of    !
      ! type 34.                                                                           !
      !------------------------------------------------------------------------------------!
      npts = csite%npatches * n_pft

      if (associated(csite%A_o_max)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%A_o_max,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'A_O_MAX :34:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%A_c_max)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%A_c_max,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'A_C_MAX :34:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(csite%repro)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%repro,nvar,igr,init,csite%paglob_id, &
           var_len,var_len_global,max_ptrs,'REPRO_PA :34:hist:mont:dcyc') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_sitetype_p34
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the patch-level variables           !
   ! (sitetype) that have three dimensions (n_pft,ff_nhgt,npatches).                                 !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_sitetype_p346(csite,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(sitetype), target        :: csite
      integer       , intent(in)    :: init
      integer       , intent(in)    :: igr
      integer       , intent(in)    :: var_len
      integer       , intent(in)    :: max_ptrs
      integer       , intent(in)    :: var_len_global
      integer       , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                       :: npts
      !------------------------------------------------------------------------------------!






      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 3-D vectors with dimensions npatches and n_dbh.   !
      !  Notice that they all use the same npts.  Here you should only add variables of    !
      ! type 346.                                                                          !
      !------------------------------------------------------------------------------------!
      npts = csite%npatches * n_pft * n_dbh

      if (associated(csite%cumlai_profile)) then
         nvar=nvar+1
           call vtable_edio_r(npts,csite%cumlai_profile,nvar,igr,init,csite%paglob_id,     &
           var_len,var_len_global,max_ptrs,'CUMLAI_PROFILE :346:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if
      !------------------------------------------------------------------------------------!

      return
      
   end subroutine filltab_sitetype_p346
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This sub-routine fills in the variable table for cohort-level variables.          !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_patchtype(igr,ipy,isi,ipa,init)

      use ed_var_tables, only : vtable_edio_r & ! sub-routine
                              , vtable_edio_i & ! sub-routine
                              , metadata_edio ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      integer          , intent(in) :: init
      integer          , intent(in) :: igr
      integer          , intent(in) :: ipy
      integer          , intent(in) :: isi
      integer          , intent(in) :: ipa
      !----- Local variables. -------------------------------------------------------------!
      type(polygontype), pointer    :: cpoly
      type(sitetype)   , pointer    :: csite
      type(patchtype)  , pointer    :: cpatch
      integer                       :: ip
      integer                       :: np
      integer                       :: is
      integer                       :: ns
      integer                       :: iy
      integer                       :: ny
      integer                       :: var_len
      integer                       :: max_ptrs
      integer                       :: var_len_global
      integer                       :: nvar
      integer                       :: npts
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !      The condition exists where some patches may have no cohorts, particularly     !
      ! over deserts.  Therefore it is possible that the first patch in the site has no    !
      ! cohorts, but the others might have some cohorts.  Therefore, we want to make sure  !
      ! we create mapping tables if "any" of the patches have cohorts, so we will provide  !
      ! for this conditions when we make the mapping tables.  The condition has to arise   !
      ! where there are no cohorts at all in any of the patches in any of the polygons on  !
      ! the current machine to not initialize these vectors.                               !
      !------------------------------------------------------------------------------------!
      if (init==0) then
         ny = edgrid_g(igr)%npolygons

         polyloop: do iy=1,ny
            cpoly => edgrid_g(igr)%polygon(iy)
            ns = cpoly%nsites
            siteloop: do is=1,ns
               csite => cpoly%site(is)
               np = csite%npatches
               patchloop: do ip=1,np
                  cpatch => edgrid_g(igr)%polygon(iy)%site(is)%patch(ip)
                  if (cpatch%ncohorts > 0) exit polyloop
               end do patchloop
            end do siteloop
         end do polyloop

      else
         cpatch => edgrid_g(igr)%polygon(ipy)%site(isi)%patch(ipa)
      end if
      !------------------------------------------------------------------------------------!


      !----- Nothing else to do here in case this is an empty patch. ----------------------!
      if (cpatch%ncohorts == 0) return
      !------------------------------------------------------------------------------------!


      !----- Define some variables to be used by all variables. ---------------------------!
      var_len        = cpatch%ncohorts
      var_len_global = edgrid_g(igr)%ncohorts_global
      max_ptrs       = edgrid_g(igr)%npatches_global
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      !     Initialise the variable number, which should be unique for each variable.      !
      ! nioglobal has the current position after all the parent structure variables have   !
      ! been positioned in the variable table.                                             !
      !------------------------------------------------------------------------------------!
      nvar=nioglobal+niogrid+niopoly+niosite
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !      We split the routine into smaller ones to avoid excessive number of calls to  !
      ! sub-routines from the same routine (this allows more optimisation for ifort, and   !
      ! reduces chances of segmentation violation during the compilation).                 !
      !------------------------------------------------------------------------------------!
      call filltab_patchtype_p40     (cpatch,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_patchtype_p41inst (cpatch,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_patchtype_p41fmean(cpatch,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_patchtype_p41dmean(cpatch,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_patchtype_p41mmean(cpatch,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_patchtype_m41     (cpatch,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_patchtype_p48     (cpatch,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_patchtype_p49     (cpatch,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_patchtype_p411    (cpatch,igr,init,var_len,var_len_global,max_ptrs,nvar)
      call filltab_patchtype_m411    (cpatch,igr,init,var_len,var_len_global,max_ptrs,nvar)
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_patchtype
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the cohort-level variables          !
   ! (patchtype) that have one dimension and are integer (type 40).                        !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_patchtype_p40(cpatch,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_i  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(patchtype), target        :: cpatch
      integer        , intent(in)    :: init
      integer        , intent(in)    :: igr
      integer        , intent(in)    :: var_len
      integer        , intent(in)    :: max_ptrs
      integer        , intent(in)    :: var_len_global
      integer        , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                        :: npts
      !------------------------------------------------------------------------------------!








      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 1-D vectors, with dimension ncohorts.  Notice     !
      ! that they all use the same npts.  Here you should only add variables of type 40    !
      ! (integers and scalars).                                                            !
      !------------------------------------------------------------------------------------!
      npts = cpatch%ncohorts



      if (associated(cpatch%pft)) then
         nvar=nvar+1
           call vtable_edio_i(npts,cpatch%pft,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'PFT :40:hist:anal:dail:mont:dcyc:year')
         call metadata_edio(nvar,igr,'Plant Functional Type','[-]','NA') 
      end if


      if (associated(cpatch%phenology_status)) then
         nvar=nvar+1
           call vtable_edio_i(npts,cpatch%phenology_status,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'PHENOLOGY_STATUS :40:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%recruit_dbh)) then
         nvar=nvar+1
           call vtable_edio_i(npts,cpatch%recruit_dbh,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'RECRUIT_DBH :40:hist:dail:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%census_status)) then
         nvar=nvar+1
           call vtable_edio_i(npts,cpatch%census_status,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'CENSUS_STATUS :40:hist:dail:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%krdepth)) then
         nvar=nvar+1
           call vtable_edio_i(npts,cpatch%krdepth,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'KRDEPTH :40:hist:dail:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%first_census)) then
         nvar=nvar+1
           call vtable_edio_i(npts,cpatch%first_census,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'FIRST_CENSUS :40:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%new_recruit_flag)) then
         nvar=nvar+1
           call vtable_edio_i(npts,cpatch%new_recruit_flag,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'NEW_RECRUIT_FLAG :40:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_patchtype_p40
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the cohort-level variables          !
   ! (patchtype) that have one dimension and are real (type 41) and not fmean, dmean,      !
   ! mmean, mmsqu, qmean, or qmsqu.                                                        !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_patchtype_p41inst (cpatch,igr,init,var_len,var_len_global,max_ptrs   &
                                        ,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(patchtype), target        :: cpatch
      integer        , intent(in)    :: init
      integer        , intent(in)    :: igr
      integer        , intent(in)    :: var_len
      integer        , intent(in)    :: max_ptrs
      integer        , intent(in)    :: var_len_global
      integer        , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                        :: npts
      !------------------------------------------------------------------------------------!








      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 1-D vectors, with dimension ncohorts.  Notice     !
      ! that they all use the same npts.  Here you should only add variables of type 41    !
      ! (real, scalar, and not one of the fmean, dmean, mmean, mmsqu variables).           !
      !------------------------------------------------------------------------------------!
      npts = cpatch%ncohorts


      if (associated(cpatch%nplant)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%nplant,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'NPLANT :41:hist:anal:dail:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'Plant density','[plant/m2]','NA') 
      end if

      if (associated(cpatch%hite)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%hite,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'HITE :41:hist:anal:dail:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%agb)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%agb,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'AGB_CO :41:hist:dail:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'Above-ground biomass','[kgC/plant]','icohort') 
      end if

      if (associated(cpatch%basarea)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%basarea,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'BA_CO :41:hist:anal:dail:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'Basal-area','[cm2]','icohort') 
      end if

      if (associated(cpatch%dagb_dt)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%dagb_dt,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'DAGB_DT :41:hist:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'Above-ground biomass growth','[kgC/plant/yr]','icohort') 
      end if

      if (associated(cpatch%dlnagb_dt)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%dlnagb_dt,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'DLNAGB_DT :41:hist:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'Relative AGB growth','[1/yr]','icohort') 
      end if

      if (associated(cpatch%dba_dt)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%dba_dt,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'DBA_DT :41:hist:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'Basal-area growth','[cm2/plant/yr]','icohort') 
      end if

      if (associated(cpatch%dlnba_dt)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%dlnba_dt,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'DLNBA_DT :41:hist:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'Relative Basal-area growth','[1/yr]','icohort') 
      end if

      if (associated(cpatch%ddbh_dt)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%ddbh_dt,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'DDBH_DT :41:hist:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'DBH growth','[cm/plant/yr]','icohort') 
      end if

      if (associated(cpatch%dlndbh_dt)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%dlndbh_dt,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'DLNDBH_DT :41:hist:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'Relative DBH growth','[1/yr]','icohort') 
      end if

      if (associated(cpatch%dbh)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%dbh,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'DBH :41:hist:anal:year:dail:mont:dcyc') 
         call metadata_edio(nvar,igr,'Diameter at breast height','[cm]','icohort') 
      end if

      if (associated(cpatch%bdead)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%bdead,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'BDEAD :41:hist:anal:mont:dail:year:dcyc') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%bleaf)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%bleaf,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'BLEAF :41:hist:anal:year:dail:mont:dcyc') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%balive)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%balive,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'BALIVE :41:hist:anal:year:dail:mont:dcyc') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%broot)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%broot,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'BROOT :41:hist:anal:year:dail:mont:dcyc') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%bsapwooda)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%bsapwooda,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs                                          &
           ,'BSAPWOODA :41:hist:anal:year:dail:anal:mont:dcyc') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%bsapwoodb)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%bsapwoodb,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs                                          &
           ,'BSAPWOODB :41:hist:anal:year:dail:anal:mont:dcyc') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%lai)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%lai,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LAI_CO :41:hist:anal:dail:mont:year:dcyc') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%wai)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%wai,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'WAI_CO :41:hist:anal:dail:mont:year:dcyc') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%crown_area)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%crown_area,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs                                           &
           ,'CROWN_AREA_CO :41:hist:anal:dail:mont:year:dcyc') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%bstorage)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%bstorage,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'BSTORAGE :41:hist:anal:year:dail:mont:dcyc') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%cbr_bar)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%cbr_bar,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'CBR_BAR :41:hist:mont:year:dcyc') 
         call metadata_edio(nvar,igr,'Relative carbon balance','[NA]','NA') 
      end if

      if (associated(cpatch%leaf_energy)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%leaf_energy,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LEAF_ENERGY :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%leaf_hcap)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%leaf_hcap,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LEAF_HCAP :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%leaf_temp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%leaf_temp,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LEAF_TEMP :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%leaf_vpdef)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%leaf_vpdef,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LEAF_VPDEF :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%leaf_temp_pv)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpatch%leaf_temp_pv,nvar,igr,init,cpatch%coglob_id, &
              var_len,var_len_global,max_ptrs,'LEAF_TEMP_PV :41:hist') 
         call metadata_edio(nvar,igr,'Leaf Temperature at previous step','[]','NA') 
      end if


      if (associated(cpatch%leaf_fliq)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%leaf_fliq,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LEAF_FLIQ :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%leaf_water)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%leaf_water,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LEAF_WATER :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%wood_energy)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%wood_energy,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'WOOD_ENERGY :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%wood_hcap)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%wood_hcap,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'WOOD_HCAP :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%wood_temp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%wood_temp,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'WOOD_TEMP :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if
      
     if (associated(cpatch%wood_temp_pv)) then
        nvar=nvar+1
         call vtable_edio_r(npts,cpatch%wood_temp_pv,nvar,igr,init,cpatch%coglob_id, &
                var_len,var_len_global,max_ptrs,'WOOD_TEMP_PV :41:hist') 
         call metadata_edio(nvar,igr,'Wood temperature at previous step','[NA]','NA') 
      end if


      if (associated(cpatch%wood_fliq)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%wood_fliq,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'WOOD_FLIQ :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%wood_water)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%wood_water,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'WOOD_WATER :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%veg_wind)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%veg_wind,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'VEG_WIND :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%lsfc_shv_closed)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%lsfc_shv_closed,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LSFC_SHV_CLOSED :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%lsfc_shv_open)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%lsfc_shv_open,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LSFC_SHV_OPEN :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%lsfc_co2_closed)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%lsfc_co2_closed,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LSFC_CO2_CLOSED :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%lsfc_co2_open)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%lsfc_co2_open,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LSFC_CO2_OPEN :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%lint_shv)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%lint_shv,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LINT_SHV :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%lint_co2_closed)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%lint_co2_closed,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LINT_CO2_CLOSED :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%lint_co2_open)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%lint_co2_open,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LINT_CO2_OPEN :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%today_leaf_resp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%today_leaf_resp,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'TODAY_LEAF_RESP :41:hist') 
         call metadata_edio(nvar,igr,'NOT A DIAGNOSTIC VARIABLE','[NA]','icohort') 
      end if

      if (associated(cpatch%today_root_resp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%today_root_resp,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'TODAY_ROOT_RESP :41:hist') 
         call metadata_edio(nvar,igr,'NOT A DIAGNOSTIC VARIABLE','[NA]','icohort') 
      end if

      if (associated(cpatch%today_gpp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%today_gpp,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'TODAY_GPP :41:hist') 
         call metadata_edio(nvar,igr,'NOT A DIAGNOSTIC VARIABLE','[NA]','icohort') 
      end if
      
      if (associated(cpatch%today_nppleaf)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%today_nppleaf,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'TODAY_NPPLEAF :41:hist') 
         call metadata_edio(nvar,igr,'NOT A DIAGNOSTIC VARIABLE','[NA]','icohort') 
      end if
      
      if (associated(cpatch%today_nppfroot)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%today_nppfroot,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'TODAY_NPPFROOT :41:hist') 
         call metadata_edio(nvar,igr,'NOT A DIAGNOSTIC VARIABLE','[NA]','icohort') 
      end if
      
      if (associated(cpatch%today_nppsapwood)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%today_nppsapwood,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'TODAY_NPPSAPWOOD :41:hist') 
         call metadata_edio(nvar,igr,'NOT A DIAGNOSTIC VARIABLE','[NA]','icohort') 
      end if
      
      if (associated(cpatch%today_nppcroot)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%today_nppcroot,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'TODAY_NPPCROOT :41:hist') 
         call metadata_edio(nvar,igr,'NOT A DIAGNOSTIC VARIABLE','[NA]','icohort') 
      end if
      
      if (associated(cpatch%today_nppseeds)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%today_nppseeds,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'TODAY_NPPSEEDS :41:hist') 
         call metadata_edio(nvar,igr,'NOT A DIAGNOSTIC VARIABLE','[NA]','icohort') 
      end if
      
      if (associated(cpatch%today_nppwood)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%today_nppwood,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'TODAY_NPPWOOD :41:hist') 
         call metadata_edio(nvar,igr,'NOT A DIAGNOSTIC VARIABLE','[NA]','icohort') 
      end if
      
      if (associated(cpatch%today_nppdaily)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%today_nppdaily,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'TODAY_NPPDAILY :41:hist') 
         call metadata_edio(nvar,igr,'NOT A DIAGNOSTIC VARIABLE','[NA]','icohort') 
      end if
      
      if (associated(cpatch%today_gpp_pot)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%today_gpp_pot,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'TODAY_GPP_POT :41:hist') 
         call metadata_edio(nvar,igr,'NOT A DIAGNOSTIC VARIABLE','[NA]','icohort') 
      end if

      if (associated(cpatch%today_gpp_lightmax)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%today_gpp_lightmax,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'TODAY_GPP_LIGHTMAX :41:hist') 
         call metadata_edio(nvar,igr,'NOT A DIAGNOSTIC VARIABLE','[NA]','icohort') 
      end if

      if (associated(cpatch%today_gpp_moistmax)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%today_gpp_moistmax,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'TODAY_GPP_MOISTMAX :41:hist') 
         call metadata_edio(nvar,igr,'NOT A DIAGNOSTIC VARIABLE','[NA]','icohort') 
      end if

      if (associated(cpatch%today_gpp_mlmax)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%today_gpp_mlmax,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'TODAY_GPP_MLMAX :41:hist') 
         call metadata_edio(nvar,igr,'NOT A DIAGNOSTIC VARIABLE','[NA]','icohort') 
      end if

      if (associated(cpatch%leaf_growth_resp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%leaf_growth_resp,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LEAF_GROWTH_RESP :41:hist:anal') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%root_growth_resp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%root_growth_resp,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'ROOT_GROWTH_RESP :41:hist:anal') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%sapa_growth_resp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%sapa_growth_resp,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'SAPA_GROWTH_RESP :41:hist:anal') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%sapb_growth_resp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%sapb_growth_resp,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'SAPB_GROWTH_RESP :41:hist:anal') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%leaf_storage_resp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%leaf_storage_resp,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LEAF_STORAGE_RESP :41:hist:anal') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%root_storage_resp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%root_storage_resp,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'ROOT_STORAGE_RESP :41:hist:anal') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%sapa_storage_resp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%sapa_storage_resp,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'SAPA_STORAGE_RESP :41:hist:anal') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%sapb_storage_resp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%sapb_storage_resp,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'SAPB_STORAGE_RESP :41:hist:anal') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%monthly_dndt)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%monthly_dndt,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'MONTHLY_DNDT :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%monthly_dlnndt)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%monthly_dlnndt,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'MONTHLY_DLNNDT :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%Psi_open)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%Psi_open,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'PSI_OPEN :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%light_level)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%light_level,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LIGHT_LEVEL :41:hist') 
         call metadata_edio(nvar,igr,'Relative light level','[NA]','icohort') 
      end if

      if (associated(cpatch%light_level_beam)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%light_level_beam,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LIGHT_LEVEL_BEAM :41:hist') 
         call metadata_edio(nvar,igr,'Relative light level, beam fraction','[NA]','icohort') 
      end if

      if (associated(cpatch%light_level_diff)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%light_level_diff,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LIGHT_LEVEL_DIFF :41:hist') 
         call metadata_edio(nvar,igr,'Relative light level, diffuse fraction','[NA]','icohort') 
      end if

      if (associated(cpatch%par_level_beam)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%par_level_beam,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'PAR_LEVEL_BEAM :41:hist') 
         call metadata_edio(nvar,igr,'PAR flux density (cohort centered), beam','[W/m2]','icohort') 
      end if
      
      if (associated(cpatch%par_level_diffu)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpatch%par_level_diffu,nvar,igr,init,cpatch%coglob_id, &
               var_len,var_len_global,max_ptrs,'PAR_LEVEL_DIFFU :41:hist') 
         call metadata_edio(nvar,igr,'PAR flux density (cohort centered), upward diffuse','[W/m2]','icohort') 
      end if
      
      if (associated(cpatch%par_level_diffd)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpatch%par_level_diffd,nvar,igr,init,cpatch%coglob_id, &
               var_len,var_len_global,max_ptrs,'PAR_LEVEL_DIFFD :41:hist') 
         call metadata_edio(nvar,igr,'PAR flux density (cohort centered), downward diffuse','[W/m2]','icohort') 
      end if
      
      if (associated(cpatch%par_l)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%par_l,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'PAR_L :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%par_l_beam)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%par_l_beam,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'PAR_L_BEAM :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%par_l_diffuse)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%par_l_diffuse,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'PAR_L_DIFFUSE :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%rshort_l)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%rshort_l,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'RSHORT_L :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%rshort_l_beam)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%rshort_l_beam,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'RSHORT_L_BEAM :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%rshort_l_diffuse)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%rshort_l_diffuse,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'RSHORT_L_DIFFUSE :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%rlong_l)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%rlong_l,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'RLONG_L :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%rshort_w)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%rshort_w,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'RSHORT_W :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%rshort_w_beam)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%rshort_w_beam,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'RSHORT_W_BEAM :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%rshort_w_diffuse)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%rshort_w_diffuse,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'RSHORT_W_DIFFUSE :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%rlong_w)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%rlong_w,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'RLONG_W :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%leaf_gbh)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%leaf_gbh,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LEAF_GBH :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%leaf_gbw)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%leaf_gbw,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LEAF_GBW :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%wood_gbh)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%wood_gbh,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'WOOD_GBH :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%wood_gbw)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%wood_gbw,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'WOOD_GBW :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if
      
      if (associated(cpatch%llspan)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%llspan,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LLSPAN :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if
      
      if (associated(cpatch%A_open)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%A_open,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'A_OPEN :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%A_closed)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%A_closed,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'A_CLOSED :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%A_light)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%A_light,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'A_LIGHT :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%A_rubp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%A_rubp,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'A_RUBP :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%A_co2)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%A_co2,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'A_CO2 :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%Psi_closed)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%Psi_closed,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'PSI_CLOSED :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%gsw_open)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%gsw_open,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'GSW_OPEN :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%gsw_closed)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%gsw_closed,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'GSW_CLOSED :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%leaf_gsw)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%leaf_gsw,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LEAF_GSW :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%fsw)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%fsw,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'FSW :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%fsn)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%fsn,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'FSN :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%fs_open)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%fs_open,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'FS_OPEN :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%water_supply)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%water_supply,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'WATER_SUPPLY :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%leaf_maintenance)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%leaf_maintenance,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LEAF_MAINTENANCE :41:hist:dail') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%root_maintenance)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%root_maintenance,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'ROOT_MAINTENANCE :41:hist:dail') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%leaf_drop)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%leaf_drop,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LEAF_DROP :41:hist:dail') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%bseeds)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%bseeds,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'BSEEDS_CO :41:hist:dail:mont:dcyc') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%leaf_respiration)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%leaf_respiration,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'LEAF_RESPIRATION :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%root_respiration)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%root_respiration,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'ROOT_RESPIRATION :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%gpp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%gpp,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'GPP :41:hist') 
         call metadata_edio(nvar,igr,'Gross Primary Production','[umol/m2/s]','icohort') 
      end if

      if (associated(cpatch%paw_avg)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%paw_avg,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'PAW_AVG :41:hist:dail:mont:dcyc') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%elongf)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%elongf,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'ELONGF :41:hist:dail:mont:dcyc') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%turnover_amp)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%turnover_amp,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'TURNOVER_AMP :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%vm_bar)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%vm_bar,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'VM_BAR :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if

      if (associated(cpatch%sla)) then
         nvar=nvar+1
           call vtable_edio_r(npts,cpatch%sla,nvar,igr,init,cpatch%coglob_id, &
           var_len,var_len_global,max_ptrs,'SLA :41:hist') 
         call metadata_edio(nvar,igr,'No metadata available','[NA]','NA') 
      end if
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_patchtype_p41inst
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the cohort-level variables          !
   ! (patchtype) that have one dimension and are real (type 41) and fmean.                 !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_patchtype_p41fmean(cpatch,igr,init,var_len,var_len_global,max_ptrs   &
                                        ,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(patchtype)       , target        :: cpatch
      integer               , intent(in)    :: init
      integer               , intent(in)    :: igr
      integer               , intent(in)    :: var_len
      integer               , intent(in)    :: max_ptrs
      integer               , intent(in)    :: var_len_global
      integer               , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                               :: npts
      character(len=str_len)                :: fast_keys
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !      Decide whether to write the sub-daily means to the history file.              !
      !------------------------------------------------------------------------------------!
      select case (iadd_cohort_means)
      case (0)
         return
      case (1)
         if (history_fast) then
            fast_keys = 'hist:anal'
         else
            fast_keys = 'anal'
         end if
      end select
      !------------------------------------------------------------------------------------!




      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 1-D vectors, with dimension ncohorts.  Notice     !
      ! that they all use the same npts.  Here you should only add variables of type 41    !
      ! (real, scalar), and a "fmean" variable.                                            !
      !------------------------------------------------------------------------------------!
      npts = cpatch%ncohorts

      if (associated(cpatch%fmean_gpp             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_gpp                                          &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_GPP_CO               :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Gross primary productivity'                  &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_npp             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_npp                                          &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_NPP_CO               :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Net primary productivity'                    &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_leaf_resp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_leaf_resp                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_LEAF_RESP_CO         :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf respiration'                            &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_root_resp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_root_resp                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_ROOT_RESP_CO         :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Root respiration'                            &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_leaf_growth_resp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_leaf_growth_resp                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_LEAF_GROWTH_RESP_CO  :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf growth respiration'                     &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_root_growth_resp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_root_growth_resp                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_ROOT_GROWTH_RESP_CO  :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Root growth respiration'                     &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_sapa_growth_resp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_sapa_growth_resp                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_SAPA_GROWTH_RESP_CO  :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Aboveground sapwood growth respiration'      &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_sapb_growth_resp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_sapb_growth_resp                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_SAPB_GROWTH_RESP_CO  :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Belowground sapwood growth respiration'      &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_leaf_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_leaf_storage_resp                            &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_LEAF_STORAGE_RESP_CO      :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf Storage respiration'                    &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_root_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_root_storage_resp                            &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_ROOT_STORAGE_RESP_CO      :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Root Storage respiration'                    &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_sapa_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_sapa_storage_resp                            &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_SAPA_STORAGE_RESP_CO      :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Aboveground Sapwood Storage respiration'     &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_sapb_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_sapb_storage_resp                            &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_SAPB_STORAGE_RESP_CO      :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Belowground Sapwood Storage respiration'     &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_plresp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_plresp                                       &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_PLRESP_CO            :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Plant respiration'                           &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_leaf_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_leaf_energy                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_LEAF_ENERGY_CO       :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf internal energy'                        &
                           ,'[       J/m2]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_leaf_water      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_leaf_water                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_LEAF_WATER_CO        :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf surface water'                          &
                           ,'[      kg/m2]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_leaf_hcap       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_leaf_hcap                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_LEAF_HCAP_CO         :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf heat capacity'                          &
                           ,'[     J/m2/K]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_leaf_vpdef      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_leaf_vpdef                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_LEAF_VPDEF_CO        :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf vapour pressure deficit'                &
                           ,'[         Pa]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_leaf_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_leaf_temp                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_LEAF_TEMP_CO         :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf temperature'                            &
                           ,'[          K]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_leaf_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_leaf_fliq                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_LEAF_FLIQ_CO         :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Liquid fraction'                             &
                           ,'[         --]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_leaf_gsw        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_leaf_gsw                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_LEAF_GSW_CO          :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Stomatal conductance'                        &
                           ,'[kg/m2leaf/s]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_leaf_gbw        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_leaf_gbw                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_LEAF_GBW_CO          :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf boundary layer conductance'             &
                           ,'[kg/m2leaf/s]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_wood_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_wood_energy                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_WOOD_ENERGY_CO       :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Wood internal energy'                        &
                           ,'[       J/m2]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_wood_water      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_wood_water                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_WOOD_WATER_CO        :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Wood surface water'                          &
                           ,'[      kg/m2]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_wood_hcap       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_wood_hcap                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_WOOD_HCAP_CO         :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Wood heat capacity'                          &
                           ,'[     J/m2/K]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_wood_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_wood_temp                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_WOOD_TEMP_CO         :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Wood temperature'                            &
                           ,'[          K]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_wood_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_wood_fliq                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_WOOD_FLIQ_CO         :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Liquid fraction'                             &
                           ,'[         --]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_wood_gbw        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_wood_gbw                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_WOOD_GBW_CO          :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Wood boundary layer conductance'             &
                           ,'[kg/m2wood/s]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_fs_open         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_fs_open                                      &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_FS_OPEN_CO           :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Net stress factor'                           &
                           ,'[         --]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_fsw             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_fsw                                          &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_FSW_CO               :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Moisture stress'                             &
                           ,'[         --]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_fsn             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_fsn                                          &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_FSN_CO               :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Nitrogen stress'                             &
                           ,'[         --]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_a_open         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_a_open                                       &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_A_OPEN_CO             :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Assimilation rate (no soil moist. stress)'   &
                           ,'[ umol/m2l/s]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_a_closed       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_a_closed                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_A_CLOSED_CO           :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Minimum assimilation rate'                   &
                           ,'[ umol/m2l/s]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_a_net           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_a_net                                        &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_A_NET_CO               :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Actual assimilation rate'                    &
                           ,'[ umol/m2l/s]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_a_light         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_a_light                                      &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_A_LIGHT_CO             :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Light-limited assimilation rate'             &
                           ,'[ umol/m2l/s]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_a_rubp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_a_rubp                                       &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_A_RUBP_CO              :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - RuBP-limited assimilation rate'              &
                           ,'[ umol/m2l/s]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_a_co2           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_a_co2                                        &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_A_CO2_CO              :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - CO2-limited assimilation rate'               &
                           ,'[ umol/m2l/s]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_psi_open        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_psi_open                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_PSI_OPEN_CO          :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Transpiration with no stress'                &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_psi_closed      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_psi_closed                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_PSI_CLOSED_CO        :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Transpiration at maximum stress'             &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_water_supply    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_water_supply                                 &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_WATER_SUPPLY_CO      :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Water supply'                                &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_light_level     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_light_level                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_LIGHT_LEVEL_CO       :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Light level'                                 &
                           ,'[         --]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_light_level_beam)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_light_level_beam                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_LIGHT_LEVEL_BEAM_CO  :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Light level (direct radiation)'              &
                           ,'[         --]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_light_level_diff)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_light_level_diff                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_LIGHT_LEVEL_DIFF_CO  :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Light level (diffuse radiation)'             &
                           ,'[         --]','(icohort)'            )
      end if

      if (associated(cpatch%fmean_par_level_beam)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpatch%fmean_par_level_beam,nvar,igr,init,cpatch%coglob_id, &
               var_len,var_len_global,max_ptrs,'FMEAN_PAR_LEVEL_BEAM :41:'//trim(fast_keys)) 
         call metadata_edio(nvar,igr, &
               'Sub-daily mean PAR flux density (cohort centered), beam','[W/m2]','icohort') 
      end if

      if (associated(cpatch%fmean_par_level_diffu)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpatch%fmean_par_level_diffu,nvar,igr,init,cpatch%coglob_id, &
               var_len,var_len_global,max_ptrs,'FMEAN_PAR_LEVEL_DIFFU :41:'//trim(fast_keys)) 
         call metadata_edio(nvar,igr, &
               'Sub-daily mean PAR flux density (cohort centered), diff up','[W/m2]','icohort') 
      end if
      
      if (associated(cpatch%fmean_par_level_diffd)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpatch%fmean_par_level_diffd,nvar,igr,init,cpatch%coglob_id, &
               var_len,var_len_global,max_ptrs,'FMEAN_PAR_LEVEL_DIFFD :41:'//trim(fast_keys)) 
         call metadata_edio(nvar,igr, &
               'Sub-daily mean PAR flux density (cohort centered), diff down','[W/m2]','icohort') 
      end if


      if (associated(cpatch%fmean_par_l           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_par_l                                        &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_PAR_L_CO             :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - PAR absorbed by leaves'                      &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_par_l_beam      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_par_l_beam                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_PAR_L_BEAM_CO        :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Direct PAR absorbed by leaves'               &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_par_l_diff      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_par_l_diff                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_PAR_L_DIFF_CO        :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Diffuse PAR absorbed by leaves'              &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_rshort_l        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_rshort_l                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_RSHORT_L_CO          :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Shortwave radiation absorbed by leaves'      &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_rlong_l         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_rlong_l                                      &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_RLONG_L_CO           :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Longwave radiation absorbed by leaves'       &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_sensible_lc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_sensible_lc                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_SENSIBLE_LC_CO       :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Sensible heat'                               &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_vapor_lc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_vapor_lc                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_VAPOR_LC_CO          :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf evaporation'                            &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_transp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_transp                                       &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_TRANSP_CO            :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf transpiration'                          &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_intercepted_al  )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_intercepted_al                               &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_INTERCEPTED_AL_CO    :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf interception'                           &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_wshed_lg        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_wshed_lg                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_WSHED_LG_CO          :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Leaf shedding'                               &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_rshort_w        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_rshort_w                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_RSHORT_W_CO          :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Shortwave radiation absorbed by wood'        &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_rlong_w         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_rlong_w                                      &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_RLONG_W_CO           :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Longwave radiation absorbed by wood'         &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_sensible_wc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_sensible_wc                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_SENSIBLE_WC_CO       :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Sensible heat'                               &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_vapor_wc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_vapor_wc                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_VAPOR_WC_CO          :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Wood evaporation'                            &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_intercepted_aw  )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_intercepted_aw                               &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_INTERCEPTED_AW_CO    :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Wood interception'                           &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%fmean_wshed_wg        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%fmean_wshed_wg                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_WSHED_WG_CO          :41:'//trim(fast_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Wood shedding'                               &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_patchtype_p41fmean
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the cohort-level variables          !
   ! (patchtype) that have one dimension and are real (type 41) and dmean.                 !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_patchtype_p41dmean(cpatch,igr,init,var_len,var_len_global,max_ptrs   &
                                        ,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(patchtype)       , target        :: cpatch
      integer               , intent(in)    :: init
      integer               , intent(in)    :: igr
      integer               , intent(in)    :: var_len
      integer               , intent(in)    :: max_ptrs
      integer               , intent(in)    :: var_len_global
      integer               , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                               :: npts
      character(len=str_len)                :: dail_keys
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !      Decide whether to write the daily means to the history file.                  !
      !------------------------------------------------------------------------------------!
      select case (iadd_cohort_means)
      case (0)
         return
      case (1)
         if (history_dail) then
            dail_keys = 'hist:dail'
         else
            dail_keys = 'dail'
         end if
      end select
      !------------------------------------------------------------------------------------!








      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 1-D vectors, with dimension ncohorts.  Notice     !
      ! that they all use the same npts.  Here you should only add variables of type 41    !
      ! (real, scalar), and a "dmean" variable.                                            !
      !------------------------------------------------------------------------------------!
      npts = cpatch%ncohorts


      if (associated(cpatch%dmean_nppleaf         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_nppleaf                                      &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_NPPLEAF_CO           :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Net primary productivity - Leaf'                 &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_nppfroot        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_nppfroot                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_NPPFROOT_CO          :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Net primary productivity - Fine root'            &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_nppsapwood      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_nppsapwood                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_NPPSAPWOOD_CO        :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Net primary productivity - Sap wood'             &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_nppcroot        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_nppcroot                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_NPPCROOT_CO          :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Net primary productivity - Coarse root'          &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_nppseeds        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_nppseeds                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_NPPSEEDS_CO          :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Net primary productivity - seeds'                &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_nppwood         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_nppwood                                      &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_NPPWOOD_CO           :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Net primary productivity - heart wood'           &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_nppdaily        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_nppdaily                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_NPPDAILY_CO          :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Net primary productivity - total'                &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_gpp             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_gpp                                          &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_GPP_CO               :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Gross primary productivity'                      &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_npp             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_npp                                          &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_NPP_CO               :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Net primary productivity'                        &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_leaf_resp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_leaf_resp                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_LEAF_RESP_CO         :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf respiration'                                &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_root_resp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_root_resp                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_ROOT_RESP_CO         :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Root respiration'                                &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_leaf_growth_resp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_leaf_growth_resp                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_LEAF_GROWTH_RESP_CO  :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf growth respiration'                         &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_root_growth_resp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_root_growth_resp                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_ROOT_GROWTH_RESP_CO  :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Root growth respiration'                         &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_sapa_growth_resp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_sapa_growth_resp                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_SAPA_GROWTH_RESP_CO  :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Aboveground sapwood growth respiration'          &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_sapb_growth_resp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_sapb_growth_resp                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_SAPB_GROWTH_RESP_CO  :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Belowground sapwood growth respiration'          &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_leaf_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_leaf_storage_resp                            &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_LEAF_STORAGE_RESP_CO      :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf Storage respiration'                        &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_root_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_root_storage_resp                            &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_ROOT_STORAGE_RESP_CO      :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Root storage respiration'                        &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_sapa_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_sapa_storage_resp                            &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_SAPA_STORAGE_RESP_CO      :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Aboveground Sapwood Storage respiration'         &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_sapb_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_sapb_storage_resp                            &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_SAPB_STORAGE_RESP_CO      :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Belowground Sapwood Storage respiration'         &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_plresp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_plresp                                       &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_PLRESP_CO            :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Plant respiration'                               &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_leaf_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_leaf_energy                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_LEAF_ENERGY_CO       :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf internal energy'                            &
                           ,'[       J/m2]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_leaf_water      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_leaf_water                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_LEAF_WATER_CO        :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf surface water'                              &
                           ,'[      kg/m2]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_leaf_hcap       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_leaf_hcap                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_LEAF_HCAP_CO         :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf heat capacity'                              &
                           ,'[     J/m2/K]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_leaf_vpdef      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_leaf_vpdef                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_LEAF_VPDEF_CO        :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf vapour pressure deficit'                    &
                           ,'[         Pa]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_leaf_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_leaf_temp                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_LEAF_TEMP_CO         :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf temperature'                                &
                           ,'[          K]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_leaf_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_leaf_fliq                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_LEAF_FLIQ_CO         :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Liquid fraction'                                 &
                           ,'[         --]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_leaf_gsw        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_leaf_gsw                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_LEAF_GSW_CO          :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Stomatal conductance'                            &
                           ,'[kg/m2leaf/s]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_leaf_gbw        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_leaf_gbw                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_LEAF_GBW_CO          :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf boundary layer conductance'                 &
                           ,'[kg/m2leaf/s]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_wood_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_wood_energy                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_WOOD_ENERGY_CO       :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Wood internal energy'                            &
                           ,'[       J/m2]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_wood_water      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_wood_water                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_WOOD_WATER_CO        :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Wood surface water'                              &
                           ,'[      kg/m2]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_wood_hcap       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_wood_hcap                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_WOOD_HCAP_CO         :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Wood heat capacity'                              &
                           ,'[     J/m2/K]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_wood_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_wood_temp                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_WOOD_TEMP_CO         :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Wood temperature'                                &
                           ,'[          K]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_wood_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_wood_fliq                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_WOOD_FLIQ_CO         :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Liquid fraction'                                 &
                           ,'[         --]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_wood_gbw        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_wood_gbw                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_WOOD_GBW_CO          :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Wood boundary layer conductance'                 &
                           ,'[kg/m2wood/s]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_fs_open         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_fs_open                                      &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_FS_OPEN_CO           :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Net stress factor'                               &
                           ,'[         --]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_fsw             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_fsw                                          &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_FSW_CO               :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Moisture stress'                                 &
                           ,'[         --]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_fsn             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_fsn                                          &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_FSN_CO               :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Nitrogen stress'                                 &
                           ,'[         --]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_a_open         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_a_open                                       &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_A_OPEN_CO             :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Assimilation rate (no soil moist. stress)'       &
                           ,'[ umol/m2l/s]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_a_closed       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_a_closed                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_A_CLOSED_CO           :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Minimum assimilation rate'                       &
                           ,'[ umol/m2l/s]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_a_net           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_a_net                                        &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_A_NET_CO               :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Actual assimilation rate'                        &
                           ,'[ umol/m2l/s]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_a_light         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_a_light                                      &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_A_LIGHT_CO             :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Light-limited assimilation rate'                 &
                           ,'[ umol/m2l/s]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_a_rubp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_a_rubp                                       &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_A_RUBP_CO              :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - RuBP-limited assimilation rate'                  &
                           ,'[ umol/m2l/s]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_a_co2           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_a_co2                                        &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_A_CO2_CO              :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - CO2-limited assimilation rate'                   &
                           ,'[ umol/m2l/s]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_psi_open        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_psi_open                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_PSI_OPEN_CO          :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Transpiration with no stress'                    &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_psi_closed      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_psi_closed                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_PSI_CLOSED_CO        :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Transpiration at maximum stress'                 &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_water_supply    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_water_supply                                 &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_WATER_SUPPLY_CO      :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Water supply'                                    &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_light_level     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_light_level                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_LIGHT_LEVEL_CO       :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Light level'                                     &
                           ,'[         --]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_light_level_beam)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_light_level_beam                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_LIGHT_LEVEL_BEAM_CO  :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Light level (direct radiation)'                  &
                           ,'[         --]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_light_level_diff)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_light_level_diff                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_LIGHT_LEVEL_DIFF_CO  :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Light level (diffuse radiation)'                 &
                           ,'[         --]','(icohort)'            )
      end if

      if (associated(cpatch%dmean_par_level_beam)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_par_level_beam                             &
               ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
               ,'DMEAN_PAR_LEVEL_BEAM_CO  :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
               ,'Daily mean - PAR flux density, beam'                  &
               ,'[ W/m2 ]','(icohort)'            )
      end if

      if (associated(cpatch%dmean_par_level_diffu)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_par_level_diffu                              &
               ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
               ,'DMEAN_PAR_LEVEL_DIFFU_CO  :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
               ,'Daily mean - PAR flux density, diffuse up'                  &
               ,'[ W/m2 ]','(icohort)'            )
      end if

      if (associated(cpatch%dmean_par_level_diffd)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_par_level_diffd                             &
               ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
               ,'DMEAN_PAR_LEVEL_DIFFD_CO  :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
               ,'Daily mean - PAR flux density, diffuse down'                  &
               ,'[ W/m2 ]','(icohort)'            )
      end if
      
      if (associated(cpatch%dmean_par_l           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_par_l                                        &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_PAR_L_CO             :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - PAR absorbed by leaves'                          &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_par_l_beam      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_par_l_beam                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_PAR_L_BEAM_CO        :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Direct PAR absorbed by leaves'                   &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_par_l_diff      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_par_l_diff                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_PAR_L_DIFF_CO        :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Diffuse PAR absorbed by leaves'                  &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_rshort_l        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_rshort_l                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_RSHORT_L_CO          :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Shortwave radiation absorbed by leaves'          &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_rlong_l         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_rlong_l                                      &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_RLONG_L_CO           :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Longwave radiation absorbed by leaves'           &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_sensible_lc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_sensible_lc                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_SENSIBLE_LC_CO       :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Sensible heat'                                   &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_vapor_lc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_vapor_lc                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_VAPOR_LC_CO          :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf evaporation'                                &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_transp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_transp                                       &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_TRANSP_CO            :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf transpiration'                              &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_intercepted_al  )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_intercepted_al                               &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_INTERCEPTED_AL_CO    :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf interception'                               &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_wshed_lg        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_wshed_lg                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_WSHED_LG_CO          :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Leaf shedding'                                   &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_rshort_w        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_rshort_w                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_RSHORT_W_CO          :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Shortwave radiation absorbed by wood'            &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_rlong_w         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_rlong_w                                      &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_RLONG_W_CO           :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Longwave radiation absorbed by wood'             &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_sensible_wc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_sensible_wc                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_SENSIBLE_WC_CO       :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Sensible heat'                                   &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_vapor_wc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_vapor_wc                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_VAPOR_WC_CO          :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Wood evaporation'                                &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_intercepted_aw  )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_intercepted_aw                               &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_INTERCEPTED_AW_CO    :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Wood interception'                               &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%dmean_wshed_wg        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%dmean_wshed_wg                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_WSHED_WG_CO          :41:'//trim(dail_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Wood shedding'                                   &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_patchtype_p41dmean
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the cohort-level variables          !
   ! (patchtype) that have one dimension and are real (type 41) and mmean.                 !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_patchtype_p41mmean(cpatch,igr,init,var_len,var_len_global,max_ptrs   &
                                        ,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(patchtype)       , target        :: cpatch
      integer               , intent(in)    :: init
      integer               , intent(in)    :: igr
      integer               , intent(in)    :: var_len
      integer               , intent(in)    :: max_ptrs
      integer               , intent(in)    :: var_len_global
      integer               , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                               :: npts
      character(len=str_len)                :: eorq_keys
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !      Decide whether to write the monthly means to the history file.                !
      !------------------------------------------------------------------------------------!
      select case (iadd_cohort_means)
      case (0)
         return
      case (1)
         if (history_eorq) then
            eorq_keys = 'hist:mont:dcyc'
         else
            eorq_keys = 'mont:dcyc'
         end if
      end select
      !------------------------------------------------------------------------------------!








      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 1-D vectors, with dimension ncohorts.  Notice     !
      ! that they all use the same npts.  Here you should only add variables of type 41    !
      ! (real, scalar), and a "mmean" or "mmsqu" variable.                                 !
      !------------------------------------------------------------------------------------!
      npts = cpatch%ncohorts


      if (associated(cpatch%mmean_gpp             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_gpp                                          &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_GPP_CO               :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Gross primary productivity'                    &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_npp             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_npp                                          &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_NPP_CO               :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Net primary productivity'                      &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_leaf_resp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_leaf_resp                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_LEAF_RESP_CO         :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf respiration'                              &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_root_resp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_root_resp                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_ROOT_RESP_CO         :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Root respiration'                              &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_leaf_growth_resp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_leaf_growth_resp                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_LEAF_GROWTH_RESP_CO  :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf growth respiration'                       &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_root_growth_resp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_root_growth_resp                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_ROOT_GROWTH_RESP_CO  :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Root growth respiration'                       &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_sapa_growth_resp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_sapa_growth_resp                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_SAPA_GROWTH_RESP_CO  :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Aboveground sapwood growth respiration'        &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_sapb_growth_resp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_sapb_growth_resp                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_SAPB_GROWTH_RESP_CO  :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Belowground sapwood growth respiration'        &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_leaf_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_leaf_storage_resp                            &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_LEAF_STORAGE_RESP_CO      :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf Storage respiration'                      &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_root_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_root_storage_resp                            &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_ROOT_STORAGE_RESP_CO      :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Root Storage respiration'                      &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_sapa_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_sapa_storage_resp                            &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_SAPA_STORAGE_RESP_CO      :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Aboveground Sapwood Storage respiration'       &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_sapb_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_sapb_storage_resp                            &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_SAPB_STORAGE_RESP_CO      :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Belowground Sapwood Storage respiration'       &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_plresp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_plresp                                       &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_PLRESP_CO            :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Plant respiration'                             &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_leaf_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_leaf_energy                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_LEAF_ENERGY_CO       :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf internal energy'                          &
                           ,'[       J/m2]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_leaf_water      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_leaf_water                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_LEAF_WATER_CO        :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf surface water'                            &
                           ,'[      kg/m2]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_leaf_hcap       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_leaf_hcap                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_LEAF_HCAP_CO         :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf heat capacity'                            &
                           ,'[     J/m2/K]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_leaf_vpdef      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_leaf_vpdef                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_LEAF_VPDEF_CO        :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf vapour pressure deficit'                  &
                           ,'[         Pa]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_leaf_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_leaf_temp                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_LEAF_TEMP_CO         :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf temperature'                              &
                           ,'[          K]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_leaf_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_leaf_fliq                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_LEAF_FLIQ_CO         :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Liquid fraction'                               &
                           ,'[         --]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_leaf_gsw        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_leaf_gsw                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_LEAF_GSW_CO          :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Stomatal conductance'                          &
                           ,'[kg/m2leaf/s]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_leaf_gbw        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_leaf_gbw                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_LEAF_GBW_CO          :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf boundary layer conductance'               &
                           ,'[kg/m2leaf/s]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_wood_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_wood_energy                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_WOOD_ENERGY_CO       :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Wood internal energy'                          &
                           ,'[       J/m2]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_wood_water      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_wood_water                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_WOOD_WATER_CO        :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Wood surface water'                            &
                           ,'[      kg/m2]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_wood_hcap       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_wood_hcap                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_WOOD_HCAP_CO         :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Wood heat capacity'                            &
                           ,'[     J/m2/K]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_wood_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_wood_temp                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_WOOD_TEMP_CO         :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Wood temperature'                              &
                           ,'[          K]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_wood_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_wood_fliq                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_WOOD_FLIQ_CO         :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Liquid fraction'                               &
                           ,'[         --]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_wood_gbw        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_wood_gbw                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_WOOD_GBW_CO          :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Wood boundary layer conductance'               &
                           ,'[kg/m2wood/s]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_fs_open         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_fs_open                                      &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_FS_OPEN_CO           :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Net stress factor'                             &
                           ,'[         --]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_fsw             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_fsw                                          &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_FSW_CO               :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Moisture stress'                               &
                           ,'[         --]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_fsn             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_fsn                                          &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_FSN_CO               :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Nitrogen stress'                               &
                           ,'[         --]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_a_open         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_a_open                                       &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_A_OPEN_CO             :41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Assimilation rate (no soil moist. stress)'     &
                           ,'[ umol/m2l/s]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_a_closed       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_a_closed                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_A_CLOSED_CO           :41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Minimum assimilation rate'                     &
                           ,'[ umol/m2l/s]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_a_net           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_a_net                                        &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_A_NET_CO               :41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Actual assimilation rate'                      &
                           ,'[ umol/m2l/s]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_a_light         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_a_light                                      &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_A_LIGHT_CO             :41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Light-limited assimilation rate'               &
                           ,'[ umol/m2l/s]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_a_rubp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_a_rubp                                       &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_A_RUBP_CO              :41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - RuBP-limited assimilation rate'                &
                           ,'[ umol/m2l/s]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_a_co2           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_a_co2                                        &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_A_CO2_CO              :41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - CO2-limited assimilation rate'                 &
                           ,'[ umol/m2l/s]','(icohort)'            )
      end if


      if (associated(cpatch%mmean_psi_open        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_psi_open                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_PSI_OPEN_CO          :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Transpiration with no stress'                  &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_psi_closed      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_psi_closed                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_PSI_CLOSED_CO        :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Transpiration at maximum stress'               &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_water_supply    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_water_supply                                 &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_WATER_SUPPLY_CO      :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Water supply'                                  &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_light_level     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_light_level                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_LIGHT_LEVEL_CO       :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Light level'                                   &
                           ,'[         --]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_light_level_beam)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_light_level_beam                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_LIGHT_LEVEL_BEAM_CO  :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Light level (direct radiation)'                &
                           ,'[         --]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_light_level_diff)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_light_level_diff                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_LIGHT_LEVEL_DIFF_CO  :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Light level (diffuse radiation)'               &
                           ,'[         --]','(icohort)'            )
      end if

      if (associated(cpatch%mmean_par_level_beam)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_par_level_beam                             &
               ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
               ,'MMEAN_PAR_LEVEL_BEAM_CO  :41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
               ,'Monthly mean - PAR flux density, beam'                  &
               ,'[ W/m2 ]','(icohort)'            )
      end if

      if (associated(cpatch%mmean_par_level_diffu)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_par_level_diffu                              &
               ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
               ,'MMEAN_PAR_LEVEL_DIFFU_CO  :41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
               ,'Monthly mean - PAR flux density, diffuse up'                  &
               ,'[ W/m2 ]','(icohort)'            )
      end if

      if (associated(cpatch%mmean_par_level_diffd)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_par_level_diffd                             &
               ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
               ,'MMEAN_PAR_LEVEL_DIFFD_CO  :41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
               ,'Monthly mean - PAR flux density, diffuse down'                  &
               ,'[ W/m2 ]','(icohort)'            )
      end if

      if (associated(cpatch%mmean_par_l           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_par_l                                        &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_PAR_L_CO             :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - PAR absorbed by leaves'                        &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_par_l_beam      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_par_l_beam                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_PAR_L_BEAM_CO        :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Direct PAR absorbed by leaves'                 &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_par_l_diff      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_par_l_diff                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_PAR_L_DIFF_CO        :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Diffuse PAR absorbed by leaves'                &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_rshort_l        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_rshort_l                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_RSHORT_L_CO          :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Shortwave radiation absorbed by leaves'        &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_rlong_l         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_rlong_l                                      &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_RLONG_L_CO           :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Longwave radiation absorbed by leaves'         &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_sensible_lc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_sensible_lc                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_SENSIBLE_LC_CO       :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Sensible heat'                                 &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_vapor_lc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_vapor_lc                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_VAPOR_LC_CO          :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf evaporation'                              &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_transp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_transp                                       &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_TRANSP_CO            :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf transpiration'                            &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_intercepted_al  )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_intercepted_al                               &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_INTERCEPTED_AL_CO    :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf interception'                             &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_wshed_lg        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_wshed_lg                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_WSHED_LG_CO          :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf shedding'                                 &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_rshort_w        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_rshort_w                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_RSHORT_W_CO          :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Shortwave radiation absorbed by wood'          &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_rlong_w         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_rlong_w                                      &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_RLONG_W_CO           :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Longwave radiation absorbed by wood'           &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_sensible_wc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_sensible_wc                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_SENSIBLE_WC_CO       :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Sensible heat'                                 &
                           ,'[       W/m2]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_vapor_wc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_vapor_wc                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_VAPOR_WC_CO          :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Wood evaporation'                              &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_intercepted_aw  )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_intercepted_aw                               &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_INTERCEPTED_AW_CO    :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Wood interception'                             &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_wshed_wg        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_wshed_wg                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_WSHED_WG_CO          :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Wood shedding'                                 &
                           ,'[    kg/m2/s]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_lai             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_lai                                          &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_LAI_CO                :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf area index'                               &
                           ,'[  m2leaf/m2]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_bleaf           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_bleaf                                        &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_BLEAF_CO             :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf biomass'                                  &
                           ,'[     kgC/pl]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_broot           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_broot                                        &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_BROOT_CO             :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Root biomass'                                  &
                           ,'[     kgC/pl]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_bstorage        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_bstorage                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_BSTORAGE_CO          :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Storage biomass'                               &
                           ,'[     kgC/pl]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_leaf_maintenance)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_leaf_maintenance                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_LEAF_MAINTENANCE_CO     :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf maintenance rate'                         &
                           ,'[  kgC/pl/yr]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_root_maintenance)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_root_maintenance                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_ROOT_MAINTENANCE_CO     :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Root maintenance rate'                         &
                           ,'[  kgC/pl/yr]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_leaf_drop       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_leaf_drop                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_LEAF_DROP_CO            :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Leaf drop'                                     &
                           ,'[  kgC/pl/yr]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_cb              )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_cb                                           &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_CB_CO                   :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Carbon balance'                                &
                           ,'[  kgC/pl]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_nppleaf         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_nppleaf                                      &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_NPPLEAF_CO           :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Net primary productivity - Leaf'               &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_nppfroot        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_nppfroot                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_NPPFROOT_CO          :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Net primary productivity - Fine root'          &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_nppsapwood      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_nppsapwood                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_NPPSAPWOOD_CO        :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Net primary productivity - Sap wood'           &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_nppcroot        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_nppcroot                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_NPPCROOT_CO          :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Net primary productivity - Coarse root'        &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_nppseeds        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_nppseeds                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_NPPSEEDS_CO          :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Net primary productivity - seeds'              &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_nppwood         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_nppwood                                      &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_NPPWOOD_CO           :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Net primary productivity - heart wood'         &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%mmean_nppdaily        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmean_nppdaily                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_NPPDAILY_CO          :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Net primary productivity - total'              &
                           ,'[  kgC/m2/yr]','(icohort)'            )
      end if
      if (associated(cpatch%mmsqu_gpp             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmsqu_gpp                                          &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMSQU_GPP_CO               :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Gross primary productivity'                    &
                           ,'[kgC2/m4/yr2]','(icohort)'            )
      end if
      if (associated(cpatch%mmsqu_npp             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmsqu_npp                                          &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMSQU_NPP_CO               :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Net primary productivity'                      &
                           ,'[kgC2/m4/yr2]','(icohort)'            )
      end if
      if (associated(cpatch%mmsqu_plresp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmsqu_plresp                                       &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMSQU_PLRESP_CO            :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Plant respiration'                             &
                           ,'[kgC2/m4/yr2]','(icohort)'            )
      end if
      if (associated(cpatch%mmsqu_sensible_lc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmsqu_sensible_lc                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMSQU_SENSIBLE_LC_CO       :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Sensible heat'                                 &
                           ,'[      W2/m4]','(icohort)'            )
      end if
      if (associated(cpatch%mmsqu_vapor_lc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmsqu_vapor_lc                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMSQU_VAPOR_LC_CO          :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Leaf evaporation'                              &
                           ,'[  kg2/m4/s2]','(icohort)'            )
      end if
      if (associated(cpatch%mmsqu_transp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmsqu_transp                                       &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMSQU_TRANSP_CO            :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Leaf transpiration'                            &
                           ,'[  kg2/m4/s2]','(icohort)'            )
      end if
      if (associated(cpatch%mmsqu_sensible_wc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmsqu_sensible_wc                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMSQU_SENSIBLE_WC_CO       :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Sensible heat'                                 &
                           ,'[      W2/m4]','(icohort)'            )
      end if
      if (associated(cpatch%mmsqu_vapor_wc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%mmsqu_vapor_wc                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMSQU_VAPOR_WC_CO          :41:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly MSSq - Wood evaporation'                              &
                           ,'[  kg2/m4/s2]','(icohort)'            )
      end if
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_patchtype_p41mmean
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the cohort-level variables          !
   ! (patchtype) that have two dimensions (ndcycle,ncohorts) and are real (type -41).      !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_patchtype_m41(cpatch,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(patchtype)       , target        :: cpatch
      integer               , intent(in)    :: init
      integer               , intent(in)    :: igr
      integer               , intent(in)    :: var_len
      integer               , intent(in)    :: max_ptrs
      integer               , intent(in)    :: var_len_global
      integer               , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                               :: npts
      character(len=str_len)                :: eorq_keys
      !------------------------------------------------------------------------------------!



      !------------------------------------------------------------------------------------!
      !      Decide whether to write the mean diel to the history file.                    !
      !------------------------------------------------------------------------------------!
      select case (iadd_cohort_means)
      case (0)
         return
      case (1)
         if (history_eorq) then
            eorq_keys = 'hist:dcyc'
         else
            eorq_keys = 'dcyc'
         end if
      end select
      !------------------------------------------------------------------------------------!




      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 2-D vectors, with dimension ncohorts and ndcycle. !
      ! Notice that they all use the same npts.  Here you should only add variables of     !
      ! type -41.                                                                          !
      !------------------------------------------------------------------------------------!
      npts = cpatch%ncohorts * ndcycle
      if (associated(cpatch%qmean_gpp             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_gpp                                          &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_GPP_CO              :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Gross primary productivity'                       &
                           ,'[  kgC/m2/yr]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_npp             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_npp                                          &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_NPP_CO              :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Net primary productivity'                         &
                           ,'[  kgC/m2/yr]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_leaf_resp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_leaf_resp                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_LEAF_RESP_CO        :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf respiration'                                 &
                           ,'[  kgC/m2/yr]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_root_resp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_root_resp                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_ROOT_RESP_CO        :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Root respiration'                                 &
                           ,'[  kgC/m2/yr]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_leaf_growth_resp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_leaf_growth_resp                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_LEAF_GROWTH_RESP_CO :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf growth respiration'                          &
                           ,'[  kgC/m2/yr]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_root_growth_resp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_root_growth_resp                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_ROOT_GROWTH_RESP_CO :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Root growth respiration'                          &
                           ,'[  kgC/m2/yr]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_sapa_growth_resp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_sapa_growth_resp                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_SAPA_GROWTH_RESP_CO :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Aboveground sapwood growth respiration'           &
                           ,'[  kgC/m2/yr]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_sapb_growth_resp)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_sapb_growth_resp                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_SAPB_GROWTH_RESP_CO :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Belowground sapwood growth respiration'           &
                           ,'[  kgC/m2/yr]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_leaf_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_leaf_storage_resp                            &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_LEAF_STORAGE_RESP_CO     :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf Storage respiration'                         &
                           ,'[  kgC/m2/yr]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_root_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_root_storage_resp                            &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_ROOT_STORAGE_RESP_CO     :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Root Storage respiration'                         &
                           ,'[  kgC/m2/yr]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_sapa_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_sapa_storage_resp                            &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_SAPA_STORAGE_RESP_CO     :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Aboveground Sapwood Storage respiration'          &
                           ,'[  kgC/m2/yr]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_sapb_storage_resp    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_sapb_storage_resp                            &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_SAPB_STORAGE_RESP_CO     :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Belowground Sapwood Storage respiration'          &
                           ,'[  kgC/m2/yr]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_plresp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_plresp                                       &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_PLRESP_CO           :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Plant respiration'                                &
                           ,'[  kgC/m2/yr]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_leaf_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_leaf_energy                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_LEAF_ENERGY_CO      :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf internal energy'                             &
                           ,'[       J/m2]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_leaf_water      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_leaf_water                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_LEAF_WATER_CO       :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf surface water'                               &
                           ,'[      kg/m2]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_leaf_hcap       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_leaf_hcap                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_LEAF_HCAP_CO        :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf heat capacity'                               &
                           ,'[     J/m2/K]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_leaf_vpdef      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_leaf_vpdef                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_LEAF_VPDEF_CO       :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf vapour pressure deficit'                     &
                           ,'[         Pa]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_leaf_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_leaf_temp                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_LEAF_TEMP_CO        :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf temperature'                                 &
                           ,'[          K]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_leaf_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_leaf_fliq                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_LEAF_FLIQ_CO        :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Liquid fraction'                                  &
                           ,'[         --]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_leaf_gsw        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_leaf_gsw                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_LEAF_GSW_CO         :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Stomatal conductance'                             &
                           ,'[kg/m2leaf/s]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_leaf_gbw        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_leaf_gbw                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_LEAF_GBW_CO         :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf boundary layer conductance'                  &
                           ,'[kg/m2leaf/s]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_wood_energy     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_wood_energy                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_WOOD_ENERGY_CO      :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Wood internal energy'                             &
                           ,'[       J/m2]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_wood_water      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_wood_water                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_WOOD_WATER_CO       :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Wood surface water'                               &
                           ,'[      kg/m2]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_wood_hcap       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_wood_hcap                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_WOOD_HCAP_CO        :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Wood heat capacity'                               &
                           ,'[     J/m2/K]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_wood_temp       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_wood_temp                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_WOOD_TEMP_CO        :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Wood temperature'                                 &
                           ,'[          K]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_wood_fliq       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_wood_fliq                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_WOOD_FLIQ_CO        :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Liquid fraction'                                  &
                           ,'[         --]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_wood_gbw        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_wood_gbw                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_WOOD_GBW_CO         :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Wood boundary layer conductance'                  &
                           ,'[kg/m2wood/s]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_fs_open         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_fs_open                                      &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_FS_OPEN_CO          :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Net stress factor'                                &
                           ,'[         --]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_fsw             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_fsw                                          &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_FSW_CO              :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Moisture stress'                                  &
                           ,'[         --]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_fsn             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_fsn                                          &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_FSN_CO              :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Nitrogen stress'                                  &
                           ,'[         --]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_a_open         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_a_open                                       &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_A_OPEN_CO             :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Assimilation rate (no soil moist. stress)'     &
                           ,'[ umol/m2l/s]','(ndcycle,icohort)'            )
      end if
      if (associated(cpatch%qmean_a_closed       )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_a_closed                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_A_CLOSED_CO           :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Minimum assimilation rate'                     &
                           ,'[ umol/m2l/s]','(ndcycle,icohort)'            )
      end if
      if (associated(cpatch%qmean_a_net           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_a_net                                        &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_A_NET_CO              :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Actual assimilation rate'                      &
                           ,'[ umol/m2l/s]','(ndcycle,icohort)'            )
      end if
      if (associated(cpatch%qmean_a_light         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_a_light                                      &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_A_LIGHT_CO             :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Light-limited assimilation rate'                  &
                           ,'[ umol/m2l/s]','(ndcycle,icohort)'            )
      end if
      if (associated(cpatch%qmean_a_rubp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_a_rubp                                       &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_A_RUBP_CO              :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - RuBP-limited assimilation rate'                   &
                           ,'[ umol/m2l/s]','(ndcycle,icohort)'            )
      end if
      if (associated(cpatch%qmean_a_co2           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_a_co2                                        &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_A_CO2_CO              :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - CO2-limited assimilation rate'                    &
                           ,'[ umol/m2l/s]','(ndcycle,icohort)'            )
      end if
      if (associated(cpatch%qmean_psi_open        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_psi_open                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_PSI_OPEN_CO         :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Transpiration with no stress'                     &
                           ,'[    kg/m2/s]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_psi_closed      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_psi_closed                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_PSI_CLOSED_CO       :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Transpiration at maximum stress'                  &
                           ,'[    kg/m2/s]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_water_supply    )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_water_supply                                 &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_WATER_SUPPLY_CO     :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Water supply'                                     &
                           ,'[    kg/m2/s]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_light_level     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_light_level                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_LIGHT_LEVEL_CO      :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Light level'                                      &
                           ,'[         --]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_light_level_beam)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_light_level_beam                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_LIGHT_LEVEL_BEAM_CO :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Light level (direct radiation)'                   &
                           ,'[         --]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_light_level_diff)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_light_level_diff                             &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_LIGHT_LEVEL_DIFF_CO :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Light level (diffuse radiation)'                  &
                           ,'[         --]','(ndcycle,icohort)'    )
      end if

      if (associated(cpatch%qmean_par_level_beam)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_par_level_beam                             &
               ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
               ,'QMEAN_PAR_LEVEL_BEAM_CO  :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                     &
               ,'Mean diel - PAR flux density, beam'                  &
               ,'[ W/m2 ]','(icohort)'            )
      end if

      if (associated(cpatch%qmean_par_level_diffu)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_par_level_diffu                              &
               ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
               ,'QMEAN_PAR_LEVEL_DIFFU_CO  :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
               ,'Mean diel - PAR flux density, diffuse up'                  &
               ,'[ W/m2 ]','(icohort)'            )
      end if

      if (associated(cpatch%qmean_par_level_diffd)) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_par_level_diffd                             &
               ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
               ,'QMEAN_PAR_LEVEL_DIFFD_CO  :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
               ,'Mean diel - PAR flux density, diffuse down'                  &
               ,'[ W/m2 ]','(icohort)'            )
      end if


      if (associated(cpatch%qmean_par_l           )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_par_l                                        &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_PAR_L_CO            :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - PAR absorbed by leaves'                           &
                           ,'[       W/m2]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_par_l_beam      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_par_l_beam                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_PAR_L_BEAM_CO       :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Direct PAR absorbed by leaves'                    &
                           ,'[       W/m2]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_par_l_diff      )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_par_l_diff                                   &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_PAR_L_DIFF_CO       :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Diffuse PAR absorbed by leaves'                   &
                           ,'[       W/m2]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_rshort_l        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_rshort_l                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_RSHORT_L_CO         :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Shortwave radiation absorbed by leaves'           &
                           ,'[       W/m2]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_rlong_l         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_rlong_l                                      &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_RLONG_L_CO          :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Longwave radiation absorbed by leaves'            &
                           ,'[       W/m2]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_sensible_lc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_sensible_lc                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_SENSIBLE_LC_CO      :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Sensible heat'                                    &
                           ,'[       W/m2]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_vapor_lc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_vapor_lc                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_VAPOR_LC_CO         :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf evaporation'                                 &
                           ,'[    kg/m2/s]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_transp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_transp                                       &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_TRANSP_CO           :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf transpiration'                               &
                           ,'[    kg/m2/s]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_intercepted_al  )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_intercepted_al                               &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_INTERCEPTED_AL_CO   :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf interception'                                &
                           ,'[    kg/m2/s]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_wshed_lg        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_wshed_lg                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_WSHED_LG_CO         :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Leaf shedding'                                    &
                           ,'[    kg/m2/s]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_rshort_w        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_rshort_w                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_RSHORT_W_CO         :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Shortwave radiation absorbed by wood'             &
                           ,'[       W/m2]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_rlong_w         )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_rlong_w                                      &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_RLONG_W_CO          :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Longwave radiation absorbed by wood'              &
                           ,'[       W/m2]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_sensible_wc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_sensible_wc                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_SENSIBLE_WC_CO      :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Sensible heat'                                    &
                           ,'[       W/m2]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_vapor_wc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_vapor_wc                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_VAPOR_WC_CO         :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Wood evaporation'                                 &
                           ,'[    kg/m2/s]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_intercepted_aw  )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_intercepted_aw                               &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_INTERCEPTED_AW_CO   :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Wood interception'                                &
                           ,'[    kg/m2/s]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmean_wshed_wg        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmean_wshed_wg                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_WSHED_WG_CO         :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Wood shedding'                                    &
                           ,'[    kg/m2/s]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmsqu_gpp             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmsqu_gpp                                          &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMSQU_GPP_CO              :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Gross primary productivity'                       &
                           ,'[kgC2/m4/yr2]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmsqu_npp             )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmsqu_npp                                          &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMSQU_NPP_CO              :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Net primary productivity'                         &
                           ,'[kgC2/m4/yr2]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmsqu_plresp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmsqu_plresp                                       &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMSQU_PLRESP_CO           :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Plant respiration'                                &
                           ,'[kgC2/m4/yr2]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmsqu_sensible_lc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmsqu_sensible_lc                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMSQU_SENSIBLE_LC_CO      :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Sensible heat'                                    &
                           ,'[      W2/m4]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmsqu_vapor_lc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmsqu_vapor_lc                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMSQU_VAPOR_LC_CO         :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Leaf evaporation'                                 &
                           ,'[  kg2/m4/s2]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmsqu_transp          )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmsqu_transp                                       &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMSQU_TRANSP_CO           :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Leaf transpiration'                               &
                           ,'[  kg2/m4/s2]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmsqu_sensible_wc     )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmsqu_sensible_wc                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMSQU_SENSIBLE_WC_CO      :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Sensible heat'                                    &
                           ,'[      W2/m4]','(ndcycle,icohort)'    )
      end if
      if (associated(cpatch%qmsqu_vapor_wc        )) then
         nvar = nvar+1
         call vtable_edio_r(npts,cpatch%qmsqu_vapor_wc                                     &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMSQU_VAPOR_WC_CO         :-41:'//trim(eorq_keys)     )
         call metadata_edio(nvar,igr                                                       &
                           ,'MSSq diel - Wood evaporation'                                 &
                           ,'[  kg2/m4/s2]','(ndcycle,icohort)'    )
      end if
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_patchtype_m41
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the cohort-level variables          !
   ! (patchtype) that have two dimensions (n_mort,ncohorts) and are real (type 48).        !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_patchtype_p48(cpatch,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(patchtype)       , target        :: cpatch
      integer               , intent(in)    :: init
      integer               , intent(in)    :: igr
      integer               , intent(in)    :: var_len
      integer               , intent(in)    :: max_ptrs
      integer               , intent(in)    :: var_len_global
      integer               , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                               :: npts
      character(len=str_len)                :: eorq_keys
      !------------------------------------------------------------------------------------!




      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 2-D vectors, with dimension ncohorts and mortal-  !
      ! ity types months.  Notice that they all use the same npts.  Only variables of type !
      ! 48 should be placed here.                                                          !
      !------------------------------------------------------------------------------------!
      npts = cpatch%ncohorts * n_mort

      if (associated(cpatch%mort_rate)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpatch%mort_rate                                          &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MORT_RATE_CO :48:hist:dail') 
         call metadata_edio(nvar,igr,'Mortality rates','[1/yr]','icohort') 
      end if
      !------------------------------------------------------------------------------------!




      !------------------------------------------------------------------------------------!
      !      Decide whether to write the monthly means to the history file.                !
      !------------------------------------------------------------------------------------!
      select case (iadd_cohort_means)
      case (0)
         return
      case (1)
         if (history_eorq) then
            eorq_keys = 'hist:mont:dcyc'
         else
            eorq_keys = 'mont:dcyc'
         end if
      end select
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      if (associated(cpatch%mmean_mort_rate)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpatch%mmean_mort_rate                                    &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_MORT_RATE_CO :48:'//trim(eorq_keys))
         call metadata_edio(nvar,igr,'Monthly mean mortality rate','[1/yr]','icohort')
      end if
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_patchtype_p48
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the cohort-level variables          !
   ! (patchtype) that have two dimensions (13 months,ncohorts) and are real (type 49).     !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_patchtype_p49(cpatch,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(patchtype), target        :: cpatch
      integer        , intent(in)    :: init
      integer        , intent(in)    :: igr
      integer        , intent(in)    :: var_len
      integer        , intent(in)    :: max_ptrs
      integer        , intent(in)    :: var_len_global
      integer        , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                        :: npts
      !------------------------------------------------------------------------------------!





      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 2-D vectors, with dimension ncohorts and 13       !
      ! months.  Notice that they all use the same npts.  Here you should only add vari-   !
      ! ables of type 49.                                                                  !
      !------------------------------------------------------------------------------------!
      npts = cpatch%ncohorts * 13


      if (associated(cpatch%cb)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpatch%cb,nvar,igr,init,cpatch%coglob_id                  &
                           ,var_len,var_len_global,max_ptrs                                &
                           ,'CB :49:hist:mont:dcyc:year')
         call metadata_edio(nvar,igr,'carbon balance previous 12 months+current'           &
                           ,'[kgC/plant]','13 - icohort') 
      end if

      if (associated(cpatch%cb_lightmax)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpatch%cb_lightmax,nvar,igr,init,cpatch%coglob_id         &
                           ,var_len,var_len_global,max_ptrs                                &
                           ,'CB_LIGHTMAX :49:hist:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'Full light carbon balance last 12 months+current'    &
                           ,'[kgC/plant]','13 - icohort') 
      end if

      if (associated(cpatch%cb_moistmax)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpatch%cb_moistmax,nvar,igr,init,cpatch%coglob_id         &
                           ,var_len,var_len_global,max_ptrs                                &
                           ,'CB_MOISTMAX :49:hist:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'Full moisture carbon balance last 12 months+current' &
                           ,'[kgC/plant]','13 - icohort') 
      end if

      if (associated(cpatch%cb_mlmax)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpatch%cb_mlmax,nvar,igr,init,cpatch%coglob_id            &
                           ,var_len,var_len_global,max_ptrs                                &
                           ,'CB_MLMAX :49:hist:mont:dcyc:year') 
         call metadata_edio(nvar,igr,'Full moisture+light C balance last 12 months+current'&
                           ,'[kgC/plant]','13 - icohort') 
      end if
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_patchtype_p49
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the cohort-level variables          !
   ! (patchtype) that have two dimensions (n_radprof,ncohorts) and are real (type 411).    !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_patchtype_p411(cpatch,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine

      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(patchtype)       , target        :: cpatch
      integer               , intent(in)    :: init
      integer               , intent(in)    :: igr
      integer               , intent(in)    :: var_len
      integer               , intent(in)    :: max_ptrs
      integer               , intent(in)    :: var_len_global
      integer               , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                               :: npts
      character(len=str_len)                :: fast_keys
      character(len=str_len)                :: dail_keys
      character(len=str_len)                :: eorq_keys
      !------------------------------------------------------------------------------------!




      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 2-D vectors, with dimension ncohorts and          !
      ! radiation types.  Notice that they all use the same npts.  Only variables of type  !
      ! 411 should be placed here.                                                         !
      !------------------------------------------------------------------------------------!
      npts = cpatch%ncohorts * n_radprof

      if (associated(cpatch%rad_profile)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpatch%rad_profile                                        &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'RAD_PROFILE_CO :411:hist') 
         call metadata_edio(nvar,igr,'Radiation profile','[W/m2]','(iradprof,icohort)') 
      end if
      !------------------------------------------------------------------------------------!




      !------------------------------------------------------------------------------------!
      !      Decide whether to write the monthly means to the history file.                !
      !------------------------------------------------------------------------------------!
      select case (iadd_cohort_means)
      case (0)
         return
      case (1)
         if (history_fast) then
            fast_keys = 'hist:anal'
         else
            fast_keys = 'anal'
         end if
         if (history_dail) then
            dail_keys = 'hist:dail'
         else
            dail_keys = 'dail'
         end if
         if (history_eorq) then
            eorq_keys = 'hist:mont:dcyc'
         else
            eorq_keys = 'mont:dcyc'
         end if
      end select
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      if (associated(cpatch%fmean_rad_profile)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpatch%fmean_rad_profile                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'FMEAN_RAD_PROFILE_CO :411:'//trim(fast_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Sub-daily mean - Radiation profile','[W/m2]'                  &
                           ,'(iradprof,icohort)')
      end if
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      if (associated(cpatch%dmean_rad_profile)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpatch%dmean_rad_profile                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'DMEAN_RAD_PROFILE_CO :411:'//trim(dail_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Daily mean - Radiation profile','[W/m2]'                      &
                           ,'(iradprof,icohort)')
      end if
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!


      !------------------------------------------------------------------------------------!
      if (associated(cpatch%mmean_rad_profile)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpatch%mmean_rad_profile                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'MMEAN_RAD_PROFILE_CO :411:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Monthly mean - Radiation profile','[W/m2]'                    &
                           ,'(iradprof,icohort)')
      end if
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_patchtype_p411
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !     This routine will fill the pointer table with the cohort-level variables          !
   ! (patchtype) that have three dimensions (n_radprof,ndcycle,ncohorts) and are real      !
   ! (type -411).                                                                          !
   !---------------------------------------------------------------------------------------!
   subroutine filltab_patchtype_m411(cpatch,igr,init,var_len,var_len_global,max_ptrs,nvar)
      use ed_var_tables, only : vtable_edio_r  & ! sub-routine
                              , metadata_edio  ! ! sub-routine
    
      implicit none
      !----- Arguments. -------------------------------------------------------------------!
      type(patchtype)       , target        :: cpatch
      integer               , intent(in)    :: init
      integer               , intent(in)    :: igr
      integer               , intent(in)    :: var_len
      integer               , intent(in)    :: max_ptrs
      integer               , intent(in)    :: var_len_global
      integer               , intent(inout) :: nvar
      !----- Local variables. -------------------------------------------------------------!
      integer                               :: npts
      character(len=str_len)                :: eorq_keys
      !------------------------------------------------------------------------------------!




      !------------------------------------------------------------------------------------!
      !      Decide whether to write the monthly means to the history file.                !
      !------------------------------------------------------------------------------------!
      select case (iadd_cohort_means)
      case (0)
         return
      case (1)
         if (history_eorq) then
            eorq_keys = 'hist:dcyc'
         else
            eorq_keys = 'dcyc'
         end if
      end select
      !------------------------------------------------------------------------------------!




      !------------------------------------------------------------------------------------!
      !------------------------------------------------------------------------------------!
      !       This part should have only 3-D vectors, with dimension ncohorts, ndcycle,    !
      ! and radiation types.  Notice that they all use the same npts.  Only variables of   !
      ! type -411 should be placed here.                                                   !
      !------------------------------------------------------------------------------------!
      npts = cpatch%ncohorts * ndcycle * n_radprof

      if (associated(cpatch%qmean_rad_profile)) then
         nvar=nvar+1
         call vtable_edio_r(npts,cpatch%qmean_rad_profile                                  &
                           ,nvar,igr,init,cpatch%coglob_id,var_len,var_len_global,max_ptrs &
                           ,'QMEAN_RAD_PROFILE_CO :-411:'//trim(eorq_keys))
         call metadata_edio(nvar,igr                                                       &
                           ,'Mean diel - Radiation profile','[W/m2]'                       &
                           ,'(iradprof,ndcycle,icohort)')
      end if
      !------------------------------------------------------------------------------------!

      return
   end subroutine filltab_patchtype_m411
   !=======================================================================================!
   !=======================================================================================!






   !=======================================================================================!
   !=======================================================================================!
   !                                   UTILITY FUNCTIONS                                   !
   !=======================================================================================!
   !=======================================================================================!





   !=======================================================================================!
   !=======================================================================================!
   !      This function gets the total number of sites.                                    !
   !---------------------------------------------------------------------------------------!
   integer function get_nsites(cgrid)

      implicit none
      !------ Arguments. ------------------------------------------------------------------!
      type(edtype)     , target  :: cgrid
      !------ Local variables. ------------------------------------------------------------!
      type(polygontype), pointer :: cpoly
      integer                    :: ipy
      integer                    :: isi
      !------------------------------------------------------------------------------------!

      get_nsites = 0

      do ipy=1,cgrid%npolygons
         cpoly=>cgrid%polygon(ipy)
         do isi=1,cpoly%nsites
            get_nsites = get_nsites + 1
         end do
      end do

      return
   end function get_nsites
   !=======================================================================================!
   !=======================================================================================!





   !=======================================================================================!
   !=======================================================================================!
   !      This function gets the total number of patches.                                  !
   !---------------------------------------------------------------------------------------!
   integer function get_npatches(cgrid)

      implicit none
      !------ Arguments. ------------------------------------------------------------------!
      type(edtype)     , target  :: cgrid
      !------ Local variables. ------------------------------------------------------------!
      type(polygontype), pointer :: cpoly
      type(sitetype)   , pointer :: csite
      integer                    :: ipy
      integer                    :: isi
      integer                    :: ipa
      !------------------------------------------------------------------------------------!


      get_npatches = 0

      do ipy=1,cgrid%npolygons
         cpoly=>cgrid%polygon(ipy)
         do isi=1,cpoly%nsites
            csite=>cpoly%site(isi)
            do ipa=1,csite%npatches
               get_npatches = get_npatches + 1
            end do
         end do
      end do
      return
   end function get_npatches
   !=======================================================================================!
   !=======================================================================================!





   !=======================================================================================!
   !=======================================================================================!
   !      This function gets the total number of ncohorts.                                 !
   !---------------------------------------------------------------------------------------!
   integer function get_ncohorts(cgrid)

      implicit none
      !------ Arguments. ------------------------------------------------------------------!
      type(edtype)     , target  :: cgrid
      !------ Local variables. ------------------------------------------------------------!
      type(polygontype), pointer :: cpoly
      type(sitetype)   , pointer :: csite
      type(patchtype)  , pointer :: cpatch
      integer                    :: ipy
      integer                    :: isi
      integer                    :: ipa
      integer                    :: ico
      !------------------------------------------------------------------------------------!

      get_ncohorts = 0

      do ipy=1,cgrid%npolygons
         cpoly=>cgrid%polygon(ipy)
         do isi=1,cpoly%nsites
            csite=>cpoly%site(isi)
            do ipa=1,csite%npatches
               cpatch=>csite%patch(ipa)
               get_ncohorts = get_ncohorts + cpatch%ncohorts
            end do
         end do
      end do
      return
   end function get_ncohorts
   !=======================================================================================!
   !=======================================================================================!
end module ed_state_vars
!==========================================================================================!
!==========================================================================================!




